﻿' MIT - Licence
'
' Copyright (c) 2010 - 2011 Nils Schmidt
' This program uses Rectifier.exe by permission of the author and copyright holder Philippe E. Hurbain - (C) 2009

' Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"),
' to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
' and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

' The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
' INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
' PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
' FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
' ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

Imports System.Text

Module MatrixCalculations ' It's partly stolen from somewhere , but I did a lot of re- and new-writing, e.g. the Gaussian elimination / Big Integer Class.. ;-)

    Public Function Inverse(ByVal sourceMatrix(,) As Double) As Double(,)
        Dim eachCol As Integer
        Dim eachRow As Integer
        Dim rowsAndCols As Integer

        If (UBound(sourceMatrix, 1) <> UBound(sourceMatrix, 2)) Then
            Throw New Exception("Matrix must be square.")
        End If
        Dim rank As Integer = UBound(sourceMatrix, 1)

        Dim workMatrix(,) As Double = _
            CType(sourceMatrix.Clone, Double(,))

        Dim destMatrix(rank, rank) As Double
        Dim rightHandSide(rank) As Double
        Dim solutions(rank) As Double
        Dim rowPivots(rank) As Integer
        Dim colPivots(rank) As Integer

        workMatrix = FormLU(workMatrix, rowPivots, colPivots, rowsAndCols)

        For eachCol = 0 To rank
            rightHandSide(eachCol) = 1
            BackSolve(workMatrix, rightHandSide, solutions, rowPivots, colPivots)
            For eachRow = 0 To rank
                destMatrix(eachRow, eachCol) = solutions(eachRow)
                rightHandSide(eachRow) = 0
            Next eachRow
        Next eachCol

        Return destMatrix
    End Function

    Private Function FormLU(ByVal sourceMatrix(,) As Double, ByRef rowPivots() As Integer, ByRef colPivots() As Integer, ByRef rowsAndCols As Integer) As Double(,)
        Dim eachRow As Integer
        Dim eachCol As Integer
        Dim pivot As Integer
        Dim rowIndex As Integer
        Dim colIndex As Integer
        Dim bestRow As Integer
        Dim bestCol As Integer
        Dim rowToPivot As Integer
        Dim colToPivot As Integer
        Dim maxValue As Double
        Dim testValue As Double
        Dim oldMax As Double
        Const Deps As Double = 0.0000000000000001

        Dim rank As Integer = UBound(sourceMatrix, 1)
        Dim destMatrix(rank, rank) As Double
        Dim rowNorm(rank) As Double
        ReDim rowPivots(rank)
        ReDim colPivots(rank)

        Array.Copy(sourceMatrix, destMatrix, sourceMatrix.Length)

        For eachRow = 0 To rank
            rowPivots(eachRow) = eachRow
            colPivots(eachRow) = eachRow
            For eachCol = 0 To rank
                rowNorm(eachRow) += Math.Abs(destMatrix(eachRow, eachCol))
            Next eachCol
            If (rowNorm(eachRow) = 0) Then
                Throw New Exception("Cannot invert a singular matrix.")
            End If
        Next eachRow

        For pivot = 0 To rank - 1
            maxValue = 0
            For eachRow = pivot To rank
                rowIndex = rowPivots(eachRow)
                For eachCol = pivot To rank
                    colIndex = colPivots(eachCol)
                    testValue = Math.Abs(destMatrix(rowIndex, colIndex)) _
                        / rowNorm(rowIndex)
                    If (testValue > maxValue) Then
                        maxValue = testValue
                        bestRow = eachRow
                        bestCol = eachCol
                    End If
                Next eachCol
            Next eachRow

            If (maxValue = 0) Then
                Throw New Exception("Singular matrix used for LU.")
            ElseIf (pivot > 1) Then
                If (maxValue < (Deps * oldMax)) Then
                    Throw New Exception("Non-invertible matrix used for LU.")
                End If
            End If
            oldMax = maxValue

            If (rowPivots(pivot) <> rowPivots(bestRow)) Then
                rowsAndCols += 1
                Swap(rowPivots(pivot), rowPivots(bestRow))
            End If

            If (colPivots(pivot) <> colPivots(bestCol)) Then
                rowsAndCols += 1
                Swap(colPivots(pivot), colPivots(bestCol))
            End If

            rowToPivot = rowPivots(pivot)
            colToPivot = colPivots(pivot)

            For eachRow = (pivot + 1) To rank
                rowIndex = rowPivots(eachRow)
                destMatrix(rowIndex, colToPivot) = _
                    -destMatrix(rowIndex, colToPivot) / _
                    destMatrix(rowToPivot, colToPivot)
                For eachCol = (pivot + 1) To rank
                    colIndex = colPivots(eachCol)
                    destMatrix(rowIndex, colIndex) += _
                        destMatrix(rowIndex, colToPivot) * _
                        destMatrix(rowToPivot, colIndex)
                Next eachCol
            Next eachRow
        Next pivot

        If (destMatrix(rowPivots(rank), colPivots(rank)) = 0) Then
            Throw New Exception("Non-invertible matrix used for LU.")
        ElseIf (Math.Abs(destMatrix(rowPivots(rank), colPivots(rank))) / _
                rowNorm(rowPivots(rank))) < (Deps * oldMax) Then
            Throw New Exception("Non-invertible matrix used for LU.")
        End If

        Return destMatrix
    End Function

    Private Sub Swap(ByRef firstValue As Integer, ByRef secondValue As Integer)
        Dim holdValue As Integer
        holdValue = firstValue
        firstValue = secondValue
        secondValue = holdValue
    End Sub

    Private Sub BackSolve(ByVal sourceMatrix(,) As Double, _
            ByVal rightHandSide() As Double, ByVal solutions() As Double, _
            ByRef rowPivots() As Integer, ByRef colPivots() As Integer)
        Dim pivot As Integer
        Dim rowToPivot As Integer
        Dim colToPivot As Integer
        Dim eachRow As Integer
        Dim eachCol As Integer
        Dim rank As Integer = UBound(sourceMatrix, 1)

        For pivot = 0 To (rank - 1)
            colToPivot = colPivots(pivot)
            For eachRow = (pivot + 1) To rank
                rowToPivot = rowPivots(eachRow)
                rightHandSide(rowToPivot) += _
                    sourceMatrix(rowToPivot, colToPivot) _
                    * rightHandSide(rowPivots(pivot))
            Next eachRow
        Next pivot

        For eachRow = rank To 0 Step -1
            colToPivot = colPivots(eachRow)
            rowToPivot = rowPivots(eachRow)
            solutions(colToPivot) = rightHandSide(rowToPivot)
            For eachCol = (eachRow + 1) To rank
                solutions(colToPivot) -= _
                    sourceMatrix(rowToPivot, colPivots(eachCol)) _
                    * solutions(colPivots(eachCol))
            Next eachCol
            solutions(colToPivot) /= sourceMatrix(rowToPivot, colToPivot)
        Next eachRow
    End Sub

    Public Function GaussianElimination(ByVal sourceMatrix(,) As Double) As Double()
        Dim numberOfEquations As Integer = UBound(sourceMatrix, 1) + 1
        Dim numberOfCoefficients As Integer = UBound(sourceMatrix, 2)
        If numberOfCoefficients > numberOfEquations Then Return Nothing Else numberOfCoefficients = numberOfEquations
        Dim destMatrix(numberOfEquations - 1, numberOfCoefficients) As Fraction
        Dim solution(numberOfEquations - 1) As Double

        For r = 0 To numberOfEquations - 1
            For c = 0 To numberOfCoefficients
                destMatrix(r, c) = New Fraction(sourceMatrix(r, c))
            Next
        Next

        For c = 0 To numberOfCoefficients - 1
            ' 1. Testen ob alle Spalten-Einträge Null sind, wenn ja, gehe zur nächsten Zeile
            Dim allRowsZero As Boolean = True
            Dim rowWithNotZero As Integer
            Do
                For r = 0 To numberOfEquations - 1
                    If Not destMatrix(r, c).isZero Then allRowsZero = False : rowWithNotZero = r : Exit Do
                Next
                Continue For
            Loop
            ' 2. Forme um..
            ' 2.1 Wenn M(c, c) Null ist, dann addiere zu M(c) die Zeile M(x) * f, mit M(x,c) <> Null, so dass M(c, c) = 1
            If destMatrix(c, c).isZero Then
                ' Teile M(rowWithNotZero) durch M(rowWithNotZero,c)
                Dim divisor As Fraction = destMatrix(rowWithNotZero, c).Clone
                For c2 = c To numberOfCoefficients
                    destMatrix(rowWithNotZero, c2) /= divisor
                Next
                ' Bringe M(rowWithNotZero,c) auf 1
                If destMatrix(rowWithNotZero, c).isNegative Then
                    For c2 = c To numberOfCoefficients
                        destMatrix(rowWithNotZero, c2).isNegative = Not destMatrix(rowWithNotZero, c2).isNegative
                    Next
                End If
                ' Addiere M(rowWithNotZero) zu M(c)
                For c2 = c To numberOfCoefficients
                    destMatrix(c, c2) = destMatrix(c, c2).Clone + destMatrix(rowWithNotZero, c2).Clone
                Next
            Else
                ' Sonst teile M(c) durch M(c,c)
                Dim divisor As Fraction = destMatrix(c, c).Clone
                destMatrix(c, c).upper = New BigInt("1")
                destMatrix(c, c).lower = New BigInt("1")
                For c2 = c + 1 To numberOfCoefficients
                    destMatrix(c, c2) /= divisor
                Next
            End If
            ' 2.2 Bringe nun alle anderen Zeilen M(x,c) mit c <> x auf 0
            For r = 0 To numberOfEquations - 1
                If Not (r = c OrElse destMatrix(r, c).isZero) Then
                    Dim factor As Fraction = destMatrix(r, c).Clone
                    ' Subtrahiere M(c) von M(r)
                    destMatrix(r, c).upper = New BigInt("0")
                    destMatrix(r, c).lower = New BigInt("1")
                    For c2 = c + 1 To numberOfCoefficients
                        destMatrix(r, c2) = destMatrix(r, c2).Clone - (destMatrix(c, c2).Clone * factor)
                    Next
                End If
            Next
        Next

        ' Berechne die Lösung:
        ' Wenn es mehrere gibt, gebe nur eine davon zurück, wenn es keine gibt, gebe Nothing zurück.
        Dim columnsWithZero As Integer = 0
        For c = 0 To numberOfCoefficients - 1
            If destMatrix(numberOfEquations - 1, c).isZero Then
                columnsWithZero += 1
            End If
        Next
        If columnsWithZero = numberOfCoefficients AndAlso Not destMatrix(numberOfEquations - 1, numberOfCoefficients).isZero Then
            Return Nothing
        Else
            For r = 0 To numberOfEquations - 1
                solution(r) = destMatrix(r, numberOfCoefficients).ToDouble
            Next
            Return solution
        End If
    End Function


    Structure Fraction
        Public upper As BigInt
        Public lower As BigInt
        Public isNegative As Boolean

        Shadows Function Clone() As Fraction
            Return New Fraction() With {.upper = Me.upper, .lower = Me.lower, .isNegative = Me.isNegative}
        End Function

        Sub New(ByVal value As Double)
            value = Math.Round(value, 6)
            If value < 0 Then
                isNegative = True
            End If
            Dim sV As String = value.ToString.ToUpperInvariant
            Dim sL As String = "1"
            If sV(0) = CType("-", Char) Then sV = Mid(sV, 2)
            Dim indexOfE As Integer = sV.IndexOf(CType("E", Char))
            Dim indexOfComma As Integer = sV.IndexOf(CType(MainForm.comma, Char))
            Dim rvalue As Decimal = 0
            If indexOfE <> -1 Then
                Dim exponent As Integer
                exponent = CType(Mid(sV, indexOfE + 2), Integer)
                sV = Mid(sV, 1, indexOfE)
                If indexOfComma = -1 Then indexOfComma = sV.Length
                If exponent < 0 Then
                    For i = 2 To sV.Length - indexOfComma
                        sL += "0"
                    Next
                    For i = exponent To -1
                        sL += "0"
                    Next
                Else
                    For i = 2 To sV.Length - indexOfComma
                        sL += "0"
                    Next
                    For i = 1 To exponent
                        sV += "0"
                    Next
                End If
            Else
                If indexOfComma = -1 Then indexOfComma = sV.Length
                For i = 2 To sV.Length - indexOfComma
                    sL += "0"
                Next
            End If
            lower = New BigInt(sL)
            upper = New BigInt(Replace(sV, MainForm.comma, ""))
            'Me.reduce()
        End Sub

        Sub New(ByVal value As Long)
            If value < 0 Then
                isNegative = True
                value *= -1
            End If
            upper = New BigInt(value.ToString)
            lower = New BigInt("1")
        End Sub

        Sub New(ByVal value As ULong)
            upper = New BigInt(value.ToString)
            lower = New BigInt("1")
        End Sub

        Sub New(ByVal value As Short)
            Me.New(CType(value, Long))
        End Sub

        Sub New(ByVal value As Integer)
            Me.New(CType(value, Long))
        End Sub

        Sub New(ByVal value As Single)
            Me.New(CType(value, Double))
        End Sub

        Sub New(ByVal value As Decimal)
            Me.New(CType(value, Double))
        End Sub

        Sub New(ByVal value As Byte)
            Me.New(CType(value, ULong))
        End Sub

        Sub New(ByVal value As UShort)
            Me.New(CType(value, ULong))
        End Sub

        Sub New(ByVal value As UInteger)
            Me.New(CType(value, ULong))
        End Sub

        Public Function isOne() As Boolean
            If isNegative Then
                Return False
            End If
            Return upper = lower
        End Function

        Public Function isZero() As Boolean
            Return upper.isZero
        End Function

        Private Sub reduce()
            Dim a, b, h As New BigInt(0)
            a = upper
            b = lower
            While Not b.isZero
                h = a Mod b
                a = b
                b = h
            End While
            upper = upper \ a
            lower = lower \ a
        End Sub

        Public Shared Operator +(ByVal v1 As Fraction, ByVal v2 As Fraction) As Fraction
            Dim frac As New Fraction()
            If v1.isZero Then
                v1.isNegative = False
                frac = v2.Clone
                Return frac
            End If
            If v2.isZero Then
                v2.isNegative = False
                frac = v1.Clone
                Return frac
            End If
            If v1.lower = v2.lower Then
                frac.lower = v1.lower
            Else
                frac.lower = v1.lower * v2.lower
                v1.upper *= v2.lower
                v2.upper *= v1.lower
                v1.lower = frac.lower
                v2.lower = frac.lower
            End If
            If v1.isNegative Then
                If v2.isNegative Then
                    frac.isNegative = True
                    frac.upper = v2.upper + v1.upper
                Else
                    If v1.upper > v2.upper Then
                        frac.isNegative = True
                        frac.upper = v1.upper - v2.upper
                    Else
                        frac.upper = v2.upper - v1.upper
                    End If
                End If
            Else
                If v2.isNegative Then
                    If v1.upper > v2.upper Then
                        frac.upper = v1.upper - v2.upper
                    Else
                        frac.isNegative = True
                        frac.upper = v2.upper - v1.upper
                    End If
                Else
                    frac.upper = v2.upper + v1.upper
                End If
            End If
            'frac.reduce()
            Return frac
        End Operator

        Public Shared Operator -(ByVal v1 As Fraction, ByVal v2 As Fraction) As Fraction
            v2.isNegative = Not v2.isNegative
            Return v1 + v2
        End Operator

        Public Shared Operator *(ByVal v1 As Fraction, ByVal v2 As Fraction) As Fraction
            Dim frac As New Fraction()
            frac.upper = v1.upper * v2.upper
            frac.lower = v1.lower * v2.lower
            frac.isNegative = v1.isNegative Xor v2.isNegative
            'frac.reduce()
            Return frac
        End Operator

        Public Shared Operator /(ByVal v1 As Fraction, ByVal v2 As Fraction) As Fraction
            Dim frac As New Fraction()
            frac.upper = v1.upper * v2.lower
            frac.lower = v1.lower * v2.upper
            frac.isNegative = v1.isNegative Xor v2.isNegative
            'frac.reduce()
            Return frac
        End Operator

        Public Function ToDouble() As Double
            reduce()
            If isNegative Then
                Return -(upper / lower)
            Else
                Return upper / lower
            End If
        End Function
    End Structure

    Structure BigInt
        Private digits As List(Of Byte)

        Public Sub New(ByVal foo As Byte)
            digits = New List(Of Byte)
        End Sub

        Sub New(ByVal s As String)
            Dim l As Integer = s.Length
            If l Mod 2 = 1 Then
                s = "0" & s : l += 1
            End If
            Dim ba(l \ 2 - 1) As Byte
            For i = 1 To l Step 2
                ba((l - i) \ 2) = CType(Mid(s, i, 2), Byte)
            Next
            digits = New List(Of Byte)
            digits.AddRange(ba)
        End Sub

        Public Shadows Function toString() As String
            Dim sb As New StringBuilder
            For i = 0 To digits.Count - 2
                If digits(i) > 9 Then
                    sb.Insert(0, digits(i))
                Else
                    sb.Insert(0, digits(i))
                    sb.Insert(0, "0")
                End If
            Next
            sb.Insert(0, digits(digits.Count - 1))
            Return sb.ToString
        End Function

        Public Shared Operator +(ByVal v1 As BigInt, ByVal v2 As BigInt) As BigInt
            Dim copy1 As New BigInt(0)
            Dim copy2 As New BigInt(0)
            ' Führende Nullen entfernen
            While v1.digits(v1.digits.Count - 1) = 0
                If v1.digits.Count = 1 Then Exit While
                v1.digits.RemoveAt(v1.digits.Count - 1)
            End While
            While v2.digits(v2.digits.Count - 1) = 0
                If v2.digits.Count = 1 Then Exit While
                v2.digits.RemoveAt(v2.digits.Count - 1)
            End While
            copy1.digits.AddRange(v1.digits)
            If v2.isZero Then
                Return copy1
            End If
            copy2.digits.AddRange(v2.digits)
            If v1.isZero Then
                Return copy2
            End If
            If copy1.digits.Count < copy2.digits.Count Then
                Dim ba(copy2.digits.Count - copy1.digits.Count - 1) As Byte
                copy1.digits.AddRange(ba)
            ElseIf copy2.digits.Count < copy1.digits.Count Then
                Dim ba(copy1.digits.Count - copy2.digits.Count - 1) As Byte
                copy2.digits.AddRange(ba)
            End If
            Dim carry(copy1.digits.Count) As Byte
            For i As Integer = 0 To copy1.digits.Count - 1
                Dim sum As Short = CType(copy1.digits(i) + copy2.digits(i), Short) + carry(i)
                Dim digit As Byte = CType(sum Mod 100, Byte)
                Dim dcarry As Byte = CType((sum - digit) \ 100, Byte)
                copy1.digits(i) = digit
                carry(i + 1) = dcarry
            Next
            If carry(copy1.digits.Count) > 0 Then
                copy1.digits.Add(carry(copy1.digits.Count))
            End If
            Return copy1
        End Operator

        Public Shared Operator -(ByVal v1 As BigInt, ByVal v2 As BigInt) As BigInt
            Dim copy1 As New BigInt(0)
            Dim copy2 As New BigInt(0)
            ' Führende Nullen entfernen
            While v1.digits(v1.digits.Count - 1) = 0
                If v1.digits.Count = 1 Then Exit While
                v1.digits.RemoveAt(v1.digits.Count - 1)
            End While
            While v2.digits(v2.digits.Count - 1) = 0
                If v2.digits.Count = 1 Then Exit While
                v2.digits.RemoveAt(v2.digits.Count - 1)
            End While
            If v1 <= v2 Then
                copy1.digits.Add(0)
                Return copy1
            End If
            copy1.digits.AddRange(v1.digits)
            copy2.digits.AddRange(v2.digits)
            If copy2.digits.Count < copy1.digits.Count Then
                Dim ba(copy1.digits.Count - copy2.digits.Count - 1) As Byte
                copy2.digits.AddRange(ba)
            End If
            For i = copy1.digits.Count - 1 To 0 Step -1
                If copy1.digits(i) < copy2.digits(i) Then
                    Dim j As Integer = i
                    Dim steps As Integer = 0
                    While copy1.digits(j + 1) = 0
                        j += 1
                        steps += 1
                    End While
                    copy1.digits(j + 1) -= CType(1, Byte)
                    If j <> i Then
                        While steps > 0 AndAlso copy1.digits(j) = 0
                            copy1.digits(j) = 99
                            j -= 1
                            steps -= 1
                        End While
                    End If
                    copy1.digits(i) = CType(100 - copy2.digits(i) + copy1.digits(i), Byte)
                Else
                    copy1.digits(i) -= copy2.digits(i)
                End If
            Next
            For i = copy1.digits.Count - 1 To 0 Step -1
                If copy1.digits(i) = 0 Then
                    If 0 <> i Then
                        copy1.digits.RemoveAt(i)
                    Else
                        copy1.digits(0) = CType(0, Byte)
                        Exit For
                    End If
                Else
                    Exit For
                End If
            Next
            Return copy1
        End Operator

        Public Shared Operator *(ByVal v1 As BigInt, ByVal v2 As BigInt) As BigInt
            Dim sum As New BigInt(0)
            ' Führende Nullen entfernen
            While v1.digits(v1.digits.Count - 1) = 0
                If v1.digits.Count = 1 Then Exit While
                v1.digits.RemoveAt(v1.digits.Count - 1)
            End While
            While v2.digits(v2.digits.Count - 1) = 0
                If v2.digits.Count = 1 Then Exit While
                v2.digits.RemoveAt(v2.digits.Count - 1)
            End While
            sum.digits.Add(0)
            If v1.digits.Count < v2.digits.Count Then
                For i = 0 To v1.digits.Count - 1
                    sum += mult(v2, v1.digits(i), i)
                Next
            Else
                For i = 0 To v2.digits.Count - 1
                    sum += mult(v1, v2.digits(i), i)
                Next
            End If
            Return sum
        End Operator

        Private Shared Function mult(ByVal vlong As BigInt, ByVal number As Byte, ByVal exponent As Integer) As BigInt
            Dim carry(vlong.digits.Count + 1) As Byte
            Dim copy As New BigInt(0)
            If number = 0 Then
                copy.digits.Add(0)
                Return copy
            ElseIf number = 1 Then
                copy.digits.AddRange(vlong.digits)
                If exponent > 0 Then
                    Dim ba(exponent - 1) As Byte
                    copy.digits.InsertRange(0, ba)
                End If
                Return copy
            End If
            copy.digits.AddRange(vlong.digits)
            For i As Integer = 0 To copy.digits.Count - 1
                Dim prod As Short = CType(number, Short) * copy.digits(i)
                Dim digit As Byte = CType(prod Mod 100, Byte)
                Dim dcarry As Byte = CType((prod - digit) \ 100, Byte)
                copy.digits(i) = digit
                carry(i + 1) = dcarry
            Next
            For i As Integer = 1 To copy.digits.Count - 1
                If carry(i) > 0 Then
                    Dim prod As Byte = copy.digits(i) + carry(i)
                    Dim digit As Byte = CType(prod Mod 100, Byte)
                    Dim dcarry As Byte = CType((prod - digit) \ 100, Byte)
                    copy.digits(i) = digit
                    carry(i + 1) += dcarry
                    carry(i) = 0
                End If
            Next
            If carry(copy.digits.Count) > 0 Then
                copy.digits.Add(carry(copy.digits.Count))
                carry(copy.digits.Count - 1) = 0
            End If
            If exponent > 0 Then
                Dim ba(exponent - 1) As Byte
                copy.digits.InsertRange(0, ba)
            End If
            Return copy
        End Function

        Public Shared Operator \(ByVal v1 As BigInt, ByVal v2 As BigInt) As BigInt
            Dim sum As New BigInt(0)
            ' Führende Nullen entfernen
            While v1.digits(v1.digits.Count - 1) = 0
                If v1.digits.Count = 1 Then Exit While
                v1.digits.RemoveAt(v1.digits.Count - 1)
            End While
            While v2.digits(v2.digits.Count - 1) = 0
                If v2.digits.Count = 1 Then Exit While
                v2.digits.RemoveAt(v2.digits.Count - 1)
            End While
            If v1.digits.Count < v2.digits.Count OrElse v1 < v2 OrElse v1.isZero Then
                sum.digits.Add(0)
                Return sum
            ElseIf v1 = v2 Then
                sum.digits.Add(1)
                Return sum
            Else
                sum.digits.Add(0)
                Dim copy2 As New BigInt(0)
                Dim delta As New BigInt(0)
                delta.digits.Add(1)
                copy2.digits.AddRange(v2.digits)
                delta.shiftLeft(v1.digits.Count - 1)
                While delta.digits.Count > 0
                    While (copy2 * sum) < v1
                        sum += delta
                    End While
                    If (copy2 * sum) > v1 Then
                        sum -= delta
                    Else
                        Return sum
                    End If
                    delta.shiftRight()
                End While
            End If
            Return sum
        End Operator

        Public Shared Operator /(ByVal v1 As BigInt, ByVal v2 As BigInt) As Double
            ' Führende Nullen entfernen
            While v1.digits(v1.digits.Count - 1) = 0
                If v1.digits.Count = 1 Then Exit While
                v1.digits.RemoveAt(v1.digits.Count - 1)
            End While
            While v2.digits(v2.digits.Count - 1) = 0
                If v2.digits.Count = 1 Then Exit While
                v2.digits.RemoveAt(v2.digits.Count - 1)
            End While
            Dim d1 As Double
            Dim d2 As Double
            Dim start As Integer = 0
            Dim ende As Integer = 0
            If v1.digits.Count < 307 Then
                ende = v1.digits.Count - 1
                If ende > 20 Then start = ende - 20
                For i As Integer = start To ende
                    d1 += v1.digits(i) * 100 ^ i
                Next
            Else
                d1 = Double.MaxValue
            End If
            If v2.digits.Count < 307 Then
                start = 0
                ende = v2.digits.Count - 1
                If ende > 20 Then start = ende - 20
                For i As Integer = start To ende
                    d2 += v2.digits(i) * 100 ^ i
                Next
            Else
                d2 = Double.MaxValue
            End If
            Return d1 / d2
        End Operator

        Public Shared Operator Mod(ByVal v1 As BigInt, ByVal v2 As BigInt) As BigInt
            ' Führende Nullen entfernen
            While v1.digits(v1.digits.Count - 1) = 0
                If v1.digits.Count = 1 Then Exit While
                v1.digits.RemoveAt(v1.digits.Count - 1)
            End While
            While v2.digits(v2.digits.Count - 1) = 0
                If v2.digits.Count = 1 Then Exit While
                v2.digits.RemoveAt(v2.digits.Count - 1)
            End While
            Dim rest As New BigInt(0)
            rest = v1 \ v2
            rest = v1 - (rest * v2)
            Return rest
        End Operator

        Public Shared Operator =(ByVal v1 As BigInt, ByVal v2 As BigInt) As Boolean
            ' Führende Nullen entfernen
            While v1.digits(v1.digits.Count - 1) = 0
                If v1.digits.Count = 1 Then Exit While
                v1.digits.RemoveAt(v1.digits.Count - 1)
            End While
            While v2.digits(v2.digits.Count - 1) = 0
                If v2.digits.Count = 1 Then Exit While
                v2.digits.RemoveAt(v2.digits.Count - 1)
            End While
            If v1.digits.Count = v2.digits.Count Then
                For i As Integer = 0 To v1.digits.Count - 1
                    If Not v1.digits(i) = v2.digits(i) Then
                        Return False
                    End If
                Next
                Return True
            Else
                Return False
            End If
        End Operator

        Public Shared Operator <>(ByVal v1 As BigInt, ByVal v2 As BigInt) As Boolean
            Return Not v1 = v2
        End Operator

        Public Shared Operator >=(ByVal v1 As BigInt, ByVal v2 As BigInt) As Boolean
            Return v1 > v2 OrElse v1 = v2
        End Operator

        Public Shared Operator <=(ByVal v1 As BigInt, ByVal v2 As BigInt) As Boolean
            Return v1 < v2 OrElse v1 = v2
        End Operator

        Public Shared Operator >(ByVal v1 As BigInt, ByVal v2 As BigInt) As Boolean
            Return v2 < v1
        End Operator

        Public Shared Operator <(ByVal v1 As BigInt, ByVal v2 As BigInt) As Boolean
            ' Führende Nullen entfernen
            While v1.digits(v1.digits.Count - 1) = 0
                If v1.digits.Count = 1 Then Exit While
                v1.digits.RemoveAt(v1.digits.Count - 1)
            End While
            While v2.digits(v2.digits.Count - 1) = 0
                If v2.digits.Count = 1 Then Exit While
                v2.digits.RemoveAt(v2.digits.Count - 1)
            End While
            Dim v1c As Integer = v1.digits.Count - 1
            Dim v2c As Integer = v2.digits.Count - 1
            If v1c < v2c Then
                Return True
            ElseIf v1c > v2c Then
                Return False
            Else
                While v1c > -1 AndAlso v1.digits(v1c) = v2.digits(v1c)
                    v1c -= 1
                End While
                Return v1c > -1 AndAlso v1.digits(v1c) < v2.digits(v1c)
            End If
        End Operator

        Public Function isZero() As Boolean
            Return digits.Count = 1 AndAlso digits(0) = 0
        End Function

        Public Function isOne() As Boolean
            Return digits.Count = 1 AndAlso digits(0) = 1
        End Function

        Public Sub shiftLeft(Optional ByVal c As Integer = 1)
            If c > 0 Then
                Dim ba(c - 1) As Byte
                digits.InsertRange(0, ba)
            End If
        End Sub

        Public Sub shiftRight(Optional ByVal c As Integer = 1)
            If c > 0 Then digits.RemoveRange(0, c)
        End Sub
    End Structure

End Module



'Public Shared Operator +(ByVal v1 As Fraction, ByVal v2 As Fraction) As Fraction
'    Dim frac As New Fraction()
'    If v1.isZero Then
'        v1.isNegative = False
'        frac = v2.Clone
'        Return frac
'    End If
'    If v2.isZero Then
'        v2.isNegative = False
'        frac = v1.Clone
'        Return frac
'    End If
'    If v1.lower = v2.lower Then
'        frac.lower = v1.lower
'        If Not v1.isNegative AndAlso Not v2.isNegative Then
'            frac.upper = v2.upper + v1.upper
'        ElseIf Not v1.isNegative AndAlso v2.isNegative Then
'            If v1.upper > v2.upper Then
'                frac.upper = v1.upper - v2.upper
'            Else
'                frac.isNegative = True
'                frac.upper = v2.upper - v1.upper
'            End If
'        ElseIf v1.isNegative AndAlso Not v2.isNegative Then
'            If v1.upper > v2.upper Then
'                frac.isNegative = True
'                frac.upper = v1.upper - v2.upper
'            Else
'                frac.upper = v2.upper - v1.upper
'            End If
'        ElseIf v1.isNegative AndAlso v2.isNegative Then
'            frac.isNegative = True
'            frac.upper = v2.upper + v1.upper
'        End If
'    Else
'        frac.lower = v1.lower * v2.lower
'        v1.upper *= v2.lower
'        v2.upper *= v1.lower
'        v1.lower = frac.lower
'        v2.lower = frac.lower
'        If Not v1.isNegative AndAlso Not v2.isNegative Then
'            frac.upper = v2.upper + v1.upper
'        ElseIf Not v1.isNegative AndAlso v2.isNegative Then
'            If v1.upper > v2.upper Then
'                frac.upper = v1.upper - v2.upper
'            Else
'                frac.isNegative = True
'                frac.upper = v2.upper - v1.upper
'            End If
'        ElseIf v1.isNegative AndAlso Not v2.isNegative Then
'            If v1.upper > v2.upper Then
'                frac.isNegative = True
'                frac.upper = v1.upper - v2.upper
'            Else
'                frac.upper = v2.upper - v1.upper
'            End If
'        ElseIf v1.isNegative AndAlso v2.isNegative Then
'            frac.isNegative = True
'            frac.upper = v2.upper + v1.upper
'        End If
'    End If
'    frac.reduce()
'    Return frac
'End Operator

'Class BigInt
'    Private digits As New List(Of Byte) With {.Capacity = 100}

'    Sub New()

'    End Sub

'    Sub New(ByVal s As String)
'        Dim l As Integer = s.Length
'        Dim ba(l - 1) As Byte
'        For i = l To 1 Step (-1)
'            ba(l - i) = CType(Mid(s, i, 1), Byte)
'        Next
'        digits.AddRange(ba)
'    End Sub

'    Public Shadows Function toString() As String
'        Dim sb As New StringBuilder
'        For i = 0 To digits.Count - 1
'            sb.Insert(0, digits(i))
'        Next
'        Return sb.ToString
'    End Function

'    Public Shared Operator +(ByVal v1 As BigInt, ByVal v2 As BigInt) As BigInt
'        Dim copy1 As New BigInt
'        Dim copy2 As New BigInt
'        copy1.digits.AddRange(v1.digits)
'        If v2.isZero Then
'            Return copy1
'        End If
'        copy2.digits.AddRange(v2.digits)
'        If v1.isZero Then
'            Return copy2
'        End If
'        If copy1.digits.Count < copy2.digits.Count Then
'            Dim ba(copy2.digits.Count - copy1.digits.Count - 1) As Byte
'            copy1.digits.AddRange(ba)
'        End If
'        If copy2.digits.Count < copy1.digits.Count Then
'            Dim ba(copy1.digits.Count - copy2.digits.Count - 1) As Byte
'            copy2.digits.AddRange(ba)
'        End If
'        Dim carry(copy1.digits.Count) As Byte
'        For i As Integer = 0 To copy1.digits.Count - 1
'            Dim sum As Byte = copy1.digits(i) + copy2.digits(i) + carry(i)
'            Dim digit As Byte = CType(sum Mod 10, Byte)
'            Dim dcarry As Byte = CType((sum - digit) \ 10, Byte)
'            copy1.digits(i) = digit
'            carry(i + 1) = dcarry
'        Next
'        If carry(copy1.digits.Count) > 0 Then
'            copy1.digits.Add(carry(copy1.digits.Count))
'        End If
'        Return copy1
'    End Operator

'    Public Shared Operator -(ByVal v1 As BigInt, ByVal v2 As BigInt) As BigInt
'        Dim copy1 As New BigInt()
'        Dim copy2 As New BigInt()
'        If v1 <= v2 Then
'            copy1.digits.Add(0)
'            Return copy1
'        End If
'        copy1.digits.AddRange(v1.digits)
'        copy2.digits.AddRange(v2.digits)
'        If copy2.digits.Count < copy1.digits.Count Then
'            Dim ba(copy1.digits.Count - copy2.digits.Count - 1) As Byte
'            copy2.digits.AddRange(ba)
'        End If
'        For i = copy1.digits.Count - 1 To 0 Step -1
'            If copy1.digits(i) < copy2.digits(i) Then
'                Dim j As Integer = i
'                Dim steps As Integer = 0
'                While copy1.digits(j + 1) = 0
'                    j += 1
'                    steps += 1
'                End While
'                copy1.digits(j + 1) -= CType(1, Byte)
'                If j <> i Then
'                    While steps > 0 AndAlso copy1.digits(j) = 0
'                        copy1.digits(j) = 9
'                        j -= 1
'                        steps -= 1
'                    End While
'                End If
'                copy1.digits(i) = CType(10 - copy2.digits(i) + copy1.digits(i), Byte)
'            Else
'                copy1.digits(i) -= copy2.digits(i)
'            End If
'        Next
'        For i = copy1.digits.Count - 1 To 0 Step -1
'            If copy1.digits(i) = 0 Then
'                If 0 <> i Then
'                    copy1.digits.RemoveAt(i)
'                Else
'                    copy1.digits(0) = CType(0, Byte)
'                    Exit For
'                End If
'            Else
'                Exit For
'            End If
'        Next
'        Return copy1
'    End Operator

'    Public Shared Operator *(ByVal v1 As BigInt, ByVal v2 As BigInt) As BigInt
'        Dim sum As New BigInt()
'        sum.digits.Add(0)
'        If v1.digits.Count < v2.digits.Count Then
'            For i = 0 To v1.digits.Count - 1
'                sum += mult(v2, v1.digits(i), i)
'            Next
'        Else
'            For i = 0 To v2.digits.Count - 1
'                sum += mult(v1, v2.digits(i), i)
'            Next
'        End If
'        Return sum
'    End Operator

'    Private Shared Function mult(ByVal vlong As BigInt, ByVal number As Byte, ByVal exponent As Integer) As BigInt
'        Dim carry(vlong.digits.Count + 1) As Byte
'        Dim copy As New BigInt
'        If number = 0 Then
'            copy.digits.Add(0)
'            Return copy
'        ElseIf number = 1 Then
'            copy.digits.AddRange(vlong.digits)
'            If exponent > 0 Then
'                Dim ba(exponent - 1) As Byte
'                copy.digits.InsertRange(0, ba)
'            End If
'            Return copy
'        End If
'        copy.digits.AddRange(vlong.digits)
'        For i As Integer = 0 To copy.digits.Count - 1
'            Dim prod As Byte = number * copy.digits(i)
'            Dim digit As Byte = CType(prod Mod 10, Byte)
'            Dim dcarry As Byte = CType((prod - digit) \ 10, Byte)
'            copy.digits(i) = digit
'            carry(i + 1) = dcarry
'        Next
'        Dim newCarry As Boolean = True
'        While newCarry
'            newCarry = False
'            For i As Integer = 1 To copy.digits.Count - 1
'                If carry(i) > 0 Then
'                    Dim prod As Byte = copy.digits(i) + carry(i)
'                    Dim digit As Byte = CType(prod Mod 10, Byte)
'                    Dim dcarry As Byte = CType((prod - digit) \ 10, Byte)
'                    copy.digits(i) = digit
'                    carry(i + 1) += dcarry
'                    carry(i) = 0
'                    If dcarry > 0 Then newCarry = True
'                End If
'            Next
'            If carry(copy.digits.Count) > 0 Then
'                copy.digits.Add(carry(copy.digits.Count))
'                carry(copy.digits.Count - 1) = 0
'            End If
'        End While
'        If exponent > 0 Then
'            Dim ba(exponent - 1) As Byte
'            copy.digits.InsertRange(0, ba)
'        End If
'        Return copy
'    End Function

'    Public Shared Operator \(ByVal v1 As BigInt, ByVal v2 As BigInt) As BigInt
'        Dim sum As New BigInt
'        If v1.digits.Count < v2.digits.Count OrElse v1 < v2 OrElse v1.isZero Then
'            sum.digits.Add(0)
'            Return sum
'        ElseIf v1 = v2 Then
'            sum.digits.Add(1)
'            Return sum
'        Else
'            Dim copy2 As New BigInt
'            Dim delta As New BigInt
'            delta.digits.Add(1)
'            copy2.digits.AddRange(v2.digits)
'            delta.shiftLeft(v1.digits.Count - 1)
'            While delta.digits.Count > 0
'                While (copy2 * sum) < v1
'                    sum += delta
'                End While
'                If (copy2 * sum) > v1 Then
'                    sum -= delta
'                Else
'                    Exit While
'                End If
'                delta.shiftRight()
'            End While
'        End If
'        Return sum
'    End Operator

'    Public Shared Operator /(ByVal v1 As BigInt, ByVal v2 As BigInt) As Double
'        Dim d1 As Double
'        Dim d2 As Double
'        Dim start As Integer = 0
'        Dim ende As Integer = 0
'        If v1.digits.Count < 307 Then
'            ende = v1.digits.Count - 1
'            If ende > 20 Then start = ende - 20
'            For i As Integer = start To ende
'                d1 += v1.digits(i) * 10 ^ i
'            Next
'        Else
'            d1 = Double.MaxValue
'        End If
'        If v2.digits.Count < 307 Then
'            start = 0
'            ende = v2.digits.Count - 1
'            If ende > 20 Then start = ende - 20
'            For i As Integer = start To ende
'                d2 += v2.digits(i) * 10 ^ i
'            Next
'        Else
'            d2 = Double.MaxValue
'        End If
'        Return d1 / d2
'    End Operator

'    Public Shared Operator Mod(ByVal v1 As BigInt, ByVal v2 As BigInt) As BigInt
'        Dim rest As New BigInt
'        rest = v1 \ v2
'        rest = v1 - (rest * v2)
'        Return rest
'    End Operator

'    Public Shared Operator =(ByVal v1 As BigInt, ByVal v2 As BigInt) As Boolean
'        If v1.digits.Count = v2.digits.Count Then
'            For i As Integer = 0 To v1.digits.Count - 1
'                If Not v1.digits(i) = v2.digits(i) Then
'                    Return False
'                End If
'            Next
'            Return True
'        Else
'            Return False
'        End If
'    End Operator

'    Public Shared Operator <>(ByVal v1 As BigInt, ByVal v2 As BigInt) As Boolean
'        Return Not v1 = v2
'    End Operator

'    Public Shared Operator >=(ByVal v1 As BigInt, ByVal v2 As BigInt) As Boolean
'        Return v1 > v2 OrElse v1 = v2
'    End Operator

'    Public Shared Operator <=(ByVal v1 As BigInt, ByVal v2 As BigInt) As Boolean
'        Return v1 < v2 OrElse v1 = v2
'    End Operator

'    Public Shared Operator >(ByVal v1 As BigInt, ByVal v2 As BigInt) As Boolean
'        Return v2 < v1
'    End Operator

'    Public Shared Operator <(ByVal v1 As BigInt, ByVal v2 As BigInt) As Boolean
'        Dim v1c As Integer = v1.digits.Count - 1
'        Dim v2c As Integer = v2.digits.Count - 1
'        If v1c < v2c Then
'            Return True
'        ElseIf v1c > v2c Then
'            Return False
'        Else
'            While v1c > -1 AndAlso v1.digits(v1c) = v2.digits(v1c)
'                v1c -= 1
'            End While
'            Return v1c > -1 AndAlso v1.digits(v1c) < v2.digits(v1c)
'        End If
'    End Operator

'    Public Function isZero() As Boolean
'        Return digits.Count = 1 AndAlso digits(0) = 0
'    End Function

'    Public Sub shiftLeft(Optional ByVal c As Integer = 1)
'        If c > 0 Then
'            Dim ba(c - 1) As Byte
'            digits.InsertRange(0, ba)
'        End If
'    End Sub

'    Public Sub shiftRight(Optional ByVal c As Integer = 1)
'        If c > 0 Then digits.RemoveRange(0, c)
'    End Sub
'End Class
