﻿' MIT - Licence
'
' Copyright (c) 2010 - 2012 Nils Schmidt
' This program uses Rectifier.exe by permission of the author and copyright holder Philippe E. Hurbain - (C) 2009

' Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"),
' to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
' and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

' The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
' INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
' PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
' FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
' ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

Option Strict Off
Imports System.Text
Imports System.IO
Imports System.Runtime.Serialization
Imports System.Runtime.Serialization.Formatters.Binary
Imports System.Globalization

Imports System.Threading
Imports System.ComponentModel


Public Class MainForm


    Declare Auto Sub mouse_event Lib "user32" (ByVal dwFlags As Int32, ByVal dx As Int32, ByVal dy As Int32, ByVal cButtons As Int32, ByVal dwExtraInfo As IntPtr)

    Const MOUSEEVENTF_MOVE As Int32 = &H1 '  mouse move
    Const MOUSEEVENTF_LEFTDOWN As Int32 = &H2 '  left button down
    Const MOUSEEVENTF_LEFTUP As Int32 = &H4 '  left button up
    Const MOUSEEVENTF_RIGHTDOWN As Int32 = &H8 '  right button down
    Const MOUSEEVENTF_RIGHTUP As Int32 = &H10 '  right button up
    Const MOUSEEVENTF_MIDDLEDOWN As Int32 = &H20 '  middle button down
    Const MOUSEEVENTF_MIDDLEUP As Int32 = &H40 '  middle button up
    Const MOUSEEVENTF_ABSOLUTE As Int32 = &H8000 '  absolute move

    Dim cur_x As Single
    Dim cur_y As Single
    Dim dest_x As Single
    Dim dest_y As Single
    Dim p_old As Point

    Dim appPath As String = System.AppDomain.CurrentDomain.BaseDirectory
    Dim ldrawPath As String = ""
    Public comma As String = Thread.CurrentThread.CurrentCulture.NumberFormat.NumberDecimalSeparator

#Region "Data Structure"
    Const twoPI As Double = Math.PI * 2
    Dim auswahl, kamera, raster, movemode, rotatemode, scalemode, isLoading, isSolving, adjustmode, doAdjust As Boolean
    Dim klickX As Integer
    Dim klickY As Integer

    Dim keylock As Boolean

    Dim segments_circle As Integer = 16
    Dim segments_oval As Integer = 16
    Dim segments_arch As Integer = 4

    Dim temp_center As Vertex
    Dim temp_vertices(1) As Vertex

    Public Shared vertexIDglobal As Integer = 0
    Public Shared triangleIDglobal As Integer = 0
    Public Shared primitiveIDglobal As Integer = 0

    Public zoomlevel As Integer
    Public zoomfactor As Single = 2

    Dim punktgroesse As Integer = 4

    Dim toffsetX As Single
    Dim toffsetY As Single
    Public offsetX As Single
    Public offsetY As Single

    Dim lastPointX As Single
    Dim lastPointY As Single

    Dim lastColour As Color = Nothing
    Dim lastColourNumber As Single = 16
    Public maxColourNumber As Single
    Dim alpha As Byte = 255

    Dim moveSnap As Single = 1
    Dim rasterSnap As Single = 1
    Dim rotateSnap As Single = 1
    Dim scaleSnap As Single = 0.01F

    Dim imgOffsetX As Integer = 0
    Dim imgOffsetY As Integer = 0
    Dim imgScale As Single = 1

    Dim bearbeitungsmodus As Byte = editMode.Selection
    Dim objectToModify As String = "Vertex"

    Dim trianglemode As Byte = 0

    Public colourHMap As New Dictionary(Of Short, Color)
    Public colourHMapName As New Dictionary(Of Short, String)

    Dim backgroundPicture As Bitmap = My.Resources.temp

    Public Vertices As New List(Of Vertex)
    Dim ClipboardVertices As New List(Of Vertex)
    Public Triangles As New List(Of Triangle)
    Dim ClipboardTriangles As New List(Of Triangle)
    Public Primitives As New List(Of Primitive)
    Dim ClipboardPrimitives As New List(Of Primitive)
    Public PrimitivesHMap As New Hashtable(1000)
    Public PrimitivesMetadataHMap As New Hashtable(100)
    Public includeMetadata As Boolean = False

    Dim historyVertices As New List(Of List(Of Vertex))
    Dim historyTriangles As New List(Of List(Of Triangle))
    Dim historySelectedVertices As New List(Of List(Of Vertex))
    Dim historySelectedTriangles As New List(Of List(Of Triangle))
    Dim historyPrimitives As New List(Of List(Of Primitive))
    Dim historyPrimitivesHash As New List(Of Hashtable)
    Dim historyPrimitivesMetadataHash As New List(Of Hashtable)
    Dim historyTemplateShape As New List(Of List(Of PointF))
    Dim historyMyMetadata As New List(Of Metadata)
    Dim historyProjectionQuads As New List(Of List(Of ProjectionQuad))
    Dim historyTemplateText As New List(Of List(Of TemplateTextInfo))

    Public SelectedVertices As New List(Of Vertex)
    Public SelectedTriangles As New List(Of Triangle)

    Public myMetadata As New Metadata() With {.isMainMetadata = True}
    Dim templateShape As New List(Of PointF)
    Dim templateProjectionQuad As New List(Of ProjectionQuad)
    Dim templateText As New List(Of TemplateTextInfo)

    Dim alreadyCalculatedQuads As Integer

    Dim offset_X As Integer = 0
    Dim offset_Y As Integer = 30
    Dim getOffset As Boolean = False

    Dim unsavedChanges As Boolean = False

    Public tempvertex As New Vertex(0, 0, False, False)
    Public tempcolour As Color

    Public myKeys As New _KeySetting(True)
    Public mySettings As New _GeneralSetting(True)
    Public myColours As New _SystemColours(True)

    Private adjustDirection As Single = 0
    Private tempAdjustDirection As Single = 0

    Structure _KeySetting
        Public ModeSelect As Integer
        Public ModeMove As Integer
        Public ModeRotate As Integer
        Public ModeScale As Integer
        Public AddVertex As Integer
        Public AddTriangle As Integer
        Public Abort As Integer
        Public Pipette As Integer
        Public Preview As Integer
        Public ShowColours As Integer
        Public Zoom As Integer
        Public Translate As Integer
        Public Sub New(ByVal newSetting As Boolean)
            ModeSelect = Keys.D1
            ModeMove = Keys.D2
            ModeRotate = Keys.D3
            ModeScale = Keys.D4
            AddVertex = Keys.D5
            AddTriangle = Keys.D6
            Abort = Keys.Escape
            Pipette = Keys.O
            Preview = Keys.P
            ShowColours = Keys.Space
            Zoom = Keys.K
            Translate = Keys.L
        End Sub
    End Structure

    Structure _GeneralSetting
        Public defaultName As String
        Public defaultUser As String
        Public defaultLicense As String
        Public max_undo As Byte
        Public performanceMode As Boolean
        Public startWithFullscreen As Boolean
        Public showImageViewAtStartup As Boolean
        Public showPreferencesViewAtStartup As Boolean
        Public useAlternativeKeys As Boolean
        Public Sub New(ByVal newSetting As Boolean)
            defaultName = "" : defaultUser = "" : defaultLicense = "" : max_undo = 30 : performanceMode = False : startWithFullscreen = True
            showImageViewAtStartup = True : showPreferencesViewAtStartup = True : useAlternativeKeys = False
        End Sub
    End Structure

    Structure _SystemColours
        Public background As Color
        Public linePen As Pen
        Public inverseLinePen As Pen
        Public selectedLinePen As Pen
        Public selectedLineInVertexModePen As Pen
        Public vertexBrush As SolidBrush
        Public selectedVertexBrush As SolidBrush
        Public originPen As Pen
        Public gridPen As Pen
        Public grid10Pen As Pen
        Public selectionRectPen As Pen
        Public selectionCrossBrush As SolidBrush
        Public Sub New(ByVal newSetting As Boolean)
            background = Color.FromArgb(0, 0, 0)
            linePen = New Pen(Color.FromArgb(234, 244, 234), 1)
            inverseLinePen = New Pen(Color.FromArgb(12, 12, 12), 1)
            selectedLinePen = New Pen(Color.FromArgb(245, 12, 12), 1)
            selectedLineInVertexModePen = New Pen(Color.FromArgb(245, 180, 180), 1)
            vertexBrush = New SolidBrush(Color.FromArgb(245, 245, 12))
            selectedVertexBrush = New SolidBrush(Color.FromArgb(245, 12, 12))
            originPen = New Pen(Color.FromArgb(140, 215, 140), 1)
            gridPen = New Pen(Color.FromArgb(0, 60, 0), 1)
            grid10Pen = New Pen(Color.FromArgb(60, 60, 105), 1)
            selectionRectPen = New Pen(Color.FromArgb(245, 245, 220), 1)
            selectionCrossBrush = New SolidBrush(Color.FromArgb(120, 0, 0))
        End Sub
    End Structure

    <Serializable()> _
    Public Class Vertex
        Implements System.Runtime.Serialization.ISerializable
        Implements IComparable(Of Vertex)
        Implements IEquatable(Of Vertex)

        Public X As Single
        Public Y As Single
        Public oldX As Single
        Public oldY As Single
        Public selected As Boolean

        Public angleFrom As Double
        Public distanceFrom As Double

        Public linkedTriangles As New List(Of Triangle)

        Public groupindex As Integer

        Public vertexID As Integer

        Overloads Function Equals(ByVal other As Vertex) As Boolean Implements IEquatable(Of Vertex).Equals
            Return Me.vertexID = other.vertexID
        End Function


        Private Sub ISerializable_GetObjectData(ByVal oInfo As SerializationInfo, ByVal oContext As StreamingContext) Implements ISerializable.GetObjectData
            oInfo.AddValue("X", Me.X, Me.X.GetType)
            oInfo.AddValue("Y", Me.Y, Me.Y.GetType)
            oInfo.AddValue("selected", Me.selected, Me.selected.GetType)
            oInfo.AddValue("linkedTriangles", Me.linkedTriangles, Me.linkedTriangles.GetType)
            oInfo.AddValue("groupindex", Me.groupindex, Me.groupindex.GetType)
            oInfo.AddValue("vertexID", Me.vertexID, Me.vertexID.GetType)
        End Sub

        Protected Sub New(ByVal info As System.Runtime.Serialization.SerializationInfo, ByVal context As System.Runtime.Serialization.StreamingContext)
            Me.X = info.GetSingle("X")
            Me.Y = info.GetSingle("Y")
            Me.selected = info.GetBoolean("selected")
            Me.linkedTriangles = info.GetValue("linkedTriangles", Me.linkedTriangles.GetType)
            Me.groupindex = info.GetInt32("groupindex")
            Me.vertexID = info.GetInt32("vertexID")
        End Sub

        Sub New(ByVal vx As Single, ByVal vy As Single, ByVal sel As Boolean, Optional ByVal needID As Boolean = True)
            Me.X = vx
            Me.Y = vy
            Me.selected = sel
            Me.groupindex = -1
            If needID Then
                vertexIDglobal += 1
                Me.vertexID = vertexIDglobal
            End If
        End Sub

        Public Shared Operator <>(ByVal v1 As Vertex, ByVal v2 As Vertex) As Boolean
            Return v1.X <> v2.X OrElse v1.Y <> v2.Y
        End Operator

        Public Shared Operator =(ByVal v1 As Vertex, ByVal v2 As Vertex) As Boolean
            Return v1.X = v2.X AndAlso v1.Y = v2.Y
        End Operator

        Public Shared Operator +(ByVal v1 As Vertex, ByVal v2 As Vertex) As Vertex
            Return New Vertex(v1.X + v2.X, v1.Y + v2.Y, False, False)
        End Operator

        Public Shared Operator -(ByVal v1 As Vertex, ByVal v2 As Vertex) As Vertex
            Return New Vertex(v1.X - v2.X, v1.Y - v2.Y, False, False)
        End Operator

        Public Shared Operator *(ByVal v1 As Vertex, ByVal v2 As Vertex) As Single
            Return v1.X * v2.X + v1.Y * v2.Y
        End Operator

        Public Shared Operator >(ByVal v1 As Vertex, ByVal v2 As Vertex) As Boolean
            Return v1.angleFrom > v2.angleFrom
        End Operator

        Public Shared Operator >=(ByVal v1 As Vertex, ByVal v2 As Vertex) As Boolean
            Return v1.angleFrom >= v2.angleFrom
        End Operator

        Public Shared Operator <(ByVal v1 As Vertex, ByVal v2 As Vertex) As Boolean
            Return v1.angleFrom < v2.angleFrom
        End Operator

        Public Shared Operator <=(ByVal v1 As Vertex, ByVal v2 As Vertex) As Boolean
            Return v1.angleFrom <= v2.angleFrom
        End Operator

        Public Overloads Function CompareTo(ByVal other As Vertex) As Integer Implements IComparable(Of Vertex).CompareTo
            If Me > other Then
                Return 1
            ElseIf Me < other Then
                Return -1
            Else
                Return 0
            End If
        End Function

        Public Function angle(ByVal v2 As Vertex) As Double
            Dim angle2, tx, ty As Double
            tx = v2.X - Me.X
            ty = v2.Y - Me.Y
            angle2 = Math.Atan2(ty, tx)
            If angle2 < 0 Then angle2 += twoPI
            Return angle2
        End Function

        Public Function MYangle() As Double
            Dim angle2 As Double
            angle2 = Math.Atan2(Me.Y, Me.X)
            If angle2 < 0 Then angle2 += twoPI
            Return angle2
        End Function

        Public Function dist(ByVal v2 As Vertex) As Double
            Return Math.Sqrt((Me.X - v2.X) ^ 2 + (Me.Y - v2.Y) ^ 2)
        End Function

    End Class

    <Serializable()> _
    Public Class Triangle
        Implements System.Runtime.Serialization.ISerializable
        Implements IEquatable(Of Triangle)

        Public vertexA As Vertex
        Public vertexB As Vertex
        Public vertexC As Vertex
        Public selected As Boolean
        Public myColour As Color
        Public myColourNumber As Short = 16
        Public groupindex As Integer
        Public triangleID As Integer

        Overloads Function Equals(ByVal other As Triangle) As Boolean Implements IEquatable(Of Triangle).Equals
            Return Me.triangleID = other.triangleID
        End Function

        'Overloads Function Equals(ByVal objA As Triangle, ByVal objB As Triangle) As Boolean
        '    Return objA.triangleID = objB.triangleID
        'End Function

        'Overloads Function ReferenceEquals(ByVal other As Triangle) As Boolean
        '    Return Me.triangleID = other.triangleID
        'End Function

        Private Sub ISerializable_GetObjectData(ByVal oInfo As SerializationInfo, ByVal oContext As StreamingContext) Implements ISerializable.GetObjectData
            oInfo.AddValue("vertexA", Me.vertexA, MainForm.tempvertex.GetType())
            oInfo.AddValue("vertexB", Me.vertexB, MainForm.tempvertex.GetType())
            oInfo.AddValue("vertexC", Me.vertexC, MainForm.tempvertex.GetType())
            oInfo.AddValue("selected", Me.selected, Me.selected.GetType())
            oInfo.AddValue("myColour", Me.myColour, Me.myColour.GetType())
            oInfo.AddValue("myColourNumber", Me.myColourNumber, Me.myColourNumber.GetType())
            oInfo.AddValue("groupindex", Me.groupindex, Me.groupindex.GetType())
            oInfo.AddValue("triangleID", Me.triangleID, Me.triangleID.GetType())
        End Sub

        Protected Sub New(ByVal info As System.Runtime.Serialization.SerializationInfo, ByVal context As System.Runtime.Serialization.StreamingContext)
            Me.vertexA = info.GetValue("vertexA", MainForm.tempvertex.GetType())
            Me.vertexB = info.GetValue("vertexB", MainForm.tempvertex.GetType())
            Me.vertexC = info.GetValue("vertexC", MainForm.tempvertex.GetType())
            Me.selected = info.GetBoolean("selected")
            Me.myColour = info.GetValue("myColour", MainForm.tempcolour.GetType())
            Me.myColourNumber = info.GetInt16("myColourNumber")
            Me.groupindex = info.GetInt32("groupindex")
            Me.triangleID = info.GetInt32("triangleID")
        End Sub

        Sub New(ByVal v1 As Vertex, ByVal v2 As Vertex, ByVal v3 As Vertex, Optional ByVal needID As Boolean = True)
            Me.vertexA = v1
            Me.vertexB = v2
            Me.vertexC = v3
            Me.groupindex = -1
            If needID Then
                triangleIDglobal += 1
                Me.triangleID = triangleIDglobal
            End If
        End Sub

        Public Sub normalize()
            Dim tmpVertex As Vertex
            tmpVertex = vertexA : If vertexA.X > vertexB.X Then vertexA = vertexB : vertexB = tmpVertex
            tmpVertex = vertexA : If vertexA.X > vertexC.X Then vertexA = vertexC : vertexC = tmpVertex
            tmpVertex = vertexB : If vertexB.X > vertexC.X Then vertexB = vertexC : vertexC = tmpVertex

            tmpVertex = vertexA : If vertexA.Y > vertexB.Y Then vertexA = vertexB : vertexB = tmpVertex
            tmpVertex = vertexA : If vertexA.Y > vertexC.Y Then vertexA = vertexC : vertexC = tmpVertex
            tmpVertex = vertexB : If vertexB.Y > vertexC.Y Then vertexB = vertexC : vertexC = tmpVertex
        End Sub

        Public Function getCenter() As Vertex
            Return New Vertex((vertexA.X + vertexB.X + vertexC.X) / 3, (vertexA.Y + vertexB.Y + vertexC.Y) / 3, False, False)
        End Function

        Public Function getFingerprint() As String
            Dim sb As New StringBuilder()
            sb.Append(vertexA.X)
            sb.Append(vertexA.Y)
            sb.Append(vertexB.X)
            sb.Append(vertexB.Y)
            sb.Append(vertexC.X)
            sb.Append(vertexC.Y)
            Return sb.ToString
        End Function

        Public Sub checkWinding()
            Dim v1 As New Vertex(0, 0, False, False)
            Dim v2 As New Vertex(0, 0, False, False)
            v1 = Me.vertexB - Me.vertexA
            v2 = Me.vertexC - Me.vertexA
            If MainForm.myMetadata.mData(7) = "" OrElse (MainForm.includeMetadata AndAlso (MainForm.myMetadata.mData(7) = "BFC CERTIFY CCW" OrElse MainForm.myMetadata.mData(7) = "BFC NOCERTIFY")) Then
                If (v1.X * v2.Y - v1.Y * v2.X) < 0 Then
                    Dim tv As Vertex
                    tv = Me.vertexB
                    Me.vertexB = Me.vertexC
                    Me.vertexC = tv
                End If
            Else
                If (v1.X * v2.Y - v1.Y * v2.X) >= 0 Then
                    Dim tv As Vertex
                    tv = Me.vertexB
                    Me.vertexB = Me.vertexC
                    Me.vertexC = tv
                End If
            End If
        End Sub

    End Class

    <Serializable()> _
 Public Class ProjectionQuad
        Implements System.Runtime.Serialization.ISerializable

        Public inCoords(3, 2) As Single ' Input Koordinaten des Quads, XY als Basis, Z ist immer 0 und wird im Template nicht angegeben
        Public outCoords(3, 2) As Single ' Ziel Koordinaten (XYZ), auf diese Koordinaten wird der Quad projeziert
        Public matrix(3, 3) As Double

        Private isTriangle As Boolean

        Sub New()
        End Sub

        Sub New(ByVal x1 As Single, ByVal y1 As Single, _
                 ByVal x2 As Single, ByVal y2 As Single, _
                 ByVal x3 As Single, ByVal y3 As Single, _
                 ByVal x4 As Single, ByVal y4 As Single, _
                 ByVal px1 As Single, ByVal py1 As Single, ByVal pz1 As Single, _
                 ByVal px2 As Single, ByVal py2 As Single, ByVal pz2 As Single, _
                 ByVal px3 As Single, ByVal py3 As Single, ByVal pz3 As Single, _
                 ByVal px4 As Single, ByVal py4 As Single, ByVal pz4 As Single)
            inCoords(0, 0) = x1 : inCoords(0, 1) = y1 : inCoords(0, 2) = 0
            inCoords(1, 0) = x2 : inCoords(1, 1) = y2 : inCoords(1, 2) = 0
            isTriangle = inCoords(0, 0) = inCoords(1, 0) AndAlso inCoords(0, 1) = inCoords(1, 1)
            inCoords(2, 0) = x3 : inCoords(2, 1) = y3 : inCoords(2, 2) = 0
            inCoords(3, 0) = x4 : inCoords(3, 1) = y4 : inCoords(3, 2) = 0
            outCoords(0, 0) = px1 : outCoords(0, 1) = py1 : outCoords(0, 2) = pz1
            outCoords(1, 0) = px2 : outCoords(1, 1) = py2 : outCoords(1, 2) = pz2
            outCoords(2, 0) = px3 : outCoords(2, 1) = py3 : outCoords(2, 2) = pz3
            outCoords(3, 0) = px4 : outCoords(3, 1) = py4 : outCoords(3, 2) = pz4
        End Sub

        Overrides Function toString() As String
            Return inCoords(0, 0) & " " & inCoords(0, 1) _
            & " " & inCoords(1, 0) & " " & inCoords(1, 1) _
            & " " & inCoords(2, 0) & " " & inCoords(2, 1) _
            & " " & inCoords(3, 0) & " " & inCoords(3, 1) _
            & " " & outCoords(0, 0) & " " & outCoords(0, 1) & " " & outCoords(0, 2) _
            & " " & outCoords(1, 0) & " " & outCoords(1, 1) & " " & outCoords(1, 2) _
            & " " & outCoords(2, 0) & " " & outCoords(2, 1) & " " & outCoords(2, 2) _
            & " " & outCoords(3, 0) & " " & outCoords(3, 1) & " " & outCoords(3, 2)
        End Function

        Shadows Function Clone() As ProjectionQuad
            Return New ProjectionQuad() With {.inCoords = Me.inCoords.Clone, .outCoords = Me.outCoords.Clone, .matrix = Me.matrix.Clone}
        End Function

        Private Sub ISerializable_GetObjectData(ByVal oInfo As SerializationInfo, ByVal oContext As StreamingContext) Implements ISerializable.GetObjectData
            oInfo.AddValue("matrix", Me.matrix, matrix.GetType())
            oInfo.AddValue("inCoords", Me.inCoords, inCoords.GetType())
            oInfo.AddValue("outCoords", Me.outCoords, outCoords.GetType())
        End Sub

        Protected Sub New(ByVal info As System.Runtime.Serialization.SerializationInfo, ByVal context As System.Runtime.Serialization.StreamingContext)
            Dim tempmatrix(3, 3) As Double
            Dim tempcoords(3, 2) As Single
            Me.matrix = info.GetValue("matrix", tempmatrix.GetType())
            Me.inCoords = info.GetValue("inCoords", tempcoords.GetType())
            Me.outCoords = info.GetValue("outCoords", tempcoords.GetType())
        End Sub

        Public Sub CalculateMatrix()
            ' Create a 4 x 5 - Matrix for each coordinate..
            ' dest_x1 = unknown_matrix(0, 0)  * source_x1 + unknown_matrix(0, 1)  * source_y1 + unknown_matrix(0, 2)  * source_z1 + unknown_matrix(0, 3) * 1

            't(0) = matrix(0, 0) * v(i, 0) + matrix(0, 1) * v(i, 1) + matrix(0, 2) * v(i, 2) + matrix(0, 3)
            't(1) = matrix(1, 0) * v(i, 0) + matrix(1, 1) * v(i, 1) + matrix(1, 2) * v(i, 2) + matrix(1, 3)
            't(2) = matrix(2, 0) * v(i, 0) + matrix(2, 1) * v(i, 1) + matrix(2, 2) * v(i, 2) + matrix(2, 3)
            Dim i_backup(2) As Single
            Dim o_backup(2) As Single
            If isTriangle Then
                For i = 0 To 2
                    i_backup(i) = inCoords(0, i)
                    o_backup(i) = outCoords(0, i)
                    inCoords(0, i) = (inCoords(1, i) + inCoords(3, i)) - inCoords(2, i)
                    outCoords(0, i) = (outCoords(1, i) + outCoords(3, i)) - outCoords(2, i)
                Next
            End If
            Do
                Dim matrix_X(,) As Double = {{inCoords(0, 0), inCoords(0, 1), 0, 1, outCoords(0, 0)}, _
                                             {inCoords(1, 0), inCoords(1, 1), 0, 1, outCoords(1, 0)}, _
                                             {inCoords(2, 0), inCoords(2, 1), 0, 1, outCoords(2, 0)}, _
                                             {inCoords(3, 0), inCoords(3, 1), 0, 1, outCoords(3, 0)}}
                Dim solution_X() As Double = GaussianElimination(matrix_X)
                If solution_X Is Nothing Then Exit Do
                Dim matrix_Y(,) As Double = {{inCoords(0, 0), inCoords(0, 1), 0, 1, outCoords(0, 1)}, _
                                             {inCoords(1, 0), inCoords(1, 1), 0, 1, outCoords(1, 1)}, _
                                             {inCoords(2, 0), inCoords(2, 1), 0, 1, outCoords(2, 1)}, _
                                             {inCoords(3, 0), inCoords(3, 1), 0, 1, outCoords(3, 1)}}
                Dim solution_Y() As Double = GaussianElimination(matrix_Y)
                If solution_Y Is Nothing Then Exit Do
                Dim matrix_Z(,) As Double = {{inCoords(0, 0), inCoords(0, 1), 0, 1, outCoords(0, 2)}, _
                                             {inCoords(1, 0), inCoords(1, 1), 0, 1, outCoords(1, 2)}, _
                                             {inCoords(2, 0), inCoords(2, 1), 0, 1, outCoords(2, 2)}, _
                                             {inCoords(3, 0), inCoords(3, 1), 0, 1, outCoords(3, 2)}}
                Dim solution_Z() As Double = GaussianElimination(matrix_Z)
                If solution_Z Is Nothing Then Exit Do
                For i As Integer = 0 To 3
                    matrix(0, i) = solution_X(i)
                    matrix(1, i) = solution_Y(i)
                    matrix(2, i) = solution_Z(i)
                Next
                matrix(3, 0) = 0 : matrix(3, 1) = 0 : matrix(3, 2) = 0 : matrix(3, 3) = 1
                ' All-Row/All Colum Zero entfernen:
                For r As Integer = 0 To 2
                    Dim rsum As Double
                    For c As Integer = 0 To 2
                        rsum += Math.Abs(matrix(r, c))
                    Next
                    If Math.Abs(rsum) < 0.00001 Then
                        For c As Integer = 0 To 2
                            Dim csum As Double = 0
                            For r2 As Integer = 0 To 2
                                csum += Math.Abs(matrix(r2, c))
                            Next
                            If Math.Abs(csum) < 0.00001 Then
                                matrix(r, c) = 1.0
                            End If
                        Next
                    End If
                Next
                For c As Integer = 0 To 2
                    Dim csum As Double = 0
                    For r2 As Integer = 0 To 2
                        csum += Math.Abs(matrix(r2, c))
                    Next
                    If Math.Abs(csum) < 0.00001 Then
                        matrix(c, c) = 1.0
                    End If
                Next

                If isTriangle Then
                    For i As Integer = 0 To 2
                        inCoords(0, i) = i_backup(i)
                        outCoords(0, i) = o_backup(i)
                    Next
                End If
                Exit Sub
            Loop
            Dim idmatrix(,) As Double = {{1, 0, 0, 0}, {0, 1, 0, 0}, {0, 0, 1, 0}, {0, 0, 0, 1}}
            matrix = idmatrix.Clone
            If isTriangle Then
                For i As Integer = 0 To 2
                    inCoords(0, i) = i_backup(i)
                    outCoords(0, i) = o_backup(i)
                Next
            End If
        End Sub

        Public Function isInQuad(ByVal tri As Triangle)
            If tri.vertexA.X = tri.vertexB.X AndAlso tri.vertexA.Y = tri.vertexB.Y Then Return True
            If tri.vertexA.X = tri.vertexC.X AndAlso tri.vertexA.Y = tri.vertexC.Y Then Return True
            If tri.vertexB.X = tri.vertexC.X AndAlso tri.vertexB.Y = tri.vertexC.Y Then Return True
            Dim tri1(,) As Single = {{inCoords(0, 0), inCoords(0, 1)}, {inCoords(1, 0), inCoords(1, 1)}, {inCoords(2, 0), inCoords(2, 1)}}
            Dim tri2(,) As Single = {{inCoords(2, 0), inCoords(2, 1)}, {inCoords(3, 0), inCoords(3, 1)}, {inCoords(0, 0), inCoords(0, 1)}}
            Dim beamdir() As Decimal = {0, 0, -1}

            Dim center As Vertex = New Vertex((tri.vertexA.X + tri.vertexB.X + tri.vertexC.X) / 3, (tri.vertexA.Y + tri.vertexB.Y + tri.vertexC.Y) / 3, False, False)

            tri.vertexA.angleFrom = tri.vertexA.angle(center) + Math.PI
            tri.vertexA.distanceFrom = tri.vertexA.dist(center)
            tri.vertexB.angleFrom = tri.vertexB.angle(center) + Math.PI
            tri.vertexB.distanceFrom = tri.vertexB.dist(center)
            tri.vertexC.angleFrom = tri.vertexC.angle(center) + Math.PI
            tri.vertexC.distanceFrom = tri.vertexC.dist(center)

            Dim beamorig1() As Decimal = {center.X + 0.98 * tri.vertexA.distanceFrom * Math.Cos(tri.vertexA.angleFrom), center.Y + 0.98 * tri.vertexA.distanceFrom * Math.Sin(tri.vertexA.angleFrom), 0}
            Dim beamorig2() As Decimal = {center.X + 0.98 * tri.vertexB.distanceFrom * Math.Cos(tri.vertexB.angleFrom), center.Y + 0.98 * tri.vertexB.distanceFrom * Math.Sin(tri.vertexB.angleFrom), 0}
            Dim beamorig3() As Decimal = {center.X + 0.98 * tri.vertexC.distanceFrom * Math.Cos(tri.vertexC.angleFrom), center.Y + 0.98 * tri.vertexC.distanceFrom * Math.Sin(tri.vertexC.angleFrom), 0}

            Dim vert00() As Decimal = {tri1(0, 0), tri1(0, 1), 0}
            Dim vert01() As Decimal = {tri1(1, 0), tri1(1, 1), 0}
            Dim vert02() As Decimal = {tri1(2, 0), tri1(2, 1), 0}
            Dim vert10() As Decimal = {tri2(0, 0), tri2(0, 1), 0}
            Dim vert11() As Decimal = {tri2(1, 0), tri2(1, 1), 0}
            Dim vert12() As Decimal = {tri2(2, 0), tri2(2, 1), 0}

            Return (PowerRay.SCHNITTPKT_DREIECK(beamorig1, beamdir, vert00, vert01, vert02) OrElse _
                    PowerRay.SCHNITTPKT_DREIECK(beamorig1, beamdir, vert10, vert11, vert12)) AndAlso _
                   (PowerRay.SCHNITTPKT_DREIECK(beamorig2, beamdir, vert00, vert01, vert02) OrElse _
                    PowerRay.SCHNITTPKT_DREIECK(beamorig2, beamdir, vert10, vert11, vert12)) AndAlso _
                   (PowerRay.SCHNITTPKT_DREIECK(beamorig3, beamdir, vert00, vert01, vert02) OrElse _
                    PowerRay.SCHNITTPKT_DREIECK(beamorig3, beamdir, vert10, vert11, vert12))
        End Function

        Public Function isInQuad(ByVal trilist As List(Of Triangle))
            For i = 0 To trilist.Count - 1
                If Not isInQuad(trilist(i)) Then
                    Return False
                End If
            Next
            Return True
        End Function

    End Class

    <Serializable()> _
    Public Class Primitive
        Implements System.Runtime.Serialization.ISerializable

        Public primitiveID As Integer
        Public centerVertexID As Integer
        Public matrix(3, 3) As Double
        Public primitiveName As String
        Public myColour As Color
        Public myColourNumber As Short = 16
        Public ox As Double
        Public oy As Double

        Sub New(ByVal vx As Double, ByVal vy As Double, ByVal offset_x As Double, ByVal offset_y As Double, ByVal name As String, ByVal centerVertexID As Integer, Optional ByVal needID As Boolean = True)
            Me.matrix(0, 3) = vx
            Me.matrix(1, 3) = vy
            Me.matrix(0, 0) = 1
            Me.matrix(1, 1) = 1
            Me.matrix(2, 2) = -1
            Me.matrix(3, 3) = 1
            Me.ox = offset_x
            Me.oy = offset_y
            If needID Then
                primitiveIDglobal += 1
                Me.primitiveID = primitiveIDglobal
            End If
            Me.primitiveName = name
            Me.centerVertexID = centerVertexID
        End Sub

        Public Sub mirror(ByVal on_x As Boolean, ByVal on_Y As Boolean)
            Dim vert As Vertex = New Vertex(ox, oy, False, False)
            Dim fx As Double = 1
            Dim fy As Double = 1
            Dim tox, toy As Double
            If on_x Then fx = -1
            If on_Y Then fy = -1
            tox = matrix(0, 3)
            toy = matrix(1, 3)
            translate(-matrix(0, 3), matrix(1, 3))
            translate(-ox, oy)

            ' Spiegeln..
            Dim matrix2(,) As Double = {{fx, 0, 0, 0}, _
                                        {0, fy, 0, 0}, _
                                        {0, 0, 1, 0}, _
                                        {0, 0, 0, 1}}
            Dim matrix3(3, 3) As Double

            matrix3(0, 0) = matrix2(0, 0) * matrix(0, 0) + matrix2(0, 1) * matrix(1, 0) + matrix2(0, 2) * matrix(2, 0)
            matrix3(0, 1) = matrix2(0, 0) * matrix(0, 1) + matrix2(0, 1) * matrix(1, 1) + matrix2(0, 2) * matrix(2, 1)
            matrix3(0, 2) = matrix2(0, 0) * matrix(0, 2) + matrix2(0, 1) * matrix(1, 2) + matrix2(0, 2) * matrix(2, 2)
            matrix3(0, 3) = matrix2(0, 0) * matrix(0, 3) + matrix2(0, 1) * matrix(1, 3) + matrix2(0, 2) * matrix(2, 3) + matrix2(0, 3)

            matrix3(1, 0) = matrix2(1, 0) * matrix(0, 0) + matrix2(1, 1) * matrix(1, 0) + matrix2(1, 2) * matrix(2, 0)
            matrix3(1, 1) = matrix2(1, 0) * matrix(0, 1) + matrix2(1, 1) * matrix(1, 1) + matrix2(1, 2) * matrix(2, 1)
            matrix3(1, 2) = matrix2(1, 0) * matrix(0, 2) + matrix2(1, 1) * matrix(1, 2) + matrix2(1, 2) * matrix(2, 2)
            matrix3(1, 3) = matrix2(1, 0) * matrix(0, 3) + matrix2(1, 1) * matrix(1, 3) + matrix2(1, 2) * matrix(2, 3) + matrix2(1, 3)

            matrix3(2, 0) = matrix2(2, 0) * matrix(0, 0) + matrix2(2, 1) * matrix(1, 0) + matrix2(2, 2) * matrix(2, 0)
            matrix3(2, 1) = matrix2(2, 0) * matrix(0, 1) + matrix2(2, 1) * matrix(1, 1) + matrix2(2, 2) * matrix(2, 1)
            matrix3(2, 2) = matrix2(2, 0) * matrix(0, 2) + matrix2(2, 1) * matrix(1, 2) + matrix2(2, 2) * matrix(2, 2)
            matrix3(2, 3) = matrix2(2, 0) * matrix(0, 3) + matrix2(2, 1) * matrix(1, 3) + matrix2(2, 2) * matrix(2, 3) + matrix2(2, 3)

            matrix3(3, 0) = 0
            matrix3(3, 1) = 0
            matrix3(3, 2) = 0
            matrix3(3, 3) = 1

            matrix = matrix3

            translate(ox, -oy)
            translate(tox, -toy)
            If on_x Then ox *= -1
            If on_Y Then oy *= -1
        End Sub

        Public Sub translate(ByVal dx As Double, ByVal dy As Double)
            Dim matrix2(,) As Double = {{1, 0, 0, dx}, _
                                        {0, 1, 0, -dy}, _
                                        {0, 0, 1, 0}, _
                                        {0, 0, 0, 1}}
            Dim matrix3(3, 3) As Double

            matrix3(0, 0) = matrix2(0, 0) * matrix(0, 0) + matrix2(0, 1) * matrix(1, 0) + matrix2(0, 2) * matrix(2, 0)
            matrix3(0, 1) = matrix2(0, 0) * matrix(0, 1) + matrix2(0, 1) * matrix(1, 1) + matrix2(0, 2) * matrix(2, 1)
            matrix3(0, 2) = matrix2(0, 0) * matrix(0, 2) + matrix2(0, 1) * matrix(1, 2) + matrix2(0, 2) * matrix(2, 2)
            matrix3(0, 3) = matrix2(0, 0) * matrix(0, 3) + matrix2(0, 1) * matrix(1, 3) + matrix2(0, 2) * matrix(2, 3) + matrix2(0, 3)

            matrix3(1, 0) = matrix2(1, 0) * matrix(0, 0) + matrix2(1, 1) * matrix(1, 0) + matrix2(1, 2) * matrix(2, 0)
            matrix3(1, 1) = matrix2(1, 0) * matrix(0, 1) + matrix2(1, 1) * matrix(1, 1) + matrix2(1, 2) * matrix(2, 1)
            matrix3(1, 2) = matrix2(1, 0) * matrix(0, 2) + matrix2(1, 1) * matrix(1, 2) + matrix2(1, 2) * matrix(2, 2)
            matrix3(1, 3) = matrix2(1, 0) * matrix(0, 3) + matrix2(1, 1) * matrix(1, 3) + matrix2(1, 2) * matrix(2, 3) + matrix2(1, 3)

            matrix3(2, 0) = matrix2(2, 0) * matrix(0, 0) + matrix2(2, 1) * matrix(1, 0) + matrix2(2, 2) * matrix(2, 0)
            matrix3(2, 1) = matrix2(2, 0) * matrix(0, 1) + matrix2(2, 1) * matrix(1, 1) + matrix2(2, 2) * matrix(2, 1)
            matrix3(2, 2) = matrix2(2, 0) * matrix(0, 2) + matrix2(2, 1) * matrix(1, 2) + matrix2(2, 2) * matrix(2, 2)
            matrix3(2, 3) = matrix2(2, 0) * matrix(0, 3) + matrix2(2, 1) * matrix(1, 3) + matrix2(2, 2) * matrix(2, 3) + matrix2(2, 3)

            matrix3(3, 0) = 0
            matrix3(3, 1) = 0
            matrix3(3, 2) = 0
            matrix3(3, 3) = 1

            matrix = matrix3
        End Sub

        Public Sub rotate(ByVal r As Double)
            Dim vert As Vertex = New Vertex(ox, oy, False, False)
            Dim angle As Double = vert.MYangle
            Dim dist As Double = Math.Sqrt(vert.X ^ 2 + vert.Y ^ 2)
            Dim tox, toy As Double
            tox = matrix(0, 3)
            toy = matrix(1, 3)
            translate(-matrix(0, 3), matrix(1, 3))
            translate(-ox, oy)

            ' Rotieren..
            Dim matrix2(,) As Double = {{Math.Cos(r), -Math.Sin(r), 0, 0}, _
                                        {Math.Sin(r), Math.Cos(r), 0, 0}, _
                                        {0, 0, 1, 0}, _
                                        {0, 0, 0, 1}}
            Dim matrix3(3, 3) As Double

            matrix3(0, 0) = matrix2(0, 0) * matrix(0, 0) + matrix2(0, 1) * matrix(1, 0) + matrix2(0, 2) * matrix(2, 0)
            matrix3(0, 1) = matrix2(0, 0) * matrix(0, 1) + matrix2(0, 1) * matrix(1, 1) + matrix2(0, 2) * matrix(2, 1)
            matrix3(0, 2) = matrix2(0, 0) * matrix(0, 2) + matrix2(0, 1) * matrix(1, 2) + matrix2(0, 2) * matrix(2, 2)
            matrix3(0, 3) = matrix2(0, 0) * matrix(0, 3) + matrix2(0, 1) * matrix(1, 3) + matrix2(0, 2) * matrix(2, 3) + matrix2(0, 3)

            matrix3(1, 0) = matrix2(1, 0) * matrix(0, 0) + matrix2(1, 1) * matrix(1, 0) + matrix2(1, 2) * matrix(2, 0)
            matrix3(1, 1) = matrix2(1, 0) * matrix(0, 1) + matrix2(1, 1) * matrix(1, 1) + matrix2(1, 2) * matrix(2, 1)
            matrix3(1, 2) = matrix2(1, 0) * matrix(0, 2) + matrix2(1, 1) * matrix(1, 2) + matrix2(1, 2) * matrix(2, 2)
            matrix3(1, 3) = matrix2(1, 0) * matrix(0, 3) + matrix2(1, 1) * matrix(1, 3) + matrix2(1, 2) * matrix(2, 3) + matrix2(1, 3)

            matrix3(2, 0) = matrix2(2, 0) * matrix(0, 0) + matrix2(2, 1) * matrix(1, 0) + matrix2(2, 2) * matrix(2, 0)
            matrix3(2, 1) = matrix2(2, 0) * matrix(0, 1) + matrix2(2, 1) * matrix(1, 1) + matrix2(2, 2) * matrix(2, 1)
            matrix3(2, 2) = matrix2(2, 0) * matrix(0, 2) + matrix2(2, 1) * matrix(1, 2) + matrix2(2, 2) * matrix(2, 2)
            matrix3(2, 3) = matrix2(2, 0) * matrix(0, 3) + matrix2(2, 1) * matrix(1, 3) + matrix2(2, 2) * matrix(2, 3) + matrix2(2, 3)

            matrix3(3, 0) = 0
            matrix3(3, 1) = 0
            matrix3(3, 2) = 0
            matrix3(3, 3) = 1

            matrix = matrix3

            translate(ox, -oy)
            translate(tox, -toy)
            ox = Math.Cos(angle + r) * dist
            oy = Math.Sin(angle + r) * dist
        End Sub

        Public Sub scale(ByVal sx As Double, ByVal sy As Double)
            Dim vert As Vertex = New Vertex(ox, oy, False, False)
            Dim angle As Double = vert.MYangle
            Dim dist As Double = Math.Sqrt(vert.X ^ 2 + vert.Y ^ 2)
            Dim tox, toy As Double
            tox = matrix(0, 3)
            toy = matrix(1, 3)
            translate(-matrix(0, 3), matrix(1, 3))
            translate(-ox, oy)

            ' Scalieren..
            Dim matrix2(,) As Double = {{sx, 0, 0, 0}, _
                                        {0, sy, 0, 0}, _
                                        {0, 0, 1, 0}, _
                                        {0, 0, 0, 1}}
            Dim matrix3(3, 3) As Double

            matrix3(0, 0) = matrix2(0, 0) * matrix(0, 0) + matrix2(0, 1) * matrix(1, 0) + matrix2(0, 2) * matrix(2, 0)
            matrix3(0, 1) = matrix2(0, 0) * matrix(0, 1) + matrix2(0, 1) * matrix(1, 1) + matrix2(0, 2) * matrix(2, 1)
            matrix3(0, 2) = matrix2(0, 0) * matrix(0, 2) + matrix2(0, 1) * matrix(1, 2) + matrix2(0, 2) * matrix(2, 2)
            matrix3(0, 3) = matrix2(0, 0) * matrix(0, 3) + matrix2(0, 1) * matrix(1, 3) + matrix2(0, 2) * matrix(2, 3) + matrix2(0, 3)

            matrix3(1, 0) = matrix2(1, 0) * matrix(0, 0) + matrix2(1, 1) * matrix(1, 0) + matrix2(1, 2) * matrix(2, 0)
            matrix3(1, 1) = matrix2(1, 0) * matrix(0, 1) + matrix2(1, 1) * matrix(1, 1) + matrix2(1, 2) * matrix(2, 1)
            matrix3(1, 2) = matrix2(1, 0) * matrix(0, 2) + matrix2(1, 1) * matrix(1, 2) + matrix2(1, 2) * matrix(2, 2)
            matrix3(1, 3) = matrix2(1, 0) * matrix(0, 3) + matrix2(1, 1) * matrix(1, 3) + matrix2(1, 2) * matrix(2, 3) + matrix2(1, 3)

            matrix3(2, 0) = matrix2(2, 0) * matrix(0, 0) + matrix2(2, 1) * matrix(1, 0) + matrix2(2, 2) * matrix(2, 0)
            matrix3(2, 1) = matrix2(2, 0) * matrix(0, 1) + matrix2(2, 1) * matrix(1, 1) + matrix2(2, 2) * matrix(2, 1)
            matrix3(2, 2) = matrix2(2, 0) * matrix(0, 2) + matrix2(2, 1) * matrix(1, 2) + matrix2(2, 2) * matrix(2, 2)
            matrix3(2, 3) = matrix2(2, 0) * matrix(0, 3) + matrix2(2, 1) * matrix(1, 3) + matrix2(2, 2) * matrix(2, 3) + matrix2(2, 3)

            matrix3(3, 0) = 0
            matrix3(3, 1) = 0
            matrix3(3, 2) = 0
            matrix3(3, 3) = 1

            matrix = matrix3

            translate(ox, -oy)
            translate(tox, -toy)
            ox *= sx
            oy *= sy
        End Sub

        Private Sub ISerializable_GetObjectData(ByVal oInfo As SerializationInfo, ByVal oContext As StreamingContext) Implements ISerializable.GetObjectData
            Dim tempinteger As Integer
            Dim tempmatrix(3, 3) As Double
            Dim tempstring As String = ""
            Dim tempdouble As Double
            Dim tempshort As Short
            Dim tempcolour As Color = Color.Black
            oInfo.AddValue("primitiveID", Me.primitiveID, tempinteger.GetType())
            oInfo.AddValue("centerVertexID", Me.centerVertexID, tempinteger.GetType())
            oInfo.AddValue("matrix", Me.matrix, tempmatrix.GetType())
            oInfo.AddValue("primitiveName", Me.primitiveName, tempstring.GetType())
            oInfo.AddValue("myColour", Me.myColour, tempcolour.GetType())
            oInfo.AddValue("myColourNumber", Me.myColourNumber, tempshort.GetType())
            oInfo.AddValue("ox", Me.ox, tempdouble.GetType())
            oInfo.AddValue("oy", Me.oy, tempdouble.GetType())
        End Sub

        Protected Sub New(ByVal info As System.Runtime.Serialization.SerializationInfo, ByVal context As System.Runtime.Serialization.StreamingContext)
            Dim tempinteger As Integer
            Dim tempmatrix(3, 3) As Double
            Dim tempstring As String = ""
            Dim tempdouble As Double
            Dim tempcolour As Color = Color.Black
            Me.primitiveID = info.GetValue("primitiveID", tempinteger.GetType())
            Me.centerVertexID = info.GetValue("centerVertexID", tempinteger.GetType())
            Me.matrix = info.GetValue("matrix", tempmatrix.GetType())
            Me.primitiveName = info.GetValue("primitiveName", tempstring.GetType())
            Me.myColour = info.GetValue("myColour", tempcolour.GetType)
            Me.myColourNumber = info.GetInt16("myColourNumber")
            Me.ox = info.GetValue("ox", tempdouble.GetType())
            Me.oy = info.GetValue("oy", tempdouble.GetType())
        End Sub
    End Class

    <Serializable()> _
   Public Class Metadata
        Implements System.Runtime.Serialization.ISerializable

        Public mData(11) As String
        Public isMainMetadata As Boolean

        Public recommendedMode As Byte = 6
        Public additionalData As String = "<br>"

        Public matrix(3, 3) As Single

        Public mAlias As String = ""

        Sub New()
            For i As Byte = 0 To 11
                mData(i) = ""
            Next i
        End Sub

        Sub New(ByVal description As String, ByVal filename As String, ByVal realname As String, ByVal username As String, _
                ByVal parttype As String, ByVal licensetype As String, ByVal help As String, ByVal bfc As String, _
                ByVal category As String, ByVal keywords As String, ByVal history As String, ByVal comments As String)
            mData(0) = description
            mData(1) = filename
            mData(2) = realname
            mData(3) = username
            mData(4) = parttype
            mData(5) = licensetype
            mData(6) = help
            mData(7) = bfc
            mData(8) = category
            mData(9) = keywords
            mData(10) = history
            mData(11) = comments
        End Sub

        Public Sub show()
            MetadataDialog.TBDescription.Text = mData(0)
            MetadataDialog.TBFilename.Text = mData(1)
            If mData(2) = "" Then
                mData(2) = MainForm.mySettings.defaultName
            End If
            If mData(3) = "" Then
                mData(3) = MainForm.mySettings.defaultUser
            End If
            MetadataDialog.TBRealName.Text = mData(2)
            MetadataDialog.TBUserName.Text = mData(3)
            If Not isMainMetadata Then
                Select Case MainForm.myMetadata.mData(4)
                    Case "Subpart"
                        mData(4) = "Subpart"
                    Case "Part"
                        mData(4) = "Subpart"
                    Case "Unofficial_Subpart"
                        mData(4) = "Unofficial_Subpart"
                    Case "Unofficial_Part"
                        mData(4) = "Unofficial_Subpart"
                    Case Else
                        mData(4) = ""
                End Select
                mData(5) = MainForm.myMetadata.mData(5)
                mData(7) = MainForm.myMetadata.mData(7)
            Else
                If mData(5) = "" Then
                    mData(5) = MainForm.mySettings.defaultLicense
                End If
            End If
            MetadataDialog.CBPartType.Text = mData(4)
            MetadataDialog.CBLicense.Text = mData(5)
            MetadataDialog.RTBHelp.Text = Replace(mData(6), "<br>", vbLf)
            MetadataDialog.CBBFC.Text = mData(7)
            MetadataDialog.TBCategory.Text = mData(8)
            MetadataDialog.RTBKeywords.Text = Replace(mData(9), "<br>", vbLf)
            MetadataDialog.RTBHistory.Text = Replace(mData(10), "<br>", vbLf)
            MetadataDialog.RTBComments.Text = Replace(mData(11), "<br>", vbLf)
        End Sub

        Shadows Function Clone() As Metadata
            Return New Metadata() With {.mData = mData.Clone, .mAlias = mAlias.Clone, .additionalData = additionalData.Clone, .matrix = matrix.Clone, .recommendedMode = recommendedMode, .isMainMetadata = isMainMetadata}
        End Function

        Private Sub ISerializable_GetObjectData(ByVal oInfo As SerializationInfo, ByVal oContext As StreamingContext) Implements ISerializable.GetObjectData
            oInfo.AddValue("mData", Me.mData, mData.GetType())
            oInfo.AddValue("recommendedMode", Me.recommendedMode, recommendedMode.GetType())
            oInfo.AddValue("additionalData", Me.additionalData, additionalData.GetType())
            oInfo.AddValue("matrix", Me.matrix, matrix.GetType())
            oInfo.AddValue("isMainMetadata", Me.isMainMetadata, isMainMetadata.GetType())
        End Sub

        Protected Sub New(ByVal info As System.Runtime.Serialization.SerializationInfo, ByVal context As System.Runtime.Serialization.StreamingContext)
            Dim tempstringArray(11) As String
            Dim tempbyte As Byte
            Dim tempstring As String = ""
            Dim tempmatrix(3, 3) As Single
            Dim tempbool As Boolean
            Me.mData = info.GetValue("mData", tempstringArray.GetType())
            Me.recommendedMode = info.GetValue("recommendedMode", tempbyte.GetType())
            Me.additionalData = info.GetValue("additionalData", tempstring.GetType())
            Me.matrix = info.GetValue("matrix", tempmatrix.GetType())
            Me.isMainMetadata = info.GetValue("isMainMetadata", tempbool.GetType())
        End Sub

    End Class

    <Serializable()> _
  Public Class TemplateTextInfo
        Implements System.Runtime.Serialization.ISerializable

        Public Text As String
        Public X As Single
        Public Y As Single

        Sub New(ByVal x As Single, ByVal y As Single, ByVal text As String)
            Me.X = x
            Me.Y = y
            Me.Text = text
        End Sub

        Shadows Function Clone() As TemplateTextInfo
            Return New TemplateTextInfo(X, Y, Text)
        End Function

        Private Sub ISerializable_GetObjectData(ByVal oInfo As SerializationInfo, ByVal oContext As StreamingContext) Implements ISerializable.GetObjectData
            oInfo.AddValue("X", Me.X, X.GetType())
            oInfo.AddValue("Y", Me.Y, Y.GetType())
            oInfo.AddValue("Text", Me.Text, Text.GetType())
        End Sub

        Protected Sub New(ByVal info As System.Runtime.Serialization.SerializationInfo, ByVal context As System.Runtime.Serialization.StreamingContext)
            Dim tempsingle As Single
            Dim tempstring As String = ""
            Me.X = info.GetValue("X", tempsingle.GetType())
            Me.Y = info.GetValue("Y", tempsingle.GetType())
            Me.Text = info.GetValue("Text", tempstring.GetType())
        End Sub

    End Class

    Public Shared Function DeepClone(ByVal source As Object) As Object
        Using m As New MemoryStream() With {.Capacity = 102400}
            Dim b As New BinaryFormatter()
            b.Serialize(m, source)
            m.Position = 0
            Return b.UnsafeDeserialize(m, Nothing)
        End Using
    End Function

#End Region
#Region "Main Form"
    Dim lastAction As Byte = 0
    Dim lastMode As Byte = 0

    Private Sub MainForm_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        If unsavedChanges AndAlso ShowAllWarningsToolStripMenuItem.Checked Then
            Dim result As MsgBoxResult = MsgBox("You have unsaved changes - would you like to save them first?", MsgBoxStyle.YesNoCancel + MsgBoxStyle.MsgBoxSetForeground + MsgBoxStyle.Information, "Question:")
            If result = MsgBoxResult.Yes Then SaveToolStripMenuItem.PerformClick()
            If result = MsgBoxResult.Cancel Then e.Cancel = True
        End If
        If Me.BackgroundBitmapImport.IsBusy Then
            BackgroundBitmapImport.CancelAsync()
            e.Cancel = True
            Timer2.Enabled = True
        End If
        If Me.BackgroundHowTo.IsBusy Then
            turorialRuns = False
            e.Cancel = True
            Timer2.Enabled = True
        End If
        If e.CloseReason = CloseReason.None OrElse e.CloseReason = CloseReason.ApplicationExitCall Then
            SaveAs.FileName = appPath & "errordump_file.lpc"
            SaveToolStripMenuItem.PerformClick()
            Using DateiOut As BinaryWriter = New BinaryWriter(File.Open(appPath & "errordump_data.txt", FileMode.Create))
                DateiOut.Write("Application Path: " & appPath & vbLf)
                DateiOut.Write("Stack Trace: " & My.Application.Info.StackTrace & vbLf)
                DateiOut.Write("Loaded Assemblies: ")
                For i = 0 To My.Application.Info.LoadedAssemblies.Count - 1
                    DateiOut.Write(My.Application.Info.LoadedAssemblies(i).FullName)
                Next
                For i = 0 To historyVertices.Count - 1
                    DateiOut.Write("History Entry: " & i)
                    DateiOut.Write(historyMyMetadata(i).ToString)
                    DateiOut.Write(historyPrimitives(i).ToString)
                    DateiOut.Write(historyPrimitivesHash(i).ToString)
                    DateiOut.Write(historyPrimitivesMetadataHash(i).ToString)
                    DateiOut.Write(historyProjectionQuads(i).ToString)
                    DateiOut.Write(historySelectedTriangles(i).ToString)
                    DateiOut.Write(historySelectedVertices(i).ToString)
                    DateiOut.Write(historyTemplateShape(i).ToString)
                    DateiOut.Write(historyTemplateText(i).ToString)
                    DateiOut.Write(historyTriangles(i).ToString)
                    DateiOut.Write(historyVertices(i).ToString)
                Next i
            End Using
            Timer2.Interval = 3000
            e.Cancel = True
            Timer2.Enabled = True
        End If
    End Sub

    Public cstep As Byte = 100
    Public cnumber As Short = 0
    Public mousePressed As Boolean
    Public zoomScroll As Boolean
    Private Sub MainForm_KeyDown(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles Me.KeyDown
        If Not isLoading AndAlso Not NUDMoveSnap.Focused AndAlso Not NUDRotateSnap.Focused AndAlso Not NUDScaleSnap.Focused AndAlso Not NUDScale.Focused AndAlso Not NUDoffsetX.Focused AndAlso Not NUDoffsetY.Focused AndAlso Not NUDGrid.Focused Then
            If mySettings.useAlternativeKeys Then
                If e.KeyCode = myKeys.Translate Then
                    If Not mousePressed Then
                        pressMouseRight()
                        mousePressed = True
                    End If
                    Exit Sub
                ElseIf e.KeyCode = myKeys.Zoom Then
                    klickY = getCursorpositionY()
                    zoomScroll = True
                End If
            End If
            If e.KeyCode = myKeys.ModeSelect Then Me.BtnSelect.PerformClick()
            If e.KeyCode = myKeys.ModeMove Then Me.BtnMove.PerformClick()
            If e.KeyCode = myKeys.ModeRotate Then Me.BtnRotate.PerformClick()
            If e.KeyCode = myKeys.ModeScale Then Me.BtnScale.PerformClick()
            If e.KeyCode = myKeys.AddVertex Then Me.BtnAddVertex.PerformClick()
            If e.KeyCode = myKeys.AddTriangle Then Me.BtnAddTriangle.PerformClick()
            If e.KeyCode = myKeys.Abort AndAlso BtnAbort.Visible AndAlso BtnAbort.Enabled Then Me.BtnAbort.PerformClick()
            If e.KeyCode = myKeys.Abort Then cstep = 100 : cnumber = 0
            If e.KeyCode = myKeys.ShowColours Then BtnColours.PerformClick()
            If e.KeyCode = myKeys.Preview AndAlso BtnColours.Checked Then
                BtnPreview.PerformClick()
            ElseIf e.KeyCode = myKeys.Preview Then
                BtnColours.PerformClick() : BtnPreview.PerformClick()
            End If
            If e.KeyCode = myKeys.Pipette Then BtnPipette.PerformClick()
            If adjustmode Then
                If e.KeyCode = Keys.Left Then NUDoffsetX.Value -= moveSnap : SBZoom_Scroll(Me, New ScrollEventArgs(ScrollEventType.SmallIncrement, 0, 1, ScrollOrientation.VerticalScroll))
                If e.KeyCode = Keys.Right Then NUDoffsetX.Value += moveSnap : SBZoom_Scroll(Me, New ScrollEventArgs(ScrollEventType.SmallDecrement, 0, -1, ScrollOrientation.VerticalScroll))
                If e.KeyCode = Keys.Up Then NUDoffsetY.Value -= moveSnap : SBZoom_Scroll(Me, New ScrollEventArgs(ScrollEventType.SmallIncrement, 0, 1, ScrollOrientation.VerticalScroll))
                If e.KeyCode = Keys.Down Then NUDoffsetY.Value += moveSnap : SBZoom_Scroll(Me, New ScrollEventArgs(ScrollEventType.SmallDecrement, 0, -1, ScrollOrientation.VerticalScroll))
            End If
            If cstep = 1 Then
                If e.KeyCode = Keys.NumPad1 Then cnumber += 1
                If e.KeyCode = Keys.NumPad2 Then cnumber += 2
                If e.KeyCode = Keys.NumPad3 Then cnumber += 3
                If e.KeyCode = Keys.NumPad4 Then cnumber += 4
                If e.KeyCode = Keys.NumPad5 Then cnumber += 5
                If e.KeyCode = Keys.NumPad6 Then cnumber += 6
                If e.KeyCode = Keys.NumPad7 Then cnumber += 7
                If e.KeyCode = Keys.NumPad8 Then cnumber += 8
                If e.KeyCode = Keys.NumPad9 Then cnumber += 9
                addHistoryI(withDataFrom.Triangles + withDataFrom.SelectedTriangles + withDataFrom.Primitives)
                If colourHMap.ContainsKey(cnumber) Then
                    setColour(colourHMap(cnumber), cnumber)
                Else
                    setColour(colourHMap(16), cnumber)
                End If
                addHistoryII(withDataFrom.Triangles + withDataFrom.SelectedTriangles + withDataFrom.Primitives)
                cstep = 100 : cnumber = 0
            Else
                If e.KeyCode = Keys.NumPad0 Then cstep /= 10
                If e.KeyCode = Keys.NumPad1 Then cnumber += 1 * cstep : cstep /= 10
                If e.KeyCode = Keys.NumPad2 Then cnumber += 2 * cstep : cstep /= 10
                If e.KeyCode = Keys.NumPad3 Then cnumber += 3 * cstep : cstep /= 10
                If e.KeyCode = Keys.NumPad4 Then cnumber += 4 * cstep : cstep /= 10
                If e.KeyCode = Keys.NumPad5 Then cnumber += 5 * cstep : cstep /= 10
                If e.KeyCode = Keys.NumPad6 Then cnumber += 6 * cstep : cstep /= 10
                If e.KeyCode = Keys.NumPad7 Then cnumber += 7 * cstep : cstep /= 10
                If e.KeyCode = Keys.NumPad8 Then cnumber += 8 * cstep : cstep /= 10
                If e.KeyCode = Keys.NumPad9 Then cnumber += 9 * cstep : cstep /= 10
            End If
        End If
    End Sub

    Private Sub MainForm_KeyUp(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles Me.KeyUp
        If mousePressed Then
            releaseMouseRight()
            mousePressed = False
        End If
        zoomScroll = False
    End Sub

    Private Sub MainForm_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        raster = True
        addHistoryI(withDataFrom.All)
        addHistoryII(withDataFrom.All)

        Me.TriangleModeToolStripMenuItem.PerformClick()
        SBZoom_Scroll(Me, New ScrollEventArgs(ScrollEventType.SmallDecrement, 0, -15, ScrollOrientation.VerticalScroll))
        If Not My.Computer.FileSystem.FileExists(appPath & "Rectifier.exe") Then
            Using DateiOut As BinaryWriter = New BinaryWriter(File.Open(appPath & "Rectifier.exe", FileMode.Create))
                For Each b As Byte In My.Resources.Rectifier
                    DateiOut.Write(b)
                Next
            End Using
        End If
        loadConfig()
        If ldrawPath = "" Then
            If Me.FBDLDrawDir.ShowDialog = Windows.Forms.DialogResult.Cancel Then
                MsgBox("The application starts with reduced functionality.. It is strongly recommend to specify the LDraw-Path to use the updated colour table (ldconfig.ldr) and all primitives.", MsgBoxStyle.OkOnly + MsgBoxStyle.MsgBoxSetForeground + MsgBoxStyle.Information, "Information:")
                Me.PrimitivesToolStripMenuItem.Visible = False
                Me.ImportToolStripMenuItem.Visible = False
                Me.LoadPatternToolStripMenuItem.Visible = False
                Me.TemplateToolStripMenuItem.Visible = False
            Else
                ldrawPath = Me.FBDLDrawDir.SelectedPath
                saveConfig()
            End If
        End If
        If Not My.Computer.FileSystem.FileExists(appPath & "LDConfig.ldr") Then
            Using DateiOut As BinaryWriter = New BinaryWriter(File.Open(appPath & "LDConfig.ldr", FileMode.Create))
                For Each b As Byte In My.Resources.LDConfig
                    DateiOut.Write(b)
                Next
            End Using
        End If
        If My.Computer.FileSystem.FileExists(ldrawPath & "\LDConfig.ldr") Then
            Using DateiIn As StreamReader = My.Computer.FileSystem.OpenTextFileReader(ldrawPath & "\LDConfig.ldr", System.Text.Encoding.GetEncoding(20127))
                Do
                    Dim s As String = Replace(DateiIn.ReadLine, "CODE", "@")
                    If s Like "*!COLOUR*" Then
                        Dim s1() As String = s.Split("@")
                        If s1.Length = 2 Then
                            s1(0) = Replace(Replace(s1(0), "0 !COLOUR ", ""), "_", " ")
                            s1(1) = Replace(s1(1), "VALUE", "@")
                            s1(1) = Replace(s1(1), "EDGE", "@")
                            Dim s2() As String = s1(1).Split("@")
                            s2(0) = Trim(s2(0))
                            s2(1) = Mid(Trim(s2(1)), 2)
                            Dim id As Short = CType(s2(0), Short)
                            If maxColourNumber < id Then maxColourNumber = id
                            If id <> 16 Then colourHMap.Add(id, Color.FromArgb(Integer.Parse("FF" & s2(1), System.Globalization.NumberStyles.HexNumber))) Else colourHMap.Add(id, Color.FromArgb(0, 0, 0, 0))
                            colourHMapName.Add(id, s1(0))
                        End If
                    End If
                Loop Until DateiIn.EndOfStream
            End Using
        Else
            Using DateiIn As StreamReader = My.Computer.FileSystem.OpenTextFileReader(appPath & "LDConfig.ldr", System.Text.Encoding.GetEncoding(20127))
                Do
                    Dim s As String = Replace(DateiIn.ReadLine, "CODE", "@")
                    If s Like "*!COLOUR*" Then
                        Dim s1() As String = s.Split("@")
                        If s1.Length = 2 Then
                            s1(0) = Replace(Replace(s1(0), "0 !COLOUR ", ""), "_", " ")
                            s1(1) = Replace(s1(1), "VALUE", "@")
                            s1(1) = Replace(s1(1), "EDGE", "@")
                            Dim s2() As String = s1(1).Split("@")
                            s2(0) = Trim(s2(0))
                            s2(1) = Mid(Trim(s2(1)), 2)
                            Dim id As Short = CType(s2(0), Short)
                            If maxColourNumber < id Then maxColourNumber = id
                            If id <> 16 Then colourHMap.Add(id, Color.FromArgb(Integer.Parse("FF" & s2(1), System.Globalization.NumberStyles.HexNumber))) Else colourHMap.Add(id, Color.FromArgb(0, 0, 0, 0))
                            colourHMapName.Add(id, s1(0))
                        End If
                    End If
                Loop Until DateiIn.EndOfStream
            End Using
        End If

        C0.BackColor = colourHMap(0)
        C1.BackColor = colourHMap(1)
        C2.BackColor = colourHMap(2)
        C3.BackColor = colourHMap(3)

        C4.BackColor = colourHMap(4)
        C5.BackColor = colourHMap(5)
        C6.BackColor = colourHMap(6)
        C7.BackColor = colourHMap(7)

        C8.BackColor = colourHMap(8)
        C9.BackColor = colourHMap(9)
        C10.BackColor = colourHMap(10)
        C11.BackColor = colourHMap(11)

        C12.BackColor = colourHMap(12)
        C13.BackColor = colourHMap(13)
        C14.BackColor = colourHMap(14)
        C15.BackColor = colourHMap(15)

        Dim templates() As String = Directory.GetFiles(appPath)

        For i As Integer = 0 To templates.Length - 1
            If templates(i) Like "*.txt" Then
                Dim ti As New ToolStripMenuItem
                ti.Text = Replace(Mid(templates(i), templates(i).LastIndexOf("\") + 2), ".txt", "")
                ti.ToolTipText = templates(i)
                AddHandler ti.Click, AddressOf ti_Click
                TemplateToolStripMenuItem.DropDownItems.Add(ti)
            End If
        Next

        If My.Application.CommandLineArgs.Count > 0 Then
            Dim v As Array
            Dim mypath As String
            Dim commandlineargs As String = Environment.CommandLine
            v = Split(commandlineargs, """ ")
            mypath = v(1)
            If My.Computer.FileSystem.FileExists(mypath) AndAlso (mypath Like "*.lpc" OrElse mypath Like "*.txt") Then
                SaveAs.FileName = mypath
                LoadLPCFile(mypath)
            End If
        End If

    End Sub

    Private Sub ti_Click(ByVal sender As ToolStripMenuItem, ByVal e As EventArgs)
        If My.Computer.FileSystem.FileExists(appPath & sender.Text & ".txt") Then
            If unsavedChanges AndAlso ShowAllWarningsToolStripMenuItem.Checked Then
                Dim result As MsgBoxResult = MsgBox("You have unsaved changes - would you like to save them first?", MsgBoxStyle.YesNoCancel + MsgBoxStyle.MsgBoxSetForeground + MsgBoxStyle.Information, "Question:")
                If result = MsgBoxResult.Yes Then SaveToolStripMenuItem.PerformClick()
                If result = MsgBoxResult.Cancel Then Exit Sub
            End If

            isLoading = True
            addHistoryI(withDataFrom.All)

            Dim result2 As MsgBoxResult = MsgBox("Would you like to start a new pattern? Choose 'No' in case you want to (re-)load the template only.", MsgBoxStyle.YesNo + MsgBoxStyle.MsgBoxSetForeground + MsgBoxStyle.DefaultButton1 + MsgBoxStyle.Information, "Question:")
            If result2 = MsgBoxResult.No Then
                templateProjectionQuad.Clear()
                templateText.Clear()
                templateShape.Clear()
                alreadyCalculatedQuads = 0
            Else
                newPattern()
            End If

            Dim lineNumber As Integer = 0
            Dim content As String = ""
            Try
                Using DateiIn As StreamReader = My.Computer.FileSystem.OpenTextFileReader(appPath & sender.Text & ".txt", System.Text.Encoding.GetEncoding(20127))
                    lineNumber += 1
                    myMetadata.recommendedMode = CType(DateiIn.ReadLine(), Byte)
                    lineNumber += 1
                    myMetadata.additionalData = DateiIn.ReadLine()
                    For r As Integer = 0 To 3
                        lineNumber += 1
                        Dim rm() As String = Replace(DateiIn.ReadLine(), ".", comma).Split(" ")
                        For c As Integer = 0 To 3
                            myMetadata.matrix(r, c) = CType(rm(c), Single)
                        Next
                    Next
                    Do
                        lineNumber += 1
                        content = DateiIn.ReadLine
                        If content <> "" Then
                            Dim s() As String = Replace(content, ".", comma).Split(" ")
                            If s.Length = 2 Then
                                templateShape.Add(New PointF(CType(s(0), Single) * 1000, CType(s(1), Single) * 1000))
                                Vertices.Add(New Vertex(CType(s(0), Single) * 1000, CType(s(1), Single) * 1000, False))
                            ElseIf Mid(content, 1, 5) = "0 // " AndAlso s.Length > 4 Then
                                Dim Text As String = s(4)
                                For i As Integer = 5 To s.Length - 1
                                    Text += " " & s(i)
                                Next
                                templateText.Add(New TemplateTextInfo(CType(s(2), Single) * 1000, CType(s(3), Single) * 1000, Text))
                            ElseIf s.Length = 16 Then
                                templateShape.Add(New PointF(CType(s(0), Single) * 1000, CType(s(1), Single) * 1000))
                                inlinePrimitive(CType(s(2), Short), _
                                                    -CType(s(6), Double) * 1000, -CType(s(8), Double), 0, -CType(s(3), Double) * 1000, _
                                                    -CType(s(12), Double), -CType(s(14), Double) * 1000, 0, -CType(s(5), Double) * 1000, _
                                                    0, 0, 1, 0, _
                                                    1, 0, 0, 0, _
                                                    0, 1, 0, 0, _
                                                    0, 0, -1, 0, _
                                                     Replace(s(15), comma, "."), New List(Of String))
                            ElseIf s.Length = 20 Then
                                Dim single_array(s.Length - 1) As Single
                                For i As Integer = 0 To s.Length - 1
                                    single_array(i) = CType(s(i), Single)
                                Next
                                templateProjectionQuad.Add(New ProjectionQuad( _
                                single_array(0), single_array(1), single_array(2), single_array(3), single_array(4), _
                                single_array(5), single_array(6), single_array(7), single_array(8), single_array(9), _
                                single_array(10), single_array(11), single_array(12), single_array(13), single_array(14), _
                                single_array(15), single_array(16), single_array(17), single_array(18), single_array(19)))
                            End If
                        End If
                    Loop Until DateiIn.EndOfStream
                    cleanupDATVertices()
                End Using
                ProjectionQuadTimer.Enabled = templateProjectionQuad.Count > 0
            Catch ex As Exception
                MsgBox(ex.Message & vbCrLf & "at line " & lineNumber & ": " & content, MsgBoxStyle.OkOnly + MsgBoxStyle.MsgBoxSetForeground + MsgBoxStyle.Critical, "Parsing-Error:")
            End Try
            addHistoryII(withDataFrom.All)
            isLoading = False
            Me.Refresh()
        End If
    End Sub

    Public Sub New()

        ' This call is mandatory for the Windows Form-Designer!
        InitializeComponent()

        Me.SetStyle(ControlStyles.DoubleBuffer, True)
        Me.SetStyle(ControlStyles.ResizeRedraw, True)
        Me.SetStyle(ControlStyles.AllPaintingInWmPaint, True)
        Me.SetStyle(ControlStyles.UserPaint, True)
    End Sub

    Private Sub MainForm_ResizeEnd(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.ResizeEnd
        Me.Refresh()
    End Sub

    Private Sub MainForm_SizeChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.SizeChanged
        Me.Refresh()
    End Sub
#End Region

#Region "Mouse"

    Function getCursorpositionX() As Integer
        Return Cursor.Position.X - offset_X
    End Function

    Function getCursorpositionY() As Integer
        Return Cursor.Position.Y - offset_Y
    End Function

    Private Sub MainForm_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseDown
        If isLoading Then Exit Sub
        Me.ContextMenuStrip = Nothing
        If Not Me.Focused Then Me.Focus()
        If Not SBZoom.Focused Then SBZoom.Focus()
        If e.Button = Windows.Forms.MouseButtons.Left Then
            If getOffset Then
                offset_X = System.Windows.Forms.Cursor.Position.X - e.X
                offset_Y = System.Windows.Forms.Cursor.Position.Y - e.Y
                moveMouseAbsoluteLeftClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2, offset_Y + Me.ClientSize.Height / 2)
                getOffset = False
            End If
            If adjustmode Then
                klickX = getCursorpositionX()
                klickY = getCursorpositionY()
                doAdjust = True
                Exit Sub
            End If
            If primitiveMode = 0 Then
                Select Case bearbeitungsmodus
                    Case editMode.Selection
                        klickX = getCursorpositionX()
                        klickY = getCursorpositionY()
                        auswahl = True
                        Me.Refresh()
                        Exit Select
                    Case editMode.Translation
                        movemode = True
                        klickX = getCursorpositionX()
                        klickY = getCursorpositionY()
                        Exit Select
                    Case editMode.Rotation
                        If SelectedVertices.Count > 0 Then
                            rotatemode = True
                            klickX = getCursorpositionX()
                            klickY = getCursorpositionY()
                            If objectToModify = "Primitive" Then
                                Dim tid As Integer = Primitives(PrimitivesHMap(SelectedVertices(1).groupindex)).centerVertexID
                                For Each vert As Vertex In SelectedVertices
                                    If vert.vertexID = tid Then
                                        temp_center = New Vertex(vert.X, vert.Y, False, False)
                                        Exit For
                                    End If
                                Next
                            Else
                                temp_center = New Vertex(0, 0, False, False)
                                For Each vert As Vertex In SelectedVertices
                                    temp_center += vert
                                Next
                                temp_center.X /= SelectedVertices.Count
                                temp_center.Y /= SelectedVertices.Count
                            End If
                            For Each vert As Vertex In SelectedVertices
                                vert.angleFrom = vert.angle(temp_center) + Math.PI
                                vert.distanceFrom = vert.dist(temp_center)
                                vert.oldX = vert.X
                                vert.oldY = vert.Y
                            Next
                        End If
                        Exit Select
                    Case editMode.Scale
                        If SelectedVertices.Count > 0 Then
                            scalemode = True
                            klickX = getCursorpositionX()
                            klickY = getCursorpositionY()
                            If objectToModify = "Primitive" Then
                                Dim tid As Integer = Primitives(PrimitivesHMap(SelectedVertices(1).groupindex)).centerVertexID
                                For Each vert As Vertex In SelectedVertices
                                    If vert.vertexID = tid Then
                                        temp_center = New Vertex(vert.X, vert.Y, False, False)
                                        Exit For
                                    End If
                                Next
                            Else
                                temp_center = New Vertex(0, 0, False, False)
                                For Each vert As Vertex In SelectedVertices
                                    temp_center += vert
                                Next
                                temp_center.X /= SelectedVertices.Count
                                temp_center.Y /= SelectedVertices.Count
                            End If
                            For Each vert As Vertex In SelectedVertices
                                vert.angleFrom = vert.angle(temp_center) + Math.PI
                                vert.distanceFrom = vert.dist(temp_center)
                                vert.oldX = vert.X
                                vert.oldY = vert.Y
                            Next
                            Me.Refresh()
                        End If
                        Exit Select
                End Select
            Else

            End If
        ElseIf e.Button = Windows.Forms.MouseButtons.Middle Then
        ElseIf e.Button = Windows.Forms.MouseButtons.Right Then
            toffsetX = offsetX
            toffsetY = offsetY
            klickX = getCursorpositionX()
            klickY = getCursorpositionY()
            kamera = True
        End If
    End Sub

    Private Sub MainForm_MouseMove(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseMove
        If Not isLoading Then
            LblCoords.Text = ""
            If SelectedTriangles.Count > 0 AndAlso SelectedTriangles(0).groupindex > -1 Then
                LblCoords.Text = Primitives(PrimitivesHMap(SelectedTriangles(0).groupindex)).primitiveName & "   "
            End If
            LblCoords.Text = LblCoords.Text & Math.Round(getXcoordinate(getCursorpositionX()) / Me.moveSnap * factor) * Me.moveSnap / 1000 & " | " & Math.Round(getYcoordinate(getCursorpositionY()) / Me.moveSnap * factor) * Me.moveSnap / 1000 & " " & unit
        End If
        Dim curX As Integer
        Dim curY As Integer
        curX = e.X
        curY = e.Y
        If curX + offset_X > Cursor.Position.X Then offset_X = Cursor.Position.X - curX
        If curX + offset_X < Cursor.Position.X Then offset_X = Cursor.Position.X - curX

        If curY + offset_Y > Cursor.Position.Y Then offset_Y = Cursor.Position.Y - curY
        If curY + offset_Y < Cursor.Position.Y Then offset_Y = Cursor.Position.Y - curY
        If auswahl OrElse kamera OrElse movemode OrElse trianglemode > 0 OrElse rotatemode OrElse scalemode OrElse adjustmode OrElse primitiveMode > 0 OrElse BtnAddVertex.Checked Then Me.Refresh()
        If adjustmode Then
            Dim tx As Integer = getCursorpositionX()
            Dim ty As Integer = getCursorpositionY()
            Dim absOffsetX As Integer = Me.offsetX * zoomfactor + CType(Me.ClientSize.Width / 2, Integer)
            Dim absOffsetY As Integer = Me.offsetY * zoomfactor + CType(Me.ClientSize.Height / 2, Integer)
            Dim pvabsOffsetX As Integer = Fix((getCursorpositionX() - klickX) / zoomfactor)
            Dim pvabsOffsetY As Integer = Fix((getCursorpositionY() - klickY) / zoomfactor)
            If Not doAdjust Then
                pvabsOffsetX = 0
                pvabsOffsetY = 0
            End If
            Dim toffsetX As Integer = imgOffsetX
            Dim toffsetY As Integer = imgOffsetY
            Dim tScale As Single = imgScale
            If Me.Cursor = Cursors.SizeAll Then
                toffsetX += pvabsOffsetX
                toffsetY += pvabsOffsetY
            ElseIf Me.Cursor = Cursors.SizeNESW OrElse Me.Cursor = Cursors.SizeNWSE OrElse Me.Cursor = Cursors.SizeWE OrElse Me.Cursor = Cursors.SizeNS Then
                If getCursorpositionX() < CType(absOffsetX + toffsetX * zoomfactor, Integer) Then pvabsOffsetX *= -1
                If getCursorpositionY() < CType(absOffsetY + toffsetY * zoomfactor, Integer) Then pvabsOffsetY *= -1
                If Me.Cursor = Cursors.SizeWE Then
                    tScale += ((backgroundPicture.Width * tScale + pvabsOffsetX) / backgroundPicture.Width - tScale) * 2
                Else
                    tScale += ((backgroundPicture.Height * tScale + pvabsOffsetY) / backgroundPicture.Height - tScale) * 2
                End If
                If Control.ModifierKeys = 131072 Then adjustDirection = -1 Else adjustDirection = tempAdjustDirection
                Select Case adjustDirection
                    Case 0 : toffsetY -= pvabsOffsetY
                    Case 1.5 : toffsetX += backgroundPicture.Width / backgroundPicture.Height * pvabsOffsetY : toffsetY -= pvabsOffsetY
                    Case 3 : toffsetX += pvabsOffsetX
                    Case 4.5 : toffsetX += backgroundPicture.Width / backgroundPicture.Height * pvabsOffsetY : toffsetY += pvabsOffsetY
                    Case 6 : toffsetY += pvabsOffsetY
                    Case 7.5 : toffsetX -= backgroundPicture.Width / backgroundPicture.Height * pvabsOffsetY : toffsetY += pvabsOffsetY
                    Case 9 : toffsetX -= pvabsOffsetX
                    Case 10.5 : toffsetX -= backgroundPicture.Width / backgroundPicture.Height * pvabsOffsetY : toffsetY -= pvabsOffsetY
                End Select
            End If
            If Not doAdjust Then
                If CType(absOffsetX - (backgroundPicture.Width / 2 * tScale - toffsetX) * zoomfactor, Integer) < tx AndAlso CType(absOffsetY - (backgroundPicture.Height / 2 * tScale - toffsetY) * zoomfactor, Integer) < ty AndAlso _
                    CType(absOffsetX - (backgroundPicture.Width / 2 * tScale - toffsetX) * zoomfactor, Integer) + backgroundPicture.Width * zoomfactor * tScale > tx AndAlso _
                    CType(absOffsetY - (backgroundPicture.Height / 2 * tScale - toffsetY) * zoomfactor, Integer) + backgroundPicture.Height * zoomfactor * tScale > ty Then
                    Me.Cursor = Cursors.SizeAll
                ElseIf CType(absOffsetX - (backgroundPicture.Width / 2 * tScale - toffsetX) * zoomfactor, Integer) <= tx + 6 AndAlso CType(absOffsetY - (backgroundPicture.Height / 2 * tScale - toffsetY) * zoomfactor, Integer) <= ty + 6 AndAlso _
                    CType(absOffsetX - (backgroundPicture.Width / 2 * tScale - toffsetX) * zoomfactor, Integer) + backgroundPicture.Width * zoomfactor * tScale >= tx - 6 AndAlso _
                    CType(absOffsetY - (backgroundPicture.Height / 2 * tScale - toffsetY) * zoomfactor, Integer) + backgroundPicture.Height * zoomfactor * tScale >= ty - 6 Then
                    If Math.Abs(CType(absOffsetY - (backgroundPicture.Height / 2 * tScale - toffsetY) * zoomfactor, Integer) - ty) < 6 OrElse Math.Abs(CType(absOffsetY - (backgroundPicture.Height / 2 * tScale - toffsetY) * zoomfactor, Integer) + backgroundPicture.Height * zoomfactor * tScale - ty) < 6 Then
                        If Math.Abs(CType(absOffsetY - (backgroundPicture.Height / 2 * tScale - toffsetY) * zoomfactor, Integer) - ty) < 6 Then
                            If Math.Abs(CType(absOffsetX - (backgroundPicture.Width / 2 * tScale - toffsetX) * zoomfactor, Integer) - tx) < 6 Then
                                Me.Cursor = Cursors.SizeNWSE
                            ElseIf Math.Abs(CType(absOffsetX - (backgroundPicture.Width / 2 * tScale - toffsetX) * zoomfactor, Integer) + backgroundPicture.Width * zoomfactor * tScale - tx) < 6 Then
                                Me.Cursor = Cursors.SizeNESW
                            Else
                                Me.Cursor = Cursors.SizeNS
                            End If
                        Else
                            If Math.Abs(CType(absOffsetX - (backgroundPicture.Width / 2 * tScale - toffsetX) * zoomfactor, Integer) - tx) < 6 Then
                                Me.Cursor = Cursors.SizeNESW
                            ElseIf Math.Abs(CType(absOffsetX - (backgroundPicture.Width / 2 * tScale - toffsetX) * zoomfactor, Integer) + backgroundPicture.Width * zoomfactor * tScale - tx) < 6 Then
                                Me.Cursor = Cursors.SizeNWSE
                            Else
                                Me.Cursor = Cursors.SizeNS
                            End If
                        End If
                        If Me.Cursor = Cursors.SizeNS Then
                            If Math.Abs(CType(absOffsetY - (backgroundPicture.Height / 2 * tScale - toffsetY) * zoomfactor, Integer) - ty) < 6 Then
                                adjustDirection = 0
                            Else
                                adjustDirection = 6
                            End If
                        End If
                        If Me.Cursor = Cursors.SizeNESW Then
                            If Math.Abs(CType(absOffsetY - (backgroundPicture.Height / 2 * tScale - toffsetY) * zoomfactor, Integer) - ty) < 6 Then
                                adjustDirection = 1.5
                            Else
                                adjustDirection = 7.5
                            End If
                        End If
                        If Me.Cursor = Cursors.SizeNWSE Then
                            If Math.Abs(CType(absOffsetY - (backgroundPicture.Height / 2 * tScale - toffsetY) * zoomfactor, Integer) - ty) < 6 Then
                                adjustDirection = 10.5
                            Else
                                adjustDirection = 4.5
                            End If
                        End If
                    ElseIf Math.Abs(CType(absOffsetX - (backgroundPicture.Width / 2 * tScale - toffsetX) * zoomfactor, Integer) - tx) < 6 OrElse Math.Abs(CType(absOffsetX - (backgroundPicture.Width / 2 * tScale - toffsetX) * zoomfactor, Integer) + backgroundPicture.Width * zoomfactor * tScale - tx) < 6 Then
                        Me.Cursor = Cursors.SizeWE
                        If Math.Abs(CType(absOffsetX - (backgroundPicture.Width / 2 * tScale - toffsetX) * zoomfactor, Integer) - tx) < 6 Then
                            adjustDirection = 9
                        Else
                            adjustDirection = 3
                        End If
                    Else
                        Me.Cursor = Cursors.Cross
                    End If
                Else
                    Me.Cursor = Cursors.Cross
                End If
                tempAdjustDirection = adjustDirection
            End If
        Else
            Me.Cursor = Cursors.Cross
        End If
        If zoomScroll Then
            If klickY - getCursorpositionY() < -10 Then
                klickY = getCursorpositionY()
                SBZoom_Scroll(Me, New ScrollEventArgs(ScrollEventType.SmallIncrement, 0, 1, ScrollOrientation.VerticalScroll))
            ElseIf klickY - getCursorpositionY() > 10 Then
                klickY = getCursorpositionY()
                SBZoom_Scroll(Me, New ScrollEventArgs(ScrollEventType.SmallDecrement, 0, -1, ScrollOrientation.VerticalScroll))
            End If
        End If
    End Sub

    Private Sub MainForm_MouseUp(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseUp
        If Not isLoading Then
            If e.Button = Windows.Forms.MouseButtons.Left Then
                If adjustmode Then
                    isLoading = True
                    doAdjust = False
                    Dim vabsOffsetX As Integer = Fix((getCursorpositionX() - klickX) / zoomfactor)
                    Dim vabsOffsetY As Integer = Fix((getCursorpositionY() - klickY) / zoomfactor)
                    Dim absOffsetX As Integer = Me.offsetX * zoomfactor + CType(Me.ClientSize.Width / 2, Integer)
                    Dim absOffsetY As Integer = Me.offsetY * zoomfactor + CType(Me.ClientSize.Height / 2, Integer)
                    klickX = getCursorpositionX()
                    klickY = getCursorpositionY()
                    If Me.Cursor = Cursors.SizeAll Then
                        NUDoffsetX.Value = clipf(NUDoffsetX.Value + vabsOffsetX, NUDoffsetX.Minimum, NUDoffsetX.Maximum)
                        NUDoffsetY.Value = clipf(NUDoffsetY.Value + vabsOffsetY, NUDoffsetY.Minimum, NUDoffsetY.Maximum)
                    ElseIf Me.Cursor = Cursors.SizeNESW OrElse Me.Cursor = Cursors.SizeNWSE OrElse Me.Cursor = Cursors.SizeWE OrElse Me.Cursor = Cursors.SizeNS Then
                        If getCursorpositionX() < CType(absOffsetX + imgOffsetX * zoomfactor, Integer) Then vabsOffsetX *= -1
                        If getCursorpositionY() < CType(absOffsetY + imgOffsetY * zoomfactor, Integer) Then vabsOffsetY *= -1
                        If Me.Cursor = Cursors.SizeWE Then
                            NUDScale.Value = clipf(NUDScale.Value + ((backgroundPicture.Width * imgScale + vabsOffsetX) / backgroundPicture.Width - imgScale) * 2, NUDScale.Minimum, NUDScale.Maximum)
                        Else
                            NUDScale.Value = clipf(NUDScale.Value + ((backgroundPicture.Height * imgScale + vabsOffsetY) / backgroundPicture.Height - imgScale) * 2, NUDScale.Minimum, NUDScale.Maximum)
                        End If
                        If Control.ModifierKeys = 131072 Then adjustDirection = -1 Else adjustDirection = tempAdjustDirection
                        Select Case adjustDirection
                            Case 0 : NUDoffsetY.Value = clipf(NUDoffsetY.Value - vabsOffsetY, NUDoffsetY.Minimum, NUDoffsetY.Maximum)
                            Case 1.5 : NUDoffsetX.Value = clipf(NUDoffsetX.Value + backgroundPicture.Width / backgroundPicture.Height * vabsOffsetY, NUDoffsetX.Minimum, NUDoffsetX.Maximum) : NUDoffsetY.Value = clipf(NUDoffsetY.Value - vabsOffsetY, NUDoffsetY.Minimum, NUDoffsetY.Maximum)
                            Case 3 : NUDoffsetX.Value = clipf(NUDoffsetX.Value + vabsOffsetX, NUDoffsetX.Minimum, NUDoffsetX.Maximum)
                            Case 4.5 : NUDoffsetX.Value = clipf(NUDoffsetX.Value + backgroundPicture.Width / backgroundPicture.Height * vabsOffsetY, NUDoffsetX.Minimum, NUDoffsetX.Maximum) : NUDoffsetY.Value = clipf(NUDoffsetY.Value + vabsOffsetY, NUDoffsetY.Minimum, NUDoffsetY.Maximum)
                            Case 6 : NUDoffsetY.Value = clipf(NUDoffsetY.Value + vabsOffsetY, NUDoffsetY.Minimum, NUDoffsetY.Maximum)
                            Case 7.5 : NUDoffsetX.Value = clipf(NUDoffsetX.Value - backgroundPicture.Width / backgroundPicture.Height * vabsOffsetY, NUDoffsetX.Minimum, NUDoffsetX.Maximum) : NUDoffsetY.Value = clipf(NUDoffsetY.Value + vabsOffsetY, NUDoffsetY.Minimum, NUDoffsetY.Maximum)
                            Case 9 : NUDoffsetX.Value = clipf(NUDoffsetX.Value - vabsOffsetX, NUDoffsetX.Minimum, NUDoffsetX.Maximum)
                            Case 10.5 : NUDoffsetX.Value = clipf(NUDoffsetX.Value - backgroundPicture.Width / backgroundPicture.Height * vabsOffsetY, NUDoffsetX.Minimum, NUDoffsetX.Maximum) : NUDoffsetY.Value = clipf(NUDoffsetY.Value - vabsOffsetY, NUDoffsetY.Minimum, NUDoffsetY.Maximum)
                        End Select
                    End If
                    isLoading = False
                    Exit Sub
                End If
                If primitiveMode = 0 Then
                    If BtnAddVertex.Checked Then
                        addHistoryI(withDataFrom.Vertices + withDataFrom.Primitives + withDataFrom.SelectedVertices)
                        Vertices.Add(New Vertex(Math.Round(getXcoordinate(getCursorpositionX()) / Me.moveSnap) * Me.moveSnap, Math.Round(getYcoordinate(getCursorpositionY()) / Me.moveSnap) * Me.moveSnap, False))
                        addHistoryII(withDataFrom.Vertices + withDataFrom.Primitives + withDataFrom.SelectedVertices)
                    ElseIf Not BtnAddTriangle.Checked Then
                        addHistoryI(withDataFrom.Vertices + withDataFrom.Primitives + withDataFrom.SelectedVertices)
                    End If
                    Select Case bearbeitungsmodus
                        Case editMode.Selection

                            ' CTRL?
                            If Not Control.ModifierKeys = 131072 Or objectToModify = "Primitive" Then
                                SelectedVertices.Clear()
                                For Each tri As Triangle In SelectedTriangles
                                    tri.selected = False
                                Next
                                SelectedTriangles.Clear()
                            End If

                            Dim minX, maxX, minY, maxY, delta As Single
                            minX = getXcoordinate(Math.Max(klickX, getCursorpositionX))
                            minY = getYcoordinate(Math.Min(klickY, getCursorpositionY))
                            maxX = getXcoordinate(Math.Min(klickX, getCursorpositionX))
                            maxY = getYcoordinate(Math.Max(klickY, getCursorpositionY))

                            delta = Math.Sqrt((klickX - getCursorpositionX()) ^ 2 + (klickY - getCursorpositionY()) ^ 2)
                            If delta < 5 Then
                                If objectToModify = "Triangle" Then
                                    beamorig(0) = getXcoordinate(getCursorpositionX())
                                    beamorig(1) = getYcoordinate(getCursorpositionY())
                                    beamorig(2) = 0
                                    Dim vert0(2) As Decimal
                                    Dim vert1(2) As Decimal
                                    Dim vert2(2) As Decimal
                                    vert0(2) = 0 : vert1(2) = 0 : vert2(2) = 0
                                    For Each tri As Triangle In Triangles
                                        vert0(0) = tri.vertexA.X
                                        vert0(1) = tri.vertexA.Y
                                        vert1(0) = tri.vertexB.X
                                        vert1(1) = tri.vertexB.Y
                                        vert2(0) = tri.vertexC.X
                                        vert2(1) = tri.vertexC.Y
                                        If PowerRay.SCHNITTPKT_DREIECK(beamorig, beamdir, vert0, vert1, vert2) AndAlso Not SelectedTriangles.Contains(tri) AndAlso tri.groupindex < 0 Then
                                            tri.selected = True
                                            If tri.groupindex < 0 Then SelectedTriangles.Add(tri)
                                            If Not SelectedVertices.Contains(tri.vertexA) AndAlso tri.vertexA.groupindex < 0 Then SelectedVertices.Add(tri.vertexA)
                                            If Not SelectedVertices.Contains(tri.vertexB) AndAlso tri.vertexB.groupindex < 0 Then SelectedVertices.Add(tri.vertexB)
                                            If Not SelectedVertices.Contains(tri.vertexC) AndAlso tri.vertexC.groupindex < 0 Then SelectedVertices.Add(tri.vertexC)
                                            Exit For
                                        End If
                                    Next
                                ElseIf objectToModify = "Vertex" Then
                                    minX -= 9 / zoomfactor : minY -= 9 / zoomfactor
                                    maxX += 9 / zoomfactor : maxY += 9 / zoomfactor
                                    Dim previous_vert As Vertex = Nothing
                                    Dim min_vert As Vertex = Nothing
                                    Dim minDelta As Single = Single.MaxValue
                                    For Each vert As Vertex In Vertices
                                        If Not Control.ModifierKeys = 131072 Then vert.selected = False
                                        If vert.X >= minX AndAlso vert.X <= maxX AndAlso _
                                           vert.Y >= minY AndAlso vert.Y <= maxY Then
                                            delta = Math.Sqrt((vert.X - (minX + maxX) / 2) ^ 2 + (vert.Y - (minY + maxY) / 2) ^ 2)
                                            If delta < minDelta Then
                                                minDelta = delta
                                                If Not vert.selected Then
                                                    min_vert = vert
                                                    If Not previous_vert Is Nothing Then previous_vert.selected = False
                                                    vert.selected = True
                                                    previous_vert = vert
                                                ElseIf Not Control.ModifierKeys = 131072 Then
                                                    vert.selected = False
                                                End If
                                            ElseIf Not Control.ModifierKeys = 131072 Then
                                                vert.selected = False
                                            End If
                                        ElseIf Not Control.ModifierKeys = 131072 Then
                                            vert.selected = False
                                        End If
                                    Next
                                    If Not min_vert Is Nothing AndAlso (min_vert.groupindex < 0 OrElse BtnAddTriangle.Checked) Then SelectedVertices.Add(min_vert)
                                End If
                            ElseIf objectToModify <> "Primitive" Then
                                For Each vert As Vertex In Vertices
                                    If Not Control.ModifierKeys = 131072 Then vert.selected = False
                                    If vert.X >= minX AndAlso vert.X <= maxX AndAlso _
                                       vert.Y >= minY AndAlso vert.Y <= maxY Then
                                        If Not vert.selected AndAlso vert.groupindex < 0 Then
                                            vert.selected = True
                                            SelectedVertices.Add(vert)
                                        End If
                                    ElseIf Not Control.ModifierKeys = 131072 Then
                                        vert.selected = False
                                    End If
                                Next
                            End If

                            If objectToModify = "Triangle" AndAlso delta >= 5 Then

                                For Each vert As Vertex In SelectedVertices
                                    vert.selected = True
                                Next
                                For Each tri As Triangle In Triangles
                                    If tri.vertexA.selected AndAlso tri.vertexB.selected AndAlso tri.vertexC.selected Then
                                        If Not tri.selected AndAlso tri.groupindex = -1 Then
                                            SelectedTriangles.Add(tri)
                                            tri.selected = True
                                        End If
                                    ElseIf Not Control.ModifierKeys = 131072 Then
                                        tri.selected = False
                                    End If
                                Next
                                For Each tri As Triangle In SelectedTriangles
                                    tri.vertexA.selected = False
                                    tri.vertexB.selected = False
                                    tri.vertexC.selected = False
                                Next
                                Dim start As Integer = 0
nextVertex:
                                For i As Integer = start To SelectedVertices.Count - 1
                                    Dim vert As Vertex = SelectedVertices(i)
                                    If vert.selected Then vert.selected = False : SelectedVertices.Remove(vert) : start = i : GoTo nextVertex
                                Next

                            End If

                            If objectToModify = "Primitive" Then
                                beamorig(0) = getXcoordinate(getCursorpositionX())
                                beamorig(1) = getYcoordinate(getCursorpositionY())
                                beamorig(2) = 0
                                Dim vert0(2) As Decimal
                                Dim vert1(2) As Decimal
                                Dim vert2(2) As Decimal
                                vert0(2) = 0 : vert1(2) = 0 : vert2(2) = 0
                                For Each tri As Triangle In Triangles
                                    vert0(0) = tri.vertexA.X
                                    vert0(1) = tri.vertexA.Y
                                    vert1(0) = tri.vertexB.X
                                    vert1(1) = tri.vertexB.Y
                                    vert2(0) = tri.vertexC.X
                                    vert2(1) = tri.vertexC.Y
                                    If PowerRay.SCHNITTPKT_DREIECK(beamorig, beamdir, vert0, vert1, vert2) AndAlso Not SelectedTriangles.Contains(tri) AndAlso tri.groupindex >= 0 Then
                                        selectGroupDirect(tri.groupindex)
                                        Exit For
                                    End If
                                Next
                            End If

                            BtnPipette.Enabled = SelectedTriangles.Count = 1 OrElse (objectToModify = "Primitive" AndAlso SelectedTriangles.Count >= 1)

                            If SelectedVertices.Count = 1 Then
                                If BtnAddTriangle.Checked Then
                                    lastPointX = SelectedVertices(0).X
                                    lastPointY = SelectedVertices(0).Y
                                    If trianglemode = 2 AndAlso Not temp_vertices(0).Equals(SelectedVertices(0)) AndAlso Not temp_vertices(1).Equals(SelectedVertices(0)) Then
                                        addHistoryI(withDataFrom.Triangles + withDataFrom.SelectedTriangles)
                                        Triangles.Add(New Triangle(temp_vertices(0), temp_vertices(1), SelectedVertices(0)))
                                        LLast(Triangles).myColour = lastColour
                                        LLast(Triangles).myColourNumber = lastColourNumber
                                        temp_vertices(0).linkedTriangles.Add(LLast(Triangles))
                                        temp_vertices(1).linkedTriangles.Add(LLast(Triangles))
                                        SelectedVertices(0).linkedTriangles.Add(LLast(Triangles))
                                        trianglemode = 0
                                        addHistoryII(withDataFrom.Triangles + withDataFrom.SelectedTriangles)
                                        If FastTriangulationToolStripMenuItem.Checked Then
                                            temp_vertices(0) = SelectedVertices(0)
                                            trianglemode = 1
                                        End If
                                    Else
                                        If trianglemode = 1 AndAlso Not temp_vertices(0).Equals(SelectedVertices(0)) Then
                                            temp_vertices(1) = SelectedVertices(0)
                                            trianglemode = 2
                                        End If
                                        If trianglemode = 0 Then
                                            temp_vertices(0) = SelectedVertices(0)
                                            trianglemode = 1
                                        End If
                                    End If
                                Else
                                    If Not BtnAddVertex.Checked Then addHistoryII(withDataFrom.SelectedTriangles + withDataFrom.SelectedVertices)
                                End If
                            Else
                                If Not BtnAddVertex.Checked AndAlso Not BtnAddTriangle.Checked Then addHistoryII(withDataFrom.SelectedTriangles + withDataFrom.SelectedVertices)
                            End If

                            If objectToModify <> "Vertex" Then
                                BtnAddToGroup.Enabled = SelectedTriangles.Count > 0 AndAlso SelectedVertices.Count > 0 AndAlso Not objectToModify = "Primitive"
                                BtnUngroup.Enabled = SelectedTriangles.Count > 0 AndAlso SelectedVertices.Count > 0 AndAlso objectToModify = "Primitive"
                            End If

                            auswahl = False
                            Exit Select
                        Case editMode.Translation
                            Dim vabsOffsetX As Integer = Fix((getCursorpositionX() - klickX) / zoomfactor / moveSnap) * moveSnap
                            Dim vabsOffsetY As Integer = Fix((getCursorpositionY() - klickY) / zoomfactor / moveSnap) * moveSnap
                            If Control.ModifierKeys = 131072 Then vabsOffsetX = 0
                            If Control.ModifierKeys = Keys.Shift Then vabsOffsetY = 0
                            addHistoryI(withDataFrom.Vertices + withDataFrom.Primitives)
                            If templateProjectionQuad.Count > 0 AndAlso SelectedVertices.Count = 1 Then
                                If SelectedVertices(0).linkedTriangles.Count > 0 Then
                                    Do
                                        For i = 0 To templateProjectionQuad.Count - 1
                                            If templateProjectionQuad(i).isInQuad(SelectedVertices(0).linkedTriangles(0)) Then
                                                Exit Do
                                            End If
                                        Next
                                        Dim newV As Vertex
                                        Dim tmpV As Vertex = SelectedVertices(0) + New Vertex(-vabsOffsetX, vabsOffsetY, False, False)
                                        tmpV.vertexID = SelectedVertices(0).vertexID
                                        For i = 0 To templateProjectionQuad.Count - 1
                                            newV = CSG2ProjectionQuadIntersectionVertex(tmpV, SelectedVertices(0).linkedTriangles(0), templateProjectionQuad(i))
                                            If Not newV Is Nothing Then
                                                vabsOffsetX = SelectedVertices(0).X - newV.X
                                                vabsOffsetY = newV.Y - SelectedVertices(0).Y
                                            End If
                                        Next
                                        Exit Do
                                    Loop
                                End If
                            End If
                            For Each vert As Vertex In SelectedVertices
                                vert.X -= vabsOffsetX
                                vert.Y += vabsOffsetY
                            Next
                            If SelectedVertices.Count > 0 AndAlso objectToModify = "Primitive" Then
                                Primitives(CType(PrimitivesHMap(SelectedVertices(0).groupindex), Integer)).translate(-vabsOffsetX, -vabsOffsetY)
                            End If
                            movemode = False
                            addHistoryII(withDataFrom.Vertices + withDataFrom.Primitives)
                            Exit Select
                        Case editMode.Rotation
                            rotatemode = False
                            Dim angleToRotate As Double = Fix((-(getCursorpositionX() - klickX) - (getCursorpositionY() - klickY)) / 100 / (rotateSnap * Math.PI / 180)) * (rotateSnap * Math.PI / 180)
                            addHistoryI(withDataFrom.Vertices + withDataFrom.Primitives)
                            For Each vert As Vertex In SelectedVertices
                                vert.X = temp_center.X + vert.distanceFrom * Math.Cos(vert.angleFrom + angleToRotate)
                                vert.Y = temp_center.Y + vert.distanceFrom * Math.Sin(vert.angleFrom + angleToRotate)
                                If Single.IsNaN(vert.X) OrElse Single.IsNaN(vert.Y) Then
                                    vert.X = vert.oldX
                                    vert.Y = vert.oldY
                                End If
                            Next
                            If SelectedVertices.Count > 0 AndAlso objectToModify = "Primitive" Then
                                Primitives(CType(PrimitivesHMap(SelectedVertices(0).groupindex), Integer)).rotate(angleToRotate)
                            End If
                            addHistoryII(withDataFrom.Vertices + withDataFrom.Primitives)
                            Exit Select
                        Case editMode.Scale
                            scalemode = False
                            Dim factorToScale As Double = 1 + Fix((getCursorpositionX() - klickX) / 100 / scaleSnap) * scaleSnap
                            If factorToScale < 0.1 Then factorToScale = 0.1
                            If factorToScale > 2 Then factorToScale = 2
                            addHistoryI(withDataFrom.Vertices + withDataFrom.Primitives)
                            For Each vert As Vertex In SelectedVertices
                                If Not Control.ModifierKeys = 131072 Then
                                    vert.X = temp_center.X + factorToScale * vert.distanceFrom * Math.Cos(vert.angleFrom)
                                Else
                                    vert.X = vert.oldX
                                End If
                                If Not Control.ModifierKeys = Keys.Shift Then
                                    vert.Y = temp_center.Y + factorToScale * vert.distanceFrom * Math.Sin(vert.angleFrom)
                                Else
                                    vert.Y = vert.oldY
                                End If
                                If Single.IsNaN(vert.X) OrElse Single.IsNaN(vert.Y) Then
                                    vert.X = vert.oldX
                                    vert.Y = vert.oldY
                                End If
                            Next
                            If SelectedVertices.Count > 0 AndAlso objectToModify = "Primitive" Then
                                Dim factorToScaleX As Double = 1
                                Dim factorToScaleY As Double = 1
                                If Not Control.ModifierKeys = 131072 Then factorToScaleX = factorToScale
                                If Not Control.ModifierKeys = Keys.Shift Then factorToScaleY = factorToScale
                                Primitives(CType(PrimitivesHMap(SelectedVertices(0).groupindex), Integer)).scale(factorToScaleX, factorToScaleY)
                            End If
                            addHistoryII(withDataFrom.Vertices + withDataFrom.Primitives)
                            Exit Select
                    End Select
                Else
                    If primitiveMode = 255 Then
                        addHistoryI(withDataFrom.Vertices + withDataFrom.Triangles + withDataFrom.SelectedVertices + withDataFrom.SelectedTriangles)
                        Vertices.Add(New Vertex(Math.Round(getXcoordinate(getCursorpositionX()) / Me.moveSnap) * Me.moveSnap, Math.Round(getYcoordinate(getCursorpositionY()) / Me.moveSnap) * Me.moveSnap, True))
                        Triangles.Add(New Triangle(Vertices(Vertices.Count - 1), Vertices(Vertices.Count - 2), Vertices(Vertices.Count - 3)) With {.myColour = lastColour, .myColourNumber = lastColourNumber})
                        Vertices(Vertices.Count - 1).linkedTriangles.Add(LLast(Triangles))
                        Vertices(Vertices.Count - 2).linkedTriangles.Add(LLast(Triangles))
                        Vertices(Vertices.Count - 3).linkedTriangles.Add(LLast(Triangles))
                        SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                        SelectedVertices.Add(LLast(Vertices))
                        addHistoryII(withDataFrom.Vertices + withDataFrom.Triangles + withDataFrom.SelectedVertices + withDataFrom.SelectedTriangles)
                    End If
                    If primitiveMode = 4 Then
                        addHistoryI(withDataFrom.Vertices + withDataFrom.Triangles + withDataFrom.SelectedVertices + withDataFrom.SelectedTriangles)
                        primitiveBordersize = Fix(getXcoordinate(getCursorpositionX()) - primitiveCenter.X) / Math.Sqrt(primitiveWidth ^ 2 + primitiveHeight ^ 2)
                        If primitiveBordersize < 0 Then primitiveBordersize *= -1
                        clip(primitiveBordersize, 0, 1)
                        If primitiveObject < 3 Then
                            For Each tri As Triangle In SelectedTriangles
                                tri.selected = False
                            Next
                            For Each vert As Vertex In SelectedVertices
                                vert.selected = False
                            Next
                            SelectedTriangles.Clear()
                            SelectedVertices.Clear()
                            Dim v11, v21, v31 As Vertex
                            v11 = New Vertex(primitiveCenter.X, primitiveCenter.Y + primitiveHeight, False)
                            v21 = New Vertex(primitiveCenter.X + primitiveHeight + primitiveWidth, primitiveCenter.Y - (primitiveHeight / 2), False)
                            v31 = New Vertex(primitiveCenter.X - primitiveHeight - primitiveWidth, primitiveCenter.Y - (primitiveHeight / 2), False)
                            Dim v12, v22, v32 As Vertex
                            v12 = New Vertex(primitiveCenter.X, primitiveCenter.Y + primitiveHeight * primitiveBordersize, False)
                            v22 = New Vertex(primitiveCenter.X + (primitiveHeight + primitiveWidth) * primitiveBordersize, primitiveCenter.Y - (primitiveHeight / 2) * primitiveBordersize, False)
                            v32 = New Vertex(primitiveCenter.X - (primitiveHeight + primitiveWidth) * primitiveBordersize, primitiveCenter.Y - (primitiveHeight / 2) * primitiveBordersize, False)
                            Vertices.Add(v11) : Vertices.Add(v12) ' 6 5
                            Vertices.Add(v21) : Vertices.Add(v22) ' 4 3 
                            Vertices.Add(v31) : Vertices.Add(v32) ' 2 1
                            If primitiveObject = 1 Then
                                Triangles.Add(New Triangle(v12, v22, v32) With {.myColour = lastColour, .myColourNumber = lastColourNumber})
                                Vertices(Vertices.Count - 1).linkedTriangles.Add(LLast(Triangles))
                                Vertices(Vertices.Count - 3).linkedTriangles.Add(LLast(Triangles))
                                Vertices(Vertices.Count - 5).linkedTriangles.Add(LLast(Triangles))
                                SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                            End If
                            Triangles.Add(New Triangle(v11, v22, v21) With {.myColour = lastColour, .myColourNumber = lastColourNumber})
                            Vertices(Vertices.Count - 6).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 3).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 4).linkedTriangles.Add(LLast(Triangles))
                            SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                            Triangles.Add(New Triangle(v11, v22, v12) With {.myColour = lastColour, .myColourNumber = lastColourNumber})
                            Vertices(Vertices.Count - 6).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 3).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 5).linkedTriangles.Add(LLast(Triangles))
                            SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                            Triangles.Add(New Triangle(v21, v32, v31) With {.myColour = lastColour, .myColourNumber = lastColourNumber})
                            Vertices(Vertices.Count - 4).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 1).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 2).linkedTriangles.Add(LLast(Triangles))
                            SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                            Triangles.Add(New Triangle(v21, v32, v22) With {.myColour = lastColour, .myColourNumber = lastColourNumber})
                            Vertices(Vertices.Count - 4).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 1).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 3).linkedTriangles.Add(LLast(Triangles))
                            SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                            Triangles.Add(New Triangle(v31, v12, v11) With {.myColour = lastColour, .myColourNumber = lastColourNumber})
                            Vertices(Vertices.Count - 2).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 5).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 6).linkedTriangles.Add(LLast(Triangles))
                            SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                            Triangles.Add(New Triangle(v31, v12, v32) With {.myColour = lastColour, .myColourNumber = lastColourNumber})
                            Vertices(Vertices.Count - 2).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 5).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 1).linkedTriangles.Add(LLast(Triangles))
                            SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                            SelectedVertices.Add(Vertices(Vertices.Count - 1))
                            SelectedVertices.Add(Vertices(Vertices.Count - 2))
                            SelectedVertices.Add(Vertices(Vertices.Count - 3))
                            SelectedVertices.Add(Vertices(Vertices.Count - 4))
                            SelectedVertices.Add(Vertices(Vertices.Count - 5))
                            SelectedVertices.Add(Vertices(Vertices.Count - 6))
                        End If
                        If primitiveObject = 4 OrElse primitiveObject = 5 Then
                            For Each tri As Triangle In SelectedTriangles
                                tri.selected = False
                            Next
                            For Each vert As Vertex In SelectedVertices
                                vert.selected = False
                            Next
                            SelectedTriangles.Clear()
                            SelectedVertices.Clear()
                            Dim v11, v21, v31, v41, v12, v22, v32, v42 As Vertex
                            v11 = New Vertex(primitiveCenter.X + primitiveWidth, primitiveCenter.Y + primitiveHeight, False)
                            v21 = New Vertex(primitiveCenter.X - primitiveWidth, primitiveCenter.Y + primitiveHeight, False)
                            v31 = New Vertex(primitiveCenter.X - primitiveWidth, primitiveCenter.Y - primitiveHeight, False)
                            v41 = New Vertex(primitiveCenter.X + primitiveWidth, primitiveCenter.Y - primitiveHeight, False)
                            v12 = New Vertex(primitiveCenter.X + primitiveWidth * primitiveBordersize, primitiveCenter.Y + primitiveHeight * primitiveBordersize, False)
                            v22 = New Vertex(primitiveCenter.X - primitiveWidth * primitiveBordersize, primitiveCenter.Y + primitiveHeight * primitiveBordersize, False)
                            v32 = New Vertex(primitiveCenter.X - primitiveWidth * primitiveBordersize, primitiveCenter.Y - primitiveHeight * primitiveBordersize, False)
                            v42 = New Vertex(primitiveCenter.X + primitiveWidth * primitiveBordersize, primitiveCenter.Y - primitiveHeight * primitiveBordersize, False)
                            Vertices.Add(v11) : Vertices.Add(v12) '8 7
                            Vertices.Add(v21) : Vertices.Add(v22) '6 5
                            Vertices.Add(v31) : Vertices.Add(v32) '4 3
                            Vertices.Add(v41) : Vertices.Add(v42) '2 1
                            If primitiveObject = 4 Then
                                Triangles.Add(New Triangle(v12, v22, v32) With {.myColour = lastColour, .myColourNumber = lastColourNumber})
                                Vertices(Vertices.Count - 7).linkedTriangles.Add(LLast(Triangles))
                                Vertices(Vertices.Count - 5).linkedTriangles.Add(LLast(Triangles))
                                Vertices(Vertices.Count - 3).linkedTriangles.Add(LLast(Triangles))
                                SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                                Triangles.Add(New Triangle(v32, v42, v12) With {.myColour = lastColour, .myColourNumber = lastColourNumber})
                                Vertices(Vertices.Count - 3).linkedTriangles.Add(LLast(Triangles))
                                Vertices(Vertices.Count - 1).linkedTriangles.Add(LLast(Triangles))
                                Vertices(Vertices.Count - 7).linkedTriangles.Add(LLast(Triangles))
                                SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                            End If
                            Triangles.Add(New Triangle(v11, v22, v21) With {.myColour = lastColour, .myColourNumber = lastColourNumber})
                            Vertices(Vertices.Count - 8).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 5).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 6).linkedTriangles.Add(LLast(Triangles))
                            SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                            Triangles.Add(New Triangle(v11, v22, v12) With {.myColour = lastColour, .myColourNumber = lastColourNumber})
                            Vertices(Vertices.Count - 8).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 5).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 7).linkedTriangles.Add(LLast(Triangles))
                            SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                            Triangles.Add(New Triangle(v21, v32, v31) With {.myColour = lastColour, .myColourNumber = lastColourNumber})
                            Vertices(Vertices.Count - 6).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 3).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 4).linkedTriangles.Add(LLast(Triangles))
                            SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                            Triangles.Add(New Triangle(v21, v32, v22) With {.myColour = lastColour, .myColourNumber = lastColourNumber})
                            Vertices(Vertices.Count - 6).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 3).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 5).linkedTriangles.Add(LLast(Triangles))
                            SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                            Triangles.Add(New Triangle(v31, v42, v41) With {.myColour = lastColour, .myColourNumber = lastColourNumber})
                            Vertices(Vertices.Count - 4).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 1).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 2).linkedTriangles.Add(LLast(Triangles))
                            SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                            Triangles.Add(New Triangle(v31, v42, v32) With {.myColour = lastColour, .myColourNumber = lastColourNumber})
                            Vertices(Vertices.Count - 4).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 1).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 3).linkedTriangles.Add(LLast(Triangles))
                            SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                            Triangles.Add(New Triangle(v41, v12, v11) With {.myColour = lastColour, .myColourNumber = lastColourNumber})
                            Vertices(Vertices.Count - 2).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 7).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 8).linkedTriangles.Add(LLast(Triangles))
                            SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                            Triangles.Add(New Triangle(v41, v12, v42) With {.myColour = lastColour, .myColourNumber = lastColourNumber})
                            Vertices(Vertices.Count - 2).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 7).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 1).linkedTriangles.Add(LLast(Triangles))
                            SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                            SelectedVertices.Add(Vertices(Vertices.Count - 1))
                            SelectedVertices.Add(Vertices(Vertices.Count - 2))
                            SelectedVertices.Add(Vertices(Vertices.Count - 3))
                            SelectedVertices.Add(Vertices(Vertices.Count - 4))
                            SelectedVertices.Add(Vertices(Vertices.Count - 5))
                            SelectedVertices.Add(Vertices(Vertices.Count - 6))
                            SelectedVertices.Add(Vertices(Vertices.Count - 7))
                            SelectedVertices.Add(Vertices(Vertices.Count - 8))
                        End If
                        If primitiveObject = 7 OrElse primitiveObject = 8 OrElse primitiveObject = 10 OrElse primitiveObject = 11 Then
                            Dim segments_temp As Integer
                            If primitiveObject < 10 Then segments_temp = segments_circle Else segments_temp = segments_oval
                            For Each tri As Triangle In SelectedTriangles
                                tri.selected = False
                            Next
                            For Each vert As Vertex In SelectedVertices
                                vert.selected = False
                            Next
                            SelectedTriangles.Clear()
                            SelectedVertices.Clear()
                            Dim vcenter As Vertex = Nothing
                            Dim vinner(segments_temp) As Vertex
                            Dim vouter(segments_temp) As Vertex
                            Dim counter As Integer = 0
                            If primitiveObject = 7 OrElse primitiveObject = 10 Then
                                vcenter = New Vertex(primitiveCenter.X, primitiveCenter.Y, False)
                                Vertices.Add(vcenter)
                                SelectedVertices.Add(LLast(Vertices))
                            End If
                            For a As Double = 0 To Math.PI * 2 Step Math.PI * 2 / segments_temp
                                vinner(counter) = New Vertex(primitiveCenter.X + Math.Cos(a) * primitiveWidth * primitiveBordersize, primitiveCenter.Y + Math.Sin(a) * primitiveHeight * primitiveBordersize, False)
                                Vertices.Add(vinner(counter))
                                counter += 1
                            Next
                            If primitiveObject = 7 OrElse primitiveObject = 10 Then
                                Triangles.Add(New Triangle(vcenter, vinner(segments_temp - 1), vinner(0)) With {.myColour = lastColour, .myColourNumber = lastColourNumber})
                                Vertices(Vertices.Count - segments_temp - 1).linkedTriangles.Add(LLast(Triangles))
                                Vertices(Vertices.Count - 1).linkedTriangles.Add(LLast(Triangles))
                                Vertices(Vertices.Count - segments_temp).linkedTriangles.Add(LLast(Triangles))
                                SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                                For counter = 1 To segments_temp - 1
                                    Triangles.Add(New Triangle(vcenter, vinner(counter), vinner(counter - 1)) With {.myColour = lastColour, .myColourNumber = lastColourNumber})
                                    Vertices(Vertices.Count - segments_temp - 1).linkedTriangles.Add(LLast(Triangles))
                                    Vertices(Vertices.Count - segments_temp + counter).linkedTriangles.Add(LLast(Triangles))
                                    Vertices(Vertices.Count - segments_temp + counter - 1).linkedTriangles.Add(LLast(Triangles))
                                    SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                                Next
                            End If
                            counter = 0
                            For a As Double = 0 To Math.PI * 2 Step Math.PI * 2 / segments_temp
                                vouter(counter) = New Vertex(primitiveCenter.X + Math.Cos(a) * primitiveWidth, primitiveCenter.Y + Math.Sin(a) * primitiveHeight, False)
                                Vertices.Add(vouter(counter))
                                counter += 1
                            Next
                            Triangles.Add(New Triangle(vouter(segments_temp - 1), vinner(segments_temp - 1), vinner(0)) With {.myColour = lastColour, .myColourNumber = lastColourNumber})
                            Vertices(Vertices.Count - 1).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 1 - segments_temp).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - segments_temp * 2).linkedTriangles.Add(LLast(Triangles))
                            SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                            Triangles.Add(New Triangle(vouter(0), vouter(segments_temp - 1), vinner(0)) With {.myColour = lastColour, .myColourNumber = lastColourNumber})
                            Vertices(Vertices.Count - segments_temp).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 1).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - segments_temp * 2).linkedTriangles.Add(LLast(Triangles))
                            SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                            For counter = 1 To segments_temp - 1
                                Triangles.Add(New Triangle(vouter(counter - 1), vinner(counter - 1), vinner(counter)) With {.myColour = lastColour, .myColourNumber = lastColourNumber})
                                Vertices(Vertices.Count - segments_temp + counter - 1).linkedTriangles.Add(LLast(Triangles))
                                Vertices(Vertices.Count - segments_temp * 2 + counter - 1).linkedTriangles.Add(LLast(Triangles))
                                Vertices(Vertices.Count - segments_temp * 2 + counter).linkedTriangles.Add(LLast(Triangles))
                                SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                                Triangles.Add(New Triangle(vouter(counter), vouter(counter - 1), vinner(counter)) With {.myColour = lastColour, .myColourNumber = lastColourNumber})
                                Vertices(Vertices.Count - segments_temp + counter).linkedTriangles.Add(LLast(Triangles))
                                Vertices(Vertices.Count - segments_temp + counter - 1).linkedTriangles.Add(LLast(Triangles))
                                Vertices(Vertices.Count - segments_temp * 2 + counter).linkedTriangles.Add(LLast(Triangles))
                                SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                            Next
                            For counter = 0 To segments_temp * 2 - 1
                                SelectedVertices.Add(Vertices(Vertices.Count - counter - 1))
                            Next
                        End If
                        addHistoryII(withDataFrom.Vertices + withDataFrom.Triangles + withDataFrom.SelectedVertices + withDataFrom.SelectedTriangles)
                        abortPrimitiveMode()
                    End If
                    If primitiveMode = 3 Then
                        If Not (primitiveObject <> 0 AndAlso primitiveObject <> 3 AndAlso primitiveObject <> 9) Then
                            addHistoryI(withDataFrom.Vertices + withDataFrom.Triangles + withDataFrom.SelectedVertices + withDataFrom.SelectedTriangles)
                        End If
                        primitiveWidth = Fix(getXcoordinate(getCursorpositionX()) - primitiveCenter.X)
                        If primitiveObject = 0 Then
                            For Each tri As Triangle In SelectedTriangles
                                tri.selected = False
                            Next
                            For Each vert As Vertex In SelectedVertices
                                vert.selected = False
                            Next
                            SelectedTriangles.Clear()
                            SelectedVertices.Clear()
                            Dim v1, v2, v3 As Vertex
                            v1 = New Vertex(primitiveCenter.X, primitiveCenter.Y + primitiveHeight, True)
                            v2 = New Vertex(primitiveCenter.X + primitiveHeight, primitiveCenter.Y - (primitiveHeight / 2), True)
                            v3 = New Vertex(primitiveCenter.X - primitiveHeight, primitiveCenter.Y - (primitiveHeight / 2), True)
                            Vertices.Add(v1) : Vertices.Add(v2) : Vertices.Add(v3)
                            Triangles.Add(New Triangle(v1, v2, v3) With {.myColour = lastColour, .myColourNumber = lastColourNumber})
                            Vertices(Vertices.Count - 1).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 2).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 3).linkedTriangles.Add(LLast(Triangles))
                            LLast(Triangles).vertexB.X += primitiveWidth
                            LLast(Triangles).vertexC.X -= primitiveWidth
                            SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                            SelectedVertices.Add(Vertices(Vertices.Count - 1))
                            SelectedVertices.Add(Vertices(Vertices.Count - 2))
                            SelectedVertices.Add(Vertices(Vertices.Count - 3))
                        End If
                        If primitiveObject = 3 Then
                            For Each tri As Triangle In SelectedTriangles
                                tri.selected = False
                            Next
                            For Each vert As Vertex In SelectedVertices
                                vert.selected = False
                            Next
                            SelectedTriangles.Clear()
                            SelectedVertices.Clear()
                            Dim v1, v2, v3, v4 As Vertex
                            v1 = New Vertex(primitiveCenter.X + primitiveWidth, primitiveCenter.Y + primitiveHeight, False)
                            v2 = New Vertex(primitiveCenter.X - primitiveWidth, primitiveCenter.Y + primitiveHeight, False)
                            v3 = New Vertex(primitiveCenter.X - primitiveWidth, primitiveCenter.Y - primitiveHeight, False)
                            v4 = New Vertex(primitiveCenter.X + primitiveWidth, primitiveCenter.Y - primitiveHeight, False)
                            Vertices.Add(v1) : Vertices.Add(v2) : Vertices.Add(v3) : Vertices.Add(v4)
                            Triangles.Add(New Triangle(v1, v2, v3) With {.myColour = lastColour, .myColourNumber = lastColourNumber})
                            Vertices(Vertices.Count - 2).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 3).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 4).linkedTriangles.Add(LLast(Triangles))
                            SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                            Triangles.Add(New Triangle(v3, v4, v1) With {.myColour = lastColour, .myColourNumber = lastColourNumber})
                            Vertices(Vertices.Count - 2).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 1).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 4).linkedTriangles.Add(LLast(Triangles))
                            SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                            SelectedVertices.Add(Vertices(Vertices.Count - 1))
                            SelectedVertices.Add(Vertices(Vertices.Count - 2))
                            SelectedVertices.Add(Vertices(Vertices.Count - 3))
                            SelectedVertices.Add(Vertices(Vertices.Count - 4))
                        End If
                        If primitiveObject = 9 Then
                            For Each tri As Triangle In SelectedTriangles
                                tri.selected = False
                            Next
                            For Each vert As Vertex In SelectedVertices
                                vert.selected = False
                            Next
                            SelectedTriangles.Clear()
                            SelectedVertices.Clear()
                            primitiveWidth = Fix(Math.Sqrt((getYcoordinate(getCursorpositionY()) - primitiveCenter.Y) ^ 2 + (getXcoordinate(getCursorpositionX()) - primitiveCenter.X) ^ 2))
                            Dim vcenter As Vertex
                            Dim vouter(segments_oval) As Vertex
                            Dim counter As Integer = 0
                            vcenter = New Vertex(primitiveCenter.X, primitiveCenter.Y, False)
                            Vertices.Add(vcenter)
                            SelectedVertices.Add(LLast(Vertices))
                            For a As Double = 0 To Math.PI * 2 Step Math.PI * 2 / segments_oval
                                vouter(counter) = New Vertex(primitiveCenter.X + Math.Cos(a) * primitiveWidth, primitiveCenter.Y + Math.Sin(a) * primitiveHeight, False)
                                Vertices.Add(vouter(counter))
                                counter += 1
                            Next
                            Triangles.Add(New Triangle(vcenter, vouter(segments_oval - 1), vouter(0)) With {.myColour = lastColour, .myColourNumber = lastColourNumber})
                            Vertices(Vertices.Count - segments_oval - 1).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 1).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - segments_oval).linkedTriangles.Add(LLast(Triangles))
                            SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                            For counter = 1 To segments_oval - 1
                                Triangles.Add(New Triangle(vcenter, vouter(counter), vouter(counter - 1)) With {.myColour = lastColour, .myColourNumber = lastColourNumber})
                                Vertices(Vertices.Count - segments_oval - 1).linkedTriangles.Add(LLast(Triangles))
                                Vertices(Vertices.Count - segments_oval + counter).linkedTriangles.Add(LLast(Triangles))
                                Vertices(Vertices.Count - segments_oval + counter - 1).linkedTriangles.Add(LLast(Triangles))
                                SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                            Next
                            For counter = 0 To segments_oval - 1
                                SelectedVertices.Add(Vertices(Vertices.Count - counter - 1))
                            Next
                        End If
                        If primitiveObject <> 0 AndAlso primitiveObject <> 3 AndAlso primitiveObject <> 9 Then
                            primitiveMode = 4
                        Else
                            addHistoryII(withDataFrom.Vertices + withDataFrom.Triangles + withDataFrom.SelectedVertices + withDataFrom.SelectedTriangles)
                            abortPrimitiveMode()
                        End If
                    End If
                    If primitiveMode = 2 Then
                        primitiveHeight = Fix(getYcoordinate(getCursorpositionY()) - primitiveCenter.Y)
                        If primitiveObject < 6 Then
                            primitiveMode = 3
                        End If
                        If primitiveObject = 6 Then
                            addHistoryI(withDataFrom.Vertices + withDataFrom.Triangles + withDataFrom.SelectedVertices + withDataFrom.SelectedTriangles)
                            For Each tri As Triangle In SelectedTriangles
                                tri.selected = False
                            Next
                            For Each vert As Vertex In SelectedVertices
                                vert.selected = False
                            Next
                            SelectedTriangles.Clear()
                            SelectedVertices.Clear()
                            primitiveHeight = Fix(Math.Sqrt((getYcoordinate(getCursorpositionY()) - primitiveCenter.Y) ^ 2 + (getXcoordinate(getCursorpositionX()) - primitiveCenter.X) ^ 2))
                            Dim vcenter As Vertex
                            Dim vouter(segments_circle) As Vertex
                            Dim counter As Integer = 0
                            vcenter = New Vertex(primitiveCenter.X, primitiveCenter.Y, False)
                            Vertices.Add(vcenter)
                            SelectedVertices.Add(LLast(Vertices))
                            For a As Double = 0 To Math.PI * 2 Step Math.PI * 2 / segments_circle
                                vouter(counter) = New Vertex(primitiveCenter.X + Math.Cos(a) * primitiveHeight, primitiveCenter.Y + Math.Sin(a) * primitiveHeight, False)
                                Vertices.Add(vouter(counter))
                                counter += 1
                            Next
                            Triangles.Add(New Triangle(vcenter, vouter(segments_circle - 1), vouter(0)) With {.myColour = lastColour, .myColourNumber = lastColourNumber})
                            Vertices(Vertices.Count - segments_circle - 1).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 1).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - segments_circle).linkedTriangles.Add(LLast(Triangles))
                            SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                            For counter = 1 To segments_circle - 1
                                Triangles.Add(New Triangle(vcenter, vouter(counter), vouter(counter - 1)) With {.myColour = lastColour, .myColourNumber = lastColourNumber})
                                Vertices(Vertices.Count - segments_circle - 1).linkedTriangles.Add(LLast(Triangles))
                                Vertices(Vertices.Count - segments_circle + counter).linkedTriangles.Add(LLast(Triangles))
                                Vertices(Vertices.Count - segments_circle + counter - 1).linkedTriangles.Add(LLast(Triangles))
                                SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                            Next
                            For counter = 0 To segments_circle - 1
                                SelectedVertices.Add(Vertices(Vertices.Count - counter - 1))
                            Next
                            abortPrimitiveMode()
                            addHistoryII(withDataFrom.Vertices + withDataFrom.Triangles + withDataFrom.SelectedVertices + withDataFrom.SelectedTriangles)
                        End If
                        If primitiveObject = 7 OrElse primitiveObject = 8 Then
                            primitiveHeight = Fix(Math.Sqrt((getYcoordinate(getCursorpositionY()) - primitiveCenter.Y) ^ 2 + (getXcoordinate(getCursorpositionX()) - primitiveCenter.X) ^ 2))
                            primitiveWidth = primitiveHeight
                            primitiveMode = 4
                        End If
                        If primitiveObject = 9 OrElse primitiveObject = 10 OrElse primitiveObject = 11 Then
                            primitiveHeight = Fix(Math.Sqrt((getYcoordinate(getCursorpositionY()) - primitiveCenter.Y) ^ 2 + (getXcoordinate(getCursorpositionX()) - primitiveCenter.X) ^ 2))
                            primitiveMode = 3
                        End If
                    End If
                    If primitiveMode = 1 Then
                        primitiveCenter = New Vertex(Math.Round(getXcoordinate(getCursorpositionX()) / Me.moveSnap) * Me.moveSnap, Math.Round(getYcoordinate(getCursorpositionY()) / Me.moveSnap) * Me.moveSnap, False, False)
                        If primitiveName = "" Then
                            primitiveMode = 2
                        Else
                            abortPrimitiveMode()
                            addHistoryI(withDataFrom.Vertices + withDataFrom.Triangles + withDataFrom.Primitives + withDataFrom.PrimitivesHash)
                            inlinePrimitive("16", _
                                1, 0, 0, 0, _
                                0, 1, 0, 0, _
                                0, 0, 1, 0, _
                                1, 0, 0, 0, _
                                0, 1, 0, 0, _
                                0, 0, 1, 0, _
                                primitiveName, New List(Of String))
                            primitiveName = ""
                            PrimitiveModeToolStripMenuItem.PerformClick()
                            addHistoryII(withDataFrom.Vertices + withDataFrom.Triangles + withDataFrom.Primitives + withDataFrom.PrimitivesHash)
                        End If

                    End If
                End If
                GBVertex.Visible = False
                GBVertex.Visible = SelectedVertices.Count = 1
            ElseIf e.Button = Windows.Forms.MouseButtons.Middle Then
            ElseIf e.Button = Windows.Forms.MouseButtons.Right Then
                If Math.Abs(getCursorpositionX() - klickX) + Math.Abs(getCursorpositionY() - klickY) < 2 AndAlso Not AbortCurrentLessonToolStripMenuItem.Enabled Then
                    Me.ContextMenuStrip = CMS
                    Me.ContextMenuStrip.Show(Me, getCursorpositionX(), getCursorpositionY())
                End If
                kamera = False
                If trianglemode > 0 Then
                    trianglemode = 0
                End If
            End If
        End If
        Me.Refresh()
    End Sub

#End Region
#Region "Mathematical Functions"

    Public Function repairMatrix(ByVal matrix(,) As Double) As Double(,)
        Dim rmatrix(,) As Double = matrix.Clone
        ' All-Row/All Colum Zero:
        For r As Integer = 0 To 2
            Dim rsum As Double
            For c As Integer = 0 To 2
                rsum += Math.Abs(rmatrix(r, c))
            Next
            If Math.Abs(rsum) < 0.00001 Then
                For c As Integer = 0 To 2
                    Dim csum As Double = 0
                    For r2 As Integer = 0 To 2
                        csum += Math.Abs(rmatrix(r2, c))
                    Next
                    If Math.Abs(csum) < 0.00001 Then
                        rmatrix(r, c) = 1.0
                    End If
                Next
            End If
        Next
        For c As Integer = 0 To 2
            Dim csum As Double = 0
            For r2 As Integer = 0 To 2
                csum += Math.Abs(rmatrix(r2, c))
            Next
            If Math.Abs(csum) < 0.00001 Then
                rmatrix(c, c) = 1.0
            End If
        Next

        ' Delete singularities:
        Dim det As Single = rmatrix(0, 0) * (rmatrix(1, 1) * rmatrix(2, 2) - rmatrix(1, 2) * rmatrix(2, 1)) + rmatrix(0, 1) * (rmatrix(1, 2) * rmatrix(2, 0) - rmatrix(1, 0) * rmatrix(2, 2)) + rmatrix(0, 2) * (rmatrix(1, 0) * rmatrix(2, 1) - rmatrix(1, 1) * rmatrix(2, 0))
        If Math.Abs(det) < 0.0001 Then
            For c As Integer = 0 To 2
                If rmatrix(c, c) = 0 Then
                    rmatrix(c, c) = 1
                End If
            Next
        End If

        Return rmatrix.Clone
    End Function

    Private Function adjustPrimitiveMatrix(ByVal pmatrix(,) As Double, ByVal projectMode As Byte, Optional ByVal group As Integer = -1) As Double(,)



        Dim matrix2(,) As Double = pmatrix.Clone
        Dim matrix3(3, 3) As Double
        ' XY mit +Z (Back) [OK]
        Dim matrix(,) As Double = {{-1, 0, 0, 0}, _
                                   {0, 0, -1, 0}, _
                                   {0, 1, 0, 0}, _
                                   {0, 0, 0, 1}}

        matrix3(0, 0) = matrix2(0, 0) * matrix(0, 0) + matrix2(0, 1) * matrix(1, 0) + matrix2(0, 2) * matrix(2, 0)
        matrix3(0, 1) = matrix2(0, 0) * matrix(0, 1) + matrix2(0, 1) * matrix(1, 1) + matrix2(0, 2) * matrix(2, 1)
        matrix3(0, 2) = matrix2(0, 0) * matrix(0, 2) + matrix2(0, 1) * matrix(1, 2) + matrix2(0, 2) * matrix(2, 2)
        matrix3(0, 3) = matrix2(0, 0) * matrix(0, 3) + matrix2(0, 1) * matrix(1, 3) + matrix2(0, 2) * matrix(2, 3) + matrix2(0, 3)

        matrix3(1, 0) = matrix2(1, 0) * matrix(0, 0) + matrix2(1, 1) * matrix(1, 0) + matrix2(1, 2) * matrix(2, 0)
        matrix3(1, 1) = matrix2(1, 0) * matrix(0, 1) + matrix2(1, 1) * matrix(1, 1) + matrix2(1, 2) * matrix(2, 1)
        matrix3(1, 2) = matrix2(1, 0) * matrix(0, 2) + matrix2(1, 1) * matrix(1, 2) + matrix2(1, 2) * matrix(2, 2)
        matrix3(1, 3) = matrix2(1, 0) * matrix(0, 3) + matrix2(1, 1) * matrix(1, 3) + matrix2(1, 2) * matrix(2, 3) + matrix2(1, 3)

        matrix3(2, 0) = matrix2(2, 0) * matrix(0, 0) + matrix2(2, 1) * matrix(1, 0) + matrix2(2, 2) * matrix(2, 0)
        matrix3(2, 1) = matrix2(2, 0) * matrix(0, 1) + matrix2(2, 1) * matrix(1, 1) + matrix2(2, 2) * matrix(2, 1)
        matrix3(2, 2) = matrix2(2, 0) * matrix(0, 2) + matrix2(2, 1) * matrix(1, 2) + matrix2(2, 2) * matrix(2, 2)
        matrix3(2, 3) = matrix2(2, 0) * matrix(0, 3) + matrix2(2, 1) * matrix(1, 3) + matrix2(2, 2) * matrix(2, 3) + matrix2(2, 3)

        matrix3(3, 0) = 0
        matrix3(3, 1) = 0
        matrix3(3, 2) = 0
        matrix3(3, 3) = 1

        matrix = matrix3.Clone

        If projectMode <> 5 AndAlso templateProjectionQuad.Count = 0 Then

            Select Case projectMode
                Case 0 ' YZ mit -X (Right) [OK]
                    Dim cmatrix(,) As Double = {{0, 0, 1, 0}, _
                                                {0, 1, 0, 0}, _
                                                {1, 0, 0, 0}, _
                                                {0, 0, 0, 1}}
                    matrix2 = cmatrix.Clone
                Case 1 ' XZ mit -Y (Top) [OK]
                    Dim cmatrix(,) As Double = {{-1, 0, 0, 0}, _
                                                {0, 0, 1, 0}, _
                                                {0, -1, 0, 0}, _
                                                {0, 0, 0, 1}}
                    matrix2 = cmatrix.Clone
                Case 2 ' XY mit -Z (Front) [OK] 
                    Dim cmatrix(,) As Double = {{-1, 0, 0, 0}, _
                                                {0, 1, 0, 0}, _
                                                {0, 0, -1, 0}, _
                                                {0, 0, 0, 1}}
                    matrix2 = cmatrix.Clone
                Case 3 ' YZ mit +X (Left) [OK]
                    Dim cmatrix(,) As Double = {{0, 0, -1, 0}, _
                                                {0, 1, 0, 0}, _
                                                {-1, 0, 0, 0}, _
                                                {0, 0, 0, 1}}
                    matrix2 = cmatrix.Clone
                Case 4 ' XZ mit -Y (Bottom) [OK]
                    Dim cmatrix(,) As Double = {{-1, 0, 0, 0}, _
                                                {0, 0, -1, 0}, _
                                                {0, 1, 0, 0}, _
                                                {0, 0, 0, 1}}
                    matrix2 = cmatrix.Clone
            End Select

            matrix3(0, 0) = matrix2(0, 0) * matrix(0, 0) + matrix2(0, 1) * matrix(1, 0) + matrix2(0, 2) * matrix(2, 0)
            matrix3(0, 1) = matrix2(0, 0) * matrix(0, 1) + matrix2(0, 1) * matrix(1, 1) + matrix2(0, 2) * matrix(2, 1)
            matrix3(0, 2) = matrix2(0, 0) * matrix(0, 2) + matrix2(0, 1) * matrix(1, 2) + matrix2(0, 2) * matrix(2, 2)
            matrix3(0, 3) = matrix2(0, 0) * matrix(0, 3) + matrix2(0, 1) * matrix(1, 3) + matrix2(0, 2) * matrix(2, 3) + matrix2(0, 3)

            matrix3(1, 0) = matrix2(1, 0) * matrix(0, 0) + matrix2(1, 1) * matrix(1, 0) + matrix2(1, 2) * matrix(2, 0)
            matrix3(1, 1) = matrix2(1, 0) * matrix(0, 1) + matrix2(1, 1) * matrix(1, 1) + matrix2(1, 2) * matrix(2, 1)
            matrix3(1, 2) = matrix2(1, 0) * matrix(0, 2) + matrix2(1, 1) * matrix(1, 2) + matrix2(1, 2) * matrix(2, 2)
            matrix3(1, 3) = matrix2(1, 0) * matrix(0, 3) + matrix2(1, 1) * matrix(1, 3) + matrix2(1, 2) * matrix(2, 3) + matrix2(1, 3)

            matrix3(2, 0) = matrix2(2, 0) * matrix(0, 0) + matrix2(2, 1) * matrix(1, 0) + matrix2(2, 2) * matrix(2, 0)
            matrix3(2, 1) = matrix2(2, 0) * matrix(0, 1) + matrix2(2, 1) * matrix(1, 1) + matrix2(2, 2) * matrix(2, 1)
            matrix3(2, 2) = matrix2(2, 0) * matrix(0, 2) + matrix2(2, 1) * matrix(1, 2) + matrix2(2, 2) * matrix(2, 2)
            matrix3(2, 3) = matrix2(2, 0) * matrix(0, 3) + matrix2(2, 1) * matrix(1, 3) + matrix2(2, 2) * matrix(2, 3) + matrix2(2, 3)

            matrix3(3, 0) = 0
            matrix3(3, 1) = 0
            matrix3(3, 2) = 0
            matrix3(3, 3) = 1
            matrix = matrix3.Clone
        End If

        If projectMode = myMetadata.recommendedMode Then

            Do
                If templateProjectionQuad.Count > 0 Then
                    selectGroupDirect(group)
                    For i = 0 To templateProjectionQuad.Count - 1
                        If templateProjectionQuad(i).isInQuad(SelectedTriangles) Then
                            For r As Integer = 0 To 3
                                For c As Integer = 0 To 3
                                    matrix2(r, c) = templateProjectionQuad(i).matrix(r, c)
                                Next
                            Next
                            Exit Do
                        End If
                    Next
                End If
                For r As Integer = 0 To 3
                    For c As Integer = 0 To 3
                        matrix2(r, c) = myMetadata.matrix(r, c)
                    Next
                Next
                matrix2(0, 3) *= 1000
                matrix2(1, 3) *= 1000
                matrix2(2, 3) *= 1000
                Exit Do
            Loop

            matrix3(0, 0) = matrix2(0, 0) * matrix(0, 0) + matrix2(0, 1) * matrix(1, 0) + matrix2(0, 2) * matrix(2, 0)
            matrix3(0, 1) = matrix2(0, 0) * matrix(0, 1) + matrix2(0, 1) * matrix(1, 1) + matrix2(0, 2) * matrix(2, 1)
            matrix3(0, 2) = matrix2(0, 0) * matrix(0, 2) + matrix2(0, 1) * matrix(1, 2) + matrix2(0, 2) * matrix(2, 2)
            matrix3(0, 3) = matrix2(0, 0) * matrix(0, 3) + matrix2(0, 1) * matrix(1, 3) + matrix2(0, 2) * matrix(2, 3) + matrix2(0, 3)

            matrix3(1, 0) = matrix2(1, 0) * matrix(0, 0) + matrix2(1, 1) * matrix(1, 0) + matrix2(1, 2) * matrix(2, 0)
            matrix3(1, 1) = matrix2(1, 0) * matrix(0, 1) + matrix2(1, 1) * matrix(1, 1) + matrix2(1, 2) * matrix(2, 1)
            matrix3(1, 2) = matrix2(1, 0) * matrix(0, 2) + matrix2(1, 1) * matrix(1, 2) + matrix2(1, 2) * matrix(2, 2)
            matrix3(1, 3) = matrix2(1, 0) * matrix(0, 3) + matrix2(1, 1) * matrix(1, 3) + matrix2(1, 2) * matrix(2, 3) + matrix2(1, 3)

            matrix3(2, 0) = matrix2(2, 0) * matrix(0, 0) + matrix2(2, 1) * matrix(1, 0) + matrix2(2, 2) * matrix(2, 0)
            matrix3(2, 1) = matrix2(2, 0) * matrix(0, 1) + matrix2(2, 1) * matrix(1, 1) + matrix2(2, 2) * matrix(2, 1)
            matrix3(2, 2) = matrix2(2, 0) * matrix(0, 2) + matrix2(2, 1) * matrix(1, 2) + matrix2(2, 2) * matrix(2, 2)
            matrix3(2, 3) = matrix2(2, 0) * matrix(0, 3) + matrix2(2, 1) * matrix(1, 3) + matrix2(2, 2) * matrix(2, 3) + matrix2(2, 3)

            matrix3(3, 0) = 0
            matrix3(3, 1) = 0
            matrix3(3, 2) = 0
            matrix3(3, 3) = 1

            matrix = matrix3.Clone

        End If

        matrix = repairMatrix(matrix)
        Dim det As Single = matrix(0, 0) * (matrix(1, 1) * matrix(2, 2) - matrix(1, 2) * matrix(2, 1)) + matrix(0, 1) * (matrix(1, 2) * matrix(2, 0) - matrix(1, 0) * matrix(2, 2)) + matrix(0, 2) * (matrix(1, 0) * matrix(2, 1) - matrix(1, 1) * matrix(2, 0))
        If myMetadata.mData(6) = "BFC CERTIFY CW" Xor det < 0 Then
            Dim cmatrix(,) As Double = {{1, 0, 0, 0}, _
                                        {0, -1, 0, 0}, _
                                        {0, 0, 1, 0}, _
                                        {0, 0, 0, 1}}
            matrix2 = cmatrix.Clone
            matrix3(0, 0) = matrix(0, 0) * matrix2(0, 0) + matrix(0, 1) * matrix2(1, 0) + matrix(0, 2) * matrix2(2, 0)
            matrix3(0, 1) = matrix(0, 0) * matrix2(0, 1) + matrix(0, 1) * matrix2(1, 1) + matrix(0, 2) * matrix2(2, 1)
            matrix3(0, 2) = matrix(0, 0) * matrix2(0, 2) + matrix(0, 1) * matrix2(1, 2) + matrix(0, 2) * matrix2(2, 2)
            matrix3(0, 3) = matrix(0, 0) * matrix2(0, 3) + matrix(0, 1) * matrix2(1, 3) + matrix(0, 2) * matrix2(2, 3) + matrix(0, 3)

            matrix3(1, 0) = matrix(1, 0) * matrix2(0, 0) + matrix(1, 1) * matrix2(1, 0) + matrix(1, 2) * matrix2(2, 0)
            matrix3(1, 1) = matrix(1, 0) * matrix2(0, 1) + matrix(1, 1) * matrix2(1, 1) + matrix(1, 2) * matrix2(2, 1)
            matrix3(1, 2) = matrix(1, 0) * matrix2(0, 2) + matrix(1, 1) * matrix2(1, 2) + matrix(1, 2) * matrix2(2, 2)
            matrix3(1, 3) = matrix(1, 0) * matrix2(0, 3) + matrix(1, 1) * matrix2(1, 3) + matrix(1, 2) * matrix2(2, 3) + matrix(1, 3)

            matrix3(2, 0) = matrix(2, 0) * matrix2(0, 0) + matrix(2, 1) * matrix2(1, 0) + matrix(2, 2) * matrix2(2, 0)
            matrix3(2, 1) = matrix(2, 0) * matrix2(0, 1) + matrix(2, 1) * matrix2(1, 1) + matrix(2, 2) * matrix2(2, 1)
            matrix3(2, 2) = matrix(2, 0) * matrix2(0, 2) + matrix(2, 1) * matrix2(1, 2) + matrix(2, 2) * matrix2(2, 2)
            matrix3(2, 3) = matrix(2, 0) * matrix2(0, 3) + matrix(2, 1) * matrix2(1, 3) + matrix(2, 2) * matrix2(2, 3) + matrix(2, 3)

            matrix3(3, 0) = 0
            matrix3(3, 1) = 0
            matrix3(3, 2) = 0
            matrix3(3, 3) = 1

            matrix = matrix3.Clone
        End If

            matrix(0, 3) /= 1000
            matrix(1, 3) /= 1000
            matrix(2, 3) /= 1000

        Return matrix
    End Function

    Private Function adjustSubfileMatrix(ByVal pmatrix(,) As Double, ByVal projectMode As Byte, Optional ByVal group As Integer = -1) As Double(,)

        Dim matrix(,) As Double = pmatrix.Clone

        Dim matrix2(3, 3) As Double
        Dim matrix3(3, 3) As Double

        If templateProjectionQuad.Count = 0 Then
            Select Case projectMode
                Case 0 ' YZ mit -X (Right) [OK]
                    Dim cmatrix(,) As Double = {{0, 0, 1, 0}, _
                                                {0, 1, 0, 0}, _
                                                {1, 0, 0, 0}, _
                                                {0, 0, 0, 1}}
                    matrix2 = cmatrix.Clone
                Case 1 ' XZ mit -Y (Top) [OK]
                    Dim cmatrix(,) As Double = {{-1, 0, 0, 0}, _
                                                {0, 0, 1, 0}, _
                                                {0, -1, 0, 0}, _
                                                {0, 0, 0, 1}}
                    matrix2 = cmatrix.Clone
                Case 2 ' XY mit -Z (Front) [OK] 
                    Dim cmatrix(,) As Double = {{-1, 0, 0, 0}, _
                                                {0, 1, 0, 0}, _
                                                {0, 0, 1, 0}, _
                                                {0, 0, 0, 1}}
                    matrix2 = cmatrix.Clone
                Case 3 ' YZ mit +X (Left) [OK]
                    Dim cmatrix(,) As Double = {{0, 0, -1, 0}, _
                                                {0, 1, 0, 0}, _
                                                {-1, 0, 0, 0}, _
                                                {0, 0, 0, 1}}
                    matrix2 = cmatrix.Clone
                Case 4 ' XZ mit -Y (Bottom) [OK]
                    Dim cmatrix(,) As Double = {{-1, 0, 0, 0}, _
                                                {0, 0, -1, 0}, _
                                                {0, 1, 0, 0}, _
                                                {0, 0, 0, 1}}
                    matrix2 = cmatrix.Clone
                Case 5 ' XY mit +Z (Back) [OK] 
                    Dim cmatrix(,) As Double = {{1, 0, 0, 0}, _
                                                {0, 1, 0, 0}, _
                                                {0, 0, -1, 0}, _
                                                {0, 0, 0, 1}}
                    matrix2 = cmatrix.Clone
            End Select
        Else
            Dim cmatrix(,) As Double = {{1, 0, 0, 0}, _
                                        {0, 1, 0, 0}, _
                                        {0, 0, -1, 0}, _
                                        {0, 0, 0, 1}}
            matrix2 = cmatrix.Clone
        End If

        matrix3(0, 0) = matrix2(0, 0) * matrix(0, 0) + matrix2(0, 1) * matrix(1, 0) + matrix2(0, 2) * matrix(2, 0)
        matrix3(0, 1) = matrix2(0, 0) * matrix(0, 1) + matrix2(0, 1) * matrix(1, 1) + matrix2(0, 2) * matrix(2, 1)
        matrix3(0, 2) = matrix2(0, 0) * matrix(0, 2) + matrix2(0, 1) * matrix(1, 2) + matrix2(0, 2) * matrix(2, 2)
        matrix3(0, 3) = matrix2(0, 0) * matrix(0, 3) + matrix2(0, 1) * matrix(1, 3) + matrix2(0, 2) * matrix(2, 3) + matrix2(0, 3)

        matrix3(1, 0) = matrix2(1, 0) * matrix(0, 0) + matrix2(1, 1) * matrix(1, 0) + matrix2(1, 2) * matrix(2, 0)
        matrix3(1, 1) = matrix2(1, 0) * matrix(0, 1) + matrix2(1, 1) * matrix(1, 1) + matrix2(1, 2) * matrix(2, 1)
        matrix3(1, 2) = matrix2(1, 0) * matrix(0, 2) + matrix2(1, 1) * matrix(1, 2) + matrix2(1, 2) * matrix(2, 2)
        matrix3(1, 3) = matrix2(1, 0) * matrix(0, 3) + matrix2(1, 1) * matrix(1, 3) + matrix2(1, 2) * matrix(2, 3) + matrix2(1, 3)

        matrix3(2, 0) = matrix2(2, 0) * matrix(0, 0) + matrix2(2, 1) * matrix(1, 0) + matrix2(2, 2) * matrix(2, 0)
        matrix3(2, 1) = matrix2(2, 0) * matrix(0, 1) + matrix2(2, 1) * matrix(1, 1) + matrix2(2, 2) * matrix(2, 1)
        matrix3(2, 2) = matrix2(2, 0) * matrix(0, 2) + matrix2(2, 1) * matrix(1, 2) + matrix2(2, 2) * matrix(2, 2)
        matrix3(2, 3) = matrix2(2, 0) * matrix(0, 3) + matrix2(2, 1) * matrix(1, 3) + matrix2(2, 2) * matrix(2, 3) + matrix2(2, 3)

        matrix3(3, 0) = 0
        matrix3(3, 1) = 0
        matrix3(3, 2) = 0
        matrix3(3, 3) = 1

        matrix = matrix3.Clone

        matrix(0, 3) /= 1000
        matrix(1, 3) /= 1000
        matrix(2, 3) /= 1000

        If projectMode = myMetadata.recommendedMode Then

            Do
                If templateProjectionQuad.Count > 0 Then
                    selectGroupDirect(group)
                    For i = 0 To templateProjectionQuad.Count - 1
                        If templateProjectionQuad(i).isInQuad(SelectedTriangles) Then
                            For r As Integer = 0 To 3
                                For c As Integer = 0 To 3
                                    matrix2(r, c) = templateProjectionQuad(i).matrix(r, c)
                                Next
                            Next
                            matrix2 = repairMatrix(matrix2)
                            Dim det As Single = matrix2(0, 0) * (matrix2(1, 1) * matrix2(2, 2) - matrix2(1, 2) * matrix2(2, 1)) + matrix2(0, 1) * (matrix2(1, 2) * matrix2(2, 0) - matrix2(1, 0) * matrix2(2, 2)) + matrix2(0, 2) * (matrix2(1, 0) * matrix2(2, 1) - matrix2(1, 1) * matrix2(2, 0))
                            If CType(PrimitivesMetadataHMap(Primitives(PrimitivesHMap(group)).primitiveName), Metadata).mData(6) = "BFC CERTIFY CW" Xor det < 0 Then matrix(2, 2) = -matrix(2, 2)
                            Exit Do
                        End If
                    Next
                End If
                For r As Integer = 0 To 3
                    For c As Integer = 0 To 3
                        matrix2(r, c) = myMetadata.matrix(r, c)
                    Next
                Next
                Exit Do
            Loop

            matrix3(0, 0) = matrix2(0, 0) * matrix(0, 0) + matrix2(0, 1) * matrix(1, 0) + matrix2(0, 2) * matrix(2, 0)
            matrix3(0, 1) = matrix2(0, 0) * matrix(0, 1) + matrix2(0, 1) * matrix(1, 1) + matrix2(0, 2) * matrix(2, 1)
            matrix3(0, 2) = matrix2(0, 0) * matrix(0, 2) + matrix2(0, 1) * matrix(1, 2) + matrix2(0, 2) * matrix(2, 2)
            matrix3(0, 3) = matrix2(0, 0) * matrix(0, 3) + matrix2(0, 1) * matrix(1, 3) + matrix2(0, 2) * matrix(2, 3) + matrix2(0, 3)

            matrix3(1, 0) = matrix2(1, 0) * matrix(0, 0) + matrix2(1, 1) * matrix(1, 0) + matrix2(1, 2) * matrix(2, 0)
            matrix3(1, 1) = matrix2(1, 0) * matrix(0, 1) + matrix2(1, 1) * matrix(1, 1) + matrix2(1, 2) * matrix(2, 1)
            matrix3(1, 2) = matrix2(1, 0) * matrix(0, 2) + matrix2(1, 1) * matrix(1, 2) + matrix2(1, 2) * matrix(2, 2)
            matrix3(1, 3) = matrix2(1, 0) * matrix(0, 3) + matrix2(1, 1) * matrix(1, 3) + matrix2(1, 2) * matrix(2, 3) + matrix2(1, 3)

            matrix3(2, 0) = matrix2(2, 0) * matrix(0, 0) + matrix2(2, 1) * matrix(1, 0) + matrix2(2, 2) * matrix(2, 0)
            matrix3(2, 1) = matrix2(2, 0) * matrix(0, 1) + matrix2(2, 1) * matrix(1, 1) + matrix2(2, 2) * matrix(2, 1)
            matrix3(2, 2) = matrix2(2, 0) * matrix(0, 2) + matrix2(2, 1) * matrix(1, 2) + matrix2(2, 2) * matrix(2, 2)
            matrix3(2, 3) = matrix2(2, 0) * matrix(0, 3) + matrix2(2, 1) * matrix(1, 3) + matrix2(2, 2) * matrix(2, 3) + matrix2(2, 3)

            matrix3(3, 0) = 0
            matrix3(3, 1) = 0
            matrix3(3, 2) = 0
            matrix3(3, 3) = 1

            matrix = matrix3.Clone

        End If

        Return repairMatrix(matrix)
    End Function

    Function adjustAndSaveTriangles(ByVal projectMode As Byte, ByRef DateiOut As StreamWriter) As Integer
        Dim lines As Integer = 0
        Dim v(2, 2) As Single
        Dim matrix(3, 3) As Double
        Dim matrix2(3, 3) As Double
        Dim matrix3(3, 3) As Double

        For r As Integer = 0 To 3
            For c As Integer = 0 To 3
                matrix2(r, c) = myMetadata.matrix(r, c)
            Next
        Next

        Select Case projectMode
            Case 0
                Dim cmatrix(,) As Double = {{0, 0, 1, 0}, _
                                            {0, 1, 0, 0}, _
                                            {1, 0, 0, 0}, _
                                            {0, 0, 0, 1}}
                matrix = cmatrix.Clone
            Case 1
                Dim cmatrix(,) As Double = {{-1, 0, 0, 0}, _
                                            {0, 0, 1, 0}, _
                                            {0, -1, 0, 0}, _
                                            {0, 0, 0, 1}}
                matrix = cmatrix.Clone
            Case 2
                Dim cmatrix(,) As Double = {{-1, 0, 0, 0}, _
                                            {0, 1, 0, 0}, _
                                            {0, 0, -1, 0}, _
                                            {0, 0, 0, 1}}
                matrix = cmatrix.Clone
            Case 3
                Dim cmatrix(,) As Double = {{0, 0, -1, 0}, _
                                            {0, 1, 0, 0}, _
                                            {-1, 0, 0, 0}, _
                                            {0, 0, 0, 1}}
                matrix = cmatrix.Clone
            Case 4
                Dim cmatrix(,) As Double = {{-1, 0, 0, 0}, _
                                            {0, 0, -1, 0}, _
                                            {0, 1, 0, 0}, _
                                            {0, 0, 0, 1}}
                matrix = cmatrix.Clone
            Case 5
                Dim cmatrix(,) As Double = {{1, 0, 0, 0}, _
                                            {0, 1, 0, 0}, _
                                            {0, 0, -1, 0}, _
                                            {0, 0, 0, 1}}
                matrix = cmatrix.Clone
        End Select

        matrix3(0, 0) = matrix2(0, 0) * matrix(0, 0) + matrix2(0, 1) * matrix(1, 0) + matrix2(0, 2) * matrix(2, 0)
        matrix3(0, 1) = matrix2(0, 0) * matrix(0, 1) + matrix2(0, 1) * matrix(1, 1) + matrix2(0, 2) * matrix(2, 1)
        matrix3(0, 2) = matrix2(0, 0) * matrix(0, 2) + matrix2(0, 1) * matrix(1, 2) + matrix2(0, 2) * matrix(2, 2)
        matrix3(0, 3) = matrix2(0, 0) * matrix(0, 3) + matrix2(0, 1) * matrix(1, 3) + matrix2(0, 2) * matrix(2, 3) + matrix2(0, 3)

        matrix3(1, 0) = matrix2(1, 0) * matrix(0, 0) + matrix2(1, 1) * matrix(1, 0) + matrix2(1, 2) * matrix(2, 0)
        matrix3(1, 1) = matrix2(1, 0) * matrix(0, 1) + matrix2(1, 1) * matrix(1, 1) + matrix2(1, 2) * matrix(2, 1)
        matrix3(1, 2) = matrix2(1, 0) * matrix(0, 2) + matrix2(1, 1) * matrix(1, 2) + matrix2(1, 2) * matrix(2, 2)
        matrix3(1, 3) = matrix2(1, 0) * matrix(0, 3) + matrix2(1, 1) * matrix(1, 3) + matrix2(1, 2) * matrix(2, 3) + matrix2(1, 3)

        matrix3(2, 0) = matrix2(2, 0) * matrix(0, 0) + matrix2(2, 1) * matrix(1, 0) + matrix2(2, 2) * matrix(2, 0)
        matrix3(2, 1) = matrix2(2, 0) * matrix(0, 1) + matrix2(2, 1) * matrix(1, 1) + matrix2(2, 2) * matrix(2, 1)
        matrix3(2, 2) = matrix2(2, 0) * matrix(0, 2) + matrix2(2, 1) * matrix(1, 2) + matrix2(2, 2) * matrix(2, 2)
        matrix3(2, 3) = matrix2(2, 0) * matrix(0, 3) + matrix2(2, 1) * matrix(1, 3) + matrix2(2, 2) * matrix(2, 3) + matrix2(2, 3)

        matrix3(3, 0) = 0
        matrix3(3, 1) = 0
        matrix3(3, 2) = 0
        matrix3(3, 3) = 1

        matrix = matrix3.Clone

        For Each tri As Triangle In Triangles
            If tri.groupindex = -1 Then
                tri.normalize()
                tri.checkWinding()

                v(0, 0) = tri.vertexA.X
                v(0, 1) = tri.vertexA.Y
                v(0, 2) = 0
                v(1, 0) = tri.vertexB.X
                v(1, 1) = tri.vertexB.Y
                v(1, 2) = 0
                v(2, 0) = tri.vertexC.X
                v(2, 1) = tri.vertexC.Y
                v(2, 2) = 0

                If templateProjectionQuad.Count > 0 Then
                    For i As Integer = 0 To templateProjectionQuad.Count - 1
                        If templateProjectionQuad(i).isInQuad(tri) Then
                            matrix = templateProjectionQuad(i).matrix.Clone
                            Exit For
                        End If
                    Next
                End If

                For i As Integer = 0 To 2
                    Dim t(2) As Single
                    t(0) = matrix(0, 0) * v(i, 0) + matrix(0, 1) * v(i, 1) + matrix(0, 2) * v(i, 2) + matrix(0, 3)
                    t(1) = matrix(1, 0) * v(i, 0) + matrix(1, 1) * v(i, 1) + matrix(1, 2) * v(i, 2) + matrix(1, 3)
                    t(2) = matrix(2, 0) * v(i, 0) + matrix(2, 1) * v(i, 1) + matrix(2, 2) * v(i, 2) + matrix(2, 3)
                    v(i, 0) = t(0)
                    v(i, 1) = t(1)
                    v(i, 2) = t(2)
                Next

                DateiOut.WriteLine(Replace("3 " & getColourString(tri) & " " & v(0, 0) & " " & v(0, 1) & " " & v(0, 2) & " " _
                                                   & v(1, 0) & " " & v(1, 1) & " " & v(1, 2) & " " _
                                                   & v(2, 0) & " " & v(2, 1) & " " & v(2, 2), comma, "."))
                lines += 1
            End If
        Next
        Return lines
    End Function

    Public Function getXcoordinate(ByVal relX As Integer) As Single
        Return -((relX - Me.ClientSize.Width / 2) / zoomfactor - Me.offsetX)
    End Function

    Public Function getYcoordinate(ByVal relY As Integer) As Single
        Return -((-relY + Me.ClientSize.Height / 2) / zoomfactor + Me.offsetY)
    End Function

    Private Sub clip(ByRef value As Single, ByVal min As Single, ByVal max As Single)
        value = Math.Max(value, min)
        value = Math.Min(value, max)
    End Sub

    Private Function clipf(ByVal value As Single, ByVal min As Single, ByVal max As Single)
        value = Math.Max(value, min)
        value = Math.Min(value, max)
        Return value
    End Function

    Private Sub paintTriangles(Optional ByVal s As Integer = 0)
        For i As Integer = s To Triangles.Count - 1
            If colourHMap.ContainsKey(Triangles(i).myColourNumber) Then Triangles(i).myColour = colourHMap(Triangles(i).myColourNumber)
        Next
    End Sub

    Private Sub paintTrianglesTransparent(Optional ByVal s As Integer = 0)
        For i As Integer = s To Triangles.Count - 1
            Dim col As Color = colourHMap(Triangles(i).myColourNumber)
            Triangles(i).myColour = Color.FromArgb(128, col.R, col.G, col.B)
        Next
    End Sub

    Private Sub scaleVertices(ByVal rescale As Boolean)
        If rescale Then
            For Each vert As Vertex In Vertices
                vert.X /= 1000
                vert.Y /= 1000
            Next
        Else
            For Each vert As Vertex In Vertices
                vert.X = vert.X * 1000
                vert.Y = vert.Y * 1000
            Next
        End If
    End Sub

    Private Sub cleanupOldData()
        Dim VIDtoVI As New Hashtable(Vertices.Count)
        Dim TIDtoTI As New Hashtable(Triangles.Count)

        For i As Integer = 0 To Vertices.Count - 1
            VIDtoVI.Add(Vertices(i).vertexID, i)
        Next

        For i As Integer = 0 To Triangles.Count - 1
            TIDtoTI.Add(Triangles(i).triangleID, i)
        Next

        Dim start As Integer
        Dim start2 As Integer
nextVert:
        For i As Integer = start To Vertices.Count - 1
            start2 = 0
nextTri:
            For j As Integer = start2 To Vertices(i).linkedTriangles.Count - 1
                If Not TIDtoTI.Contains(Vertices(i).linkedTriangles(j).triangleID) Then
                    Vertices(i).linkedTriangles.RemoveAt(j)
                    GoTo nextTri
                End If
            Next
        Next

    End Sub

    Private Sub cleanupDATTriangles(Optional ByVal s As Integer = 0)
        Dim start As Integer = s

        For i As Integer = start To Triangles.Count - 1
            Triangles(i).normalize()
        Next

        Dim triHM As New Dictionary(Of String, Byte)
nextTri:
        For i1 As Integer = start To Triangles.Count - 1
            Dim tri As Triangle = Triangles(i1)
            If triHM.ContainsKey(tri.getFingerprint) Then
                If tri.groupindex = -1 Then
                    removeTriangle(tri, i1)
                    start = i1
                    GoTo nextTri
                End If
            Else
                If tri.groupindex = -1 Then
                    If tri.vertexA.X = tri.vertexB.X AndAlso tri.vertexA.Y = tri.vertexB.Y Then removeTriangle(tri, i1) : start = i1 : GoTo nextTri
                    If tri.vertexA.X = tri.vertexC.X AndAlso tri.vertexA.Y = tri.vertexC.Y Then removeTriangle(tri, i1) : start = i1 : GoTo nextTri
                    If tri.vertexB.X = tri.vertexC.X AndAlso tri.vertexB.Y = tri.vertexC.Y Then removeTriangle(tri, i1) : start = i1 : GoTo nextTri
                    triHM.Add(tri.getFingerprint, 0)
                End If
            End If
        Next
    End Sub

    Private Sub cleanupDATTriangles2(ByRef bw As BackgroundWorker)
        CDialog.Controls("LblStatus").Text = "(3/4) Cleanup Triangles.. "
        Dim start As Integer = 0

        For i As Integer = start To Triangles.Count - 1
            Triangles(i).normalize()
        Next

        Dim triHM As New Dictionary(Of String, Byte)
nextTri:
        CType(CDialog.Controls("PBStatus"), ProgressBar).Value = Fix(start / (Triangles.Count - 1) * 100) : If bw.CancellationPending Then Exit Sub
        For i1 As Integer = start To Triangles.Count - 1
            Dim tri As Triangle = Triangles(i1)
            If triHM.ContainsKey(tri.getFingerprint) Then
                If tri.groupindex = -1 Then
                    removeTriangle(tri, i1)
                    start = i1
                    GoTo nextTri
                End If
            Else
                If tri.groupindex = -1 Then
                    If tri.vertexA.X = tri.vertexB.X AndAlso tri.vertexA.Y = tri.vertexB.Y Then removeTriangle(tri, i1) : start = i1 : GoTo nextTri
                    If tri.vertexA.X = tri.vertexC.X AndAlso tri.vertexA.Y = tri.vertexC.Y Then removeTriangle(tri, i1) : start = i1 : GoTo nextTri
                    If tri.vertexB.X = tri.vertexC.X AndAlso tri.vertexB.Y = tri.vertexC.Y Then removeTriangle(tri, i1) : start = i1 : GoTo nextTri
                    triHM.Add(tri.getFingerprint, 0)
                End If
            End If
        Next
    End Sub

    Private Sub cleanupDATVertices(Optional ByVal s As Integer = 0, Optional ByVal s2 As Integer = 0)
        Dim start As Integer = s
        Dim tc As Char = Chr(156)
        Dim sb As New StringBuilder
        Dim vertDict As New Dictionary(Of String, Vertex)
        Dim TIDtoTI As New Dictionary(Of Integer, Integer)

        For i As Integer = s2 To Triangles.Count - 1
            TIDtoTI.Add(Triangles(i).triangleID, i)
        Next
nextVert:
        For i1 As Integer = start To Vertices.Count - 1

            Dim vert As Vertex = Vertices(i1)
            sb.Remove(0, sb.Length)
            sb.Append(Math.Round(vert.X, 1))
            sb.Append(tc)
            sb.Append(Math.Round(vert.Y, 1))

            If vertDict.ContainsKey(sb.ToString) Then
                Dim vert2 As Vertex = vertDict(sb.ToString)
                If vert.groupindex = -1 Or vert2.groupindex = -1 Then
                    If vert.groupindex <> -1 Then vert2.groupindex = vert.groupindex
                    For i As Integer = 0 To vert.linkedTriangles.Count - 1
                        Dim tri As Triangle = vert.linkedTriangles(i)
                        If Not vert2.linkedTriangles.Contains(Triangles(TIDtoTI(tri.triangleID))) Then
                            vert2.linkedTriangles.Add(Triangles(TIDtoTI(tri.triangleID)))
                            If tri.vertexA.Equals(vert) Then Triangles(TIDtoTI(tri.triangleID)).vertexA = vert2
                            If tri.vertexB.Equals(vert) Then Triangles(TIDtoTI(tri.triangleID)).vertexB = vert2
                            If tri.vertexC.Equals(vert) Then Triangles(TIDtoTI(tri.triangleID)).vertexC = vert2
                        End If
                    Next
                    Vertices.RemoveAt(i1)
                    start = i1
                    GoTo nextVert
                End If
            Else
                vertDict.Add(sb.ToString, vert)
            End If
        Next

    End Sub

    Private Sub cleanupDATVerticesFast(ByVal count As Integer, ByRef bw As BackgroundWorker)
        CDialog.Controls("LblStatus").Text = "(3/4) Cleanup Vertices.. "
        Dim start As Integer = 0
        Dim tc As Char = Chr(156)
        Dim sb As New StringBuilder
        Dim vertDict As New Dictionary(Of String, Vertex)
        Dim TIDtoTI As New Dictionary(Of Integer, Integer)
        For i As Integer = 0 To Triangles.Count - 1
            TIDtoTI.Add(Triangles(i).triangleID, i)
        Next
nextVert:
        For i1 As Integer = start To Vertices.Count - 1

            Dim vert As Vertex = Vertices(i1)
            sb.Remove(0, sb.Length)
            sb.Append(Math.Round(vert.X, 1))
            sb.Append(tc)
            sb.Append(Math.Round(vert.Y, 1))

            If vertDict.ContainsKey(sb.ToString) Then
                Dim vert2 As Vertex = vertDict(sb.ToString)
                For i As Integer = 0 To vert.linkedTriangles.Count - 1
                    Dim tri As Triangle = vert.linkedTriangles(i)
                    If Not vert2.linkedTriangles.Contains(Triangles(TIDtoTI(tri.triangleID))) Then
                        vert2.linkedTriangles.Add(Triangles(TIDtoTI(tri.triangleID)))
                        If tri.vertexA.Equals(vert) Then Triangles(TIDtoTI(tri.triangleID)).vertexA = vert2
                        If tri.vertexB.Equals(vert) Then Triangles(TIDtoTI(tri.triangleID)).vertexB = vert2
                        If tri.vertexC.Equals(vert) Then Triangles(TIDtoTI(tri.triangleID)).vertexC = vert2
                    End If
                Next
                Vertices.RemoveAt(i1)
                CType(CDialog.Controls("PBStatus"), ProgressBar).Value = Fix(start / (Vertices.Count - 1) * 100) : If bw.CancellationPending Then Exit Sub

                count -= 1
                If count = 0 Then Exit Sub
                start = i1
                GoTo nextVert
            Else
                vertDict.Add(sb.ToString, vert)
            End If
        Next

    End Sub

    Public Function removeTriangle(ByVal tri As Triangle) As Boolean
        tri.vertexA.linkedTriangles.Remove(tri)
        tri.vertexB.linkedTriangles.Remove(tri)
        tri.vertexC.linkedTriangles.Remove(tri)
        Return Triangles.Remove(tri)
    End Function

    Private Sub removeTriangle(ByVal tri As Triangle, ByVal index As Integer)
        tri.vertexA.linkedTriangles.Remove(tri)
        tri.vertexB.linkedTriangles.Remove(tri)
        tri.vertexC.linkedTriangles.Remove(tri)
        Triangles.RemoveAt(index)
    End Sub

#End Region
#Region "View Properties"
    Private Sub ShowGridToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ShowGridToolStripMenuItem.Click
        raster = ShowGridToolStripMenuItem.Checked
        Me.Refresh()
    End Sub

    Private Sub ImageToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ImageToolStripMenuItem.Click
        GBImage.Visible = ImageToolStripMenuItem.Checked
        Me.Refresh()
    End Sub

    Private Sub SnappingToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SnappingToolStripMenuItem.Click
        GBSnap.Visible = SnappingToolStripMenuItem.Checked
        Me.Refresh()
    End Sub

    Dim conversionEnabled As Boolean = False
    Private Sub NUDMoveSnap_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles NUDMoveSnap.ValueChanged
        If conversionEnabled Then
            Me.moveSnap = NUDMoveSnap.Value / factor
        Else
            Me.moveSnap = Fix(NUDMoveSnap.Value * 10 ^ NUDMoveSnap.DecimalPlaces) / 10 ^ NUDMoveSnap.DecimalPlaces / factor
        End If
        Me.NUDVertX.Increment = Me.moveSnap * factor / 1000
        Me.NUDVertY.Increment = Me.moveSnap * factor / 1000
        Me.Refresh()
    End Sub

    Private Sub NUDRotateSnap_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles NUDRotateSnap.ValueChanged
        Me.rotateSnap = NUDRotateSnap.Value
    End Sub

    Private Sub NUDScaleSnap_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles NUDScaleSnap.ValueChanged
        Me.scaleSnap = NUDScaleSnap.Value
    End Sub

    Private Sub NUDGrid_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles NUDGrid.ValueChanged
        If conversionEnabled Then
            Me.rasterSnap = NUDGrid.Value / factor
        Else
            Me.rasterSnap = Fix(NUDGrid.Value * 10 ^ NUDGrid.DecimalPlaces) / 10 ^ NUDGrid.DecimalPlaces / factor
        End If
        Me.Refresh()
    End Sub

    Private Sub SBZoom_Scroll(ByVal sender As System.Object, ByVal e As System.Windows.Forms.ScrollEventArgs) Handles SBZoom.Scroll
        If Not isLoading Then
            zoomlevel += e.NewValue - e.OldValue
            clip(zoomlevel, -30, 90)
            If zoomlevel > -10 Then
                zoomfactor = 1 + zoomlevel * 0.1F
            ElseIf zoomlevel > -20 Then
                zoomfactor = 0.2F + zoomlevel * 0.01F
            Else
                zoomfactor = 0.03F + zoomlevel * 0.001F
            End If
            If zoomlevel = -30 Then zoomfactor = 0.001
            LblDebug.Text = "Zoom: " & CType(zoomfactor * 10000, Integer) / 100 & "%"
            LblCoords.Text = Math.Round(getXcoordinate(getCursorpositionX()) / Me.moveSnap * factor) * Me.moveSnap / 1000 & " | " & Math.Round(getYcoordinate(getCursorpositionY()) / Me.moveSnap * factor) * Me.moveSnap / 1000 & " " & unit
            e.NewValue = 0
            Me.Refresh()
        Else
            e.NewValue = e.OldValue
        End If
    End Sub
#End Region
#Region "Image Properties"
    Private Sub NUDoffsetX_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles NUDoffsetX.ValueChanged
        imgOffsetX = NUDoffsetX.Value
        Me.Refresh()
    End Sub

    Private Sub NUDoffsetY_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles NUDoffsetY.ValueChanged
        imgOffsetY = NUDoffsetY.Value
        Me.Refresh()
    End Sub

    Private Sub NUDScale_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles NUDScale.ValueChanged
        imgScale = NUDScale.Value
        LblSize.Text = Math.Round(imgScale * backgroundPicture.Width / 1000 * factor, 3) & " x " & vbCrLf & Math.Round(imgScale * backgroundPicture.Height / 1000 * factor, 3) & unit
        Me.Refresh()
    End Sub

    Private Sub BtnImagePath_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BtnImagePath.Click
        If OpenImage.ShowDialog() = Windows.Forms.DialogResult.OK Then
            If OpenImage.FileName <> "" Then
                Try
                    backgroundPicture.Dispose()
                    backgroundPicture = New Bitmap(OpenImage.FileName)
                    TBImage.Text = OpenImage.FileName
                    LblSize.Text = imgScale * backgroundPicture.Width / 1000 * factor & " x " & vbCrLf & imgScale * backgroundPicture.Height / 1000 * factor & unit
                Catch ex As Exception
                    MsgBox("Invalid image format!", MsgBoxStyle.Exclamation, "ERROR")
                    backgroundPicture = My.Resources.temp
                    TBImage.Text = ""
                End Try
            End If
        End If
    End Sub
#End Region
#Region "AddFunctions"
    Private Sub BtnAddVertex_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BtnAddVertex.Click
        If Not BtnAddVertex.Checked Then trianglemode = 0
        keylock = True
        BtnAddTriangle.Checked = False
        CMSAddTriangle.Checked = False
        CMSAddVertex.Checked = Not CMSAddVertex.Checked
        BtnMove.Enabled = Not BtnAddVertex.Checked
        BtnRotate.Enabled = Not BtnAddVertex.Checked
        BtnScale.Enabled = Not BtnAddVertex.Checked
        BtnSelect.PerformClick()
        keylock = False
        If Not BtnAddVertex.Checked Then
            Select Case lastAction
                Case 0 : BtnSelect.PerformClick()
                Case 1 : BtnMove.PerformClick()
                Case 2 : BtnRotate.PerformClick()
                Case 3 : BtnScale.PerformClick()
            End Select
        End If
        Me.Refresh()
    End Sub

    Private Sub BtnAddTriangle_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BtnAddTriangle.Click
        If Not BtnAddTriangle.Checked Then trianglemode = 0
        keylock = True
        BtnAddVertex.Checked = False
        CMSAddVertex.Checked = False
        CMSAddTriangle.Checked = Not CMSAddTriangle.Checked
        BtnMove.Enabled = Not BtnAddTriangle.Checked
        BtnRotate.Enabled = Not BtnAddTriangle.Checked
        BtnScale.Enabled = Not BtnAddTriangle.Checked
        BtnSelect.PerformClick()
        VerticesModeToolStripMenuItem.PerformClick()
        keylock = False
        If Not BtnAddTriangle.Checked Then
            Select Case lastAction
                Case 0 : BtnSelect.PerformClick()
                Case 1 : BtnMove.PerformClick()
                Case 2 : BtnRotate.PerformClick()
                Case 3 : BtnScale.PerformClick()
            End Select
            Select Case lastMode
                Case 0 : VerticesModeToolStripMenuItem.PerformClick()
                Case 1 : TriangleModeToolStripMenuItem.PerformClick()
                Case 2 : PrimitiveModeToolStripMenuItem.PerformClick()
            End Select
        End If
        Me.Refresh()
    End Sub
#End Region
#Region "Modi"
    Private Sub BtnSelect_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BtnSelect.Click
        If Not keylock Then lastAction = 0
        BtnSelect.Checked = True
        BtnMove.Checked = False
        BtnRotate.Checked = False
        BtnScale.Checked = False
        bearbeitungsmodus = editMode.Selection
    End Sub

    Private Sub BtnMove_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BtnMove.Click
        If Not keylock Then lastAction = 1
        BtnSelect.Checked = False
        BtnMove.Checked = True
        BtnRotate.Checked = False
        BtnScale.Checked = False
        bearbeitungsmodus = editMode.Translation
    End Sub

    Private Sub BtnRotate_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BtnRotate.Click
        If Not keylock Then lastAction = 2
        BtnSelect.Checked = False
        BtnMove.Checked = False
        BtnRotate.Checked = True
        BtnScale.Checked = False
        bearbeitungsmodus = editMode.Rotation
    End Sub

    Private Sub BtnScale_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BtnScale.Click
        If Not keylock Then lastAction = 3
        BtnSelect.Checked = False
        BtnMove.Checked = False
        BtnRotate.Checked = False
        BtnScale.Checked = True
        bearbeitungsmodus = editMode.Scale
    End Sub

    Private Sub VerticesModeToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles VerticesModeToolStripMenuItem.Click
        If Not keylock Then lastMode = 0
        BtnAddToGroup.Enabled = False
        BtnUngroup.Enabled = False
        If BtnMode.Text = "Primitive Mode" Then
            For Each vert As Vertex In SelectedVertices
                vert.selected = False
            Next
            SelectedVertices.Clear()
        End If
        BtnMode.Text = VerticesModeToolStripMenuItem.Text
        BtnMode.ToolTipText = VerticesModeToolStripMenuItem.ToolTipText
        objectToModify = "Vertex"
        BtnCSG.Enabled = True
        ColourToolStrip.Visible = BtnColours.Checked
        BtnMerge.Enabled = True
        For Each tri As Triangle In SelectedTriangles
            tri.selected = False
        Next
        SelectedTriangles.Clear()
        For Each vert As Vertex In SelectedVertices
            vert.selected = True
        Next
        Me.Refresh()
        SelectAllToolStripMenuItem.Enabled = True
        SelectConnectedToolStripMenuItem.Enabled = True
        SelectTouchingToolStripMenuItem.Enabled = True
        SelectSameColourToolStripMenuItem.Enabled = True
        WithColourToolStripMenuItem.Enabled = True
    End Sub

    Private Sub TriangleModeToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles TriangleModeToolStripMenuItem.Click
        If Not keylock Then lastMode = 1
        If BtnMode.Text = "Primitive Mode" Then
            BtnAddToGroup.Enabled = False
            BtnUngroup.Enabled = False
            For Each tri As Triangle In SelectedTriangles
                tri.selected = False
            Next
            SelectedTriangles.Clear()
        End If
        BtnMode.Text = TriangleModeToolStripMenuItem.Text
        BtnMode.ToolTipText = TriangleModeToolStripMenuItem.ToolTipText
        objectToModify = "Triangle"
        BtnCSG.Enabled = True
        ColourToolStrip.Visible = BtnColours.Checked
        GBVertex.Visible = False
        BtnMerge.Enabled = False
        For Each vert As Vertex In SelectedVertices
            vert.selected = False
        Next
        SelectedVertices.Clear()
        Me.Refresh()
        SelectAllToolStripMenuItem.Enabled = True
        SelectConnectedToolStripMenuItem.Enabled = True
        SelectTouchingToolStripMenuItem.Enabled = True
        SelectSameColourToolStripMenuItem.Enabled = True
        WithColourToolStripMenuItem.Enabled = True
    End Sub

    Private Sub PrimitiveModeToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles PrimitiveModeToolStripMenuItem.Click
        If Not keylock Then lastMode = 2
        SelectAllToolStripMenuItem.Enabled = False
        SelectConnectedToolStripMenuItem.Enabled = False
        SelectTouchingToolStripMenuItem.Enabled = False
        SelectSameColourToolStripMenuItem.Enabled = False
        WithColourToolStripMenuItem.Enabled = False
        BtnMode.Text = PrimitiveModeToolStripMenuItem.Text
        BtnMode.ToolTipText = PrimitiveModeToolStripMenuItem.ToolTipText
        objectToModify = "Primitive"
        BtnCSG.Enabled = False
        BtnColours.Enabled = True
        ColourToolStrip.Visible = BtnColours.Checked
        GBVertex.Visible = False
        BtnMerge.Enabled = False
        For Each tri As Triangle In SelectedTriangles
            tri.selected = False
        Next
        SelectedTriangles.Clear()
        For Each vert As Vertex In SelectedVertices
            vert.selected = False
        Next
        SelectedVertices.Clear()
        Me.Refresh()
    End Sub
#End Region
#Region "Copy & Paste"
    Private Sub BtnCut_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BtnCut.Click
        If SelectedVertices.Count > 0 OrElse SelectedTriangles.Count > 0 Then
            addHistoryI(withDataFrom.All)
            copy()
            delete()
            addHistoryII(withDataFrom.All)
        End If
    End Sub

    Private Sub BtnCopy_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BtnCopy.Click
        copy()
    End Sub

    Private Sub BtnPaste_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BtnPaste.Click
        addHistoryI(withDataFrom.All)
        paste()
        addHistoryII(withDataFrom.All)
    End Sub

    Private Sub ToolStripMenuItem5_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripMenuItem5.Click
        If SelectedVertices.Count > 0 OrElse SelectedTriangles.Count > 0 Then
            addHistoryI(withDataFrom.All)
            copy()
            delete()
            addHistoryII(withDataFrom.All)
        End If
    End Sub

    Private Sub ToolStripMenuItem4_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripMenuItem4.Click
        copy()
    End Sub

    Private Sub ToolStripMenuItem3_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripMenuItem3.Click
        addHistoryI(withDataFrom.All)
        paste()
        addHistoryII(withDataFrom.All)
    End Sub

    Private Sub ToolStripMenuItem2_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripMenuItem2.Click
        If SelectedVertices.Count > 0 OrElse SelectedTriangles.Count > 0 Then
            addHistoryI(withDataFrom.All)
            delete()
            addHistoryII(withDataFrom.All)
        End If
    End Sub

    Private Sub delete()
        If SelectedVertices.Count > 0 OrElse SelectedTriangles.Count > 0 Then
            trianglemode = 0
            isLoading = True
            If BtnMode.Text <> "Triangle Mode" Then
                If objectToModify = "Primitive" Then
                    Dim myindex As Integer = CType(PrimitivesHMap(SelectedVertices(0).groupindex), Integer)
                    PrimitivesHMap.Remove(SelectedVertices(0).groupindex)
                    If PrimitivesHMap.Keys.Count > 0 Then
                        Dim klist(PrimitivesHMap.Keys.Count - 1) As Integer
                        Dim counter As Integer = 0
                        For Each key As Integer In PrimitivesHMap.Keys
                            klist(counter) = key
                            counter += 1
                        Next
                        For i As Integer = 0 To PrimitivesHMap.Keys.Count - 1
                            If PrimitivesHMap.Item(klist(i)) > myindex Then
                                PrimitivesHMap.Item(klist(i)) -= 1
                            End If
                        Next i
                    End If
                    Primitives.RemoveAt(myindex)
                End If
                For Each vert As Vertex In SelectedVertices
removeVertex:
                    For iv As Integer = 0 To Vertices.Count - 1
                        Dim ref As Vertex = Vertices(iv)
                        If ref.Equals(vert) Then
nextTriangle:
                            For Each tri As Triangle In vert.linkedTriangles
                                If Triangles.Contains(tri) Then
                                    removeTriangle(tri)
                                Else
                                    vert.linkedTriangles.Remove(tri)
                                End If
                                GoTo nextTriangle
                            Next
                            Vertices.RemoveAt(iv)
                            GoTo removeVertex
                        End If
                    Next
                Next
            Else
                For Each tri As Triangle In SelectedTriangles
                    removeTriangle(tri)
                Next
            End If
            SelectedVertices.Clear()
            SelectedTriangles.Clear()
            isLoading = False
            Me.Refresh()
        End If
    End Sub

    Private Sub copy()
        If SelectedVertices.Count > 0 Then
            ClipboardVertices.Clear()
            ClipboardTriangles.Clear()
            ClipboardPrimitives.Clear()

            For Each vert As Vertex In SelectedVertices
                ClipboardVertices.Add(New Vertex(vert.X, vert.Y, True, False) With {.vertexID = vert.vertexID, .groupindex = vert.groupindex})
            Next

            If BtnMode.Text = "Triangle Mode" Then
                For Each tri As Triangle In SelectedTriangles
                    If SelectedVertices.Contains(tri.vertexA) AndAlso SelectedVertices.Contains(tri.vertexB) AndAlso SelectedVertices.Contains(tri.vertexC) Then
                        ClipboardTriangles.Add(New Triangle(New Vertex(tri.vertexA.X, tri.vertexA.Y, True, False) With {.vertexID = tri.vertexA.vertexID, .groupindex = tri.vertexA.groupindex}, New Vertex(tri.vertexB.X, tri.vertexB.Y, True, False) With {.vertexID = tri.vertexB.vertexID, .groupindex = tri.vertexB.groupindex}, New Vertex(tri.vertexC.X, tri.vertexC.Y, True, False) With {.vertexID = tri.vertexC.vertexID, .groupindex = tri.vertexC.groupindex}, False) With {.selected = True, .myColourNumber = tri.myColourNumber, .myColour = tri.myColour, .groupindex = tri.groupindex, .triangleID = tri.triangleID})
                    End If
                Next
            Else
                For Each tri As Triangle In Triangles
                    If SelectedVertices.Contains(tri.vertexA) AndAlso SelectedVertices.Contains(tri.vertexB) AndAlso SelectedVertices.Contains(tri.vertexC) Then
                        ClipboardTriangles.Add(New Triangle(New Vertex(tri.vertexA.X, tri.vertexA.Y, True, False) With {.vertexID = tri.vertexA.vertexID, .groupindex = tri.vertexA.groupindex}, New Vertex(tri.vertexB.X, tri.vertexB.Y, True, False) With {.vertexID = tri.vertexB.vertexID, .groupindex = tri.vertexB.groupindex}, New Vertex(tri.vertexC.X, tri.vertexC.Y, True, False) With {.vertexID = tri.vertexC.vertexID, .groupindex = tri.vertexC.groupindex}, False) With {.selected = True, .myColourNumber = tri.myColourNumber, .myColour = tri.myColour, .groupindex = tri.groupindex, .triangleID = tri.triangleID})
                    End If
                Next
            End If

            If SelectedVertices(0).groupindex > -1 Then
                Dim prim As Primitive = Primitives(PrimitivesHMap(SelectedVertices(0).groupindex))
                ClipboardPrimitives.Add(New Primitive(0, 0, prim.ox, prim.oy, prim.primitiveName, prim.centerVertexID, False) With {.matrix = prim.matrix, .myColourNumber = prim.myColourNumber})
            End If

            Me.Refresh()
        End If
    End Sub

    Dim vA, vB, vC As Vertex
    Private Sub paste()
        If ClipboardVertices.Count > 0 Then
            For Each vert As Vertex In SelectedVertices
                vert.selected = False
            Next

            For Each tri As Triangle In SelectedTriangles
                tri.selected = False
            Next


            SelectedVertices.Clear()
            SelectedTriangles.Clear()

            Dim VIDtoVI As New Hashtable(ClipboardVertices.Count)

            Dim startVertices As Integer = Vertices.Count
            Dim startTriangles As Integer = Triangles.Count


            If ClipboardVertices(0).groupindex > -1 Then
                ' Neues Primitive einfügen:
                Primitives.Add(New Primitive(0, 0, ClipboardPrimitives(0).ox, ClipboardPrimitives(0).oy, ClipboardPrimitives(0).primitiveName, ClipboardPrimitives(0).centerVertexID) With {.matrix = ClipboardPrimitives(0).matrix, .myColourNumber = ClipboardPrimitives(0).myColourNumber})
                PrimitivesHMap.Add(primitiveIDglobal, Primitives.Count - 1)
                For Each vert As Vertex In ClipboardVertices
                    Vertices.Add(New Vertex(vert.X, vert.Y, False) With {.angleFrom = vert.vertexID, .groupindex = primitiveIDglobal})
                    SelectedVertices.Add(LLast(Vertices))
                Next

                For Each tri As Triangle In ClipboardTriangles
                    Triangles.Add(New Triangle(tri.vertexA, tri.vertexB, tri.vertexC) With {.selected = True, .myColourNumber = tri.myColourNumber, .myColour = tri.myColour, .groupindex = primitiveIDglobal})
                    SelectedTriangles.Add(LLast(Triangles))
                Next
            Else
                ' Nur Vertices und Triangles einfügen
                For Each vert As Vertex In ClipboardVertices
                    Vertices.Add(New Vertex(vert.X, vert.Y, True) With {.angleFrom = vert.vertexID})
                    SelectedVertices.Add(LLast(Vertices))
                Next

                For Each tri As Triangle In ClipboardTriangles
                    Triangles.Add(New Triangle(tri.vertexA, tri.vertexB, tri.vertexC) With {.selected = True, .myColourNumber = tri.myColourNumber, .myColour = tri.myColour})
                    SelectedTriangles.Add(LLast(Triangles))
                Next
            End If

            ' Hashmap erstellen:
            For i As Integer = startVertices To Vertices.Count - 1
                VIDtoVI.Add(CType(Vertices(i).angleFrom, Integer), i)
                Vertices(i).angleFrom = 0
            Next

            ' Linken:
            If ClipboardVertices(0).groupindex > -1 Then
                LLast(Primitives).centerVertexID = Vertices(VIDtoVI(LLast(Primitives).centerVertexID)).vertexID
            End If
            For i As Integer = startTriangles To Triangles.Count - 1
                Vertices(VIDtoVI(Triangles(i).vertexA.vertexID)).linkedTriangles.Add(Triangles(i))
                Vertices(VIDtoVI(Triangles(i).vertexB.vertexID)).linkedTriangles.Add(Triangles(i))
                Vertices(VIDtoVI(Triangles(i).vertexC.vertexID)).linkedTriangles.Add(Triangles(i))
                Triangles(i).vertexA = Vertices(VIDtoVI(Triangles(i).vertexA.vertexID))
                Triangles(i).vertexB = Vertices(VIDtoVI(Triangles(i).vertexB.vertexID))
                Triangles(i).vertexC = Vertices(VIDtoVI(Triangles(i).vertexC.vertexID))
            Next
            Me.Refresh()
        End If
    End Sub
#End Region


#Region "Select"
    Private Sub SelectAllToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SelectAllToolStripMenuItem.Click
        addHistoryI(withDataFrom.SelectedTriangles + withDataFrom.SelectedVertices)
        selectAll()
        addHistoryII(withDataFrom.SelectedTriangles + withDataFrom.SelectedVertices)
        Me.Refresh()
    End Sub

    Private Sub SelectSameColourToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SelectSameColourToolStripMenuItem.Click
        addHistoryI(withDataFrom.SelectedTriangles + withDataFrom.SelectedVertices)
        selectSameColour()
        addHistoryII(withDataFrom.SelectedTriangles + withDataFrom.SelectedVertices)
        Me.Refresh()
    End Sub

    Private Sub SelectConnectedToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SelectConnectedToolStripMenuItem.Click
        addHistoryI(withDataFrom.SelectedTriangles + withDataFrom.SelectedVertices)
        selectConnected()
        addHistoryII(withDataFrom.SelectedTriangles + withDataFrom.SelectedVertices)
        Me.Refresh()
    End Sub

    Private Sub SelectTouchingToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SelectTouchingToolStripMenuItem.Click
        addHistoryI(withDataFrom.SelectedTriangles + withDataFrom.SelectedVertices)
        selectTouching()
        addHistoryII(withDataFrom.SelectedTriangles + withDataFrom.SelectedVertices)
        Me.Refresh()
    End Sub

    Private Sub WithColourToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles WithColourToolStripMenuItem.Click
        WithColourToolStripMenuItem.GetCurrentParent.Show()
        Me.Refresh()
    End Sub

    Private Sub selectAll()
        SelectedTriangles.Clear()
        SelectedVertices.Clear()
        For Each tri As Triangle In Triangles
            If tri.groupindex = -1 Then
                SelectedTriangles.Add(tri)
                tri.selected = True
                If Not SelectedVertices.Contains(tri.vertexA) AndAlso tri.vertexA.groupindex = -1 Then SelectedVertices.Add(tri.vertexA) : LLast(SelectedVertices).selected = True
                If Not SelectedVertices.Contains(tri.vertexB) AndAlso tri.vertexB.groupindex = -1 Then SelectedVertices.Add(tri.vertexB) : LLast(SelectedVertices).selected = True
                If Not SelectedVertices.Contains(tri.vertexC) AndAlso tri.vertexC.groupindex = -1 Then SelectedVertices.Add(tri.vertexC) : LLast(SelectedVertices).selected = True
            End If
        Next
        For Each vert As Vertex In Vertices
            If Not SelectedVertices.Contains(vert) AndAlso vert.groupindex = -1 Then
                vert.selected = True
                SelectedVertices.Add(vert)
            End If
        Next
    End Sub

    Private Sub selectSameColour()
        If SelectedTriangles.Count > 0 Then
            Dim tc As Short = SelectedTriangles(0).myColourNumber
            Dim tci As Integer = SelectedTriangles(0).myColour.ToArgb
            SelectedTriangles.Clear()
            SelectedVertices.Clear()
            If tc <> -1 Then
                For Each tri As Triangle In Triangles
                    If tri.myColourNumber = tc AndAlso tri.groupindex < 0 Then
                        SelectedTriangles.Add(tri)
                        tri.selected = True
                        If Not SelectedVertices.Contains(tri.vertexA) Then SelectedVertices.Add(tri.vertexA) : LLast(SelectedVertices).selected = True
                        If Not SelectedVertices.Contains(tri.vertexB) Then SelectedVertices.Add(tri.vertexB) : LLast(SelectedVertices).selected = True
                        If Not SelectedVertices.Contains(tri.vertexC) Then SelectedVertices.Add(tri.vertexC) : LLast(SelectedVertices).selected = True
                    End If
                Next
            Else
                For Each tri As Triangle In Triangles
                    If tri.myColour.ToArgb = tci AndAlso tri.groupindex < 0 Then
                        SelectedTriangles.Add(tri)
                        tri.selected = True
                        If Not SelectedVertices.Contains(tri.vertexA) Then SelectedVertices.Add(tri.vertexA) : LLast(SelectedVertices).selected = True
                        If Not SelectedVertices.Contains(tri.vertexB) Then SelectedVertices.Add(tri.vertexB) : LLast(SelectedVertices).selected = True
                        If Not SelectedVertices.Contains(tri.vertexC) Then SelectedVertices.Add(tri.vertexC) : LLast(SelectedVertices).selected = True
                    End If
                Next
            End If
        End If
    End Sub

    Private Sub selectConnected(Optional ByVal sameColourTest As Boolean = True)
        Dim startcount As Integer = 0
        Dim tc As Short
        Dim tci As Integer
        If WithColourToolStripMenuItem.Checked AndAlso sameColourTest AndAlso SelectedTriangles.Count > 0 Then tc = SelectedTriangles(0).myColourNumber : tci = SelectedTriangles(0).myColour.ToArgb
nextStep:
        If objectToModify = "Vertex" Then
            Dim tmpcount As Integer = SelectedVertices.Count - 1
            For verti = startcount To tmpcount
                Dim vert As Vertex = SelectedVertices(verti)
                For Each tri As Triangle In vert.linkedTriangles
                    If Not SelectedVertices.Contains(tri.vertexA) AndAlso tri.vertexA.groupindex < 0 Then tri.vertexA.selected = True : SelectedVertices.Add(tri.vertexA)
                    If Not SelectedVertices.Contains(tri.vertexB) AndAlso tri.vertexB.groupindex < 0 Then tri.vertexB.selected = True : SelectedVertices.Add(tri.vertexB)
                    If Not SelectedVertices.Contains(tri.vertexC) AndAlso tri.vertexC.groupindex < 0 Then tri.vertexC.selected = True : SelectedVertices.Add(tri.vertexC)
                Next
            Next
            startcount = tmpcount + 1
            If SelectedVertices.Count > startcount Then GoTo nextstep
        ElseIf objectToModify = "Triangle" Then
            Dim tmpcount As Integer = SelectedTriangles.Count - 1
            If WithColourToolStripMenuItem.Checked AndAlso sameColourTest Then
                For trii = startcount To tmpcount
                    Dim tri As Triangle = SelectedTriangles(trii)
                    If Not tri.vertexA Is Nothing Then
                        For Each tri2 As Triangle In tri.vertexA.linkedTriangles
                            If ((tri2.myColourNumber = tc AndAlso tc <> -1) OrElse (tc = -1 AndAlso tri2.myColour.ToArgb = tci)) AndAlso Not SelectedTriangles.Contains(tri2) AndAlso tri2.groupindex < 0 Then
                                If Not SelectedVertices.Contains(tri2.vertexA) AndAlso tri2.vertexA.groupindex < 0 Then tri2.vertexA.selected = True : SelectedVertices.Add(tri2.vertexA)
                                If Not SelectedVertices.Contains(tri2.vertexB) AndAlso tri2.vertexB.groupindex < 0 Then tri2.vertexB.selected = True : SelectedVertices.Add(tri2.vertexB)
                                If Not SelectedVertices.Contains(tri2.vertexC) AndAlso tri2.vertexC.groupindex < 0 Then tri2.vertexC.selected = True : SelectedVertices.Add(tri2.vertexC)
                                tri2.selected = True : SelectedTriangles.Add(tri2)
                            End If
                        Next
                    End If
                    If Not tri.vertexB Is Nothing Then
                        For Each tri2 As Triangle In tri.vertexB.linkedTriangles
                            If ((tri2.myColourNumber = tc AndAlso tc <> -1) OrElse (tc = -1 AndAlso tri2.myColour.ToArgb = tci)) AndAlso Not SelectedTriangles.Contains(tri2) AndAlso tri2.groupindex < 0 Then
                                If Not SelectedVertices.Contains(tri2.vertexA) AndAlso tri2.vertexA.groupindex < 0 Then tri2.vertexA.selected = True : SelectedVertices.Add(tri2.vertexA)
                                If Not SelectedVertices.Contains(tri2.vertexB) AndAlso tri2.vertexB.groupindex < 0 Then tri2.vertexB.selected = True : SelectedVertices.Add(tri2.vertexB)
                                If Not SelectedVertices.Contains(tri2.vertexC) AndAlso tri2.vertexC.groupindex < 0 Then tri2.vertexC.selected = True : SelectedVertices.Add(tri2.vertexC)
                                tri2.selected = True : SelectedTriangles.Add(tri2)
                            End If
                        Next
                    End If
                    If Not tri.vertexC Is Nothing Then
                        For Each tri2 As Triangle In tri.vertexC.linkedTriangles
                            If ((tri2.myColourNumber = tc AndAlso tc <> -1) OrElse (tc = -1 AndAlso tri2.myColour.ToArgb = tci)) AndAlso Not SelectedTriangles.Contains(tri2) AndAlso tri2.groupindex < 0 Then
                                If Not SelectedVertices.Contains(tri2.vertexA) AndAlso tri2.vertexA.groupindex < 0 Then tri2.vertexA.selected = True : SelectedVertices.Add(tri2.vertexA)
                                If Not SelectedVertices.Contains(tri2.vertexB) AndAlso tri2.vertexB.groupindex < 0 Then tri2.vertexB.selected = True : SelectedVertices.Add(tri2.vertexB)
                                If Not SelectedVertices.Contains(tri2.vertexC) AndAlso tri2.vertexC.groupindex < 0 Then tri2.vertexC.selected = True : SelectedVertices.Add(tri2.vertexC)
                                tri2.selected = True : SelectedTriangles.Add(tri2)
                            End If
                        Next
                    End If
                Next
            Else
                For trii = startcount To tmpcount
                    Dim tri As Triangle = SelectedTriangles(trii)
                    For Each tri2 As Triangle In tri.vertexA.linkedTriangles
                        If Not SelectedTriangles.Contains(tri2) AndAlso tri2.groupindex < 0 Then
                            If Not SelectedVertices.Contains(tri2.vertexA) AndAlso tri2.vertexA.groupindex < 0 Then tri2.vertexA.selected = True : SelectedVertices.Add(tri2.vertexA)
                            If Not SelectedVertices.Contains(tri2.vertexB) AndAlso tri2.vertexB.groupindex < 0 Then tri2.vertexB.selected = True : SelectedVertices.Add(tri2.vertexB)
                            If Not SelectedVertices.Contains(tri2.vertexC) AndAlso tri2.vertexC.groupindex < 0 Then tri2.vertexC.selected = True : SelectedVertices.Add(tri2.vertexC)
                            tri2.selected = True : SelectedTriangles.Add(tri2)
                        End If
                    Next
                    For Each tri2 As Triangle In tri.vertexB.linkedTriangles
                        If Not SelectedTriangles.Contains(tri2) AndAlso tri2.groupindex < 0 Then
                            If Not SelectedVertices.Contains(tri2.vertexA) AndAlso tri2.vertexA.groupindex < 0 Then tri2.vertexA.selected = True : SelectedVertices.Add(tri2.vertexA)
                            If Not SelectedVertices.Contains(tri2.vertexB) AndAlso tri2.vertexB.groupindex < 0 Then tri2.vertexB.selected = True : SelectedVertices.Add(tri2.vertexB)
                            If Not SelectedVertices.Contains(tri2.vertexC) AndAlso tri2.vertexC.groupindex < 0 Then tri2.vertexC.selected = True : SelectedVertices.Add(tri2.vertexC)
                            tri2.selected = True : SelectedTriangles.Add(tri2)
                        End If
                    Next
                    For Each tri2 As Triangle In tri.vertexC.linkedTriangles
                        If Not SelectedTriangles.Contains(tri2) AndAlso tri2.groupindex < 0 Then
                            If Not SelectedVertices.Contains(tri2.vertexA) AndAlso tri2.vertexA.groupindex < 0 Then tri2.vertexA.selected = True : SelectedVertices.Add(tri2.vertexA)
                            If Not SelectedVertices.Contains(tri2.vertexB) AndAlso tri2.vertexB.groupindex < 0 Then tri2.vertexB.selected = True : SelectedVertices.Add(tri2.vertexB)
                            If Not SelectedVertices.Contains(tri2.vertexC) AndAlso tri2.vertexC.groupindex < 0 Then tri2.vertexC.selected = True : SelectedVertices.Add(tri2.vertexC)
                            tri2.selected = True : SelectedTriangles.Add(tri2)
                        End If
                    Next
                Next
            End If
            startcount = tmpcount + 1
            If SelectedTriangles.Count > startcount Then GoTo nextstep
        End If
    End Sub

    Private Sub selectPrimitiveGroup(ByVal group As Integer)
        Dim startcount As Integer = 0
nextStep:
        Dim tmpcount As Integer = SelectedTriangles.Count - 1
        For trii = startcount To tmpcount
            Dim tri As Triangle = SelectedTriangles(trii)
            For Each tri2 As Triangle In tri.vertexA.linkedTriangles
                If Not SelectedTriangles.Contains(tri2) AndAlso tri2.groupindex = group Then
                    If Not SelectedVertices.Contains(tri2.vertexA) AndAlso tri2.vertexA.groupindex = group Then SelectedVertices.Add(tri2.vertexA)
                    If Not SelectedVertices.Contains(tri2.vertexB) AndAlso tri2.vertexB.groupindex = group Then SelectedVertices.Add(tri2.vertexB)
                    If Not SelectedVertices.Contains(tri2.vertexC) AndAlso tri2.vertexC.groupindex = group Then SelectedVertices.Add(tri2.vertexC)
                    tri2.selected = True : SelectedTriangles.Add(tri2)
                End If
            Next
            For Each tri2 As Triangle In tri.vertexB.linkedTriangles
                If Not SelectedTriangles.Contains(tri2) AndAlso tri2.groupindex = group Then
                    If Not SelectedVertices.Contains(tri2.vertexA) AndAlso tri2.vertexA.groupindex = group Then SelectedVertices.Add(tri2.vertexA)
                    If Not SelectedVertices.Contains(tri2.vertexB) AndAlso tri2.vertexB.groupindex = group Then SelectedVertices.Add(tri2.vertexB)
                    If Not SelectedVertices.Contains(tri2.vertexC) AndAlso tri2.vertexC.groupindex = group Then SelectedVertices.Add(tri2.vertexC)
                    tri2.selected = True : SelectedTriangles.Add(tri2)
                End If
            Next
            For Each tri2 As Triangle In tri.vertexC.linkedTriangles
                If Not SelectedTriangles.Contains(tri2) AndAlso tri2.groupindex = group Then
                    If Not SelectedVertices.Contains(tri2.vertexA) AndAlso tri2.vertexA.groupindex = group Then SelectedVertices.Add(tri2.vertexA)
                    If Not SelectedVertices.Contains(tri2.vertexB) AndAlso tri2.vertexB.groupindex = group Then SelectedVertices.Add(tri2.vertexB)
                    If Not SelectedVertices.Contains(tri2.vertexC) AndAlso tri2.vertexC.groupindex = group Then SelectedVertices.Add(tri2.vertexC)
                    tri2.selected = True : SelectedTriangles.Add(tri2)
                End If
            Next
        Next
        startcount = tmpcount + 1
        If SelectedTriangles.Count > startcount Then GoTo nextstep
    End Sub

    Private Sub selectColourStrideHorizontal(ByRef bmp As Bitmap)
        Dim startcount As Integer = 0
        Dim above_col As Integer
        Dim bottom_col As Integer
        If SelectedVertices(0).Y < 3 Then
            above_col = 0
        Else
            If SelectedVertices(0).X > bmp.Width Then
                above_col = bmp.GetPixel(bmp.Width - SelectedVertices(0).X + 1, SelectedVertices(0).Y - 2).ToArgb
            Else
                above_col = bmp.GetPixel(bmp.Width - SelectedVertices(0).X, SelectedVertices(0).Y - 2).ToArgb
            End If
        End If
        If SelectedVertices(0).Y > bmp.Height Then
            bottom_col = 0
        Else
            If SelectedVertices(0).X > bmp.Width Then
                bottom_col = bmp.GetPixel(bmp.Width - SelectedVertices(0).X + 1, SelectedVertices(0).Y - 1).ToArgb
            Else
                bottom_col = bmp.GetPixel(bmp.Width - SelectedVertices(0).X, SelectedVertices(0).Y - 1).ToArgb
            End If
        End If
nextStep:
        Dim tmpcount As Integer = SelectedVertices.Count - 1
        For verti = startcount To tmpcount
            Dim vert As Vertex = SelectedVertices(verti)
            For Each tri As Triangle In vert.linkedTriangles
                Dim tabove_col As Integer
                Dim tbottom_col As Integer

                If tri.vertexA.selected AndAlso Not SelectedVertices.Contains(tri.vertexA) AndAlso tri.vertexA.Y = vert.Y AndAlso tri.vertexA.X > SelectedVertices(0).X Then
                    If tri.vertexA.Y < 3 Then
                        tabove_col = 0
                    Else
                        If tri.vertexA.X > bmp.Width Then
                            tabove_col = bmp.GetPixel(bmp.Width - tri.vertexA.X + 1, tri.vertexA.Y - 2).ToArgb
                        Else
                            tabove_col = bmp.GetPixel(bmp.Width - tri.vertexA.X, tri.vertexA.Y - 2).ToArgb
                        End If
                    End If
                    If tri.vertexA.Y > bmp.Height Then
                        tbottom_col = 0
                    Else
                        If tri.vertexA.X > bmp.Width Then
                            tbottom_col = bmp.GetPixel(bmp.Width - tri.vertexA.X + 1, tri.vertexA.Y - 1).ToArgb
                        Else
                            tbottom_col = bmp.GetPixel(bmp.Width - tri.vertexA.X, tri.vertexA.Y - 1).ToArgb
                        End If
                    End If
                    SelectedVertices.Add(tri.vertexA)
                    If Not (above_col = tabove_col AndAlso bottom_col = tbottom_col) Then
                        Exit Sub
                    End If
                End If

                If tri.vertexB.selected AndAlso Not SelectedVertices.Contains(tri.vertexB) AndAlso tri.vertexB.Y = vert.Y AndAlso tri.vertexB.X > SelectedVertices(0).X Then
                    If tri.vertexB.Y < 3 Then
                        tabove_col = 0
                    Else
                        If tri.vertexB.X > bmp.Width Then
                            tabove_col = bmp.GetPixel(bmp.Width - tri.vertexB.X + 1, tri.vertexB.Y - 2).ToArgb
                        Else
                            tabove_col = bmp.GetPixel(bmp.Width - tri.vertexB.X, tri.vertexB.Y - 2).ToArgb
                        End If
                    End If
                    If tri.vertexB.Y > bmp.Height Then
                        tbottom_col = 0
                    Else
                        If tri.vertexB.X > bmp.Width Then
                            tbottom_col = bmp.GetPixel(bmp.Width - tri.vertexB.X + 1, tri.vertexB.Y - 1).ToArgb
                        Else
                            tbottom_col = bmp.GetPixel(bmp.Width - tri.vertexB.X, tri.vertexB.Y - 1).ToArgb
                        End If
                    End If
                    SelectedVertices.Add(tri.vertexB)
                    If Not (above_col = tabove_col AndAlso bottom_col = tbottom_col) Then
                        Exit Sub
                    End If

                End If

                If tri.vertexC.selected AndAlso Not SelectedVertices.Contains(tri.vertexC) AndAlso tri.vertexC.Y = vert.Y AndAlso tri.vertexB.X > SelectedVertices(0).X Then
                    If tri.vertexC.Y < 3 Then
                        tabove_col = 0
                    Else
                        If tri.vertexC.X > bmp.Width Then
                            tabove_col = bmp.GetPixel(bmp.Width - tri.vertexC.X + 1, tri.vertexC.Y - 2).ToArgb
                        Else
                            tabove_col = bmp.GetPixel(bmp.Width - tri.vertexC.X, tri.vertexC.Y - 2).ToArgb
                        End If
                    End If
                    If tri.vertexC.Y > bmp.Height Then
                        tbottom_col = 0
                    Else
                        If tri.vertexC.X > bmp.Width Then
                            tbottom_col = bmp.GetPixel(bmp.Width - tri.vertexC.X + 1, tri.vertexC.Y - 1).ToArgb
                        Else
                            tbottom_col = bmp.GetPixel(bmp.Width - tri.vertexC.X, tri.vertexC.Y - 1).ToArgb
                        End If
                    End If
                    SelectedVertices.Add(tri.vertexC)
                    If Not (above_col = tabove_col AndAlso bottom_col = tbottom_col) Then
                        Exit Sub
                    End If

                End If

            Next
        Next
        startcount = tmpcount + 1
        If SelectedVertices.Count > startcount Then GoTo nextstep
    End Sub

    Private Sub selectColourStrideVertical(ByRef bmp As Bitmap)
        Dim startcount As Integer = 0
        Dim left_col As Integer
        Dim right_col As Integer
        If SelectedVertices(0).X < 2 Then
            left_col = 0
        Else
            If SelectedVertices(0).Y > bmp.Height Then
                left_col = bmp.GetPixel(bmp.Width - SelectedVertices(0).X + 1, SelectedVertices(0).Y - 2).ToArgb
            Else
                left_col = bmp.GetPixel(bmp.Width - SelectedVertices(0).X + 1, SelectedVertices(0).Y - 1).ToArgb
            End If
        End If
        If SelectedVertices(0).X > bmp.Width Then
            right_col = 0
        Else
            If SelectedVertices(0).Y > bmp.Height Then
                right_col = bmp.GetPixel(bmp.Width - SelectedVertices(0).X, SelectedVertices(0).Y - 2).ToArgb
            Else
                right_col = bmp.GetPixel(bmp.Width - SelectedVertices(0).X, SelectedVertices(0).Y - 1).ToArgb
            End If
        End If
nextStep:
        Dim tmpcount As Integer = SelectedVertices.Count - 1
        For verti = startcount To tmpcount
            Dim vert As Vertex = SelectedVertices(verti)
            For Each tri As Triangle In vert.linkedTriangles
                Dim tleft_col As Integer
                Dim tright_col As Integer

                If tri.vertexA.selected AndAlso Not SelectedVertices.Contains(tri.vertexA) AndAlso tri.vertexA.X = vert.X AndAlso tri.vertexA.Y > SelectedVertices(0).Y Then
                    If tri.vertexA.X < 2 Then
                        tleft_col = 0
                    Else
                        If tri.vertexA.Y > bmp.Height Then
                            tleft_col = bmp.GetPixel(bmp.Width - tri.vertexA.X + 1, tri.vertexA.Y - 2).ToArgb
                        Else
                            tleft_col = bmp.GetPixel(bmp.Width - tri.vertexA.X + 1, tri.vertexA.Y - 1).ToArgb
                        End If
                    End If
                    If tri.vertexA.X > bmp.Width Then
                        tright_col = 0
                    Else
                        If tri.vertexA.Y > bmp.Height Then
                            tright_col = bmp.GetPixel(bmp.Width - tri.vertexA.X, tri.vertexA.Y - 2).ToArgb
                        Else
                            tright_col = bmp.GetPixel(bmp.Width - tri.vertexA.X, tri.vertexA.Y - 1).ToArgb
                        End If
                    End If
                    SelectedVertices.Add(tri.vertexA)
                    If Not (left_col = tleft_col AndAlso right_col = tright_col) Then
                        Exit Sub
                    End If
                End If

                If tri.vertexB.selected AndAlso Not SelectedVertices.Contains(tri.vertexB) AndAlso tri.vertexB.X = vert.X AndAlso tri.vertexB.Y > SelectedVertices(0).Y Then
                    If tri.vertexB.X < 2 Then
                        tleft_col = 0
                    Else
                        If tri.vertexB.Y > bmp.Height Then
                            tleft_col = bmp.GetPixel(bmp.Width - tri.vertexB.X + 1, tri.vertexB.Y - 2).ToArgb
                        Else
                            tleft_col = bmp.GetPixel(bmp.Width - tri.vertexB.X + 1, tri.vertexB.Y - 1).ToArgb
                        End If
                    End If
                    If tri.vertexB.X > bmp.Width Then
                        tright_col = 0
                    Else
                        If tri.vertexB.Y > bmp.Height Then
                            tright_col = bmp.GetPixel(bmp.Width - tri.vertexB.X, tri.vertexB.Y - 2).ToArgb
                        Else
                            tright_col = bmp.GetPixel(bmp.Width - tri.vertexB.X, tri.vertexB.Y - 1).ToArgb
                        End If
                    End If
                    SelectedVertices.Add(tri.vertexB)
                    If Not (left_col = tleft_col AndAlso right_col = tright_col) Then
                        Exit Sub
                    End If

                End If

                If tri.vertexC.selected AndAlso Not SelectedVertices.Contains(tri.vertexC) AndAlso tri.vertexC.X = vert.X AndAlso tri.vertexB.Y > SelectedVertices(0).Y Then
                    If tri.vertexC.X < 2 Then
                        tleft_col = 0
                    Else
                        If tri.vertexC.Y > bmp.Height Then
                            tleft_col = bmp.GetPixel(bmp.Width - tri.vertexC.X + 1, tri.vertexC.Y - 2).ToArgb
                        Else
                            tleft_col = bmp.GetPixel(bmp.Width - tri.vertexC.X + 1, tri.vertexC.Y - 1).ToArgb
                        End If
                    End If
                    If tri.vertexC.X > bmp.Width Then
                        tright_col = 0
                    Else
                        If tri.vertexC.Y > bmp.Height Then
                            tright_col = bmp.GetPixel(bmp.Width - tri.vertexC.X, tri.vertexC.Y - 2).ToArgb
                        Else
                            tright_col = bmp.GetPixel(bmp.Width - tri.vertexC.X, tri.vertexC.Y - 1).ToArgb
                        End If
                    End If
                    SelectedVertices.Add(tri.vertexC)
                    If Not (left_col = tleft_col AndAlso right_col = tright_col) Then
                        Exit Sub
                    End If

                End If

            Next
        Next
        startcount = tmpcount + 1
        If SelectedVertices.Count > startcount Then GoTo nextstep
    End Sub

    Private Sub selectTouching()
        If objectToModify = "Vertex" Then
            Dim tmpcount As Integer = SelectedVertices.Count - 1
            For verti = 0 To tmpcount
                Dim vert As Vertex = SelectedVertices(verti)
                For Each tri As Triangle In vert.linkedTriangles
                    If Not SelectedVertices.Contains(tri.vertexA) AndAlso tri.vertexA.groupindex < 0 Then tri.vertexA.selected = True : SelectedVertices.Add(tri.vertexA)
                    If Not SelectedVertices.Contains(tri.vertexB) AndAlso tri.vertexB.groupindex < 0 Then tri.vertexB.selected = True : SelectedVertices.Add(tri.vertexB)
                    If Not SelectedVertices.Contains(tri.vertexC) AndAlso tri.vertexC.groupindex < 0 Then tri.vertexC.selected = True : SelectedVertices.Add(tri.vertexC)
                Next
            Next
        ElseIf objectToModify = "Triangle" Then
            Dim tc As Short
            Dim tci As Integer
            If WithColourToolStripMenuItem.Checked AndAlso SelectedTriangles.Count > 0 Then tc = SelectedTriangles(0).myColourNumber : tci = SelectedTriangles(0).myColour.ToArgb
            Dim tmpcount As Integer = SelectedTriangles.Count - 1
            If WithColourToolStripMenuItem.Checked Then
                For trii = 0 To tmpcount
                    Dim tri As Triangle = SelectedTriangles(trii)
                    For Each tri2 As Triangle In tri.vertexA.linkedTriangles
                        If ((tri2.myColourNumber = tc AndAlso tc <> -1) OrElse (tc = -1 AndAlso tri2.myColour.ToArgb = tci)) AndAlso Not SelectedTriangles.Contains(tri2) AndAlso tri2.groupindex < 0 Then
                            If Not SelectedVertices.Contains(tri2.vertexA) AndAlso tri2.vertexA.groupindex < 0 Then tri2.vertexA.selected = True : SelectedVertices.Add(tri2.vertexA)
                            If Not SelectedVertices.Contains(tri2.vertexB) AndAlso tri2.vertexB.groupindex < 0 Then tri2.vertexB.selected = True : SelectedVertices.Add(tri2.vertexB)
                            If Not SelectedVertices.Contains(tri2.vertexC) AndAlso tri2.vertexC.groupindex < 0 Then tri2.vertexC.selected = True : SelectedVertices.Add(tri2.vertexC)
                            tri2.selected = True : SelectedTriangles.Add(tri2)
                        End If
                    Next
                    For Each tri2 As Triangle In tri.vertexB.linkedTriangles
                        If ((tri2.myColourNumber = tc AndAlso tc <> -1) OrElse (tc = -1 AndAlso tri2.myColour.ToArgb = tci)) AndAlso Not SelectedTriangles.Contains(tri2) AndAlso tri2.groupindex < 0 Then
                            If Not SelectedVertices.Contains(tri2.vertexA) AndAlso tri2.vertexA.groupindex < 0 Then tri2.vertexA.selected = True : SelectedVertices.Add(tri2.vertexA)
                            If Not SelectedVertices.Contains(tri2.vertexB) AndAlso tri2.vertexB.groupindex < 0 Then tri2.vertexB.selected = True : SelectedVertices.Add(tri2.vertexB)
                            If Not SelectedVertices.Contains(tri2.vertexC) AndAlso tri2.vertexC.groupindex < 0 Then tri2.vertexC.selected = True : SelectedVertices.Add(tri2.vertexC)
                            tri2.selected = True : SelectedTriangles.Add(tri2)
                        End If
                    Next
                    For Each tri2 As Triangle In tri.vertexC.linkedTriangles
                        If ((tri2.myColourNumber = tc AndAlso tc <> -1) OrElse (tc = -1 AndAlso tri2.myColour.ToArgb = tci)) AndAlso Not SelectedTriangles.Contains(tri2) AndAlso tri2.groupindex < 0 Then
                            If Not SelectedVertices.Contains(tri2.vertexA) AndAlso tri2.vertexA.groupindex < 0 Then tri2.vertexA.selected = True : SelectedVertices.Add(tri2.vertexA)
                            If Not SelectedVertices.Contains(tri2.vertexB) AndAlso tri2.vertexB.groupindex < 0 Then tri2.vertexB.selected = True : SelectedVertices.Add(tri2.vertexB)
                            If Not SelectedVertices.Contains(tri2.vertexC) AndAlso tri2.vertexC.groupindex < 0 Then tri2.vertexC.selected = True : SelectedVertices.Add(tri2.vertexC)
                            tri2.selected = True : SelectedTriangles.Add(tri2)
                        End If
                    Next
                Next
            Else
                For trii = 0 To tmpcount
                    Dim tri As Triangle = SelectedTriangles(trii)
                    For Each tri2 As Triangle In tri.vertexA.linkedTriangles
                        If Not SelectedTriangles.Contains(tri2) AndAlso tri2.groupindex < 0 Then
                            If Not SelectedVertices.Contains(tri2.vertexA) AndAlso tri2.vertexA.groupindex < 0 Then tri2.vertexA.selected = True : SelectedVertices.Add(tri2.vertexA)
                            If Not SelectedVertices.Contains(tri2.vertexB) AndAlso tri2.vertexB.groupindex < 0 Then tri2.vertexB.selected = True : SelectedVertices.Add(tri2.vertexB)
                            If Not SelectedVertices.Contains(tri2.vertexC) AndAlso tri2.vertexC.groupindex < 0 Then tri2.vertexC.selected = True : SelectedVertices.Add(tri2.vertexC)
                            tri2.selected = True : SelectedTriangles.Add(tri2)
                        End If
                    Next
                    For Each tri2 As Triangle In tri.vertexB.linkedTriangles
                        If Not SelectedTriangles.Contains(tri2) AndAlso tri2.groupindex < 0 Then
                            If Not SelectedVertices.Contains(tri2.vertexA) AndAlso tri2.vertexA.groupindex < 0 Then tri2.vertexA.selected = True : SelectedVertices.Add(tri2.vertexA)
                            If Not SelectedVertices.Contains(tri2.vertexB) AndAlso tri2.vertexB.groupindex < 0 Then tri2.vertexB.selected = True : SelectedVertices.Add(tri2.vertexB)
                            If Not SelectedVertices.Contains(tri2.vertexC) AndAlso tri2.vertexC.groupindex < 0 Then tri2.vertexC.selected = True : SelectedVertices.Add(tri2.vertexC)
                            tri2.selected = True : SelectedTriangles.Add(tri2)
                        End If
                    Next
                    For Each tri2 As Triangle In tri.vertexC.linkedTriangles
                        If Not SelectedTriangles.Contains(tri2) AndAlso tri2.groupindex < 0 Then
                            If Not SelectedVertices.Contains(tri2.vertexA) AndAlso tri2.vertexA.groupindex < 0 Then tri2.vertexA.selected = True : SelectedVertices.Add(tri2.vertexA)
                            If Not SelectedVertices.Contains(tri2.vertexB) AndAlso tri2.vertexB.groupindex < 0 Then tri2.vertexB.selected = True : SelectedVertices.Add(tri2.vertexB)
                            If Not SelectedVertices.Contains(tri2.vertexC) AndAlso tri2.vertexC.groupindex < 0 Then tri2.vertexC.selected = True : SelectedVertices.Add(tri2.vertexC)
                            tri2.selected = True : SelectedTriangles.Add(tri2)
                        End If
                    Next
                Next
            End If
        End If
    End Sub

    Private Sub selectTouching2()
        Dim tmpcount As Integer = SelectedVertices.Count - 1
        For verti = 0 To tmpcount
            Dim vert As Vertex = SelectedVertices(verti)
            vert.selected = True
            For Each tri As Triangle In vert.linkedTriangles
                If Not SelectedVertices.Contains(tri.vertexA) AndAlso Not tri.vertexA.selected Then SelectedVertices.Add(tri.vertexA) : LLast(SelectedVertices).selected = True
                If Not SelectedVertices.Contains(tri.vertexB) AndAlso Not tri.vertexB.selected Then SelectedVertices.Add(tri.vertexB) : LLast(SelectedVertices).selected = True
                If Not SelectedVertices.Contains(tri.vertexC) AndAlso Not tri.vertexC.selected Then SelectedVertices.Add(tri.vertexC) : LLast(SelectedVertices).selected = True
            Next
        Next
    End Sub

    Private Sub selectTouching3()
        Dim tmpcount As Integer = SelectedVertices.Count - 1
        For verti = 0 To tmpcount
            Dim vert As Vertex = SelectedVertices(verti)
            For Each tri As Triangle In vert.linkedTriangles
                If Not SelectedVertices.Contains(tri.vertexA) Then SelectedVertices.Add(tri.vertexA)
                If Not SelectedVertices.Contains(tri.vertexB) Then SelectedVertices.Add(tri.vertexB)
                If Not SelectedVertices.Contains(tri.vertexC) Then SelectedVertices.Add(tri.vertexC)
            Next
        Next
    End Sub

#End Region
#Region "Mirror"

    Dim deltaXmin As Single
    Dim deltaXmax As Single
    Dim deltaYmin As Single
    Dim deltaYmax As Single
    Private Sub MirrorXToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MirrorXToolStripMenuItem.Click
        If SelectedVertices.Count > 0 Then
            addHistoryI(withDataFrom.Vertices + withDataFrom.Primitives)
            mirrorX()
            addHistoryII(withDataFrom.Vertices + withDataFrom.Primitives)
            Me.Refresh()
        End If
    End Sub

    Private Sub MirrorYToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MirrorYToolStripMenuItem.Click
        If SelectedVertices.Count > 0 Then
            addHistoryI(withDataFrom.Vertices + withDataFrom.Primitives)
            mirrorY()
            addHistoryII(withDataFrom.Vertices + withDataFrom.Primitives)
            Me.Refresh()
        End If
    End Sub

    Private Sub MirrorXLeftToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MirrorXLeftToolStripMenuItem.Click
        If SelectedVertices.Count > 0 Then
            addHistoryI(withDataFrom.All)
            copy()
            paste()
            mirrorX()
            For i As Integer = 0 To SelectedVertices.Count - 1
                SelectedVertices(i).X += deltaXmax * 2
                SelectedVertices(i).selected = False
            Next
            If objectToModify <> "Primitive" Then
                cleanupDATVertices()
            Else
                Primitives(PrimitivesHMap(SelectedVertices(0).groupindex)).translate(deltaXmin * 2, 0)
            End If
            SelectedVertices.Clear()
            For i As Integer = 0 To SelectedTriangles.Count - 1
                SelectedTriangles(i).selected = False
            Next
            SelectedTriangles.Clear()
            addHistoryII(withDataFrom.All)
            Me.Refresh()
        End If
    End Sub

    Private Sub MirrorXRightToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MirrorXRightToolStripMenuItem.Click
        If SelectedVertices.Count > 0 Then
            addHistoryI(withDataFrom.All)
            copy()
            paste()
            mirrorX()
            For i As Integer = 0 To SelectedVertices.Count - 1
                SelectedVertices(i).X += deltaXmin * 2
                SelectedVertices(i).selected = False
            Next
            If objectToModify <> "Primitive" Then
                cleanupDATVertices()
            Else
                Primitives(PrimitivesHMap(SelectedVertices(0).groupindex)).translate(deltaXmax * 2, 0)
            End If
            SelectedVertices.Clear()
            For i As Integer = 0 To SelectedTriangles.Count - 1
                SelectedTriangles(i).selected = False
            Next
            SelectedTriangles.Clear()
            addHistoryII(withDataFrom.All)
            Me.Refresh()
        End If
    End Sub

    Private Sub MirrorYTopToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MirrorYTopToolStripMenuItem.Click
        If SelectedVertices.Count > 0 Then
            addHistoryI(withDataFrom.All)
            copy()
            paste()
            mirrorY()
            For i As Integer = 0 To SelectedVertices.Count - 1
                SelectedVertices(i).Y += deltaYmin * 2
                SelectedVertices(i).selected = False
            Next
            If objectToModify <> "Primitive" Then
                cleanupDATVertices()
            Else
                Primitives(PrimitivesHMap(SelectedVertices(0).groupindex)).translate(0, deltaYmax * 2)
            End If
            SelectedVertices.Clear()
            For i As Integer = 0 To SelectedTriangles.Count - 1
                SelectedTriangles(i).selected = False
            Next
            SelectedTriangles.Clear()
            addHistoryII(withDataFrom.All)
            Me.Refresh()
        End If
    End Sub

    Private Sub MirrorYBottomToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MirrorYBottomToolStripMenuItem.Click
        If SelectedVertices.Count > 0 Then
            addHistoryI(withDataFrom.All)
            copy()
            paste()
            mirrorY()
            For i As Integer = 0 To SelectedVertices.Count - 1
                SelectedVertices(i).Y += deltaYmax * 2
                SelectedVertices(i).selected = False
            Next
            If objectToModify <> "Primitive" Then
                cleanupDATVertices()
            Else
                Primitives(PrimitivesHMap(SelectedVertices(0).groupindex)).translate(0, deltaYmin * 2)
            End If
            SelectedVertices.Clear()
            For i As Integer = 0 To SelectedTriangles.Count - 1
                SelectedTriangles(i).selected = False
            Next
            SelectedTriangles.Clear()
            addHistoryII(withDataFrom.All)
            Me.Refresh()
        End If
    End Sub

    Private Sub mirrorX()
        deltaXmin = Single.PositiveInfinity
        deltaXmax = Single.NegativeInfinity
        If SelectedVertices.Count > 0 Then
            If objectToModify = "Primitive" Then
                Dim tid As Integer = Primitives(PrimitivesHMap(SelectedVertices(1).groupindex)).centerVertexID
                Primitives(PrimitivesHMap(SelectedVertices(1).groupindex)).mirror(True, False)
                For Each vert As Vertex In SelectedVertices
                    If vert.vertexID = tid Then
                        temp_center = New Vertex(vert.X, vert.Y, False, False)
                        Exit For
                    End If
                Next
            Else
                temp_center = New Vertex(0, 0, False, False)
                For Each vert As Vertex In SelectedVertices
                    temp_center += vert
                Next
                temp_center.X /= SelectedVertices.Count
                temp_center.Y /= SelectedVertices.Count
            End If
            For Each vert As Vertex In SelectedVertices
                vert.X = temp_center.X - (vert.X - temp_center.X)
                If (temp_center.X - vert.X) < deltaXmin Then deltaXmin = temp_center.X - vert.X
                If (temp_center.X - vert.X) > deltaXmax Then deltaXmax = temp_center.X - vert.X
            Next
        End If
    End Sub

    Private Sub mirrorY()
        deltaYmin = Single.PositiveInfinity
        deltaYmax = Single.NegativeInfinity
        If SelectedVertices.Count > 0 Then
            If objectToModify = "Primitive" Then
                Dim tid As Integer = Primitives(PrimitivesHMap(SelectedVertices(1).groupindex)).centerVertexID
                Primitives(PrimitivesHMap(SelectedVertices(1).groupindex)).mirror(False, True)
                For Each vert As Vertex In SelectedVertices
                    If vert.vertexID = tid Then
                        temp_center = New Vertex(vert.X, vert.Y, False, False)
                        Exit For
                    End If
                Next
            Else
                temp_center = New Vertex(0, 0, False, False)
                For Each vert As Vertex In SelectedVertices
                    temp_center += vert
                Next
                temp_center.X /= SelectedVertices.Count
                temp_center.Y /= SelectedVertices.Count
            End If
            For Each vert As Vertex In SelectedVertices
                vert.Y = temp_center.Y - (vert.Y - temp_center.Y)
                If (temp_center.Y - vert.Y) < deltaYmin Then deltaYmin = temp_center.Y - vert.Y
                If (temp_center.Y - vert.Y) > deltaYmax Then deltaYmax = temp_center.Y - vert.Y
            Next
        End If
    End Sub
#End Region
#Region "Colours"
    Private Sub C0_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles C0.Click
        addHistoryI(withDataFrom.Triangles + withDataFrom.SelectedTriangles + withDataFrom.Primitives)
        setColour(colourHMap(0), 0)
        addHistoryII(withDataFrom.Triangles + withDataFrom.SelectedTriangles + withDataFrom.Primitives)
    End Sub

    Private Sub C1_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles C1.Click
        addHistoryI(withDataFrom.Triangles + withDataFrom.SelectedTriangles + withDataFrom.Primitives)
        setColour(colourHMap(1), 1)
        addHistoryII(withDataFrom.Triangles + withDataFrom.SelectedTriangles + withDataFrom.Primitives)
    End Sub

    Private Sub C2_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles C2.Click
        addHistoryI(withDataFrom.Triangles + withDataFrom.SelectedTriangles + withDataFrom.Primitives)
        setColour(colourHMap(2), 2)
        addHistoryII(withDataFrom.Triangles + withDataFrom.SelectedTriangles + withDataFrom.Primitives)
    End Sub

    Private Sub C3_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles C3.Click
        addHistoryI(withDataFrom.Triangles + withDataFrom.SelectedTriangles + withDataFrom.Primitives)
        setColour(colourHMap(3), 3)
        addHistoryII(withDataFrom.Triangles + withDataFrom.SelectedTriangles + withDataFrom.Primitives)
    End Sub

    Private Sub C4_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles C4.Click
        addHistoryI(withDataFrom.Triangles + withDataFrom.SelectedTriangles + withDataFrom.Primitives)
        setColour(colourHMap(4), 4)
        addHistoryII(withDataFrom.Triangles + withDataFrom.SelectedTriangles + withDataFrom.Primitives)
    End Sub

    Private Sub C5_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles C5.Click
        addHistoryI(withDataFrom.Triangles + withDataFrom.SelectedTriangles + withDataFrom.Primitives)
        setColour(colourHMap(5), 5)
        addHistoryII(withDataFrom.Triangles + withDataFrom.SelectedTriangles + withDataFrom.Primitives)
    End Sub

    Private Sub C6_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles C6.Click
        addHistoryI(withDataFrom.Triangles + withDataFrom.SelectedTriangles + withDataFrom.Primitives)
        setColour(colourHMap(6), 6)
        addHistoryII(withDataFrom.Triangles + withDataFrom.SelectedTriangles + withDataFrom.Primitives)
    End Sub

    Private Sub C7_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles C7.Click
        addHistoryI(withDataFrom.Triangles + withDataFrom.SelectedTriangles + withDataFrom.Primitives)
        setColour(colourHMap(7), 7)
        addHistoryII(withDataFrom.Triangles + withDataFrom.SelectedTriangles + withDataFrom.Primitives)
    End Sub

    Private Sub C8_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles C8.Click
        addHistoryI(withDataFrom.Triangles + withDataFrom.SelectedTriangles + withDataFrom.Primitives)
        setColour(colourHMap(8), 8)
        addHistoryII(withDataFrom.Triangles + withDataFrom.SelectedTriangles + withDataFrom.Primitives)
    End Sub

    Private Sub C9_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles C9.Click
        addHistoryI(withDataFrom.Triangles + withDataFrom.SelectedTriangles + withDataFrom.Primitives)
        setColour(colourHMap(9), 9)
        addHistoryII(withDataFrom.Triangles + withDataFrom.SelectedTriangles + withDataFrom.Primitives)
    End Sub

    Private Sub C10_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles C10.Click
        addHistoryI(withDataFrom.Triangles + withDataFrom.SelectedTriangles + withDataFrom.Primitives)
        setColour(colourHMap(10), 10)
        addHistoryII(withDataFrom.Triangles + withDataFrom.SelectedTriangles + withDataFrom.Primitives)
    End Sub

    Private Sub C11_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles C11.Click
        addHistoryI(withDataFrom.Triangles + withDataFrom.SelectedTriangles + withDataFrom.Primitives)
        setColour(colourHMap(11), 11)
        addHistoryII(withDataFrom.Triangles + withDataFrom.SelectedTriangles + withDataFrom.Primitives)
    End Sub

    Private Sub C12_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles C12.Click
        addHistoryI(withDataFrom.Triangles + withDataFrom.SelectedTriangles + withDataFrom.Primitives)
        setColour(colourHMap(12), 12)
        addHistoryII(withDataFrom.Triangles + withDataFrom.SelectedTriangles + withDataFrom.Primitives)
    End Sub

    Private Sub C13_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles C13.Click
        addHistoryI(withDataFrom.Triangles + withDataFrom.SelectedTriangles + withDataFrom.Primitives)
        setColour(colourHMap(13), 13)
        addHistoryII(withDataFrom.Triangles + withDataFrom.SelectedTriangles + withDataFrom.Primitives)
    End Sub

    Private Sub C14_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles C14.Click
        addHistoryI(withDataFrom.Triangles + withDataFrom.SelectedTriangles + withDataFrom.Primitives)
        setColour(colourHMap(14), 14)
        addHistoryII(withDataFrom.Triangles + withDataFrom.SelectedTriangles + withDataFrom.Primitives)
    End Sub

    Private Sub C15_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles C15.Click
        addHistoryI(withDataFrom.Triangles + withDataFrom.SelectedTriangles + withDataFrom.Primitives)
        setColour(colourHMap(15), 15)
        addHistoryII(withDataFrom.Triangles + withDataFrom.SelectedTriangles + withDataFrom.Primitives)
    End Sub

    Private Sub C16_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles C16.Click
        addHistoryI(withDataFrom.Triangles + withDataFrom.SelectedTriangles + withDataFrom.Primitives)
        setColour(Nothing, 16)
        addHistoryII(withDataFrom.Triangles + withDataFrom.SelectedTriangles + withDataFrom.Primitives)
    End Sub

    Public Sub setColour(ByVal c As Color, ByVal cn As Short)
        If SelectedTriangles.Count > 0 Then
            If SelectedTriangles(0).groupindex <> -1 Then
                Primitives(PrimitivesHMap(SelectedTriangles(0).groupindex)).myColour = c
                Primitives(PrimitivesHMap(SelectedTriangles(0).groupindex)).myColourNumber = cn
            Else
                For Each tri As Triangle In SelectedTriangles
                    tri.myColour = c
                    tri.myColourNumber = cn
                Next
            End If
            Me.Refresh()
        End If
        lastColour = c
        lastColourNumber = cn
        ColourForm.NR.Value = lastColour.R
        ColourForm.NG.Value = lastColour.G
        ColourForm.NB.Value = lastColour.B
        CLast.BackColor = lastColour
        If cn = -1 Then
            CLast.Text = "0x2" & lastColour.R.ToString("X2") & lastColour.G.ToString("X2") & lastColour.B.ToString("X2")
        Else
            CLast.Text = lastColourNumber
        End If
    End Sub
#End Region
#Region "Undo & Redo"
    Private Sub UndoToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles UndoToolStripMenuItem.Click
        isLoading = True
        undo()
        isLoading = False
        Me.Refresh()
    End Sub

    Private Sub RedoToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RedoToolStripMenuItem.Click
        isLoading = True
        redo()
        isLoading = False
        Me.Refresh()
    End Sub

    Dim zeiger As Byte = 0

    Enum withDataFrom As UShort
        All = 0
        Vertices = 1
        Triangles = 2
        SelectedVertices = 4
        SelectedTriangles = 8
        Primitives = 16
        PrimitivesHash = 32
        TemplateShape = 64
        PrimitivesMetadataHash = 128
        TemplateProjectionQuads = 256
        TemplateText = 512
    End Enum

    Enum editMode As Byte
        Selection = 0
        Translation = 1
        Rotation = 2
        Scale = 3
    End Enum

    Dim last_what As Byte = 0
    Dim aHI As Boolean = False
    Public Sub addHistoryI(Optional ByVal what As UShort = withDataFrom.All)
        If Not PerformanceEnabledToolStripMenuItem.Checked Then
            If what <> last_what AndAlso historyVertices.Count > 0 Then
                last_what = what
                If what = withDataFrom.All OrElse what Mod 2 = 1 AndAlso historyVertices(historyVertices.Count - 1) Is Nothing Then historyVertices(historyVertices.Count - 1) = DeepClone(Vertices)
                If what = withDataFrom.All OrElse (what \ 2) Mod 2 = 1 AndAlso historyTriangles(historyTriangles.Count - 1) Is Nothing Then historyTriangles(historyTriangles.Count - 1) = DeepClone(Triangles)
                If what = withDataFrom.All OrElse (what \ 4) Mod 2 = 1 AndAlso historySelectedVertices(historySelectedVertices.Count - 1) Is Nothing Then historySelectedVertices(historySelectedVertices.Count - 1) = DeepClone(SelectedVertices)
                If what = withDataFrom.All OrElse (what \ 8) Mod 2 = 1 AndAlso historySelectedTriangles(historySelectedTriangles.Count - 1) Is Nothing Then historySelectedTriangles(historySelectedTriangles.Count - 1) = DeepClone(SelectedTriangles)
                If what = withDataFrom.All OrElse (what \ 16) Mod 2 = 1 AndAlso historyPrimitives(historyPrimitives.Count - 1) Is Nothing Then historyPrimitives(historyPrimitives.Count - 1) = DeepClone(Primitives)
                If what = withDataFrom.All OrElse (what \ 32) Mod 2 = 1 AndAlso historyPrimitivesHash(historyPrimitivesHash.Count - 1) Is Nothing Then historyPrimitivesHash(historyPrimitivesHash.Count - 1) = DeepClone(PrimitivesHMap)
                If what = withDataFrom.All OrElse (what \ 64) Mod 2 = 1 AndAlso historyTemplateShape(historyTemplateShape.Count - 1) Is Nothing Then historyTemplateShape(historyTemplateShape.Count - 1) = DeepClone(templateShape)
                If what = withDataFrom.All OrElse (what \ 128) Mod 2 = 1 AndAlso historyPrimitivesMetadataHash(historyPrimitivesMetadataHash.Count - 1) Is Nothing Then
                    historyPrimitivesMetadataHash(historyPrimitivesMetadataHash.Count - 1) = DeepClone(PrimitivesMetadataHMap)
                    historyMyMetadata(historyMyMetadata.Count - 1) = DeepClone(myMetadata)
                End If
                If what = withDataFrom.All OrElse (what \ 256) Mod 2 = 1 AndAlso historyProjectionQuads(historyProjectionQuads.Count - 1) Is Nothing Then historyProjectionQuads(historyProjectionQuads.Count - 1) = DeepClone(templateProjectionQuad)
                If what = withDataFrom.All OrElse (what \ 512) Mod 2 = 1 AndAlso historyTemplateText(historyTemplateText.Count - 1) Is Nothing Then historyTemplateText(historyTemplateText.Count - 1) = DeepClone(templateText)
            End If
            aHI = True
        End If
        unsavedChanges = True
    End Sub

    Public Sub addHistoryII(Optional ByVal what As UShort = withDataFrom.All)
        If Not PerformanceEnabledToolStripMenuItem.Checked Then
            If Not aHI Then
                Throw New Exception("addHistoryII was called w/o addHistoryII: " & what)
            End If
            aHI = False

            If zeiger < historyVertices.Count Then
                historyVertices.RemoveRange(zeiger + 1, historyVertices.Count - zeiger - 1)
                historyTriangles.RemoveRange(zeiger + 1, historyTriangles.Count - zeiger - 1)
                historySelectedVertices.RemoveRange(zeiger + 1, historySelectedVertices.Count - zeiger - 1)
                historySelectedTriangles.RemoveRange(zeiger + 1, historySelectedTriangles.Count - zeiger - 1)
                historyPrimitives.RemoveRange(zeiger + 1, historyPrimitives.Count - zeiger - 1)
                historyPrimitivesHash.RemoveRange(zeiger + 1, historyPrimitivesHash.Count - zeiger - 1)
                historyTemplateShape.RemoveRange(zeiger + 1, historyTemplateShape.Count - zeiger - 1)
                historyPrimitivesMetadataHash.RemoveRange(zeiger + 1, historyPrimitivesMetadataHash.Count - zeiger - 1)
                historyMyMetadata.RemoveRange(zeiger + 1, historyMyMetadata.Count - zeiger - 1)
                historyProjectionQuads.RemoveRange(zeiger + 1, historyProjectionQuads.Count - zeiger - 1)
                historyTemplateText.RemoveRange(zeiger + 1, historyTemplateText.Count - zeiger - 1)
            End If

            If what = withDataFrom.All OrElse what Mod 2 = 1 Then historyVertices.Add(DeepClone(Vertices)) Else historyVertices.Add(Nothing)
            If what = withDataFrom.All OrElse (what \ 2) Mod 2 = 1 Then historyTriangles.Add(DeepClone(Triangles)) Else historyTriangles.Add(Nothing)
            If what = withDataFrom.All OrElse (what \ 4) Mod 2 = 1 Then historySelectedVertices.Add(DeepClone(SelectedVertices)) Else historySelectedVertices.Add(Nothing)
            If what = withDataFrom.All OrElse (what \ 8) Mod 2 = 1 Then historySelectedTriangles.Add(DeepClone(SelectedTriangles)) Else historySelectedTriangles.Add(Nothing)
            If what = withDataFrom.All OrElse (what \ 16) Mod 2 = 1 Then historyPrimitives.Add(DeepClone(Primitives)) Else historyPrimitives.Add(Nothing)
            If what = withDataFrom.All OrElse (what \ 32) Mod 2 = 1 Then historyPrimitivesHash.Add(DeepClone(PrimitivesHMap)) Else historyPrimitivesHash.Add(Nothing)
            If what = withDataFrom.All OrElse (what \ 64) Mod 2 = 1 Then historyTemplateShape.Add(DeepClone(templateShape)) Else historyTemplateShape.Add(Nothing)
            If what = withDataFrom.All OrElse (what \ 128) Mod 2 = 1 Then
                historyPrimitivesMetadataHash.Add(DeepClone(PrimitivesMetadataHMap))
                historyMyMetadata.Add(DeepClone(myMetadata))
            Else
                historyPrimitivesMetadataHash.Add(Nothing)
                historyMyMetadata.Add(Nothing)
            End If
            If what = withDataFrom.All OrElse (what \ 256) Mod 2 = 1 Then historyProjectionQuads.Add(DeepClone(templateProjectionQuad)) Else historyProjectionQuads.Add(Nothing)
            If what = withDataFrom.All OrElse (what \ 512) Mod 2 = 1 Then historyTemplateText.Add(DeepClone(templateText)) Else historyTemplateText.Add(Nothing)
            If zeiger < mySettings.max_undo Then
                zeiger += 1
            Else
                historyVertices.RemoveAt(0)
                historyTriangles.RemoveAt(0)
                historySelectedVertices.RemoveAt(0)
                historySelectedTriangles.RemoveAt(0)
                historyPrimitives.RemoveAt(0)
                historyPrimitivesHash.RemoveAt(0)
                historyTemplateShape.RemoveAt(0)
                historyPrimitivesMetadataHash.RemoveAt(0)
                historyMyMetadata.RemoveAt(0)
                historyProjectionQuads.RemoveAt(0)
                historyTemplateText.RemoveAt(0)
            End If
            UndoToolStripMenuItem.Enabled = True
            RedoToolStripMenuItem.Enabled = False
        End If
    End Sub

    Private Sub undo()
        If zeiger = historyVertices.Count Then zeiger -= 1
        If zeiger > 0 Then
            zeiger -= 1
            If Not historyVertices(zeiger) Is Nothing Then Vertices.Clear() : Vertices.AddRange(historyVertices(zeiger).ToArray)
            If Not historySelectedVertices(zeiger) Is Nothing Then
                For i As Integer = 0 To SelectedVertices.Count - 1
                    SelectedVertices(i).selected = False
                Next
                SelectedVertices.Clear() : SelectedVertices.AddRange(historySelectedVertices(zeiger).ToArray)
            End If
            If Not historyTriangles(zeiger) Is Nothing Then Triangles.Clear() : Triangles.AddRange(historyTriangles(zeiger).ToArray)
            If Not historySelectedTriangles(zeiger) Is Nothing Then
                For i As Integer = 0 To SelectedTriangles.Count - 1
                    SelectedTriangles(i).selected = False
                Next
                SelectedTriangles.Clear() : SelectedTriangles.AddRange(historySelectedTriangles(zeiger).ToArray)
            End If
            If Not historyPrimitives(zeiger) Is Nothing Then Primitives.Clear() : Primitives.AddRange(historyPrimitives(zeiger).ToArray)
            If Not historyPrimitivesHash(zeiger) Is Nothing Then PrimitivesHMap = historyPrimitivesHash(zeiger).Clone
            If Not historyPrimitivesMetadataHash(zeiger) Is Nothing Then PrimitivesMetadataHMap = historyPrimitivesMetadataHash(zeiger).Clone : myMetadata = historyMyMetadata(zeiger).Clone()
            If Not historyTemplateShape(zeiger) Is Nothing Then templateShape.Clear() : templateShape.AddRange(historyTemplateShape(zeiger).ToArray)
            If Not historyProjectionQuads(zeiger) Is Nothing Then templateProjectionQuad.Clear() : templateProjectionQuad.AddRange(historyProjectionQuads(zeiger).ToArray) : alreadyCalculatedQuads = 0
            If Not historyTemplateText(zeiger) Is Nothing Then templateText.Clear() : templateText.AddRange(historyTemplateText(zeiger).ToArray)
            RedoToolStripMenuItem.Enabled = True
            cleanupHistoryData()
        End If
        UndoToolStripMenuItem.Enabled = zeiger > 0
    End Sub

    Private Sub redo()
        If zeiger < historyVertices.Count - 1 Then
            zeiger += 1
            If Not historyVertices(zeiger) Is Nothing Then Vertices.Clear() : Vertices.AddRange(historyVertices(zeiger).ToArray)
            If Not historySelectedVertices(zeiger) Is Nothing Then
                For i As Integer = 0 To SelectedVertices.Count - 1
                    SelectedVertices(i).selected = False
                Next
                SelectedVertices.Clear() : SelectedVertices.AddRange(historySelectedVertices(zeiger).ToArray)
            End If
            If Not historyTriangles(zeiger) Is Nothing Then Triangles.Clear() : Triangles.AddRange(historyTriangles(zeiger).ToArray)
            If Not historySelectedTriangles(zeiger) Is Nothing Then
                For i As Integer = 0 To SelectedTriangles.Count - 1
                    SelectedTriangles(i).selected = False
                Next
                SelectedTriangles.Clear() : SelectedTriangles.AddRange(historySelectedTriangles(zeiger).ToArray)
            End If
            If Not historyPrimitives(zeiger) Is Nothing Then Primitives.Clear() : Primitives.AddRange(historyPrimitives(zeiger).ToArray)
            If Not historyPrimitivesHash(zeiger) Is Nothing Then PrimitivesHMap = historyPrimitivesHash(zeiger).Clone : myMetadata = historyMyMetadata(zeiger).Clone()
            If Not historyPrimitivesMetadataHash(zeiger) Is Nothing Then PrimitivesMetadataHMap = historyPrimitivesMetadataHash(zeiger).Clone
            If Not historyTemplateShape(zeiger) Is Nothing Then templateShape.Clear() : templateShape.AddRange(historyTemplateShape(zeiger).ToArray)
            If Not historyProjectionQuads(zeiger) Is Nothing Then templateProjectionQuad.Clear() : templateProjectionQuad.AddRange(historyProjectionQuads(zeiger).ToArray) : alreadyCalculatedQuads = 0
            If Not historyTemplateText(zeiger) Is Nothing Then templateText.Clear() : templateText.AddRange(historyTemplateText(zeiger).ToArray)
            UndoToolStripMenuItem.Enabled = True
            cleanupHistoryData()
        End If
        RedoToolStripMenuItem.Enabled = zeiger < (historyVertices.Count - 1)
    End Sub

    Private Sub cleanupHistoryData()

        Dim VIDtoVI As New Hashtable(Vertices.Count)
        Dim TIDtoTI As New Hashtable(Triangles.Count)

        For i As Integer = 0 To Vertices.Count - 1
            VIDtoVI.Add(Vertices(i).vertexID, i)
        Next

        For i As Integer = 0 To Triangles.Count - 1
            TIDtoTI.Add(Triangles(i).triangleID, i)
            Triangles(i).vertexA = Vertices(VIDtoVI(Triangles(i).vertexA.vertexID))
            Triangles(i).vertexB = Vertices(VIDtoVI(Triangles(i).vertexB.vertexID))
            Triangles(i).vertexC = Vertices(VIDtoVI(Triangles(i).vertexC.vertexID))
        Next

        For i As Integer = 0 To SelectedTriangles.Count - 1
            SelectedTriangles(i) = Triangles(TIDtoTI(SelectedTriangles(i).triangleID))
            SelectedTriangles(i).selected = True
        Next

        For i As Integer = 0 To SelectedVertices.Count - 1
            SelectedVertices(i) = Vertices(VIDtoVI(SelectedVertices(i).vertexID))
            SelectedVertices(i).selected = True
        Next

        For i As Integer = 0 To Vertices.Count - 1
            For j As Integer = 0 To Vertices(i).linkedTriangles.Count - 1
                Vertices(i).linkedTriangles(j) = Triangles(TIDtoTI(Vertices(i).linkedTriangles(j).triangleID))
            Next
        Next

    End Sub
#End Region
#Region "File Menue"
    Private Sub LoadPatternToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles LoadPatternToolStripMenuItem.Click
        If LoadFile.ShowDialog() = Windows.Forms.DialogResult.OK Then
            If LoadFile.FileName <> "" Then
                LoadLPCFile(LoadFile.FileName)
            End If
        End If
    End Sub

    Private Sub LoadLPCFile(ByVal myFilename As String)
        isLoading = True
        addHistoryI(withDataFrom.All)
        newPattern()
        BtnMove.PerformClick()
        Try
            Using DateiIn As StreamReader = My.Computer.FileSystem.OpenTextFileReader(myFilename, System.Text.Encoding.GetEncoding(20127))
                Dim header As String = DateiIn.ReadLine()
                If header = "0 LDraw - Pattern Creator 1.1 ASCII File" OrElse header = "0 LDraw - Pattern Creator 1.2 ASCII File" Then
                    ' Load Triangles:
                    Dim trianz As Integer = CType(DateiIn.ReadLine(), Integer)
                    Dim myColour, vID1, vID2, vID3, triangleID, groupindex As Integer
                    Dim myColourNumber As Short
                    For tri As Integer = 1 To trianz
                        triangleID = CType(DateiIn.ReadLine(), Integer)
                        myColour = CType(DateiIn.ReadLine(), Integer)
                        myColourNumber = CType(DateiIn.ReadLine(), Short)
                        vID1 = CType(DateiIn.ReadLine(), Integer)
                        vID2 = CType(DateiIn.ReadLine(), Integer)
                        vID3 = CType(DateiIn.ReadLine(), Integer)
                        groupindex = CType(DateiIn.ReadLine(), Integer)
                        Triangles.Add(New Triangle(New Vertex(0, 0, False, False) With {.vertexID = vID1} _
                                                 , New Vertex(0, 0, False, False) With {.vertexID = vID2} _
                                                 , New Vertex(0, 0, False, False) With {.vertexID = vID3}, False) _
                                                 With {.myColour = Color.FromArgb(myColour), .myColourNumber = myColourNumber, .triangleID = triangleID, .groupindex = groupindex})
                    Next
                    ' Load Vertices:
                    Dim vertanz As Integer = CType(DateiIn.ReadLine(), Integer)
                    Dim X, Y As Single
                    Dim vertexID, triangleID2 As Integer
                    For vert As Integer = 1 To vertanz
                        vertexID = CType(DateiIn.ReadLine(), Integer)
                        X = CType(Replace(DateiIn.ReadLine, ".", comma), Single)
                        Y = CType(Replace(DateiIn.ReadLine, ".", comma), Single)
                        groupindex = CType(DateiIn.ReadLine(), Integer)
                        Vertices.Add(New Vertex(X, Y, False, False) With {.vertexID = vertexID, .groupindex = groupindex})
                        Dim linkedtrianz As Integer = CType(DateiIn.ReadLine(), Integer)
                        For tri As Integer = 1 To linkedtrianz
                            triangleID2 = CType(DateiIn.ReadLine(), Integer)
                            LLast(Vertices).linkedTriangles.Add(New Triangle(Nothing, Nothing, Nothing, False) With {.triangleID = triangleID2})
                        Next
                    Next
                    ' Load Primitives:
                    Dim primanz As Integer = CType(DateiIn.ReadLine(), Integer)
                    Dim ox, oy As Single
                    Dim primitiveName As String
                    Dim primitiveID As Integer
                    Dim centerVertexID As Integer
                    For prim As Integer = 1 To primanz
                        Dim matrix(3, 3) As Double
                        primitiveID = CType(DateiIn.ReadLine(), Integer)
                        centerVertexID = CType(DateiIn.ReadLine(), Integer)
                        PrimitivesHMap.Add(primitiveID, prim - 1)
                        ox = CType(Replace(DateiIn.ReadLine, ".", comma), Double)
                        oy = CType(Replace(DateiIn.ReadLine, ".", comma), Double)
                        primitiveName = DateiIn.ReadLine()
                        If header = "0 LDraw - Pattern Creator 1.2 ASCII File" Then myColour = CType(DateiIn.ReadLine(), Integer)
                        myColourNumber = CType(DateiIn.ReadLine(), Short)
                        For z As Integer = 0 To 3
                            For s As Integer = 0 To 3
                                matrix(z, s) = CType(Replace(DateiIn.ReadLine, ".", comma), Double)
                            Next
                        Next
                        Primitives.Add(New Primitive(0, 0, ox, oy, primitiveName, centerVertexID, False) With {.matrix = matrix.Clone, .primitiveID = primitiveID, .myColourNumber = myColourNumber, .myColour = Color.FromArgb(myColour)})
                    Next
                    ' Load Metadata:
                    DateiIn.ReadLine()
                    myMetadata.recommendedMode = CType(DateiIn.ReadLine(), Byte)
                    myMetadata.additionalData = DateiIn.ReadLine()
                    For r As Integer = 0 To 3
                        For c As Integer = 0 To 3
                            myMetadata.matrix(r, c) = CType(Replace(DateiIn.ReadLine, ".", comma), Single)
                        Next
                    Next
                    For i As Integer = 0 To 11
                        myMetadata.mData(i) = DateiIn.ReadLine()
                    Next i
                    Do While DateiIn.ReadLine() = "--Metadata--"
                        Dim mData(11) As String
                        Dim key As String = DateiIn.ReadLine()
                        For i As Integer = 0 To 11
                            mData(i) = DateiIn.ReadLine()
                        Next i
                        PrimitivesMetadataHMap.Add(key, New Metadata(mData(0), mData(1), mData(2), mData(3), mData(4), mData(5), mData(6), mData(7), mData(8), mData(9), mData(10), mData(11)))
                    Loop

                    Dim templateShapeCount As Integer = CType(DateiIn.ReadLine(), Integer)
                    For i As Integer = 0 To templateShapeCount - 1
                        Dim tx, ty As Single
                        tx = CType(Replace(DateiIn.ReadLine, ".", comma), Single)
                        ty = CType(Replace(DateiIn.ReadLine, ".", comma), Single)
                        templateShape.Add(New PointF(tx, ty))
                    Next
                    If header = "0 LDraw - Pattern Creator 1.2 ASCII File" Then
                        Dim templateQuadCount As Integer = CType(DateiIn.ReadLine(), Integer)
                        alreadyCalculatedQuads = CType(DateiIn.ReadLine(), Integer)
                        For i As Integer = 0 To templateQuadCount - 1
                            Dim s() As String = Replace(DateiIn.ReadLine, ".", comma).Split(" ")
                            Dim single_array(s.Length - 1) As Single
                            For j As Integer = 0 To s.Length - 1
                                single_array(j) = CType(s(j), Single)
                            Next
                            templateProjectionQuad.Add(New ProjectionQuad( _
                            single_array(0), single_array(1), single_array(2), single_array(3), single_array(4), _
                            single_array(5), single_array(6), single_array(7), single_array(8), single_array(9), _
                            single_array(10), single_array(11), single_array(12), single_array(13), single_array(14), _
                            single_array(15), single_array(16), single_array(17), single_array(18), single_array(19)))
                            For r As Integer = 0 To 3
                                For c As Integer = 0 To 3
                                    templateProjectionQuad(i).matrix(r, c) = CType(Replace(DateiIn.ReadLine, ".", comma), Double)
                                Next
                            Next
                        Next
                        Dim templateTextCount As Integer = CType(DateiIn.ReadLine(), Integer)
                        For i As Integer = 0 To templateTextCount - 1
                            Dim Text As String = Replace(DateiIn.ReadLine, ".", comma)
                            X = CType(Replace(DateiIn.ReadLine, ".", comma), Single)
                            Y = CType(Replace(DateiIn.ReadLine, ".", comma), Single)
                            templateText.Add(New TemplateTextInfo(X, Y, Text))
                        Next
                    End If
                    DateiIn.ReadLine()
                    includeMetadata = CType(DateiIn.ReadLine(), Boolean)
                    MetadataDialog.CBInclude.Checked = includeMetadata
                    vertexIDglobal = CType(DateiIn.ReadLine(), Integer)
                    triangleIDglobal = CType(DateiIn.ReadLine(), Integer)
                    primitiveIDglobal = CType(DateiIn.ReadLine(), Integer)

                    TBImage.Text = DateiIn.ReadLine()
                    Try
                        If TBImage.Text <> "" Then
                            backgroundPicture.Dispose()
                            backgroundPicture = New Bitmap(TBImage.Text)
                        Else
                            backgroundPicture = My.Resources.temp
                        End If
                    Catch
                        backgroundPicture = My.Resources.temp
                    End Try
                    NUDoffsetX.Maximum = Integer.MaxValue
                    NUDoffsetY.Maximum = Integer.MaxValue
                    NUDScale.Maximum = Decimal.MaxValue
                    NUDoffsetX.Minimum = Integer.MinValue
                    NUDoffsetY.Minimum = Integer.MinValue
                    NUDScale.Minimum = Decimal.MinValue
                    NUDoffsetX.Value = CType(Replace(DateiIn.ReadLine, ".", comma), Integer)
                    NUDoffsetY.Value = CType(Replace(DateiIn.ReadLine, ".", comma), Integer)
                    NUDScale.Value = CType(Replace(DateiIn.ReadLine, ".", comma), Decimal)
                    factor = CType(Replace(DateiIn.ReadLine, ".", comma), Single)
                    unit = DateiIn.ReadLine()
                    Select Case unit
                        Case "LDU"
                            LDUToolStripMenuItem.PerformClick()
                        Case "mm"
                            MillimeterToolStripMenuItem.PerformClick()
                        Case "inch"
                            InchToolStripMenuItem.PerformClick()
                    End Select

                    NUDMoveSnap.Value = CType(Replace(DateiIn.ReadLine, ".", comma), Decimal)
                    NUDRotateSnap.Value = CType(Replace(DateiIn.ReadLine, ".", comma), Decimal)
                    NUDScaleSnap.Value = CType(Replace(DateiIn.ReadLine, ".", comma), Decimal)
                    NUDGrid.Value = CType(Replace(DateiIn.ReadLine, ".", comma), Decimal)


                    LblSize.Text = imgScale * backgroundPicture.Width / 1000 * factor & " x " & vbCrLf & imgScale * backgroundPicture.Height / 1000 * factor & unit
                End If
                If header = "0 LDraw - Pattern Creator 1.0 ASCII File" Then
                    ' Load Triangles:
                    Dim trianz As Integer = CType(DateiIn.ReadLine(), Integer)
                    Dim myColour, vID1, vID2, vID3, triangleID As Integer
                    Dim myColourNumber As Short
                    For tri As Integer = 1 To trianz
                        triangleID = CType(DateiIn.ReadLine(), Integer)
                        myColour = CType(DateiIn.ReadLine(), Integer)
                        myColourNumber = CType(DateiIn.ReadLine(), Short)
                        vID1 = CType(DateiIn.ReadLine(), Integer)
                        vID2 = CType(DateiIn.ReadLine(), Integer)
                        vID3 = CType(DateiIn.ReadLine(), Integer)
                        Triangles.Add(New Triangle(New Vertex(0, 0, False, False) With {.vertexID = vID1} _
                                                 , New Vertex(0, 0, False, False) With {.vertexID = vID2} _
                                                 , New Vertex(0, 0, False, False) With {.vertexID = vID3}, False) _
                                                 With {.myColour = Color.FromArgb(myColour), .myColourNumber = myColourNumber, .triangleID = triangleID})
                    Next
                    ' Load Vertices:
                    Dim vertanz As Integer = CType(DateiIn.ReadLine(), Integer)
                    Dim X, Y As Single
                    Dim vertexID, triangleID2 As Integer
                    For vert As Integer = 1 To vertanz
                        vertexID = CType(DateiIn.ReadLine(), Integer)
                        X = CType(Replace(DateiIn.ReadLine, ".", comma), Single)
                        Y = CType(Replace(DateiIn.ReadLine, ".", comma), Single)
                        Vertices.Add(New Vertex(-X, -Y, False, False) With {.vertexID = vertexID})
                        Dim linkedtrianz As Integer = CType(DateiIn.ReadLine(), Integer)
                        For tri As Integer = 1 To linkedtrianz
                            triangleID2 = CType(DateiIn.ReadLine(), Integer)
                            LLast(Vertices).linkedTriangles.Add(New Triangle(Nothing, Nothing, Nothing, False) With {.triangleID = triangleID2})
                        Next
                    Next
                    vertexIDglobal = CType(DateiIn.ReadLine(), Integer)
                    triangleIDglobal = CType(DateiIn.ReadLine(), Integer)

                    TBImage.Text = DateiIn.ReadLine()
                    Try
                        If TBImage.Text <> "" Then
                            backgroundPicture.Dispose()
                            backgroundPicture = New Bitmap(TBImage.Text)
                        Else
                            backgroundPicture = My.Resources.temp
                        End If
                    Catch
                        backgroundPicture = My.Resources.temp
                    End Try
                    NUDoffsetX.Maximum = Integer.MaxValue
                    NUDoffsetY.Maximum = Integer.MaxValue
                    NUDScale.Maximum = Decimal.MaxValue
                    NUDoffsetX.Minimum = Integer.MinValue
                    NUDoffsetY.Minimum = Integer.MinValue
                    NUDScale.Minimum = Decimal.MinValue
                    NUDoffsetX.Value = CType(DateiIn.ReadLine(), Integer)
                    NUDoffsetY.Value = CType(DateiIn.ReadLine(), Integer)
                    NUDScale.Value = CType(Replace(DateiIn.ReadLine, ".", comma), Decimal)
                    LblSize.Text = imgScale * backgroundPicture.Width / 1000 * factor & " x " & vbCrLf & imgScale * backgroundPicture.Height / 1000 * factor & unit
                    cleanupOldData()
                End If
            End Using
            SaveAs.FileName = LoadFile.FileName
            ProjectionQuadTimer.Enabled = templateProjectionQuad.Count > 0
        Catch ex As Exception
            isLoading = False
            unsavedChanges = False
            Exit Sub
        End Try
        isLoading = False
        cleanupHistoryData()
        addHistoryII(withDataFrom.All)
        unsavedChanges = False
        Me.Refresh()
    End Sub


    Private Sub SaveToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SaveToolStripMenuItem.Click
        If SaveAs.FileName <> "" Then
            Using DateiOut As StreamWriter = My.Computer.FileSystem.OpenTextFileWriter(SaveAs.FileName, False, System.Text.Encoding.GetEncoding(20127))
                DateiOut.WriteLine("0 LDraw - Pattern Creator 1.2 ASCII File")
                ' Dreiecke speichern:
                DateiOut.WriteLine(Triangles.Count)
                For Each tri As Triangle In Triangles
                    DateiOut.WriteLine(tri.triangleID)
                    DateiOut.WriteLine(tri.myColour.ToArgb)
                    DateiOut.WriteLine(tri.myColourNumber)
                    DateiOut.WriteLine(tri.vertexA.vertexID)
                    DateiOut.WriteLine(tri.vertexB.vertexID)
                    DateiOut.WriteLine(tri.vertexC.vertexID)
                    DateiOut.WriteLine(tri.groupindex)
                Next
                ' Vertices speichern:
                DateiOut.WriteLine(Vertices.Count)
                For Each vert As Vertex In Vertices
                    DateiOut.WriteLine(vert.vertexID)
                    DateiOut.WriteLine(globalize(vert.X))
                    DateiOut.WriteLine(globalize(vert.Y))
                    DateiOut.WriteLine(vert.groupindex)
                    DateiOut.WriteLine(vert.linkedTriangles.Count)
                    For Each tri As Triangle In vert.linkedTriangles
                        DateiOut.WriteLine(tri.triangleID)
                    Next
                Next
                ' Primitive speichern:
                DateiOut.WriteLine(Primitives.Count)
                For Each prim As Primitive In Primitives
                    DateiOut.WriteLine(prim.primitiveID)
                    DateiOut.WriteLine(prim.centerVertexID)
                    DateiOut.WriteLine(globalize(prim.ox))
                    DateiOut.WriteLine(globalize(prim.oy))
                    DateiOut.WriteLine(prim.primitiveName)
                    DateiOut.WriteLine(prim.myColour.ToArgb)
                    DateiOut.WriteLine(prim.myColourNumber)
                    For z As Integer = 0 To 3
                        For s As Integer = 0 To 3
                            DateiOut.WriteLine(globalize(prim.matrix(z, s)))
                        Next
                    Next
                Next
                ' Metadata speichern:
                DateiOut.WriteLine("--Metadata--")
                DateiOut.WriteLine(myMetadata.recommendedMode)
                DateiOut.WriteLine(myMetadata.additionalData)
                For r As Integer = 0 To 3
                    For c As Integer = 0 To 3
                        DateiOut.WriteLine(globalize(myMetadata.matrix(r, c)))
                    Next
                Next
                For i As Integer = 0 To 11
                    DateiOut.WriteLine(myMetadata.mData(i))
                Next i

                Dim prims As New Hashtable(Primitives.Count)
                For Each i As Integer In PrimitivesHMap.Values
                    If Not prims.Contains(Primitives(i).primitiveName) Then
                        prims.Add(Primitives(i).primitiveName, Nothing)
                        If Primitives(i).primitiveName Like "subfile*" OrElse Primitives(i).primitiveName Like "s\*" Then
                            Dim m As Metadata = PrimitivesMetadataHMap.Item(Primitives(i).primitiveName)
                            DateiOut.WriteLine("--Metadata--")
                            DateiOut.WriteLine(Primitives(i).primitiveName)
                            For j As Integer = 0 To 11
                                DateiOut.WriteLine(m.mData(j))
                            Next
                        End If
                    End If
                Next
                DateiOut.WriteLine("--Template--")
                DateiOut.WriteLine(templateShape.Count)
                For i As Integer = 0 To templateShape.Count - 1
                    DateiOut.WriteLine(globalize(templateShape(i).X))
                    DateiOut.WriteLine(globalize(templateShape(i).Y))
                Next
                DateiOut.WriteLine(templateProjectionQuad.Count)
                DateiOut.WriteLine(alreadyCalculatedQuads)
                For i As Integer = 0 To templateProjectionQuad.Count - 1
                    DateiOut.WriteLine(templateProjectionQuad(i).toString())
                    For r As Integer = 0 To 3
                        For c As Integer = 0 To 3
                            DateiOut.WriteLine(globalize(templateProjectionQuad(i).matrix(r, c)))
                        Next
                    Next
                Next
                DateiOut.WriteLine(templateText.Count)
                For i = 0 To templateText.Count - 1
                    DateiOut.WriteLine(templateText(i).Text)
                    DateiOut.WriteLine(globalize(templateText(i).X))
                    DateiOut.WriteLine(globalize(templateText(i).Y))
                Next
                DateiOut.WriteLine("--Settings--")

                DateiOut.WriteLine(MetadataDialog.CBInclude.Checked.ToString)
                DateiOut.WriteLine(vertexIDglobal)
                DateiOut.WriteLine(triangleIDglobal)
                DateiOut.WriteLine(primitiveIDglobal)

                DateiOut.WriteLine(TBImage.Text)
                DateiOut.WriteLine(globalize(Fix(NUDoffsetX.Value)))
                DateiOut.WriteLine(globalize(Fix(NUDoffsetY.Value)))
                DateiOut.WriteLine(globalize(NUDScale.Value))

                DateiOut.WriteLine(globalize(factor))
                DateiOut.WriteLine(unit)
                DateiOut.WriteLine(globalize(NUDMoveSnap.Value))
                DateiOut.WriteLine(globalize(NUDRotateSnap.Value))
                DateiOut.WriteLine(globalize(NUDScaleSnap.Value))
                DateiOut.WriteLine(globalize(NUDGrid.Value))
            End Using
        Else
            SaveAsToolStripMenuItem.PerformClick()
        End If
    End Sub

    Private Function globalize(ByVal s As String) As String
        Return Replace(s, comma, ".")
    End Function

    Private Sub SaveAsToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SaveAsToolStripMenuItem.Click
        If SaveAs.ShowDialog() = Windows.Forms.DialogResult.OK Then
            If SaveAs.FileName <> "" Then
                SaveToolStripMenuItem.PerformClick()
            End If
        End If
    End Sub

    Private Function useRecommendedProjection(ByVal projectMode As Byte) As Boolean
        If myMetadata.recommendedMode <> 6 AndAlso myMetadata.recommendedMode <> projectMode Then
            Dim shouldBe As String = ""
            Select Case myMetadata.recommendedMode
                Case 0 : shouldBe = "YZ plane (-X, Right)"
                Case 1 : shouldBe = "XZ plane (-Y, Top)"
                Case 2 : shouldBe = "XY plane (-Z, Front)"
                Case 3 : shouldBe = "YZ plane (+X, Left)"
                Case 4 : shouldBe = "XZ plane (+Y, Bottom)"
                Case 5 : shouldBe = "XY plane (+Z, Back)"
            End Select
            Dim isInstead As String = ""
            Select Case projectMode
                Case 0 : isInstead = "YZ plane (-X, Right)"
                Case 1 : isInstead = "XZ plane (-Y, Top)"
                Case 2 : isInstead = "XY plane (-Z, Front)"
                Case 3 : isInstead = "YZ plane (+X, Left)"
                Case 4 : isInstead = "XZ plane (+Y, Bottom)"
                Case 5 : isInstead = "XY plane (+Z, Back)"
            End Select
            If MsgBox("This pattern should ideally be projected on " & shouldBe & ". Do you really want to project this pattern on " & isInstead & "?", MsgBoxStyle.YesNo + MsgBoxStyle.MsgBoxSetForeground + MsgBoxStyle.Question + MsgBoxStyle.DefaultButton2, "Question:") = MsgBoxResult.No Then Return False
        End If
        Return True
    End Function

    Private Function userWantsToExportDAT(ByVal projectMode As Byte) As Boolean
        Select Case projectMode
            Case 0 ' YZ mit -X (Right) [OK]
                SaveAsDAT.Title = "Export to DAT, project on YZ (Right):"
            Case 3 ' YZ mit +X (Left) [OK]
                SaveAsDAT.Title = "Export to DAT, project on YZ (Left):"
            Case 1 ' XZ mit -Y (Top) [OK]
                SaveAsDAT.Title = "Export to DAT, project on XZ (Top):"
            Case 4 ' XZ mit -Y (Bottom) [OK]
                SaveAsDAT.Title = "Export to DAT, project on XZ (Bottom):"
            Case 2 ' XY mit -Z (Front) [OK]
                SaveAsDAT.Title = "Export to DAT, project on XY (Front):"
            Case 5 ' XY mit +Z (Back) [OK]
                SaveAsDAT.Title = "Export to DAT, project on XY (Back):"
        End Select
        If (includeMetadata AndAlso myMetadata.mData(1) <> "") Then SaveAsDAT.FileName = myMetadata.mData(1)
        If SaveAsDAT.ShowDialog() = Windows.Forms.DialogResult.OK Then
            If SaveAsDAT.FileName <> "" Then
                Return True
            End If
        End If
        Return False
    End Function

    Private Sub exportDat2(ByVal Params As Object())
        If ProjectionQuadTimer.Enabled Then
            ProjectionQuadTimer.Enabled = False
            Thread.Sleep(500)
        End If
        Dim projectMode As Byte = CType(Params(0), Byte)
        CDialog.Controls("LblStatus").Text = "(1/4) Exporting subfiles and part header.."
        isLoading = True
        MenuStrip1.Enabled = False
        Me.ToolStrip1.Enabled = False
        Me.ColourToolStrip.Enabled = False
        GBImage.Enabled = False
        GBSnap.Enabled = False

        Dim filePath As String = Mid(SaveAsDAT.FileName, 1, SaveAsDAT.FileName.LastIndexOf("\"))
        Dim fileType As String = Mid(SaveAsDAT.FileName, SaveAsDAT.FileName.LastIndexOf(".") + 1, 4)
        Dim fileName As String = Replace(Replace(SaveAsDAT.FileName, filePath & "\", ""), fileType, "")
        Dim subfile As New Hashtable(Primitives.Count)
        Dim subfileToGroup As New Hashtable(Primitives.Count)
        Dim subfileToTriangle As New Hashtable(Primitives.Count)
        Dim vertexIDtoGroupNumber As New Dictionary(Of Integer, Integer)
        Dim lines As Integer = 0

        Dim counter As Integer = 1
        scaleVertices(True)

        If includeMetadata AndAlso myMetadata.mData(1) Like "*.dat" Then
            For Each prim As Primitive In Primitives
                If prim.primitiveName Like "subfile*" Then
                    If Not subfile.Contains(prim.primitiveName) Then
                        subfile.Add(prim.primitiveName, CType(PrimitivesMetadataHMap(prim.primitiveName), Metadata).mData(1))
                        counter += 1
                    End If
                End If
            Next
        Else
            For Each prim As Primitive In Primitives
                If prim.primitiveName Like "subfile*" Then
                    If Not subfile.Contains(prim.primitiveName) Then
                        subfile.Add(prim.primitiveName, "s\" & fileName & "s" & counter & ".dat")
                        counter += 1
                    End If
                End If
            Next
        End If

        If subfile.Count > 0 Then


            If Not My.Computer.FileSystem.DirectoryExists(filePath & "\s") Then
                My.Computer.FileSystem.CreateDirectory(filePath & "\s")
                CDialog.Controls("LblStatus").Text = "Your defined groups are extracted as subfiles in a new folder called 's'."
                Thread.Sleep(5000)
            Else
                Dim subfiles_in_folder As New List(Of String)
                Dim other_subfiles As Boolean = False
                subfiles_in_folder.AddRange(Directory.GetFiles(filePath & "\s"))
                For Each key As String In subfile.Keys
                    If subfiles_in_folder.Contains(filePath & "\" & subfile(key)) Then
                        subfiles_in_folder.Remove(filePath & "\" & subfile(key))
                    End If
                Next
                If subfiles_in_folder.Count > 0 Then
                    Dim files As String = ""
                    If subfiles_in_folder.Count > 1 Then
                        files = subfiles_in_folder(0) & "..." & vbCrLf
                    Else
                        files = subfiles_in_folder(0) & vbCrLf
                    End If
                    Dim result As MsgBoxResult
                    result = MsgBox("They are unknown files in the subfile-folder: " & vbCrLf & files & "Do you want to delete them?", MsgBoxStyle.YesNoCancel + MsgBoxStyle.MsgBoxSetForeground + MsgBoxStyle.Exclamation + MsgBoxStyle.DefaultButton2, "Warning:")
                    If result = MsgBoxResult.Cancel Then GoTo abort
                    If result = MsgBoxResult.Yes Then
                        For Each filen As String In subfiles_in_folder
                            My.Computer.FileSystem.DeleteFile(filen)
                        Next
                    End If
                End If
            End If

            ' Export Subfiles:
            For i As Integer = 0 To Triangles.Count - 1
                Dim tri As Triangle = Triangles(i)
                If tri.groupindex <> -1 Then
                    Dim value As String = subfile(Primitives(PrimitivesHMap(tri.groupindex)).primitiveName)
                    If Not value Is Nothing AndAlso Not subfileToGroup.Contains(value) Then
                        subfileToGroup.Add(value, tri.groupindex)
                        subfileToTriangle.Add(value, i)
                    End If
                End If
            Next

            For Each key As String In subfile.Keys
                exportSubfile(filePath & "\" & subfile(key), subfileToGroup(subfile(key)), subfileToTriangle(subfile(key)))
            Next

        End If

        Using DateiOut As StreamWriter = My.Computer.FileSystem.OpenTextFileWriter(appPath & "tmp.dat", False, System.Text.Encoding.GetEncoding(20127))
            If includeMetadata AndAlso myMetadata.mData(1) Like "*.dat" Then
                DateiOut.WriteLine("0 " & myMetadata.mData(0))
                DateiOut.WriteLine("0 Name: " & myMetadata.mData(1))
                DateiOut.WriteLine("0 Author: " & myMetadata.mData(2) & " [" & myMetadata.mData(3) & "]")
                DateiOut.WriteLine("0 !LDRAW_ORG " & myMetadata.mData(4))
                DateiOut.WriteLine("0 !LICENSE " & myMetadata.mData(5))
                DateiOut.WriteLine()
                If myMetadata.mData(6) <> "" Then
                    Dim alines() As String = Replace(myMetadata.mData(6), "<br>", "µ").Split("µ")
                    For i As Integer = 0 To alines.Length - 1
                        DateiOut.WriteLine("0 !HELP " & alines(i))
                    Next
                    If alines.Length > 0 Then DateiOut.WriteLine()
                End If
                If myMetadata.mData(7) = "BFC CERTIFY CCW" OrElse myMetadata.mData(7) = "BFC CERTIFY CW" OrElse myMetadata.mData(7) = "BFC NOCERTIFY" Then
                    DateiOut.WriteLine("0 " & myMetadata.mData(7))
                Else
                    DateiOut.WriteLine("0 BFC CERTIFY CCW")
                End If
                If myMetadata.mData(8) <> "" Then DateiOut.WriteLine("0 !CATEGORY " & myMetadata.mData(8))
                If myMetadata.mData(9) <> "" Then
                    Dim alines() As String = Replace(myMetadata.mData(9), "<br>", "µ").Split("µ")
                    For i As Integer = 0 To alines.Length - 1
                        DateiOut.WriteLine("0 !KEYWORDS " & alines(i))
                    Next
                    If alines.Length > 0 Then DateiOut.WriteLine()
                End If
                If myMetadata.mData(10) <> "" Then
                    Dim alines() As String = Replace(myMetadata.mData(10), "<br>", "µ").Split("µ")
                    For i As Integer = 0 To alines.Length - 1
                        DateiOut.WriteLine("0 !HISTORY " & alines(i))
                    Next
                    If alines.Length > 0 Then DateiOut.WriteLine()
                End If
                If myMetadata.mData(11) <> "" Then
                    Dim alines() As String = Replace(myMetadata.mData(11), "<br>", "µ").Split("µ")
                    For i As Integer = 0 To alines.Length - 1
                        DateiOut.WriteLine("0 // " & alines(i))
                    Next
                    If alines.Length > 0 Then DateiOut.WriteLine()
                End If
            Else
                DateiOut.WriteLine("0 BFC CERTIFY CCW")
            End If

            If projectMode = myMetadata.recommendedMode Then
                If myMetadata.additionalData <> "" Then
                    Dim alines() As String = Replace(myMetadata.additionalData & "<br>", "<br>", "µ").Split("µ")
                    For i As Integer = 0 To alines.Length - 1
                        DateiOut.WriteLine(alines(i))
                    Next
                End If
                If templateProjectionQuad.Count > 0 Then
                    CDialog.Controls("LblStatus").Text = "(2/4) Calculating projection matrices.."
                    CType(CDialog.Controls("PBStatus"), ProgressBar).Minimum = 0
                    CType(CDialog.Controls("PBStatus"), ProgressBar).Value = 0
                    CType(CDialog.Controls("PBStatus"), ProgressBar).Maximum = templateProjectionQuad.Count - 1
                    For i As Integer = alreadyCalculatedQuads To templateProjectionQuad.Count - 1
                        CType(CDialog.Controls("PBStatus"), ProgressBar).Value = i
                        templateProjectionQuad(i).CalculateMatrix()
                    Next
                    alreadyCalculatedQuads = templateProjectionQuad.Count
                    For i As Integer = 0 To Vertices.Count - 1
                        vertexIDtoGroupNumber.Add(Vertices(i).vertexID, Vertices(i).groupindex)
                    Next
                End If
            End If

            lines = 1
            If Primitives.Count > 0 Then
                CDialog.Controls("LblStatus").Text = "(3/4) Writing Linetype 1.."
                CType(CDialog.Controls("PBStatus"), ProgressBar).Minimum = 0
                CType(CDialog.Controls("PBStatus"), ProgressBar).Value = 0
                CType(CDialog.Controls("PBStatus"), ProgressBar).Maximum = Primitives.Count - 1
            End If
            For i As Integer = 0 To Primitives.Count - 1
                CType(CDialog.Controls("PBStatus"), ProgressBar).Value = i
                Dim prim As Primitive = Primitives(i)
                If Not prim.primitiveName Like "subfile*" Then
                    Dim matrix(3, 3) As Double
                    If templateProjectionQuad.Count > 0 AndAlso projectMode = myMetadata.recommendedMode Then
                        matrix = adjustPrimitiveMatrix(prim.matrix, projectMode, vertexIDtoGroupNumber(prim.centerVertexID))
                    Else
                        matrix = adjustPrimitiveMatrix(prim.matrix, projectMode)
                    End If
                    DateiOut.WriteLine(Replace("1 " & getColourString(prim) & " " & _
                                               matrix(0, 3) & " " & matrix(1, 3) & " " & matrix(2, 3) & " " & _
                                               matrix(0, 0) & " " & matrix(0, 1) & " " & matrix(0, 2) & " " & _
                                               matrix(1, 0) & " " & matrix(1, 1) & " " & matrix(1, 2) & " " & _
                                               matrix(2, 0) & " " & matrix(2, 1) & " " & matrix(2, 2) & _
                                               " " & prim.primitiveName, comma, "."))
                    lines += 1
                Else
                    Dim matrix(3, 3) As Double
                    If templateProjectionQuad.Count > 0 Then
                        matrix = adjustSubfileMatrix(prim.matrix, projectMode, vertexIDtoGroupNumber(prim.centerVertexID))
                    Else
                        matrix = adjustSubfileMatrix(prim.matrix, projectMode)
                    End If
                    DateiOut.WriteLine(Replace("1 " & getColourString(prim) & " " & _
                                                       matrix(0, 3) & " " & matrix(1, 3) & " " & matrix(2, 3) & " " & _
                                                       matrix(0, 0) & " " & matrix(0, 1) & " " & matrix(0, 2) & " " & _
                                                       matrix(1, 0) & " " & matrix(1, 1) & " " & matrix(1, 2) & " " & _
                                                       matrix(2, 0) & " " & matrix(2, 1) & " " & matrix(2, 2) & _
                                                       " " & subfile(prim.primitiveName), comma, "."))
                    lines += 1
                End If
            Next
            If Triangles.Count > 0 Then
                If projectMode = myMetadata.recommendedMode Then
                    CDialog.Controls("LblStatus").Text = "(4/4) Writing Linetype 3.."
                    CType(CDialog.Controls("PBStatus"), ProgressBar).Minimum = 0
                    CType(CDialog.Controls("PBStatus"), ProgressBar).Value = 0
                    CType(CDialog.Controls("PBStatus"), ProgressBar).Maximum = Triangles.Count - 1
                    lines += adjustAndSaveTriangles(projectMode, DateiOut)
                    CType(CDialog.Controls("PBStatus"), ProgressBar).Value = Triangles.Count - 1
                Else
                    CDialog.Controls("LblStatus").Text = "(4/4) Writing Linetype 3.."
                    CType(CDialog.Controls("PBStatus"), ProgressBar).Minimum = 0
                    CType(CDialog.Controls("PBStatus"), ProgressBar).Value = 0
                    CType(CDialog.Controls("PBStatus"), ProgressBar).Maximum = Triangles.Count - 1
                    For i As Integer = 0 To Triangles.Count - 1
                        CType(CDialog.Controls("PBStatus"), ProgressBar).Value = i
                        Dim tri As Triangle = Triangles(i)
                        If tri.groupindex = -1 Then
                            tri.normalize()
                            tri.checkWinding()
                            Select Case projectMode
                                Case 0 : DateiOut.WriteLine(Replace("3 " & getColourString(tri) & " 0 " & tri.vertexA.Y & " " & tri.vertexA.X & " 0 " _
                                                               & tri.vertexB.Y & " " & tri.vertexB.X & " 0 " _
                                                               & tri.vertexC.Y & " " & tri.vertexC.X & " ", comma, "."))
                                Case 1 : DateiOut.WriteLine(Replace("3 " & getColourString(tri) & " " & -tri.vertexA.X & " 0 " & -tri.vertexA.Y & " " _
                                                           & -tri.vertexB.X & " 0 " & -tri.vertexB.Y & " " _
                                                           & -tri.vertexC.X & " 0 " & -tri.vertexC.Y & " ", comma, "."))
                                Case 2 : DateiOut.WriteLine(Replace("3 " & getColourString(tri) & " " & -tri.vertexA.X & " " & tri.vertexA.Y & " 0 " _
                                                           & -tri.vertexB.X & " " & tri.vertexB.Y & " 0 " _
                                                           & -tri.vertexC.X & " " & tri.vertexC.Y & " 0", comma, "."))
                                Case 3 : DateiOut.WriteLine(Replace("3 " & getColourString(tri) & " 0 " & tri.vertexA.Y & " " & -tri.vertexA.X & " 0 " _
                                                               & tri.vertexB.Y & " " & -tri.vertexB.X & " 0 " _
                                                               & tri.vertexC.Y & " " & -tri.vertexC.X & " ", comma, "."))
                                Case 4 : DateiOut.WriteLine(Replace("3 " & getColourString(tri) & " " & -tri.vertexA.X & " 0 " & tri.vertexA.Y & " " _
                                                            & -tri.vertexB.X & " 0 " & tri.vertexB.Y & " " _
                                                            & -tri.vertexC.X & " 0 " & tri.vertexC.Y & " ", comma, "."))
                                Case 5 : DateiOut.WriteLine(Replace("3 " & getColourString(tri) & " " & tri.vertexA.X & " " & tri.vertexA.Y & " 0 " _
                                                           & tri.vertexB.X & " " & tri.vertexB.Y & " 0 " _
                                                           & tri.vertexC.X & " " & tri.vertexC.Y & " 0", comma, "."))
                            End Select
                            lines += 1
                        End If
                    Next
                End If
            End If
        End Using


        rectify(lines, SaveAsDAT.FileName, myMetadata)

abort:
        scaleVertices(False)
        If templateProjectionQuad.Count > 0 Then
            clearSelection()
        End If
        MenuStrip1.Enabled = True
        Me.ToolStrip1.Enabled = True
        Me.ColourToolStrip.Enabled = True
        GBImage.Enabled = True
        GBSnap.Enabled = True
        addHistoryII(withDataFrom.All)
        isLoading = False
        CDialog.Controls("Cancel_Button").Enabled = True
        CDialog.Close()
        Me.Refresh()
    End Sub

    Private Function getColourString(ByVal tri As Triangle) As String
        If tri.myColourNumber = -1 Then
            Return "0x2" & tri.myColour.R.ToString("X2") & tri.myColour.G.ToString("X2") & tri.myColour.B.ToString("X2")
        End If
        Return tri.myColourNumber
    End Function

    Private Function getColourString(ByVal prim As Primitive) As String
        If prim.myColourNumber = -1 Then
            Return "0x2" & prim.myColour.R.ToString("X2") & prim.myColour.G.ToString("X2") & prim.myColour.B.ToString("X2")
        End If
        Return prim.myColourNumber
    End Function

    Private Sub clearSelection()
        For i As Integer = 0 To SelectedTriangles.Count - 1
            SelectedTriangles(i).selected = False
        Next
        For i As Integer = 0 To SelectedVertices.Count - 1
            SelectedVertices(i).selected = False
        Next
        SelectedTriangles.Clear()
        SelectedVertices.Clear()
    End Sub

    Private Sub selectGroupDirect(ByVal group As Integer)
        clearSelection()
        For tri As Integer = 0 To Triangles.Count - 1
            If Triangles(tri).groupindex = group Then
                SelectedTriangles.Add(Triangles(tri)) : Triangles(tri).selected = True
                If Not SelectedVertices.Contains(Triangles(tri).vertexA) Then SelectedVertices.Add(Triangles(tri).vertexA)
                If Not SelectedVertices.Contains(Triangles(tri).vertexB) Then SelectedVertices.Add(Triangles(tri).vertexB)
                If Not SelectedVertices.Contains(Triangles(tri).vertexC) Then SelectedVertices.Add(Triangles(tri).vertexC)
            End If
        Next
    End Sub

    Private Sub exportSubfile(ByVal fileName As String, ByVal group As Integer, ByRef tri As Integer)

        Dim subfileMetadata As Metadata = PrimitivesMetadataHMap(Primitives(PrimitivesHMap(group)).primitiveName)

        clearSelection()

        SelectedTriangles.Add(Triangles(tri)) : Triangles(tri).selected = True
        SelectedVertices.Add(Triangles(tri).vertexA)
        SelectedVertices.Add(Triangles(tri).vertexB)
        SelectedVertices.Add(Triangles(tri).vertexC)

        selectPrimitiveGroup(group)

        Dim tempTris As New List(Of Triangle)

        For i As Integer = 0 To SelectedTriangles.Count - 1
            Dim v(2) As Vertex
            v(0) = SelectedTriangles(i).vertexA
            v(1) = SelectedTriangles(i).vertexB
            v(2) = SelectedTriangles(i).vertexC
            tempTris.Add(New Triangle(New Vertex(v(0).X, v(0).Y, False, False), New Vertex(v(1).X, v(1).Y, False, False), New Vertex(v(2).X, v(2).Y, False, False), False) With {.myColourNumber = SelectedTriangles(i).myColourNumber})
            SelectedTriangles(i).selected = False
        Next
        Dim prim As Primitive = Primitives(PrimitivesHMap(group))
        Dim transmatrix(,) As Double = CType(prim.matrix.Clone, Double(,))

        transmatrix(0, 3) /= 1000
        transmatrix(1, 3) /= 1000

        transmatrix = Inverse(transmatrix)

        Using DateiOut As StreamWriter = My.Computer.FileSystem.OpenTextFileWriter(appPath & "tmp.dat", False, System.Text.Encoding.GetEncoding(20127))
            If includeMetadata AndAlso myMetadata.mData(1) Like "*.dat" Then
                DateiOut.WriteLine("0 " & subfileMetadata.mData(0))
                DateiOut.WriteLine("0 Name: " & subfileMetadata.mData(1))
                DateiOut.WriteLine("0 Author: " & subfileMetadata.mData(2) & " [" & subfileMetadata.mData(3) & "]")
                DateiOut.WriteLine("0 !LDRAW_ORG " & subfileMetadata.mData(4))
                DateiOut.WriteLine("0 !LICENSE " & subfileMetadata.mData(5))
                DateiOut.WriteLine()
                If subfileMetadata.mData(6) <> "" Then
                    Dim alines() As String = Replace(subfileMetadata.mData(6), "<br>", "µ").Split("µ")
                    For i As Integer = 0 To alines.Length - 1
                        DateiOut.WriteLine("0 !HELP " & alines(i))
                    Next
                    If alines.Length > 0 Then DateiOut.WriteLine()
                End If
                If subfileMetadata.mData(7) = "BFC CERTIFY CCW" OrElse subfileMetadata.mData(7) = "BFC CERTIFY CW" OrElse subfileMetadata.mData(7) = "BFC NOCERTIFY" Then
                    DateiOut.WriteLine("0 " & subfileMetadata.mData(7))
                Else
                    DateiOut.WriteLine("0 BFC CERTIFY CCW")
                End If
                If subfileMetadata.mData(8) <> "" Then DateiOut.WriteLine("0 !CATEGORY " & subfileMetadata.mData(8))
                If subfileMetadata.mData(9) <> "" Then
                    Dim alines() As String = Replace(subfileMetadata.mData(9), "<br>", "µ").Split("µ")
                    For i As Integer = 0 To alines.Length - 1
                        DateiOut.WriteLine("0 !KEYWORDS " & alines(i))
                    Next
                    If alines.Length > 0 Then DateiOut.WriteLine()
                End If
                If subfileMetadata.mData(10) <> "" Then
                    Dim alines() As String = Replace(subfileMetadata.mData(10), "<br>", "µ").Split("µ")
                    For i As Integer = 0 To alines.Length - 1
                        DateiOut.WriteLine("0 !HISTORY " & alines(i))
                    Next
                    If alines.Length > 0 Then DateiOut.WriteLine()
                End If
                If subfileMetadata.mData(11) <> "" Then
                    Dim alines() As String = Replace(subfileMetadata.mData(11), "<br>", "µ").Split("µ")
                    For i As Integer = 0 To alines.Length - 1
                        DateiOut.WriteLine("0 // " & alines(i))
                    Next
                    If alines.Length > 0 Then DateiOut.WriteLine()
                End If
            Else
                DateiOut.WriteLine("0 BFC CERTIFY CCW")
            End If

            For i As Integer = 0 To tempTris.Count - 1
                Dim tri2 As Triangle = tempTris(i)
                Dim tx, ty As Single
                tx = transmatrix(0, 0) * tri2.vertexA.X + transmatrix(0, 1) * tri2.vertexA.Y + transmatrix(0, 3)
                ty = transmatrix(1, 0) * tri2.vertexA.X + transmatrix(1, 1) * tri2.vertexA.Y + transmatrix(1, 3)
                tri2.vertexA.X = tx
                tri2.vertexA.Y = ty
                tx = transmatrix(0, 0) * tri2.vertexB.X + transmatrix(0, 1) * tri2.vertexB.Y + transmatrix(0, 3)
                ty = transmatrix(1, 0) * tri2.vertexB.X + transmatrix(1, 1) * tri2.vertexB.Y + transmatrix(1, 3)
                tri2.vertexB.X = tx
                tri2.vertexB.Y = ty
                tx = transmatrix(0, 0) * tri2.vertexC.X + transmatrix(0, 1) * tri2.vertexC.Y + transmatrix(0, 3)
                ty = transmatrix(1, 0) * tri2.vertexC.X + transmatrix(1, 1) * tri2.vertexC.Y + transmatrix(1, 3)
                tri2.vertexC.X = tx
                tri2.vertexC.Y = ty
                tri2.normalize()
                tri2.checkWinding()
                DateiOut.WriteLine(Replace("3 " & getColourString(tri2) & " " & tri2.vertexA.X & " " & tri2.vertexA.Y & " 0 " _
                                                   & tri2.vertexB.X & " " & tri2.vertexB.Y & " 0 " _
                                                   & tri2.vertexC.X & " " & tri2.vertexC.Y & " 0", comma, "."))
            Next i
        End Using


        rectify(tempTris.Count, fileName, subfileMetadata)

        tempTris.Clear()
        SelectedTriangles.Clear()
        SelectedVertices.Clear()
    End Sub

    Private Sub rectify(ByVal lines As Integer, ByVal filename As String, ByRef meta As Metadata)
        Dim oProcess As Process

        Using DateiOut As StreamWriter = My.Computer.FileSystem.OpenTextFileWriter(appPath & "Rectifier.bat", False, System.Text.Encoding.GetEncoding(20127))
            DateiOut.WriteLine("cd " & appPath)
            DateiOut.WriteLine("Rectifier tmp.dat tmpr.dat")
        End Using
        Dim fcount As Integer = 0
        Dim fcount_max As Integer = 0

        Dim sb As New StringBuilder

newRectify:
        oProcess = System.Diagnostics.Process.Start(appPath & "Rectifier.bat")
        oProcess.StartInfo.WindowStyle = ProcessWindowStyle.Maximized
        oProcess.EnableRaisingEvents = True
        Do
            Thread.Sleep(10)
        Loop Until My.Computer.FileSystem.FileExists(appPath & "tmpr.dat")
        Do
            Thread.Sleep(10)
        Loop Until My.Computer.FileSystem.GetFileInfo(appPath & "tmpr.dat").LastWriteTime.Ticks + 1000000 < Now.Ticks
        Do
            Thread.Sleep(10)
            SendKeys.SendWait("{ENTER}")
        Loop Until oProcess.HasExited

        If My.Computer.FileSystem.GetFileInfo(appPath & "tmpr.dat").Length < 25 AndAlso fcount = 0 Then
            Using DateiIn As StreamReader = My.Computer.FileSystem.OpenTextFileReader(appPath & "tmp.dat", System.Text.Encoding.GetEncoding(20127))

                Do
                    Dim lncount As Integer = 0
                    Using DateiOut As StreamWriter = My.Computer.FileSystem.OpenTextFileWriter(appPath & "tmp" & fcount & ".dat", False, System.Text.Encoding.GetEncoding(20127))
                        Do
                            DateiOut.WriteLine(DateiIn.ReadLine)
                            lncount += 1
                        Loop Until DateiIn.EndOfStream OrElse lncount = 3000
                    End Using
                    fcount += 1
                Loop Until DateiIn.EndOfStream
            End Using
            fcount_max = fcount - 1
            Using DateiOut As StreamWriter = My.Computer.FileSystem.OpenTextFileWriter(appPath & "Rectifier.bat", False, System.Text.Encoding.GetEncoding(20127))
                DateiOut.WriteLine("cd " & appPath)
                DateiOut.WriteLine("Rectifier tmp" & fcount_max + 1 - fcount & ".dat tmpr" & fcount_max + 1 - fcount & ".dat")
            End Using
            GoTo newRectify
        End If
        If My.Computer.FileSystem.FileExists(appPath & "tmpr" & fcount_max + 1 - fcount & ".dat") Then
            Using DateiIn As StreamReader = My.Computer.FileSystem.OpenTextFileReader(appPath & "tmpr" & fcount_max + 1 - fcount & ".dat", System.Text.Encoding.GetEncoding(20127))
                Do
                    sb.Append(DateiIn.ReadLine)
                    sb.AppendLine()
                Loop Until DateiIn.EndOfStream
            End Using
            My.Computer.FileSystem.DeleteFile(appPath & "tmp" & fcount_max + 1 - fcount & ".dat")
            My.Computer.FileSystem.DeleteFile(appPath & "tmpr" & fcount_max + 1 - fcount & ".dat")
            fcount -= 1
            If My.Computer.FileSystem.FileExists(appPath & "tmp" & fcount_max + 1 - fcount & ".dat") Then
                Using DateiOut As StreamWriter = My.Computer.FileSystem.OpenTextFileWriter(appPath & "Rectifier.bat", False, System.Text.Encoding.GetEncoding(20127))
                    DateiOut.WriteLine("cd " & appPath)
                    DateiOut.WriteLine("Rectifier tmp" & fcount_max + 1 - fcount & ".dat tmpr" & fcount_max + 1 - fcount & ".dat")
                End Using
                GoTo newRectify
            Else
                Using DateiOut As StreamWriter = My.Computer.FileSystem.OpenTextFileWriter(appPath & "tmpr.dat", True, System.Text.Encoding.GetEncoding(20127))
                    DateiOut.Write(sb.ToString)
                End Using
            End If
        End If
        If includeMetadata AndAlso myMetadata.mData(1) Like "*.dat" Then
            If meta.isMainMetadata Then
                Try
                    filename = Mid(filename, 1, filename.LastIndexOf("\") + 1)
                    My.Computer.FileSystem.CopyFile(appPath & "tmpr.dat", filename + meta.mData(1), True)
                Catch
                    My.Computer.FileSystem.CopyFile(appPath & "tmpr.dat", filename, True)
                End Try
            Else
                Try
                    filename = Mid(filename, 1, filename.LastIndexOf("\") - 1)
                    My.Computer.FileSystem.CopyFile(appPath & "tmpr.dat", filename + meta.mData(1), True)
                Catch
                    My.Computer.FileSystem.CopyFile(appPath & "tmpr.dat", filename, True)
                End Try
            End If
        Else
            My.Computer.FileSystem.CopyFile(appPath & "tmpr.dat", filename, True)
        End If
        My.Computer.FileSystem.DeleteFile(appPath & "tmp.dat")
        My.Computer.FileSystem.DeleteFile(appPath & "tmpr.dat")
    End Sub


    Private Sub NewPatternToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles NewPatternToolStripMenuItem.Click
        If unsavedChanges AndAlso ShowAllWarningsToolStripMenuItem.Checked Then
            Dim result As MsgBoxResult = MsgBox("You have unsaved changes - would you like to save them first?", MsgBoxStyle.YesNoCancel + MsgBoxStyle.MsgBoxSetForeground + MsgBoxStyle.Information, "Question:")
            If result = MsgBoxResult.Yes Then SaveToolStripMenuItem.PerformClick()
            If result = MsgBoxResult.Cancel Then Exit Sub
        End If
        isLoading = True
        addHistoryI(withDataFrom.All)
        newPattern()
        addHistoryII(withDataFrom.All)
        isLoading = False
        unsavedChanges = False
        Me.Refresh()
    End Sub

    Private Sub newPattern()
        isLoading = True
        alreadyCalculatedQuads = 0
        MetadataDialog.CBInclude.Checked = False
        includeMetadata = False
        backgroundPicture.Dispose()
        backgroundPicture = My.Resources.temp
        SelectedTriangles.Clear()
        SelectedVertices.Clear()
        Vertices.Clear()
        Triangles.Clear()
        PrimitivesHMap.Clear()
        Primitives.Clear()
        PrimitivesMetadataHMap.Clear()
        myMetadata = New MainForm.Metadata() With {.isMainMetadata = True}
        For r As Integer = 0 To 3
            For c As Integer = 0 To 3
                myMetadata.matrix(r, c) = 0
                If c = r Then myMetadata.matrix(r, c) = 1
            Next
        Next
        templateShape.Clear()
        templateProjectionQuad.Clear()
        templateText.Clear()
        vertexIDglobal = 0
        Me.TBImage.Text = ""
        Me.GBVertex.Visible = False
        SaveAs.FileName = ""
        SaveAsDAT.FileName = ""
        triangleIDglobal = 0
        primitiveIDglobal = 0
        Me.NUDMoveSnap.Value = 100
        Me.NUDScaleSnap.Value = 0.01
        Me.NUDRotateSnap.Value = 15
        Me.NUDScale.Value = 1
        Me.NUDoffsetX.Value = 0
        Me.NUDoffsetY.Value = 0
        Me.NUDGrid.Value = 1
        isLoading = False
        Me.Refresh()
        GC.Collect()
    End Sub

#End Region

#Region "Primitives"

#Region "Primitive Datastructure"
    Dim primitiveMode As Byte
    Dim primitiveObject As Byte
    Dim primitiveName As String

    Dim primitiveCenter As New Vertex(0, 0, False, False)
    Dim primitiveHeight As Single
    Dim primitiveWidth As Single
    Dim primitiveBordersize As Single
#End Region

#Region "Basic Functions"
    Private Sub startPrimitiveMode()
        primitiveMode = 1
        Me.ToolStrip1.Visible = False
        Me.BtnAbort.Visible = True
        TriangleModeToolStripMenuItem.PerformClick()
    End Sub

    Private Sub abortPrimitiveMode()
        If SelectedTriangles.Count > 0 AndAlso SelectedVertices.Count > 0 Then
            BtnAddToGroup.Enabled = True
            BtnUngroup.Enabled = False
        Else
            BtnAddToGroup.Enabled = False
        End If
        primitiveMode = 0
        Me.ToolStrip1.Visible = True
        Me.BtnAbort.Visible = False
        Me.MenuStrip1.Visible = True
        Me.BtnAbort.Text = Replace(BtnAbort.Text, "OK", "ABORT")
        Me.BtnAbort.BackColor = Color.Red
    End Sub
#End Region

#Region "LDraw Primitives"

    Private Sub Disc44_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Disc44.Click
        primitiveObject = 255
        primitiveName = sender.Text
        startPrimitiveMode()
    End Sub

    Private Sub Disc34_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Disc34.Click
        primitiveObject = 255
        primitiveName = sender.Text
        startPrimitiveMode()
    End Sub

    Private Sub Disc58_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Disc58.Click
        primitiveObject = 255
        primitiveName = sender.Text
        startPrimitiveMode()
    End Sub

    Private Sub Disc24_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Disc24.Click
        primitiveObject = 255
        primitiveName = sender.Text
        startPrimitiveMode()
    End Sub

    Private Sub Disc716_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Disc716.Click
        primitiveObject = 255
        primitiveName = sender.Text
        startPrimitiveMode()
    End Sub

    Private Sub Disc38_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Disc38.Click
        primitiveObject = 255
        primitiveName = sender.Text
        startPrimitiveMode()
    End Sub

    Private Sub Disc516_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Disc516.Click
        primitiveObject = 255
        primitiveName = sender.Text
        startPrimitiveMode()
    End Sub

    Private Sub Disc14_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Disc14.Click
        primitiveObject = 255
        primitiveName = sender.Text
        startPrimitiveMode()
    End Sub

    Private Sub Disc316_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Disc316.Click
        primitiveObject = 255
        primitiveName = sender.Text
        startPrimitiveMode()
    End Sub

    Private Sub Disc18_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Disc18.Click
        primitiveObject = 255
        primitiveName = sender.Text
        startPrimitiveMode()
    End Sub

    Private Sub Disc116_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Disc116.Click
        primitiveObject = 255
        primitiveName = sender.Text
        startPrimitiveMode()
    End Sub

    Private Sub Disc4448_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Disc4448.Click
        primitiveObject = 255
        primitiveName = sender.Text
        startPrimitiveMode()
    End Sub

    Private Sub Disc2448_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Disc2448.Click
        primitiveObject = 255
        primitiveName = sender.Text
        startPrimitiveMode()
    End Sub

    Private Sub Disc1448_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Disc1448.Click
        primitiveObject = 255
        primitiveName = sender.Text
        startPrimitiveMode()
    End Sub

    Private Sub Disc31648_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Disc31648.Click
        primitiveObject = 255
        primitiveName = sender.Text
        startPrimitiveMode()
    End Sub

    Private Sub Disc1648_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Disc1648.Click
        primitiveObject = 255
        primitiveName = sender.Text
        startPrimitiveMode()
    End Sub

    Private Sub Disc1848_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Disc1848.Click
        primitiveObject = 255
        primitiveName = sender.Text
        startPrimitiveMode()
    End Sub

    Private Sub Disc54848_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Disc54848.Click
        primitiveObject = 255
        primitiveName = sender.Text
        startPrimitiveMode()
    End Sub

    Private Sub Disc11248_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Disc11248.Click
        primitiveObject = 255
        primitiveName = sender.Text
        startPrimitiveMode()
    End Sub

    Private Sub Ndis44_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Ndis44.Click
        primitiveObject = 255
        primitiveName = sender.Text
        startPrimitiveMode()
    End Sub

    Private Sub Ndis78_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Ndis78.Click
        primitiveObject = 255
        primitiveName = sender.Text
        startPrimitiveMode()
    End Sub

    Private Sub Ndis34_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Ndis34.Click
        primitiveObject = 255
        primitiveName = sender.Text
        startPrimitiveMode()
    End Sub

    Private Sub Ndis24_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Ndis24.Click
        primitiveObject = 255
        primitiveName = sender.Text
        startPrimitiveMode()
    End Sub

    Private Sub Ndis716_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Ndis716.Click
        primitiveObject = 255
        primitiveName = sender.Text
        startPrimitiveMode()
    End Sub

    Private Sub Ndis38_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Ndis38.Click
        primitiveObject = 255
        primitiveName = sender.Text
        startPrimitiveMode()
    End Sub

    Private Sub Ndis14_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Ndis14.Click
        primitiveObject = 255
        primitiveName = sender.Text
        startPrimitiveMode()
    End Sub

    Private Sub Ndis316_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Ndis316.Click
        primitiveObject = 255
        primitiveName = sender.Text
        startPrimitiveMode()
    End Sub

    Private Sub Ndis18_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Ndis18.Click
        primitiveObject = 255
        primitiveName = sender.Text
        startPrimitiveMode()
    End Sub

    Private Sub Ndis116_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Ndis116.Click
        primitiveObject = 255
        primitiveName = sender.Text
        startPrimitiveMode()
    End Sub

    Private Sub Ndis4448_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Ndis4448.Click
        primitiveObject = 255
        primitiveName = sender.Text
        startPrimitiveMode()
    End Sub

    Private Sub Ndis2448_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Ndis2448.Click
        primitiveObject = 255
        primitiveName = sender.Text
        startPrimitiveMode()
    End Sub

    Private Sub Ndis1448_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Ndis1448.Click
        primitiveObject = 255
        primitiveName = sender.Text
        startPrimitiveMode()
    End Sub

    Private Sub Ndis31648_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Ndis31648.Click
        primitiveObject = 255
        primitiveName = sender.Text
        startPrimitiveMode()
    End Sub

    Private Sub Ndis1648_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Ndis1648.Click
        primitiveObject = 255
        primitiveName = sender.Text
        startPrimitiveMode()
    End Sub

    Private Sub Ndis1848_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Ndis1848.Click
        primitiveObject = 255
        primitiveName = sender.Text
        startPrimitiveMode()
    End Sub

    Private Sub Ndis54848_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Ndis54848.Click
        primitiveObject = 255
        primitiveName = sender.Text
        startPrimitiveMode()
    End Sub

    Private Sub Ndis11248_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Ndis11248.Click
        primitiveObject = 255
        primitiveName = sender.Text
        startPrimitiveMode()
    End Sub

    Private Sub Seg716_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Seg716.Click
        primitiveObject = 255
        primitiveName = sender.Text
        startPrimitiveMode()
    End Sub

    Private Sub Seg38_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Seg38.Click
        primitiveObject = 255
        primitiveName = sender.Text
        startPrimitiveMode()
    End Sub

    Private Sub Seg516_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Seg516.Click
        primitiveObject = 255
        primitiveName = sender.Text
        startPrimitiveMode()
    End Sub

    Private Sub Seg14_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Seg14.Click
        primitiveObject = 255
        primitiveName = sender.Text
        startPrimitiveMode()
    End Sub

    Private Sub Seg316_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Seg316.Click
        primitiveObject = 255
        primitiveName = sender.Text
        startPrimitiveMode()
    End Sub

    Private Sub Seg18_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Seg18.Click
        primitiveObject = 255
        primitiveName = sender.Text
        startPrimitiveMode()
    End Sub

    Private Sub Seg1448_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Seg1448.Click
        primitiveObject = 255
        primitiveName = sender.Text
        startPrimitiveMode()
    End Sub

    Private Sub Seg52448_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Seg52448.Click
        primitiveObject = 255
        primitiveName = sender.Text
        startPrimitiveMode()
    End Sub

    Private Sub Seg1848_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Seg1848.Click
        primitiveObject = 255
        primitiveName = sender.Text
        startPrimitiveMode()
    End Sub

    Private Sub Seg11248_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Seg11248.Click
        primitiveObject = 255
        primitiveName = sender.Text
        startPrimitiveMode()
    End Sub

    Private Sub Aring44_Click(ByVal sender As System.Object, ByVal e As System.EventArgs)
        primitiveObject = 255
        primitiveName = sender.Text
        startPrimitiveMode()
    End Sub

    Private Sub Aring4448_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Aring4448.Click
        primitiveObject = 255
        primitiveName = sender.Text
        startPrimitiveMode()
    End Sub

    Private Sub CircularRingSegmentToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CircularRingSegmentToolStripMenuItem.Click
        RingDialog.hiRes = False
        If RingDialog.ShowDialog() = Windows.Forms.DialogResult.OK Then
            primitiveObject = 255
            primitiveName = RingDialog.pname
            startPrimitiveMode()
        End If
    End Sub

    Private Sub CircularRingSegment48ToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CircularRingSegment48ToolStripMenuItem.Click
        RingDialog.hiRes = True
        If RingDialog.ShowDialog() = Windows.Forms.DialogResult.OK Then
            primitiveObject = 255
            primitiveName = RingDialog.pname
            startPrimitiveMode()
        End If
    End Sub

#End Region

#Region "Event Handler"
    Private Sub BtnAbort_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BtnAbort.Click
        abortPrimitiveMode()
        adjustmode = False
        doAdjust = False
        Me.Refresh()
    End Sub

    Private Sub SolidTriangleToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SolidTriangleToolStripMenuItem.Click
        primitiveObject = 0
        startPrimitiveMode()
    End Sub

    Private Sub TriangleWithFrameToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles TriangleWithFrameToolStripMenuItem.Click
        primitiveObject = 1
        startPrimitiveMode()
    End Sub

    Private Sub HollowTriangleToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles HollowTriangleToolStripMenuItem.Click
        primitiveObject = 2
        startPrimitiveMode()
    End Sub

    Private Sub SolidRectangleToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SolidRectangleToolStripMenuItem.Click
        primitiveObject = 3
        startPrimitiveMode()
    End Sub

    Private Sub RectangleWithFrameToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RectangleWithFrameToolStripMenuItem.Click
        primitiveObject = 4
        startPrimitiveMode()
    End Sub

    Private Sub HollowRectangleToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles HollowRectangleToolStripMenuItem.Click
        primitiveObject = 5
        startPrimitiveMode()
    End Sub

    Private Sub SolidCircleToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SolidCircleToolStripMenuItem.Click
        primitiveObject = 6
        startPrimitiveMode()
    End Sub

    Private Sub CircleWithFrameToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CircleWithFrameToolStripMenuItem.Click
        primitiveObject = 7
        startPrimitiveMode()
    End Sub

    Private Sub HollowCircleToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles HollowCircleToolStripMenuItem.Click
        primitiveObject = 8
        startPrimitiveMode()
    End Sub

    Private Sub SolidOvalToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SolidOvalToolStripMenuItem.Click
        primitiveObject = 9
        startPrimitiveMode()
    End Sub

    Private Sub OvalWithFrameToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles OvalWithFrameToolStripMenuItem.Click
        primitiveObject = 10
        startPrimitiveMode()
    End Sub

    Private Sub HollowOvalToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles HollowOvalToolStripMenuItem.Click
        primitiveObject = 11
        startPrimitiveMode()
    End Sub

    Private Sub SolidPolygonToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs)

        startPrimitiveMode()
    End Sub

    Private Sub PolygonWithFrameToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs)

        startPrimitiveMode()
    End Sub

    Private Sub HollowPolygonToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs)

        startPrimitiveMode()
    End Sub

    Private Sub ChainToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ChainToolStripMenuItem.Click
        If Vertices.Count > 1 Then
            startPrimitiveMode()
            primitiveMode = 255
        End If
    End Sub

    Private Sub CircleSegments_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CircleSegments.TextChanged
        Try
            segments_circle = CType(CircleSegments.Text, Integer)
            If segments_circle < 3 Then segments_circle = 3
        Catch ex As Exception
            CircleSegments.Text = segments_circle
        End Try
    End Sub

    Private Sub OvalSegments_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles OvalSegments.TextChanged
        Try
            segments_oval = CType(OvalSegments.Text, Integer)
            If segments_oval < 3 Then segments_oval = 3
        Catch ex As Exception
            OvalSegments.Text = segments_oval
        End Try
    End Sub
#End Region

#End Region

#Region "Tutorial"

    Dim turorialRuns As Boolean = False
    Private Sub CreateABasicPolygonToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ChangeViewportToolStripMenuItem.Click
        If Not Me.BackgroundHowTo.IsBusy AndAlso (Me.Vertices.Count = 0 OrElse MsgBox("Do you really want to start this lesson and discard the current pattern?", MsgBoxStyle.YesNo, "Warning:") = Windows.Forms.DialogResult.Yes) Then
            turorialRuns = True
            AbortCurrentLessonToolStripMenuItem.Enabled = True
            If MsgBox("Please don't move the mouse or hit on your keyboard (execpt of the ENTER key) while watching this how-to." & vbCrLf & vbCrLf & "You can abort the lesson by pressing Ctrl+Q at any time." & vbCrLf & "Press [ENTER] to continue.", MsgBoxStyle.OkOnly, "Info:") = MsgBoxResult.Ok Then
                Me.Activate()
                Me.WindowState = FormWindowState.Maximized
                Me.BringToFront()
                Control.CheckForIllegalCrossThreadCalls = False
                Me.BackgroundHowTo.RunWorkerAsync(0)
            End If
        End If
    End Sub

    Private Sub CreateApolygonToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CreateApolygonToolStripMenuItem.Click
        If Not Me.BackgroundHowTo.IsBusy AndAlso (Me.Vertices.Count = 0 OrElse MsgBox("Do you really want to start this lesson and discard the current pattern?", MsgBoxStyle.YesNo, "Warning:") = Windows.Forms.DialogResult.Yes) Then
            turorialRuns = True
            AbortCurrentLessonToolStripMenuItem.Enabled = True
            If MsgBox("Please don't move the mouse or hit on your keyboard (execpt of the ENTER key) while watching this how-to." & vbCrLf & vbCrLf & "You can abort the lesson by pressing Ctrl+Q at any time." & vbCrLf & "Press [ENTER] to continue.", MsgBoxStyle.OkOnly, "Info:") = MsgBoxResult.Ok Then
                Me.Activate()
                Me.WindowState = FormWindowState.Maximized
                Me.BringToFront()
                Control.CheckForIllegalCrossThreadCalls = False
                Me.BackgroundHowTo.RunWorkerAsync(1)
            End If
        End If
    End Sub

    Private Sub BackgroundHowTo_DoWork( _
    ByVal sender As Object, ByVal e As DoWorkEventArgs) _
    Handles BackgroundHowTo.DoWork

        ' Do not access the form's BackgroundWorker reference directly.
        ' Instead, use the reference provided by the sender parameter.
        Dim bw As BackgroundWorker = CType(sender, BackgroundWorker)

        ' Extract the argument.
        Dim TutorialNumber As Integer = e.Argument

        ' Start the time-consuming operation.
        e.Result = BackgroundHowToDo(bw, TutorialNumber)

        ' If the operation was canceled by the user, 
        ' set the DoWorkEventArgs.Cancel property to true.
        If bw.CancellationPending Then
            e.Cancel = True
        End If

    End Sub

    Private Sub BackgroundHowTo_RunWorkerCompleted( _
    ByVal sender As Object, ByVal e As RunWorkerCompletedEventArgs) _
    Handles BackgroundHowTo.RunWorkerCompleted

        If e.Cancelled Then
            ' The user canceled the operation.
        ElseIf Not (e.Error Is Nothing) Then
            ' There was an error during the operation.
        Else
            ' The operation completed normally.
        End If
    End Sub

    Private Function BackgroundHowToDo(ByVal bw As BackgroundWorker, ByVal TutorialNumber As Integer) As String
        resetGUI()
        Dim mouseButtonL As String = "left mouse button"
        Dim mouseButtonR As String = "right mouse button"
        Dim upperL As String = "Left"
        Dim upperR As String = "Right"
        If My.Computer.Mouse.ButtonsSwapped Then
            mouseButtonL = "right mouse button"
            mouseButtonR = "left mouse button"
            upperL = "Right"
            upperR = "Left"
        End If

        If TutorialNumber = 0 Then
            Do
                If Not turorialRuns Then Exit Do
                While MsgBox("1.) Magnify the viewport: " & vbCrLf & "Use the mousewheel or the Page-Up- and Page-Down-keys to modify the zoom level of the viewport." & vbCrLf & "The zoom level ranges from 0.1% to 1000% with a maximum precision of.." & vbCrLf & ".. 0.001 LDU for moving objects .." & vbCrLf & ".. and more than 0.0000000001 LDU for rotating and scaling objects." & vbCrLf & vbCrLf & "Press [ENTER] to continue and watch this action.", MsgBoxStyle.OkOnly + MsgBoxStyle.MsgBoxSetForeground, "Info:") <> MsgBoxResult.Ok : End While
                moveMouseAbsoluteRightClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2, System.Windows.Forms.Screen.PrimaryScreen.Bounds.Height / 2)
                For i As Integer = 0 To 89
                    Thread.Sleep(30)
                    SBZoom_Scroll(Me, New ScrollEventArgs(ScrollEventType.SmallIncrement, 0, 1, ScrollOrientation.VerticalScroll))
                    If Not turorialRuns Then Exit Do
                Next i
                For i As Integer = 0 To 121
                    Thread.Sleep(30)
                    SBZoom_Scroll(Me, New ScrollEventArgs(ScrollEventType.SmallDecrement, 0, -1, ScrollOrientation.VerticalScroll))
                    If Not turorialRuns Then Exit Do
                Next i
                resetGUI()
                While MsgBox("2.) Move the viewport: " & vbCrLf & "Press the " & mouseButtonR & " to translate the origin of the viewport." & vbCrLf & "You can read the current positon of the cursor on the lower left corner of this form." & vbCrLf & "It is displayed in LDU coordinates." & vbCrLf & vbCrLf & "Press [ENTER] to continue and watch this action.", MsgBoxStyle.OkOnly + MsgBoxStyle.MsgBoxSetForeground, "Info:") <> MsgBoxResult.Ok : End While
                resetGUI()
                moveMouseAbsoluteRightClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2, System.Windows.Forms.Screen.PrimaryScreen.Bounds.Height / 2)
                Thread.Sleep(30)
                moveMouseRightSmooth(System.Windows.Forms.Cursor.Position.X, System.Windows.Forms.Cursor.Position.Y, System.Windows.Forms.Cursor.Position.X + 200, System.Windows.Forms.Cursor.Position.Y + 100, 100)
                While MsgBox("3.) Display/hide the grid: " & vbCrLf & "Click on 'Views' -> 'Show grid' to enable/disable this view." & vbCrLf & vbCrLf & "Press [ENTER] to continue and watch this action (I will use ALT+V and S to access the menu).", MsgBoxStyle.OkOnly + MsgBoxStyle.MsgBoxSetForeground, "Info:") <> MsgBoxResult.Ok : End While
                resetGUI()
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1000)
                SendKeys.SendWait("%v")
                ShowGridToolStripMenuItem.Select()
                If Not turorialRuns Then Exit Do Else Thread.Sleep(2000)
                SendKeys.SendWait("s%v")
                ShowGridToolStripMenuItem.Select()
                If Not turorialRuns Then Exit Do Else Thread.Sleep(2000)
                SendKeys.SendWait("s%v")
                ShowGridToolStripMenuItem.Select()
                If Not turorialRuns Then Exit Do Else Thread.Sleep(2000)
                SendKeys.SendWait("%")
                Me.Refresh()
                Me.Invalidate()
                Me.Update()
                While MsgBox("4.) Resize the grid: " & vbCrLf & "  - Click on 'Views' -> 'View-Preferences' to enable this view." & vbCrLf & "  - Adjust the 'Grid Size'-Value to increase/decrease the grid resolution." & vbCrLf & vbCrLf & "Press [ENTER] to continue and watch this action (I will use ALT+V and V to access the menu).", MsgBoxStyle.OkOnly + MsgBoxStyle.MsgBoxSetForeground, "Info:") <> MsgBoxResult.Ok : End While
                resetGUI()
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1000)
                SendKeys.SendWait("%v")
                SnappingToolStripMenuItem.Select()
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1000)
                SendKeys.SendWait("v%v")
                SnappingToolStripMenuItem.Select()
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1000)
                SendKeys.SendWait("%")
                NUDGrid.Value = 10
                While NUDGrid.Value > 0.1
                    If Not turorialRuns Then Exit Do
                    NUDGrid.Value -= 0.1
                    Thread.Sleep(100)
                End While
                If Not turorialRuns Then Exit Do Else While MsgBox("Thank you for watching this lesson." & vbCrLf & vbCrLf & "Press [ENTER] to continue.", MsgBoxStyle.OkOnly + MsgBoxStyle.MsgBoxSetForeground, "Info:") <> MsgBoxResult.Ok : End While
                Exit Do
            Loop
        End If

        If TutorialNumber = 1 Then
            Do
                If Not turorialRuns Then Exit Do
                While MsgBox("1.) Add vertices: " & vbCrLf & "Click on 'Add Vertex', then click on the grid with your " & mouseButtonL & " to add a vertex to your model." & vbCrLf & vbCrLf & "Press [ENTER] to continue and watch this action.", MsgBoxStyle.OkOnly + MsgBoxStyle.MsgBoxSetForeground, "Info:") <> MsgBoxResult.Ok : End While
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1500)
                clickOnButton(BtnAddVertex)
                moveMouseAbsoluteRightClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2, offset_Y + Me.ClientSize.Height / 2)
                If Not turorialRuns Then Exit Do Else Thread.Sleep(500)
                pressMouseLeft()
                releaseMouseLeft()
                If Not turorialRuns Then Exit Do Else Thread.Sleep(500)
                moveMouseAbsoluteRightClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2 - 100, System.Windows.Forms.Screen.PrimaryScreen.Bounds.Height / 2)
                If Not turorialRuns Then Exit Do Else Thread.Sleep(500)
                pressMouseLeft()
                releaseMouseLeft()
                If Not turorialRuns Then Exit Do Else Thread.Sleep(500)
                moveMouseAbsoluteRightClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2 - 100, System.Windows.Forms.Screen.PrimaryScreen.Bounds.Height / 2 - 100)
                If Not turorialRuns Then Exit Do Else Thread.Sleep(500)
                pressMouseLeft()
                releaseMouseLeft()
                If Not turorialRuns Then Exit Do Else Thread.Sleep(100)
                moveMouseAbsoluteRightClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2, System.Windows.Forms.Screen.PrimaryScreen.Bounds.Height / 2 - 100)
                Thread.Sleep(100)
                pressMouseLeft()
                releaseMouseLeft()
                If Not turorialRuns Then Exit Do Else Thread.Sleep(100)
                moveMouseAbsoluteRightClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2 + 100, System.Windows.Forms.Screen.PrimaryScreen.Bounds.Height / 2 - 100)
                Thread.Sleep(100)
                pressMouseLeft()
                releaseMouseLeft()
                Thread.Sleep(100)
                moveMouseAbsoluteRightClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2 + 100, System.Windows.Forms.Screen.PrimaryScreen.Bounds.Height / 2)
                If Not turorialRuns Then Exit Do Else Thread.Sleep(100)
                pressMouseLeft()
                releaseMouseLeft()
                Thread.Sleep(100)
                moveMouseAbsoluteRightClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2 + 100, System.Windows.Forms.Screen.PrimaryScreen.Bounds.Height / 2 + 100)
                Thread.Sleep(100)
                pressMouseLeft()
                releaseMouseLeft()
                If Not turorialRuns Then Exit Do Else Thread.Sleep(50)
                moveMouseAbsoluteRightClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2, System.Windows.Forms.Screen.PrimaryScreen.Bounds.Height / 2 + 100)
                Thread.Sleep(50)
                pressMouseLeft()
                releaseMouseLeft()
                Thread.Sleep(50)
                moveMouseAbsoluteRightClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2 - 100, System.Windows.Forms.Screen.PrimaryScreen.Bounds.Height / 2 + 100)
                Thread.Sleep(50)
                pressMouseLeft()
                releaseMouseLeft()
                Thread.Sleep(50)
                If Not turorialRuns Then Exit Do Else Thread.Sleep(50)
                While MsgBox("2.) Add triangles: " & vbCrLf & "Click on 'Add Triangle', then click on the vertices with your " & mouseButtonL & " to create a triangle." & vbCrLf & vbCrLf & "Press [ENTER] to continue and watch this action.", MsgBoxStyle.OkOnly + MsgBoxStyle.MsgBoxSetForeground, "Info:") <> MsgBoxResult.Ok : End While
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1500)
                clickOnButton(BtnAddTriangle)

                moveMouseAbsoluteLeftClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2, offset_Y + Me.ClientSize.Height / 2)
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1000)
                moveMouseAbsoluteLeftClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2 - 100, offset_Y + Me.ClientSize.Height / 2)
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1000)
                moveMouseAbsoluteLeftClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2 - 100, offset_Y + Me.ClientSize.Height / 2 - 100)

                moveMouseAbsoluteLeftClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2 - 100, offset_Y + Me.ClientSize.Height / 2 - 100)
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1000)
                moveMouseAbsoluteLeftClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2, offset_Y + Me.ClientSize.Height / 2 - 100)
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1000)
                moveMouseAbsoluteLeftClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2, offset_Y + Me.ClientSize.Height / 2)
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1000)


                moveMouseAbsoluteLeftClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2, offset_Y + Me.ClientSize.Height / 2 - 100)
                Thread.Sleep(500)
                moveMouseAbsoluteLeftClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2 + 100, offset_Y + Me.ClientSize.Height / 2 - 100)
                If Not turorialRuns Then Exit Do Else Thread.Sleep(500)
                moveMouseAbsoluteLeftClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2, offset_Y + Me.ClientSize.Height / 2)

                moveMouseAbsoluteLeftClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2, offset_Y + Me.ClientSize.Height / 2)
                Thread.Sleep(500)
                moveMouseAbsoluteLeftClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2 + 100, offset_Y + Me.ClientSize.Height / 2)
                If Not turorialRuns Then Exit Do Else Thread.Sleep(500)
                moveMouseAbsoluteLeftClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2 + 100, offset_Y + Me.ClientSize.Height / 2 - 100)
                Thread.Sleep(500)



                moveMouseAbsoluteLeftClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2, offset_Y + Me.ClientSize.Height / 2 + 100)
                If Not turorialRuns Then Exit Do Else Thread.Sleep(500)
                moveMouseAbsoluteLeftClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2 - 100, offset_Y + Me.ClientSize.Height / 2 + 100)
                Thread.Sleep(500)
                moveMouseAbsoluteLeftClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2 - 100, offset_Y + Me.ClientSize.Height / 2)

                moveMouseAbsoluteLeftClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2 - 100, offset_Y + Me.ClientSize.Height / 2)
                If Not turorialRuns Then Exit Do Else Thread.Sleep(100)
                moveMouseAbsoluteLeftClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2, offset_Y + Me.ClientSize.Height / 2)
                Thread.Sleep(100)
                moveMouseAbsoluteLeftClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2, offset_Y + Me.ClientSize.Height / 2 + 100)
                Thread.Sleep(100)


                moveMouseAbsoluteLeftClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2, offset_Y + Me.ClientSize.Height / 2)
                Thread.Sleep(100)
                moveMouseAbsoluteLeftClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2 + 100, offset_Y + Me.ClientSize.Height / 2)
                If Not turorialRuns Then Exit Do Else Thread.Sleep(100)
                moveMouseAbsoluteLeftClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2, offset_Y + Me.ClientSize.Height / 2 + 100)

                moveMouseAbsoluteLeftClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2, offset_Y + Me.ClientSize.Height / 2 + 100)
                Thread.Sleep(100)
                moveMouseAbsoluteLeftClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2 + 100, offset_Y + Me.ClientSize.Height / 2 + 100)
                Thread.Sleep(100)
                moveMouseAbsoluteLeftClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2 + 100, offset_Y + Me.ClientSize.Height / 2)
                Thread.Sleep(100)
                moveMouseAbsoluteLeftClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2, offset_Y + Me.ClientSize.Height / 2 + 100)
                Thread.Sleep(100)
                If Not turorialRuns Then Exit Do Else Thread.Sleep(100)
                clickOnButton(BtnAddTriangle)
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1000)
                While MsgBox("2.) Move vertices: " & vbCrLf & "  - Click on 'Triangle Mode', then select the 'Vertex Mode' entry to enable vertex-selection." & vbCrLf & "  - Hold the " & mouseButtonL & " and select all vertices." & vbCrLf & "  - Click on 'Move' to activate the movemode" & vbCrLf & "  - Hold the " & mouseButtonL & " and drag it over the screen to move the selection." & vbCrLf & vbCrLf & "Press [ENTER] to continue and watch this action.", MsgBoxStyle.OkOnly + MsgBoxStyle.MsgBoxSetForeground, "Info:") <> MsgBoxResult.Ok : End While
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1500)
                clickOnButton(BtnMode)
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1000)
                VerticesModeToolStripMenuItem.Select()
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1000)
                VerticesModeToolStripMenuItem.PerformClick()
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1000)
                moveMouseAbsoluteLeftClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2 - 200, offset_Y + Me.ClientSize.Height / 2 - 200)
                pressMouseLeft()
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1000)
                System.Windows.Forms.Cursor.Position = New Point(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2 + 200, offset_Y + Me.ClientSize.Height / 2 + 200)
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1000)
                releaseMouseLeft()
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1000)
                clickOnButton(BtnMove)
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1500)
                System.Windows.Forms.Cursor.Position = New Point(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2, offset_Y + Me.ClientSize.Height / 2)
                Thread.Sleep(30)
                moveMouseLeftSmooth(System.Windows.Forms.Cursor.Position.X, System.Windows.Forms.Cursor.Position.Y, System.Windows.Forms.Cursor.Position.X + 200, System.Windows.Forms.Cursor.Position.Y, 100)
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1500)
                While MsgBox("3.) Rotate vertices: " & vbCrLf & "  - Click on 'Rotate' to activate the rotatemode" & vbCrLf & "  - Hold the " & mouseButtonL & " and drag it over the screen to rotate the selection." & vbCrLf & vbCrLf & "Press [ENTER] to continue and watch this action.", MsgBoxStyle.OkOnly + MsgBoxStyle.MsgBoxSetForeground, "Info:") <> MsgBoxResult.Ok : End While
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1000)
                clickOnButton(BtnRotate)
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1500)
                System.Windows.Forms.Cursor.Position = New Point(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2, offset_Y + Me.ClientSize.Height / 2)
                Thread.Sleep(30)
                moveMouseLeftSmooth(System.Windows.Forms.Cursor.Position.X, System.Windows.Forms.Cursor.Position.Y, System.Windows.Forms.Cursor.Position.X + 170, System.Windows.Forms.Cursor.Position.Y, 100)
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1500)
                While MsgBox("4.) Scale vertices: " & vbCrLf & "  - Click on 'Scale' to activate the scalemode" & vbCrLf & "  - Hold the " & mouseButtonL & " and drag it over the screen to scale the selection." & vbCrLf & vbCrLf & "Press [ENTER] to continue and watch this action.", MsgBoxStyle.OkOnly + MsgBoxStyle.MsgBoxSetForeground, "Info:") <> MsgBoxResult.Ok : End While
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1000)
                clickOnButton(BtnScale)
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1500)
                System.Windows.Forms.Cursor.Position = New Point(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2, offset_Y + Me.ClientSize.Height / 2)
                Thread.Sleep(30)
                moveMouseLeftSmooth(System.Windows.Forms.Cursor.Position.X, System.Windows.Forms.Cursor.Position.Y, System.Windows.Forms.Cursor.Position.X + 50, System.Windows.Forms.Cursor.Position.Y, 100)
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1500)
                While MsgBox("5.) Colouring triangles: " & vbCrLf & "  - Click on 'Vertex Mode', then select the 'Triangle Mode' entry  and press 1 to enable triangle-selection." & vbCrLf & "  - " & upperL & "-click the upper right triangle" & vbCrLf & "  - Click on 'Edit'->'Select Touching..' (Alt+E+T) to select the triangles which are touching the current selection." & vbCrLf & "  - Click on 'Show Colours' and pick a colour" & vbCrLf & vbCrLf & "Press [ENTER] to continue and watch this action.", MsgBoxStyle.OkOnly + MsgBoxStyle.MsgBoxSetForeground, "Info:") <> MsgBoxResult.Ok : End While
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1500)
                clickOnButton(BtnMode)
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1000)
                TriangleModeToolStripMenuItem.PerformClick()
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1000)
                SendKeys.SendWait("1")
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1000)
                moveMouseAbsoluteLeftClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2 + 180, offset_Y + Me.ClientSize.Height / 2 - 16)
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1000)
                SendKeys.SendWait("%e")
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1000)
                SelectTouchingToolStripMenuItem.Select()
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1000)
                SendKeys.SendWait("t")
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1000)
                clickOnButton(BtnColours)
                C0.Select()
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1000)
                C0.PerformClick()
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1000)
                While MsgBox("Thank you for watching this lesson." & vbCrLf & vbCrLf & "Press [ENTER] to continue.", MsgBoxStyle.OkOnly + MsgBoxStyle.MsgBoxSetForeground, "Info:") <> MsgBoxResult.Ok : End While
                Exit Do
            Loop
        End If

        resetGUI()
        releaseMouseRight()
        Thread.Sleep(1000)
        releaseMouseLeft()
        Thread.Sleep(1000)
        AbortCurrentLessonToolStripMenuItem.Enabled = False
        Me.BackgroundHowTo.CancelAsync()
        Return "[ready]"
    End Function


    Private Sub resetGUI()
        If BtnPreview.Checked Then BtnPreview.PerformClick() : Thread.Sleep(500)
        If BtnAddTriangle.Checked Then BtnAddTriangle.PerformClick() : Thread.Sleep(500)
        If BtnAddVertex.Checked Then BtnAddVertex.PerformClick() : Thread.Sleep(500)
        If Not ShowGridToolStripMenuItem.Checked Then
            ViewToolStripMenuItem.PerformClick()
            Thread.Sleep(500)
            ShowGridToolStripMenuItem.PerformClick()
            Thread.Sleep(500)
        End If
        If WithColourToolStripMenuItem.Checked Then
            EditToolStripMenuItem.PerformClick()
            Thread.Sleep(500)
            WithColourToolStripMenuItem.PerformClick()
            Thread.Sleep(500)
        End If
        If BtnColours.Checked Then
            BtnColours.PerformClick()
            Thread.Sleep(500)
        End If
        abortPrimitiveMode()
        CircleSegments.Text = 16
        OvalSegments.Text = 16
        newPattern()
        BtnMode.PerformClick()
        Thread.Sleep(500)
        VerticesModeToolStripMenuItem.PerformClick()
        Thread.Sleep(500)
        BtnSelect.PerformClick()
        GBImage.Visible = False
        ImageToolStripMenuItem.Checked = False
        GBSnap.Visible = False
        SnappingToolStripMenuItem.Checked = False
        toffsetX = 0
        toffsetY = 0
        offsetX = 0
        offsetY = 0
        zoomfactor = 1
        zoomlevel = 0
        LblDebug.Text = "Zoom: 100%"
        Me.Refresh()
    End Sub

    Private Sub AbortCurrentLessonToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles AbortCurrentLessonToolStripMenuItem.Click
        turorialRuns = False
    End Sub

    Public Sub clickOnButton(ByRef tsb As ToolStripButton)
        System.Windows.Forms.Cursor.Position = New Point(tsb.Bounds.Location.X + tsb.Size.Width / 2, tsb.Size.Height / 2 + offset_Y + MenuStrip1.Height)
        Thread.Sleep(1000)
        tsb.PerformClick()
        Thread.Sleep(1000)
    End Sub

    Public Sub clickOnButton(ByRef tsb As ToolStripDropDownButton)
        System.Windows.Forms.Cursor.Position = New Point(tsb.Bounds.Location.X + tsb.Size.Width / 2, tsb.Size.Height / 2 + offset_Y + MenuStrip1.Height)
        Thread.Sleep(1000)
    End Sub


    Public Sub getMouseCoords()
        cur_x = System.Windows.Forms.Cursor.Position.X * 65535 / System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width
        cur_y = System.Windows.Forms.Cursor.Position.Y * 65535 / System.Windows.Forms.Screen.PrimaryScreen.Bounds.Height
    End Sub

    Public Sub moveMouseAbsoluteLeftPressed(ByRef dx As Single, ByRef dy As Single)
        dest_x = dx * 65535 / System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width
        dest_y = dy * 65535 / System.Windows.Forms.Screen.PrimaryScreen.Bounds.Height
        If My.Computer.Mouse.ButtonsSwapped Then
            mouse_event(MOUSEEVENTF_ABSOLUTE + MOUSEEVENTF_RIGHTDOWN + MOUSEEVENTF_MOVE, dest_x, dest_y, 0, 0)
        Else
            mouse_event(MOUSEEVENTF_ABSOLUTE + MOUSEEVENTF_LEFTDOWN + MOUSEEVENTF_MOVE, dest_x, dest_y, 0, 0)
        End If
    End Sub

    Public Sub moveMouseAbsoluteRightPressed(ByRef dx As Single, ByRef dy As Single)
        dest_x = dx * 65535 / System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width
        dest_y = dy * 65535 / System.Windows.Forms.Screen.PrimaryScreen.Bounds.Height
        If My.Computer.Mouse.ButtonsSwapped Then
            mouse_event(MOUSEEVENTF_ABSOLUTE + MOUSEEVENTF_LEFTDOWN + MOUSEEVENTF_MOVE, dest_x, dest_y, 0, 0)
        Else
            mouse_event(MOUSEEVENTF_ABSOLUTE + MOUSEEVENTF_RIGHTDOWN + MOUSEEVENTF_MOVE, dest_x, dest_y, 0, 0)
        End If
    End Sub

    Public Sub pressMouseRight()
        getMouseCoords()
        If My.Computer.Mouse.ButtonsSwapped Then
            mouse_event(MOUSEEVENTF_ABSOLUTE + MOUSEEVENTF_LEFTDOWN, cur_x, cur_y, 0, 0)
        Else
            mouse_event(MOUSEEVENTF_ABSOLUTE + MOUSEEVENTF_RIGHTDOWN, cur_x, cur_y, 0, 0)
        End If
    End Sub

    Public Sub pressMouseLeft()
        getMouseCoords()
        If My.Computer.Mouse.ButtonsSwapped Then
            mouse_event(MOUSEEVENTF_ABSOLUTE + MOUSEEVENTF_RIGHTDOWN, cur_x, cur_y, 0, 0)
        Else
            mouse_event(MOUSEEVENTF_ABSOLUTE + MOUSEEVENTF_LEFTDOWN, cur_x, cur_y, 0, 0)
        End If
    End Sub

    Public Sub releaseMouseRight()
        getMouseCoords()
        If My.Computer.Mouse.ButtonsSwapped Then
            mouse_event(MOUSEEVENTF_ABSOLUTE + MOUSEEVENTF_LEFTUP, cur_x, cur_y, 0, 0)
        Else
            mouse_event(MOUSEEVENTF_ABSOLUTE + MOUSEEVENTF_RIGHTUP, cur_x, cur_y, 0, 0)
        End If
    End Sub

    Public Sub releaseMouseLeft()
        getMouseCoords()
        If My.Computer.Mouse.ButtonsSwapped Then
            mouse_event(MOUSEEVENTF_ABSOLUTE + MOUSEEVENTF_RIGHTUP, cur_x, cur_y, 0, 0)
        Else
            mouse_event(MOUSEEVENTF_ABSOLUTE + MOUSEEVENTF_LEFTUP, cur_x, cur_y, 0, 0)
        End If
    End Sub

    Public Sub moveMouseRightSmooth(ByVal sx As Single, ByVal sy As Single, ByVal dx As Single, ByVal dy As Single, ByVal t As Integer)
        pressMouseRight()
        Thread.Sleep(30)
        Dim cx, cy As Single
        cx = (dx - sx) / t
        cy = (dy - sy) / t
        For ct As Integer = 1 To t
            If Not turorialRuns Then Exit For
            sx += cx
            sy += cy
            dest_x = sx * 65535 / System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width
            dest_y = sy * 65535 / System.Windows.Forms.Screen.PrimaryScreen.Bounds.Height
            mouse_event(MOUSEEVENTF_ABSOLUTE + MOUSEEVENTF_MOVE, dest_x, dest_y, 0, 0)
            Thread.Sleep(1)
        Next
        releaseMouseRight()
    End Sub

    Public Sub moveMouseLeftSmooth(ByVal sx As Single, ByVal sy As Single, ByVal dx As Single, ByVal dy As Single, ByVal t As Integer)
        pressMouseLeft()
        Thread.Sleep(30)
        Dim cx, cy As Single
        cx = (dx - sx) / t
        cy = (dy - sy) / t
        For ct As Integer = 1 To t
            If Not turorialRuns Then Exit For
            sx += cx
            sy += cy
            dest_x = sx * 65535 / System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width
            dest_y = sy * 65535 / System.Windows.Forms.Screen.PrimaryScreen.Bounds.Height
            mouse_event(MOUSEEVENTF_ABSOLUTE + MOUSEEVENTF_MOVE, dest_x, dest_y, 0, 0)
            Thread.Sleep(1)
        Next
        releaseMouseLeft()
    End Sub

    Public Sub moveMouseAbsoluteLeftClick(ByVal dx As Single, ByVal dy As Single)
        dest_x = dx * 65535 / System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width
        dest_y = dy * 65535 / System.Windows.Forms.Screen.PrimaryScreen.Bounds.Height
        If My.Computer.Mouse.ButtonsSwapped Then
            mouse_event(MOUSEEVENTF_ABSOLUTE + MOUSEEVENTF_MOVE + MOUSEEVENTF_RIGHTDOWN + MOUSEEVENTF_RIGHTUP, dest_x, dest_y, 0, 0)
        Else
            mouse_event(MOUSEEVENTF_ABSOLUTE + MOUSEEVENTF_MOVE + MOUSEEVENTF_LEFTDOWN + MOUSEEVENTF_LEFTUP, dest_x, dest_y, 0, 0)
        End If
    End Sub

    Public Sub moveMouseAbsoluteRightClick(ByVal dx As Single, ByVal dy As Single)
        dest_x = dx * 65535 / System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width
        dest_y = dy * 65535 / System.Windows.Forms.Screen.PrimaryScreen.Bounds.Height
        If My.Computer.Mouse.ButtonsSwapped Then
            mouse_event(MOUSEEVENTF_ABSOLUTE + MOUSEEVENTF_MOVE + MOUSEEVENTF_LEFTDOWN + MOUSEEVENTF_LEFTUP, dest_x, dest_y, 0, 0)
        Else
            mouse_event(MOUSEEVENTF_ABSOLUTE + MOUSEEVENTF_MOVE + MOUSEEVENTF_RIGHTDOWN + MOUSEEVENTF_RIGHTUP, dest_x, dest_y, 0, 0)
        End If
    End Sub

#End Region

#Region "CMS"
    Private Sub CMSSelect_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CMSSelect.Click
        BtnSelect.PerformClick()
    End Sub

    Private Sub CMSMove_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CMSMove.Click
        BtnMove.PerformClick()
    End Sub

    Private Sub CMSRotate_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CMSRotate.Click
        BtnRotate.PerformClick()
    End Sub

    Private Sub CMSScale_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CMSScale.Click
        BtnScale.PerformClick()
    End Sub

    Private Sub CMSVertex_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CMSVertex.Click
        VerticesModeToolStripMenuItem.PerformClick()
    End Sub

    Private Sub CMSTriangle_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CMSTriangle.Click
        TriangleModeToolStripMenuItem.PerformClick()
    End Sub

    Private Sub CMSPrimitive_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CMSPrimitive.Click
        PrimitiveModeToolStripMenuItem.PerformClick()
    End Sub

    Private Sub CMSAddVertex_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CMSAddVertex.Click
        BtnAddVertex.PerformClick()
        CMSAddTriangle.Checked = False
    End Sub

    Private Sub CMSAddTriangle_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CMSAddTriangle.Click
        BtnAddTriangle.PerformClick()
        CMSAddVertex.Checked = False
    End Sub

    Private Sub CMSCut_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CMSCut.Click
        BtnCut.PerformClick()
    End Sub

    Private Sub CMSCopy_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CMSCopy.Click
        BtnCopy.PerformClick()
    End Sub

    Private Sub CMSPaste_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CMSPaste.Click
        BtnPaste.PerformClick()
    End Sub

    Private Sub CMSDelete_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CMSDelete.Click
        ToolStripMenuItem2.PerformClick()
    End Sub
#End Region

    Private Sub ToAverageToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToAverageToolStripMenuItem.Click
        If SelectedVertices.Count > 1 Then
            addHistoryI(withDataFrom.All)
            mergeToAverage()
            addHistoryII(withDataFrom.All)
            Me.Refresh()
        End If
    End Sub

    Private Sub mergeToAverage()
        trianglemode = 0
        Dim sumX As Integer = 0
        Dim sumY As Integer = 0
        For i As Integer = 0 To SelectedVertices.Count - 1
            sumX += SelectedVertices(i).X
            sumY += SelectedVertices(i).Y
            SelectedVertices(i).selected = False
        Next
        sumX /= SelectedVertices.Count
        sumY /= SelectedVertices.Count
        For i As Integer = 0 To SelectedVertices.Count - 1
            SelectedVertices(i).X = sumX
            SelectedVertices(i).Y = sumY
        Next
        SelectedVertices.Clear()
        SelectedTriangles.Clear()
        cleanupDATVertices()
        cleanupDATTriangles()
    End Sub

    Private Sub mergeToAverage2()
        Dim sumX As Integer = 0
        Dim sumY As Integer = 0
        Dim temp(SelectedVertices.Count - 1, 1) As Single
        For i As Integer = 0 To SelectedVertices.Count - 1
            temp(i, 0) = SelectedVertices(i).X
            temp(i, 1) = SelectedVertices(i).Y
            sumX += temp(i, 0)
            sumY += temp(i, 1)
            For j As Integer = 0 To SelectedVertices(i).linkedTriangles.Count - 1
                If Not SelectedTriangles.Contains(SelectedVertices(i).linkedTriangles(j)) Then
                    SelectedTriangles.Add(SelectedVertices(i).linkedTriangles(j))
                End If
            Next
        Next

        sumX /= SelectedVertices.Count
        sumY /= SelectedVertices.Count

        For i As Integer = 0 To SelectedVertices.Count - 1
            SelectedVertices(i).X = sumX
            SelectedVertices(i).Y = sumY
        Next

        Dim old_count As Integer = SelectedVertices.Count
        selectTouching3()
        For i As Integer = SelectedVertices.Count To SelectedVertices.Count - 1
            For j As Integer = 0 To SelectedVertices(i).linkedTriangles.Count - 1
                If Not SelectedTriangles.Contains(SelectedVertices(i).linkedTriangles(j)) Then
                    SelectedTriangles.Add(SelectedVertices(i).linkedTriangles(j))
                End If
            Next
        Next
        For trii1 As Integer = 0 To SelectedTriangles.Count - 1
            Dim tri1 As Triangle = SelectedTriangles(trii1)
            For trii2 As Integer = trii1 + 1 To SelectedTriangles.Count - 1
                Dim tri2 As Triangle = SelectedTriangles(trii2)
                If CSG2TrianglesIntersectionsOnly(tri1, tri2, False) Then
                    For i As Integer = 0 To old_count - 1
                        SelectedVertices(i).X = temp(i, 0)
                        SelectedVertices(i).Y = temp(i, 1)
                    Next
                    GoTo endCheck
                End If
            Next
        Next
endCheck:

        SelectedVertices.Clear()
        SelectedTriangles.Clear()

    End Sub

    Private Sub ToLastSelectedToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToLastSelectedToolStripMenuItem.Click
        If SelectedVertices.Count > 1 Then
            addHistoryI(withDataFrom.All)
            mergeToLast()
            addHistoryII(withDataFrom.All)
            Me.Refresh()
        End If
    End Sub

    Private Sub mergeToLast()
        trianglemode = 0
        For i As Integer = 0 To SelectedVertices.Count - 1
            SelectedVertices(i).X = LLast(SelectedVertices).X
            SelectedVertices(i).Y = LLast(SelectedVertices).Y
            SelectedVertices(i).selected = False
        Next
        SelectedVertices.Clear()
        SelectedTriangles.Clear()
        cleanupDATVertices()
        cleanupDATTriangles()
    End Sub

    Private Sub splitAndMerge()
        Dim count As Integer
        Dim halfcount As Integer
        count = SelectedVertices.Count
        halfcount = count \ 2
        Dim temp(count - 1, 1) As Single
        For i As Integer = 1 To halfcount
            temp(i, 0) = SelectedVertices(i).X
            temp(i, 1) = SelectedVertices(i).Y
            SelectedVertices(i).X = SelectedVertices(0).X
            SelectedVertices(i).Y = SelectedVertices(0).Y
        Next
        For i As Integer = halfcount + 1 To count - 1
            temp(i, 0) = SelectedVertices(i).X
            temp(i, 1) = SelectedVertices(i).Y
            SelectedVertices(i).X = SelectedVertices(count - 1).X
            SelectedVertices(i).Y = SelectedVertices(count - 1).Y
        Next
        Dim old_count As Integer = SelectedVertices.Count
        selectTouching3()
        For i As Integer = 0 To SelectedVertices.Count - 1
            For j As Integer = 0 To SelectedVertices(i).linkedTriangles.Count - 1
                If Not SelectedTriangles.Contains(SelectedVertices(i).linkedTriangles(j)) Then
                    SelectedTriangles.Add(SelectedVertices(i).linkedTriangles(j))
                End If
            Next
        Next
        For trii1 As Integer = 0 To SelectedTriangles.Count - 1
            Dim tri1 As Triangle = SelectedTriangles(trii1)
            For trii2 As Integer = trii1 + 1 To SelectedTriangles.Count - 1
                Dim tri2 As Triangle = SelectedTriangles(trii2)
                If CSG2TrianglesIntersectionsOnly(tri1, tri2, False) Then
                    For i As Integer = 0 To old_count - 1
                        SelectedVertices(i).X = temp(i, 0)
                        SelectedVertices(i).Y = temp(i, 1)
                    Next
                    GoTo endCheck
                End If
            Next
        Next
endCheck:
        SelectedTriangles.Clear()
        SelectedVertices.Clear()
    End Sub

    Private Sub splitAndMerge2()
        Dim count As Integer
        Dim halfcount As Integer
        count = SelectedVertices.Count
        halfcount = count \ 2
        Dim temp(count - 1, 1) As Single
        For i As Integer = 1 To halfcount
            temp(i, 0) = SelectedVertices(i).X
            temp(i, 1) = SelectedVertices(i).Y
            SelectedVertices(i).X = SelectedVertices(0).X
            SelectedVertices(i).Y = SelectedVertices(0).Y
        Next
        For i As Integer = halfcount + 1 To count - 1
            temp(i, 0) = SelectedVertices(i).X
            temp(i, 1) = SelectedVertices(i).Y
            SelectedVertices(i).X = SelectedVertices(count - 1).X
            SelectedVertices(i).Y = SelectedVertices(count - 1).Y
        Next
        Dim old_count As Integer = SelectedVertices.Count
        selectTouching3()
        For i As Integer = SelectedVertices.Count To SelectedVertices.Count - 1
            For j As Integer = 0 To SelectedVertices(i).linkedTriangles.Count - 1
                If Not SelectedTriangles.Contains(SelectedVertices(i).linkedTriangles(j)) Then
                    SelectedTriangles.Add(SelectedVertices(i).linkedTriangles(j))
                End If
            Next
        Next
        For trii1 As Integer = 0 To SelectedTriangles.Count - 1
            Dim tri1 As Triangle = SelectedTriangles(trii1)
            For trii2 As Integer = trii1 + 1 To SelectedTriangles.Count - 1
                Dim tri2 As Triangle = SelectedTriangles(trii2)
                If CSG2TrianglesIntersectionsOnly(tri1, tri2, False) Then
                    For i As Integer = 0 To old_count - 1
                        SelectedVertices(i).X = temp(i, 0)
                        SelectedVertices(i).Y = temp(i, 1)
                    Next
                    GoTo endCheck
                End If
            Next
        Next
endCheck:
        SelectedTriangles.Clear()
        SelectedVertices.Clear()
    End Sub

#Region "CSG"

    Private Sub CSGAddToolStripMenuItem_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles CSGUnionToolStripMenuItem.Click
        If SelectedTriangles.Count > 1 Then
            addHistoryI(withDataFrom.All)
            CSG2Triangles(SelectedTriangles(0), SelectedTriangles(1))
            cleanupDATVertices()
            For i As Integer = 0 To Triangles.Count - 1
                Triangles(i).selected = False
            Next
            SelectedTriangles.Clear()
            SelectedVertices.Clear()
            addHistoryII(withDataFrom.All)
            Me.Refresh()
        End If
    End Sub

    Private Sub CSGIntersectionPointsToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CSGIntersectionPointsToolStripMenuItem.Click
        If SelectedTriangles.Count > 1 Then
            addHistoryI(withDataFrom.All)
            For trii1 As Integer = 0 To SelectedTriangles.Count - 1
                Dim tri1 As Triangle = SelectedTriangles(trii1)
                For trii2 As Integer = trii1 + 1 To SelectedTriangles.Count - 1
                    Dim tri2 As Triangle = SelectedTriangles(trii2)
                    CSG2TrianglesIntersectionsOnly(tri1, tri2)
                Next
            Next
            cleanupDATVertices()
            addHistoryII(withDataFrom.All)
            Me.Refresh()
        End If
    End Sub

    Private Sub CSGSubdivideToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CSGSubdivideToolStripMenuItem.Click
        If SelectedTriangles.Count = 1 Then
            addHistoryI(withDataFrom.All)
            subdivideTriangle(SelectedTriangles(0))
            SelectedVertices.Clear()
            SelectedTriangles.Clear()
            cleanupDATVertices()
            paintTrianglesTransparent()
            addHistoryII(withDataFrom.All)
            Me.Refresh()
        End If
    End Sub

    Private Sub CSGSplitToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CSGSplitToolStripMenuItem.Click
        If SelectedVertices.Count = 2 Then
            addHistoryI(withDataFrom.All)
            splitTriangle(SelectedVertices(0), SelectedVertices(1))
            SelectedVertices.Clear()
            addHistoryII(withDataFrom.All)
            Me.Refresh()
        End If
    End Sub

#End Region

    Private Sub MainForm_Paint(ByVal sender As Object, ByVal e As System.Windows.Forms.PaintEventArgs) Handles Me.Paint
        'Dim wa As Stopwatch = Stopwatch.StartNew()
        e.Graphics.SmoothingMode = Drawing2D.SmoothingMode.HighSpeed
        If kamera Then
            offsetX = toffsetX + (getCursorpositionX() - klickX) / zoomfactor
            offsetY = toffsetY + (getCursorpositionY() - klickY) / zoomfactor
        End If

        If isLoading Then Exit Sub
        Dim absOffsetX As Integer = Me.offsetX * zoomfactor + CType(Me.ClientSize.Width / 2, Integer)
        Dim absOffsetY As Integer = Me.offsetY * zoomfactor + CType(Me.ClientSize.Height / 2, Integer)

        ' Bild
        If adjustmode Then
            Dim pvabsOffsetX As Integer = Fix((getCursorpositionX() - klickX) / zoomfactor)
            Dim pvabsOffsetY As Integer = Fix((getCursorpositionY() - klickY) / zoomfactor)
            If Not doAdjust Then
                pvabsOffsetX = 0
                pvabsOffsetY = 0
            End If
            Dim toffsetX As Integer = imgOffsetX
            Dim toffsetY As Integer = imgOffsetY
            Dim tScale As Single = imgScale
            If Me.Cursor = Cursors.SizeAll Then
                toffsetX += pvabsOffsetX
                toffsetY += pvabsOffsetY
            ElseIf Me.Cursor = Cursors.SizeNESW OrElse Me.Cursor = Cursors.SizeNWSE OrElse Me.Cursor = Cursors.SizeWE OrElse Me.Cursor = Cursors.SizeNS Then
                If getCursorpositionX() < CType(absOffsetX + toffsetX * zoomfactor, Integer) Then pvabsOffsetX *= -1
                If getCursorpositionY() < CType(absOffsetY + toffsetY * zoomfactor, Integer) Then pvabsOffsetY *= -1
                If Me.Cursor = Cursors.SizeWE Then
                    tScale += ((backgroundPicture.Width * tScale + pvabsOffsetX) / backgroundPicture.Width - tScale) * 2
                Else
                    tScale += ((backgroundPicture.Height * tScale + pvabsOffsetY) / backgroundPicture.Height - tScale) * 2
                End If
                If Control.ModifierKeys = 131072 Then adjustDirection = -1 Else adjustDirection = tempAdjustDirection
                Select Case adjustDirection
                    Case 0 : toffsetY -= pvabsOffsetY
                    Case 1.5 : toffsetX += backgroundPicture.Width / backgroundPicture.Height * pvabsOffsetY : toffsetY -= pvabsOffsetY
                    Case 3 : toffsetX += pvabsOffsetX
                    Case 4.5 : toffsetX += backgroundPicture.Width / backgroundPicture.Height * pvabsOffsetY : toffsetY += pvabsOffsetY
                    Case 6 : toffsetY += pvabsOffsetY
                    Case 7.5 : toffsetX -= backgroundPicture.Width / backgroundPicture.Height * pvabsOffsetY : toffsetY += pvabsOffsetY
                    Case 9 : toffsetX -= pvabsOffsetX
                    Case 10.5 : toffsetX -= backgroundPicture.Width / backgroundPicture.Height * pvabsOffsetY : toffsetY -= pvabsOffsetY
                End Select
            End If
            Try
                e.Graphics.DrawImage(backgroundPicture, CType(absOffsetX - (backgroundPicture.Width / 2 * tScale - toffsetX) * zoomfactor, Integer), CType(absOffsetY - (backgroundPicture.Height / 2 * tScale - toffsetY) * zoomfactor, Integer), backgroundPicture.Width * zoomfactor * tScale, backgroundPicture.Height * zoomfactor * tScale)
            Catch ex As Exception
            End Try
            e.Graphics.DrawRectangle(Pens.LimeGreen, CType(absOffsetX - (backgroundPicture.Width / 2 * tScale - toffsetX) * zoomfactor, Integer), CType(absOffsetY - (backgroundPicture.Height / 2 * tScale - toffsetY) * zoomfactor, Integer), backgroundPicture.Width * zoomfactor * tScale, backgroundPicture.Height * zoomfactor * tScale)
            e.Graphics.DrawRectangle(Pens.LimeGreen, CType(absOffsetX - (backgroundPicture.Width / 2 * tScale - toffsetX) * zoomfactor, Integer) - 1, CType(absOffsetY - (backgroundPicture.Height / 2 * tScale - toffsetY) * zoomfactor, Integer) - 1, backgroundPicture.Width * zoomfactor * tScale + 2, backgroundPicture.Height * zoomfactor * tScale + 2)
        Else
            Try
                e.Graphics.DrawImage(backgroundPicture, CType(absOffsetX - (backgroundPicture.Width / 2 * imgScale - imgOffsetX) * zoomfactor, Integer), CType(absOffsetY - (backgroundPicture.Height / 2 * imgScale - imgOffsetY) * zoomfactor, Integer), backgroundPicture.Width * zoomfactor * imgScale, backgroundPicture.Height * zoomfactor * imgScale)
            Catch ex As Exception
            End Try
        End If

        ' Grid, Origin
        If raster Then
            Dim scale As Single
            If zoomlevel > 80 Then scale = Me.rasterSnap * zoomfactor : GoTo raster_zechnen
            If zoomlevel > 0 Then scale = Me.rasterSnap * zoomfactor * 10 : GoTo raster_zechnen
            If zoomlevel > -10 Then scale = Me.rasterSnap * zoomfactor * 100 : GoTo raster_zechnen
            If zoomlevel > -20 Then scale = Me.rasterSnap * zoomfactor * 1000 : GoTo raster_zechnen
            If zoomlevel > -30 Then scale = Me.rasterSnap * zoomfactor * 10000 : GoTo raster_zechnen
            scale = Me.rasterSnap * zoomfactor * 100000
raster_zechnen:
            Dim relOffsetX As Integer
            Dim relOffsetY As Integer
            relOffsetX = absOffsetX Mod scale
            relOffsetY = absOffsetY Mod scale
            For x As Single = relOffsetX To Me.ClientSize.Width Step scale
                e.Graphics.DrawLine(myColours.gridPen, x, 0, x, Me.ClientSize.Height)
            Next
            For y As Single = relOffsetY To Me.ClientSize.Height Step scale
                e.Graphics.DrawLine(myColours.gridPen, 0, y, Me.ClientSize.Width, y)
            Next
            relOffsetX = absOffsetX Mod (scale * 10)
            relOffsetY = absOffsetY Mod (scale * 10)
            For x As Single = relOffsetX To Me.ClientSize.Width Step scale * 10
                e.Graphics.DrawLine(myColours.grid10Pen, x, 0, x, Me.ClientSize.Height)
            Next
            For y As Single = relOffsetY To Me.ClientSize.Height Step scale * 10
                e.Graphics.DrawLine(myColours.grid10Pen, 0, y, Me.ClientSize.Width, y)
            Next
        End If
        e.Graphics.DrawLine(myColours.originPen, absOffsetX, 0, absOffsetX, Me.ClientSize.Height)
        e.Graphics.DrawLine(myColours.originPen, 0, absOffsetY, Me.ClientSize.Width, absOffsetY)

        Dim vabsOffsetX As Integer
        Dim vabsOffsetY As Integer
        If movemode Then
            If Not Control.ModifierKeys = 131072 Then vabsOffsetX = Fix((getCursorpositionX() - klickX) / zoomfactor / moveSnap) * moveSnap
            If Not Control.ModifierKeys = Keys.Shift Then vabsOffsetY = Fix((getCursorpositionY() - klickY) / zoomfactor / moveSnap) * moveSnap
            If SelectedVertices.Count = 1 AndAlso templateProjectionQuad.Count > 0 Then
                If SelectedVertices(0).linkedTriangles.Count > 0 Then
                    Do
                        For i = 0 To templateProjectionQuad.Count - 1
                            If templateProjectionQuad(i).isInQuad(SelectedVertices(0).linkedTriangles(0)) Then
                                Exit Do
                            End If
                        Next
                        Dim newV As Vertex
                        Dim tmpV As Vertex = SelectedVertices(0) + New Vertex(-vabsOffsetX, vabsOffsetY, False, False)
                        tmpV.vertexID = SelectedVertices(0).vertexID
                        For i = 0 To templateProjectionQuad.Count - 1
                            newV = CSG2ProjectionQuadIntersectionVertex(tmpV, SelectedVertices(0).linkedTriangles(0), templateProjectionQuad(i))
                            If Not newV Is Nothing Then
                                vabsOffsetX = SelectedVertices(0).X - newV.X
                                vabsOffsetY = newV.Y - SelectedVertices(0).Y
                            End If
                        Next
                        Exit Do
                    Loop
                End If
            End If
            For Each vert As Vertex In SelectedVertices
                vert.X -= vabsOffsetX
                vert.Y += vabsOffsetY
            Next
        ElseIf rotatemode Then
            Dim angleToRotate As Double = Fix((-(getCursorpositionX() - klickX) - (getCursorpositionY() - klickY)) / 100 / (rotateSnap * Math.PI / 180)) * (rotateSnap * Math.PI / 180)
            For Each vert As Vertex In SelectedVertices
                vert.X = temp_center.X + vert.distanceFrom * Math.Cos(vert.angleFrom + angleToRotate)
                vert.Y = temp_center.Y + vert.distanceFrom * Math.Sin(vert.angleFrom + angleToRotate)
                If Single.IsNaN(vert.X) OrElse Single.IsNaN(vert.Y) Then
                    vert.X = vert.oldX
                    vert.Y = vert.oldY
                End If
            Next
        ElseIf scalemode Then
            Dim factorToScale As Double = 1 + Fix((getCursorpositionX() - klickX) / 100 / scaleSnap) * scaleSnap
            If factorToScale < 0.1 Then factorToScale = 0.1
            If factorToScale > 2 Then factorToScale = 2
            Dim pf(4) As PointF
            pf(0).X = klickX - 100
            pf(0).Y = klickY - 5
            pf(1).X = klickX + 100
            pf(1).Y = klickY - 5
            pf(2).X = klickX + 100
            pf(2).Y = klickY + 5
            pf(3).X = klickX - 100
            pf(3).Y = klickY + 5
            pf(4).X = klickX - 100
            pf(4).Y = klickY - 5
            e.Graphics.FillPolygon(Brushes.WhiteSmoke, pf)
            e.Graphics.DrawString("0%                                    100%                                200%", New Font("Arial", 8, FontStyle.Regular, GraphicsUnit.Pixel), Brushes.Azure, klickX - 100, klickY + 5)
            For x As Integer = -100 To 100 Step 10
                e.Graphics.DrawLine(Pens.DarkGray, klickX + x, klickY - 5, klickX + x, klickY + 5)
            Next x
            e.Graphics.DrawString(Fix(factorToScale * 100) & "%", New Font("Arial", 8, FontStyle.Bold, GraphicsUnit.Pixel), Brushes.Red, klickX + 5, klickY - 3)
            e.Graphics.DrawLine(Pens.Black, klickX, klickY - 5, klickX, klickY + 5)
            For Each vert As Vertex In SelectedVertices
                If Not Control.ModifierKeys = Keys.Control Then
                    vert.X = temp_center.X + factorToScale * vert.distanceFrom * Math.Cos(vert.angleFrom)
                Else
                    vert.X = vert.oldX
                End If
                If Not Control.ModifierKeys = Keys.Shift Then
                    vert.Y = temp_center.Y + factorToScale * vert.distanceFrom * Math.Sin(vert.angleFrom)
                Else
                    vert.Y = vert.oldY
                End If
                If Single.IsNaN(vert.X) OrElse Single.IsNaN(vert.Y) Then
                    vert.X = vert.oldX
                    vert.Y = vert.oldY
                End If
            Next
        End If

        ' Filled Triangles
        If BtnColours.Checked Then
            If alpha = 255 Then
                Dim pf(3) As PointF
                For i As Integer = 0 To Triangles.Count - 1
                    Dim tri As Triangle = Triangles(i)
                    If tri.myColourNumber <> 16 Then
                        Dim b As New SolidBrush(tri.myColour)
                        pf(0).X = absOffsetX - tri.vertexA.X * zoomfactor
                        pf(0).Y = absOffsetY + tri.vertexA.Y * zoomfactor
                        pf(1).X = absOffsetX - tri.vertexB.X * zoomfactor
                        pf(1).Y = absOffsetY + tri.vertexB.Y * zoomfactor
                        pf(2).X = absOffsetX - tri.vertexC.X * zoomfactor
                        pf(2).Y = absOffsetY + tri.vertexC.Y * zoomfactor
                        pf(3).X = absOffsetX - tri.vertexA.X * zoomfactor
                        pf(3).Y = absOffsetY + tri.vertexA.Y * zoomfactor
                        e.Graphics.FillPolygon(b, pf)
                    ElseIf tri.groupindex <> -1 AndAlso tri.myColourNumber = 16 AndAlso Primitives(PrimitivesHMap(tri.groupindex)).myColourNumber <> 16 Then
                        Dim b As New SolidBrush(Primitives(PrimitivesHMap(tri.groupindex)).myColour)
                        pf(0).X = absOffsetX - tri.vertexA.X * zoomfactor
                        pf(0).Y = absOffsetY + tri.vertexA.Y * zoomfactor
                        pf(1).X = absOffsetX - tri.vertexB.X * zoomfactor
                        pf(1).Y = absOffsetY + tri.vertexB.Y * zoomfactor
                        pf(2).X = absOffsetX - tri.vertexC.X * zoomfactor
                        pf(2).Y = absOffsetY + tri.vertexC.Y * zoomfactor
                        pf(3).X = absOffsetX - tri.vertexA.X * zoomfactor
                        pf(3).Y = absOffsetY + tri.vertexA.Y * zoomfactor
                        e.Graphics.FillPolygon(b, pf)
                    End If
                Next
            Else
                Dim pf(3) As PointF
                For i As Integer = 0 To Triangles.Count - 1
                    Dim tri As Triangle = Triangles(i)
                    If tri.myColourNumber <> 16 Then
                        Dim col As Color = tri.myColour
                        Dim b As New SolidBrush(Color.FromArgb(alpha, col.R, col.G, col.B))
                        pf(0).X = absOffsetX - tri.vertexA.X * zoomfactor
                        pf(0).Y = absOffsetY + tri.vertexA.Y * zoomfactor
                        pf(1).X = absOffsetX - tri.vertexB.X * zoomfactor
                        pf(1).Y = absOffsetY + tri.vertexB.Y * zoomfactor
                        pf(2).X = absOffsetX - tri.vertexC.X * zoomfactor
                        pf(2).Y = absOffsetY + tri.vertexC.Y * zoomfactor
                        pf(3).X = absOffsetX - tri.vertexA.X * zoomfactor
                        pf(3).Y = absOffsetY + tri.vertexA.Y * zoomfactor
                        e.Graphics.FillPolygon(b, pf)
                    ElseIf tri.groupindex <> -1 AndAlso tri.myColourNumber = 16 AndAlso Primitives(PrimitivesHMap(tri.groupindex)).myColourNumber <> 16 Then
                        Dim col As Color = Primitives(PrimitivesHMap(tri.groupindex)).myColour
                        Dim b As New SolidBrush(Color.FromArgb(alpha, col.R, col.G, col.B))
                        pf(0).X = absOffsetX - tri.vertexA.X * zoomfactor
                        pf(0).Y = absOffsetY + tri.vertexA.Y * zoomfactor
                        pf(1).X = absOffsetX - tri.vertexB.X * zoomfactor
                        pf(1).Y = absOffsetY + tri.vertexB.Y * zoomfactor
                        pf(2).X = absOffsetX - tri.vertexC.X * zoomfactor
                        pf(2).Y = absOffsetY + tri.vertexC.Y * zoomfactor
                        pf(3).X = absOffsetX - tri.vertexA.X * zoomfactor
                        pf(3).Y = absOffsetY + tri.vertexA.Y * zoomfactor
                        e.Graphics.FillPolygon(b, pf)
                    End If
                Next
            End If
        End If

        If Not BtnPreview.Checked Then
            ' Triangles
            Dim pf2(3) As PointF
            For i As Integer = 0 To Triangles.Count - 1
                Dim tri As Triangle = Triangles(i)
                If Not tri.selected Then
                    If tri.myColourNumber <> 15 OrElse Not BtnColours.Checked Then
                        pf2(0).X = absOffsetX - tri.vertexA.X * zoomfactor
                        pf2(0).Y = absOffsetY + tri.vertexA.Y * zoomfactor
                        pf2(1).X = absOffsetX - tri.vertexB.X * zoomfactor
                        pf2(1).Y = absOffsetY + tri.vertexB.Y * zoomfactor
                        pf2(2).X = absOffsetX - tri.vertexC.X * zoomfactor
                        pf2(2).Y = absOffsetY + tri.vertexC.Y * zoomfactor
                        pf2(3).X = absOffsetX - tri.vertexA.X * zoomfactor
                        pf2(3).Y = absOffsetY + tri.vertexA.Y * zoomfactor
                        e.Graphics.DrawPolygon(myColours.linePen, pf2)
                    Else
                        pf2(0).X = absOffsetX - tri.vertexA.X * zoomfactor
                        pf2(0).Y = absOffsetY + tri.vertexA.Y * zoomfactor
                        pf2(1).X = absOffsetX - tri.vertexB.X * zoomfactor
                        pf2(1).Y = absOffsetY + tri.vertexB.Y * zoomfactor
                        pf2(2).X = absOffsetX - tri.vertexC.X * zoomfactor
                        pf2(2).Y = absOffsetY + tri.vertexC.Y * zoomfactor
                        pf2(3).X = absOffsetX - tri.vertexA.X * zoomfactor
                        pf2(3).Y = absOffsetY + tri.vertexA.Y * zoomfactor
                        e.Graphics.DrawPolygon(myColours.inverseLinePen, pf2)
                    End If
                End If
            Next
            If objectToModify <> "Vertex" Then
                For i As Integer = 0 To Triangles.Count - 1
                    Dim tri As Triangle = Triangles(i)
                    If tri.selected Then
                        pf2(0).X = absOffsetX - tri.vertexA.X * zoomfactor
                        pf2(0).Y = absOffsetY + tri.vertexA.Y * zoomfactor
                        pf2(1).X = absOffsetX - tri.vertexB.X * zoomfactor
                        pf2(1).Y = absOffsetY + tri.vertexB.Y * zoomfactor
                        pf2(2).X = absOffsetX - tri.vertexC.X * zoomfactor
                        pf2(2).Y = absOffsetY + tri.vertexC.Y * zoomfactor
                        pf2(3).X = absOffsetX - tri.vertexA.X * zoomfactor
                        pf2(3).Y = absOffsetY + tri.vertexA.Y * zoomfactor
                        e.Graphics.DrawPolygon(myColours.selectedLinePen, pf2)
                    End If
                Next
            Else
                For i As Integer = 0 To Triangles.Count - 1
                    Dim tri As Triangle = Triangles(i)
                    If tri.selected Then
                        pf2(0).X = absOffsetX - tri.vertexA.X * zoomfactor
                        pf2(0).Y = absOffsetY + tri.vertexA.Y * zoomfactor
                        pf2(1).X = absOffsetX - tri.vertexB.X * zoomfactor
                        pf2(1).Y = absOffsetY + tri.vertexB.Y * zoomfactor
                        pf2(2).X = absOffsetX - tri.vertexC.X * zoomfactor
                        pf2(2).Y = absOffsetY + tri.vertexC.Y * zoomfactor
                        pf2(3).X = absOffsetX - tri.vertexA.X * zoomfactor
                        pf2(3).Y = absOffsetY + tri.vertexA.Y * zoomfactor
                        e.Graphics.DrawPolygon(myColours.selectedLineInVertexModePen, pf2)
                    End If
                Next
            End If

            If trianglemode > 0 Then
                e.Graphics.DrawLine(Pens.PowderBlue, absOffsetX - lastPointX * zoomfactor, absOffsetY + lastPointY * zoomfactor, getCursorpositionX(), getCursorpositionY)
                If trianglemode = 2 Then
                    e.Graphics.DrawLine(Pens.PowderBlue, absOffsetX - temp_vertices(0).X * zoomfactor, absOffsetY + temp_vertices(0).Y * zoomfactor, getCursorpositionX(), getCursorpositionY)
                    e.Graphics.DrawLine(Pens.PowderBlue, absOffsetX - temp_vertices(1).X * zoomfactor, absOffsetY + temp_vertices(1).Y * zoomfactor, absOffsetX - temp_vertices(0).X * zoomfactor, absOffsetY + temp_vertices(0).Y * zoomfactor)
                End If
            End If

            ' Vertices

            Dim vs As New List(Of RectangleF) With {.Capacity = SelectedVertices.Count}
            Dim v As New List(Of RectangleF) With {.Capacity = Vertices.Count}
            For i As Integer = 0 To Vertices.Count - 1
                Dim vert As Vertex = Vertices(i)
                If vert.selected AndAlso objectToModify = "Vertex" Then
                    vs.Add(New RectangleF(absOffsetX - vert.X * zoomfactor - (punktgroesse / 2), absOffsetY + vert.Y * zoomfactor - (punktgroesse / 2), punktgroesse, punktgroesse))
                Else
                    v.Add(New RectangleF(absOffsetX - vert.X * zoomfactor - (punktgroesse / 2), absOffsetY + vert.Y * zoomfactor - (punktgroesse / 2), punktgroesse, punktgroesse))
                End If
            Next
            If vs.Count > 0 Then e.Graphics.FillRectangles(myColours.selectedVertexBrush, vs.ToArray)
            If v.Count > 0 Then e.Graphics.FillRectangles(myColours.vertexBrush, v.ToArray)

            If movemode Then
                For Each vert As Vertex In SelectedVertices
                    vert.X += vabsOffsetX
                    vert.Y -= vabsOffsetY
                Next
            ElseIf rotatemode OrElse scalemode Then
                For Each vert As Vertex In SelectedVertices
                    vert.X = vert.oldX
                    vert.Y = vert.oldY
                Next
            End If
        End If

        ' Auswahl
        If auswahl AndAlso objectToModify <> "Primitive" AndAlso Not adjustmode Then
            Dim tX, tY, tW, tH As Integer
            tX = Math.Min(klickX, getCursorpositionX)
            tY = Math.Min(klickY, getCursorpositionY)
            tW = Math.Abs(getCursorpositionX() - klickX)
            tH = Math.Abs(getCursorpositionY() - klickY)
            e.Graphics.DrawRectangle(myColours.selectionRectPen, tX, tY, tW, tH)
        End If

        ' Add Vertex
        If BtnAddVertex.Checked Then
            Dim v1 As New Vertex(Math.Round(getXcoordinate(getCursorpositionX()) / Me.moveSnap) * Me.moveSnap, Math.Round(getYcoordinate(getCursorpositionY()) / Me.moveSnap) * Me.moveSnap, False, False)
            e.Graphics.FillRectangle(myColours.selectedVertexBrush, New RectangleF(absOffsetX - v1.X * zoomfactor - (punktgroesse / 2), absOffsetY + v1.Y * zoomfactor - (punktgroesse / 2), punktgroesse, punktgroesse))
        End If

        ' Primitive Modes
        If primitiveMode > 0 Then
            Select Case primitiveMode
                Case 1
                    e.Graphics.DrawString("Set primitive origin", New Font("Arial", 16, FontStyle.Regular, GraphicsUnit.Pixel), Brushes.Black, 12, 32)
                    e.Graphics.DrawString("Set primitive origin", New Font("Arial", 16, FontStyle.Regular, GraphicsUnit.Pixel), Brushes.Azure, 10, 30)
                    primitiveCenter = New Vertex(Math.Round(getXcoordinate(getCursorpositionX()) / Me.moveSnap) * Me.moveSnap, Math.Round(getYcoordinate(getCursorpositionY()) / Me.moveSnap) * Me.moveSnap, False, False)
                    e.Graphics.FillRectangle(myColours.selectionCrossBrush, New RectangleF(absOffsetX - primitiveCenter.X * zoomfactor - 1, absOffsetY + primitiveCenter.Y * zoomfactor - 16, 3, 33))
                    e.Graphics.FillRectangle(myColours.selectionCrossBrush, New RectangleF(absOffsetX - primitiveCenter.X * zoomfactor - 16, absOffsetY + primitiveCenter.Y * zoomfactor - 1, 33, 3))
                    Exit Select
                Case 2
                    e.Graphics.DrawString("Set primitive height or size", New Font("Arial", 16, FontStyle.Regular, GraphicsUnit.Pixel), Brushes.Black, 12, 32)
                    e.Graphics.DrawString("Set primitive height or size", New Font("Arial", 16, FontStyle.Regular, GraphicsUnit.Pixel), Brushes.Azure, 10, 30)
                    e.Graphics.FillRectangle(myColours.selectionCrossBrush, New RectangleF(absOffsetX - primitiveCenter.X * zoomfactor - 1, absOffsetY + primitiveCenter.Y * zoomfactor - 16, 2, 32))
                    e.Graphics.FillRectangle(myColours.selectionCrossBrush, New RectangleF(absOffsetX - primitiveCenter.X * zoomfactor - 16, absOffsetY + primitiveCenter.Y * zoomfactor - 1, 32, 2))
                    primitiveHeight = Fix(getYcoordinate(getCursorpositionY()) - primitiveCenter.Y)
                    Select Case primitiveObject
                        Case Is < 3
                            Dim v1, v2, v3 As Vertex
                            v1 = New Vertex(primitiveCenter.X, primitiveCenter.Y + primitiveHeight, False, False)
                            v2 = New Vertex(primitiveCenter.X + primitiveHeight, primitiveCenter.Y - (primitiveHeight / 2), False, False)
                            v3 = New Vertex(primitiveCenter.X - primitiveHeight, primitiveCenter.Y - (primitiveHeight / 2), False, False)
                            e.Graphics.FillRectangle(myColours.selectedVertexBrush, New RectangleF(absOffsetX - v1.X * zoomfactor - (punktgroesse / 2), absOffsetY + v1.Y * zoomfactor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.FillRectangle(myColours.selectedVertexBrush, New RectangleF(absOffsetX - v2.X * zoomfactor - (punktgroesse / 2), absOffsetY + v2.Y * zoomfactor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.FillRectangle(myColours.selectedVertexBrush, New RectangleF(absOffsetX - v3.X * zoomfactor - (punktgroesse / 2), absOffsetY + v3.Y * zoomfactor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - v1.X * zoomfactor, absOffsetY + v1.Y * zoomfactor, absOffsetX - v2.X * zoomfactor, absOffsetY + v2.Y * zoomfactor)
                            e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - v2.X * zoomfactor, absOffsetY + v2.Y * zoomfactor, absOffsetX - v3.X * zoomfactor, absOffsetY + v3.Y * zoomfactor)
                            e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - v3.X * zoomfactor, absOffsetY + v3.Y * zoomfactor, absOffsetX - v1.X * zoomfactor, absOffsetY + v1.Y * zoomfactor)
                            Exit Select
                        Case Is < 6
                            Dim v1, v2, v3, v4 As Vertex
                            v1 = New Vertex(primitiveCenter.X + primitiveHeight, primitiveCenter.Y + primitiveHeight, False, False)
                            v2 = New Vertex(primitiveCenter.X - primitiveHeight, primitiveCenter.Y + primitiveHeight, False, False)
                            v3 = New Vertex(primitiveCenter.X - primitiveHeight, primitiveCenter.Y - primitiveHeight, False, False)
                            v4 = New Vertex(primitiveCenter.X + primitiveHeight, primitiveCenter.Y - primitiveHeight, False, False)
                            e.Graphics.FillRectangle(myColours.selectedVertexBrush, New RectangleF(absOffsetX - v1.X * zoomfactor - (punktgroesse / 2), absOffsetY + v1.Y * zoomfactor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.FillRectangle(myColours.selectedVertexBrush, New RectangleF(absOffsetX - v2.X * zoomfactor - (punktgroesse / 2), absOffsetY + v2.Y * zoomfactor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.FillRectangle(myColours.selectedVertexBrush, New RectangleF(absOffsetX - v3.X * zoomfactor - (punktgroesse / 2), absOffsetY + v3.Y * zoomfactor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.FillRectangle(myColours.selectedVertexBrush, New RectangleF(absOffsetX - v4.X * zoomfactor - (punktgroesse / 2), absOffsetY + v4.Y * zoomfactor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - v1.X * zoomfactor, absOffsetY + v1.Y * zoomfactor, absOffsetX - v2.X * zoomfactor, absOffsetY + v2.Y * zoomfactor)
                            e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - v2.X * zoomfactor, absOffsetY + v2.Y * zoomfactor, absOffsetX - v3.X * zoomfactor, absOffsetY + v3.Y * zoomfactor)
                            e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - v3.X * zoomfactor, absOffsetY + v3.Y * zoomfactor, absOffsetX - v4.X * zoomfactor, absOffsetY + v4.Y * zoomfactor)
                            e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - v4.X * zoomfactor, absOffsetY + v4.Y * zoomfactor, absOffsetX - v1.X * zoomfactor, absOffsetY + v1.Y * zoomfactor)
                            e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - v1.X * zoomfactor, absOffsetY + v1.Y * zoomfactor, absOffsetX - v3.X * zoomfactor, absOffsetY + v3.Y * zoomfactor)
                            Exit Select
                        Case Is < 9
                            primitiveHeight = Fix(Math.Sqrt((getYcoordinate(getCursorpositionY()) - primitiveCenter.Y) ^ 2 + (getXcoordinate(getCursorpositionX()) - primitiveCenter.X) ^ 2))
                            Dim vcenter As Vertex
                            Dim vouter(segments_circle) As Vertex
                            Dim counter As Integer = 0
                            vcenter = New Vertex(primitiveCenter.X, primitiveCenter.Y, False, False)
                            For a As Double = 0 To Math.PI * 2 Step Math.PI * 2 / segments_circle
                                vouter(counter) = New Vertex(primitiveCenter.X + Math.Cos(a) * primitiveHeight, primitiveCenter.Y + Math.Sin(a) * primitiveHeight, False, False)
                                counter += 1
                            Next
                            e.Graphics.FillRectangle(myColours.selectedVertexBrush, New RectangleF(absOffsetX - vouter(0).X * zoomfactor - (punktgroesse / 2), absOffsetY + vouter(0).Y * zoomfactor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - vouter(0).X * zoomfactor, absOffsetY + vouter(0).Y * zoomfactor, absOffsetX - vouter(segments_circle - 1).X * zoomfactor, absOffsetY + vouter(segments_circle - 1).Y * zoomfactor)
                            e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - vouter(0).X * zoomfactor, absOffsetY + vouter(0).Y * zoomfactor, absOffsetX - vcenter.X * zoomfactor, absOffsetY + vcenter.Y * zoomfactor)
                            For counter = 1 To segments_circle - 1
                                e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - vouter(counter).X * zoomfactor, absOffsetY + vouter(counter).Y * zoomfactor, absOffsetX - vouter(counter - 1).X * zoomfactor, absOffsetY + vouter(counter - 1).Y * zoomfactor)
                                e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - vouter(counter).X * zoomfactor, absOffsetY + vouter(counter).Y * zoomfactor, absOffsetX - vcenter.X * zoomfactor, absOffsetY + vcenter.Y * zoomfactor)
                                e.Graphics.FillRectangle(myColours.selectedVertexBrush, New RectangleF(absOffsetX - vouter(counter).X * zoomfactor - (punktgroesse / 2), absOffsetY + vouter(counter).Y * zoomfactor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            Next
                            e.Graphics.FillRectangle(myColours.selectedVertexBrush, New RectangleF(absOffsetX - vcenter.X * zoomfactor - (punktgroesse / 2), absOffsetY + vcenter.Y * zoomfactor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            Exit Select
                        Case Is < 12
                            primitiveHeight = Fix(Math.Sqrt((getYcoordinate(getCursorpositionY()) - primitiveCenter.Y) ^ 2 + (getXcoordinate(getCursorpositionX()) - primitiveCenter.X) ^ 2))
                            Dim vcenter As Vertex
                            Dim vouter(segments_oval) As Vertex
                            Dim counter As Integer = 0
                            vcenter = New Vertex(primitiveCenter.X, primitiveCenter.Y, False, False)
                            For a As Double = 0 To Math.PI * 2 Step Math.PI * 2 / segments_oval
                                vouter(counter) = New Vertex(primitiveCenter.X + Math.Cos(a) * primitiveHeight, primitiveCenter.Y + Math.Sin(a) * primitiveHeight, False, False)
                                counter += 1
                            Next
                            e.Graphics.FillRectangle(myColours.selectedVertexBrush, New RectangleF(absOffsetX - vouter(0).X * zoomfactor - (punktgroesse / 2), absOffsetY + vouter(0).Y * zoomfactor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - vouter(0).X * zoomfactor, absOffsetY + vouter(0).Y * zoomfactor, absOffsetX - vouter(segments_oval - 1).X * zoomfactor, absOffsetY + vouter(segments_oval - 1).Y * zoomfactor)
                            e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - vouter(0).X * zoomfactor, absOffsetY + vouter(0).Y * zoomfactor, absOffsetX - vcenter.X * zoomfactor, absOffsetY + vcenter.Y * zoomfactor)
                            For counter = 1 To segments_oval - 1
                                e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - vouter(counter).X * zoomfactor, absOffsetY + vouter(counter).Y * zoomfactor, absOffsetX - vouter(counter - 1).X * zoomfactor, absOffsetY + vouter(counter - 1).Y * zoomfactor)
                                e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - vouter(counter).X * zoomfactor, absOffsetY + vouter(counter).Y * zoomfactor, absOffsetX - vcenter.X * zoomfactor, absOffsetY + vcenter.Y * zoomfactor)
                                e.Graphics.FillRectangle(myColours.selectedVertexBrush, New RectangleF(absOffsetX - vouter(counter).X * zoomfactor - (punktgroesse / 2), absOffsetY + vouter(counter).Y * zoomfactor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            Next
                            e.Graphics.FillRectangle(myColours.selectedVertexBrush, New RectangleF(absOffsetX - vcenter.X * zoomfactor - (punktgroesse / 2), absOffsetY + vcenter.Y * zoomfactor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            Exit Select
                    End Select
                    Exit Select
                Case 3
                    e.Graphics.DrawString("Set primitive width", New Font("Arial", 16, FontStyle.Regular, GraphicsUnit.Pixel), Brushes.Black, 12, 32)
                    e.Graphics.DrawString("Set primitive width", New Font("Arial", 16, FontStyle.Regular, GraphicsUnit.Pixel), Brushes.Azure, 10, 30)
                    e.Graphics.FillRectangle(myColours.selectionCrossBrush, New RectangleF(absOffsetX - primitiveCenter.X * zoomfactor - 1, absOffsetY + primitiveCenter.Y * zoomfactor - 16, 2, 32))
                    e.Graphics.FillRectangle(myColours.selectionCrossBrush, New RectangleF(absOffsetX - primitiveCenter.X * zoomfactor - 16, absOffsetY + primitiveCenter.Y * zoomfactor - 1, 32, 2))
                    primitiveWidth = Fix(getXcoordinate(getCursorpositionX()) - primitiveCenter.X)
                    Select Case primitiveObject
                        Case Is < 3
                            Dim v1, v2, v3 As Vertex
                            v1 = New Vertex(primitiveCenter.X, primitiveCenter.Y + primitiveHeight, False, False)
                            v2 = New Vertex(primitiveCenter.X + primitiveHeight + primitiveWidth, primitiveCenter.Y - (primitiveHeight / 2), False, False)
                            v3 = New Vertex(primitiveCenter.X - primitiveHeight - primitiveWidth, primitiveCenter.Y - (primitiveHeight / 2), False, False)
                            e.Graphics.FillRectangle(myColours.selectedVertexBrush, New RectangleF(absOffsetX - v1.X * zoomfactor - (punktgroesse / 2), absOffsetY + v1.Y * zoomfactor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.FillRectangle(myColours.selectedVertexBrush, New RectangleF(absOffsetX - v2.X * zoomfactor - (punktgroesse / 2), absOffsetY + v2.Y * zoomfactor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.FillRectangle(myColours.selectedVertexBrush, New RectangleF(absOffsetX - v3.X * zoomfactor - (punktgroesse / 2), absOffsetY + v3.Y * zoomfactor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - v1.X * zoomfactor, absOffsetY + v1.Y * zoomfactor, absOffsetX - v2.X * zoomfactor, absOffsetY + v2.Y * zoomfactor)
                            e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - v2.X * zoomfactor, absOffsetY + v2.Y * zoomfactor, absOffsetX - v3.X * zoomfactor, absOffsetY + v3.Y * zoomfactor)
                            e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - v3.X * zoomfactor, absOffsetY + v3.Y * zoomfactor, absOffsetX - v1.X * zoomfactor, absOffsetY + v1.Y * zoomfactor)
                            Exit Select
                        Case Is < 6
                            Dim v1, v2, v3, v4 As Vertex
                            v1 = New Vertex(primitiveCenter.X + primitiveWidth, primitiveCenter.Y + primitiveHeight, False, False)
                            v2 = New Vertex(primitiveCenter.X - primitiveWidth, primitiveCenter.Y + primitiveHeight, False, False)
                            v3 = New Vertex(primitiveCenter.X - primitiveWidth, primitiveCenter.Y - primitiveHeight, False, False)
                            v4 = New Vertex(primitiveCenter.X + primitiveWidth, primitiveCenter.Y - primitiveHeight, False, False)
                            e.Graphics.FillRectangle(myColours.selectedVertexBrush, New RectangleF(absOffsetX - v1.X * zoomfactor - (punktgroesse / 2), absOffsetY + v1.Y * zoomfactor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.FillRectangle(myColours.selectedVertexBrush, New RectangleF(absOffsetX - v2.X * zoomfactor - (punktgroesse / 2), absOffsetY + v2.Y * zoomfactor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.FillRectangle(myColours.selectedVertexBrush, New RectangleF(absOffsetX - v3.X * zoomfactor - (punktgroesse / 2), absOffsetY + v3.Y * zoomfactor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.FillRectangle(myColours.selectedVertexBrush, New RectangleF(absOffsetX - v4.X * zoomfactor - (punktgroesse / 2), absOffsetY + v4.Y * zoomfactor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - v1.X * zoomfactor, absOffsetY + v1.Y * zoomfactor, absOffsetX - v2.X * zoomfactor, absOffsetY + v2.Y * zoomfactor)
                            e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - v2.X * zoomfactor, absOffsetY + v2.Y * zoomfactor, absOffsetX - v3.X * zoomfactor, absOffsetY + v3.Y * zoomfactor)
                            e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - v3.X * zoomfactor, absOffsetY + v3.Y * zoomfactor, absOffsetX - v4.X * zoomfactor, absOffsetY + v4.Y * zoomfactor)
                            e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - v4.X * zoomfactor, absOffsetY + v4.Y * zoomfactor, absOffsetX - v1.X * zoomfactor, absOffsetY + v1.Y * zoomfactor)
                            e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - v1.X * zoomfactor, absOffsetY + v1.Y * zoomfactor, absOffsetX - v3.X * zoomfactor, absOffsetY + v3.Y * zoomfactor)
                            Exit Select
                        Case Is < 12
                            primitiveWidth = Fix(Math.Sqrt((getYcoordinate(getCursorpositionY()) - primitiveCenter.Y) ^ 2 + (getXcoordinate(getCursorpositionX()) - primitiveCenter.X) ^ 2))
                            Dim vcenter As Vertex
                            Dim vouter(segments_oval) As Vertex
                            Dim counter As Integer = 0
                            vcenter = New Vertex(primitiveCenter.X, primitiveCenter.Y, False, False)
                            For a As Double = 0 To Math.PI * 2 Step Math.PI * 2 / segments_oval
                                vouter(counter) = New Vertex(primitiveCenter.X + Math.Cos(a) * primitiveWidth, primitiveCenter.Y + Math.Sin(a) * primitiveHeight, False, False)
                                counter += 1
                            Next
                            e.Graphics.FillRectangle(myColours.selectedVertexBrush, New RectangleF(absOffsetX - vouter(0).X * zoomfactor - (punktgroesse / 2), absOffsetY + vouter(0).Y * zoomfactor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - vouter(0).X * zoomfactor, absOffsetY + vouter(0).Y * zoomfactor, absOffsetX - vouter(segments_oval - 1).X * zoomfactor, absOffsetY + vouter(segments_oval - 1).Y * zoomfactor)
                            e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - vouter(0).X * zoomfactor, absOffsetY + vouter(0).Y * zoomfactor, absOffsetX - vcenter.X * zoomfactor, absOffsetY + vcenter.Y * zoomfactor)
                            For counter = 1 To segments_oval - 1
                                e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - vouter(counter).X * zoomfactor, absOffsetY + vouter(counter).Y * zoomfactor, absOffsetX - vouter(counter - 1).X * zoomfactor, absOffsetY + vouter(counter - 1).Y * zoomfactor)
                                e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - vouter(counter).X * zoomfactor, absOffsetY + vouter(counter).Y * zoomfactor, absOffsetX - vcenter.X * zoomfactor, absOffsetY + vcenter.Y * zoomfactor)
                                e.Graphics.FillRectangle(myColours.selectedVertexBrush, New RectangleF(absOffsetX - vouter(counter).X * zoomfactor - (punktgroesse / 2), absOffsetY + vouter(counter).Y * zoomfactor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            Next
                            e.Graphics.FillRectangle(myColours.selectedVertexBrush, New RectangleF(absOffsetX - vcenter.X * zoomfactor - (punktgroesse / 2), absOffsetY + vcenter.Y * zoomfactor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            Exit Select
                    End Select
                    Exit Select
                Case 4
                    e.Graphics.DrawString("Set primitive frame size", New Font("Arial", 16, FontStyle.Regular, GraphicsUnit.Pixel), Brushes.Black, 12, 32)
                    e.Graphics.DrawString("Set primitive frame size", New Font("Arial", 16, FontStyle.Regular, GraphicsUnit.Pixel), Brushes.Azure, 10, 30)
                    e.Graphics.FillRectangle(myColours.selectionCrossBrush, New RectangleF(absOffsetX - primitiveCenter.X * zoomfactor - 1, absOffsetY + primitiveCenter.Y * zoomfactor - 16, 2, 32))
                    e.Graphics.FillRectangle(myColours.selectionCrossBrush, New RectangleF(absOffsetX - primitiveCenter.X * zoomfactor - 16, absOffsetY + primitiveCenter.Y * zoomfactor - 1, 32, 2))
                    primitiveBordersize = Fix(getXcoordinate(getCursorpositionX()) - primitiveCenter.X) / Math.Sqrt(primitiveWidth ^ 2 + primitiveHeight ^ 2)
                    If primitiveBordersize < 0 Then primitiveBordersize *= -1
                    clip(primitiveBordersize, 0, 1)
                    Select Case primitiveObject
                        Case Is < 3
                            Dim v11, v21, v31 As Vertex
                            v11 = New Vertex(primitiveCenter.X, primitiveCenter.Y + primitiveHeight, False, False)
                            v21 = New Vertex(primitiveCenter.X + primitiveHeight + primitiveWidth, primitiveCenter.Y - (primitiveHeight / 2), False, False)
                            v31 = New Vertex(primitiveCenter.X - primitiveHeight - primitiveWidth, primitiveCenter.Y - (primitiveHeight / 2), False, False)
                            e.Graphics.FillRectangle(myColours.selectedVertexBrush, New RectangleF(absOffsetX - v11.X * zoomfactor - (punktgroesse / 2), absOffsetY + v11.Y * zoomfactor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.FillRectangle(myColours.selectedVertexBrush, New RectangleF(absOffsetX - v21.X * zoomfactor - (punktgroesse / 2), absOffsetY + v21.Y * zoomfactor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.FillRectangle(myColours.selectedVertexBrush, New RectangleF(absOffsetX - v31.X * zoomfactor - (punktgroesse / 2), absOffsetY + v31.Y * zoomfactor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - v11.X * zoomfactor, absOffsetY + v11.Y * zoomfactor, absOffsetX - v21.X * zoomfactor, absOffsetY + v21.Y * zoomfactor)
                            e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - v21.X * zoomfactor, absOffsetY + v21.Y * zoomfactor, absOffsetX - v31.X * zoomfactor, absOffsetY + v31.Y * zoomfactor)
                            e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - v31.X * zoomfactor, absOffsetY + v31.Y * zoomfactor, absOffsetX - v11.X * zoomfactor, absOffsetY + v11.Y * zoomfactor)
                            Dim v12, v22, v32 As Vertex
                            v12 = New Vertex(primitiveCenter.X, primitiveCenter.Y + primitiveHeight * primitiveBordersize, False, False)
                            v22 = New Vertex(primitiveCenter.X + (primitiveHeight + primitiveWidth) * primitiveBordersize, primitiveCenter.Y - (primitiveHeight / 2) * primitiveBordersize, False, False)
                            v32 = New Vertex(primitiveCenter.X - (primitiveHeight + primitiveWidth) * primitiveBordersize, primitiveCenter.Y - (primitiveHeight / 2) * primitiveBordersize, False, False)
                            e.Graphics.FillRectangle(myColours.selectedVertexBrush, New RectangleF(absOffsetX - v12.X * zoomfactor - (punktgroesse / 2), absOffsetY + v12.Y * zoomfactor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.FillRectangle(myColours.selectedVertexBrush, New RectangleF(absOffsetX - v22.X * zoomfactor - (punktgroesse / 2), absOffsetY + v22.Y * zoomfactor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.FillRectangle(myColours.selectedVertexBrush, New RectangleF(absOffsetX - v32.X * zoomfactor - (punktgroesse / 2), absOffsetY + v32.Y * zoomfactor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - v12.X * zoomfactor, absOffsetY + v12.Y * zoomfactor, absOffsetX - v22.X * zoomfactor, absOffsetY + v22.Y * zoomfactor)
                            e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - v22.X * zoomfactor, absOffsetY + v22.Y * zoomfactor, absOffsetX - v32.X * zoomfactor, absOffsetY + v32.Y * zoomfactor)
                            e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - v32.X * zoomfactor, absOffsetY + v32.Y * zoomfactor, absOffsetX - v12.X * zoomfactor, absOffsetY + v12.Y * zoomfactor)
                            e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - v11.X * zoomfactor, absOffsetY + v11.Y * zoomfactor, absOffsetX - v12.X * zoomfactor, absOffsetY + v12.Y * zoomfactor)
                            e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - v21.X * zoomfactor, absOffsetY + v21.Y * zoomfactor, absOffsetX - v22.X * zoomfactor, absOffsetY + v22.Y * zoomfactor)
                            e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - v31.X * zoomfactor, absOffsetY + v31.Y * zoomfactor, absOffsetX - v32.X * zoomfactor, absOffsetY + v32.Y * zoomfactor)
                            e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - v11.X * zoomfactor, absOffsetY + v11.Y * zoomfactor, absOffsetX - v22.X * zoomfactor, absOffsetY + v22.Y * zoomfactor)
                            e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - v21.X * zoomfactor, absOffsetY + v21.Y * zoomfactor, absOffsetX - v32.X * zoomfactor, absOffsetY + v32.Y * zoomfactor)
                            e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - v31.X * zoomfactor, absOffsetY + v31.Y * zoomfactor, absOffsetX - v12.X * zoomfactor, absOffsetY + v12.Y * zoomfactor)
                            Exit Select
                        Case Is < 6
                            Dim v1, v2, v3, v4, v5, v6, v7, v8 As Vertex
                            v1 = New Vertex(primitiveCenter.X + primitiveWidth, primitiveCenter.Y + primitiveHeight, False, False)
                            v2 = New Vertex(primitiveCenter.X - primitiveWidth, primitiveCenter.Y + primitiveHeight, False, False)
                            v3 = New Vertex(primitiveCenter.X - primitiveWidth, primitiveCenter.Y - primitiveHeight, False, False)
                            v4 = New Vertex(primitiveCenter.X + primitiveWidth, primitiveCenter.Y - primitiveHeight, False, False)
                            v5 = New Vertex(primitiveCenter.X + primitiveWidth * primitiveBordersize, primitiveCenter.Y + primitiveHeight * primitiveBordersize, False, False)
                            v6 = New Vertex(primitiveCenter.X - primitiveWidth * primitiveBordersize, primitiveCenter.Y + primitiveHeight * primitiveBordersize, False, False)
                            v7 = New Vertex(primitiveCenter.X - primitiveWidth * primitiveBordersize, primitiveCenter.Y - primitiveHeight * primitiveBordersize, False, False)
                            v8 = New Vertex(primitiveCenter.X + primitiveWidth * primitiveBordersize, primitiveCenter.Y - primitiveHeight * primitiveBordersize, False, False)
                            e.Graphics.FillRectangle(myColours.selectedVertexBrush, New RectangleF(absOffsetX - v1.X * zoomfactor - (punktgroesse / 2), absOffsetY + v1.Y * zoomfactor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.FillRectangle(myColours.selectedVertexBrush, New RectangleF(absOffsetX - v2.X * zoomfactor - (punktgroesse / 2), absOffsetY + v2.Y * zoomfactor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.FillRectangle(myColours.selectedVertexBrush, New RectangleF(absOffsetX - v3.X * zoomfactor - (punktgroesse / 2), absOffsetY + v3.Y * zoomfactor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.FillRectangle(myColours.selectedVertexBrush, New RectangleF(absOffsetX - v4.X * zoomfactor - (punktgroesse / 2), absOffsetY + v4.Y * zoomfactor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.FillRectangle(myColours.selectedVertexBrush, New RectangleF(absOffsetX - v5.X * zoomfactor - (punktgroesse / 2), absOffsetY + v5.Y * zoomfactor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.FillRectangle(myColours.selectedVertexBrush, New RectangleF(absOffsetX - v6.X * zoomfactor - (punktgroesse / 2), absOffsetY + v6.Y * zoomfactor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.FillRectangle(myColours.selectedVertexBrush, New RectangleF(absOffsetX - v7.X * zoomfactor - (punktgroesse / 2), absOffsetY + v7.Y * zoomfactor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.FillRectangle(myColours.selectedVertexBrush, New RectangleF(absOffsetX - v8.X * zoomfactor - (punktgroesse / 2), absOffsetY + v8.Y * zoomfactor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            ' Kontur: Rechteck aussen
                            e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - v1.X * zoomfactor, absOffsetY + v1.Y * zoomfactor, absOffsetX - v2.X * zoomfactor, absOffsetY + v2.Y * zoomfactor)
                            e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - v2.X * zoomfactor, absOffsetY + v2.Y * zoomfactor, absOffsetX - v3.X * zoomfactor, absOffsetY + v3.Y * zoomfactor)
                            e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - v3.X * zoomfactor, absOffsetY + v3.Y * zoomfactor, absOffsetX - v4.X * zoomfactor, absOffsetY + v4.Y * zoomfactor)
                            e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - v4.X * zoomfactor, absOffsetY + v4.Y * zoomfactor, absOffsetX - v1.X * zoomfactor, absOffsetY + v1.Y * zoomfactor)
                            ' Kontur: Rechteck innen
                            e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - v5.X * zoomfactor, absOffsetY + v5.Y * zoomfactor, absOffsetX - v6.X * zoomfactor, absOffsetY + v6.Y * zoomfactor)
                            e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - v6.X * zoomfactor, absOffsetY + v6.Y * zoomfactor, absOffsetX - v7.X * zoomfactor, absOffsetY + v7.Y * zoomfactor)
                            e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - v7.X * zoomfactor, absOffsetY + v7.Y * zoomfactor, absOffsetX - v8.X * zoomfactor, absOffsetY + v8.Y * zoomfactor)
                            e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - v8.X * zoomfactor, absOffsetY + v8.Y * zoomfactor, absOffsetX - v5.X * zoomfactor, absOffsetY + v5.Y * zoomfactor)
                            ' Diagonale: Innen
                            If primitiveObject = 4 Then e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - v5.X * zoomfactor, absOffsetY + v5.Y * zoomfactor, absOffsetX - v7.X * zoomfactor, absOffsetY + v7.Y * zoomfactor)
                            ' Verbindungslinien: Aussen/Innen
                            e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - v1.X * zoomfactor, absOffsetY + v1.Y * zoomfactor, absOffsetX - v5.X * zoomfactor, absOffsetY + v5.Y * zoomfactor)
                            e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - v2.X * zoomfactor, absOffsetY + v2.Y * zoomfactor, absOffsetX - v6.X * zoomfactor, absOffsetY + v6.Y * zoomfactor)
                            e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - v3.X * zoomfactor, absOffsetY + v3.Y * zoomfactor, absOffsetX - v7.X * zoomfactor, absOffsetY + v7.Y * zoomfactor)
                            e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - v4.X * zoomfactor, absOffsetY + v4.Y * zoomfactor, absOffsetX - v8.X * zoomfactor, absOffsetY + v8.Y * zoomfactor)
                            ' Verbindungslinien: Diagonal
                            e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - v1.X * zoomfactor, absOffsetY + v1.Y * zoomfactor, absOffsetX - v6.X * zoomfactor, absOffsetY + v6.Y * zoomfactor)
                            e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - v2.X * zoomfactor, absOffsetY + v2.Y * zoomfactor, absOffsetX - v7.X * zoomfactor, absOffsetY + v7.Y * zoomfactor)
                            e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - v3.X * zoomfactor, absOffsetY + v3.Y * zoomfactor, absOffsetX - v8.X * zoomfactor, absOffsetY + v8.Y * zoomfactor)
                            e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - v4.X * zoomfactor, absOffsetY + v4.Y * zoomfactor, absOffsetX - v5.X * zoomfactor, absOffsetY + v5.Y * zoomfactor)
                            Exit Select
                        Case Is < 9
                            Dim vcenter As Vertex
                            Dim vinner(segments_circle) As Vertex
                            Dim vouter(segments_circle) As Vertex
                            Dim counter As Integer = 0
                            vcenter = New Vertex(primitiveCenter.X, primitiveCenter.Y, False, False)
                            For a As Double = 0 To Math.PI * 2 Step Math.PI * 2 / segments_circle
                                vinner(counter) = New Vertex(primitiveCenter.X + Math.Cos(a) * primitiveHeight * primitiveBordersize, primitiveCenter.Y + Math.Sin(a) * primitiveHeight * primitiveBordersize, False, False)
                                vouter(counter) = New Vertex(primitiveCenter.X + Math.Cos(a) * primitiveHeight, primitiveCenter.Y + Math.Sin(a) * primitiveHeight, False, False)
                                counter += 1
                            Next
                            e.Graphics.FillRectangle(myColours.selectedVertexBrush, New RectangleF(absOffsetX - vouter(0).X * zoomfactor - (punktgroesse / 2), absOffsetY + vouter(0).Y * zoomfactor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.FillRectangle(myColours.selectedVertexBrush, New RectangleF(absOffsetX - vinner(0).X * zoomfactor - (punktgroesse / 2), absOffsetY + vinner(0).Y * zoomfactor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - vouter(0).X * zoomfactor, absOffsetY + vouter(0).Y * zoomfactor, absOffsetX - vouter(segments_circle - 1).X * zoomfactor, absOffsetY + vouter(segments_circle - 1).Y * zoomfactor)
                            e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - vinner(0).X * zoomfactor, absOffsetY + vinner(0).Y * zoomfactor, absOffsetX - vinner(segments_circle - 1).X * zoomfactor, absOffsetY + vinner(segments_circle - 1).Y * zoomfactor)
                            e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - vinner(0).X * zoomfactor, absOffsetY + vinner(0).Y * zoomfactor, absOffsetX - vouter(segments_circle - 1).X * zoomfactor, absOffsetY + vouter(segments_circle - 1).Y * zoomfactor)
                            e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - vinner(segments_circle - 1).X * zoomfactor, absOffsetY + vinner(segments_circle - 1).Y * zoomfactor, absOffsetX - vouter(segments_circle - 1).X * zoomfactor, absOffsetY + vouter(segments_circle - 1).Y * zoomfactor)
                            If primitiveObject = 7 Then e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - vinner(0).X * zoomfactor, absOffsetY + vinner(0).Y * zoomfactor, absOffsetX - vcenter.X * zoomfactor, absOffsetY + vcenter.Y * zoomfactor)
                            For counter = 1 To segments_circle - 1
                                e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - vouter(counter).X * zoomfactor, absOffsetY + vouter(counter).Y * zoomfactor, absOffsetX - vouter(counter - 1).X * zoomfactor, absOffsetY + vouter(counter - 1).Y * zoomfactor)
                                e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - vinner(counter).X * zoomfactor, absOffsetY + vinner(counter).Y * zoomfactor, absOffsetX - vinner(counter - 1).X * zoomfactor, absOffsetY + vinner(counter - 1).Y * zoomfactor)
                                e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - vinner(counter).X * zoomfactor, absOffsetY + vinner(counter).Y * zoomfactor, absOffsetX - vouter(counter - 1).X * zoomfactor, absOffsetY + vouter(counter - 1).Y * zoomfactor)
                                e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - vinner(counter - 1).X * zoomfactor, absOffsetY + vinner(counter - 1).Y * zoomfactor, absOffsetX - vouter(counter - 1).X * zoomfactor, absOffsetY + vouter(counter - 1).Y * zoomfactor)
                                If primitiveObject = 7 Then e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - vinner(counter).X * zoomfactor, absOffsetY + vinner(counter).Y * zoomfactor, absOffsetX - vcenter.X * zoomfactor, absOffsetY + vcenter.Y * zoomfactor)
                                e.Graphics.FillRectangle(myColours.selectedVertexBrush, New RectangleF(absOffsetX - vouter(counter).X * zoomfactor - (punktgroesse / 2), absOffsetY + vouter(counter).Y * zoomfactor - (punktgroesse / 2), punktgroesse, punktgroesse))
                                e.Graphics.FillRectangle(myColours.selectedVertexBrush, New RectangleF(absOffsetX - vinner(counter).X * zoomfactor - (punktgroesse / 2), absOffsetY + vinner(counter).Y * zoomfactor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            Next
                            e.Graphics.FillRectangle(myColours.selectedVertexBrush, New RectangleF(absOffsetX - vcenter.X * zoomfactor - (punktgroesse / 2), absOffsetY + vcenter.Y * zoomfactor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            Exit Select
                        Case Is < 12
                            Dim vcenter As Vertex
                            Dim vinner(segments_oval) As Vertex
                            Dim vouter(segments_oval) As Vertex
                            Dim counter As Integer = 0
                            vcenter = New Vertex(primitiveCenter.X, primitiveCenter.Y, False, False)
                            For a As Double = 0 To Math.PI * 2 Step Math.PI * 2 / segments_oval
                                vinner(counter) = New Vertex(primitiveCenter.X + Math.Cos(a) * primitiveWidth * primitiveBordersize, primitiveCenter.Y + Math.Sin(a) * primitiveHeight * primitiveBordersize, False, False)
                                vouter(counter) = New Vertex(primitiveCenter.X + Math.Cos(a) * primitiveWidth, primitiveCenter.Y + Math.Sin(a) * primitiveHeight, False, False)
                                counter += 1
                            Next
                            e.Graphics.FillRectangle(myColours.selectedVertexBrush, New RectangleF(absOffsetX - vouter(0).X * zoomfactor - (punktgroesse / 2), absOffsetY + vouter(0).Y * zoomfactor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.FillRectangle(myColours.selectedVertexBrush, New RectangleF(absOffsetX - vinner(0).X * zoomfactor - (punktgroesse / 2), absOffsetY + vinner(0).Y * zoomfactor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - vouter(0).X * zoomfactor, absOffsetY + vouter(0).Y * zoomfactor, absOffsetX - vouter(segments_oval - 1).X * zoomfactor, absOffsetY + vouter(segments_oval - 1).Y * zoomfactor)
                            e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - vinner(0).X * zoomfactor, absOffsetY + vinner(0).Y * zoomfactor, absOffsetX - vinner(segments_oval - 1).X * zoomfactor, absOffsetY + vinner(segments_oval - 1).Y * zoomfactor)
                            e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - vinner(0).X * zoomfactor, absOffsetY + vinner(0).Y * zoomfactor, absOffsetX - vouter(segments_oval - 1).X * zoomfactor, absOffsetY + vouter(segments_oval - 1).Y * zoomfactor)
                            e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - vinner(segments_oval - 1).X * zoomfactor, absOffsetY + vinner(segments_oval - 1).Y * zoomfactor, absOffsetX - vouter(segments_oval - 1).X * zoomfactor, absOffsetY + vouter(segments_oval - 1).Y * zoomfactor)
                            If primitiveObject = 10 Then e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - vinner(0).X * zoomfactor, absOffsetY + vinner(0).Y * zoomfactor, absOffsetX - vcenter.X * zoomfactor, absOffsetY + vcenter.Y * zoomfactor)
                            For counter = 1 To segments_oval - 1
                                e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - vouter(counter).X * zoomfactor, absOffsetY + vouter(counter).Y * zoomfactor, absOffsetX - vouter(counter - 1).X * zoomfactor, absOffsetY + vouter(counter - 1).Y * zoomfactor)
                                e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - vinner(counter).X * zoomfactor, absOffsetY + vinner(counter).Y * zoomfactor, absOffsetX - vinner(counter - 1).X * zoomfactor, absOffsetY + vinner(counter - 1).Y * zoomfactor)
                                e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - vinner(counter).X * zoomfactor, absOffsetY + vinner(counter).Y * zoomfactor, absOffsetX - vouter(counter - 1).X * zoomfactor, absOffsetY + vouter(counter - 1).Y * zoomfactor)
                                e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - vinner(counter - 1).X * zoomfactor, absOffsetY + vinner(counter - 1).Y * zoomfactor, absOffsetX - vouter(counter - 1).X * zoomfactor, absOffsetY + vouter(counter - 1).Y * zoomfactor)
                                If primitiveObject = 10 Then e.Graphics.DrawLine(myColours.selectedLinePen, absOffsetX - vinner(counter).X * zoomfactor, absOffsetY + vinner(counter).Y * zoomfactor, absOffsetX - vcenter.X * zoomfactor, absOffsetY + vcenter.Y * zoomfactor)
                                e.Graphics.FillRectangle(myColours.selectedVertexBrush, New RectangleF(absOffsetX - vouter(counter).X * zoomfactor - (punktgroesse / 2), absOffsetY + vouter(counter).Y * zoomfactor - (punktgroesse / 2), punktgroesse, punktgroesse))
                                e.Graphics.FillRectangle(myColours.selectedVertexBrush, New RectangleF(absOffsetX - vinner(counter).X * zoomfactor - (punktgroesse / 2), absOffsetY + vinner(counter).Y * zoomfactor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            Next
                            e.Graphics.FillRectangle(myColours.selectedVertexBrush, New RectangleF(absOffsetX - vcenter.X * zoomfactor - (punktgroesse / 2), absOffsetY + vcenter.Y * zoomfactor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            Exit Select
                    End Select
                Case 255
                    e.Graphics.DrawString("Click to add a new triangle", New Font("Arial", 16, FontStyle.Regular, GraphicsUnit.Pixel), Brushes.Black, 12, 32)
                    e.Graphics.DrawString("Click to add a new triangle", New Font("Arial", 16, FontStyle.Regular, GraphicsUnit.Pixel), Brushes.Azure, 10, 30)
                    primitiveCenter = New Vertex(Math.Round(getXcoordinate(getCursorpositionX()) / Me.moveSnap) * Me.moveSnap, Math.Round(getYcoordinate(getCursorpositionY()) / Me.moveSnap) * Me.moveSnap, False, False)
                    e.Graphics.DrawLine(Pens.RoyalBlue, absOffsetX - primitiveCenter.X * zoomfactor, absOffsetY + primitiveCenter.Y * zoomfactor, absOffsetX - LLast(Vertices).X * zoomfactor, absOffsetY + LLast(Vertices).Y * zoomfactor)
                    e.Graphics.DrawLine(Pens.PowderBlue, absOffsetX - primitiveCenter.X * zoomfactor, absOffsetY + primitiveCenter.Y * zoomfactor, absOffsetX - Vertices(Vertices.Count - 2).X * zoomfactor, absOffsetY + Vertices(Vertices.Count - 2).Y * zoomfactor)
                    e.Graphics.DrawLine(Pens.PowderBlue, absOffsetX - LLast(Vertices).X * zoomfactor, absOffsetY + LLast(Vertices).Y * zoomfactor, absOffsetX - Vertices(Vertices.Count - 2).X * zoomfactor, absOffsetY + Vertices(Vertices.Count - 2).Y * zoomfactor)
                    Exit Select
            End Select
        End If
        ' Templates:
        Dim shapeCount As Integer = templateShape.Count
        If shapeCount > 0 Then
            If BtnPreview.Checked Then Exit Sub
            If Not templateShape(0).X = Single.Epsilon Then
                Dim templateShapeArray(shapeCount - 1) As PointF
                For i As Integer = 0 To shapeCount - 1
                    templateShapeArray(i).X = absOffsetX - templateShape(i).X * zoomfactor
                    templateShapeArray(i).Y = absOffsetY + templateShape(i).Y * zoomfactor
                Next
                e.Graphics.DrawPolygon(Pens.Orange, templateShapeArray)
                For i = 0 To templateText.Count - 1
                    e.Graphics.DrawString(templateText(i).Text, New Font("Arial", (1000 * zoomfactor) + 8, FontStyle.Regular, GraphicsUnit.Pixel), Brushes.Black, absOffsetX - templateText(i).X * zoomfactor + 2, absOffsetY + templateText(i).Y * zoomfactor + 2)
                    e.Graphics.DrawString(templateText(i).Text, New Font("Arial", (1000 * zoomfactor) + 8, FontStyle.Regular, GraphicsUnit.Pixel), Brushes.Azure, absOffsetX - templateText(i).X * zoomfactor, absOffsetY + templateText(i).Y * zoomfactor)
                Next
            Else

                For i As Integer = 1 To shapeCount - 1 Step 4
                    Dim templateShapeArray(3) As PointF
                    templateShapeArray(0).X = absOffsetX - templateShape(i).X * zoomfactor
                    templateShapeArray(0).Y = absOffsetY + templateShape(i).Y * zoomfactor
                    templateShapeArray(1).X = absOffsetX - templateShape(i + 1).X * zoomfactor
                    templateShapeArray(1).Y = absOffsetY + templateShape(i + 1).Y * zoomfactor
                    templateShapeArray(2).X = absOffsetX - templateShape(i + 2).X * zoomfactor
                    templateShapeArray(2).Y = absOffsetY + templateShape(i + 2).Y * zoomfactor
                    templateShapeArray(3).X = absOffsetX - templateShape(i + 3).X * zoomfactor
                    templateShapeArray(3).Y = absOffsetY + templateShape(i + 3).Y * zoomfactor
                    e.Graphics.DrawPolygon(Pens.LimeGreen, templateShapeArray)
                Next
            End If
        End If

        ' Axis Label:
        If ShowAxisLabelToolStripMenuItem.Checked Then
            Dim scale As Single
            If zoomlevel > 80 Then scale = Me.rasterSnap * zoomfactor : GoTo label_zechnen
            If zoomlevel > 0 Then scale = Me.rasterSnap * zoomfactor * 10 : GoTo label_zechnen
            If zoomlevel > -10 Then scale = Me.rasterSnap * zoomfactor * 100 : GoTo label_zechnen
            If zoomlevel > -20 Then scale = Me.rasterSnap * zoomfactor * 1000 : GoTo label_zechnen
            If zoomlevel > -30 Then scale = Me.rasterSnap * zoomfactor * 10000 : GoTo label_zechnen
            scale = Me.rasterSnap * zoomfactor * 100000
label_zechnen:
            Dim relOffsetX As Integer
            Dim relOffsetY As Integer
            relOffsetX = absOffsetX Mod (scale * 10)
            relOffsetY = absOffsetY Mod (scale * 10)
            e.Graphics.FillRectangle(Brushes.LightGray, 0, Me.ClientSize.Height - 46, Me.ClientSize.Width, Me.ClientSize.Height)
            e.Graphics.FillRectangle(Brushes.LightGray, 0, 0, 46, Me.ClientSize.Height)
            If relOffsetX < 46 Then relOffsetX += scale * 10
            For x As Single = relOffsetX To Me.ClientSize.Width Step scale * 10
                e.Graphics.DrawString(Math.Round(getXcoordinate(x) / Me.moveSnap * factor) * Me.moveSnap / 1000, New Font("Arial", 12, FontStyle.Regular, GraphicsUnit.Pixel), Brushes.Azure, x + 2, Me.ClientSize.Height - 42)
                e.Graphics.DrawString(Math.Round(getXcoordinate(x) / Me.moveSnap * factor) * Me.moveSnap / 1000, New Font("Arial", 12, FontStyle.Regular, GraphicsUnit.Pixel), Brushes.Black, x, Me.ClientSize.Height - 44)
            Next
            For y As Single = relOffsetY To Me.ClientSize.Height Step scale * 10
                e.Graphics.DrawString(Math.Round(getYcoordinate(y) / Me.moveSnap * factor) * Me.moveSnap / 1000, New Font("Arial", 12, FontStyle.Regular, GraphicsUnit.Pixel), Brushes.Azure, 6, y + 2)
                e.Graphics.DrawString(Math.Round(getYcoordinate(y) / Me.moveSnap * factor) * Me.moveSnap / 1000, New Font("Arial", 12, FontStyle.Regular, GraphicsUnit.Pixel), Brushes.Black, 4, y)
            Next
        End If
        'wa.Stop()
    End Sub

    Private Sub ExitToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ExitToolStripMenuItem.Click
        Me.Close()
    End Sub

    Private Sub BtnColours_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BtnColours.Click
        ColourToolStrip.Visible = BtnColours.Checked
        BtnPreview.Checked = BtnPreview.Checked AndAlso BtnColours.Checked
        Me.Refresh()
    End Sub

    Private Sub BtnAutoRound_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BtnAutoRound.Click
        For Each vert As Vertex In SelectedVertices
            If vert.groupindex = -1 Then
                vert.X = Math.Round(vert.X)
                vert.Y = Math.Round(vert.Y)
            End If
        Next
        Me.Refresh()
    End Sub

    Private Sub RTBDebug_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RTBDebug.TextChanged
        If RTBDebug.Lines.Length > 35 Then
            RTBDebug.Text = RTBDebug.Lines(RTBDebug.Lines.Length - 1)
        End If
    End Sub

    Private Sub ProjectOnYZPlaneToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ProjectOnYZPlaneToolStripMenuItem.Click
        addHistoryI(withDataFrom.All)
        importDat(0)
        addHistoryII(withDataFrom.All)
    End Sub

    Private Sub ProjectOnYZPlane2ToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ProjectOnYZPlane2ToolStripMenuItem.Click
        addHistoryI(withDataFrom.All)
        importDat(3)
        addHistoryII(withDataFrom.All)
    End Sub

    Private Sub ProjectOnZXPlaneToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ProjectOnZXPlaneToolStripMenuItem.Click
        addHistoryI(withDataFrom.All)
        importDat(1)
        addHistoryII(withDataFrom.All)
    End Sub

    Private Sub ProjectOnZXPlane2ToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ProjectOnZXPlane2ToolStripMenuItem.Click
        addHistoryI(withDataFrom.All)
        importDat(4)
        addHistoryII(withDataFrom.All)
    End Sub

    Private Sub ProjectOnXYPlaneToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ProjectOnXYPlaneToolStripMenuItem.Click
        addHistoryI(withDataFrom.All)
        importDat(2)
        addHistoryII(withDataFrom.All)
    End Sub

    Private Sub ProjectOnXYPlane2ToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ProjectOnXYPlane2ToolStripMenuItem.Click
        addHistoryI(withDataFrom.All)
        importDat(5)
        addHistoryII(withDataFrom.All)
    End Sub

    Private Sub importDat(ByVal projectMode As Byte, Optional ByVal showDialog As Boolean = True, Optional ByVal filename As String = "")
newTry:
        primitiveCenter.X = 0
        primitiveCenter.Y = 0
        Dim mySubfiles As New List(Of String)
        Dim startVertex As Integer = Vertices.Count
        Dim endVertex As Integer = 0
        Dim startTriangle As Integer = Triangles.Count
        Dim endTriangle As Integer = 0
        Dim iDialog As Form = ImportDialog
        If showDialog AndAlso iDialog.ShowDialog = Windows.Forms.DialogResult.Cancel Then Exit Sub
        If Not showDialog Then
            OpenDAT.FileName = filename
            CType(iDialog.Controls("GBMode").Controls("RBappend"), RadioButton).PerformClick()
        End If
        Dim RBnew As RadioButton = CType(iDialog.Controls("GBMode").Controls("RBnew"), RadioButton)
        Dim RBtemplate As RadioButton = CType(iDialog.Controls("GBMode").Controls("RBtemplate"), RadioButton)
        If showDialog AndAlso (RBnew.Checked OrElse RBtemplate.Checked) AndAlso unsavedChanges AndAlso ShowAllWarningsToolStripMenuItem.Checked Then
            Dim result As MsgBoxResult = MsgBox("You have unsaved changes - would you like to save them first?", MsgBoxStyle.YesNoCancel + MsgBoxStyle.MsgBoxSetForeground + MsgBoxStyle.Information, "Question:")
            If result = MsgBoxResult.Yes Then SaveToolStripMenuItem.PerformClick()
            If result = MsgBoxResult.Cancel Then Exit Sub
        End If
        BtnMove.PerformClick()
        isLoading = True
        MenuStrip1.Enabled = False
        Me.ToolStrip1.Enabled = False
        Me.ColourToolStrip.Enabled = False
        GBImage.Enabled = False
        GBSnap.Enabled = False
        Select Case projectMode
            Case 0 ' YZ mit -X (Right) [OK]
                OpenDAT.Title = "Import from DAT, project on YZ (Right):"
            Case 3 ' YZ mit +X (Left) [OK]
                OpenDAT.Title = "Import from DAT, project on YZ (Left):"
            Case 1 ' XZ mit -Y (Top) [OK]
                OpenDAT.Title = "Import from DAT, project on XZ (Top):"
            Case 4 ' XZ mit -Y (Bottom) [OK]
                OpenDAT.Title = "Import from DAT, project on XZ (Bottom):"
            Case 2 ' XY mit -Z (Front) [OK]
                OpenDAT.Title = "Import from DAT, project on XY (Front):"
            Case 5 ' XY mit +Z (Back) [OK]
                OpenDAT.Title = "Import from DAT, project on XY (Back):"
        End Select
        If Not showDialog OrElse OpenDAT.ShowDialog() = Windows.Forms.DialogResult.OK Then
            If OpenDAT.FileName <> "" Then
                Try
                    If RBnew.Checked OrElse RBtemplate.Checked Then
                        If RBtemplate.Checked Then
                            isLoading = True
                            alreadyCalculatedQuads = 0
                            myMetadata = New MainForm.Metadata() With {.isMainMetadata = True}
                            For r As Integer = 0 To 3
                                For c As Integer = 0 To 3
                                    myMetadata.matrix(r, c) = 0
                                    If c = r Then myMetadata.matrix(r, c) = 1
                                Next
                            Next
                            templateShape.Clear()
                            templateProjectionQuad.Clear()
                            templateText.Clear()
                            isLoading = False
                            templateShape.Add(New PointF(Single.Epsilon, Single.Epsilon))
                            scaleVertices(True)
                        Else
                            startVertex = 0
                            startTriangle = 0
                            newPattern()
                        End If
                    Else
                        scaleVertices(True)
                    End If
                    If Me.ShowAllWarningsToolStripMenuItem.Checked AndAlso RBnew.Checked Then While MsgBox("This function does not inline subfiles but 2D-primitives." & vbCrLf & "Use 'File->Import from DAT->Project on..->Append part' to inline the desired subpart.", MsgBoxStyle.OkOnly + MsgBoxStyle.MsgBoxSetForeground + MsgBoxStyle.Information, "Info:") <> MsgBoxResult.Ok : End While

                    Dim tempMetadata As New Metadata()
                    tempMetadata.additionalData = ""
                    Using DateiIn As StreamReader = My.Computer.FileSystem.OpenTextFileReader(OpenDAT.FileName, System.Text.Encoding.GetEncoding(20127))
                        Dim textline As String
                        Dim headerState As Integer = 0

                        Do
                            textline = DateiIn.ReadLine().Trim()
                            If textline <> Nothing Then
                                ' Header parsen
                                If headerState <> -1 Then

                                    Try
nextTry:
                                        If textline Like "1 *" Then headerState = -1
                                        If textline Like "2 *" Then headerState = -1
                                        If textline Like "3 *" Then headerState = -1
                                        If textline Like "4 *" Then headerState = -1
                                        If textline Like "5 *" Then headerState = -1
                                        If headerState = 0 AndAlso textline Like "0 *" Then
                                            tempMetadata.mData(0) = Mid(textline, 3)
                                            headerState = 1
                                        ElseIf headerState = 1 AndAlso textline Like "0 Name: *" Then
                                            tempMetadata.mData(1) = Mid(textline, 9)
                                            headerState = 2
                                        ElseIf headerState = 2 AndAlso textline Like "0 Author: *" Then
                                            tempMetadata.mData(2) = Mid(textline, 11, textline.LastIndexOf("[") - 11)
                                            tempMetadata.mData(3) = Mid(textline, textline.LastIndexOf("[") + 2, textline.LastIndexOf("]") - textline.LastIndexOf("[") - 1)
                                            headerState = 3
                                        ElseIf headerState = 3 AndAlso textline Like "0 !LDRAW_ORG *" Then
                                            tempMetadata.mData(4) = Mid(textline, 14)
                                            headerState = 4
                                        ElseIf headerState = 4 AndAlso textline Like "0 !LICENSE *" Then
                                            tempMetadata.mData(5) = Mid(textline, 12)
                                            headerState = 5
                                        ElseIf headerState = 5 Then
                                            If textline Like "0 !HELP *" Then
                                                tempMetadata.mData(6) += Mid(textline, 9) & "<br>"
                                            Else
                                                headerState = 6 : GoTo nextTry
                                            End If
                                        ElseIf headerState = 6 AndAlso textline Like "0 BFC *" Then
                                            tempMetadata.mData(7) = Mid(textline, 3)
                                            headerState = 7
                                        ElseIf headerState = 7 Then
                                            If textline Like "0 !CATEGORY*" Then
                                                tempMetadata.mData(8) = Mid(textline, 11)
                                            Else
                                                headerState = 8 : GoTo nextTry
                                            End If
                                        ElseIf headerState = 8 Then
                                            If textline Like "0 !KEYWORDS*" Then
                                                tempMetadata.mData(9) += Mid(textline, 12) & "<br>"
                                            Else
                                                headerState = 9 : GoTo nextTry
                                            End If
                                        ElseIf headerState = 9 Then
                                            If textline Like "0 !HISTORY*" Then
                                                tempMetadata.mData(10) += Mid(textline, 12) & "<br>"
                                            Else
                                                headerState = 10 : GoTo nextTry
                                            End If
                                        ElseIf headerState = 10 Then
                                            If textline Like "0 //*" Then
                                                tempMetadata.mData(11) += Mid(textline, 5) & "<br>"
                                            Else
                                                If tempMetadata.mData(6).Length > 3 Then tempMetadata.mData(6) = Mid(tempMetadata.mData(6), 1, tempMetadata.mData(6).Length - 4)
                                                If tempMetadata.mData(9).Length > 3 Then tempMetadata.mData(9) = Mid(tempMetadata.mData(9), 1, tempMetadata.mData(9).Length - 4)
                                                If tempMetadata.mData(10).Length > 3 Then tempMetadata.mData(10) = Mid(tempMetadata.mData(10), 1, tempMetadata.mData(10).Length - 4)
                                                If tempMetadata.mData(11).Length > 3 Then tempMetadata.mData(11) = Mid(tempMetadata.mData(11), 1, tempMetadata.mData(11).Length - 4)
                                                headerState = -1
                                            End If
                                        ElseIf textline Like "*!CMDLINE*" Then
                                        Else
                                            headerState = -1
                                        End If
                                        If headerState <> -1 Then Continue Do
                                    Catch
                                        headerState += 1
                                    End Try
                                End If
                                Dim ttextline As String = textline
                                textline = Replace(Replace(textline, ".", comma).ToLower, "  ", " ")
                                Dim oldlenght As Integer = textline.Length
                                Do
                                    textline = Replace(textline, "  ", " ")
                                Loop Until oldlenght = textline.Length
                                Dim words() As String = textline.Split(CType(" ", Char))
                                If RBtemplate.Checked AndAlso words(0) Like "*0*" Then
                                    tempMetadata.additionalData += ttextline & "<br>"
                                End If
                                If words(0) Like "*1*" AndAlso words.Length = 15 Then
                                    If RBtemplate.Checked Then
                                        tempMetadata.additionalData += ttextline & "<br>"
                                    Else
                                        Select Case projectMode
                                            ' YZ mit -X (Right) []
                                            Case 0 : inlinePrimitive(words(1), _
                                                    -CType(words(11), Double), -CType(words(13), Double), 0, CType(words(4), Double), _
                                                    -CType(words(8), Double), -CType(words(10), Double), 0, CType(words(3), Double), _
                                                    0, 0, 1, 0, _
                                                    1, 0, 0, 0, _
                                                    0, 1, 0, 0, _
                                                    0, 0, -1, 0, _
                                                    Replace(words(14), comma, "."), mySubfiles, False)
                                                ' YZ mit +X (Left) []
                                            Case 3 : inlinePrimitive(words(1), _
                                                    CType(words(11), Double), CType(words(13), Double), 0, -CType(words(4), Double), _
                                                    -CType(words(8), Double), -CType(words(10), Double), 0, CType(words(3), Double), _
                                                    0, 0, 1, 0, _
                                                    1, 0, 0, 0, _
                                                    0, 1, 0, 0, _
                                                    0, 0, -1, 0, _
                                                    Replace(words(14), comma, "."), mySubfiles, False)
                                                ' XZ mit -Y (Top) [OK]
                                            Case 1 : inlinePrimitive(words(1), _
                                                    -CType(words(5), Double), -CType(words(7), Double), 0, -CType(words(2), Double), _
                                                    -CType(words(11), Double), -CType(words(13), Double), 0, -CType(words(4), Double), _
                                                    0, 0, 1, 0, _
                                                    -1, 0, 0, 0, _
                                                    0, -1, 0, 0, _
                                                    0, 0, -1, 0, _
                                                    Replace(words(14), comma, "."), mySubfiles, False)
                                                ' XZ mit -Y (Bottom) [OK]
                                            Case 4 : inlinePrimitive(words(1), _
                                                CType(words(5), Double), CType(words(7), Double), 0, -CType(words(2), Double), _
                                                -CType(words(11), Double), -CType(words(13), Double), 0, CType(words(4), Double), _
                                                0, 0, 1, 0, _
                                                1, 0, 0, 0, _
                                                0, 1, 0, 0, _
                                                0, 0, -1, 0, _
                                                Replace(words(14), comma, "."), mySubfiles, False)
                                                ' XY mit -Z (Front) [OK]
                                            Case 2 : inlinePrimitive(words(1), _
                                               CType(words(5), Double), CType(words(7), Double), 0, -CType(words(2), Double), _
                                                -CType(words(8), Double), -CType(words(10), Double), 0, CType(words(3), Double), _
                                                    0, 0, 1, 0, _
                                                    1, 0, 0, 0, _
                                                    0, 1, 0, 0, _
                                                    0, 0, -1, 0, _
                                                    Replace(words(14), comma, "."), mySubfiles, False)
                                                ' XY mit +Z (Back) [OK]
                                            Case 5 : inlinePrimitive(words(1), _
                                                -CType(words(5), Double), -CType(words(7), Double), 0, CType(words(2), Double), _
                                                -CType(words(8), Double), -CType(words(10), Double), 0, CType(words(3), Double), _
                                                    0, 0, 1, 0, _
                                                    1, 0, 0, 0, _
                                                    0, 1, 0, 0, _
                                                    0, 0, -1, 0, _
                                                    Replace(words(14), comma, "."), mySubfiles, False)
                                        End Select
                                    End If
                                ElseIf words(0) Like "*3*" AndAlso words.Length = 11 Then
                                    Select Case projectMode
                                        Case 0
                                            '3 4, 6 7, 9 10
                                            Vertices.Add(New Vertex(CType(words(4), Single), CType(words(3), Single), False))
                                            Vertices.Add(New Vertex(CType(words(7), Single), CType(words(6), Single), False))
                                            Vertices.Add(New Vertex(CType(words(10), Single), CType(words(9), Single), False))
                                        Case 1
                                            '4 2, 7 5, 10 8
                                            Vertices.Add(New Vertex(-CType(words(2), Single), CType(-words(4), Single), False))
                                            Vertices.Add(New Vertex(-CType(words(5), Single), CType(-words(7), Single), False))
                                            Vertices.Add(New Vertex(-CType(words(8), Single), CType(-words(10), Single), False))
                                        Case 2
                                            '2 3, 5 6, 8 9
                                            Vertices.Add(New Vertex(-CType(words(2), Single), CType(words(3), Single), False))
                                            Vertices.Add(New Vertex(-CType(words(5), Single), CType(words(6), Single), False))
                                            Vertices.Add(New Vertex(-CType(words(8), Single), CType(words(9), Single), False))
                                        Case 3
                                            '3 4, 6 7, 9 10
                                            Vertices.Add(New Vertex(-CType(words(4), Single), CType(words(3), Single), False))
                                            Vertices.Add(New Vertex(-CType(words(7), Single), CType(words(6), Single), False))
                                            Vertices.Add(New Vertex(-CType(words(10), Single), CType(words(9), Single), False))
                                        Case 4
                                            '4 2, 7 5, 10 8
                                            Vertices.Add(New Vertex(-CType(words(2), Single), CType(words(4), Single), False))
                                            Vertices.Add(New Vertex(-CType(words(5), Single), CType(words(7), Single), False))
                                            Vertices.Add(New Vertex(-CType(words(8), Single), CType(words(10), Single), False))
                                        Case 5
                                            '2 3, 5 6, 8 9
                                            Vertices.Add(New Vertex(CType(words(2), Single), CType(words(3), Single), False))
                                            Vertices.Add(New Vertex(CType(words(5), Single), CType(words(6), Single), False))
                                            Vertices.Add(New Vertex(CType(words(8), Single), CType(words(9), Single), False))
                                    End Select
                                    If Not RBtemplate.Checked Then
                                        Triangles.Add(New Triangle(Vertices(Vertices.Count - 1), Vertices(Vertices.Count - 2), Vertices(Vertices.Count - 3)))
                                        If words(1) Like "0x2*" Then
                                            LLast(Triangles).myColour = Color.FromArgb(255, _
                                                                                       Convert.ToInt32("0x" & Mid(words(1), 4, 2), 16), _
                                                                                       Convert.ToInt32("0x" & Mid(words(1), 6, 2), 16), _
                                                                                       Convert.ToInt32("0x" & Mid(words(1), 8, 2), 16))
                                            LLast(Triangles).myColourNumber = -1
                                        ElseIf words(1) Like "0x*" Then
                                            LLast(Triangles).myColourNumber = 16
                                        Else
                                            LLast(Triangles).myColourNumber = CType(words(1), Short)
                                        End If
                                        Vertices(Vertices.Count - 1).linkedTriangles.Add(LLast(Triangles))
                                        Vertices(Vertices.Count - 2).linkedTriangles.Add(LLast(Triangles))
                                        Vertices(Vertices.Count - 3).linkedTriangles.Add(LLast(Triangles))
                                    Else
                                        templateShape.Add(New PointF(Vertices(Vertices.Count - 3).X * 1000, Vertices(Vertices.Count - 3).Y * 1000))
                                        templateShape.Add(New PointF(Vertices(Vertices.Count - 3).X * 1000, Vertices(Vertices.Count - 3).Y * 1000))
                                        templateShape.Add(New PointF(Vertices(Vertices.Count - 2).X * 1000, Vertices(Vertices.Count - 2).Y * 1000))
                                        templateShape.Add(New PointF(Vertices(Vertices.Count - 1).X * 1000, Vertices(Vertices.Count - 1).Y * 1000))
                                        templateProjectionQuad.Add(New ProjectionQuad(Vertices(Vertices.Count - 3).X, Vertices(Vertices.Count - 3).Y, _
                                                                                      Vertices(Vertices.Count - 3).X, Vertices(Vertices.Count - 3).Y, _
                                                                                      Vertices(Vertices.Count - 2).X, Vertices(Vertices.Count - 2).Y, _
                                                                                      Vertices(Vertices.Count - 1).X, Vertices(Vertices.Count - 1).Y, _
                                                                                      CType(words(2), Single), CType(words(3), Single), CType(words(4), Single), _
                                                                                      CType(words(2), Single), CType(words(3), Single), CType(words(4), Single), _
                                                                                      CType(words(5), Single), CType(words(6), Single), CType(words(7), Single), _
                                                                                      CType(words(8), Single), CType(words(9), Single), CType(words(10), Single)))
                                    End If
                                ElseIf words(0) Like "*4*" AndAlso words.Length = 14 Then
                                    Select Case projectMode
                                        Case 0
                                            '3 4, 6 7, 9 10, 12 13
                                            Vertices.Add(New Vertex(CType(words(4), Single), CType(words(3), Single), False))
                                            Vertices.Add(New Vertex(CType(words(7), Single), CType(words(6), Single), False))
                                            Vertices.Add(New Vertex(CType(words(10), Single), CType(words(9), Single), False))
                                            If Not RBtemplate.Checked Then
                                                Triangles.Add(New Triangle(Vertices(Vertices.Count - 1), Vertices(Vertices.Count - 2), Vertices(Vertices.Count - 3)))
                                                If words(1) Like "0x2*" Then
                                                    LLast(Triangles).myColour = Color.FromArgb(255, _
                                                                                               Convert.ToInt32("0x" & Mid(words(1), 4, 2), 16), _
                                                                                               Convert.ToInt32("0x" & Mid(words(1), 6, 2), 16), _
                                                                                               Convert.ToInt32("0x" & Mid(words(1), 8, 2), 16))
                                                    LLast(Triangles).myColourNumber = -1
                                                ElseIf words(1) Like "0x*" Then
                                                    LLast(Triangles).myColourNumber = 16
                                                Else
                                                    LLast(Triangles).myColourNumber = CType(words(1), Short)
                                                End If
                                                Vertices(Vertices.Count - 1).linkedTriangles.Add(LLast(Triangles))
                                                Vertices(Vertices.Count - 2).linkedTriangles.Add(LLast(Triangles))
                                                Vertices(Vertices.Count - 3).linkedTriangles.Add(LLast(Triangles))
                                            End If
                                            Vertices.Add(New Vertex(CType(words(13), Single), CType(words(12), Single), False))
                                        Case 1
                                            '4 2, 7 5, 10 8, 13 11
                                            Vertices.Add(New Vertex(-CType(words(2), Single), -CType(words(4), Single), False))
                                            Vertices.Add(New Vertex(-CType(words(5), Single), -CType(words(7), Single), False))
                                            Vertices.Add(New Vertex(-CType(words(8), Single), -CType(words(10), Single), False))
                                            If Not RBtemplate.Checked Then
                                                Triangles.Add(New Triangle(Vertices(Vertices.Count - 1), Vertices(Vertices.Count - 2), Vertices(Vertices.Count - 3)))
                                                If words(1) Like "0x2*" Then
                                                    LLast(Triangles).myColour = Color.FromArgb(255, _
                                                                                               Convert.ToInt32("0x" & Mid(words(1), 4, 2), 16), _
                                                                                               Convert.ToInt32("0x" & Mid(words(1), 6, 2), 16), _
                                                                                               Convert.ToInt32("0x" & Mid(words(1), 8, 2), 16))
                                                    LLast(Triangles).myColourNumber = -1
                                                ElseIf words(1) Like "0x*" Then
                                                    LLast(Triangles).myColourNumber = 16
                                                Else
                                                    LLast(Triangles).myColourNumber = CType(words(1), Short)
                                                End If
                                                Vertices(Vertices.Count - 1).linkedTriangles.Add(LLast(Triangles))
                                                Vertices(Vertices.Count - 2).linkedTriangles.Add(LLast(Triangles))
                                                Vertices(Vertices.Count - 3).linkedTriangles.Add(LLast(Triangles))
                                            End If
                                            Vertices.Add(New Vertex(-CType(words(11), Single), -CType(words(13), Single), False))
                                        Case 2
                                            '2 3, 5 6, 8 9, 11 12
                                            Vertices.Add(New Vertex(-CType(words(2), Single), CType(words(3), Single), False))
                                            Vertices.Add(New Vertex(-CType(words(5), Single), CType(words(6), Single), False))
                                            Vertices.Add(New Vertex(-CType(words(8), Single), CType(words(9), Single), False))
                                            If Not RBtemplate.Checked Then
                                                Triangles.Add(New Triangle(Vertices(Vertices.Count - 1), Vertices(Vertices.Count - 2), Vertices(Vertices.Count - 3)))
                                                If words(1) Like "0x2*" Then
                                                    LLast(Triangles).myColour = Color.FromArgb(255, _
                                                                                               Convert.ToInt32("0x" & Mid(words(1), 4, 2), 16), _
                                                                                               Convert.ToInt32("0x" & Mid(words(1), 6, 2), 16), _
                                                                                               Convert.ToInt32("0x" & Mid(words(1), 8, 2), 16))
                                                    LLast(Triangles).myColourNumber = -1
                                                ElseIf words(1) Like "0x*" Then
                                                    LLast(Triangles).myColourNumber = 16
                                                Else
                                                    LLast(Triangles).myColourNumber = CType(words(1), Short)
                                                End If
                                                Vertices(Vertices.Count - 1).linkedTriangles.Add(LLast(Triangles))
                                                Vertices(Vertices.Count - 2).linkedTriangles.Add(LLast(Triangles))
                                                Vertices(Vertices.Count - 3).linkedTriangles.Add(LLast(Triangles))
                                            End If
                                            Vertices.Add(New Vertex(-CType(words(11), Single), CType(words(12), Single), False))
                                        Case 3
                                            '3 4, 6 7, 9 10, 12 13
                                            Vertices.Add(New Vertex(-CType(words(4), Single), CType(words(3), Single), False))
                                            Vertices.Add(New Vertex(-CType(words(7), Single), CType(words(6), Single), False))
                                            Vertices.Add(New Vertex(-CType(words(10), Single), CType(words(9), Single), False))
                                            If Not RBtemplate.Checked Then
                                                Triangles.Add(New Triangle(Vertices(Vertices.Count - 1), Vertices(Vertices.Count - 2), Vertices(Vertices.Count - 3)))
                                                If words(1) Like "0x2*" Then
                                                    LLast(Triangles).myColour = Color.FromArgb(255, _
                                                                                               Convert.ToInt32("0x" & Mid(words(1), 4, 2), 16), _
                                                                                               Convert.ToInt32("0x" & Mid(words(1), 6, 2), 16), _
                                                                                               Convert.ToInt32("0x" & Mid(words(1), 8, 2), 16))
                                                    LLast(Triangles).myColourNumber = -1
                                                ElseIf words(1) Like "0x*" Then
                                                    LLast(Triangles).myColourNumber = 16
                                                Else
                                                    LLast(Triangles).myColourNumber = CType(words(1), Short)
                                                End If
                                                Vertices(Vertices.Count - 1).linkedTriangles.Add(LLast(Triangles))
                                                Vertices(Vertices.Count - 2).linkedTriangles.Add(LLast(Triangles))
                                                Vertices(Vertices.Count - 3).linkedTriangles.Add(LLast(Triangles))
                                            End If
                                            Vertices.Add(New Vertex(-CType(words(13), Single), CType(words(12), Single), False))
                                        Case 4
                                            '4 2, 7 5, 10 8, 13 11
                                            Vertices.Add(New Vertex(-CType(words(2), Single), CType(words(4), Single), False))
                                            Vertices.Add(New Vertex(-CType(words(5), Single), CType(words(7), Single), False))
                                            Vertices.Add(New Vertex(-CType(words(8), Single), CType(words(10), Single), False))
                                            If Not RBtemplate.Checked Then
                                                Triangles.Add(New Triangle(Vertices(Vertices.Count - 1), Vertices(Vertices.Count - 2), Vertices(Vertices.Count - 3)))
                                                If words(1) Like "0x2*" Then
                                                    LLast(Triangles).myColour = Color.FromArgb(255, _
                                                                                               Convert.ToInt32("0x" & Mid(words(1), 4, 2), 16), _
                                                                                               Convert.ToInt32("0x" & Mid(words(1), 6, 2), 16), _
                                                                                               Convert.ToInt32("0x" & Mid(words(1), 8, 2), 16))
                                                    LLast(Triangles).myColourNumber = -1
                                                ElseIf words(1) Like "0x*" Then
                                                    LLast(Triangles).myColourNumber = 16
                                                Else
                                                    LLast(Triangles).myColourNumber = CType(words(1), Short)
                                                End If
                                                Vertices(Vertices.Count - 1).linkedTriangles.Add(LLast(Triangles))
                                                Vertices(Vertices.Count - 2).linkedTriangles.Add(LLast(Triangles))
                                                Vertices(Vertices.Count - 3).linkedTriangles.Add(LLast(Triangles))
                                            End If
                                            Vertices.Add(New Vertex(-CType(words(11), Single), CType(words(13), Single), False))
                                        Case 5
                                            '2 3, 5 6, 8 9, 11 12
                                            Vertices.Add(New Vertex(CType(words(2), Single), CType(words(3), Single), False))
                                            Vertices.Add(New Vertex(CType(words(5), Single), CType(words(6), Single), False))
                                            Vertices.Add(New Vertex(CType(words(8), Single), CType(words(9), Single), False))
                                            If Not RBtemplate.Checked Then
                                                Triangles.Add(New Triangle(Vertices(Vertices.Count - 1), Vertices(Vertices.Count - 2), Vertices(Vertices.Count - 3)))
                                                If words(1) Like "0x2*" Then
                                                    LLast(Triangles).myColour = Color.FromArgb(255, _
                                                                                               Convert.ToInt32("0x" & Mid(words(1), 4, 2), 16), _
                                                                                               Convert.ToInt32("0x" & Mid(words(1), 6, 2), 16), _
                                                                                               Convert.ToInt32("0x" & Mid(words(1), 8, 2), 16))
                                                    LLast(Triangles).myColourNumber = -1
                                                ElseIf words(1) Like "0x*" Then
                                                    LLast(Triangles).myColourNumber = 16
                                                Else
                                                    LLast(Triangles).myColourNumber = CType(words(1), Short)
                                                End If
                                                Vertices(Vertices.Count - 1).linkedTriangles.Add(LLast(Triangles))
                                                Vertices(Vertices.Count - 2).linkedTriangles.Add(LLast(Triangles))
                                                Vertices(Vertices.Count - 3).linkedTriangles.Add(LLast(Triangles))
                                            End If
                                            Vertices.Add(New Vertex(CType(words(11), Single), CType(words(12), Single), False))
                                    End Select
                                    If Not RBtemplate.Checked Then
                                        Triangles.Add(New Triangle(Vertices(Vertices.Count - 1), Vertices(Vertices.Count - 2), Vertices(Vertices.Count - 4)))
                                        If words(1) Like "0x2*" Then
                                            LLast(Triangles).myColour = Color.FromArgb(255, _
                                                                                       Convert.ToInt32("0x" & Mid(words(1), 4, 2), 16), _
                                                                                       Convert.ToInt32("0x" & Mid(words(1), 6, 2), 16), _
                                                                                       Convert.ToInt32("0x" & Mid(words(1), 8, 2), 16))
                                            LLast(Triangles).myColourNumber = -1
                                        ElseIf words(1) Like "0x*" Then
                                            LLast(Triangles).myColourNumber = 16
                                        Else
                                            LLast(Triangles).myColourNumber = CType(words(1), Short)
                                        End If
                                        Vertices(Vertices.Count - 1).linkedTriangles.Add(LLast(Triangles))
                                        Vertices(Vertices.Count - 2).linkedTriangles.Add(LLast(Triangles))
                                        Vertices(Vertices.Count - 4).linkedTriangles.Add(LLast(Triangles))
                                    Else
                                        templateShape.Add(New PointF(Vertices(Vertices.Count - 4).X * 1000, Vertices(Vertices.Count - 4).Y * 1000))
                                        templateShape.Add(New PointF(Vertices(Vertices.Count - 3).X * 1000, Vertices(Vertices.Count - 3).Y * 1000))
                                        templateShape.Add(New PointF(Vertices(Vertices.Count - 2).X * 1000, Vertices(Vertices.Count - 2).Y * 1000))
                                        templateShape.Add(New PointF(Vertices(Vertices.Count - 1).X * 1000, Vertices(Vertices.Count - 1).Y * 1000))
                                        templateProjectionQuad.Add(New ProjectionQuad(Vertices(Vertices.Count - 4).X, Vertices(Vertices.Count - 4).Y, _
                                                                                      Vertices(Vertices.Count - 3).X, Vertices(Vertices.Count - 3).Y, _
                                                                                      Vertices(Vertices.Count - 2).X, Vertices(Vertices.Count - 2).Y, _
                                                                                      Vertices(Vertices.Count - 1).X, Vertices(Vertices.Count - 1).Y, _
                                                                                      CType(words(2), Single), CType(words(3), Single), CType(words(4), Single), _
                                                                                      CType(words(5), Single), CType(words(6), Single), CType(words(7), Single), _
                                                                                      CType(words(8), Single), CType(words(9), Single), CType(words(10), Single), _
                                                                                      CType(words(11), Single), CType(words(12), Single), CType(words(13), Single)))
                                    End If
                                End If
                            End If
                        Loop Until DateiIn.EndOfStream
                    End Using
                    scaleVertices(False)
                    endTriangle = Triangles.Count
                    endVertex = Vertices.Count
                    If (endVertex <> startVertex AndAlso endTriangle <> startTriangle) OrElse RBtemplate.Checked Then
                        If RBnew.Checked OrElse RBtemplate.Checked Then
                            cleanupDATTriangles()
                            cleanupDATVertices()
                            paintTriangles()
                        Else
                            cleanupDATTriangles(startTriangle)
                            cleanupDATVertices(startVertex, startTriangle)
                            paintTriangles(startTriangle)
                        End If
                        endTriangle = Triangles.Count
                        endVertex = Vertices.Count
                        If RBnew.Checked OrElse RBtemplate.Checked Then
                            myMetadata = tempMetadata.Clone
                            myMetadata.isMainMetadata = True
                            For r As Integer = 0 To 3
                                For c As Integer = 0 To 3
                                    myMetadata.matrix(r, c) = 0
                                    If c = r Then myMetadata.matrix(r, c) = 1
                                Next
                            Next
                        Else
                            temp_center = New Vertex(0, 0, False, False)
                            For i As Integer = startVertex To endVertex - 1
                                Dim vert As Vertex = Vertices(i)
                                temp_center += vert
                            Next
                            temp_center.X /= (endVertex - startVertex)
                            temp_center.Y /= (endVertex - startVertex)
                            Dim mindist As Double = Double.PositiveInfinity
                            Dim minvert As Vertex = New Vertex(0, 0, False, False)
                            For i As Integer = startVertex To endVertex - 1
                                Dim vert As Vertex = Vertices(i)
                                Dim dist As Double
                                dist = vert.dist(temp_center)
                                If dist < mindist Then
                                    minvert = vert
                                    mindist = dist
                                End If
                            Next
                            Primitives.Add(New Primitive(minvert.X, minvert.Y, 0, 0, "subfile" & Now.Ticks, minvert.vertexID))
                            PrimitivesHMap.Add(primitiveIDglobal, Primitives.Count - 1)
                            PrimitivesMetadataHMap.Add(LLast(Primitives).primitiveName, tempMetadata.Clone)
                            PrimitivesMetadataHMap.Item(LLast(Primitives).primitiveName).mData(1) = ""
                            For i As Integer = startVertex To endVertex - 1
                                Vertices(i).groupindex = primitiveIDglobal
                            Next
                            For i As Integer = startTriangle To endTriangle - 1
                                Triangles(i).groupindex = primitiveIDglobal
                            Next
                        End If
                    End If
                    If RBtemplate.Checked Then myMetadata.recommendedMode = projectMode
                    If Vertices.Count = 0 AndAlso showDialog Then
                        If tempMetadata.mData(1) = "" Then
                            Dim result As MsgBoxResult = MsgBox("The DAT file contains no information which could be evaluated. Please try to open a subfile of this part.", MsgBoxStyle.OkOnly + MsgBoxStyle.MsgBoxSetForeground + MsgBoxStyle.Information, "Info:")
                        Else
                            Dim result As MsgBoxResult = MsgBox("The DAT file contains no information which could be evaluated. Please try to open a subfile of " & tempMetadata.mData(1) & ".", MsgBoxStyle.OkOnly + MsgBoxStyle.MsgBoxSetForeground + MsgBoxStyle.Information, "Info:")
                        End If
                    End If
                    ProjectionQuadTimer.Enabled = templateProjectionQuad.Count > 0
                Catch ex As Exception
                    If MsgBox("Invalid DAT format!", MsgBoxStyle.RetryCancel + MsgBoxStyle.MsgBoxSetForeground + MsgBoxStyle.Exclamation, "ERROR") = MsgBoxResult.Retry Then
                        GoTo newTry
                    End If
                    backgroundPicture = My.Resources.temp
                    TBImage.Text = ""
                    If showDialog Then newPattern()
                End Try
            End If
        End If
        unsavedChanges = False
        isLoading = False
        MenuStrip1.Enabled = True
        Me.ToolStrip1.Enabled = True
        Me.ColourToolStrip.Enabled = True
        GBImage.Enabled = True
        GBSnap.Enabled = True
        Me.Refresh()
    End Sub

    Private Sub inlinePrimitive(ByVal colour As String, _
                                ByVal M11 As Double, ByVal M12 As Double, ByVal M13 As Double, ByVal M14 As Double, _
                                ByVal M21 As Double, ByVal M22 As Double, ByVal M23 As Double, ByVal M24 As Double, _
                                ByVal M31 As Double, ByVal M32 As Double, ByVal M33 As Double, ByVal M34 As Double, _
 _
                                ByVal N11 As Double, ByVal N12 As Double, ByVal N13 As Double, ByVal N14 As Double, _
                                ByVal N21 As Double, ByVal N22 As Double, ByVal N23 As Double, ByVal N24 As Double, _
                                ByVal N31 As Double, ByVal N32 As Double, ByVal N33 As Double, ByVal N34 As Double, _
                                ByVal name As String, ByRef subFileList As List(Of String), Optional ByVal scale As Boolean = True)

        Dim matrix(3, 3) As Double

        Dim matrix2(3, 3) As Double
        Dim matrix3(3, 3) As Double

        matrix2(0, 0) = M11
        matrix2(0, 1) = M12
        matrix2(0, 2) = M13
        matrix2(0, 3) = M14
        matrix2(1, 0) = M21
        matrix2(1, 1) = M22
        matrix2(1, 2) = M23
        matrix2(1, 3) = M24
        matrix2(2, 0) = M31
        matrix(2, 1) = M32
        matrix2(2, 2) = M33
        matrix2(2, 3) = M34
        matrix2(3, 0) = 0
        matrix2(3, 1) = 0
        matrix2(3, 2) = 0
        matrix2(3, 3) = 1
        matrix(0, 0) = N11
        matrix(0, 1) = N12
        matrix(0, 2) = N13
        matrix(0, 3) = N14
        matrix(1, 0) = N21
        matrix(1, 1) = N22
        matrix(1, 2) = N23
        matrix(1, 3) = N24
        matrix(2, 0) = N31
        matrix(2, 1) = N32
        matrix(2, 2) = N33
        matrix(2, 3) = N34
        matrix(3, 0) = 0
        matrix(3, 1) = 0
        matrix(3, 2) = 0
        matrix(3, 3) = 1

        matrix3(0, 0) = matrix2(0, 0) * matrix(0, 0) + matrix2(0, 1) * matrix(1, 0) + matrix2(0, 2) * matrix(2, 0)
        matrix3(0, 1) = matrix2(0, 0) * matrix(0, 1) + matrix2(0, 1) * matrix(1, 1) + matrix2(0, 2) * matrix(2, 1)
        matrix3(0, 2) = matrix2(0, 0) * matrix(0, 2) + matrix2(0, 1) * matrix(1, 2) + matrix2(0, 2) * matrix(2, 2)
        matrix3(0, 3) = matrix2(0, 0) * matrix(0, 3) + matrix2(0, 1) * matrix(1, 3) + matrix2(0, 2) * matrix(2, 3) + matrix2(0, 3)

        matrix3(1, 0) = matrix2(1, 0) * matrix(0, 0) + matrix2(1, 1) * matrix(1, 0) + matrix2(1, 2) * matrix(2, 0)
        matrix3(1, 1) = matrix2(1, 0) * matrix(0, 1) + matrix2(1, 1) * matrix(1, 1) + matrix2(1, 2) * matrix(2, 1)
        matrix3(1, 2) = matrix2(1, 0) * matrix(0, 2) + matrix2(1, 1) * matrix(1, 2) + matrix2(1, 2) * matrix(2, 2)
        matrix3(1, 3) = matrix2(1, 0) * matrix(0, 3) + matrix2(1, 1) * matrix(1, 3) + matrix2(1, 2) * matrix(2, 3) + matrix2(1, 3)

        matrix3(2, 0) = matrix2(2, 0) * matrix(0, 0) + matrix2(2, 1) * matrix(1, 0) + matrix2(2, 2) * matrix(2, 0)
        matrix3(2, 1) = matrix2(2, 0) * matrix(0, 1) + matrix2(2, 1) * matrix(1, 1) + matrix2(2, 2) * matrix(2, 1)
        matrix3(2, 2) = matrix2(2, 0) * matrix(0, 2) + matrix2(2, 1) * matrix(1, 2) + matrix2(2, 2) * matrix(2, 2)
        matrix3(2, 3) = matrix2(2, 0) * matrix(0, 3) + matrix2(2, 1) * matrix(1, 3) + matrix2(2, 2) * matrix(2, 3) + matrix2(2, 3)

        matrix3(3, 0) = 0
        matrix3(3, 1) = 0
        matrix3(3, 2) = 0
        matrix3(3, 3) = 1

        Dim start As Integer = Vertices.Count
        Dim start2 As Integer = Triangles.Count
        If (name Like "48\*" AndAlso (name Like "48\#-#ring#.dat" OrElse name Like "48\#-#rin##.dat" OrElse name Like "48\#-##rin#.dat" OrElse name Like "48\#-##ri##.dat" OrElse name Like "48\##-##ri#.dat")) OrElse _
            (name Like "48\*aring.dat") OrElse _
            (name Like "48\*chrd.dat") OrElse _
            (name Like "*chrd.dat") OrElse _
            (name Like "48\*ndis.dat") OrElse _
            (name Like "*ndis.dat") OrElse _
            (name Like "#-#ring#.dat" OrElse name Like "#-#rin##.dat" OrElse name Like "#-##rin#.dat" OrElse name Like "#-##ri##.dat" OrElse name Like "##-##ri#.dat") OrElse _
            (name Like "48\*disc.dat") OrElse _
            (name Like "*disc.dat") Then
            If My.Computer.FileSystem.FileExists(ldrawPath & "\P\" & name) Then
newTry:
                Dim mySubfiles As New List(Of String)
                Dim startVertex As Integer = Vertices.Count
                Dim endVertex As Integer = 0
                Dim startTriangle As Integer = Triangles.Count
                Dim endTriangle As Integer = 0
                BtnMove.PerformClick()
                isLoading = True

                Try

                    Using DateiIn As StreamReader = My.Computer.FileSystem.OpenTextFileReader(ldrawPath & "\P\" & name, System.Text.Encoding.GetEncoding(20127))
                        Dim textline As String
                        Vertices.Add(New Vertex(0, 0, False))
                        Do
                            textline = DateiIn.ReadLine().Trim()
                            If textline <> Nothing Then

                                Dim ttextline As String = textline
                                textline = Replace(Replace(textline, ".", comma).ToLower, "  ", " ")
                                Dim oldlenght As Integer = textline.Length
                                Do
                                    textline = Replace(textline, "  ", " ")
                                Loop Until oldlenght = textline.Length
                                Dim words() As String = textline.Split(CType(" ", Char))

                                If words(0) Like "*3*" AndAlso words.Length = 11 Then

                                    Vertices.Add(New Vertex(-CType(words(2), Single), CType(-words(4), Single), False))
                                    Vertices.Add(New Vertex(-CType(words(5), Single), CType(-words(7), Single), False))
                                    Vertices.Add(New Vertex(-CType(words(8), Single), CType(-words(10), Single), False))

                                    Triangles.Add(New Triangle(Vertices(Vertices.Count - 1), Vertices(Vertices.Count - 2), Vertices(Vertices.Count - 3)))
                                    If words(1) Like "0x2*" Then
                                        LLast(Triangles).myColour = Color.FromArgb(255, _
                                                                                   Convert.ToInt32("0x" & Mid(words(1), 4, 2), 16), _
                                                                                   Convert.ToInt32("0x" & Mid(words(1), 6, 2), 16), _
                                                                                   Convert.ToInt32("0x" & Mid(words(1), 8, 2), 16))
                                        LLast(Triangles).myColourNumber = -1
                                    ElseIf words(1) Like "0x*" Then
                                        LLast(Triangles).myColourNumber = 16
                                    Else
                                        LLast(Triangles).myColourNumber = CType(words(1), Short)
                                    End If
                                    Vertices(Vertices.Count - 1).linkedTriangles.Add(LLast(Triangles))
                                    Vertices(Vertices.Count - 2).linkedTriangles.Add(LLast(Triangles))
                                    Vertices(Vertices.Count - 3).linkedTriangles.Add(LLast(Triangles))

                                ElseIf words(0) Like "*4*" AndAlso words.Length = 14 Then

                                    '4 2, 7 5, 10 8, 13 11
                                    Vertices.Add(New Vertex(-CType(words(2), Single), -CType(words(4), Single), False))
                                    Vertices.Add(New Vertex(-CType(words(5), Single), -CType(words(7), Single), False))
                                    Vertices.Add(New Vertex(-CType(words(8), Single), -CType(words(10), Single), False))
                                    Triangles.Add(New Triangle(Vertices(Vertices.Count - 1), Vertices(Vertices.Count - 2), Vertices(Vertices.Count - 3)))
                                    If words(1) Like "0x2*" Then
                                        LLast(Triangles).myColour = Color.FromArgb(255, _
                                                                                   Convert.ToInt32("0x" & Mid(words(1), 4, 2), 16), _
                                                                                   Convert.ToInt32("0x" & Mid(words(1), 6, 2), 16), _
                                                                                   Convert.ToInt32("0x" & Mid(words(1), 8, 2), 16))
                                        LLast(Triangles).myColourNumber = -1
                                    ElseIf words(1) Like "0x*" Then
                                        LLast(Triangles).myColourNumber = 16
                                    Else
                                        LLast(Triangles).myColourNumber = CType(words(1), Short)
                                    End If
                                    Vertices(Vertices.Count - 1).linkedTriangles.Add(LLast(Triangles))
                                    Vertices(Vertices.Count - 2).linkedTriangles.Add(LLast(Triangles))
                                    Vertices(Vertices.Count - 3).linkedTriangles.Add(LLast(Triangles))
                                    Vertices.Add(New Vertex(-CType(words(11), Single), -CType(words(13), Single), False))
                                    Triangles.Add(New Triangle(Vertices(Vertices.Count - 1), Vertices(Vertices.Count - 2), Vertices(Vertices.Count - 4)))
                                    If words(1) Like "0x2*" Then
                                        LLast(Triangles).myColour = Color.FromArgb(255, _
                                                                                   Convert.ToInt32("0x" & Mid(words(1), 4, 2), 16), _
                                                                                   Convert.ToInt32("0x" & Mid(words(1), 6, 2), 16), _
                                                                                   Convert.ToInt32("0x" & Mid(words(1), 8, 2), 16))
                                        LLast(Triangles).myColourNumber = -1
                                    ElseIf words(1) Like "0x*" Then
                                        LLast(Triangles).myColourNumber = 16
                                    Else
                                        LLast(Triangles).myColourNumber = CType(words(1), Short)
                                    End If
                                    Vertices(Vertices.Count - 1).linkedTriangles.Add(LLast(Triangles))
                                    Vertices(Vertices.Count - 2).linkedTriangles.Add(LLast(Triangles))
                                    Vertices(Vertices.Count - 4).linkedTriangles.Add(LLast(Triangles))
                                End If
                            End If
                        Loop Until DateiIn.EndOfStream
                    End Using
                    endTriangle = Triangles.Count
                    endVertex = Vertices.Count
                    If (endVertex <> startVertex AndAlso endTriangle <> startTriangle) Then

                        cleanupDATTriangles(startTriangle)
                        cleanupDATVertices(startVertex, startTriangle)
                        paintTriangles(startTriangle)
                        endTriangle = Triangles.Count
                        endVertex = Vertices.Count

                        Triangles.Add(New Triangle(Vertices(startVertex), Vertices(startVertex), Vertices(startVertex + 1)))
                        Vertices(startVertex).linkedTriangles.Add(LLast(Triangles))
                        Vertices(startVertex + 1).linkedTriangles.Add(LLast(Triangles))

                        Primitives.Add(New Primitive(primitiveCenter.X, primitiveCenter.Y, 0, 0, name, Vertices(startVertex).vertexID))
                        For i As Integer = startVertex To endVertex - 1
                            Vertices(i).groupindex = primitiveIDglobal
                        Next
                        For i As Integer = startTriangle To endTriangle
                            Triangles(i).groupindex = primitiveIDglobal
                        Next
                    Else
                        isLoading = False
                        Me.Refresh()
                        GoTo doNothing
                    End If
                Catch ex As Exception
                    isLoading = False
                    Me.Refresh()
                    GoTo doNothing
                End Try
                isLoading = False
                Me.Refresh()
            Else
                isLoading = False
                Me.Refresh()
                GoTo doNothing
            End If
            GoTo doMatrix
        Else
            isLoading = False
            Me.Refresh()
            GoTo doNothing
        End If
        If name Like "*s\*.dat" Then
            subFileList.Add(name)
        End If
        GoTo doNothing
doMatrix:
        PrimitivesHMap.Add(primitiveIDglobal, Primitives.Count - 1)
        Dim lastPrimitive As Primitive = LLast(Primitives)
        If scale Then

            lastPrimitive.myColourNumber = CType(lastColourNumber, Short)
            lastPrimitive.myColour = lastColour
        Else
            If colour Like "0x*" Then
                lastPrimitive.myColour = Color.FromArgb(Convert.ToInt32(colour, 16))
                lastPrimitive.myColourNumber = -1
            Else
                lastPrimitive.myColourNumber = CType(colour, Short)
                If colourHMap.ContainsKey(lastPrimitive.myColourNumber) Then lastPrimitive.myColour = colourHMap(lastPrimitive.myColourNumber) Else lastPrimitive.myColour = Nothing
            End If
        End If
        lastPrimitive.matrix = matrix3.Clone
        lastPrimitive.matrix(0, 3) = lastPrimitive.matrix(0, 3) * 1000 + primitiveCenter.X
        lastPrimitive.matrix(1, 3) = lastPrimitive.matrix(1, 3) * 1000 + primitiveCenter.Y
        Dim tx, ty As Single
        If scale Then
            For i = start To Vertices.Count - 1
                tx = matrix3(0, 0) * Vertices(i).X + matrix3(0, 1) * Vertices(i).Y + matrix3(0, 3) * 1000
                ty = matrix3(1, 0) * Vertices(i).X + matrix3(1, 1) * Vertices(i).Y + matrix3(1, 3) * 1000
                Vertices(i).X = tx * 1000 + primitiveCenter.X
                Vertices(i).Y = ty * 1000 + primitiveCenter.Y
            Next
        Else
            For i = start To Vertices.Count - 1
                tx = matrix3(0, 0) * Vertices(i).X + matrix3(0, 1) * Vertices(i).Y + matrix3(0, 3)
                ty = matrix3(1, 0) * Vertices(i).X + matrix3(1, 1) * Vertices(i).Y + matrix3(1, 3)
                Vertices(i).X = tx
                Vertices(i).Y = ty
            Next
        End If
        Exit Sub
doNothing:
    End Sub

    Private Sub DebugToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles DebugToolStripMenuItem.Click
        Label1.Visible = DebugToolStripMenuItem.Checked
        RTBDebug.Visible = DebugToolStripMenuItem.Checked
    End Sub

    Public Shared Function LLast(Of TSource)( _
     ByVal source As List(Of TSource) _
    ) As TSource
        Return source(source.Count - 1)
    End Function

    Public Shared Function Atan2(ByVal y As Double, ByVal x As Double) As Double
        Dim ta As Double
        ta = Math.Atan2(y, x)
        If ta < 0 Then ta += 2 * Math.PI
        Return ta
    End Function

    Private Sub ToolStripMenuItem1_Click_1(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ResetViewToolStripMenuItem.Click
        toffsetX = 0
        toffsetY = 0
        offsetX = 0
        offsetY = 0
        zoomfactor = 1
        zoomlevel = 0
        LblDebug.Text = "Zoom: 100%"
        Me.Refresh()
    End Sub

    Private Sub AboutToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles AboutToolStripMenuItem.Click
        MsgBox("LD - Pattern Creator - Version 1.2.1 Beta" & vbCrLf & "[Full backwards compatibility to version 1.0 and 1.1]" & vbCrLf & vbCrLf & "This software is licensed under the terms of the MIT - License." & vbCrLf & "(C) Nils Schmidt 2010 - 2011" & vbCrLf & "This program uses Rectifier.exe by permission of the author Philippe E. Hurbain - (C) 2009.", MsgBoxStyle.OkOnly + MsgBoxStyle.MsgBoxSetForeground, "About LD - Pattern Creator")
    End Sub

    Private Sub Timer1_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Timer1.Tick
        SBZoom.Focus()
        Timer1.Enabled = False
        Timer1.Dispose()
    End Sub

    Private Sub Timer2_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Timer2.Tick
        Me.Close()
    End Sub

    Private Sub MainForm_Shown(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Shown
        Me.BackColor = myColours.background
        Dim tmx, tmy As Integer
        tmx = System.Windows.Forms.Cursor.Position.X
        tmy = System.Windows.Forms.Cursor.Position.Y
        Me.Activate()
        If mySettings.startWithFullscreen Then
            Me.WindowState = FormWindowState.Maximized
            Me.BringToFront()
            getOffset = True
            moveMouseAbsoluteLeftClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2, offset_Y + Me.ClientSize.Height / 2)
            Me.Activate()
            Me.BringToFront()
        Else
            Me.WindowState = FormWindowState.Normal
            Me.BringToFront()
            getOffset = True
            moveMouseAbsoluteLeftClick(Me.Location.X + Me.Width / 2 + offset_X, Me.Location.Y + offset_Y + Me.Height / 2)
            Me.Activate()
            Me.BringToFront()
        End If
    End Sub

    Private Sub ToolStrip1_MouseEnter(ByVal sender As Object, ByVal e As System.EventArgs) Handles ToolStrip1.MouseEnter
        Me.Cursor = Cursors.Arrow
    End Sub

    Private Sub ToolStrip2_MouseEnter(ByVal sender As Object, ByVal e As System.EventArgs) Handles ColourToolStrip.MouseEnter
        Me.Cursor = Cursors.Arrow
    End Sub

    Private Sub GBImage_MouseEnter(ByVal sender As Object, ByVal e As System.EventArgs) Handles GBImage.MouseEnter
        Me.Cursor = Cursors.Arrow
    End Sub

    Private Sub MenuStrip1_MouseEnter(ByVal sender As Object, ByVal e As System.EventArgs) Handles MenuStrip1.MouseEnter
        Me.Cursor = Cursors.Arrow
    End Sub

    Private Sub GBSnap_MouseEnter(ByVal sender As Object, ByVal e As System.EventArgs) Handles GBSnap.MouseEnter
        Me.Cursor = Cursors.Arrow
    End Sub

    Private Sub BtnMode_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles BtnMode.Click
        If BtnMode.Text = "Primitive Mode" Then
            For Each tri As Triangle In SelectedTriangles
                tri.selected = False
            Next
            For Each vert As Vertex In SelectedVertices
                vert.selected = False
            Next
            SelectedTriangles.Clear()
            SelectedVertices.Clear()
            VerticesModeToolStripMenuItem.PerformClick()
        Else
            If BtnMode.Text = "Triangle Mode" Then
                PrimitiveModeToolStripMenuItem.PerformClick()
            Else
                TriangleModeToolStripMenuItem.PerformClick()
            End If
        End If
    End Sub

    Dim unit As String = "LDU"
    Public factor As Single = 1
    Private Sub LDUToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles LDUToolStripMenuItem.Click
        conversionEnabled = True
        NUDMoveSnap.DecimalPlaces = 0
        NUDGrid.DecimalPlaces = 1
        If Not LDUToolStripMenuItem.Checked Then LDUToolStripMenuItem.Checked = True
        InchToolStripMenuItem.Checked = False
        MillimeterToolStripMenuItem.Checked = False
        unit = "LDU"
        factor = 1
        LblSize.Text = imgScale * backgroundPicture.Width / 1000 * factor & " x " & vbCrLf & imgScale * backgroundPicture.Height / 1000 * factor & unit
        LblMoveSnap.Text = "Move Snap (1/1000 " & unit & "):"
        LblGridSize.Text = "Grid Size (1/1000 " & unit & "):"
        LblUnit1.Text = unit
        LblUnit2.Text = unit
        Dim temp As Single
        temp = Me.moveSnap * factor
        NUDMoveSnap.Minimum = 1
        NUDMoveSnap.Maximum = 10000
        clip(temp, NUDMoveSnap.Minimum, NUDMoveSnap.Maximum)
        NUDMoveSnap.Value = temp
        temp = Me.rasterSnap * factor
        NUDGrid.Minimum = 0.1
        NUDGrid.Maximum = 10
        clip(temp, NUDGrid.Minimum, NUDGrid.Maximum)
        NUDGrid.Value = temp
        If GBVertex.Visible AndAlso SelectedVertices.Count = 1 Then
            NUDVertX.Value = SelectedVertices(0).X * factor / 1000
            NUDVertY.Value = SelectedVertices(0).Y * factor / 1000
        End If
        conversionEnabled = False
    End Sub

    Private Sub MillimeterToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MillimeterToolStripMenuItem.Click
        conversionEnabled = True
        NUDMoveSnap.DecimalPlaces = 1
        NUDGrid.DecimalPlaces = 2
        If Not MillimeterToolStripMenuItem.Checked Then MillimeterToolStripMenuItem.Checked = True
        LDUToolStripMenuItem.Checked = False
        InchToolStripMenuItem.Checked = False
        unit = "mm"
        factor = 0.4F
        LblSize.Text = imgScale * backgroundPicture.Width / 1000 * factor & " x " & vbCrLf & imgScale * backgroundPicture.Height / 1000 * factor & unit
        LblMoveSnap.Text = "Move Snap (1/1000 " & unit & "):"
        LblGridSize.Text = "Grid Size (1/1000 " & unit & "):"
        LblUnit1.Text = unit
        LblUnit2.Text = unit
        Dim temp As Single
        temp = Me.moveSnap * factor
        NUDMoveSnap.Minimum = 0.4
        NUDMoveSnap.Maximum = 4000
        clip(temp, NUDMoveSnap.Minimum, NUDMoveSnap.Maximum)
        NUDMoveSnap.Value = temp
        temp = Me.rasterSnap * factor
        NUDGrid.Minimum = 0.4 * 0.1
        NUDGrid.Maximum = 0.4 * 10
        clip(temp, NUDGrid.Minimum, NUDGrid.Maximum)
        NUDGrid.Value = temp
        If GBVertex.Visible AndAlso SelectedVertices.Count = 1 Then
            NUDVertX.Value = SelectedVertices(0).X * factor / 1000
            NUDVertY.Value = SelectedVertices(0).Y * factor / 1000
        End If
        conversionEnabled = False
    End Sub

    Private Sub InchToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles InchToolStripMenuItem.Click
        conversionEnabled = True
        NUDMoveSnap.DecimalPlaces = 3
        NUDGrid.DecimalPlaces = 4
        If Not InchToolStripMenuItem.Checked Then InchToolStripMenuItem.Checked = True
        LDUToolStripMenuItem.Checked = False
        MillimeterToolStripMenuItem.Checked = False
        unit = "inch"
        factor = 0.016F
        LblSize.Text = imgScale * backgroundPicture.Width / 1000 * factor & " x " & vbCrLf & imgScale * backgroundPicture.Height / 1000 * factor & unit
        LblMoveSnap.Text = "Move Snap (1/1000 " & unit & "):"
        LblGridSize.Text = "Grid Size (1/1000 " & unit & "):"
        LblUnit1.Text = unit
        LblUnit2.Text = unit
        Dim temp As Single
        temp = Me.moveSnap * factor
        NUDMoveSnap.Minimum = 0.016
        NUDMoveSnap.Maximum = 160
        clip(temp, NUDMoveSnap.Minimum, NUDMoveSnap.Maximum)
        NUDMoveSnap.Value = temp
        temp = Me.rasterSnap * factor
        NUDGrid.Minimum = 0.016 * 0.1
        NUDGrid.Maximum = 0.016 * 10
        clip(temp, NUDGrid.Minimum, NUDGrid.Maximum)
        NUDGrid.Value = temp
        If GBVertex.Visible AndAlso SelectedVertices.Count = 1 Then
            NUDVertX.Value = SelectedVertices(0).X * factor / 1000
            NUDVertY.Value = SelectedVertices(0).Y * factor / 1000
        End If
        conversionEnabled = False
    End Sub

    Private Sub BtnImageClose_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BtnImageClose.Click
        ImageToolStripMenuItem.PerformClick()
    End Sub

    Private Sub BtnPrefsClose_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BtnPrefsClose.Click
        Me.SnappingToolStripMenuItem.PerformClick()
    End Sub

    Private Sub BtnImageClose_MouseEnter(ByVal sender As Object, ByVal e As System.EventArgs) Handles BtnImageClose.MouseEnter
        Me.Cursor = Cursors.Arrow
    End Sub

    Private Sub BtnPrefsClose_MouseEnter(ByVal sender As Object, ByVal e As System.EventArgs) Handles BtnPrefsClose.MouseEnter
        Me.Cursor = Cursors.Arrow
    End Sub

    Private Sub PerformanceEnabledToolStripMenuItem_CheckedChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles PerformanceEnabledToolStripMenuItem.CheckedChanged
        If PerformanceEnabledToolStripMenuItem.Checked Then
            If Me.isLoading OrElse MsgBox("Enabling the performance mode will speed up the reaction time of this program, but the undo function will be deactivated!" & vbCrLf & "Do you really want to activate it?", MsgBoxStyle.YesNo + MsgBoxStyle.MsgBoxSetForeground + MsgBoxStyle.DefaultButton2 + MsgBoxStyle.Exclamation, "Warning:") = MsgBoxResult.Yes Then
                mySettings.performanceMode = True
                UndoToolStripMenuItem.Enabled = False
                RedoToolStripMenuItem.Enabled = False
                historyVertices.Clear()
                historyTriangles.Clear()
                historySelectedVertices.Clear()
                historySelectedTriangles.Clear()
                historyPrimitives.Clear()
                historyPrimitivesHash.Clear()
                historyPrimitivesMetadataHash.Clear()
            Else
                PerformanceEnabledToolStripMenuItem.Checked = False
                mySettings.performanceMode = False
            End If
        ElseIf historyVertices.Count = 0 Then
            zeiger = 0
            addHistoryI(withDataFrom.All)
            addHistoryII(withDataFrom.All)
        End If
    End Sub

    Private Sub BtnAddToGroup_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BtnAddToGroup.Click
        addHistoryI(withDataFrom.Primitives + withDataFrom.PrimitivesHash + withDataFrom.SelectedVertices + withDataFrom.SelectedTriangles)

        For Each tri As Triangle In SelectedTriangles
            tri.selected = False
        Next

        temp_center = New Vertex(0, 0, False, False)
        For Each vert As Vertex In SelectedVertices
            temp_center += vert
        Next
        temp_center.X /= SelectedVertices.Count
        temp_center.Y /= SelectedVertices.Count

        Dim mindist As Double = Double.PositiveInfinity
        Dim minvert As Vertex = New Vertex(0, 0, False, False)
        For Each vert As Vertex In SelectedVertices
            Dim dist As Double
            dist = vert.dist(temp_center)
            If dist < mindist Then
                minvert = vert
                mindist = dist
            End If
        Next
        Primitives.Add(New Primitive(minvert.X, minvert.Y, 0, 0, "subfile" & Now.Ticks, minvert.vertexID))
        PrimitivesHMap.Add(primitiveIDglobal, Primitives.Count - 1)
        PrimitivesMetadataHMap.Add(LLast(Primitives).primitiveName, New Metadata())
        For Each vert As Vertex In SelectedVertices
            vert.groupindex = primitiveIDglobal
            vert.selected = False
        Next
        For Each tri As Triangle In SelectedTriangles
            tri.groupindex = primitiveIDglobal
            tri.selected = False
        Next
        SelectedVertices.Clear()
        SelectedTriangles.Clear()
        BtnAddToGroup.Enabled = False
        Me.Refresh()

        addHistoryII(withDataFrom.Primitives + withDataFrom.PrimitivesHash + withDataFrom.SelectedVertices + withDataFrom.SelectedTriangles)
    End Sub

    Private Sub BtnUngroup_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BtnUngroup.Click
        Dim myindex As Integer = CType(PrimitivesHMap(SelectedVertices(0).groupindex), Integer)
        addHistoryI(withDataFrom.Primitives + withDataFrom.PrimitivesHash + withDataFrom.SelectedVertices + withDataFrom.SelectedTriangles)
        PrimitivesHMap.Remove(SelectedVertices(0).groupindex)
        If PrimitivesHMap.Keys.Count > 0 Then
            Dim klist(PrimitivesHMap.Keys.Count - 1) As Integer
            Dim counter As Integer = 0
            For Each key As Integer In PrimitivesHMap.Keys
                klist(counter) = key
                counter += 1
            Next
            For i As Integer = 0 To PrimitivesHMap.Keys.Count - 1
                If PrimitivesHMap.Item(klist(i)) > myindex Then
                    PrimitivesHMap.Item(klist(i)) -= 1
                End If
            Next i
        End If
        Primitives.RemoveAt(myindex)
        For Each vert As Vertex In SelectedVertices
            vert.groupindex = -1
            vert.selected = False
        Next
        For Each tri As Triangle In SelectedTriangles
            tri.groupindex = -1
            tri.selected = False
        Next
        SelectedVertices.Clear()
        SelectedTriangles.Clear()
        BtnUngroup.Enabled = False
        addHistoryII(withDataFrom.Primitives + withDataFrom.PrimitivesHash + withDataFrom.SelectedVertices + withDataFrom.SelectedTriangles)
        Me.Refresh()
    End Sub

    Private Sub MetadataToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MetadataToolStripMenuItem.Click
        addHistoryI(withDataFrom.PrimitivesMetadataHash)
        MetadataDialog.ShowDialog()
        addHistoryII(withDataFrom.PrimitivesMetadataHash)
    End Sub

    Private CDialog As Form
    Private Sub ImportFromBitmapToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ImportFromBitmapToolStripMenuItem.Click
        If unsavedChanges AndAlso ShowAllWarningsToolStripMenuItem.Checked Then
            Dim result As MsgBoxResult = MsgBox("You have unsaved changes - would you like to save them first?", MsgBoxStyle.YesNoCancel + MsgBoxStyle.MsgBoxSetForeground + MsgBoxStyle.Information, "Question:")
            If result = MsgBoxResult.Yes Then SaveToolStripMenuItem.PerformClick()
            If result = MsgBoxResult.Cancel Then Exit Sub
        End If
        If OpenBitmap.ShowDialog = Windows.Forms.DialogResult.OK Then
            If OpenBitmap.FileName <> "" Then
                Try
                    Dim bmp2 As New Bitmap(OpenBitmap.FileName)
                Catch ex As Exception
                    While MsgBox("This is not a vaild image file.", MsgBoxStyle.OkOnly + MsgBoxStyle.MsgBoxSetForeground + MsgBoxStyle.Information, "Info:") <> MsgBoxResult.Ok : End While
                    Exit Sub
                End Try

                Control.CheckForIllegalCrossThreadCalls = False
                Me.BackgroundBitmapImport.RunWorkerAsync()
                CDialog = New StatusDialog
                If CDialog.ShowDialog() = Windows.Forms.DialogResult.Cancel Then
                    Try
                        BackgroundBitmapImport.CancelAsync()
                    Catch
                    End Try
                End If
            End If
        End If
    End Sub

    Private Sub BackgroundBitmapImport_DoWork( _
    ByVal sender As Object, ByVal e As DoWorkEventArgs) _
    Handles BackgroundBitmapImport.DoWork

        ' Do not access the form's BackgroundWorker reference directly.
        ' Instead, use the reference provided by the sender parameter.
        Dim bw As BackgroundWorker = CType(sender, BackgroundWorker)

        ' Extract the argument.
        Dim TutorialNumber As Integer = e.Argument

        ' Start the time-consuming operation.
        e.Result = BackgroundBitmapImportDo(bw)

        ' If the operation was canceled by the user, 
        ' set the DoWorkEventArgs.Cancel property to true.
        If bw.CancellationPending Then
            e.Cancel = True
        End If

    End Sub

    Private Sub BackgroundBitmapImport_RunWorkerCompleted( _
    ByVal sender As Object, ByVal e As RunWorkerCompletedEventArgs) _
    Handles BackgroundBitmapImport.RunWorkerCompleted

        If e.Cancelled Then
            ' The user canceled the operation.
        ElseIf Not (e.Error Is Nothing) Then
            ' There was an error during the operation.
        Else
            ' The operation completed normally.
        End If
    End Sub

    Private Function BackgroundBitmapImportDo(ByVal bw As BackgroundWorker) As String
        CType(CDialog.Controls("PBStatus"), ProgressBar).Minimum = 0
        CType(CDialog.Controls("PBStatus"), ProgressBar).Value = 0
        CType(CDialog.Controls("PBStatus"), ProgressBar).Maximum = 100
        Thread.Sleep(500)
        BtnMove.PerformClick()
        TriangleModeToolStripMenuItem.PerformClick()
        If Not BtnColours.Checked Then BtnColours.PerformClick()
        VerticesModeToolStripMenuItem.PerformClick()

        CDialog.Controls("LblStatus").Text = "(1/4) Loading.."
        isLoading = True
        MenuStrip1.Enabled = False
        Me.ToolStrip1.Enabled = False
        Me.ColourToolStrip.Enabled = False
        GBImage.Enabled = False
        GBSnap.Enabled = False
        Dim bmp As New Bitmap(OpenBitmap.FileName)

        addHistoryI(withDataFrom.All)
        newPattern()

        Dim vertexmap(bmp.Width + 1, bmp.Height + 1) As Vertex
        CDialog.Controls("LblStatus").Text = "(2/4) Parsing.. "
        For y As Integer = 2 To bmp.Height + 1
            CType(CDialog.Controls("PBStatus"), ProgressBar).Value = Fix(y / bmp.Height * 50) : If bw.CancellationPending Then GoTo Abort

            For x As Integer = 1 To bmp.Width + 1
                Vertices.Add(New Vertex(x, y, False))
                vertexmap(x, y) = LLast(Vertices)
            Next
        Next
        CDialog.Controls("LblStatus").Text = "(2/4) Parsing.. "
        For y As Integer = 2 To bmp.Height
            CType(CDialog.Controls("PBStatus"), ProgressBar).Value = Fix(y / bmp.Height * 50 + 50) : If bw.CancellationPending Then GoTo Abort
            For x As Integer = bmp.Width To 1 Step -1
                Dim col1 As Color = bmp.GetPixel(bmp.Width - x, y - 1)
                Dim tCol As Color
                Dim index As Integer = -1
                If Color.FromArgb(255, 111, 255).ToArgb = col1.ToArgb Then
                    index = 16
                    tCol = Nothing
                Else
                    For Each j As Single In colourHMap.Keys
                        If colourHMap.Item(j).ToArgb = col1.ToArgb Then
                            tCol = colourHMap.Item(j)
                            index = j : Exit For
                        End If
                    Next
                    If index = -1 Then
                        tCol = col1
                    End If
                End If
                Triangles.Add(New Triangle(vertexmap(x, y), vertexmap(x + 1, y), vertexmap(x + 1, y + 1))) : LLast(Triangles).myColourNumber = index : LLast(Triangles).myColour = tCol
                vertexmap(x, y).linkedTriangles.Add(LLast(Triangles))
                vertexmap(x + 1, y).linkedTriangles.Add(LLast(Triangles))
                vertexmap(x + 1, y + 1).linkedTriangles.Add(LLast(Triangles))
                Triangles.Add(New Triangle(vertexmap(x + 1, y + 1), vertexmap(x, y + 1), vertexmap(x, y))) : LLast(Triangles).myColourNumber = index : LLast(Triangles).myColour = tCol
                vertexmap(x + 1, y + 1).linkedTriangles.Add(LLast(Triangles))
                vertexmap(x, y + 1).linkedTriangles.Add(LLast(Triangles))
                vertexmap(x, y).linkedTriangles.Add(LLast(Triangles))
            Next
        Next
        paintTriangles()
        Dim old_Count As Integer = 0
        For it As Integer = 0 To 1000
            CDialog.Controls("LblStatus").Text = "(3/4) Optimize.. "
            ' Kanten/Ränder schützen
            For i As Integer = 0 To Vertices.Count - 1
                If bw.CancellationPending Then GoTo Abort
                Dim count As Integer
                count = Vertices(i).linkedTriangles.Count
                If count > 3 Then
                    For j As Integer = 0 To count - 1
                        Dim tri As Triangle = Vertices(i).linkedTriangles(j)
                        If tri.vertexA.linkedTriangles.Count < 4 OrElse tri.vertexB.linkedTriangles.Count < 4 OrElse tri.vertexC.linkedTriangles.Count < 4 OrElse tri.myColourNumber <> Vertices(i).linkedTriangles(0).myColourNumber Then
                            Vertices(i).selected = True
                            Exit For
                        End If
                    Next
                Else
                    Vertices(i).selected = True
                End If
            Next

            Dim start As Integer = 0
            If it = 0 Then
                Dim tcount As Integer = 0
                CDialog.Controls("LblStatus").Text = "(3/4) Optimize I.. "
newVertex2:
                For i As Integer = start To Vertices.Count - 1
                    If Vertices(i).selected Then
                        SelectedVertices.Add(Vertices(i))
                        selectColourStrideHorizontal(bmp)
                        If SelectedVertices.Count > 2 Then
                            start = i + 1
                            tcount += SelectedVertices.Count - 1
                            splitAndMerge()

                            CType(CDialog.Controls("PBStatus"), ProgressBar).Value = Fix(i / Vertices.Count * 100) : If bw.CancellationPending Then GoTo Abort

                            GoTo newVertex2
                        Else
                            SelectedVertices.Clear()
                        End If
                    End If
                Next
                Try
                    cleanupDATVerticesFast(tcount, bw)
                    cleanupDATTriangles2(bw)
                Catch
                End Try
                start = 0
                tcount = 0
                CDialog.Controls("LblStatus").Text = "(3/4) Optimize II.. "
newVertex3:
                For i As Integer = start To Vertices.Count - 1
                    If Vertices(i).selected Then
                        SelectedVertices.Add(Vertices(i))
                        selectColourStrideVertical(bmp)
                        If SelectedVertices.Count > 2 Then
                            start = i + 1
                            tcount += SelectedVertices.Count - 1
                            splitAndMerge()

                            CType(CDialog.Controls("PBStatus"), ProgressBar).Value = Fix(i / Vertices.Count * 100) : If bw.CancellationPending Then GoTo Abort
                            GoTo newVertex3
                        Else
                            SelectedVertices.Clear()
                        End If
                    End If
                Next
                Try
                    cleanupDATVerticesFast(tcount, bw)
                    cleanupDATTriangles2(bw)
                Catch
                End Try
                start = 0
            End If
            old_Count = Vertices.Count
            CDialog.Controls("LblStatus").Text = "(3/4) Optimize III.. "
newVertex:
            For i As Integer = start To Vertices.Count - 1
                Dim count As Integer
                count = Vertices(i).linkedTriangles.Count
                If Not Vertices(i).selected AndAlso count > 3 Then
                    Do
                        For j As Integer = 0 To count - 1
                            Dim tri As Triangle = Vertices(i).linkedTriangles(j)
                            If tri.myColourNumber <> Vertices(i).linkedTriangles(0).myColourNumber OrElse tri.vertexA.linkedTriangles.Count < 4 OrElse tri.vertexB.linkedTriangles.Count < 4 OrElse tri.vertexC.linkedTriangles.Count < 4 Then
                                Vertices(i).selected = True
                                Exit Do
                            End If
                        Next
                        SelectedVertices.Add(Vertices(i))
                        selectTouching2()
                        Dim oc As Integer = Vertices.Count
                        mergeToAverage2()
                        If oc = Vertices.Count Then
                            start = i + 1
                        Else
                            start = i
                        End If
                        CType(CDialog.Controls("PBStatus"), ProgressBar).Value = Fix(i / Vertices.Count * 100) : If bw.CancellationPending Then GoTo Abort
                        GoTo newVertex
                    Loop
                Else
                    Vertices(i).selected = True
                End If
            Next i
            Try
                cleanupDATTriangles2(bw)
                cleanupDATVerticesFast(Integer.MaxValue, bw)
            Catch
            End Try
            For i As Integer = 0 To Vertices.Count - 1
                Vertices(i).selected = False
            Next
            If old_Count = Vertices.Count Then Exit For
        Next it
        CDialog.Controls("LblStatus").Text = "(4/4) Paint.. "
        'paintTriangles()
        MenuStrip1.Enabled = True
        Me.ToolStrip1.Enabled = True
        Me.ColourToolStrip.Enabled = True
        GBImage.Enabled = True
        GBSnap.Enabled = True
        isLoading = False
        CDialog.Close()
        Me.Refresh()
        addHistoryII(withDataFrom.All)
        Return ""
Abort:
        newPattern()
        addHistoryII(withDataFrom.All)
        MenuStrip1.Enabled = True
        Me.ToolStrip1.Enabled = True
        Me.ColourToolStrip.Enabled = True
        GBImage.Enabled = True
        GBSnap.Enabled = True
        isLoading = False
        Me.Refresh()
        Return "Abort"
    End Function

    Function strCol(ByVal c As Color) As String
        If c.R > c.G Then
            If c.B > c.R Then
                Return "BRG"
            Else
                If c.B > c.G Then
                    Return "RBG"
                Else
                    Return "RGB"
                End If
            End If
        Else
            If c.B > c.G Then
                Return "BGR"
            Else
                If c.B > c.R Then
                    Return "GBR"
                Else
                    Return "GRB"
                End If
            End If
        End If
        Return ""
    End Function

    Private Sub EProjectOnYZPlaneToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles EProjectOnYZPlaneToolStripMenuItem.Click
        exportDat(0)
    End Sub

    Private Sub EProjectOnYZPlane2ToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles EProjectOnYZPlane2ToolStripMenuItem.Click
        exportDat(3)
    End Sub

    Private Sub EProjectOnZXPlaneToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles EProjectOnZXPlaneToolStripMenuItem.Click
        exportDat(1)
    End Sub

    Private Sub EProjectOnZXPlane2ToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles EProjectOnZXPlane2ToolStripMenuItem.Click
        exportDat(4)
    End Sub

    Private Sub EProjectOnXYPlaneToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles EProjectOnXYPlaneToolStripMenuItem.Click
        exportDat(2)
    End Sub

    Private Sub EProjectOnXYPlane2ToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles EProjectOnXYPlane2ToolStripMenuItem.Click
        exportDat(5)
    End Sub

    Private Sub exportDat(ByVal projectMode As Byte)
        If Not isLoading AndAlso useRecommendedProjection(projectMode) AndAlso userWantsToExportDAT(projectMode) Then
            addHistoryI(withDataFrom.All)
            Control.CheckForIllegalCrossThreadCalls = False
            CDialog = New StatusDialog
            CDialog.Controls("Cancel_Button").Enabled = False
            Dim Thread As New Threading.Thread(AddressOf exportDat2)
            Thread.Start(New Object() {projectMode})
            If CDialog.ShowDialog() = Windows.Forms.DialogResult.Cancel Then

            End If
        End If
    End Sub

    Private Sub BtnShowPalette_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BtnShowPalette.Click
        ColourForm.Show()
    End Sub

    Private Sub BtnPipette_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BtnPipette.Click
        If SelectedTriangles.Count = 1 Then
            lastColour = SelectedTriangles(0).myColour
            lastColourNumber = SelectedTriangles(0).myColourNumber
            CLast.BackColor = lastColour
            If lastColourNumber = -1 Then
                CLast.Text = "0x2" & lastColour.R.ToString("X2") & lastColour.G.ToString("X2") & lastColour.B.ToString("X2")
            Else
                CLast.Text = lastColourNumber
            End If
        ElseIf objectToModify = "Primitive" AndAlso SelectedTriangles.Count > 0 AndAlso SelectedTriangles(0).groupindex <> -1 Then
            lastColourNumber = Primitives(PrimitivesHMap(SelectedTriangles(0).groupindex)).myColourNumber
            lastColour = colourHMap(lastColourNumber)
            CLast.BackColor = lastColour
            If lastColourNumber = -1 Then
                CLast.Text = "0x2" & lastColour.R.ToString("X2") & lastColour.G.ToString("X2") & lastColour.B.ToString("X2")
            Else
                CLast.Text = lastColourNumber
            End If
        Else
            BtnPipette.Enabled = False
        End If
        ColourForm.NR.Value = lastColour.R
        ColourForm.NG.Value = lastColour.G
        ColourForm.NB.Value = lastColour.B
    End Sub

    Private Sub NUDVertX_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles NUDVertX.ValueChanged
        If SelectedVertices.Count = 1 Then
            SelectedVertices(0).X = NUDVertX.Value * 1000 / factor
            Me.Refresh()
        Else
            GBVertex.Visible = False
        End If
    End Sub

    Private Sub NUDVertY_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles NUDVertY.ValueChanged
        If SelectedVertices.Count = 1 Then
            SelectedVertices(0).Y = NUDVertY.Value * 1000 / factor
            Me.Refresh()
        Else
            GBVertex.Visible = False
        End If
    End Sub

    Private Sub GBVertex_VisibleChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles GBVertex.VisibleChanged
        If GBVertex.Visible Then
            NUDVertX.Value = SelectedVertices(0).X * factor / 1000
            NUDVertY.Value = SelectedVertices(0).Y * factor / 1000
        End If
    End Sub

    Private Sub CLast_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CLast.Click
        setColour(lastColour, lastColourNumber)
    End Sub

    Public Sub loadConfig()
        If My.Computer.FileSystem.FileExists(appPath & "Config.cfg") Then
            isLoading = True
            Try
                Using DateiIn As StreamReader = My.Computer.FileSystem.OpenTextFileReader(appPath & "Config.cfg", System.Text.Encoding.GetEncoding(20127))
                    Me.ShowAllWarningsToolStripMenuItem.Checked = CType(DateiIn.ReadLine, Boolean)
                    Me.mySettings.defaultName = DateiIn.ReadLine
                    Me.mySettings.defaultUser = DateiIn.ReadLine
                    Me.mySettings.defaultLicense = DateiIn.ReadLine
                    Me.mySettings.max_undo = CType(DateiIn.ReadLine, Byte)
                    Me.mySettings.performanceMode = CType(DateiIn.ReadLine, Boolean)

                    Me.mySettings.startWithFullscreen = CType(DateiIn.ReadLine, Boolean)
                    Me.mySettings.showImageViewAtStartup = CType(DateiIn.ReadLine, Boolean)
                    Me.mySettings.showPreferencesViewAtStartup = CType(DateiIn.ReadLine, Boolean)

                    Me.mySettings.useAlternativeKeys = CType(DateiIn.ReadLine, Boolean)

                    myKeys.Abort = CType(DateiIn.ReadLine, Integer) : KeyToSet.setKey(BtnAbort, myKeys.Abort)
                    myKeys.AddTriangle = CType(DateiIn.ReadLine, Integer) : KeyToSet.setKey(BtnAddTriangle, myKeys.AddTriangle)
                    myKeys.AddVertex = CType(DateiIn.ReadLine, Integer) : KeyToSet.setKey(BtnAddVertex, myKeys.AddVertex)
                    myKeys.ModeMove = CType(DateiIn.ReadLine, Integer) : KeyToSet.setKey(BtnMove, myKeys.ModeMove)

                    myKeys.ModeRotate = CType(DateiIn.ReadLine, Integer) : KeyToSet.setKey(BtnRotate, myKeys.ModeRotate)
                    myKeys.ModeScale = CType(DateiIn.ReadLine, Integer) : KeyToSet.setKey(BtnScale, myKeys.ModeScale)
                    myKeys.ModeSelect = CType(DateiIn.ReadLine, Integer) : KeyToSet.setKey(BtnSelect, myKeys.ModeSelect)

                    myKeys.Pipette = CType(DateiIn.ReadLine, Integer) : KeyToSet.setKey(BtnPipette, myKeys.Pipette)
                    myKeys.Preview = CType(DateiIn.ReadLine, Integer) : KeyToSet.setKey(BtnPreview, myKeys.Preview)
                    myKeys.ShowColours = CType(DateiIn.ReadLine, Integer) : KeyToSet.setKey(BtnColours, myKeys.ShowColours)
                    myKeys.Translate = CType(DateiIn.ReadLine, Integer) : KeyToSet.setKey(BtnTranslate, myKeys.Translate)
                    myKeys.Zoom = CType(DateiIn.ReadLine, Integer) : KeyToSet.setKey(BtnZoom, myKeys.Zoom)

                    NewPatternToolStripMenuItem.ShortcutKeys = CType(DateiIn.ReadLine, Integer)
                    NewPatternToolStripMenuItem.ShortcutKeyDisplayString = KeyToSet.keyToString(New System.Windows.Forms.KeyEventArgs(NewPatternToolStripMenuItem.ShortcutKeys))
                    LoadPatternToolStripMenuItem.ShortcutKeys = CType(DateiIn.ReadLine, Integer)
                    LoadPatternToolStripMenuItem.ShortcutKeyDisplayString = KeyToSet.keyToString(New System.Windows.Forms.KeyEventArgs(LoadPatternToolStripMenuItem.ShortcutKeys))
                    SaveToolStripMenuItem.ShortcutKeys = CType(DateiIn.ReadLine, Integer)
                    SaveToolStripMenuItem.ShortcutKeyDisplayString = KeyToSet.keyToString(New System.Windows.Forms.KeyEventArgs(SaveToolStripMenuItem.ShortcutKeys))
                    SelectAllToolStripMenuItem.ShortcutKeys = CType(DateiIn.ReadLine, Integer)
                    SelectAllToolStripMenuItem.ShortcutKeyDisplayString = KeyToSet.keyToString(New System.Windows.Forms.KeyEventArgs(SelectAllToolStripMenuItem.ShortcutKeys))
                    SelectSameColourToolStripMenuItem.ShortcutKeys = CType(DateiIn.ReadLine, Integer)
                    SelectSameColourToolStripMenuItem.ShortcutKeyDisplayString = KeyToSet.keyToString(New System.Windows.Forms.KeyEventArgs(SelectSameColourToolStripMenuItem.ShortcutKeys))

                    SelectConnectedToolStripMenuItem.ShortcutKeys = CType(DateiIn.ReadLine, Integer)
                    SelectConnectedToolStripMenuItem.ShortcutKeyDisplayString = KeyToSet.keyToString(New System.Windows.Forms.KeyEventArgs(SelectConnectedToolStripMenuItem.ShortcutKeys))
                    SelectTouchingToolStripMenuItem.ShortcutKeys = CType(DateiIn.ReadLine, Integer)
                    SelectTouchingToolStripMenuItem.ShortcutKeyDisplayString = KeyToSet.keyToString(New System.Windows.Forms.KeyEventArgs(SelectTouchingToolStripMenuItem.ShortcutKeys))
                    WithColourToolStripMenuItem.ShortcutKeys = CType(DateiIn.ReadLine, Integer)
                    WithColourToolStripMenuItem.ShortcutKeyDisplayString = KeyToSet.keyToString(New System.Windows.Forms.KeyEventArgs(WithColourToolStripMenuItem.ShortcutKeys))
                    ResetViewToolStripMenuItem.ShortcutKeys = CType(DateiIn.ReadLine, Integer)
                    ResetViewToolStripMenuItem.ShortcutKeyDisplayString = KeyToSet.keyToString(New System.Windows.Forms.KeyEventArgs(ResetViewToolStripMenuItem.ShortcutKeys))
                    ImageToolStripMenuItem.ShortcutKeys = CType(DateiIn.ReadLine, Integer)
                    ImageToolStripMenuItem.ShortcutKeyDisplayString = KeyToSet.keyToString(New System.Windows.Forms.KeyEventArgs(ImageToolStripMenuItem.ShortcutKeys))

                    SnappingToolStripMenuItem.ShortcutKeys = CType(DateiIn.ReadLine, Integer)
                    SnappingToolStripMenuItem.ShortcutKeyDisplayString = KeyToSet.keyToString(New System.Windows.Forms.KeyEventArgs(SnappingToolStripMenuItem.ShortcutKeys))
                    ToAverageToolStripMenuItem.ShortcutKeys = CType(DateiIn.ReadLine, Integer)
                    ToAverageToolStripMenuItem.ShortcutKeyDisplayString = KeyToSet.keyToString(New System.Windows.Forms.KeyEventArgs(ToAverageToolStripMenuItem.ShortcutKeys))
                    ToLastSelectedToolStripMenuItem.ShortcutKeys = CType(DateiIn.ReadLine, Integer)
                    ToLastSelectedToolStripMenuItem.ShortcutKeyDisplayString = KeyToSet.keyToString(New System.Windows.Forms.KeyEventArgs(ToLastSelectedToolStripMenuItem.ShortcutKeys))

                    myColours.background = Color.FromArgb(CType(DateiIn.ReadLine, Integer))
                    Me.BackColor = myColours.background
                    myColours.linePen = New Pen(Color.FromArgb(CType(DateiIn.ReadLine, Integer)))
                    myColours.inverseLinePen = New Pen(Color.FromArgb(CType(DateiIn.ReadLine, Integer)))
                    myColours.selectedLinePen = New Pen(Color.FromArgb(CType(DateiIn.ReadLine, Integer)))
                    myColours.selectedLineInVertexModePen = New Pen(Color.FromArgb(CType(DateiIn.ReadLine, Integer)))
                    myColours.vertexBrush = New SolidBrush(Color.FromArgb(CType(DateiIn.ReadLine, Integer)))
                    myColours.selectedVertexBrush = New SolidBrush(Color.FromArgb(CType(DateiIn.ReadLine, Integer)))

                    myColours.originPen = New Pen(Color.FromArgb(CType(DateiIn.ReadLine, Integer)))
                    myColours.gridPen = New Pen(Color.FromArgb(CType(DateiIn.ReadLine, Integer)))
                    myColours.grid10Pen = New Pen(Color.FromArgb(CType(DateiIn.ReadLine, Integer)))
                    myColours.selectionRectPen = New Pen(Color.FromArgb(CType(DateiIn.ReadLine, Integer)))

                    myColours.selectionCrossBrush = New SolidBrush(Color.FromArgb(CType(DateiIn.ReadLine, Integer)))
                    Me.ldrawPath = DateiIn.ReadLine
                End Using
                Me.MaxUndoToolStripTextBox.Text = mySettings.max_undo
                Me.PerformanceEnabledToolStripMenuItem.Checked = mySettings.performanceMode
                If Not mySettings.showImageViewAtStartup Then Me.ImageToolStripMenuItem.PerformClick()
                If Not mySettings.showPreferencesViewAtStartup Then Me.SnappingToolStripMenuItem.PerformClick()
            Catch
            End Try
            isLoading = False
        End If
    End Sub

    Public Sub saveConfig()
        Using DateiOut As StreamWriter = My.Computer.FileSystem.OpenTextFileWriter(appPath & "Config.cfg", False, System.Text.Encoding.GetEncoding(20127))
            DateiOut.WriteLine(ShowAllWarningsToolStripMenuItem.Checked)
            DateiOut.WriteLine(mySettings.defaultName)
            DateiOut.WriteLine(mySettings.defaultUser)
            DateiOut.WriteLine(mySettings.defaultLicense)
            DateiOut.WriteLine(mySettings.max_undo)
            DateiOut.WriteLine(mySettings.performanceMode)
            DateiOut.WriteLine(mySettings.startWithFullscreen)
            DateiOut.WriteLine(mySettings.showImageViewAtStartup)
            DateiOut.WriteLine(mySettings.showPreferencesViewAtStartup)
            DateiOut.WriteLine(mySettings.useAlternativeKeys)

            DateiOut.WriteLine(myKeys.Abort)
            DateiOut.WriteLine(myKeys.AddTriangle)
            DateiOut.WriteLine(myKeys.AddVertex)
            DateiOut.WriteLine(myKeys.ModeMove)

            DateiOut.WriteLine(myKeys.ModeRotate)
            DateiOut.WriteLine(myKeys.ModeScale)
            DateiOut.WriteLine(myKeys.ModeSelect)

            DateiOut.WriteLine(myKeys.Pipette)
            DateiOut.WriteLine(myKeys.Preview)
            DateiOut.WriteLine(myKeys.ShowColours)
            DateiOut.WriteLine(myKeys.Translate)
            DateiOut.WriteLine(myKeys.Zoom)

            DateiOut.WriteLine(CType(NewPatternToolStripMenuItem.ShortcutKeys, Integer))
            DateiOut.WriteLine(CType(LoadPatternToolStripMenuItem.ShortcutKeys, Integer))
            DateiOut.WriteLine(CType(SaveToolStripMenuItem.ShortcutKeys, Integer))
            DateiOut.WriteLine(CType(SelectAllToolStripMenuItem.ShortcutKeys, Integer))
            DateiOut.WriteLine(CType(SelectSameColourToolStripMenuItem.ShortcutKeys, Integer))

            DateiOut.WriteLine(CType(SelectConnectedToolStripMenuItem.ShortcutKeys, Integer))
            DateiOut.WriteLine(CType(SelectTouchingToolStripMenuItem.ShortcutKeys, Integer))
            DateiOut.WriteLine(CType(WithColourToolStripMenuItem.ShortcutKeys, Integer))
            DateiOut.WriteLine(CType(ResetViewToolStripMenuItem.ShortcutKeys, Integer))
            DateiOut.WriteLine(CType(ImageToolStripMenuItem.ShortcutKeys, Integer))

            DateiOut.WriteLine(CType(SnappingToolStripMenuItem.ShortcutKeys, Integer))
            DateiOut.WriteLine(CType(ToAverageToolStripMenuItem.ShortcutKeys, Integer))
            DateiOut.WriteLine(CType(ToLastSelectedToolStripMenuItem.ShortcutKeys, Integer))

            DateiOut.WriteLine(myColours.background.ToArgb)
            DateiOut.WriteLine(myColours.linePen.Color.ToArgb)
            DateiOut.WriteLine(myColours.inverseLinePen.Color.ToArgb)
            DateiOut.WriteLine(myColours.selectedLinePen.Color.ToArgb)
            DateiOut.WriteLine(myColours.selectedLineInVertexModePen.Color.ToArgb)
            DateiOut.WriteLine(myColours.vertexBrush.Color.ToArgb)
            DateiOut.WriteLine(myColours.selectedVertexBrush.Color.ToArgb)
            DateiOut.WriteLine(myColours.originPen.Color.ToArgb)
            DateiOut.WriteLine(myColours.gridPen.Color.ToArgb)
            DateiOut.WriteLine(myColours.grid10Pen.Color.ToArgb)
            DateiOut.WriteLine(myColours.selectionRectPen.Color.ToArgb)
            DateiOut.WriteLine(myColours.selectionCrossBrush.Color.ToArgb)

            DateiOut.WriteLine(ldrawPath)
        End Using
    End Sub

    Private Sub ShowAllWarningsToolStripMenuItem_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles ShowAllWarningsToolStripMenuItem.Click
        saveConfig()
    End Sub

    Private Sub BtnPreview_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles BtnPreview.Click
        Me.Refresh()
    End Sub

    Private Sub CreateTemplateBitmapToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CreateTemplateBitmapToolStripMenuItem.Click
        If BitmapDialog.ShowDialog() = Windows.Forms.DialogResult.OK Then

        End If
    End Sub

    Private Sub CreateATemplateToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CreateATemplateToolStripMenuItem.Click
        ChooseTemplateDialog.selectedText = ""
        If TemplateEditor.ShowDialog() = Windows.Forms.DialogResult.OK Then
            Dim ti As New ToolStripMenuItem
            ti.Text = TemplateEditor.oldName
            ti.ToolTipText = appPath & TemplateEditor.oldName & ".txt"
            AddHandler ti.Click, AddressOf ti_Click
            TemplateToolStripMenuItem.DropDownItems.Add(ti)
        End If
    End Sub

    Private Sub EditTemplateToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles EditTemplateToolStripMenuItem.Click
        ChooseTemplateDialog.ShowDialog()
    End Sub

    Private Sub MaxUndoToolStripTextBox_TextChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles MaxUndoToolStripTextBox.TextChanged
        Try
            If mySettings.max_undo <> CType(MaxUndoToolStripTextBox.Text, Byte) Then
                mySettings.max_undo = CType(MaxUndoToolStripTextBox.Text, Byte)
                If mySettings.max_undo < 3 Then mySettings.max_undo = 3
                MaxUndoToolStripTextBox.Text = mySettings.max_undo
                saveConfig()
            End If
        Catch ex As Exception
            MaxUndoToolStripTextBox.Text = mySettings.max_undo
        End Try
    End Sub

    Private Sub ProjectionQuadTimer_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ProjectionQuadTimer.Tick
        If Not isLoading AndAlso templateProjectionQuad.Count > alreadyCalculatedQuads Then
            ProjectionQuadTimer.Interval = 30000
            isSolving = True
            templateProjectionQuad(alreadyCalculatedQuads).CalculateMatrix()
            alreadyCalculatedQuads += 1
            isSolving = False
        ElseIf templateProjectionQuad.Count = alreadyCalculatedQuads Then
            ProjectionQuadTimer.Enabled = False
        End If
    End Sub

    Private Sub O100ToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles O100ToolStripMenuItem.Click
        OpacityToolStripDropDownButton.Text = "Opacity: 100%"
        alpha = 255
        Me.Refresh()
    End Sub

    Private Sub O50ToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles O50ToolStripMenuItem.Click
        OpacityToolStripDropDownButton.Text = "Opacity: 50%"
        alpha = 127
        Me.Refresh()
    End Sub

    Private Sub O25ToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles O25ToolStripMenuItem.Click
        OpacityToolStripDropDownButton.Text = "Opacity: 25%"
        alpha = 64
        Me.Refresh()
    End Sub

    Private Sub O10ToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles O10ToolStripMenuItem.Click
        OpacityToolStripDropDownButton.Text = "Opacity: 10%"
        alpha = 26
        Me.Refresh()
    End Sub

    Private Sub LoadBackgroundImageToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles LoadBackgroundImageToolStripMenuItem.Click
        If Not GBImage.Visible Then
            GBImage.Visible = True
            BtnImagePath.PerformClick()
            GBImage.Visible = False
        Else
            BtnImagePath.PerformClick()
        End If
    End Sub

    Private Sub MoreOptionsToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles AdvancedOptionsToolStripMenuItem.Click
        If OptionsDialog.ShowDialog = Windows.Forms.DialogResult.OK Then
            saveConfig()
            Me.Refresh()
        End If
    End Sub

    Private Sub ShowAxisLabelToolStripMenuItem_CheckedChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles ShowAxisLabelToolStripMenuItem.CheckedChanged
        Me.Refresh()
    End Sub

    Private Sub AdjustBackgroundImageToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles AdjustBackgroundImageToolStripMenuItem.Click
        If Not GBImage.Visible Then
            GBImage.Visible = True
            BtnAdjust.PerformClick()
            GBImage.Visible = False
        Else
            BtnAdjust.PerformClick()
        End If
    End Sub

    Private Sub BtnAdjust_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BtnAdjust.Click
        Me.BtnAbort.Text = Replace(Me.BtnAbort.Text, "ABORT", "OK")
        Me.BtnAbort.BackColor = Color.Green
        Me.BtnAbort.Visible = True
        Me.ToolStrip1.Visible = False
        Me.MenuStrip1.Visible = False
        Me.BtnAbort.Visible = True
        adjustmode = True
        If BtnAddVertex.Checked Then BtnAddVertex.PerformClick()
        If BtnAddTriangle.Checked Then BtnAddTriangle.PerformClick()
        BtnSelect.PerformClick()
        Me.Refresh()
    End Sub

    Private Sub LblDebug_DoubleClick(ByVal sender As Object, ByVal e As System.EventArgs) Handles LblDebug.DoubleClick
        If ZoomDialog.ShowDialog = Windows.Forms.DialogResult.OK Then

            If zoomfactor = 0.001 Then
                zoomlevel = -30
            ElseIf zoomfactor <= 0.01F Then
                zoomlevel = (zoomfactor - 1.0F) * 10.0F
            ElseIf zoomfactor <= 0.1F Then
                zoomlevel = (zoomfactor - 0.2F) * 100.0F
            ElseIf zoomfactor <= 10.0F Then
                zoomlevel = (zoomfactor - 1.0F) * 10.0F
            End If

            clip(zoomlevel, -30, 90)

            LblDebug.Text = "Zoom: " & CType(zoomfactor * 10000, Integer) / 100 & "%"
            LblCoords.Text = Math.Round(getXcoordinate(getCursorpositionX()) / Me.moveSnap * factor) * Me.moveSnap / 1000 & " | " & Math.Round(getYcoordinate(getCursorpositionY()) / Me.moveSnap * factor) * Me.moveSnap / 1000 & " " & unit
            Me.Refresh()
        End If
    End Sub

    Private Sub RemoveIsolatedVerticesToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RemoveIsolatedVerticesToolStripMenuItem.Click
        addHistoryI(withDataFrom.Vertices + withDataFrom.SelectedVertices)
        clearSelection()
        For i As Integer = 0 To Vertices.Count - 1
            If Vertices(i).linkedTriangles.Count = 0 Then
                SelectedVertices.Add(Vertices(i))
            End If
        Next
        delete()
        Me.Refresh()
        addHistoryII(withDataFrom.Vertices + withDataFrom.SelectedVertices)
    End Sub

    Private Sub MergeToNearestPrimvertexToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MergeToNearestPrimvertexToolStripMenuItem.Click
        If SelectedVertices.Count >= 1 Then
            addHistoryI(withDataFrom.All)
            If SelectedVertices.Count > 1 Then
                Dim sumX As Integer = 0
                Dim sumY As Integer = 0
                For i As Integer = 0 To SelectedVertices.Count - 1
                    sumX += SelectedVertices(i).X
                    sumY += SelectedVertices(i).Y
                Next
                sumX /= SelectedVertices.Count
                sumY /= SelectedVertices.Count
                For i As Integer = 0 To SelectedVertices.Count - 1
                    SelectedVertices(i).X = sumX
                    SelectedVertices(i).Y = sumY
                Next
                SelectedTriangles.Clear()
                cleanupDATVertices()
                cleanupDATTriangles()
                For i As Integer = 0 To Vertices.Count - 1
                    For j As Integer = 0 To SelectedVertices.Count - 1
                        If Vertices(i).vertexID = SelectedVertices(j).vertexID Then
                            SelectedVertices(j).selected = False
                        End If
                    Next
                Next
newDelete:
                For j As Integer = 0 To SelectedVertices.Count - 1
                    If SelectedVertices(j).selected Then
                        SelectedVertices.RemoveAt(j)
                        GoTo newDelete
                    End If
                Next
            End If
            SelectedVertices(0).selected = False
            Dim x As Single = 0
            Dim y As Single = 0
            Dim min_dist As Double = Double.MaxValue
            For i As Integer = 0 To Vertices.Count - 1
                If Vertices(i).vertexID <> SelectedVertices(0).vertexID Then
                    If Vertices(i).groupindex <> -1 AndAlso min_dist > Vertices(i).dist(SelectedVertices(0)) Then
                        min_dist = Vertices(i).dist(SelectedVertices(0))
                        x = Vertices(i).X
                        y = Vertices(i).Y
                    End If
                End If
            Next
            SelectedVertices(0).X = x
            SelectedVertices(0).Y = y
            SelectedVertices.Clear()
            addHistoryII(withDataFrom.All)
            Me.Refresh()
        End If
    End Sub

    Private Sub SetPathToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SetPathToolStripMenuItem.Click
        If Me.FBDLDrawDir.ShowDialog = Windows.Forms.DialogResult.Cancel Then
        Else
            ldrawPath = Me.FBDLDrawDir.SelectedPath
            saveConfig()
        End If
    End Sub

End Class