﻿' MIT - Licence
'
' Copyright (c) 2010 - 2011 Nils Schmidt
' This program uses Rectifier.exe by permission of the author and copyright holder Philippe E. Hurbain - (C) 2009

' Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"),
' to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
' and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

' The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
' INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
' PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
' FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
' ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

Option Strict Off
Imports System.Text
Imports System.IO
Imports System.Runtime.Serialization
Imports System.Runtime.Serialization.Formatters.Binary

Imports System.Threading
Imports System.ComponentModel


Public Class MainForm


    Declare Auto Sub mouse_event Lib "user32" (ByVal dwFlags As Int32, ByVal dx As Int32, ByVal dy As Int32, ByVal cButtons As Int32, ByVal dwExtraInfo As IntPtr)

    Const MOUSEEVENTF_MOVE As Int32 = &H1 '  mouse move
    Const MOUSEEVENTF_LEFTDOWN As Int32 = &H2 '  left button down
    Const MOUSEEVENTF_LEFTUP As Int32 = &H4 '  left button up
    Const MOUSEEVENTF_RIGHTDOWN As Int32 = &H8 '  right button down
    Const MOUSEEVENTF_RIGHTUP As Int32 = &H10 '  right button up
    Const MOUSEEVENTF_MIDDLEDOWN As Int32 = &H20 '  middle button down
    Const MOUSEEVENTF_MIDDLEUP As Int32 = &H40 '  middle button up
    Const MOUSEEVENTF_ABSOLUTE As Int32 = &H8000 '  absolute move

    Dim cur_x As Single
    Dim cur_y As Single
    Dim dest_x As Single
    Dim dest_y As Single
    Dim p_old As Point

#Region "Data Structure"
    Const twoPI As Double = Math.PI * 2
    Dim auswahl, kamera, raster, movemode, rotatemode, scalemode, isLoading As Boolean
    Dim klickX As Integer
    Dim klickY As Integer


    Dim segments_circle As Integer = 16
    Dim segments_oval As Integer = 16
    Dim segments_arch As Integer = 4

    Dim temp_center As Vertex

    Public Shared vertexIDglobal As Integer = 0
    Public Shared triangleIDglobal As Integer = 0
    Public Shared primitiveIDglobal As Integer = 0


    Dim zoomlevel As Integer
    Dim zoomfaktor As Single = 1

    Dim punktgroesse As Integer = 4

    Dim toffsetX As Single
    Dim toffsetY As Single
    Dim offsetX As Single
    Dim offsetY As Single

    Dim lastPointX As Single
    Dim lastPointY As Single

    Dim moveSnap As Single = 1
    Dim rasterSnap As Single = 1
    Dim rotateSnap As Single = 1
    Dim scaleSnap As Single = 0.01F

    Dim imgOffsetX As Integer = 0
    Dim imgOffsetY As Integer = 0
    Dim imgScale As Single = 1

    Dim beamdir() As Decimal = {0, 0, -1}
    Dim beamorig(2) As Decimal

    Dim schnittpunkt As New Vertex(0, 0, False, False)

    Dim bearbeitungsmodus As String = "Auswahl"
    Dim bearbeitungsobjekt As String = "Vertex"

    Dim dreiecksmodus As Byte = 0

    Dim hintergrundbild As Bitmap = My.Resources.temp

    Dim Vertices As New List(Of Vertex)
    Dim ClipboardVertices As New List(Of Vertex)
    Dim Triangles As New List(Of Triangle)
    Dim ClipboardTriangles As New List(Of Triangle)
    Dim Primitives As New List(Of Primitive)
    Dim ClipboardPrimitives As New List(Of Primitive)
    Dim PrimitivesHMap As New Hashtable(1000)

    Dim historyVertices As New List(Of List(Of Vertex))
    Dim historyTriangles As New List(Of List(Of Triangle))
    Dim historySelectedVertices As New List(Of List(Of Vertex))
    Dim historySelectedTriangles As New List(Of List(Of Triangle))
    Dim historyPrimitives As New List(Of List(Of Primitive))
    Dim historyHashtable As New List(Of Hashtable)

    Dim SelectedVertices As New List(Of Vertex)
    Dim SelectedTriangles As New List(Of Triangle)


    Dim offset_X As Integer = 0
    Dim offset_Y As Integer = 30
    Dim getOffset As Boolean = False

    Public tempvertex As New Vertex(0, 0, False, False)
    Public tempcolour As Color

    <Serializable()> _
    Public Class Vertex
        Implements System.Runtime.Serialization.ISerializable
        Implements IComparable(Of Vertex)

        Public X As Single
        Public Y As Single
        Public oldX As Single
        Public oldY As Single
        Public selected As Boolean

        Public angleFrom As Double
        Public distanceFrom As Double

        Public linkedTriangles As New List(Of Triangle)

        Public groupindex As Integer

        Public vertexID As Integer

        Private Sub ISerializable_GetObjectData(ByVal oInfo As SerializationInfo, ByVal oContext As StreamingContext) Implements ISerializable.GetObjectData
            oInfo.AddValue("X", Me.X, Me.X.GetType)
            oInfo.AddValue("Y", Me.Y, Me.Y.GetType)
            oInfo.AddValue("selected", Me.selected, Me.selected.GetType)
            oInfo.AddValue("linkedTriangles", Me.linkedTriangles, Me.linkedTriangles.GetType)
            oInfo.AddValue("groupindex", Me.groupindex, Me.groupindex.GetType)
            oInfo.AddValue("vertexID", Me.vertexID, Me.vertexID.GetType)
        End Sub

        Protected Sub New(ByVal info As System.Runtime.Serialization.SerializationInfo, ByVal context As System.Runtime.Serialization.StreamingContext)
            ' Konstruktor für das Deserialisieren
            Me.X = info.GetSingle("X")
            Me.Y = info.GetSingle("Y")
            Me.selected = info.GetBoolean("selected")
            Me.linkedTriangles = info.GetValue("linkedTriangles", Me.linkedTriangles.GetType)
            Me.groupindex = info.GetInt32("groupindex")
            Me.vertexID = info.GetInt32("vertexID")
        End Sub

        Sub New(ByVal vx As Single, ByVal vy As Single, ByVal sel As Boolean, Optional ByVal needID As Boolean = True)
            Me.X = vx
            Me.Y = vy
            Me.selected = sel
            Me.groupindex = -1
            If needID Then
                vertexIDglobal += 1
                Me.vertexID = vertexIDglobal
            End If
        End Sub

        Public Shared Operator <>(ByVal v1 As Vertex, ByVal v2 As Vertex) As Boolean
            Return v1.X <> v2.X Or v1.Y <> v2.Y
        End Operator

        Public Shared Operator =(ByVal v1 As Vertex, ByVal v2 As Vertex) As Boolean
            Return v1.X = v2.X And v1.Y = v2.Y
        End Operator

        Public Shared Operator +(ByVal v1 As Vertex, ByVal v2 As Vertex) As Vertex
            Return New Vertex(v1.X + v2.X, v1.Y + v2.Y, False, False)
        End Operator

        Public Shared Operator -(ByVal v1 As Vertex, ByVal v2 As Vertex) As Vertex
            Return New Vertex(v1.X - v2.X, v1.Y - v2.Y, False, False)
        End Operator

        Public Shared Operator *(ByVal v1 As Vertex, ByVal v2 As Vertex) As Single
            Return v1.X * v2.X + v1.Y * v2.Y
        End Operator

        Public Shared Operator >(ByVal v1 As Vertex, ByVal v2 As Vertex) As Boolean
            Return v1.angleFrom > v2.angleFrom
        End Operator

        Public Shared Operator >=(ByVal v1 As Vertex, ByVal v2 As Vertex) As Boolean
            Return v1.angleFrom >= v2.angleFrom
        End Operator

        Public Shared Operator <(ByVal v1 As Vertex, ByVal v2 As Vertex) As Boolean
            Return v1.angleFrom < v2.angleFrom
        End Operator

        Public Shared Operator <=(ByVal v1 As Vertex, ByVal v2 As Vertex) As Boolean
            Return v1.angleFrom <= v2.angleFrom
        End Operator

        Public Overloads Function CompareTo(ByVal other As Vertex) As Integer Implements IComparable(Of Vertex).CompareTo
            If Me > other Then
                Return 1
            ElseIf Me < other Then
                Return -1
            Else
                Return 0
            End If
        End Function

        Public Function angle(ByVal v2 As Vertex) As Double
            Dim angle2, tx, ty As Double
            tx = v2.X - Me.X
            ty = v2.Y - Me.Y
            angle2 = Math.Atan2(ty, tx)
            If angle2 < 0 Then angle2 += twoPI
            Return angle2
        End Function

        Public Function MYangle() As Double
            Dim angle2 As Double
            angle2 = Math.Atan2(Me.Y, Me.X)
            If angle2 < 0 Then angle2 += twoPI
            Return angle2
        End Function

        Public Function dist(ByVal v2 As Vertex) As Double
            Return Math.Sqrt((Me.X - v2.X) ^ 2 + (Me.Y - v2.Y) ^ 2)
        End Function


    End Class

    <Serializable()> _
    Public Class Triangle
        Implements System.Runtime.Serialization.ISerializable
        Public vertexA As Vertex
        Public vertexB As Vertex
        Public vertexC As Vertex
        Public selected As Boolean
        Public myColour As Color
        Public myColourNumber As Short = 16
        Public groupindex As Integer
        Public triangleID As Integer

        Private Sub ISerializable_GetObjectData(ByVal oInfo As SerializationInfo, ByVal oContext As StreamingContext) Implements ISerializable.GetObjectData
            oInfo.AddValue("vertexA", Me.vertexA, MainForm.tempvertex.GetType())
            oInfo.AddValue("vertexB", Me.vertexB, MainForm.tempvertex.GetType())
            oInfo.AddValue("vertexC", Me.vertexC, MainForm.tempvertex.GetType())
            oInfo.AddValue("selected", Me.selected, Me.selected.GetType())
            oInfo.AddValue("myColour", Me.myColour, Me.myColour.GetType())
            oInfo.AddValue("myColourNumber", Me.myColourNumber, Me.myColourNumber.GetType())
            oInfo.AddValue("groupindex", Me.groupindex, Me.groupindex.GetType())
            oInfo.AddValue("triangleID", Me.triangleID, Me.triangleID.GetType())
        End Sub

        Protected Sub New(ByVal info As System.Runtime.Serialization.SerializationInfo, ByVal context As System.Runtime.Serialization.StreamingContext)
            ' Konstruktor für das Deserialisieren
            Me.vertexA = info.GetValue("vertexA", MainForm.tempvertex.GetType())
            Me.vertexB = info.GetValue("vertexB", MainForm.tempvertex.GetType())
            Me.vertexC = info.GetValue("vertexC", MainForm.tempvertex.GetType())
            Me.selected = info.GetBoolean("selected")
            Me.myColour = info.GetValue("myColour", MainForm.tempcolour.GetType())
            Me.myColourNumber = info.GetInt16("myColourNumber")
            Me.groupindex = info.GetInt32("groupindex")
            Me.triangleID = info.GetInt32("triangleID")
        End Sub

        Sub New(ByVal v1 As Vertex, ByVal v2 As Vertex, ByVal v3 As Vertex, Optional ByVal needID As Boolean = True)
            Me.vertexA = v1
            Me.vertexB = v2
            Me.vertexC = v3
            Me.groupindex = -1
            If needID Then
                triangleIDglobal += 1
                Me.triangleID = triangleIDglobal
            End If
        End Sub

        Public Sub normalize()
            Dim tmpVertex As Vertex
            tmpVertex = vertexA : If vertexA.X > vertexB.X Then vertexA = vertexB : vertexB = tmpVertex
            tmpVertex = vertexA : If vertexA.X > vertexC.X Then vertexA = vertexC : vertexC = tmpVertex
            tmpVertex = vertexB : If vertexB.X > vertexC.X Then vertexB = vertexC : vertexC = tmpVertex

            tmpVertex = vertexA : If vertexA.Y > vertexB.Y Then vertexA = vertexB : vertexB = tmpVertex
            tmpVertex = vertexA : If vertexA.Y > vertexC.Y Then vertexA = vertexC : vertexC = tmpVertex
            tmpVertex = vertexB : If vertexB.Y > vertexC.Y Then vertexB = vertexC : vertexC = tmpVertex
        End Sub

        Public Function getCenter() As Vertex
            Return New Vertex((vertexA.X + vertexB.X + vertexC.X) / 3, (vertexA.Y + vertexB.Y + vertexC.Y) / 3, False, False)
        End Function

        Public Function getFingerprint() As String
            Dim sb As New StringBuilder()
            sb.Append(vertexA.X)
            sb.Append(vertexA.Y)
            sb.Append(vertexB.X)
            sb.Append(vertexB.Y)
            sb.Append(vertexC.X)
            sb.Append(vertexC.Y)
            Return sb.ToString
        End Function

        Public Sub checkWinding()
            Dim v1 As New Vertex(0, 0, False, False)
            Dim v2 As New Vertex(0, 0, False, False)
            v1 = Me.vertexB - Me.vertexA
            v2 = Me.vertexC - Me.vertexA
            If (v1.X * v2.Y - v1.Y * v2.X) < 0 Then
                Dim tv As Vertex
                tv = Me.vertexB
                Me.vertexB = Me.vertexC
                Me.vertexC = tv
            End If
        End Sub

    End Class

    <Serializable()> _
    Public Class Primitive
        Implements System.Runtime.Serialization.ISerializable

        Public primitiveID As Integer
        Public centerVertexID As Integer
        Public matrix(3, 3) As Double
        Public primitiveName As String
        Public ox As Double
        Public oy As Double

        Sub New(ByVal vx As Single, ByVal vy As Single, ByVal offset_x As Single, ByVal offset_y As Single, ByVal name As String, ByVal centerVertexID As Integer, Optional ByVal needID As Boolean = True)
            Me.matrix(0, 3) = vx
            Me.matrix(1, 3) = vy
            Me.matrix(0, 0) = 1
            Me.matrix(1, 1) = 1
            Me.matrix(2, 2) = 1
            Me.matrix(3, 3) = 1
            Me.ox = offset_x
            Me.oy = offset_y
            If needID Then
                primitiveIDglobal += 1
                Me.primitiveID = primitiveIDglobal
            End If
            Me.primitiveName = name
            Me.centerVertexID = centerVertexID
        End Sub

        Public Sub translate(ByVal dx As Single, ByVal dy As Single)
            Dim matrix2(,) As Double = {{1, 0, 0, dx}, _
                                        {0, 1, 0, dy}, _
                                        {0, 0, 1, 0}, _
                                        {0, 0, 0, 1}}
            Dim matrix3(3, 3) As Double

            matrix3(0, 0) = matrix2(0, 0) * matrix(0, 0) + matrix2(0, 1) * matrix(1, 0) + matrix2(0, 2) * matrix(2, 0)
            matrix3(0, 1) = matrix2(0, 0) * matrix(0, 1) + matrix2(0, 1) * matrix(1, 1) + matrix2(0, 2) * matrix(2, 1)
            matrix3(0, 2) = matrix2(0, 0) * matrix(0, 2) + matrix2(0, 1) * matrix(1, 2) + matrix2(0, 2) * matrix(2, 2)
            matrix3(0, 3) = matrix2(0, 0) * matrix(0, 3) + matrix2(0, 1) * matrix(1, 3) + matrix2(0, 2) * matrix(2, 3) + matrix2(0, 3)

            matrix3(1, 0) = matrix2(1, 0) * matrix(0, 0) + matrix2(1, 1) * matrix(1, 0) + matrix2(1, 2) * matrix(2, 0)
            matrix3(1, 1) = matrix2(1, 0) * matrix(0, 1) + matrix2(1, 1) * matrix(1, 1) + matrix2(1, 2) * matrix(2, 1)
            matrix3(1, 2) = matrix2(1, 0) * matrix(0, 2) + matrix2(1, 1) * matrix(1, 2) + matrix2(1, 2) * matrix(2, 2)
            matrix3(1, 3) = matrix2(1, 0) * matrix(0, 3) + matrix2(1, 1) * matrix(1, 3) + matrix2(1, 2) * matrix(2, 3) + matrix2(1, 3)

            matrix3(2, 0) = matrix2(2, 0) * matrix(0, 0) + matrix2(2, 1) * matrix(1, 0) + matrix2(2, 2) * matrix(2, 0)
            matrix3(2, 1) = matrix2(2, 0) * matrix(0, 1) + matrix2(2, 1) * matrix(1, 1) + matrix2(2, 2) * matrix(2, 1)
            matrix3(2, 2) = matrix2(2, 0) * matrix(0, 2) + matrix2(2, 1) * matrix(1, 2) + matrix2(2, 2) * matrix(2, 2)
            matrix3(2, 3) = matrix2(2, 0) * matrix(0, 3) + matrix2(2, 1) * matrix(1, 3) + matrix2(2, 2) * matrix(2, 3) + matrix2(2, 3)

            matrix = matrix3
        End Sub

        Public Sub rotate(ByVal r As Double)
            Dim vert As Vertex = New Vertex(ox, oy, False, False)
            Dim angle As Double = vert.MYangle
            Dim dist As Double = Math.Sqrt(vert.X ^ 2 + vert.Y ^ 2)
            Dim tox, toy As Double
            tox = matrix(0, 3)
            toy = matrix(1, 3)
            translate(-matrix(0, 3), -matrix(1, 3))
            translate(-ox, -oy)

            ' Rotieren..
            Dim matrix2(,) As Double = {{Math.Cos(r), -Math.Sin(r), 0, 0}, _
                                        {Math.Sin(r), Math.Cos(r), 0, 0}, _
                                        {0, 0, 1, 0}, _
                                        {0, 0, 0, 1}}
            Dim matrix3(3, 3) As Double

            matrix3(0, 0) = matrix2(0, 0) * matrix(0, 0) + matrix2(0, 1) * matrix(1, 0) + matrix2(0, 2) * matrix(2, 0)
            matrix3(0, 1) = matrix2(0, 0) * matrix(0, 1) + matrix2(0, 1) * matrix(1, 1) + matrix2(0, 2) * matrix(2, 1)
            matrix3(0, 2) = matrix2(0, 0) * matrix(0, 2) + matrix2(0, 1) * matrix(1, 2) + matrix2(0, 2) * matrix(2, 2)
            matrix3(0, 3) = matrix2(0, 0) * matrix(0, 3) + matrix2(0, 1) * matrix(1, 3) + matrix2(0, 2) * matrix(2, 3) + matrix2(0, 3)

            matrix3(1, 0) = matrix2(1, 0) * matrix(0, 0) + matrix2(1, 1) * matrix(1, 0) + matrix2(1, 2) * matrix(2, 0)
            matrix3(1, 1) = matrix2(1, 0) * matrix(0, 1) + matrix2(1, 1) * matrix(1, 1) + matrix2(1, 2) * matrix(2, 1)
            matrix3(1, 2) = matrix2(1, 0) * matrix(0, 2) + matrix2(1, 1) * matrix(1, 2) + matrix2(1, 2) * matrix(2, 2)
            matrix3(1, 3) = matrix2(1, 0) * matrix(0, 3) + matrix2(1, 1) * matrix(1, 3) + matrix2(1, 2) * matrix(2, 3) + matrix2(1, 3)

            matrix3(2, 0) = matrix2(2, 0) * matrix(0, 0) + matrix2(2, 1) * matrix(1, 0) + matrix2(2, 2) * matrix(2, 0)
            matrix3(2, 1) = matrix2(2, 0) * matrix(0, 1) + matrix2(2, 1) * matrix(1, 1) + matrix2(2, 2) * matrix(2, 1)
            matrix3(2, 2) = matrix2(2, 0) * matrix(0, 2) + matrix2(2, 1) * matrix(1, 2) + matrix2(2, 2) * matrix(2, 2)
            matrix3(2, 3) = matrix2(2, 0) * matrix(0, 3) + matrix2(2, 1) * matrix(1, 3) + matrix2(2, 2) * matrix(2, 3) + matrix2(2, 3)

            matrix = matrix3

            translate(ox, oy)
            translate(tox, toy)
            ox = Math.Cos(angle + r) * dist
            oy = Math.Sin(angle + r) * dist
        End Sub

        Public Sub scale(ByVal sx As Double, ByVal sy As Double)
            Dim vert As Vertex = New Vertex(ox, oy, False, False)
            Dim angle As Double = vert.MYangle
            Dim dist As Double = Math.Sqrt(vert.X ^ 2 + vert.Y ^ 2)
            Dim tox, toy As Double
            tox = matrix(0, 3)
            toy = matrix(1, 3)
            translate(-matrix(0, 3), -matrix(1, 3))
            translate(-ox, -oy)

            ' Scalieren..
            Dim matrix2(,) As Double = {{sx, 0, 0, 0}, _
                                        {0, sy, 0, 0}, _
                                        {0, 0, 1, 0}, _
                                        {0, 0, 0, 1}}
            Dim matrix3(3, 3) As Double

            matrix3(0, 0) = matrix2(0, 0) * matrix(0, 0) + matrix2(0, 1) * matrix(1, 0) + matrix2(0, 2) * matrix(2, 0)
            matrix3(0, 1) = matrix2(0, 0) * matrix(0, 1) + matrix2(0, 1) * matrix(1, 1) + matrix2(0, 2) * matrix(2, 1)
            matrix3(0, 2) = matrix2(0, 0) * matrix(0, 2) + matrix2(0, 1) * matrix(1, 2) + matrix2(0, 2) * matrix(2, 2)
            matrix3(0, 3) = matrix2(0, 0) * matrix(0, 3) + matrix2(0, 1) * matrix(1, 3) + matrix2(0, 2) * matrix(2, 3) + matrix2(0, 3)

            matrix3(1, 0) = matrix2(1, 0) * matrix(0, 0) + matrix2(1, 1) * matrix(1, 0) + matrix2(1, 2) * matrix(2, 0)
            matrix3(1, 1) = matrix2(1, 0) * matrix(0, 1) + matrix2(1, 1) * matrix(1, 1) + matrix2(1, 2) * matrix(2, 1)
            matrix3(1, 2) = matrix2(1, 0) * matrix(0, 2) + matrix2(1, 1) * matrix(1, 2) + matrix2(1, 2) * matrix(2, 2)
            matrix3(1, 3) = matrix2(1, 0) * matrix(0, 3) + matrix2(1, 1) * matrix(1, 3) + matrix2(1, 2) * matrix(2, 3) + matrix2(1, 3)

            matrix3(2, 0) = matrix2(2, 0) * matrix(0, 0) + matrix2(2, 1) * matrix(1, 0) + matrix2(2, 2) * matrix(2, 0)
            matrix3(2, 1) = matrix2(2, 0) * matrix(0, 1) + matrix2(2, 1) * matrix(1, 1) + matrix2(2, 2) * matrix(2, 1)
            matrix3(2, 2) = matrix2(2, 0) * matrix(0, 2) + matrix2(2, 1) * matrix(1, 2) + matrix2(2, 2) * matrix(2, 2)
            matrix3(2, 3) = matrix2(2, 0) * matrix(0, 3) + matrix2(2, 1) * matrix(1, 3) + matrix2(2, 2) * matrix(2, 3) + matrix2(2, 3)

            matrix = matrix3

            translate(ox, oy)
            translate(tox, toy)
            ox *= sx
            oy *= sy
        End Sub

        Private Sub ISerializable_GetObjectData(ByVal oInfo As SerializationInfo, ByVal oContext As StreamingContext) Implements ISerializable.GetObjectData
            Dim tempinteger As Integer
            Dim tempmatrix(3, 3) As Double
            Dim tempstring As String = ""
            Dim tempdouble As Double
            oInfo.AddValue("primitiveID", Me.primitiveID, tempinteger.GetType())
            oInfo.AddValue("centerVertexID", Me.centerVertexID, tempinteger.GetType())
            oInfo.AddValue("matrix", Me.matrix, tempmatrix.GetType())
            oInfo.AddValue("primitiveName", Me.primitiveName, tempstring.GetType())
            oInfo.AddValue("ox", Me.ox, tempdouble.GetType())
            oInfo.AddValue("oy", Me.oy, tempdouble.GetType())
        End Sub

        Protected Sub New(ByVal info As System.Runtime.Serialization.SerializationInfo, ByVal context As System.Runtime.Serialization.StreamingContext)
            ' Konstruktor für das Deserialisieren
            Dim tempinteger As Integer
            Dim tempmatrix(3, 3) As Double
            Dim tempstring As String = ""
            Dim tempdouble As Double
            Me.primitiveID = info.GetValue("primitiveID", tempinteger.GetType())
            Me.centerVertexID = info.GetValue("centerVertexID", tempinteger.GetType())
            Me.matrix = info.GetValue("matrix", tempmatrix.GetType())
            Me.primitiveName = info.GetValue("primitiveName", tempstring.GetType())
            Me.ox = info.GetValue("ox", tempdouble.GetType())
            Me.oy = info.GetValue("oy", tempdouble.GetType())
        End Sub
    End Class

    Public Shared Function DeepClone(ByVal source As Object) As Object
        Dim m As New MemoryStream() With {.Capacity = 102400}
        Dim b As New BinaryFormatter()
        b.Serialize(m, source)
        m.Position = 0
        Return b.Deserialize(m)
    End Function

#End Region
#Region "Main Form"
    Dim lastAction As Byte = 0
    Dim keyLock As Boolean = True

    Private Sub MainForm_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        If Me.BackgroundHowTo.IsBusy Then
            turorialRuns = False
            e.Cancel = True
            Timer2.Enabled = True
        End If
    End Sub

    Private Sub MainForm_KeyDown(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles Me.KeyDown
        If Not NUDMoveSnap.Focused AndAlso Not NUDRotateSnap.Focused AndAlso Not NUDScaleSnap.Focused AndAlso Not NUDScale.Focused AndAlso Not NUDoffsetX.Focused AndAlso Not NUDoffsetY.Focused Then
            If e.KeyCode = Keys.D1 Then Me.BtnSelect.PerformClick() : lastAction = 0
            If e.KeyCode = Keys.D2 Then Me.BtnMove.PerformClick() : lastAction = 1
            If e.KeyCode = Keys.D3 Then Me.BtnRotate.PerformClick() : lastAction = 2
            If e.KeyCode = Keys.D4 Then Me.BtnScale.PerformClick() : lastAction = 3
            If keyLock AndAlso e.KeyCode = Keys.D5 Then Me.BtnAddVertex.PerformClick() : keyLock = False
            If keyLock AndAlso e.KeyCode = Keys.D6 Then Me.BtnAddTriangle.PerformClick() : keyLock = False
            If e.KeyCode = Keys.S Then If Not Me.BtnSelect.Checked Then Me.BtnSelect.PerformClick()
        End If
    End Sub

    Private Sub MainForm_KeyUp(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles Me.KeyUp
        If e.KeyCode = Keys.S OrElse Not keyLock Then
            Select Case lastAction
                Case 1 : If Not Me.BtnMove.Checked Then Me.BtnMove.PerformClick()
                Case 2 : If Not Me.BtnRotate.Checked Then Me.BtnRotate.PerformClick()
                Case 3 : If Not Me.BtnScale.Checked Then Me.BtnScale.PerformClick()
            End Select
        End If
        keyLock = True
    End Sub

    Private Sub MainForm_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        'Dim dummy As New Primitive(0, 0, 1, 1, "4-4disc.dat")
        'dummy.translate(1, 1)
        ''dummy.scale(2, 2)
        'dummy.rotate(Math.PI / 2)
        'dummy.translate(2, -1.0)
        'dummy.rotate(Math.PI / 2)
        'dummy.scale(2, 0.5)
        raster = True
        'Me.SetStyle(ControlStyles.DoubleBuffer Or ControlStyles.UserPaint Or ControlStyles.AllPaintingInWmPaint, True)
        addHistory()
        addHistory()
        Me.TriangleModeToolStripMenuItem.PerformClick()
    End Sub

    Public Sub New()

        ' Dieser Aufruf ist für den Windows Form-Designer erforderlich.
        InitializeComponent()

        ' Fügen Sie Initialisierungen nach dem InitializeComponent()-Aufruf hinzu.
        Me.SetStyle(ControlStyles.DoubleBuffer, True)
        Me.SetStyle(ControlStyles.ResizeRedraw, True)
        Me.SetStyle(ControlStyles.AllPaintingInWmPaint, True)
        Me.SetStyle(ControlStyles.UserPaint, True)
    End Sub

    Private Sub MainForm_ResizeEnd(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.ResizeEnd
        Me.Refresh()
    End Sub

    Private Sub MainForm_SizeChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.SizeChanged
        Me.Refresh()
    End Sub
#End Region

#Region "Mouse"
    Function getCursorpositionX() As Integer
        Return Cursor.Position.X - Me.Location.X
    End Function

    Function getCursorpositionY() As Integer
        Return Cursor.Position.Y - Me.Location.Y - offset_Y
    End Function

    Private Sub MainForm_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseDown
        If Not SBZoom.Focused Then SBZoom.Focus()
        If e.Button = Windows.Forms.MouseButtons.Left Then
            If getOffset Then
                offset_X = System.Windows.Forms.Cursor.Position.X - e.X
                offset_Y = System.Windows.Forms.Cursor.Position.Y - e.Y + 2
                moveMouseAbsoluteRightClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2, offset_Y + Me.ClientSize.Height / 2)
                getOffset = False
            End If
            If primitiveMode = 0 Then
                Select Case bearbeitungsmodus
                    Case "Auswahl"
                        klickX = getCursorpositionX()
                        klickY = getCursorpositionY()
                        auswahl = True
                        ' CTRL?
                        If Not Control.ModifierKeys = 131072 Or bearbeitungsobjekt = "Primitive" Then
                            SelectedVertices.Clear()
                            For Each tri As Triangle In SelectedTriangles
                                tri.selected = False
                            Next
                            SelectedTriangles.Clear()
                        End If

                        Me.Refresh()
                        Exit Select
                    Case "Translation"
                        movemode = True
                        klickX = getCursorpositionX()
                        klickY = getCursorpositionY()
                        Exit Select
                    Case "Rotation"

                        If SelectedVertices.Count > 0 Then
                            rotatemode = True
                            klickX = getCursorpositionX()
                            klickY = getCursorpositionY()
                            If bearbeitungsobjekt = "Primitive" Then
                                Dim tid As Integer = Primitives(PrimitivesHMap(SelectedVertices(0).groupindex)).centerVertexID
                                For Each vert As Vertex In SelectedVertices
                                    If vert.vertexID = tid Then
                                        temp_center = New Vertex(vert.X, vert.Y, False, False)
                                        Exit For
                                    End If
                                Next
                            Else
                                temp_center = New Vertex(0, 0, False, False)
                                For Each vert As Vertex In SelectedVertices
                                    temp_center += vert
                                Next
                                temp_center.X /= SelectedVertices.Count
                                temp_center.Y /= SelectedVertices.Count
                            End If
                            For Each vert As Vertex In SelectedVertices
                                vert.angleFrom = vert.angle(temp_center) + Math.PI
                                vert.distanceFrom = vert.dist(temp_center)
                                vert.oldX = vert.X
                                vert.oldY = vert.Y
                            Next
                        End If
                        Exit Select
                    Case "Skalierung"
                        If SelectedVertices.Count > 0 Then
                            scalemode = True
                            klickX = getCursorpositionX()
                            klickY = getCursorpositionY()
                            temp_center = New Vertex(0, 0, False, False)
                            For Each vert As Vertex In SelectedVertices
                                temp_center += vert
                            Next
                            temp_center.X /= SelectedVertices.Count
                            temp_center.Y /= SelectedVertices.Count
                            For Each vert As Vertex In SelectedVertices
                                vert.angleFrom = vert.angle(temp_center) + Math.PI
                                vert.distanceFrom = vert.dist(temp_center)
                                vert.oldX = vert.X
                                vert.oldY = vert.Y
                            Next
                            Me.Refresh()
                        End If
                        Exit Select
                End Select
            Else

            End If
        ElseIf e.Button = Windows.Forms.MouseButtons.Middle Then
        ElseIf e.Button = Windows.Forms.MouseButtons.Right Then
            toffsetX = offsetX
            toffsetY = offsetY
            klickX = getCursorpositionX()
            klickY = getCursorpositionY()
            kamera = True
        End If
    End Sub

    Private Sub MainForm_MouseMove(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseMove
        'If Not (NUDGrid.Focused OrElse NUDMoveSnap.Focused OrElse NUDMoveSnap.Focused OrElse NUDRotateSnap.Focused OrElse NUDScaleSnap.Focused OrElse NUDScale.Focused) Then SBZoom.Focus()
        LblCoords.Text = Math.Round(getXcoordinate(getCursorpositionX() - 3) / Me.moveSnap * factor) * Me.moveSnap / 1000 & " | " & Math.Round(getYcoordinate(getCursorpositionY()) / Me.moveSnap * factor) * Me.moveSnap / 1000 & " " & unit
        Me.Cursor = Cursors.Cross
        Dim curX As Integer
        Dim curY As Integer
        curX = e.X
        curY = e.Y
        If curX + offset_X > Cursor.Position.X Then offset_X = Cursor.Position.X - cur_x
        If curX + offset_X < Cursor.Position.X Then offset_X = Cursor.Position.X - cur_x

        If curY + offset_Y > Cursor.Position.Y Then offset_Y = Cursor.Position.Y - curY + 4
        If curY + offset_Y < Cursor.Position.Y Then offset_Y = Cursor.Position.Y - curY + 4
        If auswahl OrElse kamera OrElse movemode OrElse dreiecksmodus > 0 OrElse rotatemode OrElse scalemode OrElse primitiveMode > 0 OrElse BtnAddVertex.Checked Then Me.Refresh()
    End Sub

    Private Sub MainForm_MouseUp(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseUp
        If Not isLoading Then
            If e.Button = Windows.Forms.MouseButtons.Left Then
                If primitiveMode = 0 Then
                    If BtnAddVertex.Checked Then
                        Vertices.Add(New Vertex(Math.Round(getXcoordinate(getCursorpositionX() - 3) / Me.moveSnap) * Me.moveSnap, Math.Round(getYcoordinate(getCursorpositionY()) / Me.moveSnap) * Me.moveSnap, False))
                        'Vertices.Add(New Vertex(Math.Round(getXcoordinate(getCursorpositionX() - 3)), Math.Round(getYcoordinate(getCursorpositionY())), False))
                    End If
                    Select Case bearbeitungsmodus
                        Case "Auswahl"
                            Dim minX, maxX, minY, maxY, delta As Single
                            minX = getXcoordinate(Math.Min(klickX, getCursorpositionX))
                            minY = getYcoordinate(Math.Max(klickY, getCursorpositionY))
                            maxX = getXcoordinate(Math.Max(klickX, getCursorpositionX))
                            maxY = getYcoordinate(Math.Min(klickY, getCursorpositionY))

                            delta = Math.Sqrt((klickX - getCursorpositionX()) ^ 2 + (klickY - getCursorpositionY()) ^ 2)
                            If delta < 5 Then
                                If bearbeitungsobjekt = "Dreieck" Then
                                    beamorig(0) = getXcoordinate(getCursorpositionX() - 3)
                                    beamorig(1) = getYcoordinate(getCursorpositionY())
                                    beamorig(2) = 0
                                    Dim vert0(2) As Decimal
                                    Dim vert1(2) As Decimal
                                    Dim vert2(2) As Decimal
                                    vert0(2) = 0 : vert1(2) = 0 : vert2(2) = 0
                                    For Each tri As Triangle In Triangles
                                        vert0(0) = tri.vertexA.X
                                        vert0(1) = tri.vertexA.Y
                                        vert1(0) = tri.vertexB.X
                                        vert1(1) = tri.vertexB.Y
                                        vert2(0) = tri.vertexC.X
                                        vert2(1) = tri.vertexC.Y
                                        If PowerRay.SCHNITTPKT_DREIECK(beamorig, beamdir, vert0, vert1, vert2) AndAlso Not SelectedTriangles.Contains(tri) AndAlso tri.groupindex < 0 Then
                                            tri.selected = True
                                            SelectedTriangles.Add(tri)
                                            If Not SelectedVertices.Contains(tri.vertexA) AndAlso tri.vertexA.groupindex < 0 Then SelectedVertices.Add(tri.vertexA)
                                            If Not SelectedVertices.Contains(tri.vertexB) AndAlso tri.vertexB.groupindex < 0 Then SelectedVertices.Add(tri.vertexB)
                                            If Not SelectedVertices.Contains(tri.vertexC) AndAlso tri.vertexC.groupindex < 0 Then SelectedVertices.Add(tri.vertexC)
                                            Exit For
                                        End If
                                    Next
                                ElseIf bearbeitungsobjekt = "Vertex" Then
                                    minX -= 9 / zoomfaktor : minY -= 9 / zoomfaktor
                                    maxX += 9 / zoomfaktor : maxY += 9 / zoomfaktor
                                    Dim previous_vert As Vertex = Nothing
                                    Dim min_vert As Vertex = Nothing
                                    Dim minDelta As Single = Single.MaxValue
                                    For Each vert As Vertex In Vertices
                                        If Not Control.ModifierKeys = 131072 Then vert.selected = False
                                        If vert.X >= minX AndAlso vert.X <= maxX AndAlso _
                                           vert.Y >= minY AndAlso vert.Y <= maxY Then
                                            delta = Math.Sqrt((vert.X - (minX + maxX) / 2) ^ 2 + (vert.Y - (minY + maxY) / 2) ^ 2)
                                            If delta < minDelta Then
                                                minDelta = delta
                                                If Not vert.selected Then
                                                    min_vert = vert
                                                    If Not previous_vert Is Nothing Then previous_vert.selected = False
                                                    vert.selected = True
                                                    previous_vert = vert
                                                ElseIf Not Control.ModifierKeys = 131072 Then
                                                    vert.selected = False
                                                End If
                                            ElseIf Not Control.ModifierKeys = 131072 Then
                                                vert.selected = False
                                            End If
                                        ElseIf Not Control.ModifierKeys = 131072 Then
                                            vert.selected = False
                                        End If
                                    Next
                                    If Not min_vert Is Nothing AndAlso (min_vert.groupindex < 0 OrElse BtnAddTriangle.Checked) Then SelectedVertices.Add(min_vert)
                                End If
                            ElseIf bearbeitungsobjekt <> "Primitive" Then
                                For Each vert As Vertex In Vertices
                                    If Not Control.ModifierKeys = 131072 Then vert.selected = False
                                    If vert.X >= minX AndAlso vert.X <= maxX AndAlso _
                                       vert.Y >= minY AndAlso vert.Y <= maxY Then
                                        If Not vert.selected AndAlso vert.groupindex < 0 Then
                                            vert.selected = True
                                            SelectedVertices.Add(vert)
                                        End If
                                    ElseIf Not Control.ModifierKeys = 131072 Then
                                        vert.selected = False
                                    End If
                                Next
                            End If

                            If bearbeitungsobjekt = "Dreieck" AndAlso delta >= 5 Then
nextVertex:
                                For Each vert As Vertex In SelectedVertices
                                    vert.selected = False
                                    If vert.linkedTriangles.Count > 0 Then
                                        Dim notContainsTri As Boolean = True
                                        For Each tri As Triangle In vert.linkedTriangles
                                            If SelectedVertices.Contains(tri.vertexA) AndAlso SelectedVertices.Contains(tri.vertexB) AndAlso SelectedVertices.Contains(tri.vertexC) Then
                                                If Not SelectedTriangles.Contains(tri) AndAlso tri.groupindex < 0 Then
                                                    SelectedTriangles.Add(tri)
                                                    tri.selected = True
                                                End If
                                                notContainsTri = False
                                            ElseIf Not Control.ModifierKeys = 131072 Then
                                                tri.selected = False
                                            End If
                                        Next
                                        If notContainsTri Then
                                            SelectedVertices.Remove(vert)
                                            GoTo nextVertex
                                        End If
                                    Else
                                        SelectedVertices.Remove(vert)
                                        GoTo nextVertex
                                    End If
                                Next
                            End If

                            If bearbeitungsobjekt = "Primitive" Then
                                beamorig(0) = getXcoordinate(getCursorpositionX() - 3)
                                beamorig(1) = getYcoordinate(getCursorpositionY())
                                beamorig(2) = 0
                                Dim vert0(2) As Decimal
                                Dim vert1(2) As Decimal
                                Dim vert2(2) As Decimal
                                vert0(2) = 0 : vert1(2) = 0 : vert2(2) = 0
                                For Each tri As Triangle In Triangles
                                    vert0(0) = tri.vertexA.X
                                    vert0(1) = tri.vertexA.Y
                                    vert1(0) = tri.vertexB.X
                                    vert1(1) = tri.vertexB.Y
                                    vert2(0) = tri.vertexC.X
                                    vert2(1) = tri.vertexC.Y
                                    If PowerRay.SCHNITTPKT_DREIECK(beamorig, beamdir, vert0, vert1, vert2) AndAlso Not SelectedTriangles.Contains(tri) AndAlso tri.groupindex >= 0 Then
                                        tri.selected = True
                                        SelectedTriangles.Add(tri)
                                        If Not SelectedVertices.Contains(tri.vertexA) AndAlso tri.vertexA.groupindex >= 0 Then SelectedVertices.Add(tri.vertexA)
                                        If Not SelectedVertices.Contains(tri.vertexB) AndAlso tri.vertexB.groupindex >= 0 Then SelectedVertices.Add(tri.vertexB)
                                        If Not SelectedVertices.Contains(tri.vertexC) AndAlso tri.vertexC.groupindex >= 0 Then SelectedVertices.Add(tri.vertexC)
                                        selectPrimitiveGroup(tri.groupindex)
                                        Exit For
                                    End If
                                Next
                            End If


                            If SelectedVertices.Count = 1 Then
                                If BtnAddTriangle.Checked Then
                                    lastPointX = SelectedVertices(0).X
                                    lastPointY = SelectedVertices(0).Y
                                    If dreiecksmodus = 2 AndAlso Not LLast(Triangles).vertexA.Equals(SelectedVertices(0)) AndAlso Not LLast(Triangles).vertexB.Equals(SelectedVertices(0)) Then
                                        SelectedVertices(0).linkedTriangles.Add(LLast(Triangles))
                                        LLast(Triangles).vertexC = SelectedVertices(0)
                                        dreiecksmodus = 0
                                    Else
                                        If dreiecksmodus = 1 AndAlso Not LLast(Triangles).vertexA.Equals(SelectedVertices(0)) Then
                                            SelectedVertices(0).linkedTriangles.Add(LLast(Triangles))
                                            LLast(Triangles).vertexB = SelectedVertices(0)
                                            dreiecksmodus = 2
                                        End If
                                        If dreiecksmodus = 0 Then
                                            Triangles.Add(New Triangle(SelectedVertices(0), SelectedVertices(0), SelectedVertices(0)))
                                            SelectedVertices(0).linkedTriangles.Add(LLast(Triangles))
                                            dreiecksmodus = 1
                                        End If
                                    End If
                                End If
                            End If

                            auswahl = False
                            addHistory()
                            Exit Select
                        Case "Translation"
                            Dim vabsOffsetX As Integer = Fix((getCursorpositionX() - klickX) / zoomfaktor / moveSnap) * moveSnap
                            Dim vabsOffsetY As Integer = Fix((getCursorpositionY() - klickY) / zoomfaktor / moveSnap) * moveSnap
                            If Control.ModifierKeys = 131072 Then vabsOffsetX = 0
                            If Control.ModifierKeys = Keys.Shift Then vabsOffsetY = 0

                            For Each vert As Vertex In SelectedVertices
                                vert.X += vabsOffsetX
                                vert.Y -= vabsOffsetY
                            Next
                            If SelectedVertices.Count > 0 AndAlso bearbeitungsobjekt = "Primitive" Then
                                Primitives(CType(PrimitivesHMap(SelectedVertices(0).groupindex), Integer)).translate(vabsOffsetX, vabsOffsetY)
                            End If
                            movemode = False
                            addHistory()
                            Exit Select
                        Case "Rotation"
                            rotatemode = False
                            Dim angleToRotate As Double = Fix((-(getCursorpositionX() - klickX) - (getCursorpositionY() - klickY)) / 100 / (rotateSnap * Math.PI / 180)) * (rotateSnap * Math.PI / 180)
                            For Each vert As Vertex In SelectedVertices
                                vert.X = temp_center.X + vert.distanceFrom * Math.Cos(vert.angleFrom + angleToRotate)
                                vert.Y = temp_center.Y + vert.distanceFrom * Math.Sin(vert.angleFrom + angleToRotate)
                                If Single.IsNaN(vert.X) OrElse Single.IsNaN(vert.Y) Then
                                    vert.X = vert.oldX
                                    vert.Y = vert.oldY
                                End If
                            Next
                            If SelectedVertices.Count > 0 AndAlso bearbeitungsobjekt = "Primitive" Then
                                Primitives(CType(PrimitivesHMap(SelectedVertices(0).groupindex), Integer)).rotate(angleToRotate)
                            End If
                            addHistory()
                            Exit Select
                        Case "Skalierung"
                            scalemode = False
                            Dim factorToScale As Double = 1 + Fix((getCursorpositionX() - klickX) / 100 / scaleSnap) * scaleSnap
                            If factorToScale < 0.1 Then factorToScale = 0.1
                            If factorToScale > 2 Then factorToScale = 2
                            For Each vert As Vertex In SelectedVertices
                                If Not Control.ModifierKeys = 131072 Then
                                    vert.X = temp_center.X + factorToScale * vert.distanceFrom * Math.Cos(vert.angleFrom)
                                Else
                                    vert.X = vert.oldX
                                End If
                                If Not Control.ModifierKeys = Keys.Shift Then
                                    vert.Y = temp_center.Y + factorToScale * vert.distanceFrom * Math.Sin(vert.angleFrom)
                                Else
                                    vert.Y = vert.oldY
                                End If
                                If Single.IsNaN(vert.X) OrElse Single.IsNaN(vert.Y) Then
                                    vert.X = vert.oldX
                                    vert.Y = vert.oldY
                                End If
                            Next
                            If SelectedVertices.Count > 0 AndAlso bearbeitungsobjekt = "Primitive" Then
                                Dim factorToScaleX As Double = 1
                                Dim factorToScaleY As Double = 1
                                If Not Control.ModifierKeys = 131072 Then factorToScaleX = factorToScale
                                If Not Control.ModifierKeys = Keys.Shift Then factorToScaleY = factorToScale
                                Primitives(CType(PrimitivesHMap(SelectedVertices(0).groupindex), Integer)).scale(factorToScaleX, factorToScaleY)
                            End If
                            addHistory()
                            Exit Select
                    End Select
                Else
                    If primitiveMode = 255 Then
                        'Vertices.Add(New Vertex(Fix(getXcoordinate(getCursorpositionX() - 3)), Fix(getYcoordinate(getCursorpositionY())), True)) : SelectedVertices.Add(LLast(Vertices))
                        Vertices.Add(New Vertex(Math.Round(getXcoordinate(getCursorpositionX() - 3) / Me.moveSnap) * Me.moveSnap, Math.Round(getYcoordinate(getCursorpositionY()) / Me.moveSnap) * Me.moveSnap, True)) : SelectedVertices.Add(LLast(Vertices))
                        Triangles.Add(New Triangle(Vertices(Vertices.Count - 1), Vertices(Vertices.Count - 2), Vertices(Vertices.Count - 3)))
                        Vertices(Vertices.Count - 1).linkedTriangles.Add(LLast(Triangles))
                        Vertices(Vertices.Count - 2).linkedTriangles.Add(LLast(Triangles))
                        Vertices(Vertices.Count - 3).linkedTriangles.Add(LLast(Triangles))
                        SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                    End If
                    If primitiveMode = 4 Then
                        primitiveBordersize = Fix(getXcoordinate(getCursorpositionX()) - primitiveCenter.X) / Math.Sqrt(primitiveWidth ^ 2 + primitiveHeight ^ 2)
                        clip(primitiveBordersize, 0, 1)
                        If primitiveObject < 3 Then
                            For Each tri As Triangle In SelectedTriangles
                                tri.selected = False
                            Next
                            For Each vert As Vertex In SelectedVertices
                                vert.selected = False
                            Next
                            SelectedTriangles.Clear()
                            SelectedVertices.Clear()
                            Dim v11, v21, v31 As Vertex
                            v11 = New Vertex(primitiveCenter.X, primitiveCenter.Y + primitiveHeight, False)
                            v21 = New Vertex(primitiveCenter.X + primitiveHeight + primitiveWidth, primitiveCenter.Y - (primitiveHeight / 2), False)
                            v31 = New Vertex(primitiveCenter.X - primitiveHeight - primitiveWidth, primitiveCenter.Y - (primitiveHeight / 2), False)
                            Dim v12, v22, v32 As Vertex
                            v12 = New Vertex(primitiveCenter.X, primitiveCenter.Y + primitiveHeight * primitiveBordersize, False)
                            v22 = New Vertex(primitiveCenter.X + (primitiveHeight + primitiveWidth) * primitiveBordersize, primitiveCenter.Y - (primitiveHeight / 2) * primitiveBordersize, False)
                            v32 = New Vertex(primitiveCenter.X - (primitiveHeight + primitiveWidth) * primitiveBordersize, primitiveCenter.Y - (primitiveHeight / 2) * primitiveBordersize, False)
                            Vertices.Add(v11) : Vertices.Add(v12) ' 6 5
                            Vertices.Add(v21) : Vertices.Add(v22) ' 4 3 
                            Vertices.Add(v31) : Vertices.Add(v32) ' 2 1
                            If primitiveObject = 1 Then
                                Triangles.Add(New Triangle(v12, v22, v32))
                                Vertices(Vertices.Count - 1).linkedTriangles.Add(LLast(Triangles))
                                Vertices(Vertices.Count - 3).linkedTriangles.Add(LLast(Triangles))
                                Vertices(Vertices.Count - 5).linkedTriangles.Add(LLast(Triangles))
                                SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                            End If
                            Triangles.Add(New Triangle(v11, v22, v21))
                            Vertices(Vertices.Count - 6).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 3).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 4).linkedTriangles.Add(LLast(Triangles))
                            SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                            Triangles.Add(New Triangle(v11, v22, v12))
                            Vertices(Vertices.Count - 6).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 3).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 5).linkedTriangles.Add(LLast(Triangles))
                            SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                            Triangles.Add(New Triangle(v21, v32, v31))
                            Vertices(Vertices.Count - 4).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 1).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 2).linkedTriangles.Add(LLast(Triangles))
                            SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                            Triangles.Add(New Triangle(v21, v32, v22))
                            Vertices(Vertices.Count - 4).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 1).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 3).linkedTriangles.Add(LLast(Triangles))
                            SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                            Triangles.Add(New Triangle(v31, v12, v11))
                            Vertices(Vertices.Count - 2).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 5).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 6).linkedTriangles.Add(LLast(Triangles))
                            SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                            Triangles.Add(New Triangle(v31, v12, v32))
                            Vertices(Vertices.Count - 2).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 5).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 1).linkedTriangles.Add(LLast(Triangles))
                            SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                            SelectedVertices.Add(Vertices(Vertices.Count - 1)) : Vertices(Vertices.Count - 1).selected = True
                            SelectedVertices.Add(Vertices(Vertices.Count - 2)) : Vertices(Vertices.Count - 2).selected = True
                            SelectedVertices.Add(Vertices(Vertices.Count - 3)) : Vertices(Vertices.Count - 3).selected = True
                            SelectedVertices.Add(Vertices(Vertices.Count - 4)) : Vertices(Vertices.Count - 4).selected = True
                            SelectedVertices.Add(Vertices(Vertices.Count - 5)) : Vertices(Vertices.Count - 5).selected = True
                            SelectedVertices.Add(Vertices(Vertices.Count - 6)) : Vertices(Vertices.Count - 6).selected = True
                        End If
                        If primitiveObject = 4 OrElse primitiveObject = 5 Then
                            For Each tri As Triangle In SelectedTriangles
                                tri.selected = False
                            Next
                            For Each vert As Vertex In SelectedVertices
                                vert.selected = False
                            Next
                            SelectedTriangles.Clear()
                            SelectedVertices.Clear()
                            Dim v11, v21, v31, v41, v12, v22, v32, v42 As Vertex
                            v11 = New Vertex(primitiveCenter.X + primitiveWidth, primitiveCenter.Y + primitiveHeight, False)
                            v21 = New Vertex(primitiveCenter.X - primitiveWidth, primitiveCenter.Y + primitiveHeight, False)
                            v31 = New Vertex(primitiveCenter.X - primitiveWidth, primitiveCenter.Y - primitiveHeight, False)
                            v41 = New Vertex(primitiveCenter.X + primitiveWidth, primitiveCenter.Y - primitiveHeight, False)
                            v12 = New Vertex(primitiveCenter.X + primitiveWidth * primitiveBordersize, primitiveCenter.Y + primitiveHeight * primitiveBordersize, False)
                            v22 = New Vertex(primitiveCenter.X - primitiveWidth * primitiveBordersize, primitiveCenter.Y + primitiveHeight * primitiveBordersize, False)
                            v32 = New Vertex(primitiveCenter.X - primitiveWidth * primitiveBordersize, primitiveCenter.Y - primitiveHeight * primitiveBordersize, False)
                            v42 = New Vertex(primitiveCenter.X + primitiveWidth * primitiveBordersize, primitiveCenter.Y - primitiveHeight * primitiveBordersize, False)
                            Vertices.Add(v11) : Vertices.Add(v12) '8 7
                            Vertices.Add(v21) : Vertices.Add(v22) '6 5
                            Vertices.Add(v31) : Vertices.Add(v32) '4 3
                            Vertices.Add(v41) : Vertices.Add(v42) '2 1
                            If primitiveObject = 4 Then
                                Triangles.Add(New Triangle(v12, v22, v32))
                                Vertices(Vertices.Count - 7).linkedTriangles.Add(LLast(Triangles))
                                Vertices(Vertices.Count - 5).linkedTriangles.Add(LLast(Triangles))
                                Vertices(Vertices.Count - 3).linkedTriangles.Add(LLast(Triangles))
                                SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                                Triangles.Add(New Triangle(v32, v42, v12))
                                Vertices(Vertices.Count - 3).linkedTriangles.Add(LLast(Triangles))
                                Vertices(Vertices.Count - 1).linkedTriangles.Add(LLast(Triangles))
                                Vertices(Vertices.Count - 7).linkedTriangles.Add(LLast(Triangles))
                                SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                            End If
                            Triangles.Add(New Triangle(v11, v22, v21))
                            Vertices(Vertices.Count - 8).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 5).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 6).linkedTriangles.Add(LLast(Triangles))
                            SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                            Triangles.Add(New Triangle(v11, v22, v12))
                            Vertices(Vertices.Count - 8).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 5).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 7).linkedTriangles.Add(LLast(Triangles))
                            SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                            Triangles.Add(New Triangle(v21, v32, v31))
                            Vertices(Vertices.Count - 6).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 3).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 4).linkedTriangles.Add(LLast(Triangles))
                            SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                            Triangles.Add(New Triangle(v21, v32, v22))
                            Vertices(Vertices.Count - 6).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 3).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 5).linkedTriangles.Add(LLast(Triangles))
                            SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                            Triangles.Add(New Triangle(v31, v42, v41))
                            Vertices(Vertices.Count - 4).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 1).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 2).linkedTriangles.Add(LLast(Triangles))
                            SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                            Triangles.Add(New Triangle(v31, v42, v32))
                            Vertices(Vertices.Count - 4).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 1).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 3).linkedTriangles.Add(LLast(Triangles))
                            SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                            Triangles.Add(New Triangle(v41, v12, v11))
                            Vertices(Vertices.Count - 2).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 7).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 8).linkedTriangles.Add(LLast(Triangles))
                            SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                            Triangles.Add(New Triangle(v41, v12, v42))
                            Vertices(Vertices.Count - 2).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 7).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 1).linkedTriangles.Add(LLast(Triangles))
                            SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                            SelectedVertices.Add(Vertices(Vertices.Count - 1)) : Vertices(Vertices.Count - 1).selected = True
                            SelectedVertices.Add(Vertices(Vertices.Count - 2)) : Vertices(Vertices.Count - 2).selected = True
                            SelectedVertices.Add(Vertices(Vertices.Count - 3)) : Vertices(Vertices.Count - 3).selected = True
                            SelectedVertices.Add(Vertices(Vertices.Count - 4)) : Vertices(Vertices.Count - 4).selected = True
                            SelectedVertices.Add(Vertices(Vertices.Count - 5)) : Vertices(Vertices.Count - 5).selected = True
                            SelectedVertices.Add(Vertices(Vertices.Count - 6)) : Vertices(Vertices.Count - 6).selected = True
                            SelectedVertices.Add(Vertices(Vertices.Count - 7)) : Vertices(Vertices.Count - 7).selected = True
                            SelectedVertices.Add(Vertices(Vertices.Count - 8)) : Vertices(Vertices.Count - 8).selected = True
                        End If
                        If primitiveObject = 7 OrElse primitiveObject = 8 OrElse primitiveObject = 10 OrElse primitiveObject = 11 Then
                            Dim segments_temp As Integer
                            If primitiveObject < 10 Then segments_temp = segments_circle Else segments_temp = segments_oval
                            For Each tri As Triangle In SelectedTriangles
                                tri.selected = False
                            Next
                            For Each vert As Vertex In SelectedVertices
                                vert.selected = False
                            Next
                            SelectedTriangles.Clear()
                            SelectedVertices.Clear()
                            Dim vcenter As Vertex = Nothing
                            Dim vinner(segments_temp) As Vertex
                            Dim vouter(segments_temp) As Vertex
                            Dim counter As Integer = 0
                            If primitiveObject = 7 OrElse primitiveObject = 10 Then
                                vcenter = New Vertex(primitiveCenter.X, primitiveCenter.Y, False)
                                Vertices.Add(vcenter)
                                SelectedVertices.Add(LLast(Vertices)) : LLast(Vertices).selected = True
                            End If
                            For a As Double = 0 To Math.PI * 2 Step Math.PI * 2 / segments_temp
                                vinner(counter) = New Vertex(primitiveCenter.X + Math.Cos(a) * primitiveWidth * primitiveBordersize, primitiveCenter.Y + Math.Sin(a) * primitiveHeight * primitiveBordersize, False)
                                Vertices.Add(vinner(counter))
                                counter += 1
                            Next
                            If primitiveObject = 7 OrElse primitiveObject = 10 Then
                                Triangles.Add(New Triangle(vcenter, vinner(segments_temp - 1), vinner(0)))
                                Vertices(Vertices.Count - segments_temp - 1).linkedTriangles.Add(LLast(Triangles))
                                Vertices(Vertices.Count - 1).linkedTriangles.Add(LLast(Triangles))
                                Vertices(Vertices.Count - segments_temp).linkedTriangles.Add(LLast(Triangles))
                                SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                                For counter = 1 To segments_temp - 1
                                    Triangles.Add(New Triangle(vcenter, vinner(counter), vinner(counter - 1)))
                                    Vertices(Vertices.Count - segments_temp - 1).linkedTriangles.Add(LLast(Triangles))
                                    Vertices(Vertices.Count - segments_temp + counter).linkedTriangles.Add(LLast(Triangles))
                                    Vertices(Vertices.Count - segments_temp + counter - 1).linkedTriangles.Add(LLast(Triangles))
                                    SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                                Next
                            End If
                            counter = 0
                            For a As Double = 0 To Math.PI * 2 Step Math.PI * 2 / segments_temp
                                vouter(counter) = New Vertex(primitiveCenter.X + Math.Cos(a) * primitiveWidth, primitiveCenter.Y + Math.Sin(a) * primitiveHeight, False)
                                Vertices.Add(vouter(counter))
                                counter += 1
                            Next


                            Triangles.Add(New Triangle(vouter(segments_temp - 1), vinner(segments_temp - 1), vinner(0)))
                            Vertices(Vertices.Count - 1).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 1 - segments_temp).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - segments_temp * 2).linkedTriangles.Add(LLast(Triangles))
                            SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                            Triangles.Add(New Triangle(vouter(0), vouter(segments_temp - 1), vinner(0)))
                            Vertices(Vertices.Count - segments_temp).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 1).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - segments_temp * 2).linkedTriangles.Add(LLast(Triangles))
                            SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                            For counter = 1 To segments_temp - 1
                                Triangles.Add(New Triangle(vouter(counter - 1), vinner(counter - 1), vinner(counter)))
                                Vertices(Vertices.Count - segments_temp + counter - 1).linkedTriangles.Add(LLast(Triangles))
                                Vertices(Vertices.Count - segments_temp * 2 + counter - 1).linkedTriangles.Add(LLast(Triangles))
                                Vertices(Vertices.Count - segments_temp * 2 + counter).linkedTriangles.Add(LLast(Triangles))
                                SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                                Triangles.Add(New Triangle(vouter(counter), vouter(counter - 1), vinner(counter)))
                                Vertices(Vertices.Count - segments_temp + counter).linkedTriangles.Add(LLast(Triangles))
                                Vertices(Vertices.Count - segments_temp + counter - 1).linkedTriangles.Add(LLast(Triangles))
                                Vertices(Vertices.Count - segments_temp * 2 + counter).linkedTriangles.Add(LLast(Triangles))
                                SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                            Next

                            For counter = 0 To segments_temp * 2 - 1
                                SelectedVertices.Add(Vertices(Vertices.Count - counter - 1)) : Vertices(Vertices.Count - counter - 1).selected = True
                            Next
                            abortPrimitiveMode()
                        End If
                        abortPrimitiveMode()
                    End If
                    If primitiveMode = 3 Then
                        primitiveWidth = Fix(getXcoordinate(getCursorpositionX()) - primitiveCenter.X)
                        If primitiveObject = 0 Then
                            For Each tri As Triangle In SelectedTriangles
                                tri.selected = False
                            Next
                            For Each vert As Vertex In SelectedVertices
                                vert.selected = False
                            Next
                            SelectedTriangles.Clear()
                            SelectedVertices.Clear()
                            Dim v1, v2, v3 As Vertex
                            v1 = New Vertex(primitiveCenter.X, primitiveCenter.Y + primitiveHeight, False)
                            v2 = New Vertex(primitiveCenter.X + primitiveHeight, primitiveCenter.Y - (primitiveHeight / 2), False)
                            v3 = New Vertex(primitiveCenter.X - primitiveHeight, primitiveCenter.Y - (primitiveHeight / 2), False)
                            Vertices.Add(v1) : Vertices.Add(v2) : Vertices.Add(v3)
                            Triangles.Add(New Triangle(v1, v2, v3))
                            Vertices(Vertices.Count - 1).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 2).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 3).linkedTriangles.Add(LLast(Triangles))
                            LLast(Triangles).vertexB.X += primitiveWidth
                            LLast(Triangles).vertexC.X -= primitiveWidth
                            SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                            SelectedVertices.Add(Vertices(Vertices.Count - 1)) : Vertices(Vertices.Count - 1).selected = True
                            SelectedVertices.Add(Vertices(Vertices.Count - 2)) : Vertices(Vertices.Count - 2).selected = True
                            SelectedVertices.Add(Vertices(Vertices.Count - 3)) : Vertices(Vertices.Count - 3).selected = True
                            abortPrimitiveMode()
                        End If
                        If primitiveObject = 3 Then
                            For Each tri As Triangle In SelectedTriangles
                                tri.selected = False
                            Next
                            For Each vert As Vertex In SelectedVertices
                                vert.selected = False
                            Next
                            SelectedTriangles.Clear()
                            SelectedVertices.Clear()
                            Dim v1, v2, v3, v4 As Vertex
                            v1 = New Vertex(primitiveCenter.X + primitiveWidth, primitiveCenter.Y + primitiveHeight, False)
                            v2 = New Vertex(primitiveCenter.X - primitiveWidth, primitiveCenter.Y + primitiveHeight, False)
                            v3 = New Vertex(primitiveCenter.X - primitiveWidth, primitiveCenter.Y - primitiveHeight, False)
                            v4 = New Vertex(primitiveCenter.X + primitiveWidth, primitiveCenter.Y - primitiveHeight, False)
                            Vertices.Add(v1) : Vertices.Add(v2) : Vertices.Add(v3) : Vertices.Add(v4)
                            Triangles.Add(New Triangle(v1, v2, v3))
                            Vertices(Vertices.Count - 2).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 3).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 4).linkedTriangles.Add(LLast(Triangles))
                            SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                            Triangles.Add(New Triangle(v3, v4, v1))
                            Vertices(Vertices.Count - 2).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 1).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 4).linkedTriangles.Add(LLast(Triangles))
                            SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                            SelectedVertices.Add(Vertices(Vertices.Count - 1)) : Vertices(Vertices.Count - 1).selected = True
                            SelectedVertices.Add(Vertices(Vertices.Count - 2)) : Vertices(Vertices.Count - 2).selected = True
                            SelectedVertices.Add(Vertices(Vertices.Count - 3)) : Vertices(Vertices.Count - 3).selected = True
                            SelectedVertices.Add(Vertices(Vertices.Count - 4)) : Vertices(Vertices.Count - 4).selected = True
                            abortPrimitiveMode()
                        End If
                        If primitiveObject = 9 Then
                            For Each tri As Triangle In SelectedTriangles
                                tri.selected = False
                            Next
                            For Each vert As Vertex In SelectedVertices
                                vert.selected = False
                            Next
                            SelectedTriangles.Clear()
                            SelectedVertices.Clear()
                            primitiveWidth = Fix(Math.Sqrt((getYcoordinate(getCursorpositionY()) - primitiveCenter.Y) ^ 2 + (getXcoordinate(getCursorpositionX()) - primitiveCenter.X) ^ 2))
                            Dim vcenter As Vertex
                            Dim vouter(segments_oval) As Vertex
                            Dim counter As Integer = 0
                            vcenter = New Vertex(primitiveCenter.X, primitiveCenter.Y, False)
                            Vertices.Add(vcenter)
                            SelectedVertices.Add(LLast(Vertices)) : LLast(Vertices).selected = True
                            For a As Double = 0 To Math.PI * 2 Step Math.PI * 2 / segments_oval
                                vouter(counter) = New Vertex(primitiveCenter.X + Math.Cos(a) * primitiveWidth, primitiveCenter.Y + Math.Sin(a) * primitiveHeight, False)
                                Vertices.Add(vouter(counter))
                                counter += 1
                            Next
                            Triangles.Add(New Triangle(vcenter, vouter(segments_oval - 1), vouter(0)))
                            Vertices(Vertices.Count - segments_oval - 1).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 1).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - segments_oval).linkedTriangles.Add(LLast(Triangles))
                            SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                            For counter = 1 To segments_oval - 1
                                Triangles.Add(New Triangle(vcenter, vouter(counter), vouter(counter - 1)))
                                Vertices(Vertices.Count - segments_oval - 1).linkedTriangles.Add(LLast(Triangles))
                                Vertices(Vertices.Count - segments_oval + counter).linkedTriangles.Add(LLast(Triangles))
                                Vertices(Vertices.Count - segments_oval + counter - 1).linkedTriangles.Add(LLast(Triangles))
                                SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                            Next
                            For counter = 0 To segments_oval - 1
                                SelectedVertices.Add(Vertices(Vertices.Count - counter - 1)) : Vertices(Vertices.Count - counter - 1).selected = True
                            Next
                            abortPrimitiveMode()
                        End If
                        If primitiveObject <> 0 AndAlso primitiveObject <> 3 AndAlso primitiveObject <> 9 Then
                            primitiveMode = 4
                        End If
                    End If
                    If primitiveMode = 2 Then
                        primitiveHeight = Fix(getYcoordinate(getCursorpositionY()) - primitiveCenter.Y)
                        If primitiveObject < 6 Then
                            primitiveMode = 3
                        End If
                        If primitiveObject = 6 Then
                            For Each tri As Triangle In SelectedTriangles
                                tri.selected = False
                            Next
                            For Each vert As Vertex In SelectedVertices
                                vert.selected = False
                            Next
                            SelectedTriangles.Clear()
                            SelectedVertices.Clear()
                            primitiveHeight = Fix(Math.Sqrt((getYcoordinate(getCursorpositionY()) - primitiveCenter.Y) ^ 2 + (getXcoordinate(getCursorpositionX()) - primitiveCenter.X) ^ 2))
                            Dim vcenter As Vertex
                            Dim vouter(segments_circle) As Vertex
                            Dim counter As Integer = 0
                            vcenter = New Vertex(primitiveCenter.X, primitiveCenter.Y, False)
                            Vertices.Add(vcenter)
                            SelectedVertices.Add(LLast(Vertices)) : LLast(Vertices).selected = True
                            For a As Double = 0 To Math.PI * 2 Step Math.PI * 2 / segments_circle
                                vouter(counter) = New Vertex(primitiveCenter.X + Math.Cos(a) * primitiveHeight, primitiveCenter.Y + Math.Sin(a) * primitiveHeight, False)
                                Vertices.Add(vouter(counter))
                                counter += 1
                            Next
                            Triangles.Add(New Triangle(vcenter, vouter(segments_circle - 1), vouter(0)))
                            Vertices(Vertices.Count - segments_circle - 1).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - 1).linkedTriangles.Add(LLast(Triangles))
                            Vertices(Vertices.Count - segments_circle).linkedTriangles.Add(LLast(Triangles))
                            SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                            For counter = 1 To segments_circle - 1
                                Triangles.Add(New Triangle(vcenter, vouter(counter), vouter(counter - 1)))
                                Vertices(Vertices.Count - segments_circle - 1).linkedTriangles.Add(LLast(Triangles))
                                Vertices(Vertices.Count - segments_circle + counter).linkedTriangles.Add(LLast(Triangles))
                                Vertices(Vertices.Count - segments_circle + counter - 1).linkedTriangles.Add(LLast(Triangles))
                                SelectedTriangles.Add(LLast(Triangles)) : LLast(Triangles).selected = True
                            Next
                            For counter = 0 To segments_circle - 1
                                SelectedVertices.Add(Vertices(Vertices.Count - counter - 1)) : Vertices(Vertices.Count - counter - 1).selected = True
                            Next
                            abortPrimitiveMode()
                        End If
                        If primitiveObject = 7 OrElse primitiveObject = 8 Then
                            primitiveHeight = Fix(Math.Sqrt((getYcoordinate(getCursorpositionY()) - primitiveCenter.Y) ^ 2 + (getXcoordinate(getCursorpositionX()) - primitiveCenter.X) ^ 2))
                            primitiveWidth = primitiveHeight
                            primitiveMode = 4
                        End If
                        If primitiveObject = 9 OrElse primitiveObject = 10 OrElse primitiveObject = 11 Then
                            primitiveHeight = Fix(Math.Sqrt((getYcoordinate(getCursorpositionY()) - primitiveCenter.Y) ^ 2 + (getXcoordinate(getCursorpositionX()) - primitiveCenter.X) ^ 2))
                            primitiveMode = 3
                        End If
                    End If
                    If primitiveMode = 1 Then
                        'primitiveCenter = New Vertex(Fix(getXcoordinate(getCursorpositionX() - 3)), Fix(getYcoordinate(getCursorpositionY())), False, False)
                        primitiveCenter = New Vertex(Math.Round(getXcoordinate(getCursorpositionX() - 3) / Me.moveSnap) * Me.moveSnap, Math.Round(getYcoordinate(getCursorpositionY()) / Me.moveSnap) * Me.moveSnap, False, False)
                        If primitiveName = "" Then
                            primitiveMode = 2
                        Else
                            abortPrimitiveMode()
                            Select Case primitiveName
                                Case "disc44" : ndiscGenerator(4, 4, 16)
                                Case "disc34" : ndiscGenerator(3, 4, 16)
                                Case "disc58" : ndiscGenerator(5, 8, 16)
                                Case "disc24" : ndiscGenerator(2, 4, 16)
                                Case "disc716" : ndiscGenerator(7, 16, 16)
                                Case "disc38" : ndiscGenerator(3, 8, 16)
                                Case "disc516" : ndiscGenerator(5, 16, 16)
                                Case "disc14" : ndiscGenerator(1, 4, 16)
                                Case "disc316" : ndiscGenerator(3, 16, 16)
                                Case "disc18" : ndiscGenerator(1, 8, 16)
                                Case "disc116" : ndiscGenerator(1, 16, 16)

                                Case "disc4448" : discGenerator(4, 4, 48)
                                Case "disc2448" : discGenerator(2, 4, 48)
                                Case "disc1448" : discGenerator(1, 4, 48)
                                Case "disc31648" : discGenerator(3, 16, 48)
                                Case "disc1648" : discGenerator(1, 6, 48)
                                Case "disc1848" : discGenerator(1, 8, 48)
                                Case "disc54848" : discGenerator(5, 48, 48)
                                Case "disc11248" : discGenerator(1, 12, 48)

                            End Select
                            PrimitivesHMap.Add(primitiveIDglobal, Primitives.Count - 1)
                            primitiveName = ""
                            PrimitiveModeToolStripMenuItem.PerformClick()
                            addHistory()
                        End If

                    End If
                End If
            ElseIf e.Button = Windows.Forms.MouseButtons.Middle Then
            ElseIf e.Button = Windows.Forms.MouseButtons.Right Then
                kamera = False
                If BtnAddTriangle.Checked AndAlso dreiecksmodus > 0 Then
                    Triangles.Remove(LLast(Triangles))
                    dreiecksmodus = 0
                End If
            End If
        End If
        Me.Refresh()
    End Sub

#End Region
#Region "Mathematical Functions"
    Function getXcoordinate(ByVal relX As Integer) As Single
        Return (relX - Me.ClientSize.Width / 2) / zoomfaktor - Me.offsetX
    End Function

    Function getYcoordinate(ByVal relY As Integer) As Single
        Return (-relY + Me.ClientSize.Height / 2) / zoomfaktor + Me.offsetY
    End Function

    Private Sub clip(ByRef value As Single, ByVal min As Single, ByVal max As Single)
        value = Math.Max(value, min)
        value = Math.Min(value, max)
    End Sub

    Private Sub paintTriangles()
        For Each tri As Triangle In Triangles
            Select Case tri.myColourNumber
                Case 0 : tri.myColour = Me.C0.BackColor : Exit Select
                Case 1 : tri.myColour = Me.C1.BackColor : Exit Select
                Case 2 : tri.myColour = Me.C2.BackColor : Exit Select
                Case 3 : tri.myColour = Me.C3.BackColor : Exit Select
                Case 4 : tri.myColour = Me.C4.BackColor : Exit Select
                Case 5 : tri.myColour = Me.C5.BackColor : Exit Select
                Case 6 : tri.myColour = Me.C6.BackColor : Exit Select
                Case 7 : tri.myColour = Me.C7.BackColor : Exit Select
                Case 8 : tri.myColour = Me.C8.BackColor : Exit Select
                Case 9 : tri.myColour = Me.C9.BackColor : Exit Select
                Case 10 : tri.myColour = Me.C10.BackColor : Exit Select
                Case 11 : tri.myColour = Me.C11.BackColor : Exit Select
                Case 12 : tri.myColour = Me.C12.BackColor : Exit Select
                Case 13 : tri.myColour = Me.C13.BackColor : Exit Select
                Case 14 : tri.myColour = Me.C14.BackColor : Exit Select
                Case 15 : tri.myColour = Me.C15.BackColor : Exit Select
                Case 16 : tri.myColour = Nothing : Exit Select
            End Select
        Next
    End Sub

    Private Sub scaleVertices(ByVal rescale As Boolean)
        If rescale Then
            For Each vert As Vertex In Vertices
                vert.X /= 1000
                vert.Y /= 1000
            Next
        Else
            For Each vert As Vertex In Vertices
                vert.X = Fix(vert.X * 1000)
                vert.Y = Fix(vert.Y * 1000)
            Next
        End If
    End Sub

    Private Sub cleanupMergedTriangles()
        cleanupMergedTriangles(0)
    End Sub

    Private Sub cleanupMergedVertices()
        cleanupMergedVertices(0)
    End Sub

    Private Sub cleanupMergedTriangles(ByVal start As Integer)
        PBStatus.Value = 0
        PBStatus.Visible = True
        PBStatus.Minimum = 0
        Dim tstart As Integer = start
        For i As Integer = start To Triangles.Count - 1
            Triangles(i).normalize()
        Next
nextTri:
        If Triangles.Count > 0 Then
            If PBStatus.Value > (Triangles.Count - 1) Then PBStatus.Value = Triangles.Count - 1
            PBStatus.Maximum = Triangles.Count - 1
        End If
        For i1 As Integer = start To Triangles.Count - 1
            PBStatus.Value = i1
            Dim tri As Triangle = Triangles(i1)
            For i2 As Integer = start To Triangles.Count - 1
                Dim tri2 As Triangle = Triangles(i2)
                If tri.getFingerprint = tri2.getFingerprint AndAlso Not tri.Equals(tri2) Then
                    removeTriangle(tri)
                    start = i1
                    GoTo nextTri
                End If
            Next
        Next
        start = tstart
nextTri2:
        If Triangles.Count > 0 Then
            If PBStatus.Value > (Triangles.Count - 1) Then PBStatus.Value = Triangles.Count - 1
            PBStatus.Maximum = Triangles.Count - 1
        End If
        For i1 As Integer = start To Triangles.Count - 1
            PBStatus.Value = i1
            Dim tri As Triangle = Triangles(i1)
            If tri.vertexA.X = tri.vertexB.X AndAlso tri.vertexA.Y = tri.vertexB.Y Then removeTriangle(tri) : start = i1 : GoTo nextTri2
            If tri.vertexA.X = tri.vertexC.X AndAlso tri.vertexA.Y = tri.vertexC.Y Then removeTriangle(tri) : start = i1 : GoTo nextTri2
            If tri.vertexB.X = tri.vertexC.X AndAlso tri.vertexB.Y = tri.vertexC.Y Then removeTriangle(tri) : start = i1 : GoTo nextTri2
        Next
        PBStatus.Visible = False
    End Sub

    Private Sub cleanupMergedVertices(ByVal start As Integer)
        PBStatus.Value = 0
        PBStatus.Visible = True
        PBStatus.Minimum = 0
nextVert:
        For i1 As Integer = start To Vertices.Count - 1
            Dim vert As Vertex = Vertices(i1)
            For i2 As Integer = start To Vertices.Count - 1
                Dim vert2 As Vertex = Vertices(i2)
                If vert.X >= vert2.X - 0.2 AndAlso vert.X <= vert2.X + 0.2 AndAlso vert.Y >= vert2.Y - 0.2 AndAlso vert.Y <= vert2.Y + 0.2 AndAlso Not vert.Equals(vert2) Then
                    For Each tri As Triangle In vert.linkedTriangles
                        If Not vert2.linkedTriangles.Contains(tri) Then
                            vert2.linkedTriangles.Add(tri)
                            If tri.vertexA.Equals(vert) Then tri.vertexA = vert2
                            If tri.vertexB.Equals(vert) Then tri.vertexB = vert2
                            If tri.vertexC.Equals(vert) Then tri.vertexC = vert2
                        End If
                    Next
                    Vertices.Remove(vert)
                    start = i1
                    GoTo nextVert
                End If
            Next
        Next
        PBStatus.Visible = False
    End Sub

    Private Sub removeTriangle(ByVal tri As Triangle)
        tri.vertexA.linkedTriangles.Remove(tri)
        tri.vertexB.linkedTriangles.Remove(tri)
        tri.vertexC.linkedTriangles.Remove(tri)
        Triangles.Remove(tri)
    End Sub

    Private Function CSG2Triangles(ByRef tri1 As Triangle, ByRef tri2 As Triangle) As Boolean

        Dim verticesIntersections As New List(Of Vertex)
        Dim verticesInner As New List(Of Vertex)
        Dim verticesOuter As New List(Of Vertex)
        Dim verticesComplete As New List(Of Vertex)


        schnittpunkt = intersectionBetweenTwoLines(tri1.vertexA, tri1.vertexB, tri2.vertexA, tri2.vertexB)
        If Not schnittpunkt Is Nothing Then verticesIntersections.Add(New Vertex(schnittpunkt.X, schnittpunkt.Y, False))
        schnittpunkt = intersectionBetweenTwoLines(tri1.vertexA, tri1.vertexB, tri2.vertexB, tri2.vertexC)
        If Not schnittpunkt Is Nothing Then verticesIntersections.Add(New Vertex(schnittpunkt.X, schnittpunkt.Y, False))
        schnittpunkt = intersectionBetweenTwoLines(tri1.vertexA, tri1.vertexB, tri2.vertexC, tri2.vertexA)
        If Not schnittpunkt Is Nothing Then verticesIntersections.Add(New Vertex(schnittpunkt.X, schnittpunkt.Y, False))

        schnittpunkt = intersectionBetweenTwoLines(tri1.vertexB, tri1.vertexC, tri2.vertexA, tri2.vertexB)
        If Not schnittpunkt Is Nothing Then verticesIntersections.Add(New Vertex(schnittpunkt.X, schnittpunkt.Y, False))
        schnittpunkt = intersectionBetweenTwoLines(tri1.vertexB, tri1.vertexC, tri2.vertexB, tri2.vertexC)
        If Not schnittpunkt Is Nothing Then verticesIntersections.Add(New Vertex(schnittpunkt.X, schnittpunkt.Y, False))
        schnittpunkt = intersectionBetweenTwoLines(tri1.vertexB, tri1.vertexC, tri2.vertexC, tri2.vertexA)
        If Not schnittpunkt Is Nothing Then verticesIntersections.Add(New Vertex(schnittpunkt.X, schnittpunkt.Y, False))

        schnittpunkt = intersectionBetweenTwoLines(tri1.vertexC, tri1.vertexA, tri2.vertexA, tri2.vertexB)
        If Not schnittpunkt Is Nothing Then verticesIntersections.Add(New Vertex(schnittpunkt.X, schnittpunkt.Y, False))
        schnittpunkt = intersectionBetweenTwoLines(tri1.vertexC, tri1.vertexA, tri2.vertexB, tri2.vertexC)
        If Not schnittpunkt Is Nothing Then verticesIntersections.Add(New Vertex(schnittpunkt.X, schnittpunkt.Y, False))
        schnittpunkt = intersectionBetweenTwoLines(tri1.vertexC, tri1.vertexA, tri2.vertexC, tri2.vertexA)
        If Not schnittpunkt Is Nothing Then verticesIntersections.Add(New Vertex(schnittpunkt.X, schnittpunkt.Y, False))

        ' Bestimme innere Punkte:

        Dim vert0(2) As Decimal
        Dim vert1(2) As Decimal
        Dim vert2(2) As Decimal
        vert0(2) = 0 : vert1(2) = 0 : vert2(2) = 0 : beamorig(2) = 1
        ' -> Test auf Dreieck 1
        vert0(0) = tri1.vertexA.X
        vert0(1) = tri1.vertexA.Y
        vert1(0) = tri1.vertexB.X
        vert1(1) = tri1.vertexB.Y
        vert2(0) = tri1.vertexC.X
        vert2(1) = tri1.vertexC.Y
        If tri2.vertexA <> tri1.vertexA AndAlso tri2.vertexA <> tri1.vertexB AndAlso tri2.vertexA <> tri1.vertexC Then
            beamorig(0) = tri2.vertexA.X : beamorig(1) = tri2.vertexA.Y
            If PowerRay.SCHNITTPKT_DREIECK(beamorig, beamdir, vert0, vert1, vert2) Then
                verticesInner.Add(tri2.vertexA)
                tri2.vertexA.selected = True
            End If
        End If
        If tri2.vertexB <> tri1.vertexA AndAlso tri2.vertexB <> tri1.vertexB AndAlso tri2.vertexB <> tri1.vertexC Then
            beamorig(0) = tri2.vertexB.X : beamorig(1) = tri2.vertexB.Y
            If PowerRay.SCHNITTPKT_DREIECK(beamorig, beamdir, vert0, vert1, vert2) Then
                verticesInner.Add(tri2.vertexB)
                tri2.vertexB.selected = True
            End If
        End If
        If tri2.vertexC <> tri1.vertexA AndAlso tri2.vertexC <> tri1.vertexB AndAlso tri2.vertexC <> tri1.vertexC Then
            beamorig(0) = tri2.vertexC.X : beamorig(1) = tri2.vertexC.Y
            If PowerRay.SCHNITTPKT_DREIECK(beamorig, beamdir, vert0, vert1, vert2) Then
                verticesInner.Add(tri2.vertexC)
                tri2.vertexC.selected = True
            End If
        End If

        ' -> Test auf Dreieck 2
        vert0(0) = tri2.vertexA.X
        vert0(1) = tri2.vertexA.Y
        vert1(0) = tri2.vertexB.X
        vert1(1) = tri2.vertexB.Y
        vert2(0) = tri2.vertexC.X
        vert2(1) = tri2.vertexC.Y
        If tri1.vertexA <> tri2.vertexA AndAlso tri1.vertexA <> tri2.vertexB AndAlso tri1.vertexA <> tri2.vertexC Then
            beamorig(0) = tri1.vertexA.X : beamorig(1) = tri1.vertexA.Y
            If PowerRay.SCHNITTPKT_DREIECK(beamorig, beamdir, vert0, vert1, vert2) Then
                verticesInner.Add(tri1.vertexA)
                tri1.vertexA.selected = True
            End If
        End If
        If tri1.vertexB <> tri2.vertexA AndAlso tri1.vertexB <> tri2.vertexB AndAlso tri1.vertexB <> tri2.vertexC Then
            beamorig(0) = tri1.vertexB.X : beamorig(1) = tri1.vertexB.Y
            If PowerRay.SCHNITTPKT_DREIECK(beamorig, beamdir, vert0, vert1, vert2) Then
                verticesInner.Add(tri1.vertexB)
                tri1.vertexB.selected = True
            End If
        End If
        If tri1.vertexC <> tri2.vertexA AndAlso tri1.vertexC <> tri2.vertexB AndAlso tri1.vertexC <> tri2.vertexC Then
            beamorig(0) = tri1.vertexC.X : beamorig(1) = tri1.vertexC.Y
            If PowerRay.SCHNITTPKT_DREIECK(beamorig, beamdir, vert0, vert1, vert2) Then
                verticesInner.Add(tri1.vertexC)
                tri1.vertexC.selected = True
            End If
        End If


        If verticesInner.Count > 0 OrElse verticesIntersections.Count > 0 Then

            ' Bestimme äußere Punkte:
            If Not verticesInner.Contains(tri1.vertexA) Then verticesOuter.Add(tri1.vertexA)
            If Not verticesInner.Contains(tri1.vertexB) Then verticesOuter.Add(tri1.vertexB)
            If Not verticesInner.Contains(tri1.vertexC) Then verticesOuter.Add(tri1.vertexC)

            If Not verticesInner.Contains(tri2.vertexA) Then verticesOuter.Add(tri2.vertexA)
            If Not verticesInner.Contains(tri2.vertexB) Then verticesOuter.Add(tri2.vertexB)
            If Not verticesInner.Contains(tri2.vertexC) Then verticesOuter.Add(tri2.vertexC)

            If verticesInner.Count = 1 AndAlso verticesIntersections.Count = 2 Then

                ' 1. Fall: Ein Punkt innerhalb des anderen / 2 Schnittpunkte [okay]
                ' [a] Alle Schnittpunkte zu den Gesamtdaten hinzufügen
                Vertices.AddRange(verticesIntersections)
                verticesInner.AddRange(verticesIntersections)

                ' [b] Inneres Dreieck erstellen
                Triangles.Add(New Triangle(verticesInner(0), verticesInner(1), verticesInner(2)))
                verticesInner(0).linkedTriangles.Add(LLast(Triangles))
                verticesInner(1).linkedTriangles.Add(LLast(Triangles))
                verticesInner(2).linkedTriangles.Add(LLast(Triangles))
                SelectedTriangles.Add(LLast(Triangles))

                Dim center As Vertex = LLast(Triangles).getCenter() 'tri1.getCenter() + tri2.getCenter()
                verticesComplete.AddRange(verticesInner)
                verticesComplete.AddRange(verticesOuter)

                For Each vert As Vertex In verticesComplete
                    vert.angleFrom = center.angle(vert)
                Next

                verticesComplete.Sort()

                ' [c] zwei vor, einen zurück..

                Triangles.Add(New Triangle(verticesComplete(0), verticesComplete(1), verticesComplete(2)))
                If verticesIntersections.Contains(verticesComplete(0)) AndAlso verticesComplete(1).linkedTriangles.Contains(tri1) <> verticesComplete(2).linkedTriangles.Contains(tri1) AndAlso verticesOuter.Contains(verticesComplete(1)) AndAlso verticesOuter.Contains(verticesComplete(2)) Then LLast(Triangles).selected = True
                If verticesIntersections.Contains(verticesComplete(1)) AndAlso verticesComplete(0).linkedTriangles.Contains(tri1) <> verticesComplete(2).linkedTriangles.Contains(tri1) AndAlso verticesOuter.Contains(verticesComplete(0)) AndAlso verticesOuter.Contains(verticesComplete(2)) Then LLast(Triangles).selected = True
                If verticesIntersections.Contains(verticesComplete(2)) AndAlso verticesComplete(0).linkedTriangles.Contains(tri1) <> verticesComplete(1).linkedTriangles.Contains(tri1) AndAlso verticesOuter.Contains(verticesComplete(0)) AndAlso verticesOuter.Contains(verticesComplete(1)) Then LLast(Triangles).selected = True
                verticesComplete(0).linkedTriangles.Add(LLast(Triangles))
                verticesComplete(1).linkedTriangles.Add(LLast(Triangles))
                verticesComplete(2).linkedTriangles.Add(LLast(Triangles))
                SelectedTriangles.Add(LLast(Triangles))


                For i As Integer = 1 To 7
                    Dim j, k
                    j = i + 1
                    k = i + 2
                    If j > 7 Then j -= 8
                    If k > 7 Then k -= 8
                    If (Not verticesInner.Contains(verticesComplete(i)) AndAlso verticesInner.Contains(verticesComplete(j))) OrElse _
                  (verticesInner.Contains(verticesComplete(i)) AndAlso Not verticesInner.Contains(verticesComplete(j))) Then
                        Triangles.Add(New Triangle(verticesComplete(i), verticesComplete(j), verticesComplete(k)))
                        If verticesIntersections.Contains(verticesComplete(i)) AndAlso verticesComplete(j).linkedTriangles.Contains(tri1) <> verticesComplete(k).linkedTriangles.Contains(tri1) AndAlso verticesOuter.Contains(verticesComplete(j)) AndAlso verticesOuter.Contains(verticesComplete(k)) Then LLast(Triangles).selected = True
                        If verticesIntersections.Contains(verticesComplete(j)) AndAlso verticesComplete(i).linkedTriangles.Contains(tri1) <> verticesComplete(k).linkedTriangles.Contains(tri1) AndAlso verticesOuter.Contains(verticesComplete(i)) AndAlso verticesOuter.Contains(verticesComplete(k)) Then LLast(Triangles).selected = True
                        If verticesIntersections.Contains(verticesComplete(k)) AndAlso verticesComplete(i).linkedTriangles.Contains(tri1) <> verticesComplete(j).linkedTriangles.Contains(tri1) AndAlso verticesOuter.Contains(verticesComplete(i)) AndAlso verticesOuter.Contains(verticesComplete(j)) Then LLast(Triangles).selected = True
                        verticesComplete(i).linkedTriangles.Add(LLast(Triangles))
                        verticesComplete(j).linkedTriangles.Add(LLast(Triangles))
                        verticesComplete(k).linkedTriangles.Add(LLast(Triangles))
                        SelectedTriangles.Add(LLast(Triangles))
                    Else
                        Triangles.Add(New Triangle(verticesComplete(i - 1), verticesComplete(j), verticesComplete(k)))
                        If verticesIntersections.Contains(verticesComplete(i - 1)) AndAlso verticesComplete(j).linkedTriangles.Contains(tri1) <> verticesComplete(k).linkedTriangles.Contains(tri1) AndAlso verticesOuter.Contains(verticesComplete(j)) AndAlso verticesOuter.Contains(verticesComplete(k)) Then LLast(Triangles).selected = True
                        If verticesIntersections.Contains(verticesComplete(j)) AndAlso verticesComplete(i - 1).linkedTriangles.Contains(tri1) <> verticesComplete(k).linkedTriangles.Contains(tri1) AndAlso verticesOuter.Contains(verticesComplete(i - 1)) AndAlso verticesOuter.Contains(verticesComplete(k)) Then LLast(Triangles).selected = True
                        If verticesIntersections.Contains(verticesComplete(k)) AndAlso verticesComplete(i - 1).linkedTriangles.Contains(tri1) <> verticesComplete(j).linkedTriangles.Contains(tri1) AndAlso verticesOuter.Contains(verticesComplete(i - 1)) AndAlso verticesOuter.Contains(verticesComplete(j)) Then LLast(Triangles).selected = True
                        verticesComplete(i - 1).linkedTriangles.Add(LLast(Triangles))
                        verticesComplete(j).linkedTriangles.Add(LLast(Triangles))
                        verticesComplete(k).linkedTriangles.Add(LLast(Triangles))
                        SelectedTriangles.Add(LLast(Triangles))
                    End If
                Next

                ' [c] CleanUp

                For i As Integer = Triangles.Count - 8 To Triangles.Count - 1
                    If Triangles(i).selected Then
                        SelectedTriangles.Remove(Triangles(i))
                        Triangles.Remove(Triangles(i))
                        Exit For
                    End If
                Next

                For i As Integer = Triangles.Count - 7 To Triangles.Count - 1
                    If Triangles(i).selected Then
                        SelectedTriangles.Remove(Triangles(i))
                        Triangles.Remove(Triangles(i))
                        Exit For
                    End If
                Next

                Return True
            End If

            If verticesInner.Count = 3 AndAlso verticesIntersections.Count = 0 Then

                ' 2. Fall: Dreieck komplett innerhalb des anderen [okay]
                ' [a] Alle Schnittpunkte zu den Gesamtdaten hinzufügen
                Vertices.AddRange(verticesIntersections)

                ' [b] Inneres Dreieck erstellen
                Triangles.Add(New Triangle(verticesInner(0), verticesInner(1), verticesInner(2)))
                verticesInner(0).linkedTriangles.Add(LLast(Triangles))
                verticesInner(1).linkedTriangles.Add(LLast(Triangles))
                verticesInner(2).linkedTriangles.Add(LLast(Triangles))
                SelectedTriangles.Add(LLast(Triangles))

                Dim center As Vertex = LLast(Triangles).getCenter()

                verticesComplete.AddRange(verticesInner)
                verticesComplete.AddRange(verticesOuter)

                For Each vert As Vertex In verticesComplete
                    vert.angleFrom = center.angle(vert)
                Next

                verticesComplete.Sort()

                ' [c] zwei vor, einen zurück..

                Triangles.Add(New Triangle(verticesComplete(0), verticesComplete(1), verticesComplete(2)))
                verticesComplete(0).linkedTriangles.Add(LLast(Triangles))
                verticesComplete(1).linkedTriangles.Add(LLast(Triangles))
                verticesComplete(2).linkedTriangles.Add(LLast(Triangles))
                SelectedTriangles.Add(LLast(Triangles))

                For i As Integer = 1 To 5
                    Dim j, k
                    j = i + 1
                    k = i + 2
                    If j > 5 Then j -= 6
                    If k > 5 Then k -= 6
                    If (Not verticesInner.Contains(verticesComplete(i)) AndAlso verticesInner.Contains(verticesComplete(j))) OrElse _
                  (verticesInner.Contains(verticesComplete(i)) AndAlso Not verticesInner.Contains(verticesComplete(j))) Then
                        Triangles.Add(New Triangle(verticesComplete(i), verticesComplete(j), verticesComplete(k)))
                        verticesComplete(i).linkedTriangles.Add(LLast(Triangles))
                        verticesComplete(j).linkedTriangles.Add(LLast(Triangles))
                        verticesComplete(k).linkedTriangles.Add(LLast(Triangles))
                        SelectedTriangles.Add(LLast(Triangles))
                    Else
                        Triangles.Add(New Triangle(verticesComplete(i - 1), verticesComplete(j), verticesComplete(k)))
                        verticesComplete(i - 1).linkedTriangles.Add(LLast(Triangles))
                        verticesComplete(j).linkedTriangles.Add(LLast(Triangles))
                        verticesComplete(k).linkedTriangles.Add(LLast(Triangles))
                        SelectedTriangles.Add(LLast(Triangles))
                    End If
                Next

                Return True
            End If

            If verticesInner.Count = 2 AndAlso verticesIntersections.Count = 2 Then

                ' 3. Fall: Zwei Punkte innerhalb des anderen / 2 Schnittpunkte [okay]
                ' [a] Alle Schnittpunkte zu den Gesamtdaten hinzufügen
                Vertices.AddRange(verticesIntersections)
                verticesInner.AddRange(verticesIntersections)

                ' [b] Innere Dreiecke erstellen
                Dim center As Vertex = verticesInner(0) + verticesInner(1) + verticesInner(2) + verticesInner(3)
                center.X /= 4
                center.Y /= 4

                If verticesInner(0).linkedTriangles.Contains(tri1) = verticesInner(1).linkedTriangles.Contains(tri1) Then
                    Triangles.Add(New Triangle(verticesInner(0), verticesInner(1), verticesInner(2)))
                    verticesInner(0).linkedTriangles.Add(LLast(Triangles))
                    verticesInner(1).linkedTriangles.Add(LLast(Triangles))
                    verticesInner(2).linkedTriangles.Add(LLast(Triangles))
                    SelectedTriangles.Add(LLast(Triangles))

                    Triangles.Add(New Triangle(verticesInner(2), verticesInner(3), verticesInner(0)))
                    verticesInner(2).linkedTriangles.Add(LLast(Triangles))
                    verticesInner(3).linkedTriangles.Add(LLast(Triangles))
                    verticesInner(0).linkedTriangles.Add(LLast(Triangles))
                    SelectedTriangles.Add(LLast(Triangles))
                Else
                    Triangles.Add(New Triangle(verticesInner(2), verticesInner(3), verticesInner(0)))
                    verticesInner(2).linkedTriangles.Add(LLast(Triangles))
                    verticesInner(3).linkedTriangles.Add(LLast(Triangles))
                    verticesInner(0).linkedTriangles.Add(LLast(Triangles))
                    SelectedTriangles.Add(LLast(Triangles))

                    Triangles.Add(New Triangle(verticesInner(2), verticesInner(3), verticesInner(1)))
                    verticesInner(2).linkedTriangles.Add(LLast(Triangles))
                    verticesInner(3).linkedTriangles.Add(LLast(Triangles))
                    verticesInner(1).linkedTriangles.Add(LLast(Triangles))
                    SelectedTriangles.Add(LLast(Triangles))
                End If

                verticesComplete.AddRange(verticesInner)
                verticesComplete.AddRange(verticesOuter)

                For Each vert As Vertex In verticesComplete
                    vert.angleFrom = center.angle(vert)
                Next

                verticesComplete.Sort()

                ' [c] zwei vor, einen zurück..

                Triangles.Add(New Triangle(verticesComplete(0), verticesComplete(1), verticesComplete(2)))
                verticesComplete(0).linkedTriangles.Add(LLast(Triangles))
                verticesComplete(1).linkedTriangles.Add(LLast(Triangles))
                verticesComplete(2).linkedTriangles.Add(LLast(Triangles))
                If verticesIntersections.Contains(verticesComplete(0)) AndAlso verticesComplete(1).linkedTriangles.Contains(tri1) <> verticesComplete(2).linkedTriangles.Contains(tri1) AndAlso verticesOuter.Contains(verticesComplete(1)) AndAlso verticesOuter.Contains(verticesComplete(2)) Then LLast(Triangles).selected = True
                If verticesIntersections.Contains(verticesComplete(1)) AndAlso verticesComplete(0).linkedTriangles.Contains(tri1) <> verticesComplete(2).linkedTriangles.Contains(tri1) AndAlso verticesOuter.Contains(verticesComplete(0)) AndAlso verticesOuter.Contains(verticesComplete(2)) Then LLast(Triangles).selected = True
                If verticesIntersections.Contains(verticesComplete(2)) AndAlso verticesComplete(0).linkedTriangles.Contains(tri1) <> verticesComplete(1).linkedTriangles.Contains(tri1) AndAlso verticesOuter.Contains(verticesComplete(0)) AndAlso verticesOuter.Contains(verticesComplete(1)) Then LLast(Triangles).selected = True
                SelectedTriangles.Add(LLast(Triangles))


                For i As Integer = 1 To 7
                    Dim j, k
                    j = i + 1
                    k = i + 2
                    If j > 7 Then j -= 8
                    If k > 7 Then k -= 8
                    If (Not verticesInner.Contains(verticesComplete(i)) AndAlso verticesInner.Contains(verticesComplete(j))) OrElse _
                  (verticesInner.Contains(verticesComplete(i)) AndAlso Not verticesInner.Contains(verticesComplete(j))) Then
                        Triangles.Add(New Triangle(verticesComplete(i), verticesComplete(j), verticesComplete(k)))
                        If verticesIntersections.Contains(verticesComplete(i)) AndAlso verticesComplete(j).linkedTriangles.Contains(tri1) <> verticesComplete(k).linkedTriangles.Contains(tri1) AndAlso verticesOuter.Contains(verticesComplete(j)) AndAlso verticesOuter.Contains(verticesComplete(k)) Then LLast(Triangles).selected = True
                        If verticesIntersections.Contains(verticesComplete(j)) AndAlso verticesComplete(i).linkedTriangles.Contains(tri1) <> verticesComplete(k).linkedTriangles.Contains(tri1) AndAlso verticesOuter.Contains(verticesComplete(i)) AndAlso verticesOuter.Contains(verticesComplete(k)) Then LLast(Triangles).selected = True
                        If verticesIntersections.Contains(verticesComplete(k)) AndAlso verticesComplete(i).linkedTriangles.Contains(tri1) <> verticesComplete(j).linkedTriangles.Contains(tri1) AndAlso verticesOuter.Contains(verticesComplete(i)) AndAlso verticesOuter.Contains(verticesComplete(j)) Then LLast(Triangles).selected = True
                        verticesComplete(i).linkedTriangles.Add(LLast(Triangles))
                        verticesComplete(j).linkedTriangles.Add(LLast(Triangles))
                        verticesComplete(k).linkedTriangles.Add(LLast(Triangles))
                        SelectedTriangles.Add(LLast(Triangles))
                    Else
                        Triangles.Add(New Triangle(verticesComplete(i - 1), verticesComplete(j), verticesComplete(k)))
                        If verticesIntersections.Contains(verticesComplete(i - 1)) AndAlso verticesComplete(j).linkedTriangles.Contains(tri1) <> verticesComplete(k).linkedTriangles.Contains(tri1) AndAlso verticesOuter.Contains(verticesComplete(j)) AndAlso verticesOuter.Contains(verticesComplete(k)) Then LLast(Triangles).selected = True
                        If verticesIntersections.Contains(verticesComplete(j)) AndAlso verticesComplete(i - 1).linkedTriangles.Contains(tri1) <> verticesComplete(k).linkedTriangles.Contains(tri1) AndAlso verticesOuter.Contains(verticesComplete(i - 1)) AndAlso verticesOuter.Contains(verticesComplete(k)) Then LLast(Triangles).selected = True
                        If verticesIntersections.Contains(verticesComplete(k)) AndAlso verticesComplete(i - 1).linkedTriangles.Contains(tri1) <> verticesComplete(j).linkedTriangles.Contains(tri1) AndAlso verticesOuter.Contains(verticesComplete(i - 1)) AndAlso verticesOuter.Contains(verticesComplete(j)) Then LLast(Triangles).selected = True
                        verticesComplete(i - 1).linkedTriangles.Add(LLast(Triangles))
                        verticesComplete(j).linkedTriangles.Add(LLast(Triangles))
                        verticesComplete(k).linkedTriangles.Add(LLast(Triangles))
                        SelectedTriangles.Add(LLast(Triangles))
                    End If
                Next

                ' [c] CleanUp

                For i As Integer = Triangles.Count - 8 To Triangles.Count - 1
                    If Triangles(i).selected Then
                        SelectedTriangles.Remove(Triangles(i))
                        Triangles.Remove(Triangles(i))
                        Exit For
                    End If
                Next

                For i As Integer = Triangles.Count - 7 To Triangles.Count - 1
                    If Triangles(i).selected Then
                        SelectedTriangles.Remove(Triangles(i))
                        Triangles.Remove(Triangles(i))
                        Exit For
                    End If
                Next

                Return True
            End If

            If verticesInner.Count = 1 AndAlso verticesIntersections.Count = 5 Then

                ' 4. Fall: Ein Punkt innerhalb des anderen / 5 Schnittpunkte (Punkt auf Punkt)
                ' [a1] Schnittpunkte säubern
                Dim temp_vertex As Vertex = Nothing
                If verticesIntersections(0) <> verticesIntersections(1) AndAlso verticesIntersections(0) <> verticesIntersections(2) Then
                    temp_vertex = verticesIntersections(0)
                ElseIf verticesIntersections(1) <> verticesIntersections(2) AndAlso verticesIntersections(1) <> verticesIntersections(3) Then
                    temp_vertex = verticesIntersections(1)
                ElseIf verticesIntersections(2) <> verticesIntersections(3) AndAlso verticesIntersections(2) <> verticesIntersections(4) Then
                    temp_vertex = verticesIntersections(2)
                ElseIf verticesIntersections(3) <> verticesIntersections(2) AndAlso verticesIntersections(3) <> verticesIntersections(4) Then
                    temp_vertex = verticesIntersections(3)
                ElseIf verticesIntersections(4) <> verticesIntersections(2) AndAlso verticesIntersections(4) <> verticesIntersections(3) Then
                    temp_vertex = verticesIntersections(4)
                End If
                verticesIntersections.Clear()
                verticesIntersections.Add(temp_vertex)

                ' [a2] Alle Schnittpunkte zu den Gesamtdaten hinzufügen
                Vertices.AddRange(verticesIntersections)
                verticesInner.AddRange(verticesIntersections)

                ' [b] Inneres Dreieck erstellen
                Dim temp_vertex2 As Vertex = Nothing
                If tri1.vertexA = tri2.vertexA Then temp_vertex = tri1.vertexA : temp_vertex2 = tri2.vertexA
                If tri1.vertexA = tri2.vertexB Then temp_vertex = tri1.vertexA : temp_vertex2 = tri2.vertexB
                If tri1.vertexA = tri2.vertexC Then temp_vertex = tri1.vertexA : temp_vertex2 = tri2.vertexC

                If tri1.vertexB = tri2.vertexA Then temp_vertex = tri1.vertexB : temp_vertex2 = tri2.vertexA
                If tri1.vertexB = tri2.vertexB Then temp_vertex = tri1.vertexB : temp_vertex2 = tri2.vertexB
                If tri1.vertexB = tri2.vertexC Then temp_vertex = tri1.vertexB : temp_vertex2 = tri2.vertexC

                If tri1.vertexC = tri2.vertexA Then temp_vertex = tri1.vertexC : temp_vertex2 = tri2.vertexA
                If tri1.vertexC = tri2.vertexB Then temp_vertex = tri1.vertexC : temp_vertex2 = tri2.vertexB
                If tri1.vertexC = tri2.vertexC Then temp_vertex = tri1.vertexC : temp_vertex2 = tri2.vertexC

                verticesOuter.Remove(temp_vertex)
                verticesOuter.Remove(temp_vertex2)
                verticesInner.Add(temp_vertex)

                Triangles.Add(New Triangle(verticesInner(0), verticesInner(1), verticesInner(2)))
                verticesInner(0).linkedTriangles.Add(LLast(Triangles))
                verticesInner(1).linkedTriangles.Add(LLast(Triangles))
                verticesInner(2).linkedTriangles.Add(LLast(Triangles))
                SelectedTriangles.Add(LLast(Triangles))

                Dim center As Vertex = LLast(Triangles).getCenter()

                verticesComplete.AddRange(verticesInner)
                verticesComplete.AddRange(verticesOuter)

                For Each vert As Vertex In verticesComplete
                    vert.angleFrom = center.angle(vert)
                Next

                verticesComplete.Sort()

                ' [c] zwei vor, einen zurück..

                For i As Integer = 1 To 5
                    Dim j, k
                    j = i + 1
                    k = i + 2
                    If j > 5 Then j -= 6
                    If k > 5 Then k -= 6
                    If (Not verticesInner.Contains(verticesComplete(i)) AndAlso verticesInner.Contains(verticesComplete(j))) OrElse _
                  (verticesInner.Contains(verticesComplete(i)) AndAlso Not verticesInner.Contains(verticesComplete(j))) Then
                        Triangles.Add(New Triangle(verticesComplete(i), verticesComplete(j), verticesComplete(k)))
                        verticesComplete(i).linkedTriangles.Add(LLast(Triangles))
                        verticesComplete(j).linkedTriangles.Add(LLast(Triangles))
                        verticesComplete(k).linkedTriangles.Add(LLast(Triangles))
                        SelectedTriangles.Add(LLast(Triangles))
                    Else
                        Triangles.Add(New Triangle(verticesComplete(i - 1), verticesComplete(j), verticesComplete(k)))
                        verticesComplete(i - 1).linkedTriangles.Add(LLast(Triangles))
                        verticesComplete(j).linkedTriangles.Add(LLast(Triangles))
                        verticesComplete(k).linkedTriangles.Add(LLast(Triangles))
                        SelectedTriangles.Add(LLast(Triangles))
                    End If
                Next


                For Each tri As Triangle In Triangles
                    If tri.vertexA.Equals(temp_vertex) Then
                        If verticesOuter.Contains(tri.vertexB) AndAlso verticesOuter.Contains(tri.vertexC) Then
                            SelectedTriangles.Remove(tri)
                            Triangles.Remove(tri)
                            Exit For
                        End If
                    End If
                    If tri.vertexB.Equals(temp_vertex) Then
                        If verticesOuter.Contains(tri.vertexA) AndAlso verticesOuter.Contains(tri.vertexC) Then
                            SelectedTriangles.Remove(tri)
                            Triangles.Remove(tri)
                            Exit For
                        End If
                    End If
                    If tri.vertexC.Equals(temp_vertex) Then
                        If verticesOuter.Contains(tri.vertexB) AndAlso verticesOuter.Contains(tri.vertexA) Then
                            SelectedTriangles.Remove(tri)
                            Triangles.Remove(tri)
                            Exit For
                        End If
                    End If
                Next

                Return True

            End If
        End If

        Return False
    End Function

    Private Function CSG2TrianglesIntersectionsOnly(ByRef tri1 As Triangle, ByRef tri2 As Triangle) As Boolean

        Dim verticesIntersections As New List(Of Vertex)

        schnittpunkt = intersectionBetweenTwoLines(tri1.vertexA, tri1.vertexB, tri2.vertexA, tri2.vertexB)
        If Not schnittpunkt Is Nothing Then verticesIntersections.Add(New Vertex(schnittpunkt.X, schnittpunkt.Y, False))
        schnittpunkt = intersectionBetweenTwoLines(tri1.vertexA, tri1.vertexB, tri2.vertexB, tri2.vertexC)
        If Not schnittpunkt Is Nothing Then verticesIntersections.Add(New Vertex(schnittpunkt.X, schnittpunkt.Y, False))
        schnittpunkt = intersectionBetweenTwoLines(tri1.vertexA, tri1.vertexB, tri2.vertexC, tri2.vertexA)
        If Not schnittpunkt Is Nothing Then verticesIntersections.Add(New Vertex(schnittpunkt.X, schnittpunkt.Y, False))

        schnittpunkt = intersectionBetweenTwoLines(tri1.vertexB, tri1.vertexC, tri2.vertexA, tri2.vertexB)
        If Not schnittpunkt Is Nothing Then verticesIntersections.Add(New Vertex(schnittpunkt.X, schnittpunkt.Y, False))
        schnittpunkt = intersectionBetweenTwoLines(tri1.vertexB, tri1.vertexC, tri2.vertexB, tri2.vertexC)
        If Not schnittpunkt Is Nothing Then verticesIntersections.Add(New Vertex(schnittpunkt.X, schnittpunkt.Y, False))
        schnittpunkt = intersectionBetweenTwoLines(tri1.vertexB, tri1.vertexC, tri2.vertexC, tri2.vertexA)
        If Not schnittpunkt Is Nothing Then verticesIntersections.Add(New Vertex(schnittpunkt.X, schnittpunkt.Y, False))

        schnittpunkt = intersectionBetweenTwoLines(tri1.vertexC, tri1.vertexA, tri2.vertexA, tri2.vertexB)
        If Not schnittpunkt Is Nothing Then verticesIntersections.Add(New Vertex(schnittpunkt.X, schnittpunkt.Y, False))
        schnittpunkt = intersectionBetweenTwoLines(tri1.vertexC, tri1.vertexA, tri2.vertexB, tri2.vertexC)
        If Not schnittpunkt Is Nothing Then verticesIntersections.Add(New Vertex(schnittpunkt.X, schnittpunkt.Y, False))
        schnittpunkt = intersectionBetweenTwoLines(tri1.vertexC, tri1.vertexA, tri2.vertexC, tri2.vertexA)
        If Not schnittpunkt Is Nothing Then verticesIntersections.Add(New Vertex(schnittpunkt.X, schnittpunkt.Y, False))

        Vertices.AddRange(verticesIntersections)

    End Function

    Private Function intersectionBetweenTwoLines(ByRef v11 As Vertex, ByRef v12 As Vertex, ByRef v21 As Vertex, ByRef v22 As Vertex) As Vertex
        If v11.X = v21.X AndAlso v11.Y = v21.Y AndAlso v12.X = v22.X AndAlso v12.Y = v22.Y Then Return Nothing
        If v11.X = v22.X AndAlso v11.Y = v22.Y AndAlso v12.X = v21.X AndAlso v12.Y = v21.Y Then Return Nothing

        Dim za1, za2, zb1, zb2, zc1, zc2 As Single
        Dim sx, sy As Single

        za1 = v12.Y - v11.Y
        za2 = v22.Y - v21.Y
        zb1 = v11.X - v12.X
        zb2 = v21.X - v22.X
        zc1 = za1 * v11.X + zb1 * v11.Y
        zc2 = za2 * v21.X + zb2 * v21.Y

        sx = (zc1 * zb2 - zc2 * zb1) / (za1 * zb2 - za2 * zb1)
        sy = (za1 * zc2 - za2 * zc1) / (za1 * zb2 - za2 * zb1)

        If Single.IsNaN(sx) OrElse Single.IsNaN(sy) Then Return Nothing

        If sy > v11.Y AndAlso sy > v12.Y Then Return Nothing
        If sy > v21.Y AndAlso sy > v22.Y Then Return Nothing
        If sy < v11.Y AndAlso sy < v12.Y Then Return Nothing
        If sy < v21.Y AndAlso sy < v22.Y Then Return Nothing

        If sx > v11.X AndAlso sx > v12.X Then Return Nothing
        If sx > v21.X AndAlso sx > v22.X Then Return Nothing
        If sx < v11.X AndAlso sx < v12.X Then Return Nothing
        If sx < v21.X AndAlso sx < v22.X Then Return Nothing

        Return New Vertex(sx, sy, False)
    End Function

#End Region
#Region "View Properties"
    Private Sub ShowGridToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ShowGridToolStripMenuItem.Click
        raster = ShowGridToolStripMenuItem.Checked
        Me.Refresh()
    End Sub

    Private Sub ImageToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ImageToolStripMenuItem.Click
        GBImage.Visible = ImageToolStripMenuItem.Checked
        Me.Refresh()
    End Sub

    Private Sub SnappingToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SnappingToolStripMenuItem.Click
        GBSnap.Visible = SnappingToolStripMenuItem.Checked
        Me.Refresh()
    End Sub

    Dim conversionEnabled As Boolean = False
    Private Sub NUDMoveSnap_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles NUDMoveSnap.ValueChanged
        If conversionEnabled Then
            Me.moveSnap = NUDMoveSnap.Value / factor
        Else
            Me.moveSnap = Fix(NUDMoveSnap.Value * 10 ^ NUDMoveSnap.DecimalPlaces) / 10 ^ NUDMoveSnap.DecimalPlaces / factor
        End If
        Me.Refresh()
    End Sub

    Private Sub NUDRotateSnap_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles NUDRotateSnap.ValueChanged
        Me.rotateSnap = NUDRotateSnap.Value
    End Sub

    Private Sub NUDScaleSnap_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles NUDScaleSnap.ValueChanged
        Me.scaleSnap = NUDScaleSnap.Value
    End Sub

    Private Sub NUDGrid_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles NUDGrid.ValueChanged
        If conversionEnabled Then
            Me.rasterSnap = NUDGrid.Value / factor
        Else
            Me.rasterSnap = Fix(NUDGrid.Value * 10 ^ NUDGrid.DecimalPlaces) / 10 ^ NUDGrid.DecimalPlaces / factor
        End If
        Me.Refresh()
    End Sub

    Private Sub SBZoom_Scroll(ByVal sender As System.Object, ByVal e As System.Windows.Forms.ScrollEventArgs) Handles SBZoom.Scroll
        zoomlevel += e.NewValue - e.OldValue
        clip(zoomlevel, -30, 90)
        If zoomlevel > -10 Then
            zoomfaktor = 1 + zoomlevel * 0.1F
        ElseIf zoomlevel > -20 Then
            zoomfaktor = 0.2F + zoomlevel * 0.01F
        Else
            zoomfaktor = 0.03F + zoomlevel * 0.001F
        End If
        If zoomlevel = -30 Then zoomfaktor = 0.001
        LblDebug.Text = "Zoom: " & CType(zoomfaktor * 10000, Integer) / 100 & "%"
        e.NewValue = 0
        Me.Refresh()
    End Sub
#End Region
#Region "Image Properties"
    Private Sub NUDoffsetX_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles NUDoffsetX.ValueChanged
        imgOffsetX = NUDoffsetX.Value
        Me.Refresh()
    End Sub

    Private Sub NUDoffsetY_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles NUDoffsetY.ValueChanged
        imgOffsetY = NUDoffsetY.Value
        Me.Refresh()
    End Sub

    Private Sub NUDScale_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles NUDScale.ValueChanged
        imgScale = NUDScale.Value
        LblSize.Text = imgScale * hintergrundbild.Width / 1000 * factor & " x " & vbCrLf & imgScale * hintergrundbild.Height / 1000 * factor & unit
        Me.Refresh()
    End Sub

    Private Sub BtnImagePath_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BtnImagePath.Click
        If OpenImage.ShowDialog() = Windows.Forms.DialogResult.OK Then
            If OpenImage.FileName <> "" Then
                Try
                    hintergrundbild.Dispose()
                    hintergrundbild = New Bitmap(OpenImage.FileName)
                    TBImage.Text = OpenImage.FileName
                    LblSize.Text = imgScale * hintergrundbild.Width / 1000 * factor & " x " & vbCrLf & imgScale * hintergrundbild.Height / 1000 * factor & unit
                Catch ex As Exception
                    MsgBox("Invalid image format!", MsgBoxStyle.Exclamation, "ERROR")
                    hintergrundbild = My.Resources.temp
                    TBImage.Text = ""
                End Try
            End If
        End If
    End Sub
#End Region
#Region "AddFunction"
    Private Sub BtnAddVertex_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BtnAddVertex.Click
        BtnAddTriangle.Checked = False
        BtnMove.Enabled = Not BtnAddVertex.Checked
        BtnRotate.Enabled = Not BtnAddVertex.Checked
        BtnScale.Enabled = Not BtnAddVertex.Checked
        BtnSelect.PerformClick()
        Me.Refresh()
    End Sub

    Dim restoreTriangleMode As Boolean
    Private Sub BtnAddTriangle_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BtnAddTriangle.Click
        BtnAddVertex.Checked = False
        BtnMove.Enabled = Not BtnAddTriangle.Checked
        BtnRotate.Enabled = Not BtnAddTriangle.Checked
        BtnScale.Enabled = Not BtnAddTriangle.Checked
        BtnSelect.PerformClick()
        VerticesModeToolStripMenuItem.PerformClick()
        If Not BtnAddTriangle.Checked AndAlso restoreTriangleMode Then TriangleModeToolStripMenuItem.PerformClick()
        Me.Refresh()
    End Sub
#End Region
#Region "Modi"
    Private Sub BtnSelect_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BtnSelect.Click
        BtnSelect.Checked = True
        BtnMove.Checked = False
        BtnRotate.Checked = False
        BtnScale.Checked = False
        bearbeitungsmodus = "Auswahl"
    End Sub

    Private Sub BtnMove_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BtnMove.Click
        BtnSelect.Checked = False
        BtnMove.Checked = True
        BtnRotate.Checked = False
        BtnScale.Checked = False
        bearbeitungsmodus = "Translation"
    End Sub

    Private Sub BtnRotate_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BtnRotate.Click
        BtnSelect.Checked = False
        BtnMove.Checked = False
        BtnRotate.Checked = True
        BtnScale.Checked = False
        bearbeitungsmodus = "Rotation"
    End Sub

    Private Sub BtnScale_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BtnScale.Click
        BtnSelect.Checked = False
        BtnMove.Checked = False
        BtnRotate.Checked = False
        BtnScale.Checked = True
        bearbeitungsmodus = "Skalierung"
    End Sub

    Private Sub VerticesModeToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles VerticesModeToolStripMenuItem.Click
        If BtnMode.Text = "Primitive Mode" Then
            For Each vert As Vertex In SelectedVertices
                vert.selected = False
            Next
            SelectedVertices.Clear()
        End If
        BtnMode.Text = VerticesModeToolStripMenuItem.Text
        bearbeitungsobjekt = "Vertex"
        BtnCSG.Enabled = False
        BtnColours.Enabled = False
        ToolStrip2.Visible = False
        BtnMerge.Enabled = True
        For Each tri As Triangle In SelectedTriangles
            tri.selected = False
        Next
        SelectedTriangles.Clear()
        For Each vert As Vertex In SelectedVertices
            vert.selected = True
        Next
        Me.Refresh()
        SelectAllToolStripMenuItem.Enabled = True
        SelectConnectedToolStripMenuItem.Enabled = True
        SelectTouchingToolStripMenuItem.Enabled = True
        SelectSameColourToolStripMenuItem.Enabled = True
        WithColourToolStripMenuItem.Enabled = True
    End Sub

    Private Sub TriangleModeToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles TriangleModeToolStripMenuItem.Click
        If BtnMode.Text = "Primitive Mode" Then
            For Each tri As Triangle In SelectedTriangles
                tri.selected = False
            Next
            SelectedTriangles.Clear()
        End If
        restoreTriangleMode = True
        BtnMode.Text = TriangleModeToolStripMenuItem.Text
        bearbeitungsobjekt = "Dreieck"
        BtnCSG.Enabled = True
        BtnColours.Enabled = True
        ToolStrip2.Visible = BtnColours.Checked
        BtnMerge.Enabled = False
        For Each vert As Vertex In SelectedVertices
            vert.selected = False
        Next
        SelectedVertices.Clear()
        Me.Refresh()
        SelectAllToolStripMenuItem.Enabled = True
        SelectConnectedToolStripMenuItem.Enabled = True
        SelectTouchingToolStripMenuItem.Enabled = True
        SelectSameColourToolStripMenuItem.Enabled = True
        WithColourToolStripMenuItem.Enabled = True
    End Sub

    Private Sub PrimitiveModeToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles PrimitiveModeToolStripMenuItem.Click
        SelectAllToolStripMenuItem.Enabled = False
        SelectConnectedToolStripMenuItem.Enabled = False
        SelectTouchingToolStripMenuItem.Enabled = False
        SelectSameColourToolStripMenuItem.Enabled = False
        WithColourToolStripMenuItem.Enabled = False
        BtnMode.Text = PrimitiveModeToolStripMenuItem.Text
        bearbeitungsobjekt = "Primitive"
        BtnCSG.Enabled = False
        BtnColours.Enabled = True
        ToolStrip2.Visible = BtnColours.Checked
        BtnMerge.Enabled = False
        For Each vert As Vertex In SelectedVertices
            vert.selected = False
        Next
        SelectedVertices.Clear()
        Me.Refresh()
    End Sub
#End Region
#Region "Copy & Paste"
    Private Sub BtnCut_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BtnCut.Click
        copy()
        delete()
    End Sub

    Private Sub BtnCopy_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BtnCopy.Click
        copy()
    End Sub

    Private Sub BtnPaste_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BtnPaste.Click
        paste()
    End Sub

    Private Sub ToolStripMenuItem5_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripMenuItem5.Click
        copy()
        delete()
    End Sub

    Private Sub ToolStripMenuItem4_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripMenuItem4.Click
        copy()
    End Sub

    Private Sub ToolStripMenuItem3_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripMenuItem3.Click
        paste()
    End Sub

    Private Sub ToolStripMenuItem2_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripMenuItem2.Click
        delete()
    End Sub

    Private Sub delete()
        If BtnMode.Text <> "Triangle Mode" Then
            If bearbeitungsobjekt = "Primitive" Then
                If SelectedVertices.Count > 0 Then
                    Dim myindex As Integer = CType(PrimitivesHMap(SelectedVertices(0).groupindex), Integer)
                    PrimitivesHMap.Remove(SelectedVertices(0).groupindex)
                    If PrimitivesHMap.Keys.Count > 0 Then
                        Dim klist(PrimitivesHMap.Keys.Count - 1) As Integer
                        Dim counter As Integer = 0
                        For Each key As Integer In PrimitivesHMap.Keys
                            klist(counter) = key
                            counter += 1
                        Next
                        For i As Integer = 0 To PrimitivesHMap.Keys.Count - 1
                            If PrimitivesHMap.Item(klist(i)) > myindex Then
                                PrimitivesHMap.Item(klist(i)) -= 1
                            End If
                        Next i
                    End If
                    Primitives.RemoveAt(myindex)
                End If
            End If
            For Each vert As Vertex In SelectedVertices
removeVertex:
                For Each ref As Vertex In Vertices
                    If ref.Equals(vert) Then
nextTriangle:
                        For Each tri As Triangle In vert.linkedTriangles
                            If Triangles.Contains(tri) Then
                                removeTriangle(tri)
                            Else
                                vert.linkedTriangles.Remove(tri)
                            End If
                            GoTo nextTriangle
                        Next
                        Vertices.Remove(ref)
                        GoTo removeVertex
                    End If
                Next
            Next
        Else
            For Each tri As Triangle In SelectedTriangles
                removeTriangle(tri)
            Next
        End If
        SelectedVertices.Clear()
        SelectedTriangles.Clear()
        addHistory()
        Me.Refresh()
    End Sub

    Private Sub copy()
        If SelectedVertices.Count > 0 Then
            ClipboardVertices.Clear()
            ClipboardTriangles.Clear()
            ClipboardPrimitives.Clear()

            For Each vert As Vertex In SelectedVertices
                ClipboardVertices.Add(New Vertex(vert.X, vert.Y, True, False) With {.vertexID = vert.vertexID, .groupindex = vert.groupindex})
            Next

            If BtnMode.Text = "Triangle Mode" Then
                For Each tri As Triangle In SelectedTriangles
                    If SelectedVertices.Contains(tri.vertexA) AndAlso SelectedVertices.Contains(tri.vertexB) AndAlso SelectedVertices.Contains(tri.vertexC) Then
                        ClipboardTriangles.Add(New Triangle(New Vertex(tri.vertexA.X, tri.vertexA.Y, True, False) With {.vertexID = tri.vertexA.vertexID, .groupindex = tri.vertexA.groupindex}, New Vertex(tri.vertexB.X, tri.vertexB.Y, True, False) With {.vertexID = tri.vertexB.vertexID, .groupindex = tri.vertexB.groupindex}, New Vertex(tri.vertexC.X, tri.vertexC.Y, True, False) With {.vertexID = tri.vertexC.vertexID, .groupindex = tri.vertexC.groupindex}, False) With {.selected = True, .myColourNumber = tri.myColourNumber, .myColour = tri.myColour, .groupindex = tri.groupindex, .triangleID = tri.triangleID})
                    End If
                Next
            Else
                For Each tri As Triangle In Triangles
                    If SelectedVertices.Contains(tri.vertexA) AndAlso SelectedVertices.Contains(tri.vertexB) AndAlso SelectedVertices.Contains(tri.vertexC) Then
                        ClipboardTriangles.Add(New Triangle(New Vertex(tri.vertexA.X, tri.vertexA.Y, True, False) With {.vertexID = tri.vertexA.vertexID, .groupindex = tri.vertexA.groupindex}, New Vertex(tri.vertexB.X, tri.vertexB.Y, True, False) With {.vertexID = tri.vertexB.vertexID, .groupindex = tri.vertexB.groupindex}, New Vertex(tri.vertexC.X, tri.vertexC.Y, True, False) With {.vertexID = tri.vertexC.vertexID, .groupindex = tri.vertexC.groupindex}, False) With {.selected = True, .myColourNumber = tri.myColourNumber, .myColour = tri.myColour, .groupindex = tri.groupindex, .triangleID = tri.triangleID})
                    End If
                Next
            End If

            If SelectedVertices(0).groupindex > -1 Then
                Dim prim As Primitive = Primitives(PrimitivesHMap(SelectedVertices(0).groupindex))
                ClipboardPrimitives.Add(New Primitive(0, 0, prim.ox, prim.oy, prim.primitiveName, False) With {.matrix = prim.matrix})
            End If

            Me.Refresh()
        End If
    End Sub

    Dim vA, vB, vC As Vertex
    Private Sub paste()
        If ClipboardVertices.Count > 0 Then
            For Each vert As Vertex In SelectedVertices
                vert.selected = False
            Next

            For Each tri As Triangle In SelectedTriangles
                tri.selected = False
            Next


            SelectedVertices.Clear()
            SelectedTriangles.Clear()

            Dim VIDtoVI As New Hashtable(ClipboardVertices.Count)

            Dim startVertices As Integer = Vertices.Count
            Dim startTriangles As Integer = Triangles.Count


            If ClipboardVertices(0).groupindex > -1 Then
                ' Neues Primitive einfügen:
                Primitives.Add(New Primitive(0, 0, ClipboardPrimitives(0).ox, ClipboardPrimitives(0).oy, ClipboardPrimitives(0).primitiveName, ClipboardPrimitives(0).centerVertexID) With {.matrix = ClipboardPrimitives(0).matrix})
                PrimitivesHMap.Add(primitiveIDglobal, Primitives.Count - 1)
                For Each vert As Vertex In ClipboardVertices
                    Vertices.Add(New Vertex(vert.X, vert.Y, True) With {.angleFrom = vert.vertexID, .groupindex = primitiveIDglobal})
                    SelectedVertices.Add(LLast(Vertices))
                Next

                For Each tri As Triangle In ClipboardTriangles
                    Triangles.Add(New Triangle(tri.vertexA, tri.vertexB, tri.vertexC) With {.selected = True, .myColourNumber = tri.myColourNumber, .myColour = tri.myColour, .groupindex = primitiveIDglobal})
                    SelectedTriangles.Add(LLast(Triangles))
                Next
            Else
                ' Nur Vertices und Triangles einfügen
                For Each vert As Vertex In ClipboardVertices
                    Vertices.Add(New Vertex(vert.X, vert.Y, True) With {.angleFrom = vert.vertexID})
                    SelectedVertices.Add(LLast(Vertices))
                Next

                For Each tri As Triangle In ClipboardTriangles
                    Triangles.Add(New Triangle(tri.vertexA, tri.vertexB, tri.vertexC) With {.selected = True, .myColourNumber = tri.myColourNumber, .myColour = tri.myColour})
                    SelectedTriangles.Add(LLast(Triangles))
                Next
            End If

            ' Hashmap erstellen:
            For i As Integer = startVertices To Vertices.Count - 1
                VIDtoVI.Add(CType(Vertices(i).angleFrom, Integer), i)
                Vertices(i).angleFrom = 0
            Next

            ' Linken:
            For i As Integer = startTriangles To Triangles.Count - 1
                Vertices(VIDtoVI(Triangles(i).vertexA.vertexID)).linkedTriangles.Add(Triangles(i))
                Vertices(VIDtoVI(Triangles(i).vertexB.vertexID)).linkedTriangles.Add(Triangles(i))
                Vertices(VIDtoVI(Triangles(i).vertexC.vertexID)).linkedTriangles.Add(Triangles(i))
                Triangles(i).vertexA = Vertices(VIDtoVI(Triangles(i).vertexA.vertexID))
                Triangles(i).vertexB = Vertices(VIDtoVI(Triangles(i).vertexB.vertexID))
                Triangles(i).vertexC = Vertices(VIDtoVI(Triangles(i).vertexC.vertexID))
            Next

            addHistory()
            Me.Refresh()
        End If
    End Sub
#End Region


#Region "Select"
    Private Sub SelectAllToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SelectAllToolStripMenuItem.Click
        selectAll()
        Me.Refresh()
    End Sub

    Private Sub SelectSameColourToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SelectSameColourToolStripMenuItem.Click
        selectSameColour()
        Me.Refresh()
    End Sub

    Private Sub SelectConnectedToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SelectConnectedToolStripMenuItem.Click
        selectConnected()
        Me.Refresh()
    End Sub

    Private Sub SelectTouchingToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SelectTouchingToolStripMenuItem.Click
        selectTouching()
        Me.Refresh()
    End Sub

    Private Sub WithColourToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles WithColourToolStripMenuItem.Click
        WithColourToolStripMenuItem.GetCurrentParent.Show()
        Me.Refresh()
    End Sub

    Private Sub selectAll()
        SelectedTriangles.Clear()
        SelectedVertices.Clear()
        For Each tri As Triangle In Triangles
            If tri.groupindex = -1 Then
                SelectedTriangles.Add(tri)
                tri.selected = True
                If Not SelectedVertices.Contains(tri.vertexA) AndAlso tri.vertexA.groupindex = -1 Then SelectedVertices.Add(tri.vertexA) : LLast(SelectedVertices).selected = True
                If Not SelectedVertices.Contains(tri.vertexB) AndAlso tri.vertexB.groupindex = -1 Then SelectedVertices.Add(tri.vertexB) : LLast(SelectedVertices).selected = True
                If Not SelectedVertices.Contains(tri.vertexC) AndAlso tri.vertexC.groupindex = -1 Then SelectedVertices.Add(tri.vertexC) : LLast(SelectedVertices).selected = True
            End If
        Next
        For Each vert As Vertex In Vertices
            If Not SelectedVertices.Contains(vert) AndAlso vert.groupindex = -1 Then
                vert.selected = True
                SelectedVertices.Add(vert)
            End If
        Next
    End Sub

    Private Sub selectSameColour()
        If SelectedTriangles.Count > 0 Then
            Dim tc As Short = SelectedTriangles(0).myColourNumber
            SelectedTriangles.Clear()
            SelectedVertices.Clear()
            For Each tri As Triangle In Triangles
                If tri.myColourNumber = tc Then
                    SelectedTriangles.Add(tri)
                    tri.selected = True
                    If Not SelectedVertices.Contains(tri.vertexA) Then SelectedVertices.Add(tri.vertexA) : LLast(SelectedVertices).selected = True
                    If Not SelectedVertices.Contains(tri.vertexB) Then SelectedVertices.Add(tri.vertexB) : LLast(SelectedVertices).selected = True
                    If Not SelectedVertices.Contains(tri.vertexC) Then SelectedVertices.Add(tri.vertexC) : LLast(SelectedVertices).selected = True
                End If
            Next
        End If
    End Sub

    Private Sub selectConnected()
        Dim startcount As Integer = 0
        Dim tc As Short
        If WithColourToolStripMenuItem.Checked AndAlso SelectedTriangles.Count > 0 Then tc = SelectedTriangles(0).myColourNumber
nextStep:
        If bearbeitungsobjekt = "Vertex" Then
            Dim tmpcount As Integer = SelectedVertices.Count - 1
            For verti = startcount To tmpcount
                Dim vert As Vertex = SelectedVertices(verti)
                For Each tri As Triangle In vert.linkedTriangles
                    If Not SelectedVertices.Contains(tri.vertexA) AndAlso tri.vertexA.groupindex < 0 Then tri.vertexA.selected = True : SelectedVertices.Add(tri.vertexA)
                    If Not SelectedVertices.Contains(tri.vertexB) AndAlso tri.vertexB.groupindex < 0 Then tri.vertexB.selected = True : SelectedVertices.Add(tri.vertexB)
                    If Not SelectedVertices.Contains(tri.vertexC) AndAlso tri.vertexC.groupindex < 0 Then tri.vertexC.selected = True : SelectedVertices.Add(tri.vertexC)
                Next
            Next
            startcount = tmpcount + 1
            If SelectedVertices.Count > startcount Then GoTo nextstep
        ElseIf bearbeitungsobjekt = "Dreieck" Then
            Dim tmpcount As Integer = SelectedTriangles.Count - 1
            If WithColourToolStripMenuItem.Checked Then
                For trii = startcount To tmpcount
                    Dim tri As Triangle = SelectedTriangles(trii)
                    If Not tri.vertexA Is Nothing Then
                        For Each tri2 As Triangle In tri.vertexA.linkedTriangles
                            If tri2.myColourNumber = tc AndAlso Not SelectedTriangles.Contains(tri2) AndAlso tri2.groupindex < 0 Then
                                If Not SelectedVertices.Contains(tri2.vertexA) AndAlso tri2.vertexA.groupindex < 0 Then tri2.vertexA.selected = True : SelectedVertices.Add(tri2.vertexA)
                                If Not SelectedVertices.Contains(tri2.vertexB) AndAlso tri2.vertexB.groupindex < 0 Then tri2.vertexB.selected = True : SelectedVertices.Add(tri2.vertexB)
                                If Not SelectedVertices.Contains(tri2.vertexC) AndAlso tri2.vertexC.groupindex < 0 Then tri2.vertexC.selected = True : SelectedVertices.Add(tri2.vertexC)
                                tri2.selected = True : SelectedTriangles.Add(tri2)
                            End If
                        Next
                    End If
                    If Not tri.vertexB Is Nothing Then
                        For Each tri2 As Triangle In tri.vertexB.linkedTriangles
                            If tri2.myColourNumber = tc AndAlso Not SelectedTriangles.Contains(tri2) AndAlso tri2.groupindex < 0 Then
                                If Not SelectedVertices.Contains(tri2.vertexA) AndAlso tri2.vertexA.groupindex < 0 Then tri2.vertexA.selected = True : SelectedVertices.Add(tri2.vertexA)
                                If Not SelectedVertices.Contains(tri2.vertexB) AndAlso tri2.vertexB.groupindex < 0 Then tri2.vertexB.selected = True : SelectedVertices.Add(tri2.vertexB)
                                If Not SelectedVertices.Contains(tri2.vertexC) AndAlso tri2.vertexC.groupindex < 0 Then tri2.vertexC.selected = True : SelectedVertices.Add(tri2.vertexC)
                                tri2.selected = True : SelectedTriangles.Add(tri2)
                            End If
                        Next
                    End If
                    If Not tri.vertexC Is Nothing Then
                        For Each tri2 As Triangle In tri.vertexC.linkedTriangles
                            If tri2.myColourNumber = tc AndAlso Not SelectedTriangles.Contains(tri2) AndAlso tri2.groupindex < 0 Then
                                If Not SelectedVertices.Contains(tri2.vertexA) AndAlso tri2.vertexA.groupindex < 0 Then tri2.vertexA.selected = True : SelectedVertices.Add(tri2.vertexA)
                                If Not SelectedVertices.Contains(tri2.vertexB) AndAlso tri2.vertexB.groupindex < 0 Then tri2.vertexB.selected = True : SelectedVertices.Add(tri2.vertexB)
                                If Not SelectedVertices.Contains(tri2.vertexC) AndAlso tri2.vertexC.groupindex < 0 Then tri2.vertexC.selected = True : SelectedVertices.Add(tri2.vertexC)
                                tri2.selected = True : SelectedTriangles.Add(tri2)
                            End If
                        Next
                    End If
                Next
            Else
                For trii = startcount To tmpcount
                    Dim tri As Triangle = SelectedTriangles(trii)
                    For Each tri2 As Triangle In tri.vertexA.linkedTriangles
                        If Not SelectedTriangles.Contains(tri2) AndAlso tri2.groupindex < 0 Then
                            If Not SelectedVertices.Contains(tri2.vertexA) AndAlso tri2.vertexA.groupindex < 0 Then tri2.vertexA.selected = True : SelectedVertices.Add(tri2.vertexA)
                            If Not SelectedVertices.Contains(tri2.vertexB) AndAlso tri2.vertexB.groupindex < 0 Then tri2.vertexB.selected = True : SelectedVertices.Add(tri2.vertexB)
                            If Not SelectedVertices.Contains(tri2.vertexC) AndAlso tri2.vertexC.groupindex < 0 Then tri2.vertexC.selected = True : SelectedVertices.Add(tri2.vertexC)
                            tri2.selected = True : SelectedTriangles.Add(tri2)
                        End If
                    Next
                    For Each tri2 As Triangle In tri.vertexB.linkedTriangles
                        If Not SelectedTriangles.Contains(tri2) AndAlso tri2.groupindex < 0 Then
                            If Not SelectedVertices.Contains(tri2.vertexA) AndAlso tri2.vertexA.groupindex < 0 Then tri2.vertexA.selected = True : SelectedVertices.Add(tri2.vertexA)
                            If Not SelectedVertices.Contains(tri2.vertexB) AndAlso tri2.vertexB.groupindex < 0 Then tri2.vertexB.selected = True : SelectedVertices.Add(tri2.vertexB)
                            If Not SelectedVertices.Contains(tri2.vertexC) AndAlso tri2.vertexC.groupindex < 0 Then tri2.vertexC.selected = True : SelectedVertices.Add(tri2.vertexC)
                            tri2.selected = True : SelectedTriangles.Add(tri2)
                        End If
                    Next
                    For Each tri2 As Triangle In tri.vertexC.linkedTriangles
                        If Not SelectedTriangles.Contains(tri2) AndAlso tri2.groupindex < 0 Then
                            If Not SelectedVertices.Contains(tri2.vertexA) AndAlso tri2.vertexA.groupindex < 0 Then tri2.vertexA.selected = True : SelectedVertices.Add(tri2.vertexA)
                            If Not SelectedVertices.Contains(tri2.vertexB) AndAlso tri2.vertexB.groupindex < 0 Then tri2.vertexB.selected = True : SelectedVertices.Add(tri2.vertexB)
                            If Not SelectedVertices.Contains(tri2.vertexC) AndAlso tri2.vertexC.groupindex < 0 Then tri2.vertexC.selected = True : SelectedVertices.Add(tri2.vertexC)
                            tri2.selected = True : SelectedTriangles.Add(tri2)
                        End If
                    Next
                Next
            End If
            startcount = tmpcount + 1
            If SelectedTriangles.Count > startcount Then GoTo nextstep
        End If
    End Sub

    Private Sub selectPrimitiveGroup(ByVal group As Integer)
        Dim startcount As Integer = 0
nextStep:
        Dim tmpcount As Integer = SelectedTriangles.Count - 1
        For trii = startcount To tmpcount
            Dim tri As Triangle = SelectedTriangles(trii)
            For Each tri2 As Triangle In tri.vertexA.linkedTriangles
                If Not SelectedTriangles.Contains(tri2) AndAlso tri2.groupindex = group Then
                    If Not SelectedVertices.Contains(tri2.vertexA) AndAlso tri2.vertexA.groupindex = group Then tri2.vertexA.selected = True : SelectedVertices.Add(tri2.vertexA)
                    If Not SelectedVertices.Contains(tri2.vertexB) AndAlso tri2.vertexB.groupindex = group Then tri2.vertexB.selected = True : SelectedVertices.Add(tri2.vertexB)
                    If Not SelectedVertices.Contains(tri2.vertexC) AndAlso tri2.vertexC.groupindex = group Then tri2.vertexC.selected = True : SelectedVertices.Add(tri2.vertexC)
                    tri2.selected = True : SelectedTriangles.Add(tri2)
                End If
            Next
            For Each tri2 As Triangle In tri.vertexB.linkedTriangles
                If Not SelectedTriangles.Contains(tri2) AndAlso tri2.groupindex = group Then
                    If Not SelectedVertices.Contains(tri2.vertexA) AndAlso tri2.vertexA.groupindex = group Then tri2.vertexA.selected = True : SelectedVertices.Add(tri2.vertexA)
                    If Not SelectedVertices.Contains(tri2.vertexB) AndAlso tri2.vertexB.groupindex = group Then tri2.vertexB.selected = True : SelectedVertices.Add(tri2.vertexB)
                    If Not SelectedVertices.Contains(tri2.vertexC) AndAlso tri2.vertexC.groupindex = group Then tri2.vertexC.selected = True : SelectedVertices.Add(tri2.vertexC)
                    tri2.selected = True : SelectedTriangles.Add(tri2)
                End If
            Next
            For Each tri2 As Triangle In tri.vertexC.linkedTriangles
                If Not SelectedTriangles.Contains(tri2) AndAlso tri2.groupindex = group Then
                    If Not SelectedVertices.Contains(tri2.vertexA) AndAlso tri2.vertexA.groupindex = group Then tri2.vertexA.selected = True : SelectedVertices.Add(tri2.vertexA)
                    If Not SelectedVertices.Contains(tri2.vertexB) AndAlso tri2.vertexB.groupindex = group Then tri2.vertexB.selected = True : SelectedVertices.Add(tri2.vertexB)
                    If Not SelectedVertices.Contains(tri2.vertexC) AndAlso tri2.vertexC.groupindex = group Then tri2.vertexC.selected = True : SelectedVertices.Add(tri2.vertexC)
                    tri2.selected = True : SelectedTriangles.Add(tri2)
                End If
            Next
        Next
        startcount = tmpcount + 1
        If SelectedTriangles.Count > startcount Then GoTo nextstep
    End Sub

    Private Sub selectTouching()
        If bearbeitungsobjekt = "Vertex" Then
            Dim tmpcount As Integer = SelectedVertices.Count - 1
            For verti = 0 To tmpcount
                Dim vert As Vertex = SelectedVertices(verti)
                For Each tri As Triangle In vert.linkedTriangles
                    If Not SelectedVertices.Contains(tri.vertexA) AndAlso tri.vertexA.groupindex < 0 Then tri.vertexA.selected = True : SelectedVertices.Add(tri.vertexA)
                    If Not SelectedVertices.Contains(tri.vertexB) AndAlso tri.vertexB.groupindex < 0 Then tri.vertexB.selected = True : SelectedVertices.Add(tri.vertexB)
                    If Not SelectedVertices.Contains(tri.vertexC) AndAlso tri.vertexC.groupindex < 0 Then tri.vertexC.selected = True : SelectedVertices.Add(tri.vertexC)
                Next
            Next
        ElseIf bearbeitungsobjekt = "Dreieck" Then
            Dim tc As Short
            If WithColourToolStripMenuItem.Checked AndAlso SelectedTriangles.Count > 0 Then tc = SelectedTriangles(0).myColourNumber
            Dim tmpcount As Integer = SelectedTriangles.Count - 1
            If WithColourToolStripMenuItem.Checked Then
                For trii = 0 To tmpcount
                    Dim tri As Triangle = SelectedTriangles(trii)
                    For Each tri2 As Triangle In tri.vertexA.linkedTriangles
                        If tri2.myColourNumber = tc AndAlso Not SelectedTriangles.Contains(tri2) AndAlso tri2.groupindex < 0 Then
                            If Not SelectedVertices.Contains(tri2.vertexA) AndAlso tri2.vertexA.groupindex < 0 Then tri2.vertexA.selected = True : SelectedVertices.Add(tri2.vertexA)
                            If Not SelectedVertices.Contains(tri2.vertexB) AndAlso tri2.vertexB.groupindex < 0 Then tri2.vertexB.selected = True : SelectedVertices.Add(tri2.vertexB)
                            If Not SelectedVertices.Contains(tri2.vertexC) AndAlso tri2.vertexC.groupindex < 0 Then tri2.vertexC.selected = True : SelectedVertices.Add(tri2.vertexC)
                            tri2.selected = True : SelectedTriangles.Add(tri2)
                        End If
                    Next
                    For Each tri2 As Triangle In tri.vertexB.linkedTriangles
                        If tri2.myColourNumber = tc AndAlso Not SelectedTriangles.Contains(tri2) AndAlso tri2.groupindex < 0 Then
                            If Not SelectedVertices.Contains(tri2.vertexA) AndAlso tri2.vertexA.groupindex < 0 Then tri2.vertexA.selected = True : SelectedVertices.Add(tri2.vertexA)
                            If Not SelectedVertices.Contains(tri2.vertexB) AndAlso tri2.vertexB.groupindex < 0 Then tri2.vertexB.selected = True : SelectedVertices.Add(tri2.vertexB)
                            If Not SelectedVertices.Contains(tri2.vertexC) AndAlso tri2.vertexC.groupindex < 0 Then tri2.vertexC.selected = True : SelectedVertices.Add(tri2.vertexC)
                            tri2.selected = True : SelectedTriangles.Add(tri2)
                        End If
                    Next
                    For Each tri2 As Triangle In tri.vertexC.linkedTriangles
                        If tri2.myColourNumber = tc AndAlso Not SelectedTriangles.Contains(tri2) AndAlso tri2.groupindex < 0 Then
                            If Not SelectedVertices.Contains(tri2.vertexA) AndAlso tri2.vertexA.groupindex < 0 Then tri2.vertexA.selected = True : SelectedVertices.Add(tri2.vertexA)
                            If Not SelectedVertices.Contains(tri2.vertexB) AndAlso tri2.vertexB.groupindex < 0 Then tri2.vertexB.selected = True : SelectedVertices.Add(tri2.vertexB)
                            If Not SelectedVertices.Contains(tri2.vertexC) AndAlso tri2.vertexC.groupindex < 0 Then tri2.vertexC.selected = True : SelectedVertices.Add(tri2.vertexC)
                            tri2.selected = True : SelectedTriangles.Add(tri2)
                        End If
                    Next
                Next
            Else
                For trii = 0 To tmpcount
                    Dim tri As Triangle = SelectedTriangles(trii)
                    For Each tri2 As Triangle In tri.vertexA.linkedTriangles
                        If Not SelectedTriangles.Contains(tri2) AndAlso tri2.groupindex < 0 Then
                            If Not SelectedVertices.Contains(tri2.vertexA) AndAlso tri2.vertexA.groupindex < 0 Then tri2.vertexA.selected = True : SelectedVertices.Add(tri2.vertexA)
                            If Not SelectedVertices.Contains(tri2.vertexB) AndAlso tri2.vertexB.groupindex < 0 Then tri2.vertexB.selected = True : SelectedVertices.Add(tri2.vertexB)
                            If Not SelectedVertices.Contains(tri2.vertexC) AndAlso tri2.vertexC.groupindex < 0 Then tri2.vertexC.selected = True : SelectedVertices.Add(tri2.vertexC)
                            tri2.selected = True : SelectedTriangles.Add(tri2)
                        End If
                    Next
                    For Each tri2 As Triangle In tri.vertexB.linkedTriangles
                        If Not SelectedTriangles.Contains(tri2) AndAlso tri2.groupindex < 0 Then
                            If Not SelectedVertices.Contains(tri2.vertexA) AndAlso tri2.vertexA.groupindex < 0 Then tri2.vertexA.selected = True : SelectedVertices.Add(tri2.vertexA)
                            If Not SelectedVertices.Contains(tri2.vertexB) AndAlso tri2.vertexB.groupindex < 0 Then tri2.vertexB.selected = True : SelectedVertices.Add(tri2.vertexB)
                            If Not SelectedVertices.Contains(tri2.vertexC) AndAlso tri2.vertexC.groupindex < 0 Then tri2.vertexC.selected = True : SelectedVertices.Add(tri2.vertexC)
                            tri2.selected = True : SelectedTriangles.Add(tri2)
                        End If
                    Next
                    For Each tri2 As Triangle In tri.vertexC.linkedTriangles
                        If Not SelectedTriangles.Contains(tri2) AndAlso tri2.groupindex < 0 Then
                            If Not SelectedVertices.Contains(tri2.vertexA) AndAlso tri2.vertexA.groupindex < 0 Then tri2.vertexA.selected = True : SelectedVertices.Add(tri2.vertexA)
                            If Not SelectedVertices.Contains(tri2.vertexB) AndAlso tri2.vertexB.groupindex < 0 Then tri2.vertexB.selected = True : SelectedVertices.Add(tri2.vertexB)
                            If Not SelectedVertices.Contains(tri2.vertexC) AndAlso tri2.vertexC.groupindex < 0 Then tri2.vertexC.selected = True : SelectedVertices.Add(tri2.vertexC)
                            tri2.selected = True : SelectedTriangles.Add(tri2)
                        End If
                    Next
                Next
            End If
        End If
    End Sub

    Private Sub selectTouching2()
        Dim tmpcount As Integer = SelectedVertices.Count - 1
        For verti = 0 To tmpcount
            Dim vert As Vertex = SelectedVertices(verti)
            For Each tri As Triangle In vert.linkedTriangles
                If Not SelectedVertices.Contains(tri.vertexA) AndAlso Not tri.vertexA.selected Then SelectedVertices.Add(tri.vertexA)
                If Not SelectedVertices.Contains(tri.vertexB) AndAlso Not tri.vertexB.selected Then SelectedVertices.Add(tri.vertexB)
                If Not SelectedVertices.Contains(tri.vertexC) AndAlso Not tri.vertexC.selected Then SelectedVertices.Add(tri.vertexC)
            Next
        Next
    End Sub

#End Region
#Region "Mirror"

    Dim deltaXmin As Single
    Dim deltaXmax As Single
    Dim deltaYmin As Single
    Dim deltaYmax As Single
    Private Sub MirrorXToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MirrorXToolStripMenuItem.Click
        mirrorX()
        Me.Refresh()
    End Sub

    Private Sub MirrorYToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MirrorYToolStripMenuItem.Click
        mirrorY()
        Me.Refresh()
    End Sub

    Private Sub MirrorXLeftToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MirrorXLeftToolStripMenuItem.Click
        copy()
        paste()
        mirrorX()
        For Each vert As Vertex In SelectedVertices
            vert.X += deltaXmin * 2
        Next
        cleanupMergedVertices()
        Me.Refresh()
    End Sub

    Private Sub MirrorXRightToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MirrorXRightToolStripMenuItem.Click
        copy()
        paste()
        mirrorX()
        For Each vert As Vertex In SelectedVertices
            vert.X += deltaXmax * 2
        Next
        cleanupMergedVertices()
        Me.Refresh()
    End Sub

    Private Sub MirrorYTopToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MirrorYTopToolStripMenuItem.Click
        copy()
        paste()
        mirrorY()
        For Each vert As Vertex In SelectedVertices
            vert.Y += deltaYmax * 2
        Next
        cleanupMergedVertices()
        Me.Refresh()
    End Sub

    Private Sub MirrorYBottomToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MirrorYBottomToolStripMenuItem.Click
        copy()
        paste()
        mirrorY()
        For Each vert As Vertex In SelectedVertices
            vert.Y += deltaYmin * 2
        Next
        cleanupMergedVertices()
        Me.Refresh()
    End Sub

    Private Sub mirrorX()
        deltaXmin = Single.PositiveInfinity
        deltaXmax = Single.NegativeInfinity
        If SelectedVertices.Count > 0 Then
            temp_center = New Vertex(0, 0, False, False)
            For Each vert As Vertex In SelectedVertices
                temp_center += vert
            Next
            temp_center.X /= SelectedVertices.Count
            temp_center.Y /= SelectedVertices.Count
            For Each vert As Vertex In SelectedVertices
                vert.X = temp_center.X - (vert.X - temp_center.X)
                If (temp_center.X - vert.X) < deltaXmin Then deltaXmin = temp_center.X - vert.X
                If (temp_center.X - vert.X) > deltaXmax Then deltaXmax = temp_center.X - vert.X
            Next
        End If
    End Sub

    Private Sub mirrorY()
        deltaYmin = Single.PositiveInfinity
        deltaYmax = Single.NegativeInfinity
        If SelectedVertices.Count > 0 Then
            temp_center = New Vertex(0, 0, False, False)
            For Each vert As Vertex In SelectedVertices
                temp_center += vert
            Next
            temp_center.X /= SelectedVertices.Count
            temp_center.Y /= SelectedVertices.Count
            For Each vert As Vertex In SelectedVertices
                vert.Y = temp_center.Y - (vert.Y - temp_center.Y)
                If (temp_center.Y - vert.Y) < deltaYmin Then deltaYmin = temp_center.Y - vert.Y
                If (temp_center.Y - vert.Y) > deltaYmax Then deltaYmax = temp_center.Y - vert.Y
            Next
        End If
    End Sub
#End Region
#Region "Colours"
    Private Sub C0_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles C0.Click
        setColour(C0.BackColor, 0)
    End Sub

    Private Sub C1_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles C1.Click
        setColour(C1.BackColor, 1)
    End Sub

    Private Sub C2_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles C2.Click
        setColour(C2.BackColor, 2)
    End Sub

    Private Sub C3_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles C3.Click
        setColour(C3.BackColor, 3)
    End Sub

    Private Sub C4_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles C4.Click
        setColour(C4.BackColor, 4)
    End Sub

    Private Sub C5_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles C5.Click
        setColour(C5.BackColor, 5)
    End Sub

    Private Sub C6_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles C6.Click
        setColour(C6.BackColor, 6)
    End Sub

    Private Sub C7_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles C7.Click
        setColour(C7.BackColor, 7)
    End Sub

    Private Sub C8_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles C8.Click
        setColour(C8.BackColor, 8)
    End Sub

    Private Sub C9_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles C9.Click
        setColour(C9.BackColor, 9)
    End Sub

    Private Sub C10_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles C10.Click
        setColour(C10.BackColor, 10)
    End Sub

    Private Sub C11_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles C11.Click
        setColour(C11.BackColor, 11)
    End Sub

    Private Sub C12_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles C12.Click
        setColour(C12.BackColor, 12)
    End Sub

    Private Sub C13_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles C13.Click
        setColour(C13.BackColor, 13)
    End Sub

    Private Sub C14_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles C14.Click
        setColour(C14.BackColor, 14)
    End Sub

    Private Sub C15_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles C15.Click
        setColour(C15.BackColor, 15)
    End Sub

    Private Sub C16_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles C16.Click
        setColour(Nothing, 16)
    End Sub

    Private Sub setColour(ByVal c As Color, ByVal cn As Short)
        For Each tri As Triangle In SelectedTriangles
            tri.myColour = c
            tri.myColourNumber = cn
        Next
        Me.Refresh()
    End Sub
#End Region
#Region "Undo & Redo"
    Private Sub UndoToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles UndoToolStripMenuItem.Click
        undo()
    End Sub

    Private Sub RedoToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RedoToolStripMenuItem.Click
        redo()
    End Sub

    Dim zeiger As Byte = 0
    Dim max_undo As Byte = 20

    Private Sub addHistory()
        If dreiecksmodus = 0 AndAlso Not PerformanceEnabledToolStripMenuItem.Checked Then
            If zeiger < historyVertices.Count Then
                historyVertices.RemoveRange(zeiger + 1, historyVertices.Count - zeiger - 1)
                historyTriangles.RemoveRange(zeiger + 1, historyTriangles.Count - zeiger - 1)
                historySelectedVertices.RemoveRange(zeiger + 1, historySelectedVertices.Count - zeiger - 1)
                historySelectedTriangles.RemoveRange(zeiger + 1, historySelectedTriangles.Count - zeiger - 1)
                historyPrimitives.RemoveRange(zeiger + 1, historyPrimitives.Count - zeiger - 1)
                historyHashtable.RemoveRange(zeiger + 1, historyHashtable.Count - zeiger - 1)
            End If

            historyVertices.Add(DeepClone(Vertices))
            historyTriangles.Add(DeepClone(Triangles))
            historySelectedVertices.Add(DeepClone(SelectedVertices))
            historySelectedTriangles.Add(DeepClone(SelectedTriangles))
            historyPrimitives.Add(DeepClone(Primitives))
            historyHashtable.Add(DeepClone(PrimitivesHMap))

            If zeiger < max_undo Then
                zeiger += 1
            Else
                historyVertices.RemoveAt(0)
                historyTriangles.RemoveAt(0)
                historySelectedVertices.RemoveAt(0)
                historySelectedTriangles.RemoveAt(0)
                historyPrimitives.RemoveAt(0)
                historyHashtable.RemoveAt(0)
            End If

            UndoToolStripMenuItem.Enabled = True
            RedoToolStripMenuItem.Enabled = False
        End If
    End Sub

    Private Sub undo()
        If zeiger = historyVertices.Count Then zeiger -= 1
        If zeiger > 0 Then
            zeiger -= 1
            Vertices.Clear() : Vertices.AddRange(historyVertices(zeiger).ToArray)
            SelectedVertices.Clear() : SelectedVertices.AddRange(historySelectedVertices(zeiger).ToArray)
            Triangles.Clear() : Triangles.AddRange(historyTriangles(zeiger).ToArray)
            SelectedTriangles.Clear() : SelectedTriangles.AddRange(historySelectedTriangles(zeiger).ToArray)
            Primitives.Clear() : Primitives.AddRange(historyPrimitives(zeiger).ToArray)
            PrimitivesHMap = historyHashtable(zeiger).Clone
            RedoToolStripMenuItem.Enabled = True
            cleanupHistoryData()
            Me.Refresh()
        End If
        UndoToolStripMenuItem.Enabled = zeiger > 0
    End Sub

    Private Sub redo()
        If zeiger < historyVertices.Count - 1 Then
            zeiger += 1
            Vertices.Clear() : Vertices.AddRange(historyVertices(zeiger).ToArray)
            SelectedVertices.Clear() : SelectedVertices.AddRange(historySelectedVertices(zeiger).ToArray)
            Triangles.Clear() : Triangles.AddRange(historyTriangles(zeiger).ToArray)
            SelectedTriangles.Clear() : SelectedTriangles.AddRange(historySelectedTriangles(zeiger).ToArray)
            Primitives.Clear() : Primitives.AddRange(historyPrimitives(zeiger).ToArray)
            PrimitivesHMap = historyHashtable(zeiger).Clone
            UndoToolStripMenuItem.Enabled = True
            cleanupHistoryData()
            Me.Refresh()
        End If
        RedoToolStripMenuItem.Enabled = zeiger < (historyVertices.Count - 1)
    End Sub

    Private Sub cleanupHistoryData()

        Dim VIDtoVI As New Hashtable(Vertices.Count)
        Dim TIDtoTI As New Hashtable(Triangles.Count)

        For i As Integer = 0 To Vertices.Count - 1
            VIDtoVI.Add(Vertices(i).vertexID, i)
        Next

        For i As Integer = 0 To Triangles.Count - 1
            TIDtoTI.Add(Triangles(i).triangleID, i)
            Triangles(i).vertexA = Vertices(VIDtoVI(Triangles(i).vertexA.vertexID))
            Triangles(i).vertexB = Vertices(VIDtoVI(Triangles(i).vertexB.vertexID))
            Triangles(i).vertexC = Vertices(VIDtoVI(Triangles(i).vertexC.vertexID))
        Next

        For i As Integer = 0 To SelectedTriangles.Count - 1
            SelectedTriangles(i) = Triangles(TIDtoTI(SelectedTriangles(i).triangleID))
        Next

        For i As Integer = 0 To SelectedVertices.Count - 1
            SelectedVertices(i) = Vertices(VIDtoVI(SelectedVertices(i).vertexID))
        Next

        For i As Integer = 0 To Vertices.Count - 1
            For j As Integer = 0 To Vertices(i).linkedTriangles.Count - 1
                Vertices(i).linkedTriangles(j) = Triangles(TIDtoTI(Vertices(i).linkedTriangles(j).triangleID))
            Next
        Next

    End Sub
#End Region
#Region "File Menue"
    Private Sub LoadPatternToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles LoadPatternToolStripMenuItem.Click
        isLoading = True
        If LoadFile.ShowDialog() = Windows.Forms.DialogResult.OK Then
            If LoadFile.FileName <> "" Then
                newPattern()
                Try
                    Using DateiIn As StreamReader = My.Computer.FileSystem.OpenTextFileReader(LoadFile.FileName, System.Text.Encoding.GetEncoding(20127))
                        Dim header As String = DateiIn.ReadLine()
                        If header = "0 LDraw - Pattern Creator 1.1 ASCII File" Then
                            ' Dreiecke laden:
                            Dim trianz As Integer = CType(DateiIn.ReadLine(), Integer)
                            Dim myColour, vID1, vID2, vID3, triangleID, groupindex As Integer
                            Dim myColourNumber As Short
                            For tri As Integer = 1 To trianz
                                triangleID = CType(DateiIn.ReadLine(), Integer)
                                myColour = CType(DateiIn.ReadLine(), Integer)
                                myColourNumber = CType(DateiIn.ReadLine(), Short)
                                vID1 = CType(DateiIn.ReadLine(), Integer)
                                vID2 = CType(DateiIn.ReadLine(), Integer)
                                vID3 = CType(DateiIn.ReadLine(), Integer)
                                groupindex = CType(DateiIn.ReadLine(), Integer)
                                Triangles.Add(New Triangle(New Vertex(0, 0, False, False) With {.vertexID = vID1} _
                                                         , New Vertex(0, 0, False, False) With {.vertexID = vID2} _
                                                         , New Vertex(0, 0, False, False) With {.vertexID = vID3}, False) _
                                                         With {.myColour = Color.FromArgb(myColour), .myColourNumber = myColourNumber, .triangleID = triangleID, .groupindex = groupindex})
                            Next
                            ' Vertices laden:
                            Dim vertanz As Integer = CType(DateiIn.ReadLine(), Integer)
                            Dim X, Y As Single
                            Dim vertexID, triangleID2 As Integer
                            For vert As Integer = 1 To vertanz
                                vertexID = CType(DateiIn.ReadLine(), Integer)
                                X = CType(DateiIn.ReadLine(), Single)
                                Y = CType(DateiIn.ReadLine(), Single)
                                groupindex = CType(DateiIn.ReadLine(), Integer)
                                Vertices.Add(New Vertex(X, Y, False, False) With {.vertexID = vertexID, .groupindex = groupindex})
                                Dim linkedtrianz As Integer = CType(DateiIn.ReadLine(), Integer)
                                For tri As Integer = 1 To linkedtrianz
                                    triangleID2 = CType(DateiIn.ReadLine(), Integer)
                                    LLast(Vertices).linkedTriangles.Add(New Triangle(Nothing, Nothing, Nothing, False) With {.triangleID = triangleID2})
                                Next
                            Next
                            ' Primitive laden:
                            Dim primanz As Integer = CType(DateiIn.ReadLine(), Integer)
                            Dim ox, oy As Single
                            Dim primitiveName As String
                            Dim primitiveID As Integer
                            Dim centerVertexID As Integer
                            Dim matrix(3, 3) As Double
                            For prim As Integer = 1 To primanz
                                primitiveID = CType(DateiIn.ReadLine(), Integer)
                                centerVertexID = CType(DateiIn.ReadLine(), Integer)
                                PrimitivesHMap.Add(primitiveID, prim - 1)
                                ox = CType(DateiIn.ReadLine(), Single)
                                oy = CType(DateiIn.ReadLine(), Single)
                                primitiveName = DateiIn.ReadLine()
                                For z As Integer = 0 To 3
                                    For s As Integer = 0 To 3
                                        matrix(z, s) = CType(DateiIn.ReadLine(), Double)
                                    Next
                                Next
                                Primitives.Add(New Primitive(0, 0, ox, oy, primitiveName, centerVertexID, False) With {.matrix = matrix, .primitiveID = primitiveID})
                            Next

                            vertexIDglobal = CType(DateiIn.ReadLine(), Integer)
                            triangleIDglobal = CType(DateiIn.ReadLine(), Integer)
                            primitiveIDglobal = CType(DateiIn.ReadLine(), Integer)

                            TBImage.Text = DateiIn.ReadLine()
                            Try
                                If TBImage.Text <> "" Then
                                    hintergrundbild.Dispose()
                                    hintergrundbild = New Bitmap(TBImage.Text)
                                Else
                                    hintergrundbild = My.Resources.temp
                                End If
                            Catch
                                hintergrundbild = My.Resources.temp
                            End Try

                            NUDoffsetX.Value = CType(DateiIn.ReadLine(), Integer)
                            NUDoffsetY.Value = CType(DateiIn.ReadLine(), Integer)
                            NUDScale.Value = CType(DateiIn.ReadLine(), Decimal)

                            factor = CType(DateiIn.ReadLine(), Single)
                            unit = DateiIn.ReadLine()
                            Select Case unit
                                Case "LDU"
                                    LDUToolStripMenuItem.PerformClick()
                                Case "mm"
                                    MillimeterToolStripMenuItem.PerformClick()
                                Case "inch"
                                    InchToolStripMenuItem.PerformClick()
                            End Select

                            NUDMoveSnap.Value = CType(DateiIn.ReadLine(), Decimal)
                            NUDRotateSnap.Value = CType(DateiIn.ReadLine(), Decimal)
                            NUDScaleSnap.Value = CType(DateiIn.ReadLine(), Decimal)
                            NUDGrid.Value = CType(DateiIn.ReadLine(), Decimal)


                            LblSize.Text = imgScale * hintergrundbild.Width / 1000 * factor & " x " & vbCrLf & imgScale * hintergrundbild.Height / 1000 * factor & unit
                        End If
                        If header = "0 LDraw - Pattern Creator 1.0 ASCII File" Then
                            ' Dreiecke laden:
                            Dim trianz As Integer = CType(DateiIn.ReadLine(), Integer)
                            Dim myColour, vID1, vID2, vID3, triangleID As Integer
                            Dim myColourNumber As Short
                            For tri As Integer = 1 To trianz
                                triangleID = CType(DateiIn.ReadLine(), Integer)
                                myColour = CType(DateiIn.ReadLine(), Integer)
                                myColourNumber = CType(DateiIn.ReadLine(), Short)
                                vID1 = CType(DateiIn.ReadLine(), Integer)
                                vID2 = CType(DateiIn.ReadLine(), Integer)
                                vID3 = CType(DateiIn.ReadLine(), Integer)
                                Triangles.Add(New Triangle(New Vertex(0, 0, False, False) With {.vertexID = vID1} _
                                                         , New Vertex(0, 0, False, False) With {.vertexID = vID2} _
                                                         , New Vertex(0, 0, False, False) With {.vertexID = vID3}, False) _
                                                         With {.myColour = Color.FromArgb(myColour), .myColourNumber = myColourNumber, .triangleID = triangleID})
                            Next
                            ' Vertices laden:
                            Dim vertanz As Integer = CType(DateiIn.ReadLine(), Integer)
                            Dim X, Y As Single
                            Dim vertexID, triangleID2 As Integer
                            For vert As Integer = 1 To vertanz
                                vertexID = CType(DateiIn.ReadLine(), Integer)
                                X = CType(DateiIn.ReadLine(), Single)
                                Y = CType(DateiIn.ReadLine(), Single)
                                Vertices.Add(New Vertex(X, Y, False, False) With {.vertexID = vertexID})
                                Dim linkedtrianz As Integer = CType(DateiIn.ReadLine(), Integer)
                                For tri As Integer = 1 To linkedtrianz
                                    triangleID2 = CType(DateiIn.ReadLine(), Integer)
                                    LLast(Vertices).linkedTriangles.Add(New Triangle(Nothing, Nothing, Nothing, False) With {.triangleID = triangleID2})
                                Next
                            Next
                            vertexIDglobal = CType(DateiIn.ReadLine(), Integer)
                            triangleIDglobal = CType(DateiIn.ReadLine(), Integer)

                            TBImage.Text = DateiIn.ReadLine()
                            Try
                                If TBImage.Text <> "" Then
                                    hintergrundbild.Dispose()
                                    hintergrundbild = New Bitmap(TBImage.Text)
                                Else
                                    hintergrundbild = My.Resources.temp
                                End If
                            Catch
                                hintergrundbild = My.Resources.temp
                            End Try
                            NUDoffsetX.Value = CType(DateiIn.ReadLine(), Integer)
                            NUDoffsetY.Value = CType(DateiIn.ReadLine(), Integer)
                            NUDScale.Value = CType(DateiIn.ReadLine(), Decimal)
                            LblSize.Text = imgScale * hintergrundbild.Width / 1000 * factor & " x " & vbCrLf & imgScale * hintergrundbild.Height / 1000 * factor & unit
                        End If
                    End Using
                    SaveAs.FileName = LoadFile.FileName
                Catch ex As Exception
                    isLoading = False
                    Exit Sub
                End Try
                isLoading = False
                cleanupHistoryData()
                addHistory()
                Me.Refresh()
            End If
        End If
    End Sub

    Private Sub SaveToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SaveToolStripMenuItem.Click
        If SaveAs.FileName <> "" Then
            Using DateiOut As StreamWriter = My.Computer.FileSystem.OpenTextFileWriter(SaveAs.FileName, False, System.Text.Encoding.GetEncoding(20127))
                DateiOut.WriteLine("0 LDraw - Pattern Creator 1.1 ASCII File")
                ' Dreiecke speichern:
                DateiOut.WriteLine(Triangles.Count)
                For Each tri As Triangle In Triangles
                    DateiOut.WriteLine(tri.triangleID)
                    DateiOut.WriteLine(tri.myColour.ToArgb)
                    DateiOut.WriteLine(tri.myColourNumber)
                    DateiOut.WriteLine(tri.vertexA.vertexID)
                    DateiOut.WriteLine(tri.vertexB.vertexID)
                    DateiOut.WriteLine(tri.vertexC.vertexID)
                    DateiOut.WriteLine(tri.groupindex)
                Next
                ' Vertices speichern:
                DateiOut.WriteLine(Vertices.Count)
                For Each vert As Vertex In Vertices
                    DateiOut.WriteLine(vert.vertexID)
                    DateiOut.WriteLine(vert.X)
                    DateiOut.WriteLine(vert.Y)
                    DateiOut.WriteLine(vert.groupindex)
                    DateiOut.WriteLine(vert.linkedTriangles.Count)
                    For Each tri As Triangle In vert.linkedTriangles
                        DateiOut.WriteLine(tri.triangleID)
                    Next
                Next
                ' Primitive speichern:
                DateiOut.WriteLine(Primitives.Count)
                For Each prim As Primitive In Primitives
                    DateiOut.WriteLine(prim.primitiveID)
                    DateiOut.WriteLine(prim.centerVertexID)
                    DateiOut.WriteLine(prim.ox)
                    DateiOut.WriteLine(prim.oy)
                    DateiOut.WriteLine(prim.primitiveName)
                    For z As Integer = 0 To 3
                        For s As Integer = 0 To 3
                            DateiOut.WriteLine(prim.matrix(z, s))
                        Next
                    Next
                Next

                DateiOut.WriteLine(vertexIDglobal)
                DateiOut.WriteLine(triangleIDglobal)
                DateiOut.WriteLine(primitiveIDglobal)

                DateiOut.WriteLine(TBImage.Text)
                DateiOut.WriteLine(NUDoffsetX.Value)
                DateiOut.WriteLine(NUDoffsetY.Value)
                DateiOut.WriteLine(NUDScale.Value)

                DateiOut.WriteLine(factor)
                DateiOut.WriteLine(unit)
                DateiOut.WriteLine(NUDMoveSnap.Value)
                DateiOut.WriteLine(NUDRotateSnap.Value)
                DateiOut.WriteLine(NUDScaleSnap.Value)
                DateiOut.WriteLine(NUDGrid.Value)

            End Using
        Else
            SaveAsToolStripMenuItem.PerformClick()
        End If
    End Sub

    Private Sub SaveAsToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SaveAsToolStripMenuItem.Click
        If SaveAs.ShowDialog() = Windows.Forms.DialogResult.OK Then
            If SaveAs.FileName <> "" Then
                SaveToolStripMenuItem.PerformClick()
            End If
        End If
    End Sub

    Private Sub ToolStripMenuItem1_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ExportToolStripMenuItem.Click
        If SaveAsDAT.ShowDialog() = Windows.Forms.DialogResult.OK Then
            If SaveAsDAT.FileName <> "" Then
                If MessageBox.Show("Do you want to release this pattern and auto-correct BFC orientation and scale?", "Post optimisation:", MessageBoxButtons.YesNo) = Windows.Forms.DialogResult.Yes Then
                    cleanupMergedTriangles()
                    scaleVertices(True)
                    If MessageBox.Show("Do you want to auto-insert LDraw-Primitives?", "Post optimisation:", MessageBoxButtons.YesNo) = Windows.Forms.DialogResult.Yes Then
                        test_x.Clear()
                        test_y.Clear()
                        Using DateiOut As StreamWriter = My.Computer.FileSystem.OpenTextFileWriter(SaveAsDAT.FileName, False, System.Text.Encoding.GetEncoding(20127))
                            DateiOut.WriteLine("0 BFC CERTIFY CCW")
                            SelectedVertices.Clear()
                            SelectedTriangles.Clear()
                            For Each tri As Triangle In Triangles
                                tri.selected = False
                            Next
                            For Each vert As Vertex In Vertices
                                vert.selected = False
                            Next
                            For Each vert As Vertex In Vertices

                                ' 4-4ndis.dat
                                If Not vert.selected AndAlso vert.linkedTriangles.Count >= 4 Then
                                    Dim tx As Single = 0
                                    Dim ty As Single = 0
                                    Dim dist As Single = 0
                                    Dim dist2 As Single = Single.MaxValue
                                    Dim dvert(1, 1) As Single
                                    Dim tcolor As Short = 0
                                    Dim index1 As Byte = 0
                                    Dim index2 As Byte = 0

                                    Dim everts(3) As Vertex
                                    Dim bverts(3) As Vertex
                                    Dim iverts(11) As Vertex

                                    Dim rvx As Single = Single.NaN
                                    Dim rvy As Single = Single.NaN

                                    ' Punkte mit rechtem Winkel bestimmen
                                    For Each tri As Triangle In vert.linkedTriangles
                                        index2 = 0
                                        Dim tv1(2) As Vertex
                                        tv1(0) = tri.vertexA : tv1(1) = tri.vertexB : tv1(2) = tri.vertexC
                                        For Each tri2 As Triangle In vert.linkedTriangles
                                            If tri.triangleID <> tri2.triangleID Then
                                                Dim tv2(2) As Vertex
                                                tv2(0) = tri2.vertexA : tv2(1) = tri2.vertexB : tv2(2) = tri2.vertexC
                                                Dim v1(1), v2(1) As Single
                                                For i As Integer = 0 To 2
                                                    For j As Integer = 0 To 2
                                                        If tv1(i).vertexID <> tv2(j).vertexID AndAlso tv1(i).vertexID <> vert.vertexID AndAlso tv2(j).vertexID <> vert.vertexID Then
                                                            v1(0) = tv1(i).X - vert.X : v1(1) = tv1(i).Y - vert.Y
                                                            v2(0) = tv2(j).X - vert.X : v2(1) = tv2(j).Y - vert.Y
                                                            If v1(0) * v2(0) + v1(1) * v2(1) > -0.01 AndAlso v1(0) * v2(0) + v1(1) * v2(1) < 0.01 Then
                                                                rvx = (v1(0) + v2(0)) / 2
                                                                rvy = (v1(1) + v2(1)) / 2
                                                                everts(0) = vert
                                                                If Atan2(tv1(i).Y, tv1(i).X) < Atan2(tv2(j).Y, tv2(j).X) Then
                                                                    bverts(0) = tv1(i)
                                                                    bverts(1) = tv2(j)
                                                                Else
                                                                    bverts(0) = tv2(j)
                                                                    bverts(1) = tv1(i)
                                                                End If
                                                                Exit For
                                                            End If
                                                        End If
                                                    Next j
                                                    If Not Single.IsNaN(rvx) Then Exit For
                                                Next i
                                            End If
                                            index2 += 1
                                            If Not Single.IsNaN(rvx) Then Exit For
                                        Next
                                        index1 += 1
                                        If Not Single.IsNaN(rvx) Then Exit For
                                    Next

                                    If Not Single.IsNaN(rvx) Then
                                        testcx = vert.X
                                        testcy = vert.Y
                                        testdx11 = bverts(0).X
                                        testdy11 = bverts(0).Y
                                        testdx12 = bverts(1).X
                                        testdy12 = bverts(1).Y
                                        testdx21 = rvx
                                        testdy21 = rvy

                                        dist = Math.Sqrt((vert.X - bverts(0).X) ^ 2 + (vert.Y - bverts(0).Y) ^ 2)
                                        dist2 = Math.Sqrt((vert.X - bverts(1).X) ^ 2 + (vert.Y - bverts(1).Y) ^ 2)
                                        Dim angle As Single = Math.Atan2(rvy, rvx)
                                        tx = vert.X + Math.Cos(angle) * Math.Sqrt(dist ^ 2 + dist2 ^ 2)
                                        ty = vert.Y + Math.Sin(angle) * Math.Sqrt(dist ^ 2 + dist2 ^ 2)
                                        testcx = tx
                                        testcy = ty
                                        SelectedVertices.Add(vert)
                                        For i As Integer = 1 To 5
                                            selectTouching2()
                                        Next
                                        angle = Math.Atan2(bverts(0).Y - ty, bverts(0).X - tx)
                                        Dim notmatchincircle As Boolean = True
                                        Dim reverse As Boolean = False
                                        Dim hits As Integer = 0
                                        For tangle As Single = angle To angle + 2 * Math.PI Step (2 / 16 * Math.PI)
                                            Dim tax As Single = tx - dist * Math.Cos(tangle) * Math.Cos(angle) + dist2 * Math.Sin(tangle) * Math.Sin(angle)
                                            Dim tay As Single = ty + dist * Math.Cos(tangle) * Math.Sin(angle) + dist2 * Math.Sin(tangle) * Math.Cos(angle)
                                            notmatchincircle = True
                                            For Each tvert As Vertex In SelectedVertices
                                                If (tax >= tvert.X - 0.1F AndAlso tax <= tvert.X + 0.1F AndAlso tay >= tvert.Y - 0.1F AndAlso tay <= tvert.Y + 0.1F) Then
                                                    notmatchincircle = False
                                                    hits += 1
                                                    Exit For
                                                End If
                                            Next
                                            If notmatchincircle Then Exit For
                                        Next tangle
                                        If hits < 2 Then
                                            reverse = True
                                            hits = 0
                                            For tangle As Single = angle + 2 * Math.PI To angle Step -(2 / 16 * Math.PI)
                                                Dim tax As Single = tx - dist * Math.Cos(tangle) * Math.Cos(angle) + dist2 * Math.Sin(tangle) * Math.Sin(angle)
                                                Dim tay As Single = ty + dist * Math.Cos(tangle) * Math.Sin(angle) + dist2 * Math.Sin(tangle) * Math.Cos(angle)
                                                notmatchincircle = True
                                                For Each tvert As Vertex In SelectedVertices
                                                    If (tax >= tvert.X - 0.1F AndAlso tax <= tvert.X + 0.1F AndAlso tay >= tvert.Y - 0.1F AndAlso tay <= tvert.Y + 0.1F) Then
                                                        notmatchincircle = False
                                                        hits += 1
                                                        Exit For
                                                    End If
                                                Next
                                                If notmatchincircle Then Exit For
                                            Next tangle
                                        End If
                                        If hits = 4 OrElse hits = 5 OrElse hits = 8 OrElse hits = 9 OrElse hits = 12 OrElse hits = 13 OrElse hits = 16 OrElse hits = 17 Then
                                            vert.selected = True
                                            Dim tempset As String = ""
                                            For tangle As Single = angle To angle + 2 * Math.PI Step (2 / 16 * Math.PI)
                                                Dim tax As Single = tx - dist * Math.Cos(tangle) * Math.Cos(angle) + dist2 * Math.Sin(tangle) * Math.Sin(angle)
                                                Dim tay As Single = ty + dist * Math.Cos(tangle) * Math.Sin(angle) + dist2 * Math.Sin(tangle) * Math.Cos(angle)
                                                tempset += "(" & tax & " | " & tay & ")" & vbCrLf
                                            Next
                                            tempset += ""
                                        End If
                                        SelectedVertices.Clear()
                                    End If

                                End If


                                ' 4-4disc.dat
                                If False AndAlso Not vert.selected AndAlso vert.linkedTriangles.Count = 16 Then
                                    Dim tx As Single = 0
                                    Dim ty As Single = 0
                                    Dim dist As Single = 0
                                    Dim dist2 As Single = Single.MaxValue
                                    Dim dvert(1, 1) As Single
                                    Dim tcolor As Short = vert.linkedTriangles(0).myColourNumber
                                    Dim index1 As Byte = 0
                                    Dim index2 As Byte = 0

                                    Dim rvx As Single
                                    Dim rvy As Single

                                    ' Punkte mit größtem Abstand und Mittelpunkt bestimmen
                                    For Each tri As Triangle In vert.linkedTriangles
                                        If tcolor <> tri.myColourNumber OrElse tri.selected Then tx = vert.X - 2 : Exit For
                                        index2 = 0
                                        tx += tri.vertexA.X + tri.vertexB.X + tri.vertexC.X
                                        ty += tri.vertexA.Y + tri.vertexB.Y + tri.vertexC.Y
                                        For Each tri2 As Triangle In vert.linkedTriangles

                                            If (tri.vertexA.X - tri2.vertexA.X) ^ 2 + (tri.vertexA.Y - tri2.vertexA.Y) ^ 2 > dist Then
                                                dist = (tri.vertexA.X - tri2.vertexA.X) ^ 2 + (tri.vertexA.Y - tri2.vertexA.Y) ^ 2
                                                dvert(0, 0) = tri.vertexA.X : dvert(0, 1) = tri.vertexA.Y : dvert(1, 0) = tri2.vertexA.X : dvert(1, 1) = tri2.vertexA.Y
                                            End If
                                            If (tri.vertexB.X - tri2.vertexA.X) ^ 2 + (tri.vertexB.Y - tri2.vertexA.Y) ^ 2 > dist Then
                                                dist = (tri.vertexB.X - tri2.vertexA.X) ^ 2 + (tri.vertexB.Y - tri2.vertexA.Y) ^ 2
                                                dvert(0, 0) = tri.vertexB.X : dvert(0, 1) = tri.vertexB.Y : dvert(1, 0) = tri2.vertexA.X : dvert(1, 1) = tri2.vertexA.Y
                                            End If
                                            If (tri.vertexC.X - tri2.vertexA.X) ^ 2 + (tri.vertexC.Y - tri2.vertexA.Y) ^ 2 > dist Then
                                                dist = (tri.vertexC.X - tri2.vertexA.X) ^ 2 + (tri.vertexC.Y - tri2.vertexA.Y) ^ 2
                                                dvert(0, 0) = tri.vertexC.X : dvert(0, 1) = tri.vertexC.Y : dvert(1, 0) = tri2.vertexA.X : dvert(1, 1) = tri2.vertexA.Y
                                            End If


                                            If (tri.vertexA.X - tri2.vertexB.X) ^ 2 + (tri.vertexA.Y - tri2.vertexB.Y) ^ 2 > dist Then
                                                dist = (tri.vertexA.X - tri2.vertexB.X) ^ 2 + (tri.vertexA.Y - tri2.vertexB.Y) ^ 2
                                                dvert(0, 0) = tri.vertexA.X : dvert(0, 1) = tri.vertexA.Y : dvert(1, 0) = tri2.vertexB.X : dvert(1, 1) = tri2.vertexB.Y
                                            End If
                                            If (tri.vertexB.X - tri2.vertexB.X) ^ 2 + (tri.vertexB.Y - tri2.vertexB.Y) ^ 2 > dist Then
                                                dist = (tri.vertexB.X - tri2.vertexB.X) ^ 2 + (tri.vertexB.Y - tri2.vertexB.Y) ^ 2
                                                dvert(0, 0) = tri.vertexB.X : dvert(0, 1) = tri.vertexB.Y : dvert(1, 0) = tri2.vertexB.X : dvert(1, 1) = tri2.vertexB.Y
                                            End If
                                            If (tri.vertexC.X - tri2.vertexB.X) ^ 2 + (tri.vertexC.Y - tri2.vertexB.Y) ^ 2 > dist Then
                                                dist = (tri.vertexC.X - tri2.vertexB.X) ^ 2 + (tri.vertexC.Y - tri2.vertexB.Y) ^ 2
                                                dvert(0, 0) = tri.vertexC.X : dvert(0, 1) = tri.vertexC.Y : dvert(1, 0) = tri2.vertexB.X : dvert(1, 1) = tri2.vertexB.Y
                                            End If

                                            If (tri.vertexA.X - tri2.vertexC.X) ^ 2 + (tri.vertexA.Y - tri2.vertexC.Y) ^ 2 > dist Then
                                                dist = (tri.vertexA.X - tri2.vertexC.X) ^ 2 + (tri.vertexA.Y - tri2.vertexC.Y) ^ 2
                                                dvert(0, 0) = tri.vertexA.X : dvert(0, 1) = tri.vertexA.Y : dvert(1, 0) = tri2.vertexC.X : dvert(1, 1) = tri2.vertexC.Y
                                            End If
                                            If (tri.vertexB.X - tri2.vertexC.X) ^ 2 + (tri.vertexB.Y - tri2.vertexC.Y) ^ 2 > dist Then
                                                dist = (tri.vertexB.X - tri2.vertexC.X) ^ 2 + (tri.vertexB.Y - tri2.vertexC.Y) ^ 2
                                                dvert(0, 0) = tri.vertexB.X : dvert(0, 1) = tri.vertexB.Y : dvert(1, 0) = tri2.vertexC.X : dvert(1, 1) = tri2.vertexC.Y
                                            End If
                                            If (tri.vertexC.X - tri2.vertexC.X) ^ 2 + (tri.vertexC.Y - tri2.vertexC.Y) ^ 2 > dist Then
                                                dist = (tri.vertexC.X - tri2.vertexC.X) ^ 2 + (tri.vertexC.Y - tri2.vertexC.Y) ^ 2
                                                dvert(0, 0) = tri.vertexC.X : dvert(0, 1) = tri.vertexC.Y : dvert(1, 0) = tri2.vertexC.X : dvert(1, 1) = tri2.vertexC.Y
                                            End If
                                            index2 += 1
                                        Next
                                        index1 += 1
                                    Next

                                    tx /= 48
                                    ty /= 48

                                    If tx >= vert.X - 0.1F AndAlso tx <= vert.X + 0.1F AndAlso ty >= vert.Y - 0.1F AndAlso ty <= vert.Y + 0.1F Then

                                        dist = Math.Sqrt(dist) / 2
                                        rvx = dvert(0, 0) - dvert(1, 0)
                                        rvy = dvert(0, 1) - dvert(1, 1)

                                        Dim angle As Single = -Math.Atan2(rvy, rvx)

                                        ' dist2 für Ovale berechnen und 16 Punkte testen..
                                        For Each tri As Triangle In vert.linkedTriangles
                                            If Not vert.Equals(tri.vertexA) Then
                                                If (tri.vertexA.X - tx) ^ 2 + (tri.vertexA.Y - ty) ^ 2 < dist2 Then
                                                    dist2 = (tri.vertexA.X - tx) ^ 2 + (tri.vertexA.Y - ty) ^ 2
                                                    testdx22 = tri.vertexA.X : testdy22 = tri.vertexA.Y
                                                End If
                                            End If
                                            If Not vert.Equals(tri.vertexB) Then
                                                If (tri.vertexB.X - tx) ^ 2 + (tri.vertexB.Y - ty) ^ 2 < dist2 Then
                                                    dist2 = (tri.vertexB.X - tx) ^ 2 + (tri.vertexB.Y - ty) ^ 2
                                                    testdx22 = tri.vertexB.X : testdy22 = tri.vertexB.Y
                                                End If
                                            End If
                                            If Not vert.Equals(tri.vertexC) Then
                                                If (tri.vertexC.X - tx) ^ 2 + (tri.vertexC.Y - ty) ^ 2 < dist2 Then
                                                    dist2 = (tri.vertexC.X - tx) ^ 2 + (tri.vertexC.Y - ty) ^ 2
                                                    testdx22 = tri.vertexC.X : testdy22 = tri.vertexC.Y
                                                End If
                                            End If
                                        Next

                                        dist2 = Math.Sqrt(dist2)

                                        Dim notmatchincircle As Boolean = True
                                        For tangle As Single = 0 To 2 * Math.PI Step (2 / 16 * Math.PI)
                                            Dim tax As Single = vert.X - dist * Math.Cos(tangle) * Math.Cos(angle) + dist2 * Math.Sin(tangle) * Math.Sin(angle)
                                            Dim tay As Single = vert.Y + dist * Math.Cos(tangle) * Math.Sin(angle) + dist2 * Math.Sin(tangle) * Math.Cos(angle)
                                            notmatchincircle = True
                                            For Each tri As Triangle In vert.linkedTriangles
                                                If (tax >= tri.vertexA.X - 0.1F AndAlso tax <= tri.vertexA.X + 0.1F AndAlso tay >= tri.vertexA.Y - 0.1F AndAlso tay <= tri.vertexA.Y + 0.1F) OrElse _
                                                   (tax >= tri.vertexB.X - 0.1F AndAlso tax <= tri.vertexB.X + 0.1F AndAlso tay >= tri.vertexB.Y - 0.1F AndAlso tay <= tri.vertexB.Y + 0.1F) OrElse _
                                                   (tax >= tri.vertexC.X - 0.1F AndAlso tax <= tri.vertexC.X + 0.1F AndAlso tay >= tri.vertexC.Y - 0.1F AndAlso tay <= tri.vertexC.Y + 0.1F) Then
                                                    notmatchincircle = False
                                                    Exit For
                                                End If
                                            Next
                                            If notmatchincircle Then Exit For
                                        Next tangle
                                        If notmatchincircle Then Continue For
                                        vert.selected = True
                                        For Each tri As Triangle In vert.linkedTriangles
                                            tri.selected = True
                                            tri.vertexA.selected = True
                                            tri.vertexB.selected = True
                                            tri.vertexC.selected = True
                                        Next

                                        testcx = tx
                                        testcy = ty
                                        testdx11 = dvert(0, 0)
                                        testdy11 = dvert(0, 1)
                                        testdx12 = dvert(1, 0)
                                        testdy12 = dvert(1, 1)

                                        testdx21 = tx
                                        testdy21 = ty

                                        DateiOut.WriteLine(Replace("1 " & tcolor & " " & _
                                        vert.X & " " & vert.Y & " 0 " & _
                                        dist * Math.Cos(angle) & " 0 " & dist2 * Math.Sin(angle) & " " & _
                                        -dist * Math.Sin(angle) & " 0 " & dist2 * Math.Cos(angle) & " " & _
                                        "0 -1 0" & _
                                        " 4-4disc.dat", ",", "."))
                                    End If
                                End If


                            Next

                            For Each tri As Triangle In Triangles
                                If Not tri.selected Then
                                    tri.normalize()
                                    tri.checkWinding()
                                    DateiOut.WriteLine(Replace("3 " & tri.myColourNumber & " " & tri.vertexA.X & " " & tri.vertexA.Y & " 0 " _
                                                                       & tri.vertexB.X & " " & tri.vertexB.Y & " 0 " _
                                                                       & tri.vertexC.X & " " & tri.vertexC.Y & " 0", ",", "."))
                                End If
                                tri.selected = False
                            Next
                        End Using
                        Exit Sub
                    End If
                End If
                Using DateiOut As StreamWriter = My.Computer.FileSystem.OpenTextFileWriter(SaveAsDAT.FileName, False, System.Text.Encoding.GetEncoding(20127))
                    DateiOut.WriteLine("0 BFC CERTIFY CCW")
                    For Each tri As Triangle In Triangles
                        tri.normalize()
                        tri.checkWinding()
                        DateiOut.WriteLine(Replace("3 " & tri.myColourNumber & " " & tri.vertexA.X & " " & tri.vertexA.Y & " 0 " _
                                                           & tri.vertexB.X & " " & tri.vertexB.Y & " 0 " _
                                                           & tri.vertexC.X & " " & tri.vertexC.Y & " 0", ",", "."))
                    Next
                End Using
            End If
        End If
    End Sub

    Private Sub NewPatternToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles NewPatternToolStripMenuItem.Click
        isLoading = True
        newPattern()
        addHistory()
        isLoading = False
    End Sub

    Private Sub newPattern()
        hintergrundbild.Dispose()
        hintergrundbild = My.Resources.temp
        SelectedTriangles.Clear()
        SelectedVertices.Clear()
        Vertices.Clear()
        Triangles.Clear()
        Primitives.Clear()
        PrimitivesHMap.Clear()
        vertexIDglobal = 0
        Me.TBImage.Text = ""
        triangleIDglobal = 0
        primitiveIDglobal = 0
        Me.NUDMoveSnap.Value = 100
        Me.NUDScaleSnap.Value = 0.01
        Me.NUDRotateSnap.Value = 15
        Me.NUDScale.Value = 1
        Me.NUDoffsetX.Value = 0
        Me.NUDoffsetY.Value = 0
        Me.NUDGrid.Value = 1
        Me.Refresh()
    End Sub

#End Region
#Region "Primitives"
    Dim primitiveMode As Byte
    Dim primitiveObject As Byte
    Dim primitiveName As String

    Dim primitiveCenter As New Vertex(0, 0, False, False)
    Dim primitiveHeight As Single
    Dim primitiveWidth As Single
    Dim primitiveBordersize As Single


    Private Sub startPrimitiveMode()
        primitiveMode = 1
        Me.ToolStrip1.Visible = False
        Me.BtnAbort.Visible = True
        TriangleModeToolStripMenuItem.PerformClick()
    End Sub

    Private Sub abortPrimitiveMode()
        primitiveMode = 0
        Me.ToolStrip1.Visible = True
        Me.BtnAbort.Visible = False
    End Sub

    Private Sub discGenerator(ByVal n As Double, ByVal m As Double, ByVal segments As Integer)
        If segments = 48 Then
            Primitives.Add(New Primitive(primitiveCenter.X, primitiveCenter.Y, 0, 0, "48\" & n & "-" & m & "disc.dat", vertexIDglobal + 1))
        Else
            Primitives.Add(New Primitive(primitiveCenter.X, primitiveCenter.Y, 0, 0, n & "-" & m & "disc.dat", vertexIDglobal + 1))
        End If
        Vertices.Add(New Vertex(primitiveCenter.X, primitiveCenter.Y, False) With {.groupindex = primitiveIDglobal})
        Dim cvert As Vertex = LLast(Vertices)
        Vertices.Add(New Vertex(primitiveCenter.X + 1000, primitiveCenter.Y, False) With {.groupindex = primitiveIDglobal})
        Dim fvert As Vertex = LLast(Vertices)
        If n <> m Then
            For r As Double = twoPI / segments To twoPI * (n / m) + 0.5 * twoPI / segments Step twoPI / segments
                Vertices.Add(New Vertex(primitiveCenter.X + 1000 * Math.Cos(r), primitiveCenter.Y + 1000 * Math.Sin(r), False) With {.groupindex = primitiveIDglobal})
                Triangles.Add(New Triangle(Vertices(Vertices.Count - 1), Vertices(Vertices.Count - 2), cvert) With {.groupindex = primitiveIDglobal})
                cvert.linkedTriangles.Add(LLast(Triangles))
                Vertices(Vertices.Count - 1).linkedTriangles.Add(LLast(Triangles))
                Vertices(Vertices.Count - 2).linkedTriangles.Add(LLast(Triangles))
            Next
        Else
            For r As Double = twoPI / segments To twoPI Step twoPI / segments
                Vertices.Add(New Vertex(primitiveCenter.X + 1000 * Math.Cos(r), primitiveCenter.Y + 1000 * Math.Sin(r), False) With {.groupindex = primitiveIDglobal})
                Triangles.Add(New Triangle(Vertices(Vertices.Count - 1), Vertices(Vertices.Count - 2), cvert) With {.groupindex = primitiveIDglobal})
                cvert.linkedTriangles.Add(LLast(Triangles))
                Vertices(Vertices.Count - 1).linkedTriangles.Add(LLast(Triangles))
                Vertices(Vertices.Count - 2).linkedTriangles.Add(LLast(Triangles))
            Next
            Triangles.Add(New Triangle(Vertices(Vertices.Count - 1), fvert, cvert) With {.groupindex = primitiveIDglobal})
            cvert.linkedTriangles.Add(LLast(Triangles))
            Vertices(Vertices.Count - 1).linkedTriangles.Add(LLast(Triangles))
            fvert.linkedTriangles.Add(LLast(Triangles))
        End If
    End Sub

    Private Sub ndiscGenerator(ByVal n As Double, ByVal m As Double, ByVal segments As Integer)
        If segments = 48 Then
            Primitives.Add(New Primitive(primitiveCenter.X, primitiveCenter.Y, 0, 0, "48\" & n & "-" & m & "ndis.dat", vertexIDglobal + 1))
        Else
            Primitives.Add(New Primitive(primitiveCenter.X, primitiveCenter.Y, 0, 0, n & "-" & m & "ndis.dat", vertexIDglobal + 1))
        End If
        Vertices.Add(New Vertex(primitiveCenter.X, primitiveCenter.Y, False) With {.groupindex = primitiveIDglobal})
        Dim sindex As Integer = Vertices.Count
        Dim cvert As Vertex = LLast(Vertices)
        Vertices.Add(New Vertex(primitiveCenter.X + 1000, primitiveCenter.Y, False) With {.groupindex = primitiveIDglobal})
        Dim fvert As Vertex = LLast(Vertices)
        Dim evert(3) As Vertex
        If n <> m Then
            Dim frad As Double = twoPI * (n / m) + 0.5 * twoPI / segments
            For r As Double = twoPI / segments To frad Step twoPI / segments
                Vertices.Add(New Vertex(primitiveCenter.X + 1000 * Math.Cos(r), primitiveCenter.Y + 1000 * Math.Sin(r), False) With {.groupindex = primitiveIDglobal})
            Next

            Vertices.Add(New Vertex(primitiveCenter.X + 1000, primitiveCenter.Y + 1000, False) With {.groupindex = primitiveIDglobal})
            evert(0) = LLast(Vertices)
            Dim offset As Integer = 1
            If frad > twoPI * (1 / 4) + 0.5 * twoPI / segments Then
                Vertices.Add(New Vertex(primitiveCenter.X - 1000, primitiveCenter.Y + 1000, False) With {.groupindex = primitiveIDglobal})
                evert(1) = LLast(Vertices)
                offset += 1
            End If
            If frad > twoPI * (2 / 4) + 0.5 * twoPI / segments Then
                Vertices.Add(New Vertex(primitiveCenter.X - 1000, primitiveCenter.Y - 1000, False) With {.groupindex = primitiveIDglobal})
                evert(2) = LLast(Vertices)
                offset += 1
            End If
            If frad > twoPI * (3 / 4) + 0.5 * twoPI / segments Then
                Vertices.Add(New Vertex(primitiveCenter.X + 1000, primitiveCenter.Y - 1000, False) With {.groupindex = primitiveIDglobal})
                evert(3) = LLast(Vertices)
                offset += 1
            End If

            For i As Integer = sindex To sindex + segments / 4 - 1
                If i >= Vertices.Count - 1 - offset Then Exit For
                Triangles.Add(New Triangle(Vertices(i), Vertices(i + 1), evert(0)) With {.groupindex = primitiveIDglobal})
                Vertices(i).linkedTriangles.Add(LLast(Triangles))
                Vertices(i + 1).linkedTriangles.Add(LLast(Triangles))
                evert(0).linkedTriangles.Add(LLast(Triangles))
            Next i

            For i As Integer = sindex + segments / 4 To sindex + 2 * (segments / 4) - 1
                If i >= Vertices.Count - 1 - offset Then Exit For
                Triangles.Add(New Triangle(Vertices(i), Vertices(i + 1), evert(1)) With {.groupindex = primitiveIDglobal})
                Vertices(i).linkedTriangles.Add(LLast(Triangles))
                Vertices(i + 1).linkedTriangles.Add(LLast(Triangles))
                evert(1).linkedTriangles.Add(LLast(Triangles))
            Next i

            For i As Integer = sindex + 2 * (segments / 4) To sindex + 3 * (segments / 4) - 1
                If i >= Vertices.Count - 1 - offset Then Exit For
                Triangles.Add(New Triangle(Vertices(i), Vertices(i + 1), evert(2)) With {.groupindex = primitiveIDglobal})
                Vertices(i).linkedTriangles.Add(LLast(Triangles))
                Vertices(i + 1).linkedTriangles.Add(LLast(Triangles))
                evert(2).linkedTriangles.Add(LLast(Triangles))
            Next i

            For i As Integer = sindex + 3 * (segments / 4) To sindex + segments - 2
                If i >= Vertices.Count - 1 - offset Then Exit For
                Triangles.Add(New Triangle(Vertices(i), Vertices(i + 1), evert(3)) With {.groupindex = primitiveIDglobal})
                Vertices(i).linkedTriangles.Add(LLast(Triangles))
                Vertices(i + 1).linkedTriangles.Add(LLast(Triangles))
                evert(3).linkedTriangles.Add(LLast(Triangles))
            Next i

            Triangles.Add(New Triangle(Vertices(sindex - 1), Vertices(sindex), Vertices(sindex - 1)) With {.groupindex = primitiveIDglobal})
            Vertices(sindex - 1).linkedTriangles.Add(LLast(Triangles))
            Vertices(sindex).linkedTriangles.Add(LLast(Triangles))

        Else
            For r As Double = twoPI / segments To twoPI Step twoPI / segments
                Vertices.Add(New Vertex(primitiveCenter.X + 1000 * Math.Cos(r), primitiveCenter.Y + 1000 * Math.Sin(r), False) With {.groupindex = primitiveIDglobal})
            Next

            Vertices.Add(New Vertex(primitiveCenter.X + 1000, primitiveCenter.Y + 1000, False) With {.groupindex = primitiveIDglobal})
            evert(0) = LLast(Vertices)
            Vertices.Add(New Vertex(primitiveCenter.X - 1000, primitiveCenter.Y + 1000, False) With {.groupindex = primitiveIDglobal})
            evert(1) = LLast(Vertices)
            Vertices.Add(New Vertex(primitiveCenter.X - 1000, primitiveCenter.Y - 1000, False) With {.groupindex = primitiveIDglobal})
            evert(2) = LLast(Vertices)
            Vertices.Add(New Vertex(primitiveCenter.X + 1000, primitiveCenter.Y - 1000, False) With {.groupindex = primitiveIDglobal})
            evert(3) = LLast(Vertices)
            For i As Integer = sindex To sindex + segments / 4 - 1
                Triangles.Add(New Triangle(Vertices(i), Vertices(i + 1), evert(0)) With {.groupindex = primitiveIDglobal})
                Vertices(i).linkedTriangles.Add(LLast(Triangles))
                Vertices(i + 1).linkedTriangles.Add(LLast(Triangles))
                evert(0).linkedTriangles.Add(LLast(Triangles))
            Next i

            For i As Integer = sindex + segments / 4 To sindex + 2 * (segments / 4) - 1
                Triangles.Add(New Triangle(Vertices(i), Vertices(i + 1), evert(1)) With {.groupindex = primitiveIDglobal})
                Vertices(i).linkedTriangles.Add(LLast(Triangles))
                Vertices(i + 1).linkedTriangles.Add(LLast(Triangles))
                evert(1).linkedTriangles.Add(LLast(Triangles))
            Next i

            For i As Integer = sindex + 2 * (segments / 4) To sindex + 3 * (segments / 4) - 1
                Triangles.Add(New Triangle(Vertices(i), Vertices(i + 1), evert(2)) With {.groupindex = primitiveIDglobal})
                Vertices(i).linkedTriangles.Add(LLast(Triangles))
                Vertices(i + 1).linkedTriangles.Add(LLast(Triangles))
                evert(2).linkedTriangles.Add(LLast(Triangles))
            Next i

            For i As Integer = sindex + 3 * (segments / 4) To sindex + segments - 2
                Triangles.Add(New Triangle(Vertices(i), Vertices(i + 1), evert(3)) With {.groupindex = primitiveIDglobal})
                Vertices(i).linkedTriangles.Add(LLast(Triangles))
                Vertices(i + 1).linkedTriangles.Add(LLast(Triangles))
                evert(3).linkedTriangles.Add(LLast(Triangles))
            Next i
            Triangles.Add(New Triangle(Vertices(sindex), Vertices(sindex + segments - 1), evert(3)) With {.groupindex = primitiveIDglobal})
            Vertices(sindex).linkedTriangles.Add(LLast(Triangles))
            Vertices(sindex + segments - 1).linkedTriangles.Add(LLast(Triangles))
            evert(3).linkedTriangles.Add(LLast(Triangles))
            Triangles.Add(New Triangle(Vertices(sindex - 1), Vertices(sindex), Vertices(sindex - 1)) With {.groupindex = primitiveIDglobal})
            Vertices(sindex - 1).linkedTriangles.Add(LLast(Triangles))
            Vertices(sindex).linkedTriangles.Add(LLast(Triangles))
        End If
    End Sub

    Private Sub BtnAbort_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BtnAbort.Click
        abortPrimitiveMode()
        Me.Refresh()
    End Sub

    Private Sub EmbedOvalToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles EmbedOvalToolStripMenuItem.Click
        EmbedOvalToolStripMenuItem.GetCurrentParent.Show()
    End Sub

    Private Sub EmbedCircleToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles EmbedCircleToolStripMenuItem.Click
        EmbedCircleToolStripMenuItem.GetCurrentParent.Show()
    End Sub

    Private Sub SolidTriangleToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SolidTriangleToolStripMenuItem.Click
        primitiveObject = 0
        startPrimitiveMode()
    End Sub

    Private Sub TriangleWithFrameToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles TriangleWithFrameToolStripMenuItem.Click
        primitiveObject = 1
        startPrimitiveMode()
    End Sub

    Private Sub HollowTriangleToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles HollowTriangleToolStripMenuItem.Click
        primitiveObject = 2
        startPrimitiveMode()
    End Sub

    Private Sub SolidRectangleToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SolidRectangleToolStripMenuItem.Click
        primitiveObject = 3
        startPrimitiveMode()
    End Sub

    Private Sub RectangleWithFrameToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RectangleWithFrameToolStripMenuItem.Click
        primitiveObject = 4
        startPrimitiveMode()
    End Sub

    Private Sub HollowRectangleToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles HollowRectangleToolStripMenuItem.Click
        primitiveObject = 5
        startPrimitiveMode()
    End Sub

    Private Sub SolidCircleToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SolidCircleToolStripMenuItem.Click
        primitiveObject = 6
        startPrimitiveMode()
    End Sub

    Private Sub CircleWithFrameToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CircleWithFrameToolStripMenuItem.Click
        primitiveObject = 7
        startPrimitiveMode()
    End Sub

    Private Sub HollowCircleToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles HollowCircleToolStripMenuItem.Click
        primitiveObject = 8
        startPrimitiveMode()
    End Sub

    Private Sub SolidOvalToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SolidOvalToolStripMenuItem.Click
        primitiveObject = 9
        startPrimitiveMode()
    End Sub

    Private Sub OvalWithFrameToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles OvalWithFrameToolStripMenuItem.Click
        primitiveObject = 10
        startPrimitiveMode()
    End Sub

    Private Sub HollowOvalToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles HollowOvalToolStripMenuItem.Click
        primitiveObject = 11
        startPrimitiveMode()
    End Sub

    Private Sub SolidArchToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SolidArchToolStripMenuItem.Click

        startPrimitiveMode()
    End Sub

    Private Sub SolidPolygonToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs)

        startPrimitiveMode()
    End Sub

    Private Sub PolygonWithFrameToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs)

        startPrimitiveMode()
    End Sub

    Private Sub HollowPolygonToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs)

        startPrimitiveMode()
    End Sub

    Private Sub ChainToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ChainToolStripMenuItem.Click
        If Vertices.Count > 1 Then
            startPrimitiveMode()
            primitiveMode = 255
        End If
    End Sub

    Private Sub CircleSegments_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CircleSegments.TextChanged
        Try
            segments_circle = CType(CircleSegments.Text, Integer)
            If segments_circle < 3 Then segments_circle = 3
        Catch ex As Exception
            CircleSegments.Text = segments_circle
        End Try
    End Sub

    Private Sub OvalSegments_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles OvalSegments.TextChanged
        Try
            segments_oval = CType(OvalSegments.Text, Integer)
            If segments_oval < 3 Then segments_oval = 3
        Catch ex As Exception
            OvalSegments.Text = segments_oval
        End Try
    End Sub

    Private Sub ArchSegments_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ArchSegments.TextChanged
        Try
            segments_arch = CType(ArchSegments.Text, Integer)
            If segments_arch < 3 Then segments_arch = 3
        Catch ex As Exception
            ArchSegments.Text = segments_arch
        End Try
    End Sub

#End Region

#Region "Tutorial"

    Dim turorialRuns As Boolean = False
    Private Sub CreateABasicPolygonToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ChangeViewportToolStripMenuItem.Click
        If Not Me.BackgroundHowTo.IsBusy AndAlso (Me.Vertices.Count = 0 OrElse MsgBox("Do you really want to start this lesson and discard the current pattern?", MsgBoxStyle.YesNo, "Warning:") = Windows.Forms.DialogResult.Yes) Then
            turorialRuns = True
            AbortCurrentLessonToolStripMenuItem.Enabled = True
            If MsgBox("Please don't move the mouse or hit on your keyboard (execpt of the ENTER key) while watching this how-to." & vbCrLf & vbCrLf & "You can abort the lesson by pressing Ctrl+Q at any time." & vbCrLf & "Press [ENTER] to continue.", MsgBoxStyle.OkOnly, "Info:") = MsgBoxResult.Ok Then
                Me.Activate()
                Me.WindowState = FormWindowState.Maximized
                Me.BringToFront()
                Control.CheckForIllegalCrossThreadCalls = False
                Me.BackgroundHowTo.RunWorkerAsync(0)
            End If
        End If
    End Sub

    Private Sub CreateApolygonToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CreateApolygonToolStripMenuItem.Click
        If Not Me.BackgroundHowTo.IsBusy AndAlso (Me.Vertices.Count = 0 OrElse MsgBox("Do you really want to start this lesson and discard the current pattern?", MsgBoxStyle.YesNo, "Warning:") = Windows.Forms.DialogResult.Yes) Then
            turorialRuns = True
            AbortCurrentLessonToolStripMenuItem.Enabled = True
            If MsgBox("Please don't move the mouse or hit on your keyboard (execpt of the ENTER key) while watching this how-to." & vbCrLf & vbCrLf & "You can abort the lesson by pressing Ctrl+Q at any time." & vbCrLf & "Press [ENTER] to continue.", MsgBoxStyle.OkOnly, "Info:") = MsgBoxResult.Ok Then
                Me.Activate()
                Me.WindowState = FormWindowState.Maximized
                Me.BringToFront()
                Control.CheckForIllegalCrossThreadCalls = False
                Me.BackgroundHowTo.RunWorkerAsync(1)
            End If
        End If
    End Sub

    Private Sub BackgroundHowTo_DoWork( _
    ByVal sender As Object, ByVal e As DoWorkEventArgs) _
    Handles BackgroundHowTo.DoWork

        ' Do not access the form's BackgroundWorker reference directly.
        ' Instead, use the reference provided by the sender parameter.
        Dim bw As BackgroundWorker = CType(sender, BackgroundWorker)

        ' Extract the argument.
        Dim TutorialNumber As Integer = e.Argument

        ' Start the time-consuming operation.
        e.Result = BackgroundHowToDo(bw, TutorialNumber)

        ' If the operation was canceled by the user, 
        ' set the DoWorkEventArgs.Cancel property to true.
        If bw.CancellationPending Then
            e.Cancel = True
        End If

    End Sub

    Private Sub BackgroundHowTo_RunWorkerCompleted( _
    ByVal sender As Object, ByVal e As RunWorkerCompletedEventArgs) _
    Handles BackgroundHowTo.RunWorkerCompleted

        If e.Cancelled Then
            ' The user canceled the operation.
        ElseIf Not (e.Error Is Nothing) Then
            ' There was an error during the operation.
        Else
            ' The operation completed normally.
        End If
    End Sub

    Private Function BackgroundHowToDo(ByVal bw As BackgroundWorker, ByVal TutorialNumber As Integer) As String
        resetGUI()
        If TutorialNumber = 0 Then
            Do
                If Not turorialRuns Then Exit Do
                While MsgBox("1.) Magnify the viewport: " & vbCrLf & "Use the mousewheel or the Page-Up- and Page-Down-keys to modify the zoom level of the viewport." & vbCrLf & "The zoom level ranges from 0.1% to 1000% with a maximum precision of.." & vbCrLf & ".. 0.001 LDU for moving objects .." & vbCrLf & ".. and more than 0.0000000001 LDU for rotating and scaling objects." & vbCrLf & vbCrLf & "Press [ENTER] to continue and watch this action.", MsgBoxStyle.OkOnly + MsgBoxStyle.MsgBoxSetForeground, "Info:") <> MsgBoxResult.Ok : End While
                moveMouseAbsoluteRightClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2, System.Windows.Forms.Screen.PrimaryScreen.Bounds.Height / 2)
                For i As Integer = 0 To 89
                    Thread.Sleep(30)
                    SBZoom_Scroll(Me, New ScrollEventArgs(ScrollEventType.SmallIncrement, 0, 1, ScrollOrientation.VerticalScroll))
                    If Not turorialRuns Then Exit Do
                Next i
                For i As Integer = 0 To 121
                    Thread.Sleep(30)
                    SBZoom_Scroll(Me, New ScrollEventArgs(ScrollEventType.SmallDecrement, 0, -1, ScrollOrientation.VerticalScroll))
                    If Not turorialRuns Then Exit Do
                Next i
                resetGUI()
                While MsgBox("2.) Move the viewport: " & vbCrLf & "Press the right mouse button to translate the origin of the viewport." & vbCrLf & "You can read the current positon of the cursor on the lower left corner of this form." & vbCrLf & "It is displayed in LDU coordinates." & vbCrLf & vbCrLf & "Press [ENTER] to continue and watch this action.", MsgBoxStyle.OkOnly + MsgBoxStyle.MsgBoxSetForeground, "Info:") <> MsgBoxResult.Ok : End While
                resetGUI()
                moveMouseAbsoluteRightClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2, System.Windows.Forms.Screen.PrimaryScreen.Bounds.Height / 2)
                Thread.Sleep(30)
                moveMouseRightSmooth(System.Windows.Forms.Cursor.Position.X, System.Windows.Forms.Cursor.Position.Y, System.Windows.Forms.Cursor.Position.X + 200, System.Windows.Forms.Cursor.Position.Y + 100, 100)
                While MsgBox("3.) Display/hide the grid: " & vbCrLf & "Click on 'Views' -> 'Show grid' to enable/disable this view." & vbCrLf & vbCrLf & "Press [ENTER] to continue and watch this action (I will use ALT+V and S to access the menu).", MsgBoxStyle.OkOnly + MsgBoxStyle.MsgBoxSetForeground, "Info:") <> MsgBoxResult.Ok : End While
                resetGUI()
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1000)
                SendKeys.SendWait("%v")
                ShowGridToolStripMenuItem.Select()
                If Not turorialRuns Then Exit Do Else Thread.Sleep(2000)
                SendKeys.SendWait("s%v")
                ShowGridToolStripMenuItem.Select()
                If Not turorialRuns Then Exit Do Else Thread.Sleep(2000)
                SendKeys.SendWait("s%v")
                ShowGridToolStripMenuItem.Select()
                If Not turorialRuns Then Exit Do Else Thread.Sleep(2000)
                SendKeys.SendWait("%")
                Me.Refresh()
                Me.Invalidate()
                Me.Update()
                While MsgBox("4.) Resize the grid: " & vbCrLf & "  - Click on 'Views' -> 'View-Preferences' to enable this view." & vbCrLf & "  - Adjust the 'Grid Size'-Value to increase/decrease the grid resolution." & vbCrLf & vbCrLf & "Press [ENTER] to continue and watch this action (I will use ALT+V and V to access the menu).", MsgBoxStyle.OkOnly + MsgBoxStyle.MsgBoxSetForeground, "Info:") <> MsgBoxResult.Ok : End While
                resetGUI()
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1000)
                SendKeys.SendWait("%v")
                SnappingToolStripMenuItem.Select()
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1000)
                SendKeys.SendWait("v%v")
                SnappingToolStripMenuItem.Select()
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1000)
                SendKeys.SendWait("%")
                NUDGrid.Value = 10
                While NUDGrid.Value > 0.1
                    If Not turorialRuns Then Exit Do
                    NUDGrid.Value -= 0.1
                    Thread.Sleep(100)
                End While
                If Not turorialRuns Then Exit Do Else While MsgBox("Thank you for watching this lesson." & vbCrLf & vbCrLf & "Press [ENTER] to continue.", MsgBoxStyle.OkOnly + MsgBoxStyle.MsgBoxSetForeground, "Info:") <> MsgBoxResult.Ok : End While
                'moveMouseAbsoluteRightClick(200, 200)
                'Thread.Sleep(1000)
                'moveMouseRightSmooth(200, 200, 300, 300, 100)
                'If Not turorialRuns Then Exit Do
                'Thread.Sleep(100)
                'releaseMouseRight()
                'Thread.Sleep(1000)
                'moveOnPosition("0,1", "0,2")
                'Thread.Sleep(1000)
                'moveOnPosition("0", "0")
                'Thread.Sleep(1000)
                'moveMouseRightSmooth(System.Windows.Forms.Cursor.Position.X, System.Windows.Forms.Cursor.Position.Y, p_old.X, p_old.Y, 100)
                Exit Do
            Loop
        End If

        If TutorialNumber = 1 Then
            Do
                If Not turorialRuns Then Exit Do
                While MsgBox("1.) Add vertices: " & vbCrLf & "Click on '[5] Add Vertex', then click on the grid with your left mouse button to add a vertex to your model." & vbCrLf & vbCrLf & "Press [ENTER] to continue and watch this action.", MsgBoxStyle.OkOnly + MsgBoxStyle.MsgBoxSetForeground, "Info:") <> MsgBoxResult.Ok : End While
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1500)
                clickOnButton(BtnAddVertex)
                moveMouseAbsoluteRightClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2, offset_Y + Me.ClientSize.Height / 2)
                If Not turorialRuns Then Exit Do Else Thread.Sleep(500)
                pressMouseLeft()
                releaseMouseLeft()
                If Not turorialRuns Then Exit Do Else Thread.Sleep(500)
                moveMouseAbsoluteRightClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2 - 100, System.Windows.Forms.Screen.PrimaryScreen.Bounds.Height / 2)
                If Not turorialRuns Then Exit Do Else Thread.Sleep(500)
                pressMouseLeft()
                releaseMouseLeft()
                If Not turorialRuns Then Exit Do Else Thread.Sleep(500)
                moveMouseAbsoluteRightClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2 - 100, System.Windows.Forms.Screen.PrimaryScreen.Bounds.Height / 2 - 100)
                If Not turorialRuns Then Exit Do Else Thread.Sleep(500)
                pressMouseLeft()
                releaseMouseLeft()
                If Not turorialRuns Then Exit Do Else Thread.Sleep(100)
                moveMouseAbsoluteRightClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2, System.Windows.Forms.Screen.PrimaryScreen.Bounds.Height / 2 - 100)
                Thread.Sleep(100)
                pressMouseLeft()
                releaseMouseLeft()
                If Not turorialRuns Then Exit Do Else Thread.Sleep(100)
                moveMouseAbsoluteRightClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2 + 100, System.Windows.Forms.Screen.PrimaryScreen.Bounds.Height / 2 - 100)
                Thread.Sleep(100)
                pressMouseLeft()
                releaseMouseLeft()
                Thread.Sleep(100)
                moveMouseAbsoluteRightClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2 + 100, System.Windows.Forms.Screen.PrimaryScreen.Bounds.Height / 2)
                If Not turorialRuns Then Exit Do Else Thread.Sleep(100)
                pressMouseLeft()
                releaseMouseLeft()
                Thread.Sleep(100)
                moveMouseAbsoluteRightClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2 + 100, System.Windows.Forms.Screen.PrimaryScreen.Bounds.Height / 2 + 100)
                Thread.Sleep(100)
                pressMouseLeft()
                releaseMouseLeft()
                If Not turorialRuns Then Exit Do Else Thread.Sleep(50)
                moveMouseAbsoluteRightClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2, System.Windows.Forms.Screen.PrimaryScreen.Bounds.Height / 2 + 100)
                Thread.Sleep(50)
                pressMouseLeft()
                releaseMouseLeft()
                Thread.Sleep(50)
                moveMouseAbsoluteRightClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2 - 100, System.Windows.Forms.Screen.PrimaryScreen.Bounds.Height / 2 + 100)
                Thread.Sleep(50)
                pressMouseLeft()
                releaseMouseLeft()
                If Not turorialRuns Then Exit Do Else Thread.Sleep(50)
                While MsgBox("2.) Add triangles: " & vbCrLf & "Click on '[6] Add Triangle', then click on the vertices with your left mouse button to create a triangle." & vbCrLf & vbCrLf & "Press [ENTER] to continue and watch this action.", MsgBoxStyle.OkOnly + MsgBoxStyle.MsgBoxSetForeground, "Info:") <> MsgBoxResult.Ok : End While
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1500)
                clickOnButton(BtnAddTriangle)

                moveMouseAbsoluteLeftClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2, offset_Y + Me.ClientSize.Height / 2)
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1000)
                moveMouseAbsoluteLeftClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2 - 100, offset_Y + Me.ClientSize.Height / 2)
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1000)
                moveMouseAbsoluteLeftClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2 - 100, offset_Y + Me.ClientSize.Height / 2 - 100)

                moveMouseAbsoluteLeftClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2 - 100, offset_Y + Me.ClientSize.Height / 2 - 100)
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1000)
                moveMouseAbsoluteLeftClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2, offset_Y + Me.ClientSize.Height / 2 - 100)
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1000)
                moveMouseAbsoluteLeftClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2, offset_Y + Me.ClientSize.Height / 2)
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1000)


                moveMouseAbsoluteLeftClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2, offset_Y + Me.ClientSize.Height / 2 - 100)
                Thread.Sleep(500)
                moveMouseAbsoluteLeftClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2 + 100, offset_Y + Me.ClientSize.Height / 2 - 100)
                If Not turorialRuns Then Exit Do Else Thread.Sleep(500)
                moveMouseAbsoluteLeftClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2, offset_Y + Me.ClientSize.Height / 2)

                moveMouseAbsoluteLeftClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2, offset_Y + Me.ClientSize.Height / 2)
                Thread.Sleep(500)
                moveMouseAbsoluteLeftClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2 + 100, offset_Y + Me.ClientSize.Height / 2)
                If Not turorialRuns Then Exit Do Else Thread.Sleep(500)
                moveMouseAbsoluteLeftClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2 + 100, offset_Y + Me.ClientSize.Height / 2 - 100)
                Thread.Sleep(500)



                moveMouseAbsoluteLeftClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2, offset_Y + Me.ClientSize.Height / 2 + 100)
                If Not turorialRuns Then Exit Do Else Thread.Sleep(500)
                moveMouseAbsoluteLeftClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2 - 100, offset_Y + Me.ClientSize.Height / 2 + 100)
                Thread.Sleep(500)
                moveMouseAbsoluteLeftClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2 - 100, offset_Y + Me.ClientSize.Height / 2)

                moveMouseAbsoluteLeftClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2 - 100, offset_Y + Me.ClientSize.Height / 2)
                If Not turorialRuns Then Exit Do Else Thread.Sleep(100)
                moveMouseAbsoluteLeftClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2, offset_Y + Me.ClientSize.Height / 2)
                Thread.Sleep(100)
                moveMouseAbsoluteLeftClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2, offset_Y + Me.ClientSize.Height / 2 + 100)
                Thread.Sleep(100)


                moveMouseAbsoluteLeftClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2, offset_Y + Me.ClientSize.Height / 2)
                Thread.Sleep(100)
                moveMouseAbsoluteLeftClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2 + 100, offset_Y + Me.ClientSize.Height / 2)
                If Not turorialRuns Then Exit Do Else Thread.Sleep(100)
                moveMouseAbsoluteLeftClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2, offset_Y + Me.ClientSize.Height / 2 + 100)

                moveMouseAbsoluteLeftClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2, offset_Y + Me.ClientSize.Height / 2 + 100)
                Thread.Sleep(100)
                moveMouseAbsoluteLeftClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2 + 100, offset_Y + Me.ClientSize.Height / 2 + 100)
                Thread.Sleep(100)
                moveMouseAbsoluteLeftClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2 + 100, offset_Y + Me.ClientSize.Height / 2)
                If Not turorialRuns Then Exit Do Else Thread.Sleep(100)
                clickOnButton(BtnAddTriangle)
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1000)
                While MsgBox("2.) Move vertices: " & vbCrLf & "  - Click on 'Triangle Mode', then select the 'Vertex Mode' entry to enable vertex-selection." & vbCrLf & "  - Hold the left mouse button and select all vertices." & vbCrLf & "  - Click on '[2] Move' to activate the movemode" & vbCrLf & "  - Hold the left mouse button and drag it over the screen to move the selection." & vbCrLf & vbCrLf & "Press [ENTER] to continue and watch this action.", MsgBoxStyle.OkOnly + MsgBoxStyle.MsgBoxSetForeground, "Info:") <> MsgBoxResult.Ok : End While
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1500)
                clickOnButton(BtnMode)
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1000)
                VerticesModeToolStripMenuItem.Select()
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1000)
                VerticesModeToolStripMenuItem.PerformClick()
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1000)
                moveMouseAbsoluteLeftClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2 - 200, offset_Y + Me.ClientSize.Height / 2 - 200)
                pressMouseLeft()
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1000)
                System.Windows.Forms.Cursor.Position = New Point(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2 + 200, offset_Y + Me.ClientSize.Height / 2 + 200)
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1000)
                releaseMouseLeft()
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1000)
                clickOnButton(BtnMove)
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1500)
                System.Windows.Forms.Cursor.Position = New Point(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2, offset_Y + Me.ClientSize.Height / 2)
                Thread.Sleep(30)
                moveMouseLeftSmooth(System.Windows.Forms.Cursor.Position.X, System.Windows.Forms.Cursor.Position.Y, System.Windows.Forms.Cursor.Position.X + 200, System.Windows.Forms.Cursor.Position.Y, 100)
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1500)
                While MsgBox("3.) Rotate vertices: " & vbCrLf & "  - Click on '[3] Rotate' to activate the rotatemode" & vbCrLf & "  - Hold the left mouse button and drag it over the screen to rotate the selection." & vbCrLf & vbCrLf & "Press [ENTER] to continue and watch this action.", MsgBoxStyle.OkOnly + MsgBoxStyle.MsgBoxSetForeground, "Info:") <> MsgBoxResult.Ok : End While
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1000)
                clickOnButton(BtnRotate)
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1500)
                System.Windows.Forms.Cursor.Position = New Point(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2, offset_Y + Me.ClientSize.Height / 2)
                Thread.Sleep(30)
                moveMouseLeftSmooth(System.Windows.Forms.Cursor.Position.X, System.Windows.Forms.Cursor.Position.Y, System.Windows.Forms.Cursor.Position.X + 170, System.Windows.Forms.Cursor.Position.Y, 100)
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1500)
                While MsgBox("4.) Scale vertices: " & vbCrLf & "  - Click on '[4] Scale' to activate the scalemode" & vbCrLf & "  - Hold the left mouse button and drag it over the screen to scale the selection." & vbCrLf & vbCrLf & "Press [ENTER] to continue and watch this action.", MsgBoxStyle.OkOnly + MsgBoxStyle.MsgBoxSetForeground, "Info:") <> MsgBoxResult.Ok : End While
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1000)
                clickOnButton(BtnScale)
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1500)
                System.Windows.Forms.Cursor.Position = New Point(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2, offset_Y + Me.ClientSize.Height / 2)
                Thread.Sleep(30)
                moveMouseLeftSmooth(System.Windows.Forms.Cursor.Position.X, System.Windows.Forms.Cursor.Position.Y, System.Windows.Forms.Cursor.Position.X + 50, System.Windows.Forms.Cursor.Position.Y, 100)
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1500)
                While MsgBox("5.) Colouring triangles: " & vbCrLf & "  - Click on 'Vertex Mode', then select the 'Triangle Mode' entry  and press 1 to enable triangle-selection." & vbCrLf & "  - Left-click the upper right triangle" & vbCrLf & "  - Click on 'Edit'->'Select Touching..' (Alt+E+T) to select the triangles which are touching the current selection." & vbCrLf & "  - Click on 'Show Colours' and pick a colour" & vbCrLf & vbCrLf & "Press [ENTER] to continue and watch this action.", MsgBoxStyle.OkOnly + MsgBoxStyle.MsgBoxSetForeground, "Info:") <> MsgBoxResult.Ok : End While
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1500)
                clickOnButton(BtnMode)
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1000)
                TriangleModeToolStripMenuItem.PerformClick()
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1000)
                SendKeys.SendWait("1")
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1000)
                moveMouseAbsoluteLeftClick(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width / 2 + 180, offset_Y + Me.ClientSize.Height / 2 - 16)
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1000)
                SendKeys.SendWait("%e")
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1000)
                SelectTouchingToolStripMenuItem.Select()
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1000)
                SendKeys.SendWait("t")
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1000)
                clickOnButton(BtnColours)
                C0.Select()
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1000)
                C0.PerformClick()
                If Not turorialRuns Then Exit Do Else Thread.Sleep(1000)
                While MsgBox("Thank you for watching this lesson." & vbCrLf & vbCrLf & "Press [ENTER] to continue.", MsgBoxStyle.OkOnly + MsgBoxStyle.MsgBoxSetForeground, "Info:") <> MsgBoxResult.Ok : End While
                Exit Do
            Loop
        End If

        resetGUI()
        releaseMouseRight()
        Thread.Sleep(1000)
        releaseMouseLeft()
        Thread.Sleep(1000)
        AbortCurrentLessonToolStripMenuItem.Enabled = False
        Me.BackgroundHowTo.CancelAsync()
        Return "[ready]"
    End Function


    Private Sub resetGUI()
        If BtnAddTriangle.Checked Then BtnAddTriangle.PerformClick() : Thread.Sleep(500)
        If BtnAddVertex.Checked Then BtnAddVertex.PerformClick() : Thread.Sleep(500)
        If Not ShowGridToolStripMenuItem.Checked Then
            ViewToolStripMenuItem.PerformClick()
            Thread.Sleep(500)
            ShowGridToolStripMenuItem.PerformClick()
            Thread.Sleep(500)
        End If
        If WithColourToolStripMenuItem.Checked Then
            EditToolStripMenuItem.PerformClick()
            Thread.Sleep(500)
            WithColourToolStripMenuItem.PerformClick()
            Thread.Sleep(500)
        End If
        abortPrimitiveMode()
        CircleSegments.Text = 16
        OvalSegments.Text = 16
        newPattern()
        BtnMode.PerformClick()
        Thread.Sleep(500)
        VerticesModeToolStripMenuItem.PerformClick()
        Thread.Sleep(500)
        BtnSelect.PerformClick()
        GBImage.Visible = False
        ImageToolStripMenuItem.Checked = False
        GBSnap.Visible = False
        SnappingToolStripMenuItem.Checked = False
        toffsetX = 0
        toffsetY = 0
        offsetX = 0
        offsetY = 0
        zoomfaktor = 1
        zoomlevel = 0
        LblDebug.Text = "Zoom: 100%"
        Me.Refresh()
    End Sub

    Private Sub AbortCurrentLessonToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles AbortCurrentLessonToolStripMenuItem.Click
        turorialRuns = False
    End Sub

    Public Sub clickOnButton(ByRef tsb As ToolStripButton)
        System.Windows.Forms.Cursor.Position = New Point(tsb.Bounds.Location.X + tsb.Size.Width / 2, tsb.Size.Height / 2 + offset_Y + MenuStrip1.Height) 'Me.ClientSize.Height + System.Windows.Forms.Screen.PrimaryScreen.Bounds.Height)
        Thread.Sleep(1000)
        tsb.PerformClick()
        Thread.Sleep(1000)
    End Sub

    Public Sub clickOnButton(ByRef tsb As ToolStripDropDownButton)
        System.Windows.Forms.Cursor.Position = New Point(tsb.Bounds.Location.X + tsb.Size.Width / 2, tsb.Size.Height / 2 + offset_Y + MenuStrip1.Height) 'Me.ClientSize.Height + System.Windows.Forms.Screen.PrimaryScreen.Bounds.Height)
        Thread.Sleep(1000)
    End Sub


    Public Sub getMouseCoords()
        cur_x = System.Windows.Forms.Cursor.Position.X * 65535 / System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width
        cur_y = System.Windows.Forms.Cursor.Position.Y * 65535 / System.Windows.Forms.Screen.PrimaryScreen.Bounds.Height
    End Sub

    Public Sub moveMouseAbsoluteLeftPressed(ByRef dx As Single, ByRef dy As Single)
        dest_x = dx * 65535 / System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width
        dest_y = dy * 65535 / System.Windows.Forms.Screen.PrimaryScreen.Bounds.Height
        mouse_event(MOUSEEVENTF_ABSOLUTE + MOUSEEVENTF_LEFTDOWN + MOUSEEVENTF_MOVE, dest_x, dest_y, 0, 0)
    End Sub

    Public Sub moveMouseAbsoluteRightPressed(ByRef dx As Single, ByRef dy As Single)
        dest_x = dx * 65535 / System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width
        dest_y = dy * 65535 / System.Windows.Forms.Screen.PrimaryScreen.Bounds.Height
        mouse_event(MOUSEEVENTF_ABSOLUTE + MOUSEEVENTF_RIGHTDOWN + MOUSEEVENTF_MOVE, dest_x, dest_y, 0, 0)
    End Sub

    Public Sub pressMouseRight()
        getMouseCoords()
        mouse_event(MOUSEEVENTF_ABSOLUTE + MOUSEEVENTF_RIGHTDOWN, cur_x, cur_y, 0, 0)
    End Sub

    Public Sub pressMouseLeft()
        getMouseCoords()
        mouse_event(MOUSEEVENTF_ABSOLUTE + MOUSEEVENTF_LEFTDOWN, cur_x, cur_y, 0, 0)
    End Sub

    Public Sub releaseMouseRight()
        getMouseCoords()
        mouse_event(MOUSEEVENTF_ABSOLUTE + MOUSEEVENTF_RIGHTUP, cur_x, cur_y, 0, 0)
    End Sub

    Public Sub releaseMouseLeft()
        getMouseCoords()
        mouse_event(MOUSEEVENTF_ABSOLUTE + MOUSEEVENTF_LEFTUP, cur_x, cur_y, 0, 0)
    End Sub

    Public Sub moveMouseRightSmooth(ByVal sx As Single, ByVal sy As Single, ByVal dx As Single, ByVal dy As Single, ByVal t As Integer)
        pressMouseRight()
        Thread.Sleep(30)
        Dim cx, cy As Single
        cx = (dx - sx) / t
        cy = (dy - sy) / t
        For ct As Integer = 1 To t
            If Not turorialRuns Then Exit For
            sx += cx
            sy += cy
            dest_x = sx * 65535 / System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width
            dest_y = sy * 65535 / System.Windows.Forms.Screen.PrimaryScreen.Bounds.Height
            mouse_event(MOUSEEVENTF_ABSOLUTE + MOUSEEVENTF_MOVE, dest_x, dest_y, 0, 0)
            Thread.Sleep(1)
        Next
        releaseMouseRight()
    End Sub

    Public Sub moveMouseLeftSmooth(ByVal sx As Single, ByVal sy As Single, ByVal dx As Single, ByVal dy As Single, ByVal t As Integer)
        pressMouseLeft()
        Thread.Sleep(30)
        Dim cx, cy As Single
        cx = (dx - sx) / t
        cy = (dy - sy) / t
        For ct As Integer = 1 To t
            If Not turorialRuns Then Exit For
            sx += cx
            sy += cy
            dest_x = sx * 65535 / System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width
            dest_y = sy * 65535 / System.Windows.Forms.Screen.PrimaryScreen.Bounds.Height
            mouse_event(MOUSEEVENTF_ABSOLUTE + MOUSEEVENTF_MOVE, dest_x, dest_y, 0, 0)
            Thread.Sleep(1)
        Next
        releaseMouseLeft()
    End Sub

    Public Sub moveMouseAbsoluteLeftClick(ByVal dx As Single, ByVal dy As Single)
        dest_x = dx * 65535 / System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width
        dest_y = dy * 65535 / System.Windows.Forms.Screen.PrimaryScreen.Bounds.Height
        mouse_event(MOUSEEVENTF_ABSOLUTE + MOUSEEVENTF_MOVE + MOUSEEVENTF_LEFTDOWN + MOUSEEVENTF_LEFTUP, dest_x, dest_y, 0, 0)
    End Sub

    Public Sub moveMouseAbsoluteRightClick(ByVal dx As Single, ByVal dy As Single)
        dest_x = dx * 65535 / System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width
        dest_y = dy * 65535 / System.Windows.Forms.Screen.PrimaryScreen.Bounds.Height
        mouse_event(MOUSEEVENTF_ABSOLUTE + MOUSEEVENTF_MOVE + MOUSEEVENTF_RIGHTDOWN + MOUSEEVENTF_RIGHTUP, dest_x, dest_y, 0, 0)
    End Sub

#End Region

    Private Sub ToAverageToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToAverageToolStripMenuItem.Click
        If SelectedVertices.Count > 1 Then
            Dim sumX As Integer = 0
            Dim sumY As Integer = 0
            For Each vert As Vertex In SelectedVertices
                sumX += vert.X
                sumY += vert.Y
            Next
            sumX /= SelectedVertices.Count
            sumY /= SelectedVertices.Count
            For Each vert As Vertex In SelectedVertices
                vert.X = sumX
                vert.Y = sumY
            Next
            cleanupMergedTriangles()
            cleanupMergedVertices()
            addHistory()
            Me.Refresh()
        End If
    End Sub

    Private Sub ToLastSelectedToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToLastSelectedToolStripMenuItem.Click
        If SelectedVertices.Count > 1 Then
            For Each vert As Vertex In SelectedVertices
                vert.X = LLast(SelectedVertices).X
                vert.Y = LLast(SelectedVertices).Y
            Next
            cleanupMergedTriangles()
            cleanupMergedVertices()
            addHistory()
            Me.Refresh()
        End If
    End Sub

    Private Sub CSGAddToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CSGAddToolStripMenuItem.Click
        Dim Anzahl_Alt As Integer = SelectedTriangles.Count
nextTri:
        For Each tri1 As Triangle In SelectedTriangles
            For Each tri2 As Triangle In SelectedTriangles
                If Not tri1.Equals(tri2) Then
                    If CSG2Triangles(tri1, tri2) Then
                        SelectedTriangles.Remove(tri1)
                        SelectedTriangles.Remove(tri2)
                        removeTriangle(tri1)
                        removeTriangle(tri2)
                        cleanupMergedVertices()
                        If SelectedTriangles.Count > Anzahl_Alt * 4.1 Then
                            cleanupMergedTriangles()
                            cleanupMergedVertices()
                            Me.Refresh()
                            Exit Sub
                        End If
                        GoTo nextTri
                    End If
                End If
            Next
        Next
        cleanupMergedVertices()
        cleanupMergedTriangles()
        addHistory()
        Me.Refresh()
    End Sub

    Private Sub CSGIntersectionPointsToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CSGIntersectionPointsToolStripMenuItem.Click
        Dim Anzahl_Alt As Integer = SelectedTriangles.Count
        For trii1 As Integer = 0 To SelectedTriangles.Count - 1
            Dim tri1 As Triangle = SelectedTriangles(trii1)
            For trii2 As Integer = trii1 + 1 To SelectedTriangles.Count - 1
                Dim tri2 As Triangle = SelectedTriangles(trii2)
                CSG2TrianglesIntersectionsOnly(tri1, tri2)
            Next
        Next
        cleanupMergedVertices()
        addHistory()
        Me.Refresh()
    End Sub

    Private Sub MainForm_Paint(ByVal sender As Object, ByVal e As System.Windows.Forms.PaintEventArgs) Handles Me.Paint
        Dim wa As Stopwatch = Stopwatch.StartNew()
        e.Graphics.SmoothingMode = Drawing2D.SmoothingMode.HighSpeed
        If kamera Then
            offsetX = toffsetX + (getCursorpositionX() - klickX) / zoomfaktor
            offsetY = toffsetY + (getCursorpositionY() - klickY) / zoomfaktor
        End If



        Dim absOffsetX As Integer = Me.offsetX * zoomfaktor + CType(Me.ClientSize.Width / 2, Integer)
        Dim absOffsetY As Integer = Me.offsetY * zoomfaktor + CType(Me.ClientSize.Height / 2, Integer)

        ' Bild
        Try
            e.Graphics.DrawImage(hintergrundbild, CType(absOffsetX - (hintergrundbild.Width / 2 * imgScale - imgOffsetX) * zoomfaktor, Integer), CType(absOffsetY - (hintergrundbild.Height / 2 * imgScale - imgOffsetY) * zoomfaktor, Integer), hintergrundbild.Width * zoomfaktor * imgScale, hintergrundbild.Height * zoomfaktor * imgScale)
        Catch ex As Exception
        End Try

        ' Raster, Ursprung
        If raster Then
            Dim scale As Single
            If zoomlevel > 80 Then scale = Me.rasterSnap * zoomfaktor : GoTo raster_zechnen
            If zoomlevel > 0 Then scale = Me.rasterSnap * zoomfaktor * 10 : GoTo raster_zechnen
            If zoomlevel > -10 Then scale = Me.rasterSnap * zoomfaktor * 100 : GoTo raster_zechnen
            If zoomlevel > -20 Then scale = Me.rasterSnap * zoomfaktor * 1000 : GoTo raster_zechnen
            If zoomlevel > -30 Then scale = Me.rasterSnap * zoomfaktor * 10000 : GoTo raster_zechnen
            scale = Me.rasterSnap * zoomfaktor * 100000
raster_zechnen:
            Dim relOffsetX As Integer
            Dim relOffsetY As Integer
            relOffsetX = absOffsetX Mod scale
            relOffsetY = absOffsetY Mod scale
            For x As Single = relOffsetX To Me.ClientSize.Width Step scale
                e.Graphics.DrawLine(Pens.DarkGreen, x, 0, x, Me.ClientSize.Height)
            Next
            For y As Single = relOffsetY To Me.ClientSize.Height Step scale
                e.Graphics.DrawLine(Pens.DarkGreen, 0, y, Me.ClientSize.Width, y)
            Next
            relOffsetX = absOffsetX Mod (scale * 10)
            relOffsetY = absOffsetY Mod (scale * 10)
            For x As Single = relOffsetX To Me.ClientSize.Width Step scale * 10
                e.Graphics.DrawLine(Pens.LightSlateGray, x, 0, x, Me.ClientSize.Height)
            Next
            For y As Single = relOffsetY To Me.ClientSize.Height Step scale * 10
                e.Graphics.DrawLine(Pens.LightSlateGray, 0, y, Me.ClientSize.Width, y)
            Next
        End If
        e.Graphics.DrawLine(Pens.LightGreen, absOffsetX, 0, absOffsetX, Me.ClientSize.Height)
        e.Graphics.DrawLine(Pens.LightGreen, 0, absOffsetY, Me.ClientSize.Width, absOffsetY)

        Dim vabsOffsetX As Integer
        Dim vabsOffsetY As Integer
        If movemode Then
            If Not Control.ModifierKeys = 131072 Then vabsOffsetX = Fix((getCursorpositionX() - klickX) / zoomfaktor / moveSnap) * moveSnap
            If Not Control.ModifierKeys = Keys.Shift Then vabsOffsetY = Fix((getCursorpositionY() - klickY) / zoomfaktor / moveSnap) * moveSnap
            For Each vert As Vertex In SelectedVertices
                vert.X += vabsOffsetX
                vert.Y -= vabsOffsetY
            Next
        ElseIf rotatemode Then
            Dim angleToRotate As Double = Fix((-(getCursorpositionX() - klickX) - (getCursorpositionY() - klickY)) / 100 / (rotateSnap * Math.PI / 180)) * (rotateSnap * Math.PI / 180)
            For Each vert As Vertex In SelectedVertices
                vert.X = temp_center.X + vert.distanceFrom * Math.Cos(vert.angleFrom + angleToRotate)
                vert.Y = temp_center.Y + vert.distanceFrom * Math.Sin(vert.angleFrom + angleToRotate)
                If Single.IsNaN(vert.X) OrElse Single.IsNaN(vert.Y) Then
                    vert.X = vert.oldX
                    vert.Y = vert.oldY
                End If
            Next
        ElseIf scalemode Then
            Dim factorToScale As Double = 1 + Fix((getCursorpositionX() - klickX) / 100 / scaleSnap) * scaleSnap
            If factorToScale < 0.1 Then factorToScale = 0.1
            If factorToScale > 2 Then factorToScale = 2
            Dim pf(4) As PointF
            pf(0).X = klickX - 100
            pf(0).Y = klickY - 5
            pf(1).X = klickX + 100
            pf(1).Y = klickY - 5
            pf(2).X = klickX + 100
            pf(2).Y = klickY + 5
            pf(3).X = klickX - 100
            pf(3).Y = klickY + 5
            pf(4).X = klickX - 100
            pf(4).Y = klickY - 5
            e.Graphics.FillPolygon(Brushes.WhiteSmoke, pf)
            e.Graphics.DrawString("0%                                    100%                                200%", New Font("Arial", 8, FontStyle.Regular, GraphicsUnit.Pixel), Brushes.Azure, klickX - 100, klickY + 5)
            For x As Integer = -100 To 100 Step 10
                e.Graphics.DrawLine(Pens.DarkGray, klickX + x, klickY - 5, klickX + x, klickY + 5)
            Next x
            e.Graphics.DrawString(Fix(factorToScale * 100) & "%", New Font("Arial", 8, FontStyle.Bold, GraphicsUnit.Pixel), Brushes.Red, klickX + 5, klickY - 3)
            e.Graphics.DrawLine(Pens.Black, klickX, klickY - 5, klickX, klickY + 5)
            For Each vert As Vertex In SelectedVertices
                If Not Control.ModifierKeys = Keys.Control Then
                    vert.X = temp_center.X + factorToScale * vert.distanceFrom * Math.Cos(vert.angleFrom)
                Else
                    vert.X = vert.oldX
                End If
                If Not Control.ModifierKeys = Keys.Shift Then
                    vert.Y = temp_center.Y + factorToScale * vert.distanceFrom * Math.Sin(vert.angleFrom)
                Else
                    vert.Y = vert.oldY
                End If
                If Single.IsNaN(vert.X) OrElse Single.IsNaN(vert.Y) Then
                    vert.X = vert.oldX
                    vert.Y = vert.oldY
                End If
            Next
        End If

        ' Filled Triangles
        If BtnColours.Checked Then
            Dim pf(3) As PointF
            For Each tri As Triangle In Triangles
                If tri.myColour <> Nothing Then
                    Dim b As New SolidBrush(tri.myColour)
                    pf(0).X = absOffsetX + tri.vertexA.X * zoomfaktor
                    pf(0).Y = absOffsetY - tri.vertexA.Y * zoomfaktor
                    pf(1).X = absOffsetX + tri.vertexB.X * zoomfaktor
                    pf(1).Y = absOffsetY - tri.vertexB.Y * zoomfaktor
                    pf(2).X = absOffsetX + tri.vertexC.X * zoomfaktor
                    pf(2).Y = absOffsetY - tri.vertexC.Y * zoomfaktor
                    pf(3).X = absOffsetX + tri.vertexA.X * zoomfaktor
                    pf(3).Y = absOffsetY - tri.vertexA.Y * zoomfaktor
                    e.Graphics.FillPolygon(b, pf)
                End If
            Next
        End If

        ' Triangles
        Dim pf2(3) As PointF
        For Each tri As Triangle In Triangles
            If Not tri.selected Then
                If tri.myColourNumber <> 15 OrElse Not BtnColours.Checked Then
                    'e.Graphics.DrawLine(Pens.WhiteSmoke, absOffsetX + tri.vertexA.X * zoomfaktor, absOffsetY - tri.vertexA.Y * zoomfaktor, absOffsetX + tri.vertexB.X * zoomfaktor, absOffsetY - tri.vertexB.Y * zoomfaktor)
                    'e.Graphics.DrawLine(Pens.WhiteSmoke, absOffsetX + tri.vertexB.X * zoomfaktor, absOffsetY - tri.vertexB.Y * zoomfaktor, absOffsetX + tri.vertexC.X * zoomfaktor, absOffsetY - tri.vertexC.Y * zoomfaktor)
                    'e.Graphics.DrawLine(Pens.WhiteSmoke, absOffsetX + tri.vertexC.X * zoomfaktor, absOffsetY - tri.vertexC.Y * zoomfaktor, absOffsetX + tri.vertexA.X * zoomfaktor, absOffsetY - tri.vertexA.Y * zoomfaktor)
                    pf2(0).X = absOffsetX + tri.vertexA.X * zoomfaktor
                    pf2(0).Y = absOffsetY - tri.vertexA.Y * zoomfaktor
                    pf2(1).X = absOffsetX + tri.vertexB.X * zoomfaktor
                    pf2(1).Y = absOffsetY - tri.vertexB.Y * zoomfaktor
                    pf2(2).X = absOffsetX + tri.vertexC.X * zoomfaktor
                    pf2(2).Y = absOffsetY - tri.vertexC.Y * zoomfaktor
                    pf2(3).X = absOffsetX + tri.vertexA.X * zoomfaktor
                    pf2(3).Y = absOffsetY - tri.vertexA.Y * zoomfaktor
                    e.Graphics.DrawPolygon(Pens.WhiteSmoke, pf2)
                Else
                    pf2(0).X = absOffsetX + tri.vertexA.X * zoomfaktor
                    pf2(0).Y = absOffsetY - tri.vertexA.Y * zoomfaktor
                    pf2(1).X = absOffsetX + tri.vertexB.X * zoomfaktor
                    pf2(1).Y = absOffsetY - tri.vertexB.Y * zoomfaktor
                    pf2(2).X = absOffsetX + tri.vertexC.X * zoomfaktor
                    pf2(2).Y = absOffsetY - tri.vertexC.Y * zoomfaktor
                    pf2(3).X = absOffsetX + tri.vertexA.X * zoomfaktor
                    pf2(3).Y = absOffsetY - tri.vertexA.Y * zoomfaktor
                    e.Graphics.DrawPolygon(Pens.Black, pf2)
                End If
            End If
        Next
        If bearbeitungsobjekt <> "Vertex" Then
            For Each tri As Triangle In Triangles
                If tri.selected Then
                    pf2(0).X = absOffsetX + tri.vertexA.X * zoomfaktor
                    pf2(0).Y = absOffsetY - tri.vertexA.Y * zoomfaktor
                    pf2(1).X = absOffsetX + tri.vertexB.X * zoomfaktor
                    pf2(1).Y = absOffsetY - tri.vertexB.Y * zoomfaktor
                    pf2(2).X = absOffsetX + tri.vertexC.X * zoomfaktor
                    pf2(2).Y = absOffsetY - tri.vertexC.Y * zoomfaktor
                    pf2(3).X = absOffsetX + tri.vertexA.X * zoomfaktor
                    pf2(3).Y = absOffsetY - tri.vertexA.Y * zoomfaktor
                    e.Graphics.DrawPolygon(Pens.Red, pf2)
                End If
            Next
        End If

        If dreiecksmodus > 0 Then
            e.Graphics.DrawLine(Pens.PowderBlue, absOffsetX + lastPointX * zoomfaktor, absOffsetY - lastPointY * zoomfaktor, getCursorpositionX() - 3, getCursorpositionY)
            If dreiecksmodus = 2 Then e.Graphics.DrawLine(Pens.PowderBlue, absOffsetX + LLast(Triangles).vertexA.X * zoomfaktor, absOffsetY - LLast(Triangles).vertexA.Y * zoomfaktor, getCursorpositionX() - 3, getCursorpositionY)
        End If

        ' Vertices

        Dim vs As New List(Of RectangleF) With {.Capacity = SelectedVertices.Count}
        Dim v As New List(Of RectangleF) With {.Capacity = Vertices.Count}
        For Each vert As Vertex In Vertices
            If vert.selected AndAlso bearbeitungsobjekt = "Vertex" Then
                'e.Graphics.FillRectangle(Brushes.Red, New RectangleF(absOffsetX + vert.X * zoomfaktor - (punktgroesse / 2), absOffsetY - vert.Y * zoomfaktor - (punktgroesse / 2), punktgroesse, punktgroesse))
                vs.Add(New RectangleF(absOffsetX + vert.X * zoomfaktor - (punktgroesse / 2), absOffsetY - vert.Y * zoomfaktor - (punktgroesse / 2), punktgroesse, punktgroesse))
            Else
                'e.Graphics.FillRectangle(Brushes.Yellow, New RectangleF(absOffsetX + vert.X * zoomfaktor - (punktgroesse / 2), absOffsetY - vert.Y * zoomfaktor - (punktgroesse / 2), punktgroesse, punktgroesse))
                v.Add(New RectangleF(absOffsetX + vert.X * zoomfaktor - (punktgroesse / 2), absOffsetY - vert.Y * zoomfaktor - (punktgroesse / 2), punktgroesse, punktgroesse))
            End If
        Next
        If vs.Count > 0 Then e.Graphics.FillRectangles(Brushes.Red, vs.ToArray)
        If v.Count > 0 Then e.Graphics.FillRectangles(Brushes.Yellow, v.ToArray)

        If movemode Then
            For Each vert As Vertex In SelectedVertices
                vert.X -= vabsOffsetX
                vert.Y += vabsOffsetY
            Next
        ElseIf rotatemode OrElse scalemode Then
            For Each vert As Vertex In SelectedVertices
                vert.X = vert.oldX
                vert.Y = vert.oldY
            Next
        End If

        ' Auswahl
        If auswahl AndAlso bearbeitungsobjekt <> "Primitive" Then
            Dim tX, tY, tW, tH As Integer
            tX = Math.Min(klickX, getCursorpositionX)
            tY = Math.Min(klickY, getCursorpositionY)
            tW = Math.Abs(getCursorpositionX() - klickX)
            tH = Math.Abs(getCursorpositionY() - klickY)
            e.Graphics.DrawRectangle(Pens.Beige, tX, tY, tW, tH)
        End If

        ' Add Vertex
        If BtnAddVertex.Checked Then
            Dim v1 As New Vertex(Math.Round(getXcoordinate(getCursorpositionX() - 3) / Me.moveSnap) * Me.moveSnap, Math.Round(getYcoordinate(getCursorpositionY()) / Me.moveSnap) * Me.moveSnap, False, False)
            'Dim v1 As New Vertex(Math.Round(getXcoordinate(getCursorpositionX() - 3)), Math.Round(getYcoordinate(getCursorpositionY())), False, False)
            e.Graphics.FillRectangle(Brushes.Red, New RectangleF(absOffsetX + v1.X * zoomfaktor - (punktgroesse / 2), absOffsetY - v1.Y * zoomfaktor - (punktgroesse / 2), punktgroesse, punktgroesse))
        End If

        ' Primitive Modes
        If primitiveMode > 0 Then
            Select Case primitiveMode
                Case 1
                    e.Graphics.DrawString("Set primitive origin", New Font("Arial", 16, FontStyle.Regular, GraphicsUnit.Pixel), Brushes.Black, 12, 32)
                    e.Graphics.DrawString("Set primitive origin", New Font("Arial", 16, FontStyle.Regular, GraphicsUnit.Pixel), Brushes.Azure, 10, 30)
                    primitiveCenter = New Vertex(Math.Round(getXcoordinate(getCursorpositionX() - 3) / Me.moveSnap) * Me.moveSnap, Math.Round(getYcoordinate(getCursorpositionY()) / Me.moveSnap) * Me.moveSnap, False, False)
                    e.Graphics.FillRectangle(Brushes.DarkRed, New RectangleF(absOffsetX + primitiveCenter.X * zoomfaktor - 1, absOffsetY - primitiveCenter.Y * zoomfaktor - 16, 2, 32))
                    e.Graphics.FillRectangle(Brushes.DarkRed, New RectangleF(absOffsetX + primitiveCenter.X * zoomfaktor - 16, absOffsetY - primitiveCenter.Y * zoomfaktor - 1, 32, 2))
                    Exit Select
                Case 2
                    e.Graphics.DrawString("Set primitive height or size", New Font("Arial", 16, FontStyle.Regular, GraphicsUnit.Pixel), Brushes.Black, 12, 32)
                    e.Graphics.DrawString("Set primitive height or size", New Font("Arial", 16, FontStyle.Regular, GraphicsUnit.Pixel), Brushes.Azure, 10, 30)
                    e.Graphics.FillRectangle(Brushes.DarkRed, New RectangleF(absOffsetX + primitiveCenter.X * zoomfaktor - 1, absOffsetY - primitiveCenter.Y * zoomfaktor - 16, 2, 32))
                    e.Graphics.FillRectangle(Brushes.DarkRed, New RectangleF(absOffsetX + primitiveCenter.X * zoomfaktor - 16, absOffsetY - primitiveCenter.Y * zoomfaktor - 1, 32, 2))
                    primitiveHeight = Fix(getYcoordinate(getCursorpositionY()) - primitiveCenter.Y)
                    Select Case primitiveObject
                        Case Is < 3
                            Dim v1, v2, v3 As Vertex
                            v1 = New Vertex(primitiveCenter.X, primitiveCenter.Y + primitiveHeight, False, False)
                            v2 = New Vertex(primitiveCenter.X + primitiveHeight, primitiveCenter.Y - (primitiveHeight / 2), False, False)
                            v3 = New Vertex(primitiveCenter.X - primitiveHeight, primitiveCenter.Y - (primitiveHeight / 2), False, False)
                            e.Graphics.FillRectangle(Brushes.Red, New RectangleF(absOffsetX + v1.X * zoomfaktor - (punktgroesse / 2), absOffsetY - v1.Y * zoomfaktor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.FillRectangle(Brushes.Red, New RectangleF(absOffsetX + v2.X * zoomfaktor - (punktgroesse / 2), absOffsetY - v2.Y * zoomfaktor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.FillRectangle(Brushes.Red, New RectangleF(absOffsetX + v3.X * zoomfaktor - (punktgroesse / 2), absOffsetY - v3.Y * zoomfaktor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.DrawLine(Pens.Red, absOffsetX + v1.X * zoomfaktor, absOffsetY - v1.Y * zoomfaktor, absOffsetX + v2.X * zoomfaktor, absOffsetY - v2.Y * zoomfaktor)
                            e.Graphics.DrawLine(Pens.Red, absOffsetX + v2.X * zoomfaktor, absOffsetY - v2.Y * zoomfaktor, absOffsetX + v3.X * zoomfaktor, absOffsetY - v3.Y * zoomfaktor)
                            e.Graphics.DrawLine(Pens.Red, absOffsetX + v3.X * zoomfaktor, absOffsetY - v3.Y * zoomfaktor, absOffsetX + v1.X * zoomfaktor, absOffsetY - v1.Y * zoomfaktor)
                            Exit Select
                        Case Is < 6
                            Dim v1, v2, v3, v4 As Vertex
                            v1 = New Vertex(primitiveCenter.X + primitiveHeight, primitiveCenter.Y + primitiveHeight, False, False)
                            v2 = New Vertex(primitiveCenter.X - primitiveHeight, primitiveCenter.Y + primitiveHeight, False, False)
                            v3 = New Vertex(primitiveCenter.X - primitiveHeight, primitiveCenter.Y - primitiveHeight, False, False)
                            v4 = New Vertex(primitiveCenter.X + primitiveHeight, primitiveCenter.Y - primitiveHeight, False, False)
                            e.Graphics.FillRectangle(Brushes.Red, New RectangleF(absOffsetX + v1.X * zoomfaktor - (punktgroesse / 2), absOffsetY - v1.Y * zoomfaktor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.FillRectangle(Brushes.Red, New RectangleF(absOffsetX + v2.X * zoomfaktor - (punktgroesse / 2), absOffsetY - v2.Y * zoomfaktor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.FillRectangle(Brushes.Red, New RectangleF(absOffsetX + v3.X * zoomfaktor - (punktgroesse / 2), absOffsetY - v3.Y * zoomfaktor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.FillRectangle(Brushes.Red, New RectangleF(absOffsetX + v4.X * zoomfaktor - (punktgroesse / 2), absOffsetY - v4.Y * zoomfaktor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.DrawLine(Pens.Red, absOffsetX + v1.X * zoomfaktor, absOffsetY - v1.Y * zoomfaktor, absOffsetX + v2.X * zoomfaktor, absOffsetY - v2.Y * zoomfaktor)
                            e.Graphics.DrawLine(Pens.Red, absOffsetX + v2.X * zoomfaktor, absOffsetY - v2.Y * zoomfaktor, absOffsetX + v3.X * zoomfaktor, absOffsetY - v3.Y * zoomfaktor)
                            e.Graphics.DrawLine(Pens.Red, absOffsetX + v3.X * zoomfaktor, absOffsetY - v3.Y * zoomfaktor, absOffsetX + v4.X * zoomfaktor, absOffsetY - v4.Y * zoomfaktor)
                            e.Graphics.DrawLine(Pens.Red, absOffsetX + v4.X * zoomfaktor, absOffsetY - v4.Y * zoomfaktor, absOffsetX + v1.X * zoomfaktor, absOffsetY - v1.Y * zoomfaktor)
                            e.Graphics.DrawLine(Pens.Red, absOffsetX + v1.X * zoomfaktor, absOffsetY - v1.Y * zoomfaktor, absOffsetX + v3.X * zoomfaktor, absOffsetY - v3.Y * zoomfaktor)
                            Exit Select
                        Case Is < 9
                            primitiveHeight = Fix(Math.Sqrt((getYcoordinate(getCursorpositionY()) - primitiveCenter.Y) ^ 2 + (getXcoordinate(getCursorpositionX()) - primitiveCenter.X) ^ 2))
                            Dim vcenter As Vertex
                            Dim vouter(segments_circle) As Vertex
                            Dim counter As Integer = 0
                            vcenter = New Vertex(primitiveCenter.X, primitiveCenter.Y, False, False)
                            For a As Double = 0 To Math.PI * 2 Step Math.PI * 2 / segments_circle
                                vouter(counter) = New Vertex(primitiveCenter.X + Math.Cos(a) * primitiveHeight, primitiveCenter.Y + Math.Sin(a) * primitiveHeight, False, False)
                                counter += 1
                            Next
                            e.Graphics.FillRectangle(Brushes.Red, New RectangleF(absOffsetX + vouter(0).X * zoomfaktor - (punktgroesse / 2), absOffsetY - vouter(0).Y * zoomfaktor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.DrawLine(Pens.Red, absOffsetX + vouter(0).X * zoomfaktor, absOffsetY - vouter(0).Y * zoomfaktor, absOffsetX + vouter(segments_circle - 1).X * zoomfaktor, absOffsetY - vouter(segments_circle - 1).Y * zoomfaktor)
                            e.Graphics.DrawLine(Pens.Red, absOffsetX + vouter(0).X * zoomfaktor, absOffsetY - vouter(0).Y * zoomfaktor, absOffsetX + vcenter.X * zoomfaktor, absOffsetY - vcenter.Y * zoomfaktor)
                            For counter = 1 To segments_circle - 1
                                e.Graphics.DrawLine(Pens.Red, absOffsetX + vouter(counter).X * zoomfaktor, absOffsetY - vouter(counter).Y * zoomfaktor, absOffsetX + vouter(counter - 1).X * zoomfaktor, absOffsetY - vouter(counter - 1).Y * zoomfaktor)
                                e.Graphics.DrawLine(Pens.Red, absOffsetX + vouter(counter).X * zoomfaktor, absOffsetY - vouter(counter).Y * zoomfaktor, absOffsetX + vcenter.X * zoomfaktor, absOffsetY - vcenter.Y * zoomfaktor)
                                e.Graphics.FillRectangle(Brushes.Red, New RectangleF(absOffsetX + vouter(counter).X * zoomfaktor - (punktgroesse / 2), absOffsetY - vouter(counter).Y * zoomfaktor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            Next
                            e.Graphics.FillRectangle(Brushes.Red, New RectangleF(absOffsetX + vcenter.X * zoomfaktor - (punktgroesse / 2), absOffsetY - vcenter.Y * zoomfaktor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            Exit Select
                        Case Is < 12
                            primitiveHeight = Fix(Math.Sqrt((getYcoordinate(getCursorpositionY()) - primitiveCenter.Y) ^ 2 + (getXcoordinate(getCursorpositionX()) - primitiveCenter.X) ^ 2))
                            Dim vcenter As Vertex
                            Dim vouter(segments_oval) As Vertex
                            Dim counter As Integer = 0
                            vcenter = New Vertex(primitiveCenter.X, primitiveCenter.Y, False, False)
                            For a As Double = 0 To Math.PI * 2 Step Math.PI * 2 / segments_oval
                                vouter(counter) = New Vertex(primitiveCenter.X + Math.Cos(a) * primitiveHeight, primitiveCenter.Y + Math.Sin(a) * primitiveHeight, False, False)
                                counter += 1
                            Next
                            e.Graphics.FillRectangle(Brushes.Red, New RectangleF(absOffsetX + vouter(0).X * zoomfaktor - (punktgroesse / 2), absOffsetY - vouter(0).Y * zoomfaktor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.DrawLine(Pens.Red, absOffsetX + vouter(0).X * zoomfaktor, absOffsetY - vouter(0).Y * zoomfaktor, absOffsetX + vouter(segments_oval - 1).X * zoomfaktor, absOffsetY - vouter(segments_oval - 1).Y * zoomfaktor)
                            e.Graphics.DrawLine(Pens.Red, absOffsetX + vouter(0).X * zoomfaktor, absOffsetY - vouter(0).Y * zoomfaktor, absOffsetX + vcenter.X * zoomfaktor, absOffsetY - vcenter.Y * zoomfaktor)
                            For counter = 1 To segments_oval - 1
                                e.Graphics.DrawLine(Pens.Red, absOffsetX + vouter(counter).X * zoomfaktor, absOffsetY - vouter(counter).Y * zoomfaktor, absOffsetX + vouter(counter - 1).X * zoomfaktor, absOffsetY - vouter(counter - 1).Y * zoomfaktor)
                                e.Graphics.DrawLine(Pens.Red, absOffsetX + vouter(counter).X * zoomfaktor, absOffsetY - vouter(counter).Y * zoomfaktor, absOffsetX + vcenter.X * zoomfaktor, absOffsetY - vcenter.Y * zoomfaktor)
                                e.Graphics.FillRectangle(Brushes.Red, New RectangleF(absOffsetX + vouter(counter).X * zoomfaktor - (punktgroesse / 2), absOffsetY - vouter(counter).Y * zoomfaktor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            Next
                            e.Graphics.FillRectangle(Brushes.Red, New RectangleF(absOffsetX + vcenter.X * zoomfaktor - (punktgroesse / 2), absOffsetY - vcenter.Y * zoomfaktor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            Exit Select
                    End Select
                    Exit Select
                Case 3
                    e.Graphics.DrawString("Set primitive width", New Font("Arial", 16, FontStyle.Regular, GraphicsUnit.Pixel), Brushes.Black, 12, 32)
                    e.Graphics.DrawString("Set primitive width", New Font("Arial", 16, FontStyle.Regular, GraphicsUnit.Pixel), Brushes.Azure, 10, 30)
                    e.Graphics.FillRectangle(Brushes.DarkRed, New RectangleF(absOffsetX + primitiveCenter.X * zoomfaktor - 1, absOffsetY - primitiveCenter.Y * zoomfaktor - 16, 2, 32))
                    e.Graphics.FillRectangle(Brushes.DarkRed, New RectangleF(absOffsetX + primitiveCenter.X * zoomfaktor - 16, absOffsetY - primitiveCenter.Y * zoomfaktor - 1, 32, 2))
                    primitiveWidth = Fix(getXcoordinate(getCursorpositionX()) - primitiveCenter.X)
                    Select Case primitiveObject
                        Case Is < 3
                            Dim v1, v2, v3 As Vertex
                            v1 = New Vertex(primitiveCenter.X, primitiveCenter.Y + primitiveHeight, False, False)
                            v2 = New Vertex(primitiveCenter.X + primitiveHeight + primitiveWidth, primitiveCenter.Y - (primitiveHeight / 2), False, False)
                            v3 = New Vertex(primitiveCenter.X - primitiveHeight - primitiveWidth, primitiveCenter.Y - (primitiveHeight / 2), False, False)
                            e.Graphics.FillRectangle(Brushes.Red, New RectangleF(absOffsetX + v1.X * zoomfaktor - (punktgroesse / 2), absOffsetY - v1.Y * zoomfaktor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.FillRectangle(Brushes.Red, New RectangleF(absOffsetX + v2.X * zoomfaktor - (punktgroesse / 2), absOffsetY - v2.Y * zoomfaktor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.FillRectangle(Brushes.Red, New RectangleF(absOffsetX + v3.X * zoomfaktor - (punktgroesse / 2), absOffsetY - v3.Y * zoomfaktor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.DrawLine(Pens.Red, absOffsetX + v1.X * zoomfaktor, absOffsetY - v1.Y * zoomfaktor, absOffsetX + v2.X * zoomfaktor, absOffsetY - v2.Y * zoomfaktor)
                            e.Graphics.DrawLine(Pens.Red, absOffsetX + v2.X * zoomfaktor, absOffsetY - v2.Y * zoomfaktor, absOffsetX + v3.X * zoomfaktor, absOffsetY - v3.Y * zoomfaktor)
                            e.Graphics.DrawLine(Pens.Red, absOffsetX + v3.X * zoomfaktor, absOffsetY - v3.Y * zoomfaktor, absOffsetX + v1.X * zoomfaktor, absOffsetY - v1.Y * zoomfaktor)
                            Exit Select
                        Case Is < 6
                            Dim v1, v2, v3, v4 As Vertex
                            v1 = New Vertex(primitiveCenter.X + primitiveWidth, primitiveCenter.Y + primitiveHeight, False, False)
                            v2 = New Vertex(primitiveCenter.X - primitiveWidth, primitiveCenter.Y + primitiveHeight, False, False)
                            v3 = New Vertex(primitiveCenter.X - primitiveWidth, primitiveCenter.Y - primitiveHeight, False, False)
                            v4 = New Vertex(primitiveCenter.X + primitiveWidth, primitiveCenter.Y - primitiveHeight, False, False)
                            e.Graphics.FillRectangle(Brushes.Red, New RectangleF(absOffsetX + v1.X * zoomfaktor - (punktgroesse / 2), absOffsetY - v1.Y * zoomfaktor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.FillRectangle(Brushes.Red, New RectangleF(absOffsetX + v2.X * zoomfaktor - (punktgroesse / 2), absOffsetY - v2.Y * zoomfaktor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.FillRectangle(Brushes.Red, New RectangleF(absOffsetX + v3.X * zoomfaktor - (punktgroesse / 2), absOffsetY - v3.Y * zoomfaktor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.FillRectangle(Brushes.Red, New RectangleF(absOffsetX + v4.X * zoomfaktor - (punktgroesse / 2), absOffsetY - v4.Y * zoomfaktor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.DrawLine(Pens.Red, absOffsetX + v1.X * zoomfaktor, absOffsetY - v1.Y * zoomfaktor, absOffsetX + v2.X * zoomfaktor, absOffsetY - v2.Y * zoomfaktor)
                            e.Graphics.DrawLine(Pens.Red, absOffsetX + v2.X * zoomfaktor, absOffsetY - v2.Y * zoomfaktor, absOffsetX + v3.X * zoomfaktor, absOffsetY - v3.Y * zoomfaktor)
                            e.Graphics.DrawLine(Pens.Red, absOffsetX + v3.X * zoomfaktor, absOffsetY - v3.Y * zoomfaktor, absOffsetX + v4.X * zoomfaktor, absOffsetY - v4.Y * zoomfaktor)
                            e.Graphics.DrawLine(Pens.Red, absOffsetX + v4.X * zoomfaktor, absOffsetY - v4.Y * zoomfaktor, absOffsetX + v1.X * zoomfaktor, absOffsetY - v1.Y * zoomfaktor)
                            e.Graphics.DrawLine(Pens.Red, absOffsetX + v1.X * zoomfaktor, absOffsetY - v1.Y * zoomfaktor, absOffsetX + v3.X * zoomfaktor, absOffsetY - v3.Y * zoomfaktor)
                            Exit Select
                        Case Is < 12
                            primitiveWidth = Fix(Math.Sqrt((getYcoordinate(getCursorpositionY()) - primitiveCenter.Y) ^ 2 + (getXcoordinate(getCursorpositionX()) - primitiveCenter.X) ^ 2))
                            Dim vcenter As Vertex
                            Dim vouter(segments_oval) As Vertex
                            Dim counter As Integer = 0
                            vcenter = New Vertex(primitiveCenter.X, primitiveCenter.Y, False, False)
                            For a As Double = 0 To Math.PI * 2 Step Math.PI * 2 / segments_oval
                                vouter(counter) = New Vertex(primitiveCenter.X + Math.Cos(a) * primitiveWidth, primitiveCenter.Y + Math.Sin(a) * primitiveHeight, False, False)
                                counter += 1
                            Next
                            e.Graphics.FillRectangle(Brushes.Red, New RectangleF(absOffsetX + vouter(0).X * zoomfaktor - (punktgroesse / 2), absOffsetY - vouter(0).Y * zoomfaktor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.DrawLine(Pens.Red, absOffsetX + vouter(0).X * zoomfaktor, absOffsetY - vouter(0).Y * zoomfaktor, absOffsetX + vouter(segments_oval - 1).X * zoomfaktor, absOffsetY - vouter(segments_oval - 1).Y * zoomfaktor)
                            e.Graphics.DrawLine(Pens.Red, absOffsetX + vouter(0).X * zoomfaktor, absOffsetY - vouter(0).Y * zoomfaktor, absOffsetX + vcenter.X * zoomfaktor, absOffsetY - vcenter.Y * zoomfaktor)
                            For counter = 1 To segments_oval - 1
                                e.Graphics.DrawLine(Pens.Red, absOffsetX + vouter(counter).X * zoomfaktor, absOffsetY - vouter(counter).Y * zoomfaktor, absOffsetX + vouter(counter - 1).X * zoomfaktor, absOffsetY - vouter(counter - 1).Y * zoomfaktor)
                                e.Graphics.DrawLine(Pens.Red, absOffsetX + vouter(counter).X * zoomfaktor, absOffsetY - vouter(counter).Y * zoomfaktor, absOffsetX + vcenter.X * zoomfaktor, absOffsetY - vcenter.Y * zoomfaktor)
                                e.Graphics.FillRectangle(Brushes.Red, New RectangleF(absOffsetX + vouter(counter).X * zoomfaktor - (punktgroesse / 2), absOffsetY - vouter(counter).Y * zoomfaktor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            Next
                            e.Graphics.FillRectangle(Brushes.Red, New RectangleF(absOffsetX + vcenter.X * zoomfaktor - (punktgroesse / 2), absOffsetY - vcenter.Y * zoomfaktor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            Exit Select
                    End Select
                    Exit Select
                Case 4
                    e.Graphics.DrawString("Set primitive frame size", New Font("Arial", 16, FontStyle.Regular, GraphicsUnit.Pixel), Brushes.Black, 12, 32)
                    e.Graphics.DrawString("Set primitive frame size", New Font("Arial", 16, FontStyle.Regular, GraphicsUnit.Pixel), Brushes.Azure, 10, 30)
                    e.Graphics.FillRectangle(Brushes.DarkRed, New RectangleF(absOffsetX + primitiveCenter.X * zoomfaktor - 1, absOffsetY - primitiveCenter.Y * zoomfaktor - 16, 2, 32))
                    e.Graphics.FillRectangle(Brushes.DarkRed, New RectangleF(absOffsetX + primitiveCenter.X * zoomfaktor - 16, absOffsetY - primitiveCenter.Y * zoomfaktor - 1, 32, 2))
                    primitiveBordersize = Fix(getXcoordinate(getCursorpositionX()) - primitiveCenter.X) / Math.Sqrt(primitiveWidth ^ 2 + primitiveHeight ^ 2)
                    clip(primitiveBordersize, 0, 1)
                    Select Case primitiveObject
                        Case Is < 3
                            Dim v11, v21, v31 As Vertex
                            v11 = New Vertex(primitiveCenter.X, primitiveCenter.Y + primitiveHeight, False, False)
                            v21 = New Vertex(primitiveCenter.X + primitiveHeight + primitiveWidth, primitiveCenter.Y - (primitiveHeight / 2), False, False)
                            v31 = New Vertex(primitiveCenter.X - primitiveHeight - primitiveWidth, primitiveCenter.Y - (primitiveHeight / 2), False, False)
                            e.Graphics.FillRectangle(Brushes.Red, New RectangleF(absOffsetX + v11.X * zoomfaktor - (punktgroesse / 2), absOffsetY - v11.Y * zoomfaktor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.FillRectangle(Brushes.Red, New RectangleF(absOffsetX + v21.X * zoomfaktor - (punktgroesse / 2), absOffsetY - v21.Y * zoomfaktor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.FillRectangle(Brushes.Red, New RectangleF(absOffsetX + v31.X * zoomfaktor - (punktgroesse / 2), absOffsetY - v31.Y * zoomfaktor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.DrawLine(Pens.Red, absOffsetX + v11.X * zoomfaktor, absOffsetY - v11.Y * zoomfaktor, absOffsetX + v21.X * zoomfaktor, absOffsetY - v21.Y * zoomfaktor)
                            e.Graphics.DrawLine(Pens.Red, absOffsetX + v21.X * zoomfaktor, absOffsetY - v21.Y * zoomfaktor, absOffsetX + v31.X * zoomfaktor, absOffsetY - v31.Y * zoomfaktor)
                            e.Graphics.DrawLine(Pens.Red, absOffsetX + v31.X * zoomfaktor, absOffsetY - v31.Y * zoomfaktor, absOffsetX + v11.X * zoomfaktor, absOffsetY - v11.Y * zoomfaktor)
                            Dim v12, v22, v32 As Vertex
                            v12 = New Vertex(primitiveCenter.X, primitiveCenter.Y + primitiveHeight * primitiveBordersize, False, False)
                            v22 = New Vertex(primitiveCenter.X + (primitiveHeight + primitiveWidth) * primitiveBordersize, primitiveCenter.Y - (primitiveHeight / 2) * primitiveBordersize, False, False)
                            v32 = New Vertex(primitiveCenter.X - (primitiveHeight + primitiveWidth) * primitiveBordersize, primitiveCenter.Y - (primitiveHeight / 2) * primitiveBordersize, False, False)
                            e.Graphics.FillRectangle(Brushes.Red, New RectangleF(absOffsetX + v12.X * zoomfaktor - (punktgroesse / 2), absOffsetY - v12.Y * zoomfaktor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.FillRectangle(Brushes.Red, New RectangleF(absOffsetX + v22.X * zoomfaktor - (punktgroesse / 2), absOffsetY - v22.Y * zoomfaktor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.FillRectangle(Brushes.Red, New RectangleF(absOffsetX + v32.X * zoomfaktor - (punktgroesse / 2), absOffsetY - v32.Y * zoomfaktor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.DrawLine(Pens.Red, absOffsetX + v12.X * zoomfaktor, absOffsetY - v12.Y * zoomfaktor, absOffsetX + v22.X * zoomfaktor, absOffsetY - v22.Y * zoomfaktor)
                            e.Graphics.DrawLine(Pens.Red, absOffsetX + v22.X * zoomfaktor, absOffsetY - v22.Y * zoomfaktor, absOffsetX + v32.X * zoomfaktor, absOffsetY - v32.Y * zoomfaktor)
                            e.Graphics.DrawLine(Pens.Red, absOffsetX + v32.X * zoomfaktor, absOffsetY - v32.Y * zoomfaktor, absOffsetX + v12.X * zoomfaktor, absOffsetY - v12.Y * zoomfaktor)
                            e.Graphics.DrawLine(Pens.Red, absOffsetX + v11.X * zoomfaktor, absOffsetY - v11.Y * zoomfaktor, absOffsetX + v12.X * zoomfaktor, absOffsetY - v12.Y * zoomfaktor)
                            e.Graphics.DrawLine(Pens.Red, absOffsetX + v21.X * zoomfaktor, absOffsetY - v21.Y * zoomfaktor, absOffsetX + v22.X * zoomfaktor, absOffsetY - v22.Y * zoomfaktor)
                            e.Graphics.DrawLine(Pens.Red, absOffsetX + v31.X * zoomfaktor, absOffsetY - v31.Y * zoomfaktor, absOffsetX + v32.X * zoomfaktor, absOffsetY - v32.Y * zoomfaktor)
                            e.Graphics.DrawLine(Pens.Red, absOffsetX + v11.X * zoomfaktor, absOffsetY - v11.Y * zoomfaktor, absOffsetX + v22.X * zoomfaktor, absOffsetY - v22.Y * zoomfaktor)
                            e.Graphics.DrawLine(Pens.Red, absOffsetX + v21.X * zoomfaktor, absOffsetY - v21.Y * zoomfaktor, absOffsetX + v32.X * zoomfaktor, absOffsetY - v32.Y * zoomfaktor)
                            e.Graphics.DrawLine(Pens.Red, absOffsetX + v31.X * zoomfaktor, absOffsetY - v31.Y * zoomfaktor, absOffsetX + v12.X * zoomfaktor, absOffsetY - v12.Y * zoomfaktor)
                            Exit Select
                        Case Is < 6
                            Dim v1, v2, v3, v4, v5, v6, v7, v8 As Vertex
                            v1 = New Vertex(primitiveCenter.X + primitiveWidth, primitiveCenter.Y + primitiveHeight, False, False)
                            v2 = New Vertex(primitiveCenter.X - primitiveWidth, primitiveCenter.Y + primitiveHeight, False, False)
                            v3 = New Vertex(primitiveCenter.X - primitiveWidth, primitiveCenter.Y - primitiveHeight, False, False)
                            v4 = New Vertex(primitiveCenter.X + primitiveWidth, primitiveCenter.Y - primitiveHeight, False, False)
                            v5 = New Vertex(primitiveCenter.X + primitiveWidth * primitiveBordersize, primitiveCenter.Y + primitiveHeight * primitiveBordersize, False, False)
                            v6 = New Vertex(primitiveCenter.X - primitiveWidth * primitiveBordersize, primitiveCenter.Y + primitiveHeight * primitiveBordersize, False, False)
                            v7 = New Vertex(primitiveCenter.X - primitiveWidth * primitiveBordersize, primitiveCenter.Y - primitiveHeight * primitiveBordersize, False, False)
                            v8 = New Vertex(primitiveCenter.X + primitiveWidth * primitiveBordersize, primitiveCenter.Y - primitiveHeight * primitiveBordersize, False, False)
                            e.Graphics.FillRectangle(Brushes.Red, New RectangleF(absOffsetX + v1.X * zoomfaktor - (punktgroesse / 2), absOffsetY - v1.Y * zoomfaktor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.FillRectangle(Brushes.Red, New RectangleF(absOffsetX + v2.X * zoomfaktor - (punktgroesse / 2), absOffsetY - v2.Y * zoomfaktor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.FillRectangle(Brushes.Red, New RectangleF(absOffsetX + v3.X * zoomfaktor - (punktgroesse / 2), absOffsetY - v3.Y * zoomfaktor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.FillRectangle(Brushes.Red, New RectangleF(absOffsetX + v4.X * zoomfaktor - (punktgroesse / 2), absOffsetY - v4.Y * zoomfaktor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.FillRectangle(Brushes.Red, New RectangleF(absOffsetX + v5.X * zoomfaktor - (punktgroesse / 2), absOffsetY - v5.Y * zoomfaktor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.FillRectangle(Brushes.Red, New RectangleF(absOffsetX + v6.X * zoomfaktor - (punktgroesse / 2), absOffsetY - v6.Y * zoomfaktor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.FillRectangle(Brushes.Red, New RectangleF(absOffsetX + v7.X * zoomfaktor - (punktgroesse / 2), absOffsetY - v7.Y * zoomfaktor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.FillRectangle(Brushes.Red, New RectangleF(absOffsetX + v8.X * zoomfaktor - (punktgroesse / 2), absOffsetY - v8.Y * zoomfaktor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            ' Kontur: Rechteck aussen
                            e.Graphics.DrawLine(Pens.Red, absOffsetX + v1.X * zoomfaktor, absOffsetY - v1.Y * zoomfaktor, absOffsetX + v2.X * zoomfaktor, absOffsetY - v2.Y * zoomfaktor)
                            e.Graphics.DrawLine(Pens.Red, absOffsetX + v2.X * zoomfaktor, absOffsetY - v2.Y * zoomfaktor, absOffsetX + v3.X * zoomfaktor, absOffsetY - v3.Y * zoomfaktor)
                            e.Graphics.DrawLine(Pens.Red, absOffsetX + v3.X * zoomfaktor, absOffsetY - v3.Y * zoomfaktor, absOffsetX + v4.X * zoomfaktor, absOffsetY - v4.Y * zoomfaktor)
                            e.Graphics.DrawLine(Pens.Red, absOffsetX + v4.X * zoomfaktor, absOffsetY - v4.Y * zoomfaktor, absOffsetX + v1.X * zoomfaktor, absOffsetY - v1.Y * zoomfaktor)
                            ' Kontur: Rechteck innen
                            e.Graphics.DrawLine(Pens.Red, absOffsetX + v5.X * zoomfaktor, absOffsetY - v5.Y * zoomfaktor, absOffsetX + v6.X * zoomfaktor, absOffsetY - v6.Y * zoomfaktor)
                            e.Graphics.DrawLine(Pens.Red, absOffsetX + v6.X * zoomfaktor, absOffsetY - v6.Y * zoomfaktor, absOffsetX + v7.X * zoomfaktor, absOffsetY - v7.Y * zoomfaktor)
                            e.Graphics.DrawLine(Pens.Red, absOffsetX + v7.X * zoomfaktor, absOffsetY - v7.Y * zoomfaktor, absOffsetX + v8.X * zoomfaktor, absOffsetY - v8.Y * zoomfaktor)
                            e.Graphics.DrawLine(Pens.Red, absOffsetX + v8.X * zoomfaktor, absOffsetY - v8.Y * zoomfaktor, absOffsetX + v5.X * zoomfaktor, absOffsetY - v5.Y * zoomfaktor)
                            ' Diagonale: Innen
                            If primitiveObject = 4 Then e.Graphics.DrawLine(Pens.Red, absOffsetX + v5.X * zoomfaktor, absOffsetY - v5.Y * zoomfaktor, absOffsetX + v7.X * zoomfaktor, absOffsetY - v7.Y * zoomfaktor)
                            ' Verbindungslinien: Aussen/Innen
                            e.Graphics.DrawLine(Pens.Red, absOffsetX + v1.X * zoomfaktor, absOffsetY - v1.Y * zoomfaktor, absOffsetX + v5.X * zoomfaktor, absOffsetY - v5.Y * zoomfaktor)
                            e.Graphics.DrawLine(Pens.Red, absOffsetX + v2.X * zoomfaktor, absOffsetY - v2.Y * zoomfaktor, absOffsetX + v6.X * zoomfaktor, absOffsetY - v6.Y * zoomfaktor)
                            e.Graphics.DrawLine(Pens.Red, absOffsetX + v3.X * zoomfaktor, absOffsetY - v3.Y * zoomfaktor, absOffsetX + v7.X * zoomfaktor, absOffsetY - v7.Y * zoomfaktor)
                            e.Graphics.DrawLine(Pens.Red, absOffsetX + v4.X * zoomfaktor, absOffsetY - v4.Y * zoomfaktor, absOffsetX + v8.X * zoomfaktor, absOffsetY - v8.Y * zoomfaktor)
                            ' Verbindungslinien: Diagonal
                            e.Graphics.DrawLine(Pens.Red, absOffsetX + v1.X * zoomfaktor, absOffsetY - v1.Y * zoomfaktor, absOffsetX + v6.X * zoomfaktor, absOffsetY - v6.Y * zoomfaktor)
                            e.Graphics.DrawLine(Pens.Red, absOffsetX + v2.X * zoomfaktor, absOffsetY - v2.Y * zoomfaktor, absOffsetX + v7.X * zoomfaktor, absOffsetY - v7.Y * zoomfaktor)
                            e.Graphics.DrawLine(Pens.Red, absOffsetX + v3.X * zoomfaktor, absOffsetY - v3.Y * zoomfaktor, absOffsetX + v8.X * zoomfaktor, absOffsetY - v8.Y * zoomfaktor)
                            e.Graphics.DrawLine(Pens.Red, absOffsetX + v4.X * zoomfaktor, absOffsetY - v4.Y * zoomfaktor, absOffsetX + v5.X * zoomfaktor, absOffsetY - v5.Y * zoomfaktor)
                            Exit Select
                        Case Is < 9
                            Dim vcenter As Vertex
                            Dim vinner(segments_circle) As Vertex
                            Dim vouter(segments_circle) As Vertex
                            Dim counter As Integer = 0
                            vcenter = New Vertex(primitiveCenter.X, primitiveCenter.Y, False, False)
                            For a As Double = 0 To Math.PI * 2 Step Math.PI * 2 / segments_circle
                                vinner(counter) = New Vertex(primitiveCenter.X + Math.Cos(a) * primitiveHeight * primitiveBordersize, primitiveCenter.Y + Math.Sin(a) * primitiveHeight * primitiveBordersize, False, False)
                                vouter(counter) = New Vertex(primitiveCenter.X + Math.Cos(a) * primitiveHeight, primitiveCenter.Y + Math.Sin(a) * primitiveHeight, False, False)
                                counter += 1
                            Next
                            e.Graphics.FillRectangle(Brushes.Red, New RectangleF(absOffsetX + vouter(0).X * zoomfaktor - (punktgroesse / 2), absOffsetY - vouter(0).Y * zoomfaktor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.FillRectangle(Brushes.Red, New RectangleF(absOffsetX + vinner(0).X * zoomfaktor - (punktgroesse / 2), absOffsetY - vinner(0).Y * zoomfaktor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.DrawLine(Pens.Red, absOffsetX + vouter(0).X * zoomfaktor, absOffsetY - vouter(0).Y * zoomfaktor, absOffsetX + vouter(segments_circle - 1).X * zoomfaktor, absOffsetY - vouter(segments_circle - 1).Y * zoomfaktor)
                            e.Graphics.DrawLine(Pens.Red, absOffsetX + vinner(0).X * zoomfaktor, absOffsetY - vinner(0).Y * zoomfaktor, absOffsetX + vinner(segments_circle - 1).X * zoomfaktor, absOffsetY - vinner(segments_circle - 1).Y * zoomfaktor)
                            e.Graphics.DrawLine(Pens.Red, absOffsetX + vinner(0).X * zoomfaktor, absOffsetY - vinner(0).Y * zoomfaktor, absOffsetX + vouter(segments_circle - 1).X * zoomfaktor, absOffsetY - vouter(segments_circle - 1).Y * zoomfaktor)
                            e.Graphics.DrawLine(Pens.Red, absOffsetX + vinner(segments_circle - 1).X * zoomfaktor, absOffsetY - vinner(segments_circle - 1).Y * zoomfaktor, absOffsetX + vouter(segments_circle - 1).X * zoomfaktor, absOffsetY - vouter(segments_circle - 1).Y * zoomfaktor)
                            If primitiveObject = 7 Then e.Graphics.DrawLine(Pens.Red, absOffsetX + vinner(0).X * zoomfaktor, absOffsetY - vinner(0).Y * zoomfaktor, absOffsetX + vcenter.X * zoomfaktor, absOffsetY - vcenter.Y * zoomfaktor)
                            For counter = 1 To segments_circle - 1
                                e.Graphics.DrawLine(Pens.Red, absOffsetX + vouter(counter).X * zoomfaktor, absOffsetY - vouter(counter).Y * zoomfaktor, absOffsetX + vouter(counter - 1).X * zoomfaktor, absOffsetY - vouter(counter - 1).Y * zoomfaktor)
                                e.Graphics.DrawLine(Pens.Red, absOffsetX + vinner(counter).X * zoomfaktor, absOffsetY - vinner(counter).Y * zoomfaktor, absOffsetX + vinner(counter - 1).X * zoomfaktor, absOffsetY - vinner(counter - 1).Y * zoomfaktor)
                                e.Graphics.DrawLine(Pens.Red, absOffsetX + vinner(counter).X * zoomfaktor, absOffsetY - vinner(counter).Y * zoomfaktor, absOffsetX + vouter(counter - 1).X * zoomfaktor, absOffsetY - vouter(counter - 1).Y * zoomfaktor)
                                e.Graphics.DrawLine(Pens.Red, absOffsetX + vinner(counter - 1).X * zoomfaktor, absOffsetY - vinner(counter - 1).Y * zoomfaktor, absOffsetX + vouter(counter - 1).X * zoomfaktor, absOffsetY - vouter(counter - 1).Y * zoomfaktor)
                                If primitiveObject = 7 Then e.Graphics.DrawLine(Pens.Red, absOffsetX + vinner(counter).X * zoomfaktor, absOffsetY - vinner(counter).Y * zoomfaktor, absOffsetX + vcenter.X * zoomfaktor, absOffsetY - vcenter.Y * zoomfaktor)
                                e.Graphics.FillRectangle(Brushes.Red, New RectangleF(absOffsetX + vouter(counter).X * zoomfaktor - (punktgroesse / 2), absOffsetY - vouter(counter).Y * zoomfaktor - (punktgroesse / 2), punktgroesse, punktgroesse))
                                e.Graphics.FillRectangle(Brushes.Red, New RectangleF(absOffsetX + vinner(counter).X * zoomfaktor - (punktgroesse / 2), absOffsetY - vinner(counter).Y * zoomfaktor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            Next
                            e.Graphics.FillRectangle(Brushes.Red, New RectangleF(absOffsetX + vcenter.X * zoomfaktor - (punktgroesse / 2), absOffsetY - vcenter.Y * zoomfaktor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            Exit Select
                        Case Is < 12
                            Dim vcenter As Vertex
                            Dim vinner(segments_oval) As Vertex
                            Dim vouter(segments_oval) As Vertex
                            Dim counter As Integer = 0
                            vcenter = New Vertex(primitiveCenter.X, primitiveCenter.Y, False, False)
                            For a As Double = 0 To Math.PI * 2 Step Math.PI * 2 / segments_oval
                                vinner(counter) = New Vertex(primitiveCenter.X + Math.Cos(a) * primitiveWidth * primitiveBordersize, primitiveCenter.Y + Math.Sin(a) * primitiveHeight * primitiveBordersize, False, False)
                                vouter(counter) = New Vertex(primitiveCenter.X + Math.Cos(a) * primitiveWidth, primitiveCenter.Y + Math.Sin(a) * primitiveHeight, False, False)
                                counter += 1
                            Next
                            e.Graphics.FillRectangle(Brushes.Red, New RectangleF(absOffsetX + vouter(0).X * zoomfaktor - (punktgroesse / 2), absOffsetY - vouter(0).Y * zoomfaktor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.FillRectangle(Brushes.Red, New RectangleF(absOffsetX + vinner(0).X * zoomfaktor - (punktgroesse / 2), absOffsetY - vinner(0).Y * zoomfaktor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            e.Graphics.DrawLine(Pens.Red, absOffsetX + vouter(0).X * zoomfaktor, absOffsetY - vouter(0).Y * zoomfaktor, absOffsetX + vouter(segments_oval - 1).X * zoomfaktor, absOffsetY - vouter(segments_oval - 1).Y * zoomfaktor)
                            e.Graphics.DrawLine(Pens.Red, absOffsetX + vinner(0).X * zoomfaktor, absOffsetY - vinner(0).Y * zoomfaktor, absOffsetX + vinner(segments_oval - 1).X * zoomfaktor, absOffsetY - vinner(segments_oval - 1).Y * zoomfaktor)
                            e.Graphics.DrawLine(Pens.Red, absOffsetX + vinner(0).X * zoomfaktor, absOffsetY - vinner(0).Y * zoomfaktor, absOffsetX + vouter(segments_oval - 1).X * zoomfaktor, absOffsetY - vouter(segments_oval - 1).Y * zoomfaktor)
                            e.Graphics.DrawLine(Pens.Red, absOffsetX + vinner(segments_oval - 1).X * zoomfaktor, absOffsetY - vinner(segments_oval - 1).Y * zoomfaktor, absOffsetX + vouter(segments_oval - 1).X * zoomfaktor, absOffsetY - vouter(segments_oval - 1).Y * zoomfaktor)
                            If primitiveObject = 10 Then e.Graphics.DrawLine(Pens.Red, absOffsetX + vinner(0).X * zoomfaktor, absOffsetY - vinner(0).Y * zoomfaktor, absOffsetX + vcenter.X * zoomfaktor, absOffsetY - vcenter.Y * zoomfaktor)
                            For counter = 1 To segments_oval - 1
                                e.Graphics.DrawLine(Pens.Red, absOffsetX + vouter(counter).X * zoomfaktor, absOffsetY - vouter(counter).Y * zoomfaktor, absOffsetX + vouter(counter - 1).X * zoomfaktor, absOffsetY - vouter(counter - 1).Y * zoomfaktor)
                                e.Graphics.DrawLine(Pens.Red, absOffsetX + vinner(counter).X * zoomfaktor, absOffsetY - vinner(counter).Y * zoomfaktor, absOffsetX + vinner(counter - 1).X * zoomfaktor, absOffsetY - vinner(counter - 1).Y * zoomfaktor)
                                e.Graphics.DrawLine(Pens.Red, absOffsetX + vinner(counter).X * zoomfaktor, absOffsetY - vinner(counter).Y * zoomfaktor, absOffsetX + vouter(counter - 1).X * zoomfaktor, absOffsetY - vouter(counter - 1).Y * zoomfaktor)
                                e.Graphics.DrawLine(Pens.Red, absOffsetX + vinner(counter - 1).X * zoomfaktor, absOffsetY - vinner(counter - 1).Y * zoomfaktor, absOffsetX + vouter(counter - 1).X * zoomfaktor, absOffsetY - vouter(counter - 1).Y * zoomfaktor)
                                If primitiveObject = 10 Then e.Graphics.DrawLine(Pens.Red, absOffsetX + vinner(counter).X * zoomfaktor, absOffsetY - vinner(counter).Y * zoomfaktor, absOffsetX + vcenter.X * zoomfaktor, absOffsetY - vcenter.Y * zoomfaktor)
                                e.Graphics.FillRectangle(Brushes.Red, New RectangleF(absOffsetX + vouter(counter).X * zoomfaktor - (punktgroesse / 2), absOffsetY - vouter(counter).Y * zoomfaktor - (punktgroesse / 2), punktgroesse, punktgroesse))
                                e.Graphics.FillRectangle(Brushes.Red, New RectangleF(absOffsetX + vinner(counter).X * zoomfaktor - (punktgroesse / 2), absOffsetY - vinner(counter).Y * zoomfaktor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            Next
                            e.Graphics.FillRectangle(Brushes.Red, New RectangleF(absOffsetX + vcenter.X * zoomfaktor - (punktgroesse / 2), absOffsetY - vcenter.Y * zoomfaktor - (punktgroesse / 2), punktgroesse, punktgroesse))
                            Exit Select
                    End Select
                Case 255
                    e.Graphics.DrawString("Click to add a new triangle", New Font("Arial", 16, FontStyle.Regular, GraphicsUnit.Pixel), Brushes.Black, 12, 32)
                    e.Graphics.DrawString("Click to add a new triangle", New Font("Arial", 16, FontStyle.Regular, GraphicsUnit.Pixel), Brushes.Azure, 10, 30)
                    'primitiveCenter = New Vertex(Fix(getXcoordinate(getCursorpositionX() - 3)), Fix(getYcoordinate(getCursorpositionY())), False, False)
                    primitiveCenter = New Vertex(Math.Round(getXcoordinate(getCursorpositionX() - 3) / Me.moveSnap) * Me.moveSnap, Math.Round(getYcoordinate(getCursorpositionY()) / Me.moveSnap) * Me.moveSnap, False, False)
                    e.Graphics.DrawLine(Pens.RoyalBlue, absOffsetX + primitiveCenter.X * zoomfaktor, absOffsetY - primitiveCenter.Y * zoomfaktor, absOffsetX + LLast(Vertices).X * zoomfaktor, absOffsetY - LLast(Vertices).Y * zoomfaktor)
                    e.Graphics.DrawLine(Pens.PowderBlue, absOffsetX + primitiveCenter.X * zoomfaktor, absOffsetY - primitiveCenter.Y * zoomfaktor, absOffsetX + Vertices(Vertices.Count - 2).X * zoomfaktor, absOffsetY - Vertices(Vertices.Count - 2).Y * zoomfaktor)
                    e.Graphics.DrawLine(Pens.PowderBlue, absOffsetX + LLast(Vertices).X * zoomfaktor, absOffsetY - LLast(Vertices).Y * zoomfaktor, absOffsetX + Vertices(Vertices.Count - 2).X * zoomfaktor, absOffsetY - Vertices(Vertices.Count - 2).Y * zoomfaktor)
                    Exit Select
            End Select
        End If
        wa.Stop()
        'If wa.ElapsedMilliseconds > 0 Then Me.Text = 1 / (wa.ElapsedMilliseconds / 1000) & " FPS"
        'e.Graphics.DrawLine(Pens.Green, absOffsetX + testdx11 * zoomfaktor, absOffsetY - testdy11 * zoomfaktor, absOffsetX + testdx12 * zoomfaktor, absOffsetY - testdy12 * zoomfaktor)
        'e.Graphics.DrawLine(Pens.Green, absOffsetX + testdx21 * zoomfaktor, absOffsetY - testdy21 * zoomfaktor, absOffsetX + testdx22 * zoomfaktor, absOffsetY - testdy22 * zoomfaktor)
        'e.Graphics.FillRectangle(Brushes.PowderBlue, New RectangleF(absOffsetX + testcx * zoomfaktor - (punktgroesse / 2), absOffsetY - testcy * zoomfaktor - (punktgroesse / 2), punktgroesse, punktgroesse))

        'For i As Integer = 0 To test_x.Count - 1
        'e.Graphics.FillRectangle(Brushes.Orange, New RectangleF(absOffsetX + test_x(i) * zoomfaktor - (punktgroesse / 2), absOffsetY - test_y(i) * zoomfaktor - (punktgroesse / 2), punktgroesse, punktgroesse))
        'Next i
    End Sub

    Dim testcx As Single
    Dim testcy As Single

    Dim testdx11 As Single
    Dim testdy11 As Single
    Dim testdx12 As Single
    Dim testdy12 As Single

    Dim testdx21 As Single
    Dim testdy21 As Single
    Dim testdx22 As Single
    Dim testdy22 As Single

    Dim test_x As New List(Of Single)
    Dim test_y As New List(Of Single)

    Private Sub ExitToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ExitToolStripMenuItem.Click
        Me.Close()
    End Sub

    Private Sub BtnColours_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BtnColours.Click
        ToolStrip2.Visible = BtnColours.Checked
        Me.Refresh()
    End Sub

    Private Sub BtnAutoRound_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BtnAutoRound.Click
        For Each vert As Vertex In SelectedVertices
            If vert.groupindex = -1 Then
                vert.X = Math.Round(vert.X)
                vert.Y = Math.Round(vert.Y)
            End If
        Next
        Me.Refresh()
    End Sub

    Private Sub RTBDebug_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RTBDebug.TextChanged
        If RTBDebug.Lines.Length > 35 Then
            RTBDebug.Text = RTBDebug.Lines(RTBDebug.Lines.Length - 1)
        End If
    End Sub

    Private Sub ProjectOnYZPlaneToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ProjectOnYZPlaneToolStripMenuItem.Click
        importDat(0)
    End Sub

    Private Sub ProjectOnZXPlaneToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ProjectOnZXPlaneToolStripMenuItem.Click
        importDat(1)
    End Sub

    Private Sub ProjectOnXYPlaneToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ProjectOnXYPlaneToolStripMenuItem.Click
        importDat(2)
    End Sub

    Private Sub importDat(ByVal projectMode As Byte)
        If OpenDAT.ShowDialog() = Windows.Forms.DialogResult.OK Then
            If OpenDAT.FileName <> "" Then
                Try
                    newPattern()
                    Using DateiIn As StreamReader = My.Computer.FileSystem.OpenTextFileReader(OpenDAT.FileName, System.Text.Encoding.GetEncoding(20127))
                        Dim textline As String
                        Do
                            textline = Replace(DateiIn.ReadLine().Trim(), ".", ",")
                            If textline <> Nothing Then
                                Dim words() As String = textline.Split(CType(" ", Char))
                                If words(0) = "3" AndAlso words.Length = 11 Then
                                    Select Case projectMode
                                        Case 0
                                            '3 4, 6 7, 9 10
                                            Vertices.Add(New Vertex(CType(words(3), Single), CType(words(4), Single), False))
                                            Vertices.Add(New Vertex(CType(words(6), Single), CType(words(7), Single), False))
                                            Vertices.Add(New Vertex(CType(words(9), Single), CType(words(10), Single), False))
                                            Triangles.Add(New Triangle(Vertices(Vertices.Count - 1), Vertices(Vertices.Count - 2), Vertices(Vertices.Count - 3)) With {.myColourNumber = CType(words(1), Short)})
                                            Vertices(Vertices.Count - 1).linkedTriangles.Add(LLast(Triangles))
                                            Vertices(Vertices.Count - 2).linkedTriangles.Add(LLast(Triangles))
                                            Vertices(Vertices.Count - 3).linkedTriangles.Add(LLast(Triangles))
                                        Case 1
                                            '4 2, 7 5, 10 8
                                            Vertices.Add(New Vertex(CType(words(4), Single), CType(words(2), Single), False))
                                            Vertices.Add(New Vertex(CType(words(7), Single), CType(words(5), Single), False))
                                            Vertices.Add(New Vertex(CType(words(10), Single), CType(words(8), Single), False))
                                            Triangles.Add(New Triangle(Vertices(Vertices.Count - 1), Vertices(Vertices.Count - 2), Vertices(Vertices.Count - 3)) With {.myColourNumber = CType(words(1), Short)})
                                            Vertices(Vertices.Count - 1).linkedTriangles.Add(LLast(Triangles))
                                            Vertices(Vertices.Count - 2).linkedTriangles.Add(LLast(Triangles))
                                            Vertices(Vertices.Count - 3).linkedTriangles.Add(LLast(Triangles))
                                        Case 2
                                            '2 3, 5 6, 8 9
                                            Vertices.Add(New Vertex(CType(words(2), Single), CType(words(3), Single), False))
                                            Vertices.Add(New Vertex(CType(words(5), Single), CType(words(6), Single), False))
                                            Vertices.Add(New Vertex(CType(words(8), Single), CType(words(9), Single), False))
                                            Triangles.Add(New Triangle(Vertices(Vertices.Count - 1), Vertices(Vertices.Count - 2), Vertices(Vertices.Count - 3)) With {.myColourNumber = CType(words(1), Short)})
                                            Vertices(Vertices.Count - 1).linkedTriangles.Add(LLast(Triangles))
                                            Vertices(Vertices.Count - 2).linkedTriangles.Add(LLast(Triangles))
                                            Vertices(Vertices.Count - 3).linkedTriangles.Add(LLast(Triangles))
                                    End Select
                                End If
                                If words(0) = "4" AndAlso words.Length = 14 Then
                                    Select Case projectMode
                                        Case 0
                                            '3 4, 6 7, 9 10, 12 13
                                            Vertices.Add(New Vertex(CType(words(3), Single), CType(words(4), Single), False))
                                            Vertices.Add(New Vertex(CType(words(6), Single), CType(words(7), Single), False))
                                            Vertices.Add(New Vertex(CType(words(9), Single), CType(words(10), Single), False))
                                            Triangles.Add(New Triangle(Vertices(Vertices.Count - 1), Vertices(Vertices.Count - 2), Vertices(Vertices.Count - 3)) With {.myColourNumber = CType(words(1), Short)})
                                            Vertices(Vertices.Count - 1).linkedTriangles.Add(LLast(Triangles))
                                            Vertices(Vertices.Count - 2).linkedTriangles.Add(LLast(Triangles))
                                            Vertices(Vertices.Count - 3).linkedTriangles.Add(LLast(Triangles))
                                            Vertices.Add(New Vertex(CType(words(3), Single), CType(words(4), Single), False))
                                            Vertices.Add(New Vertex(CType(words(9), Single), CType(words(10), Single), False))
                                            Vertices.Add(New Vertex(CType(words(12), Single), CType(words(13), Single), False))
                                            Triangles.Add(New Triangle(Vertices(Vertices.Count - 1), Vertices(Vertices.Count - 2), Vertices(Vertices.Count - 3)) With {.myColourNumber = CType(words(1), Short)})
                                            Vertices(Vertices.Count - 1).linkedTriangles.Add(LLast(Triangles))
                                            Vertices(Vertices.Count - 2).linkedTriangles.Add(LLast(Triangles))
                                            Vertices(Vertices.Count - 3).linkedTriangles.Add(LLast(Triangles))
                                        Case 1
                                            '4 2, 7 5, 10 8, 13 11
                                            Vertices.Add(New Vertex(CType(words(4), Single), CType(words(2), Single), False))
                                            Vertices.Add(New Vertex(CType(words(7), Single), CType(words(5), Single), False))
                                            Vertices.Add(New Vertex(CType(words(10), Single), CType(words(8), Single), False))
                                            Triangles.Add(New Triangle(Vertices(Vertices.Count - 1), Vertices(Vertices.Count - 2), Vertices(Vertices.Count - 3)) With {.myColourNumber = CType(words(1), Short)})
                                            Vertices(Vertices.Count - 1).linkedTriangles.Add(LLast(Triangles))
                                            Vertices(Vertices.Count - 2).linkedTriangles.Add(LLast(Triangles))
                                            Vertices(Vertices.Count - 3).linkedTriangles.Add(LLast(Triangles))
                                            Vertices.Add(New Vertex(CType(words(4), Single), CType(words(2), Single), False))
                                            Vertices.Add(New Vertex(CType(words(10), Single), CType(words(8), Single), False))
                                            Vertices.Add(New Vertex(CType(words(13), Single), CType(words(11), Single), False))
                                            Triangles.Add(New Triangle(Vertices(Vertices.Count - 1), Vertices(Vertices.Count - 2), Vertices(Vertices.Count - 3)) With {.myColourNumber = CType(words(1), Short)})
                                            Vertices(Vertices.Count - 1).linkedTriangles.Add(LLast(Triangles))
                                            Vertices(Vertices.Count - 2).linkedTriangles.Add(LLast(Triangles))
                                            Vertices(Vertices.Count - 3).linkedTriangles.Add(LLast(Triangles))
                                        Case 2
                                            '2 3, 5 6, 8 9, 11 12
                                            Vertices.Add(New Vertex(CType(words(2), Single), CType(words(3), Single), False))
                                            Vertices.Add(New Vertex(CType(words(5), Single), CType(words(6), Single), False))
                                            Vertices.Add(New Vertex(CType(words(8), Single), CType(words(9), Single), False))
                                            Triangles.Add(New Triangle(Vertices(Vertices.Count - 1), Vertices(Vertices.Count - 2), Vertices(Vertices.Count - 3)) With {.myColourNumber = CType(words(1), Short)})
                                            Vertices(Vertices.Count - 1).linkedTriangles.Add(LLast(Triangles))
                                            Vertices(Vertices.Count - 2).linkedTriangles.Add(LLast(Triangles))
                                            Vertices(Vertices.Count - 3).linkedTriangles.Add(LLast(Triangles))
                                            Vertices.Add(New Vertex(CType(words(2), Single), CType(words(3), Single), False))
                                            Vertices.Add(New Vertex(CType(words(8), Single), CType(words(9), Single), False))
                                            Vertices.Add(New Vertex(CType(words(11), Single), CType(words(12), Single), False))
                                            Triangles.Add(New Triangle(Vertices(Vertices.Count - 1), Vertices(Vertices.Count - 2), Vertices(Vertices.Count - 3)) With {.myColourNumber = CType(words(1), Short)})
                                            Vertices(Vertices.Count - 1).linkedTriangles.Add(LLast(Triangles))
                                            Vertices(Vertices.Count - 2).linkedTriangles.Add(LLast(Triangles))
                                            Vertices(Vertices.Count - 3).linkedTriangles.Add(LLast(Triangles))
                                    End Select
                                End If
                            End If
                        Loop Until DateiIn.EndOfStream
                    End Using
                    scaleVertices(False)
                    cleanupMergedTriangles()
                    cleanupMergedVertices()
                    paintTriangles()
                    Me.Refresh()
                Catch ex As Exception
                    MsgBox("Invalid DAT format!", MsgBoxStyle.Exclamation, "ERROR")
                    hintergrundbild = My.Resources.temp
                    TBImage.Text = ""
                End Try
            End If
        End If
    End Sub

    Private Sub DebugToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles DebugToolStripMenuItem.Click
        Label1.Visible = DebugToolStripMenuItem.Checked
        RTBDebug.Visible = DebugToolStripMenuItem.Checked
    End Sub

    Private Sub CleanupToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CleanupToolStripMenuItem.Click
        Dim start As Integer = 0
nextTri:
        For i1 As Integer = start To Triangles.Count - 1
            Dim tri As Triangle = Triangles(i1)

            If tri.vertexA Is Nothing Or tri.vertexB Is Nothing Or tri.vertexC Is Nothing Then
                removeTriangle(tri)
                start = i1
                GoTo nextTri
            End If
        Next
    End Sub


    Public Shared Function LLast(Of TSource)( _
     ByVal source As List(Of TSource) _
    ) As TSource
        Return source(source.Count - 1)
    End Function

    Public Shared Function Atan2(ByVal y As Double, ByVal x As Double) As Double
        Dim ta As Double
        ta = Math.Atan2(y, x)
        If ta < 0 Then ta += 2 * Math.PI
        Return ta
    End Function

    Private Sub ToolStripMenuItem1_Click_1(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripMenuItem1.Click
        toffsetX = 0
        toffsetY = 0
        offsetX = 0
        offsetY = 0
        zoomfaktor = 1
        zoomlevel = 0
        LblDebug.Text = "Zoom: 100%"
        Me.Refresh()
    End Sub

    Private Sub AboutToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles AboutToolStripMenuItem.Click
        MsgBox("LDraw Pattern Creator - Version 1.1 Beta" & vbCrLf & vbCrLf & "This software is licensed under the terms of the MIT - License." & vbCrLf & "(C) Nils Schmidt 2010 - 2011" & vbCrLf & "This program uses Rectifier.exe by permission of the author Philippe E. Hurbain - (C) 2009.", MsgBoxStyle.OkOnly + MsgBoxStyle.MsgBoxSetForeground, "About LDraw-Pattern-Creator")
    End Sub

    Private Sub Timer1_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Timer1.Tick
        SBZoom.Focus()
        Timer1.Enabled = False
        Timer1.Dispose()
    End Sub


    Private Sub Timer2_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Timer2.Tick
        Me.Close()
    End Sub

    Private Sub MainForm_Shown(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Shown
        Dim tmx, tmy As Integer
        tmx = System.Windows.Forms.Cursor.Position.X
        tmy = System.Windows.Forms.Cursor.Position.Y
        Me.Activate()
        Me.WindowState = FormWindowState.Maximized
        Me.BringToFront()
        getOffset = True
        moveMouseAbsoluteLeftClick(200, 200)
        Me.Activate()
        Me.BringToFront()
    End Sub

    Private Sub ToolStrip1_MouseEnter(ByVal sender As Object, ByVal e As System.EventArgs) Handles ToolStrip1.MouseEnter
        Me.Cursor = Cursors.Arrow
    End Sub

    Private Sub ToolStrip2_MouseEnter(ByVal sender As Object, ByVal e As System.EventArgs) Handles ToolStrip2.MouseEnter
        Me.Cursor = Cursors.Arrow
    End Sub

    Private Sub GBImage_MouseEnter(ByVal sender As Object, ByVal e As System.EventArgs) Handles GBImage.MouseEnter
        Me.Cursor = Cursors.Arrow
    End Sub

    Private Sub MenuStrip1_MouseEnter(ByVal sender As Object, ByVal e As System.EventArgs) Handles MenuStrip1.MouseEnter
        Me.Cursor = Cursors.Arrow
    End Sub

    Private Sub GBSnap_MouseEnter(ByVal sender As Object, ByVal e As System.EventArgs) Handles GBSnap.MouseEnter
        Me.Cursor = Cursors.Arrow
    End Sub

    Private Sub BtnMode_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles BtnMode.Click
        If BtnMode.Text = "Primitive Mode" Then
            For Each tri As Triangle In SelectedTriangles
                tri.selected = False
            Next
            For Each vert As Vertex In SelectedVertices
                vert.selected = False
            Next
            SelectedTriangles.Clear()
            SelectedVertices.Clear()
            TriangleModeToolStripMenuItem.PerformClick()
        Else
            If BtnMode.Text = "Triangle Mode" Then
                VerticesModeToolStripMenuItem.PerformClick()
            Else
                TriangleModeToolStripMenuItem.PerformClick()
            End If
        End If
    End Sub

    Private Sub Disc44_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Disc44.Click
        primitiveObject = 255
        primitiveName = "disc44"
        startPrimitiveMode()
    End Sub

    Private Sub Disc34_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Disc34.Click
        primitiveObject = 255
        primitiveName = "disc34"
        startPrimitiveMode()
    End Sub

    Private Sub Disc58_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Disc58.Click
        primitiveObject = 255
        primitiveName = "disc58"
        startPrimitiveMode()
    End Sub

    Private Sub Disc24_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Disc24.Click
        primitiveObject = 255
        primitiveName = "disc24"
        startPrimitiveMode()
    End Sub

    Private Sub Disc716_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Disc716.Click
        primitiveObject = 255
        primitiveName = "disc716"
        startPrimitiveMode()
    End Sub

    Private Sub Disc38_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Disc38.Click
        primitiveObject = 255
        primitiveName = "disc38"
        startPrimitiveMode()
    End Sub

    Private Sub Disc516_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Disc516.Click
        primitiveObject = 255
        primitiveName = "disc516"
        startPrimitiveMode()
    End Sub

    Private Sub Disc14_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Disc14.Click
        primitiveObject = 255
        primitiveName = "disc14"
        startPrimitiveMode()
    End Sub

    Private Sub Disc316_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Disc316.Click
        primitiveObject = 255
        primitiveName = "disc316"
        startPrimitiveMode()
    End Sub

    Private Sub Disc18_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Disc18.Click
        primitiveObject = 255
        primitiveName = "disc18"
        startPrimitiveMode()
    End Sub

    Private Sub Disc116_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Disc116.Click
        primitiveObject = 255
        primitiveName = "disc116"
        startPrimitiveMode()
    End Sub

    Private Sub Disc4448_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Disc4448.Click
        primitiveObject = 255
        primitiveName = "disc4448"
        startPrimitiveMode()
    End Sub

    Private Sub Disc2448_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Disc2448.Click
        primitiveObject = 255
        primitiveName = "disc2448"
        startPrimitiveMode()
    End Sub

    Private Sub Disc1448_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Disc1448.Click
        primitiveObject = 255
        primitiveName = "disc1448"
        startPrimitiveMode()
    End Sub

    Private Sub Disc31648_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Disc31648.Click
        primitiveObject = 255
        primitiveName = "disc31648"
        startPrimitiveMode()
    End Sub

    Private Sub Disc1648_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Disc1648.Click
        primitiveObject = 255
        primitiveName = "disc1648"
        startPrimitiveMode()
    End Sub

    Private Sub Disc1848_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Disc1848.Click
        primitiveObject = 255
        primitiveName = "disc1848"
        startPrimitiveMode()
    End Sub

    Private Sub Disc54848_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Disc54848.Click
        primitiveObject = 255
        primitiveName = "disc54848"
        startPrimitiveMode()
    End Sub

    Private Sub Disc11248_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Disc11248.Click
        primitiveObject = 255
        primitiveName = "disc11248"
        startPrimitiveMode()
    End Sub

    Dim unit As String = "LDU"
    Dim factor As Single = 1
    Private Sub LDUToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles LDUToolStripMenuItem.Click
        conversionEnabled = True
        NUDMoveSnap.DecimalPlaces = 0
        NUDGrid.DecimalPlaces = 1
        If Not LDUToolStripMenuItem.Checked Then LDUToolStripMenuItem.Checked = True
        InchToolStripMenuItem.Checked = False
        MillimeterToolStripMenuItem.Checked = False
        unit = "LDU"
        factor = 1
        LblSize.Text = imgScale * hintergrundbild.Width / 1000 * factor & " x " & vbCrLf & imgScale * hintergrundbild.Height / 1000 * factor & unit
        LblMoveSnap.Text = "Move Snap (1/1000 " & unit & "):"
        LblGridSize.Text = "Grid Size (1/1000 " & unit & "):"
        Dim temp As Single
        temp = Me.moveSnap * factor
        NUDMoveSnap.Minimum = 1
        NUDMoveSnap.Maximum = 10000
        clip(temp, NUDMoveSnap.Minimum, NUDMoveSnap.Maximum)
        NUDMoveSnap.Value = temp
        temp = Me.rasterSnap * factor
        NUDGrid.Minimum = 0.1
        NUDGrid.Maximum = 10
        clip(temp, NUDGrid.Minimum, NUDGrid.Maximum)
        NUDGrid.Value = temp
        conversionEnabled = False
    End Sub

    Private Sub MillimeterToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MillimeterToolStripMenuItem.Click
        conversionEnabled = True
        NUDMoveSnap.DecimalPlaces = 1
        NUDGrid.DecimalPlaces = 2
        If Not MillimeterToolStripMenuItem.Checked Then MillimeterToolStripMenuItem.Checked = True
        LDUToolStripMenuItem.Checked = False
        InchToolStripMenuItem.Checked = False
        unit = "mm"
        factor = 0.4F
        LblSize.Text = imgScale * hintergrundbild.Width / 1000 * factor & " x " & vbCrLf & imgScale * hintergrundbild.Height / 1000 * factor & unit
        LblMoveSnap.Text = "Move Snap (1/1000 " & unit & "):"
        LblGridSize.Text = "Grid Size (1/1000 " & unit & "):"
        Dim temp As Single
        temp = Me.moveSnap * factor
        NUDMoveSnap.Minimum = 0.4
        NUDMoveSnap.Maximum = 4000
        clip(temp, NUDMoveSnap.Minimum, NUDMoveSnap.Maximum)
        NUDMoveSnap.Value = temp
        temp = Me.rasterSnap * factor
        NUDGrid.Minimum = 0.4 * 0.1
        NUDGrid.Maximum = 0.4 * 10
        clip(temp, NUDGrid.Minimum, NUDGrid.Maximum)
        NUDGrid.Value = temp
        conversionEnabled = False
    End Sub

    Private Sub InchToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles InchToolStripMenuItem.Click
        conversionEnabled = True
        NUDMoveSnap.DecimalPlaces = 3
        NUDGrid.DecimalPlaces = 4
        If Not InchToolStripMenuItem.Checked Then InchToolStripMenuItem.Checked = True
        LDUToolStripMenuItem.Checked = False
        MillimeterToolStripMenuItem.Checked = False
        unit = "inch"
        factor = 0.016F
        LblSize.Text = imgScale * hintergrundbild.Width / 1000 * factor & " x " & vbCrLf & imgScale * hintergrundbild.Height / 1000 * factor & unit
        LblMoveSnap.Text = "Move Snap (1/1000 " & unit & "):"
        LblGridSize.Text = "Grid Size (1/1000 " & unit & "):"
        Dim temp As Single
        temp = Me.moveSnap * factor
        NUDMoveSnap.Minimum = 0.016
        NUDMoveSnap.Maximum = 160
        clip(temp, NUDMoveSnap.Minimum, NUDMoveSnap.Maximum)
        NUDMoveSnap.Value = temp
        temp = Me.rasterSnap * factor
        NUDGrid.Minimum = 0.016 * 0.1
        NUDGrid.Maximum = 0.016 * 10
        clip(temp, NUDGrid.Minimum, NUDGrid.Maximum)
        NUDGrid.Value = temp
        conversionEnabled = False
    End Sub

    Private Sub BtnImageClose_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BtnImageClose.Click
        ImageToolStripMenuItem.PerformClick()
    End Sub

    Private Sub BtnPrefsClose_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BtnPrefsClose.Click
        Me.SnappingToolStripMenuItem.PerformClick()
    End Sub

    Private Sub BtnImageClose_MouseEnter(ByVal sender As Object, ByVal e As System.EventArgs) Handles BtnImageClose.MouseEnter
        Me.Cursor = Cursors.Arrow
    End Sub

    Private Sub BtnPrefsClose_MouseEnter(ByVal sender As Object, ByVal e As System.EventArgs) Handles BtnPrefsClose.MouseEnter
        Me.Cursor = Cursors.Arrow
    End Sub

    Private Sub PerformanceEnabledToolStripMenuItem_CheckedChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles PerformanceEnabledToolStripMenuItem.CheckedChanged
        If PerformanceEnabledToolStripMenuItem.Checked Then
            While MsgBox("Enabling this mode will speed up the reaction time of this program, but the undo function is deactivated!", MsgBoxStyle.OkOnly + MsgBoxStyle.MsgBoxSetForeground, "Warning:") <> MsgBoxResult.Ok : End While
        End If
    End Sub

End Class


'If (Not verticesInner.Contains(verticesComplete(1)) AndAlso verticesInner.Contains(verticesComplete(2))) OrElse _
'   (verticesInner.Contains(verticesComplete(1)) AndAlso Not verticesInner.Contains(verticesComplete(2))) Then
'    Triangles.Add(New Triangle(verticesComplete(1), verticesComplete(2), verticesComplete(3)))
'    verticesComplete(1).linkedTriangles.Add(LLast(Triangles))
'    verticesComplete(2).linkedTriangles.Add(LLast(Triangles))
'    verticesComplete(3).linkedTriangles.Add(LLast(Triangles))
'Else
'    Triangles.Add(New Triangle(verticesComplete(0), verticesComplete(2), verticesComplete(3)))
'    verticesComplete(0).linkedTriangles.Add(LLast(Triangles))
'    verticesComplete(2).linkedTriangles.Add(LLast(Triangles))
'    verticesComplete(3).linkedTriangles.Add(LLast(Triangles))
'End If

'If (Not verticesInner.Contains(verticesComplete(2)) AndAlso verticesInner.Contains(verticesComplete(3))) OrElse _
'   (verticesInner.Contains(verticesComplete(2)) AndAlso Not verticesInner.Contains(verticesComplete(3))) Then
'    Triangles.Add(New Triangle(verticesComplete(2), verticesComplete(3), verticesComplete(4)))
'    verticesComplete(2).linkedTriangles.Add(LLast(Triangles))
'    verticesComplete(3).linkedTriangles.Add(LLast(Triangles))
'    verticesComplete(4).linkedTriangles.Add(LLast(Triangles))
'Else
'    Triangles.Add(New Triangle(verticesComplete(1), verticesComplete(3), verticesComplete(4)))
'    verticesComplete(1).linkedTriangles.Add(LLast(Triangles))
'    verticesComplete(3).linkedTriangles.Add(LLast(Triangles))
'    verticesComplete(4).linkedTriangles.Add(LLast(Triangles))
'End If

'If (Not verticesInner.Contains(verticesComplete(3)) AndAlso verticesInner.Contains(verticesComplete(4))) OrElse _
'   (verticesInner.Contains(verticesComplete(3)) AndAlso Not verticesInner.Contains(verticesComplete(4))) Then
'    Triangles.Add(New Triangle(verticesComplete(3), verticesComplete(4), verticesComplete(5)))
'    verticesComplete(3).linkedTriangles.Add(LLast(Triangles))
'    verticesComplete(4).linkedTriangles.Add(LLast(Triangles))
'    verticesComplete(5).linkedTriangles.Add(LLast(Triangles))
'Else
'    Triangles.Add(New Triangle(verticesComplete(2), verticesComplete(4), verticesComplete(5)))
'    verticesComplete(2).linkedTriangles.Add(LLast(Triangles))
'    verticesComplete(4).linkedTriangles.Add(LLast(Triangles))
'    verticesComplete(5).linkedTriangles.Add(LLast(Triangles))
'End If

'If (Not verticesInner.Contains(verticesComplete(4)) AndAlso verticesInner.Contains(verticesComplete(5))) OrElse _
'   (verticesInner.Contains(verticesComplete(4)) AndAlso Not verticesInner.Contains(verticesComplete(5))) Then
'    Triangles.Add(New Triangle(verticesComplete(4), verticesComplete(5), verticesComplete(0)))
'    verticesComplete(4).linkedTriangles.Add(LLast(Triangles))
'    verticesComplete(5).linkedTriangles.Add(LLast(Triangles))
'    verticesComplete(0).linkedTriangles.Add(LLast(Triangles))
'Else
'    Triangles.Add(New Triangle(verticesComplete(3), verticesComplete(5), verticesComplete(0)))
'    verticesComplete(3).linkedTriangles.Add(LLast(Triangles))
'    verticesComplete(5).linkedTriangles.Add(LLast(Triangles))
'    verticesComplete(0).linkedTriangles.Add(LLast(Triangles))
'End If

'If (Not verticesInner.Contains(verticesComplete(5)) AndAlso verticesInner.Contains(verticesComplete(0))) OrElse _
'   (verticesInner.Contains(verticesComplete(5)) AndAlso Not verticesInner.Contains(verticesComplete(0))) Then
'    Triangles.Add(New Triangle(verticesComplete(5), verticesComplete(0), verticesComplete(1)))
'    verticesComplete(5).linkedTriangles.Add(LLast(Triangles))
'    verticesComplete(0).linkedTriangles.Add(LLast(Triangles))
'    verticesComplete(1).linkedTriangles.Add(LLast(Triangles))
'Else
'    Triangles.Add(New Triangle(verticesComplete(4), verticesComplete(0), verticesComplete(1)))
'    verticesComplete(4).linkedTriangles.Add(LLast(Triangles))
'    verticesComplete(0).linkedTriangles.Add(LLast(Triangles))
'    verticesComplete(1).linkedTriangles.Add(LLast(Triangles))
'End If




'If (Not verticesInner.Contains(verticesComplete(1)) AndAlso verticesInner.Contains(verticesComplete(2))) OrElse _
'   (verticesInner.Contains(verticesComplete(1)) AndAlso Not verticesInner.Contains(verticesComplete(2))) Then
'    Triangles.Add(New Triangle(verticesComplete(1), verticesComplete(2), verticesComplete(3)))
'    verticesComplete(1).linkedTriangles.Add(LLast(Triangles))
'    verticesComplete(2).linkedTriangles.Add(LLast(Triangles))
'    verticesComplete(3).linkedTriangles.Add(LLast(Triangles))
'Else
'    Triangles.Add(New Triangle(verticesComplete(0), verticesComplete(2), verticesComplete(3)))
'    verticesComplete(0).linkedTriangles.Add(LLast(Triangles))
'    verticesComplete(2).linkedTriangles.Add(LLast(Triangles))
'    verticesComplete(3).linkedTriangles.Add(LLast(Triangles))
'End If

'If (Not verticesInner.Contains(verticesComplete(2)) AndAlso verticesInner.Contains(verticesComplete(3))) OrElse _
'   (verticesInner.Contains(verticesComplete(2)) AndAlso Not verticesInner.Contains(verticesComplete(3))) Then
'    Triangles.Add(New Triangle(verticesComplete(2), verticesComplete(3), verticesComplete(4)))
'    verticesComplete(2).linkedTriangles.Add(LLast(Triangles))
'    verticesComplete(3).linkedTriangles.Add(LLast(Triangles))
'    verticesComplete(4).linkedTriangles.Add(LLast(Triangles))
'Else
'    Triangles.Add(New Triangle(verticesComplete(1), verticesComplete(3), verticesComplete(4)))
'    verticesComplete(1).linkedTriangles.Add(LLast(Triangles))
'    verticesComplete(3).linkedTriangles.Add(LLast(Triangles))
'    verticesComplete(4).linkedTriangles.Add(LLast(Triangles))
'End If

'If (Not verticesInner.Contains(verticesComplete(3)) AndAlso verticesInner.Contains(verticesComplete(4))) OrElse _
'   (verticesInner.Contains(verticesComplete(3)) AndAlso Not verticesInner.Contains(verticesComplete(4))) Then
'    Triangles.Add(New Triangle(verticesComplete(3), verticesComplete(4), verticesComplete(5)))
'    verticesComplete(3).linkedTriangles.Add(LLast(Triangles))
'    verticesComplete(4).linkedTriangles.Add(LLast(Triangles))
'    verticesComplete(5).linkedTriangles.Add(LLast(Triangles))
'Else
'    Triangles.Add(New Triangle(verticesComplete(2), verticesComplete(4), verticesComplete(5)))
'    verticesComplete(2).linkedTriangles.Add(LLast(Triangles))
'    verticesComplete(4).linkedTriangles.Add(LLast(Triangles))
'    verticesComplete(5).linkedTriangles.Add(LLast(Triangles))
'End If

'If (Not verticesInner.Contains(verticesComplete(4)) AndAlso verticesInner.Contains(verticesComplete(5))) OrElse _
'   (verticesInner.Contains(verticesComplete(4)) AndAlso Not verticesInner.Contains(verticesComplete(5))) Then
'    Triangles.Add(New Triangle(verticesComplete(4), verticesComplete(5), verticesComplete(6)))
'    verticesComplete(4).linkedTriangles.Add(LLast(Triangles))
'    verticesComplete(5).linkedTriangles.Add(LLast(Triangles))
'    verticesComplete(6).linkedTriangles.Add(LLast(Triangles))
'Else
'    Triangles.Add(New Triangle(verticesComplete(3), verticesComplete(5), verticesComplete(6)))
'    verticesComplete(3).linkedTriangles.Add(LLast(Triangles))
'    verticesComplete(5).linkedTriangles.Add(LLast(Triangles))
'    verticesComplete(6).linkedTriangles.Add(LLast(Triangles))
'End If

'If (Not verticesInner.Contains(verticesComplete(5)) AndAlso verticesInner.Contains(verticesComplete(6))) OrElse _
'   (verticesInner.Contains(verticesComplete(5)) AndAlso Not verticesInner.Contains(verticesComplete(6))) Then
'    Triangles.Add(New Triangle(verticesComplete(5), verticesComplete(6), verticesComplete(7)))
'    verticesComplete(5).linkedTriangles.Add(LLast(Triangles))
'    verticesComplete(6).linkedTriangles.Add(LLast(Triangles))
'    verticesComplete(7).linkedTriangles.Add(LLast(Triangles))
'Else
'    Triangles.Add(New Triangle(verticesComplete(4), verticesComplete(6), verticesComplete(7)))
'    verticesComplete(4).linkedTriangles.Add(LLast(Triangles))
'    verticesComplete(6).linkedTriangles.Add(LLast(Triangles))
'    verticesComplete(7).linkedTriangles.Add(LLast(Triangles))
'End If

'If (Not verticesInner.Contains(verticesComplete(6)) AndAlso verticesInner.Contains(verticesComplete(7))) OrElse _
'   (verticesInner.Contains(verticesComplete(6)) AndAlso Not verticesInner.Contains(verticesComplete(7))) Then
'    Triangles.Add(New Triangle(verticesComplete(6), verticesComplete(7), verticesComplete(0)))
'    verticesComplete(6).linkedTriangles.Add(LLast(Triangles))
'    verticesComplete(7).linkedTriangles.Add(LLast(Triangles))
'    verticesComplete(0).linkedTriangles.Add(LLast(Triangles))
'Else
'    Triangles.Add(New Triangle(verticesComplete(5), verticesComplete(7), verticesComplete(0)))
'    verticesComplete(5).linkedTriangles.Add(LLast(Triangles))
'    verticesComplete(7).linkedTriangles.Add(LLast(Triangles))
'    verticesComplete(0).linkedTriangles.Add(LLast(Triangles))
'End If

'If (Not verticesInner.Contains(verticesComplete(7)) AndAlso verticesInner.Contains(verticesComplete(0))) OrElse _
'   (verticesInner.Contains(verticesComplete(7)) AndAlso Not verticesInner.Contains(verticesComplete(0))) Then
'    Triangles.Add(New Triangle(verticesComplete(7), verticesComplete(0), verticesComplete(1)))
'    verticesComplete(7).linkedTriangles.Add(LLast(Triangles))
'    verticesComplete(0).linkedTriangles.Add(LLast(Triangles))
'    verticesComplete(1).linkedTriangles.Add(LLast(Triangles))
'Else
'    Triangles.Add(New Triangle(verticesComplete(6), verticesComplete(0), verticesComplete(1)))
'    verticesComplete(6).linkedTriangles.Add(LLast(Triangles))
'    verticesComplete(0).linkedTriangles.Add(LLast(Triangles))
'    verticesComplete(1).linkedTriangles.Add(LLast(Triangles))
'End If